@@ -7,10 +7,10 @@
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @author Joerg Lehrke <jlehrke@noc.de>
  * @package addressbook
- * @copyright (c) 2005-12 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
+ * @copyright (c) 2005-15 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright (c) 2005/6 by Cornelius Weiss <egw@von-und-zu-weiss.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.addressbook_bo.inc.php 48495 2014-09-06 12:09:58Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -52,6 +52,7 @@ class addressbook_bo extends addressbook_so
 		'n_family, n_given',
 		'n_family, n_prefix',
 		'n_fn',
+		'n_family, n_given (bday)',
 	);
 
 	/**
@@ -367,7 +368,8 @@ class addressbook_bo extends addressbook_so
 			asort($to_sort);
 			$addressbooks += $to_sort;
 		}
-		if (!$preferences['addressbook']['hide_accounts'] && (
+		if ($required != EGW_ACL_ADD &&	// do NOT allow to set accounts as default addressbook (AB can add accounts)
+			!$preferences['addressbook']['hide_accounts'] && (
 				($grants[0] & $required) == $required ||
 				$preferences['common']['account_selection'] == 'groupmembers' &&
 				$this->account_repository != 'ldap' && ($required & EGW_ACL_READ)))
@@ -413,7 +415,7 @@ class addressbook_bo extends addressbook_so
 
 		if($isUpdate)
 		{
-			$fileas_fields = array('n_prefix','n_given','n_middle','n_family','n_suffix','n_fn','org_name','org_unit','adr_one_locality');
+			$fileas_fields = array('n_prefix','n_given','n_middle','n_family','n_suffix','n_fn','org_name','org_unit','adr_one_locality','bday');
 			$old = null;
 			foreach($fileas_fields as $field)
 			{
@@ -426,16 +428,30 @@ class addressbook_bo extends addressbook_so
 			unset($old);
 		}
 
-		$fileas = str_replace(array('n_prefix','n_given','n_middle','n_family','n_suffix','n_fn','org_name','org_unit','adr_one_locality'),
-			array($contact['n_prefix'],$contact['n_given'],$contact['n_middle'],$contact['n_family'],$contact['n_suffix'],
-				$contact['n_fn'],$contact['org_name'],$contact['org_unit'],$contact['adr_one_locality']),$type);
-
 		// removing empty delimiters, caused by empty contact fields
-		$fileas = str_replace(array(', , : ',', : ',': , ',', , ',': : ',' ()'),array(': ',': ',': ',', ',': ',''),$fileas);
-		while ($fileas[0] == ':' ||  $fileas[0] == ',') $fileas = substr($fileas,2);
-		while (substr($fileas,-2) == ': ' || substr($fileas,-2) == ', ') $fileas = substr($fileas,0,-2);
+		$fileas = str_replace(array(', , : ',', : ',': , ',', , ',': : ',' ()'),
+			array(': ',': ',': ',', ',': ',''),
+			strtr($type, array(
+				'n_prefix' => $contact['n_prefix'],
+				'n_given'  => $contact['n_given'],
+				'n_middle' => $contact['n_middle'],
+				'n_family' => $contact['n_family'],
+				'n_suffix' => $contact['n_suffix'],
+				'n_fn'     => $contact['n_fn'],
+				'org_name' => $contact['org_name'],
+				'org_unit' => $contact['org_unit'],
+				'adr_one_locality' => $contact['adr_one_locality'],
+				'bday'     => (int)$contact['bday'] ? egw_time::to($contact['bday'], true) : $contact['bday'],
+			)));
 
-		//echo "<p align=right>bocontacts::fileas(,$type)='$fileas'</p>\n";
+		while ($fileas[0] == ':' ||  $fileas[0] == ',')
+		{
+			$fileas = substr($fileas,2);
+		}
+		while (substr($fileas,-2) == ': ' || substr($fileas,-2) == ', ')
+		{
+			$fileas = substr($fileas,0,-2);
+		}
 		return $fileas;
 	}
 
@@ -497,6 +513,7 @@ class addressbook_bo extends addressbook_so
 			'org_name' => lang('company'),
 			'org_unit' => lang('department'),
 			'adr_one_locality' => lang('city'),
+			'bday'     => lang('Birthday'),
 		);
 		foreach($labels as $name => $label)
 		{
@@ -862,6 +879,11 @@ class addressbook_bo extends addressbook_so
 			// if no owner/addressbook set use the setting of the add_default prefs (if set, otherwise the users personal addressbook)
 			if (!isset($contact['owner'])) $contact['owner'] = $this->default_addressbook;
 			if (!isset($contact['private'])) $contact['private'] = (int)$this->default_private;
+			// do NOT allow to create new accounts via addressbook, they are broken without an account_id
+			if (!$contact['owner'] && empty($contact['account_id']))
+			{
+				$contact['owner'] = $this->default_addressbook ? $this->default_addressbook : $this->user;
+			}
 			// allow admins to import contacts with creator / created date set
 			if (!$contact['creator'] || !$this->is_admin($contact)) $contact['creator'] = $this->user;
 			if (!$contact['created'] || !$this->is_admin($contact)) $contact['created'] = $this->now_su;
@@ -919,24 +941,32 @@ class addressbook_bo extends addressbook_so
 		}
 		if (isset($contact['org_name'])) $contact['n_fileas'] = $this->fileas($contact, null, false);
 
-		$to_write = $contact;
-		// (non-admin) user editing his own account, make sure he does not change fields he is not allowed to (eg. via SyncML or xmlrpc)
-		if (!$ignore_acl && !$contact['owner'] && !$this->is_admin($contact))
-		{
-			foreach($contact as $field => $value)
-			{
-				if (!in_array($field,$this->own_account_acl) && !in_array($field,array('id','owner','account_id','modified','modifier')))
-				{
-					unset($to_write[$field]);	// user is now allowed to change that
-				}
-			}
-		}
-
 		// Get old record for tracking changes
 		if (!isset($old) && $isUpdate)
 		{
 			$old = $this->read($contact['id']);
 		}
+		$to_write = $contact;
+		// (non-admin) user editing his own account, make sure he does not change fields he is not allowed to (eg. via SyncML or xmlrpc)
+		if (!$ignore_acl && !$contact['owner'] && !($this->is_admin($contact) || $this->allow_account_edit()))
+		{
+			foreach($contact as $field => $value)
+			{
+				if (!in_array($field,$this->own_account_acl) && !in_array($field,array('id','owner','account_id','modified','modifier')))
+				{
+					// user is not allowed to change that
+					if ($old)
+					{
+						$to_write[$field] = $contact[$field] = $old[$field];
+					}
+					else
+					{
+						unset($to_write[$field]);
+					}
+				}
+			}
+		}
+
 		// IF THE OLD ENTRY IS A ACCOUNT, dont allow to change the owner/location
 		// maybe we need that for id and account_id as well.
 		if (is_array($old) && (!isset($old['owner']) || empty($old['owner'])))
@@ -1043,15 +1073,22 @@ class addressbook_bo extends addressbook_so
 	* reads contacts matched by key and puts all cols in the data array
 	*
 	* @param int|string $contact_id
+	* @param boolean $ignore_acl =false true: no acl check
 	* @return array|boolean array with contact data, null if not found or false on no view perms
 	*/
-	function read($contact_id)
+	function read($contact_id, $ignore_acl=false)
 	{
+		// get so_sql_cf to read private customfields too, if we ignore acl
+		if ($ignore_acl && is_a($this->somain, 'addressbook_sql'))
+		{
+			$cf_backup = (array)$this->somain->customfields;
+			$this->somain->customfields = egw_customfields::get('addressbook', true);
+		}
 		if (!($data = parent::read($contact_id)))
 		{
 			$data = null;	// not found
 		}
-		elseif (!$this->check_perms(EGW_ACL_READ,$data))
+		elseif (!$ignore_acl && !$this->check_perms(EGW_ACL_READ,$data))
 		{
 			$data = false;	// no view perms
 		}
@@ -1068,6 +1105,10 @@ class addressbook_bo extends addressbook_so
 				$data['adr_two_countryname'] = $GLOBALS['egw']->country->get_full_name($data['adr_two_countrycode'], true);
 			}
 		}
+		if (isset($cf_backup))
+		{
+			$this->somain->customfields = $cf_backup;
+		}
 		//error_log(__METHOD__.'('.array2string($contact_id).') returning '.array2string($data));
 		return $data;
 	}
@@ -1461,7 +1502,7 @@ class addressbook_bo extends addressbook_so
 	 */
 	function link_query($pattern, Array &$options = array())
 	{
-		$filter = $result = $criteria = array();
+		$result = $criteria = array();
 		$limit = false;
 		if ($pattern)
 		{
@@ -1483,8 +1524,14 @@ class addressbook_bo extends addressbook_so
 			$cfs = $this->read_customfields($ids);
 			foreach($contacts as $contact)
 			{
-				$result[$contact['id']] = $this->link_title($contact+(array)$cfs[$contact['id']]).
-					($options['type'] === 'email' ? ' <'.$contact['email'].'>' : '');
+				$result[$contact['id']] = $this->link_title($contact+(array)$cfs[$contact['id']]);
+				// make sure to return a correctly quoted rfc822 address, if requested
+				if ($options['type'] === 'email')
+				{
+					$args = explode('@', $contact['email']);
+					$args[] = $result[$contact['id']];
+					$result[$contact['id']] = call_user_func_array('imap_rfc822_write_address', $args);
+				}
 				// show category color
 				if ($contact['cat_id'] && ($color = etemplate::cats2color($contact['cat_id'])))
 				{
