@@ -7,8 +7,8 @@
  * @package addressbook
  * @subpackage groupdav
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @copyright (c) 2007-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @version $Id: class.addressbook_groupdav.inc.php 49644 2014-11-30 09:24:44Z ralfbecker $
+ * @copyright (c) 2007-15 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
+ * @version $Id$
  */
 
 /**
@@ -185,12 +185,13 @@ class addressbook_groupdav extends groupdav_handler
 	 * Callback for profind interator
 	 *
 	 * @param string $path
-	 * @param array $filter
+	 * @param array& $filter
 	 * @param array|boolean $start=false false=return all or array(start,num)
 	 * @return array with "files" array with values for keys path and props
 	 */
-	function &propfind_callback($path,array $filter,$start=false,$report_not_found_multiget_ids=true)
+	function &propfind_callback($path,array &$filter,$start=false,$report_not_found_multiget_ids=true)
 	{
+		//error_log(__METHOD__."('$path', ".array2string($filter).", ".array2string($start).", $report_not_found_multiget_ids)");
 		$starttime = microtime(true);
 		$filter_in = $filter;
 
@@ -213,11 +214,11 @@ class addressbook_groupdav extends groupdav_handler
 		$sync_collection_report = $filter['sync-collection'];
 		unset($filter['sync-collection']);
 
-		if (isset($filter_in[self::$path_attr]) && !is_array($filter_in[self::$path_attr]))
+		if (isset($filter[self::$path_attr]))
 		{
-			$filter_in[self::$path_attr] = (array)$filter_in[self::$path_attr];
+			if (!is_array($filter[self::$path_attr])) $filter[self::$path_attr] = (array)$filter[self::$path_attr];
+			$requested_multiget_ids =& $filter[self::$path_attr];
 		}
-		$requested_multiget_ids =& $filter_in[self::$path_attr];
 
 		$files = array();
 		// we query etag and modified, as LDAP does not have the strong sql etag
@@ -259,8 +260,10 @@ class addressbook_groupdav extends groupdav_handler
 				$this->sync_collection_token = $contact['modified'];
 			}
 		}
+		// last chunk or no chunking: add accounts from different repo and report missing multiget urls
 		if (!$start || count($contacts) < $start[1])
 		{
+			//error_log(__METHOD__."('$path', ".array2string($filter).", ".array2string($start)."; $report_not_found_multiget_ids) last chunk detected: count()=".count($contacts)." < $start[1]");
 			// add accounts after contacts, if enabled and stored in different repository
 			if ($this->bo->so_accounts && is_array($filter['owner']) && in_array('0', $filter['owner']))
 			{
@@ -332,13 +335,13 @@ class addressbook_groupdav extends groupdav_handler
 					}
 				}
 			}
-		}
-		// report not found multiget urls
-		if ($report_not_found_multiget_ids && $requested_multiget_ids)
-		{
-			foreach($requested_multiget_ids as $id)
+			// report not found multiget urls
+			if ($report_not_found_multiget_ids && $requested_multiget_ids)
 			{
-				$files[] = array('path' => $path.$id.self::$path_extension);
+				foreach($requested_multiget_ids as $id)
+				{
+					$files[] = array('path' => $path.$id.self::$path_extension);
+				}
 			}
 		}
 
@@ -640,12 +643,12 @@ class addressbook_groupdav extends groupdav_handler
 			$contact['carddav_name'] = $id;
 
 			// only set owner, if user is explicitly specified in URL (check via prefix, NOT for /addressbook/) or sync-all-in-one!)
-			if ($prefix && !in_array('O',$this->home_set_pref))
+			if ($prefix && !in_array('O',$this->home_set_pref) && $user)
 			{
 				$contact['owner'] = $user;
 			}
-			// check if default addressbook is synced, if not use (always synced) personal addressbook
-			elseif($this->bo->default_addressbook && !in_array($this->bo->default_addressbook,$this->home_set_pref))
+			// check if default addressbook is synced and not accounts, if not use (always synced) personal addressbook
+			elseif(!$this->bo->default_addressbook || !in_array($this->bo->default_addressbook,$this->home_set_pref))
 			{
 				$contact['owner'] = $GLOBALS['egw_info']['user']['account_id'];
 			}
@@ -669,6 +672,8 @@ class addressbook_groupdav extends groupdav_handler
 			if ($this->debug) error_log(__METHOD__."(,$id) save(".array2string($contact).") failed, Ok=$save_ok");
 			if ($save_ok === 0)
 			{
+				// honor Prefer: return=representation for 412 too (no need for client to explicitly reload)
+				$this->check_return_representation($options, $id, $user);
 				return '412 Precondition Failed';
 			}
 			return '403 Forbidden';	// happens when writing new entries in AB's without ADD rights
@@ -1024,7 +1029,14 @@ class addressbook_groupdav extends groupdav_handler
 	 */
 	public function get_grants()
 	{
-		return $this->bo->get_grants($this->bo->user);
+		$grants = $this->bo->get_grants($this->bo->user);
+
+		// remove add and delete grants for accounts (for admins too)
+		// as accounts can not be created as contacts, they eg. need further data
+		// and admins might not recognice they delete an account incl. its data
+		if (isset($grants[0])) $grants[0] &= ~(EGW_ACL_ADD|EGW_ACL_DELETE);
+
+		return $grants;
 	}
 
 	/**
@@ -1064,6 +1076,27 @@ class addressbook_groupdav extends groupdav_handler
 		return $shared;
 	}
 
+	/**
+	 * Hook to add properties to CardDAV root
+	 *
+	 * OS X 10.11.4 addressbook does a propfind for "addressbook-home-set" and "directory-gateway"
+	 * in the root and does not continue without it.
+	 *
+	 * @param array $data
+	 */
+	public static function groupdav_root_props(array $data)
+	{
+		$data['props']['addressbook-home-set'] = groupdav::mkprop(groupdav::CARDDAV, 'addressbook-home-set', array(
+			groupdav::mkprop('href',$data['caldav']->base_uri.'/'.$GLOBALS['egw_info']['user']['account_lid'].'/')));
+
+		$data['props']['principal-address'] = groupdav::mkprop(groupdav::CARDDAV, 'principal-address',
+				$GLOBALS['egw_info']['user']['preferences']['addressbook']['hide_accounts'] ? '' : array(
+				groupdav::mkprop('href',$data['caldav']->base_uri.'/addressbook-accounts/'.$GLOBALS['egw_info']['user']['person_id'].'.vcf')));
+
+		$data['props']['directory-gateway'] = groupdav::mkprop(groupdav::CARDDAV, 'directory-gateway', array(
+			groupdav::mkprop('href',$data['caldav']->base_uri.'/addressbook/')));
+	}
+
 	/**
 	 * Return appliction specific settings
 	 *
