@@ -9,15 +9,11 @@
  * @package addressbook
  * @subpackage export
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.addressbook_vcal.inc.php 41773 2013-02-19 19:53:20Z jlehrke $
+ * @version $Id$
  */
 
-require_once EGW_SERVER_ROOT.'/phpgwapi/inc/horde/lib/core.php';
-require_once(EGW_SERVER_ROOT.'/phpgwapi/inc/horde/Horde/SyncML/State.php');
-
 /**
  * Addressbook - vCard parser
- *
  */
 class addressbook_vcal extends addressbook_bo
 {
@@ -75,7 +71,7 @@ class addressbook_vcal extends addressbook_bo
 			'REV'				=> array('modified'),
 			//set for Apple: 'X-ABSHOWAS'	=> array('fileas_type'),	// Horde vCard class uses uppercase prop-names!
 		);
-		
+
 	var $supportedFields;
 
 	/**
@@ -130,8 +126,8 @@ class addressbook_vcal extends addressbook_bo
 	* import a vard into addressbook
 	*
 	* @param string	$_vcard		the vcard
-	* @param int/string	$_abID=null		the internal addressbook id or !$_abID for a new enty
-	* @param boolean $merge=false	merge data with existing entry
+	* @param int/string	$_abID =null		the internal addressbook id or !$_abID for a new enty
+	* @param boolean $merge =false	merge data with existing entry
 	* @param string $charset  The encoding charset for $text. Defaults to
     *                         utf-8 for new format, iso-8859-1 for old format.
 	* @return int contact id
@@ -146,7 +142,7 @@ class addressbook_vcal extends addressbook_bo
 			{
 				if ($merge)
 				{
-					foreach ($contact as $key => $value)
+					foreach (array_keys($contact) as $key)
 					{
 						if (!empty($old_contact[$key]))
 						{
@@ -197,17 +193,13 @@ class addressbook_vcal extends addressbook_bo
 	* return a vcard
 	*
 	* @param int/string	$_id the id of the contact
-	* @param string $_charset='UTF-8' encoding of the vcard, default UTF-8
-	* @param boolean $extra_charset_attribute=true GroupDAV/CalDAV dont need the charset attribute and some clients have problems with it
+	* @param string $_charset ='UTF-8' encoding of the vcard, default UTF-8
+	* @param boolean $extra_charset_attribute =true GroupDAV/CalDAV dont need the charset attribute and some clients have problems with it
 	* @return string containing the vcard
 	*/
 	function getVCard($_id,$_charset='UTF-8',$extra_charset_attribute=true)
 	{
-		require_once(EGW_SERVER_ROOT.'/phpgwapi/inc/horde/Horde/iCalendar/vcard.php');
-
-		#Horde::logMessage("vCalAddressbook clientProperties:\n" . print_r($this->clientProperties, true), __FILE__, __LINE__, PEAR_LOG_DEBUG);
-
-		$vCard = new Horde_iCalendar_vcard($this->version);
+		$vCard = new Horde_Icalendar_Vcard($this->version);
 		$vCard->setAttribute('PRODID','-//EGroupware//NONSGML EGroupware Addressbook '.$GLOBALS['egw_info']['apps']['phpgwapi']['version'].'//'.
 			strtoupper($GLOBALS['egw_info']['user']['preferences']['common']['lang']));
 
@@ -326,6 +318,8 @@ class addressbook_vcal extends addressbook_bo
 								$options['ENCODING'] = 'BASE64';
 							}
 							$hasdata++;
+							// need to encode binary image, not done in Horde Icalendar
+							$value = base64_encode($value);
 						}
 						else
 						{
@@ -376,7 +370,7 @@ class addressbook_vcal extends addressbook_bo
 								}
 								elseif (!$extra_charset_attribute)
 								{
-									$options['ENCODING'] = '';
+									unset($options['ENCODING']);
 								}
 							}
 							$hasdata++;
@@ -455,7 +449,7 @@ class addressbook_vcal extends addressbook_bo
 								}
 								elseif (!$extra_charset_attribute)
 								{
-									$options['ENCODING'] = '';
+									unset($options['ENCODING']);
 								}
 							}
 							if ($vcardField == 'TEL' && $entry['tel_prefer'] &&
@@ -487,7 +481,11 @@ class addressbook_vcal extends addressbook_bo
 			$vCard->setAttribute($vcardField, $value, $options, true, $values);
 		}
 
-		$result = $vCard->exportvCalendar($_charset);
+		// current iOS 8.4 shows TEL;TYPE="WORK,VOICE":+49 123 4567890 as '"WORK'
+		// old (patched) Horde iCalendar, did not enclosed parameter in quotes
+		$result = preg_replace('/^TEL;TYPE="([^"]+)":/m', 'TEL;TYPE=$1:',
+			$vCard->exportvCalendar($_charset));
+
 		if ($this->log)
 		{
 			error_log(__FILE__.'['.__LINE__.'] '.__METHOD__ .
@@ -525,7 +523,7 @@ class addressbook_vcal extends addressbook_bo
 
 		if (is_array($_supportedFields)) $this->supportedFields = $_supportedFields;
 	}
-	
+
 	function setDatabaseFields($_databaseFields)
 	{
 		if (is_array($_databaseFields)) $this->databaseFields = $_databaseFields;
@@ -551,18 +549,22 @@ class addressbook_vcal extends addressbook_bo
 				array2string($_vcard)."\n",3,$this->logfile);
 		}
 
-		require_once(EGW_SERVER_ROOT.'/phpgwapi/inc/horde/Horde/iCalendar.php');
-
-		if(!($_vcard instanceof Horde_iCalendar))
+		if(!($_vcard instanceof Horde_Icalendar))
 		{
 			$container = false;
-			$vCard = Horde_iCalendar::newComponent('vcard', $container);
+			$vCard = Horde_Icalendar::newComponent('vcard', $container);
 
-			if (!$vCard->parsevCalendar($_vcard, 'VCARD', $charset))
+			if ($charset && $charset != 'utf-8')
+			{
+				$_vcard = translation::convert($_vcard, $charset, 'utf-8');
+			}
+			if (!$vCard->parsevCalendar($_vcard, 'VCARD'))
 			{
 				return False;
 			}
-		} else {
+		}
+		else
+		{
 			$vCard = $_vcard;
 		}
 		$vcardValues = $vCard->getAllAttributes();
@@ -933,6 +935,7 @@ class addressbook_vcal extends addressbook_bo
 					if (!empty($fieldName))
 					{
 						$value = trim($vcardValues[$vcardKey]['values'][$fieldKey]);
+
 						if ($pref_tel && (($vcardKey == $pref_tel) ||
 								($vcardValues[$vcardKey]['name'] == 'TEL') &&
 								($vcardValues[$vcardKey]['value'] == $vcardValues[$pref_tel]['value'])))
@@ -942,9 +945,9 @@ class addressbook_vcal extends addressbook_bo
 						switch($fieldName)
 						{
 							case 'bday':
-								$contact[$fieldName] = $vcardValues[$vcardKey]['values']['year'] .
-									'-' . $vcardValues[$vcardKey]['values']['month'] .
-									'-' . $vcardValues[$vcardKey]['values']['mday'];
+								$contact[$fieldName] = $vcardValues[$vcardKey]['value']['year'] .
+									'-' . $vcardValues[$vcardKey]['value']['month'] .
+									'-' . $vcardValues[$vcardKey]['value']['mday'];
 								break;
 
 							case 'private':
@@ -957,6 +960,10 @@ class addressbook_vcal extends addressbook_bo
 
 							case 'jpegphoto':
 								$contact[$fieldName] = $vcardValues[$vcardKey]['value'];
+								if(in_array($vcardValues[$vcardKey]['params']['ENCODING'],array('b','B','BASE64')))
+								{
+									$contact[$fieldName] = base64_decode($contact[$fieldName]);
+								}
 								break;
 
 							case 'note':
@@ -1053,14 +1060,12 @@ class addressbook_vcal extends addressbook_bo
 	 * return a groupVCard
 	 *
 	 * @param array $list values for 'list_uid', 'list_name', 'list_modified', 'members'
-	 * @param string $version='3.0' vcard version
+	 * @param string $version ='3.0' vcard version
 	 * @return string containing the vcard
 	 */
 	function getGroupVCard(array $list,$version='3.0')
 	{
-		require_once(EGW_SERVER_ROOT.'/phpgwapi/inc/horde/Horde/iCalendar/vcard.php');
-
-		$vCard = new Horde_iCalendar_vcard($version);
+		$vCard = new Horde_Icalendar_Vcard($version);
 		$vCard->setAttribute('PRODID','-//EGroupware//NONSGML EGroupware Addressbook '.$GLOBALS['egw_info']['apps']['phpgwapi']['version'].'//'.
 			strtoupper($GLOBALS['egw_info']['user']['preferences']['common']['lang']));
 
