@@ -6,7 +6,7 @@
  * @author Hadi Nategh	<hn-AT-stylite.de>
  * @copyright (c) 2008-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: app.js 49726 2014-12-05 12:38:20Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -48,6 +48,11 @@ app.classes.addressbook = AppJS.extend(
 	 */
 	et2_ready: function(et2, name)
 	{
+		// r49769 let's CRM view run under currentapp == "addressbook", which causes
+		// app.addressbook.et2_ready called before app.infolog.et2_ready and therefore
+		// app.addressbook.et2 would point to infolog template, if we not stop here
+		if (name.match(/^infolog|tracker\./)) return;
+
 		// call parent
 		this._super.apply(this, arguments);
 
@@ -66,6 +71,14 @@ app.classes.addressbook = AppJS.extend(
 						window.app.infolog = new window.app.classes.infolog();
 					}
 				}
+				// Call check value if the AB got opened with presets
+				if (window.location.href.match(/&presets\[email\]/g) && content.presets_fields)
+				{
+					for(var i=0;i< content.presets_fields.length;i++)
+					{
+						this.check_value(this.et2.getWidgetById(content.presets_fields),0);
+					}
+				}
 				break;
 		}
 
@@ -144,6 +157,57 @@ app.classes.addressbook = AppJS.extend(
 		}
 	},
 
+	/**
+	 * Open the calender to view the selected contacts
+	 * @param {egwAction} _action
+	 * @param {egwActionObject[]} _senders
+	 */
+	view_calendar: function(_action, _senders)
+	{
+		var extras = {
+			filter: 'all',
+			owner: []
+		};
+		var orgs = [];
+		for(var i = 0; i < _senders.length; i++)
+		{
+			// Remove UID prefix for just contact_id
+			var ids = _senders[i].id.split('::');
+			ids.shift();
+			ids = ids.join('::');
+
+			// Orgs need to get all the contact IDs first
+			if (ids.substr(0,9) == 'org_name:')
+			{
+				orgs.push(ids);
+			}
+			else
+			{
+				extras.owner.push('c'+ids);
+			}
+		}
+
+		if(orgs.length > 0)
+		{
+			// Get organisation contacts, then show infolog list
+			this.egw.json('addressbook.addressbook_ui.ajax_organisation_contacts',
+				[orgs],
+				function(contacts) {
+					for(var i = 0; i < contacts.length; i++)
+					{
+						extras.owner.push('c'+contacts[i]);
+					}
+					extras.owner = extras.owner.join(',');
+					this.egw.open('','calendar','list',extras,'calendar');
+				},this,true,this
+			).sendRequest();
+		}
+		else
+		{
+			extras.owner = extras.owner.join(',');
+			egw.open('', 'calendar', 'list', extras, 'calendar');
+		}
+	},
 	/**
 	 * Add appointment or show calendar for selected contacts, call default nm_action after some checks
 	 *
@@ -168,7 +232,7 @@ app.classes.addressbook = AppJS.extend(
 				ids += "c" + id[1] + ((i < _senders.length - 1) ? "," : "");
 			}
 			var extra = {};
-			extra[_action.data.url.indexOf('owner') > 0 ? 'owner' : 'participants'] = ids;
+			extra[_action.data && _action.data.url && _action.data.url.indexOf('owner') > 0 ? 'owner' : 'participants'] = ids;
 
 			// Use framework to add calendar entry
 			egw.open('','calendar','add',extra);
@@ -460,6 +524,9 @@ app.classes.addressbook = AppJS.extend(
 
 								// Set to new list so they can see it easily
 								lists.set_value(result);
+								// Call cahnge event manually after setting the value
+								// Not sure why our selectbox does not trigger change event
+								jQuery(lists.node).change();
 							}
 						}
 					).sendRequest(true);
@@ -568,14 +635,14 @@ app.classes.addressbook = AppJS.extend(
 		if(!index)
 		{
 			alert('Could not find index');
-			window.close();
+			egw(window).close();
 			return false;
 		}
 		var nm = index.widgetContainer.getWidgetById('nm');
 		if(!index)
 		{
 			window.opener.egw.message('Could not find list', 'error');
-			window.close();
+			egw(window).close();
 			return false;
 		}
 		// Reset filters first
@@ -593,13 +660,13 @@ app.classes.addressbook = AppJS.extend(
 	adb_mail_vcard: function(_action, _elems)
 	{
 		var app_registry = egw.link_get_registry('mail');
-		if (typeof app_registry['view'] == 'undefined') app_registry = egw.link_get_registry('felamimail');
 		var link = egw().link("/index.php","menuaction="+app_registry['add']['menuaction']);
 		for (var i = 0; i < _elems.length; i++)
 		{
 			var idToUse = _elems[i].id;
 			var idToUseArray = idToUse.split('::');
 			idToUse = idToUseArray[1];
+			link += "&preset[type][]="+encodeURIComponent("text/vcard; charset="+(egw.preference('vcard_charset', 'addressbook') || 'utf-8'));
 			link += "&preset[file][]="+encodeURIComponent("vfs://default/apps/addressbook/"+idToUse+"/.entry");
 		}
 		if (typeof app_registry['view'] != 'undefined' && typeof app_registry['view_popup'] != 'undefined' )
@@ -791,5 +858,15 @@ app.classes.addressbook = AppJS.extend(
 				}, this).sendRequest();
 				break;
 		}
+	},
+
+	/**
+	 * Get title in order to set it as document title
+	 * @returns {string}
+	 */
+	getWindowTitle: function()
+	{
+		var widget = this.et2.getWidgetById('n_fn');
+		if(widget) return widget.options.value;
 	}
 });
