@@ -7,7 +7,7 @@
  * @package admin
  * @copyright (c) 2010-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.admin_categories.inc.php 48751 2014-09-25 16:03:53Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -57,7 +57,7 @@ class admin_categories
 		{
 			throw new egw_exception_no_permission_admin();
 		}
-		if ($GLOBALS['egw']->acl->check('global_categories_access',1,'admin'))
+		if ($GLOBALS['egw']->acl->check('global_categorie',1,'admin'))
 		{
 			$GLOBALS['egw']->redirect_link('/index.php');
 		}
@@ -71,12 +71,12 @@ class admin_categories
 	{
 		if (is_null(self::$acl_search))
 		{
-			self::$acl_search = !$GLOBALS['egw']->acl->check('global_categories_access',2,'admin');
-			self::$acl_add    = !$GLOBALS['egw']->acl->check('global_categories_access',4,'admin');
-			self::$acl_view   = !$GLOBALS['egw']->acl->check('global_categories_access',8,'admin');
-			self::$acl_edit   = !$GLOBALS['egw']->acl->check('global_categories_access',16,'admin');
-			self::$acl_delete = !$GLOBALS['egw']->acl->check('global_categories_access',32,'admin');
-			self::$acl_add_sub= !$GLOBALS['egw']->acl->check('global_categories_access',64,'admin');
+			self::$acl_search = !$GLOBALS['egw']->acl->check('global_categorie',2,'admin');
+			self::$acl_add    = !$GLOBALS['egw']->acl->check('global_categorie',4,'admin');
+			self::$acl_view   = !$GLOBALS['egw']->acl->check('global_categorie',8,'admin');
+			self::$acl_edit   = !$GLOBALS['egw']->acl->check('global_categorie',16,'admin');
+			self::$acl_delete = !$GLOBALS['egw']->acl->check('global_categorie',32,'admin');
+			self::$acl_add_sub= !$GLOBALS['egw']->acl->check('global_categorie',64,'admin');
 		}
 	}
 
@@ -153,7 +153,39 @@ class admin_categories
 						// to reload the whole nextmatch instead of just the row
 						$data = $cats->id2name($content['id'],'data');
 						$change_color = ($data['color'] != $content['data']['color']);
-
+						// Nicely reload just the category window / iframe
+						if($change_color)
+						{
+							if(egw_json_response::isJSONResponse())
+							{
+								// Update category styles
+								egw_json_response::get()->apply('opener.egw.includeCSS',array(categories::css($refresh_app == 'admin' ? categories::GLOBAL_APPNAME : $refresh_app)));
+								if($this->appname != 'admin')
+								{
+									egw_json_response::get()->apply('opener.egw.show_preferences',array(
+										'cats',
+										$this->appname == 'admin' ? categories::GLOBAL_APPNAME : array($refresh_app)
+									));
+									$change_color = false;
+								}
+								else
+								{
+									categories::css($refresh_app == 'admin' ? categories::GLOBAL_APPNAME : $refresh_app);
+									egw_json_response::get()->apply('opener.app.admin.load',array(
+										egw_framework::link('/index.php', array(
+											'menuaction' => $this->list_link,
+											'appname' => $appname
+										)
+									)));
+								}
+							}
+							else
+							{
+								categories::css($refresh_app == 'admin' ? categories::GLOBAL_APPNAME : $refresh_app);
+								egw_framework::refresh_opener('', null, null);
+								return;
+							}
+						}
 						try {
 							$cats->edit($content);
 							$msg = lang('Category saved.');
@@ -177,7 +209,7 @@ class admin_categories
 					}
 					if ($button == 'save')
 					{
-						egw_framework::refresh_opener($msg, $refresh_app, $content['id'], $change_color ? 'edit' : 'update', 'admin');
+						egw_framework::refresh_opener($msg, $change_color ? null : $refresh_app, $change_color ? null : $content['id'], $change_color ? null : 'update', $this->appname);
 						egw_framework::window_close();
 					}
 					break;
@@ -187,8 +219,8 @@ class admin_categories
 					{
 						$cats->delete($content['id'],$delete_subs,!$delete_subs);
 						$msg = lang('Category deleted.');
-						// Refresh internally looks for template name, which starts with 'admin'
-						egw_framework::refresh_opener($msg, $refresh_app, $content['id'],'delete', 'admin');
+
+						egw_framework::refresh_opener($msg, $refresh_app, $content['id'],'delete', $this->appname);
 						egw_framework::window_close();
 						return;
 					}
@@ -199,8 +231,7 @@ class admin_categories
 					}
 					break;
 			}
-			// Refresh internally looks for template name, which starts with 'admin'
-			egw_framework::refresh_opener($msg, $refresh_app, $content['id'], $change_color ? 'edit' : 'update', 'admin');
+			egw_framework::refresh_opener($msg, $change_color ? null : $refresh_app, $content['id'], $change_color ? null : 'update', $this->appname);
 		}
 		$content['msg'] = $msg;
 		if(!$content['appname']) $content['appname'] = $appname;
@@ -365,29 +396,37 @@ class admin_categories
 			$row['subs'] = count($row['children']);
 
 			$row['class'] = 'level'.$row['level'];
-			if($row['owner'] > 0 && !$GLOBALS['egw_info']['user']['apps']['admin'] && $row['owner'] != $GLOBALS['egw_info']['user']['account_id'])
+			if($row['owner'][0] > 0 && !$GLOBALS['egw_info']['user']['apps']['admin'] && $row['owner'][0] != $GLOBALS['egw_info']['user']['account_id'])
 			{
 				$row['class'] .= ' rowNoEdit rowNoDelete ';
 			}
+			else if (!$GLOBALS['egw_info']['user']['apps']['admin'])
+			{
+				if(!$cats->check_perms(EGW_ACL_EDIT, $row['id']) || !self::$acl_edit)
+				{
+					$row['class'] .= ' rowNoEdit';
+				}
+				if(!$cats->check_perms(EGW_ACL_DELETE, $row['id']) || !self::$acl_delete ||
+					// Only admins can delete globals
+					$cats->is_global($row['id']) && !$GLOBALS['egw_info']['user']['apps']['admin'])
+				{
+					$row['class'] .= ' rowNoDelete';
+				}
+			}
 			// Can only edit (via context menu) categories for the selected app (backend restriction)
 			if($row['appname'] != $query['appname'] || (array_sum($row['owner']) > 0))
 			{
 				$row['class'] .= ' rowNoEdit ';
 			}
-			$readonlys["edit[$row[id]]"]   = !self::$acl_edit;
-			$readonlys["add[$row[id]]"]    = !self::$acl_add_sub;
-			$readonlys["delete[$row[id]]"] = !self::$acl_delete;
+			$readonlys['nm']["edit[$row[id]]"]   = !self::$acl_edit;
+			$readonlys['nm']["add[$row[id]]"]    = !self::$acl_add_sub;
+			$readonlys['nm']["delete[$row[id]]"] = !self::$acl_delete;
 		}
 		if (true) $rows = $count <= $query['num_rows'] ? array_values($rows) : array_slice($rows, $query['start'], $query['num_rows']);
 		// make appname available for actions
 		$rows['appname'] = $query['appname'];
 		$rows['edit_link'] = $this->edit_link;
 
-		// Send categories to make sure row colors stay up to date
-		// We use update types to prevent nm from doing unneeded calls to get_rows()
-		// TODO: figure out how to only send this if a category color has changed
-		$rows['sel_options']['cat_id'] = etemplate_widget_menupopup::typeOptions('select-cat', ',,,'.$query['appname'].',0');
-
 		// disable access column for global categories
 		if ($GLOBALS['egw_info']['flags']['currentapp'] == 'admin') $rows['no_access'] = true;
 
@@ -471,6 +510,7 @@ class admin_categories
 		}
 		elseif($content['nm']['action'])
 		{
+			$appname = $content['nm']['appname'];
 			// Old buttons
 			foreach(array('delete') as $button)
 			{
@@ -515,10 +555,12 @@ class admin_categories
 				{
 					$msg .= lang('%1 category(s) %2, %3 failed because of insufficent rights !!!',$success,$action_msg,$failed);
 				}
+				egw_framework::refresh_opener($msg, $this->appname);
+				$msg = '';
 			}
 		}
 		$content['msg'] = $msg;
-		$content['add_link']= $this->add_link.'&appname='.$appname;
+		$content['nm']['add_link']= egw_framework::link('/index.php','menuaction='.$this->add_link . '&cat_id=&appname='.$appname);
 		$content['edit_link']= $this->edit_link.'&appname='.$appname;
 		$content['owner'] = '';
 
@@ -557,8 +599,8 @@ class admin_categories
 
 		if($tmpl instanceof etemplate_widget_template)
 		{
-			// Send categories for row colors
-			$sel_options['cat_id'] = etemplate_widget_menupopup::typeOptions('select-cat', ',,,'.$appname.',0');
+			// Category styles
+			categories::css($appname);
 		}
 
 		$tmpl->exec($this->list_link,$content,$sel_options,$readonlys,array(
@@ -658,11 +700,18 @@ class admin_categories
 		switch($action)
 		{
 			case 'delete':
+				$action_msg = lang('deleted');
 				foreach($checked as $id)
 				{
-					$cats->delete($id,$settings == 'sub',$settings != 'sub');
-					$action_msg = lang('deleted');
-					$success++;
+					if($cats->check_perms(EGW_ACL_DELETE, $id, (boolean)$GLOBALS['egw_info']['user']['apps']['admin']))
+					{
+						$cats->delete($id,$settings == 'sub',$settings != 'sub');
+						$success++;
+					}
+					else
+					{
+						$failed++;
+					}
 				}
 				break;
 			case 'owner':
