@@ -7,7 +7,7 @@
  * @package admin
  * @copyright (c) 2007-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.admin_cmd.inc.php 46316 2014-03-18 15:10:57Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -22,6 +22,13 @@ abstract class admin_cmd
 	const pending    = 4;
 	const queued     = 5;	// command waits to be fetched from remote
 
+	/**
+	 * Status which stil need passwords available
+	 *
+	 * @var array
+	 */
+	static $require_pw_stati = array(self::scheduled,self::pending,self::queued);
+
 	/**
 	 * The status of the command, one of either scheduled, successful, failed or deleted
 	 *
@@ -124,7 +131,7 @@ abstract class admin_cmd
 
 		foreach($data as $name => $value)
 		{
-			$this->$name = $name == 'data' && !is_array($value) ? unserialize($value) : $value;
+			$this->$name = $name == 'data' && !is_array($value) ? json_php_unserialize($value) : $value;
 		}
 		//_debug_array($this); exit;
 	}
@@ -243,7 +250,7 @@ abstract class admin_cmd
 		}
 		//echo "got: $message\n";
 
-		if (($value = unserialize($message)) !== false && $message !== serialize(false))
+		if (($value = json_php_unserialize($message)) !== false && $message !== serialize(false))
 		{
 			$message = $value;
 		}
@@ -304,7 +311,10 @@ abstract class admin_cmd
 				$vars[$name] = $this->$name;
 			}
 		}
-		$vars['data'] = serialize($this->data);	// data is stored serialized
+		// data is stored serialized
+		// passwords are masked / removed, if we dont need them anymore
+		$vars['data'] = in_array($this->status, self::$require_pw_stati) ?
+			json_encode($this->data) : self::mask_passwords($this->data);
 
 		admin_cmd::$sql->init($vars);
 		if (admin_cmd::$sql->save() != 0)
@@ -329,6 +339,33 @@ abstract class admin_cmd
 		return true;
 	}
 
+	/**
+	 * Mask / remove passwords in $data
+	 *
+	 * @param string|array $data json or php-encoded string or array
+	 * @param boolean $return_serialized =true true: return json serialized string, false: return array
+	 * @return string|array see $return_serialized
+	 */
+	static function mask_passwords($data, $return_serialized=true)
+	{
+		if (!is_array($data))
+		{
+			$data = json_php_unserialize($data);
+		}
+		foreach($data as $key => &$value)
+		{
+			if (is_array($value))
+			{
+				$value = self::mask_passwords($value, false);
+			}
+			elseif (preg_match('/(pw|passwd_?\d*|(?<!change)password|db_pass)$/i', $key))
+			{
+				$value = str_repeat('*', strlen($value));
+			}
+		}
+		return $return_serialized ? json_encode($data) : $data;
+	}
+
 	/**
 	 * reading a command from the queue returning the comand object
 	 *
@@ -361,7 +398,7 @@ abstract class admin_cmd
 	{
 		if (isset($data['data']) && !is_array($data['data']))
 		{
-			$data['data'] = unserialize($data['data']);
+			$data['data'] = json_php_unserialize($data['data']);
 		}
 		if (!class_exists($class = $data['type']) || $class == 'admin_cmd')
 		{
@@ -838,6 +875,7 @@ abstract class admin_cmd
 					'status' => admin_cmd::failed,
 					'error'  => lang('Unknown command %1!',$job['type']),
 					'errno'  => 0,
+					'data'   => self::mask_passwords($job['data']),
 				));
 			}
 		}
