@@ -6,7 +6,7 @@
  * @package setup
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: hook_config_validate.inc.php 46706 2014-04-23 14:36:08Z ralfbecker $
+ * @version $Id$
  */
 
 /*
@@ -33,5 +33,8 @@ function vfs_image_dir($vfs_image_dir)
 	if ($vfs_image_dir != (string)$GLOBALS['egw_info']['server']['vfs_image_dir'])
 	{
 		common::delete_image_map();
+
+		// Set the global now, or the old value will get re-loaded
+		$GLOBALS['egw_info']['server']['vfs_image_dir'] = $vfs_image_dir;
 	}
 }
\ No newline at end of file
