@@ -4,9 +4,9 @@
  * @link http://www.egroupware.org
  * @package filemanager
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @copyright (c) 2013-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
+ * @copyright (c) 2013-16 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: app.js 51499 2015-01-30 11:30:03Z ralfbecker $
+ * @version $Id: app.js 56051 2016-05-06 07:58:37Z ralfbecker $
  */
 
 /**
@@ -95,14 +95,16 @@ app.classes.admin = AppJS.extend(
 							self._hide_navbar.call(self);
 						}
 					);
-
-					// Register app refresh now that iframe is available
-					register_app_refresh('admin',jQuery.proxy(this.refresh,this));
 				}
 				break;
 
 			case 'admin.categories.index':
 				break;
+			case 'admin.customfield_edit':
+				// Load settings appropriate to currently set type
+				var widget = _et2.widgetContainer.getWidgetById('cf_type');
+				this.cf_type_change(null,widget);
+				break;
 		}
 	},
 
@@ -113,7 +115,9 @@ app.classes.admin = AppJS.extend(
 	 */
 	load: function(_url)
 	{
-		if (this.iframe && this.iframe.getDOMNode().contentDocument.location.href.match(/menuaction=admin.admin_statistics.submit/))
+		if (this.iframe && this.iframe.getDOMNode().contentDocument.location.href
+			.match(/menuaction=admin.admin_statistics.submit/) &&
+			!_url.match(/statistics=(postpone|canceled|submitted)/))
 		{
 			this.egw.message(this.egw.lang('Please submit (or postpone) statistic first'), 'info');
 			return;	// do not allow to leave statistics submit
@@ -161,6 +165,28 @@ app.classes.admin = AppJS.extend(
 		switch(_app)
 		{
 			case 'admin':
+				// if iframe is used --> refresh it
+				var iframe_node = this.iframe ? this.iframe.getDOMNode() : undefined;
+				var iframe_url = iframe_node ? iframe_node.contentDocument.location.href : undefined;
+				if (_id && iframe_url != 'about:blank')
+				{
+					var refresh_done = false;
+					// Try for intelligent et2 refresh inside iframe
+					if(iframe_node && iframe_node.contentWindow && iframe_node.contentWindow.etemplate2)
+					{
+						var templates = iframe_node.contentWindow.etemplate2.getByApplication('admin');
+						for(var i = 0; i < templates.length; i++)
+						{
+							templates[i].refresh(_msg,_app,_id,_type);
+							refresh_done = true;
+						}
+					}
+					if (!refresh_done)	// --> reload iframe
+					{
+						this.load(iframe_url);
+					}
+					return false;	// --> no regular refresh
+				}
 				// invalidate client-side account-cache
 				this.egw.invalidate_account(_id, _type);
 				// group deleted, added or updated
@@ -198,73 +224,6 @@ app.classes.admin = AppJS.extend(
 		}
 	},
 
-	/**
-	 * Special handling for egw_refresh() in admin, to refresh the iframe when
-	 * the framework detects a simple refresh can be used (same URL).
-	 *
-	 * All parameters ignored.
-	 *
-	 * @param {string} _msg Message to display
-	 * @param {string} _app Application being refreshed, should be 'admin'
-	 * @param {string} _id Unique record ID.
-	 * @param {string} _type Type of refresh.  Either 'edit', 'delete',
-	 *	'add' or null
-	 */
-	refresh: function(_msg, _app, _id, _type)
-	{
-		var refresh_done = false;
-
-		// group deleted, added or updated
-		if (_app === 'admin' && _id < 0)
-		{
-			var tree = this.et2.getWidgetById('tree');
-			switch(_type)
-			{
-				case 'delete':
-					tree.deleteItem('/groups/'+_id, false);
-					break;
-
-				default:	// add, update, edit, null
-					tree.refreshItem('/groups');
-					break;
-			}
-		}
-
-		// Try for intelligent et2 refresh inside iframe
-		var node = _app && _id && this.iframe ? this.iframe.getDOMNode(this.iframe) : null;
-		if(node && node.contentWindow && node.contentWindow.etemplate2)
-		{
-			var templates = node.contentWindow.etemplate2.getByApplication('admin');
-			for(var i = 0; i < templates.length; i++)
-			{
-				templates[i].refresh(_msg,_app,_id,_type);
-				refresh_done = true;
-			}
-		}
-
-		// update of account list eg. from addressbook.edit
-		if(!refresh_done && _app == 'admin' && _id)
-		{
-			var templates = etemplate2.getByApplication('admin');
-			for(var i = 0; i < templates.length; i++)
-			{
-				templates[i].refresh(_msg,_app,_id,_type);
-				refresh_done = true;
-			}
-		}
-
-		// update iframe
-		if (!refresh_done && framework)
-		{
-			var app = framework.getApplicationByName(_app);
-
-			if (app && app.browser && app.browser.currentLocation)
-			{
-				this.linkHandler(app.browser.currentLocation);
-			}
-		}
-	},
-
 	/**
 	 * Hide navbar for idots template
 	 *
@@ -447,35 +406,33 @@ app.classes.admin = AppJS.extend(
 		// Determine which application we're running as
 		var app = egw.app_name();	// can be either admin or preferences!
 		if (app != 'admin') app = 'preferences';
+		// Get by ID, since this.et2 isn't always the ACL list
+		var et2 = etemplate2.getById('admin-acl').widgetContainer;
 		var className = app+'_acl';
 		var acl_rights = {};
 		var readonlys = {acl: {}};
 
 		// Select options are already here, just pull them and pass along
-		sel_options = jQuery.extend({}, this.et2.getArrayMgr('sel_options').data||{}, {
-			'apps': this.et2.getArrayMgr('sel_options').getEntry('filter2')
-		},sel_options);
-		// Remove 'All applications'
-		delete sel_options.apps[''];
+		sel_options = et2.getArrayMgr('sel_options').data||{};
 
 		// Some defaults
-		if(this.et2 && this.et2.getWidgetById('nm'))
+		if(et2 && et2.getWidgetById('nm'))
 		{
 			// This is which checkboxes are available for each app
-			var acl_rights = this.et2.getWidgetById('nm').getArrayMgr('content').getEntry('acl_rights')||{};
+			var acl_rights = et2.getWidgetById('nm').getArrayMgr('content').getEntry('acl_rights')||{};
 
 			if(!content.acl_appname)
 			{
 				// Pre-set appname to currently selected
-				content.acl_appname = this.et2.getWidgetById('filter2').getValue()||"";
+				content.acl_appname = et2.getWidgetById('filter2').getValue()||"";
 			}
 			if(!content.acl_account)
 			{
-				content.acl_account = this.et2.getWidgetById('nm').getArrayMgr('content').getEntry('account_id');
+				content.acl_account = et2.getWidgetById('nm').getArrayMgr('content').getEntry('account_id');
 			}
 			if(!content.acl_location)
 			{
-				content.acl_location = this.et2.getWidgetById('filter').getValue() == 'run' ? 'run' : null;
+				content.acl_location = et2.getWidgetById('filter').getValue() == 'run' ? 'run' : null;
 			}
 			// If no admin rights, change UI to not allow adding access to apps
 			if(content.acl_location == 'run' && !egw.user('apps')['admin'])
@@ -492,11 +449,17 @@ app.classes.admin = AppJS.extend(
 			else
 			{
 				// Restrict application selectbox to only apps that support ACL
-				sel_options.acl_appname = {};
+				sel_options.acl_appname = [];
 				for(var app in acl_rights)
 				{
-					sel_options.acl_appname[app] = this.egw.lang(app);
+					sel_options.acl_appname.push({value: app, label: this.egw.lang(app)});
 				}
+				// Sort list
+				sel_options.acl_appname.sort(function(a,b) {
+					if(a.label > b.label) return 1;
+					if(a.label < b.label) return -1;
+					return 0;
+				});
 			}
 
 		}
@@ -569,8 +532,10 @@ app.classes.admin = AppJS.extend(
 						var removed = [];
 
 						// Loop through all apps, remove the ones with no permission
-						for(var app in sel_options.apps)
+						for(var idx in sel_options.filter2)
 						{
+							var app = sel_options.filter2[idx].value || false;
+							if (!app) continue;
 							var run_id = app+":"+_value.acl_account+":run";
 							if(_value.apps.indexOf(app) < 0 && (content.apps.indexOf(app) >= 0 || content.apps.length == 0))
 							{
@@ -635,7 +600,9 @@ app.classes.admin = AppJS.extend(
 	{
 		// Avoid the window / framework / app and just refresh the etemplate
 		// Framework will try to refresh the opener
-		this.et2.getInstanceManager().refresh(_data.msg, this.appname,_data.ids,_data.type);
+		// Get by ID, since this.et2 isn't always the ACL list
+		var et2 = etemplate2.getById('admin-acl').widgetContainer;
+		et2.getInstanceManager().refresh(_data.msg, this.appname,_data.ids,_data.type);
 	},
 
 	/**
@@ -752,39 +719,50 @@ app.classes.admin = AppJS.extend(
 	 *
 	 * @param {DOM} form
 	 * @param {string} submit_url
-	 * @param {string} confirm_msg
-	 * @param {string} action own action, if called via window_set_timeout
-	 * @param {string} exec_id own exec_id
 	 * @return {boolean}
 	 */
-	submit_statistic: function(form,submit_url,confirm_msg,action,exec_id)
+	submit_statistic: function(form, submit_url)
 	{
-		if (submit_url) {
-			if (!confirm(confirm_msg)) return false;
-
-			var own_action = form.action;
-			var own_exec_id = form['etemplate_exec_id'].value;
-			var that = this;
-
-			// submit to own webserver
-			window.setTimeout(function() {
-				that.submit_statistic.call(this, form, '', '', own_action, own_exec_id);
-			},100);
-
+		var that = this;
+		var submit = function(_button)
+		{
 			// submit to egroupware.org
+			var method=form.method;
+			form.method='POST';
+			var action = form.action;
 			form.action=submit_url;
-			form['etemplate_exec_id'].value='';
+			var target = form.target;
 			form.target='_blank';
-		} else {
-			// submit to own webserver
-			form.action = action;
-			form['etemplate_exec_id'].value=exec_id;
-			form.target='';
-
 			form.submit();
-		}
 
-		return true;
+			// submit to own webserver
+			form.method=method;
+			form.action=action;
+			form.target=target;
+			that.et2.getInstanceManager().submit('submit');
+		};
+
+		// Safari does NOT allow to call form.submit() outside of onclick callback
+		// so we have to use browsers ugly synchron confirm
+		if (navigator.userAgent.match(/Safari/) && !navigator.userAgent.match(/Chrome/))
+		{
+			if (confirm(this.egw.lang('Submit displayed information?')))
+			{
+				submit();
+			}
+		}
+		else
+		{
+			et2_dialog.show_dialog(function(_button)
+			{
+				if (_button == et2_dialog.YES_BUTTON)
+				{
+					submit();
+				}
+			}, this.egw.lang('Submit displayed information?'), '', {},
+				et2_dialog.BUTTON_YES_NO, et2_dialog.QUESTION_MESSAGE, undefined, egw);
+		}
+		return false;
 	},
 
 	/**
@@ -828,5 +806,57 @@ app.classes.admin = AppJS.extend(
 			}
 		};
 		et2_dialog.show_dialog(delDialog_callBack,this.egw.lang("Are you sure you want to delete this category ?"),this.egw.lang("Delete"),{},_buttons,et2_dialog.WARNING_MESSAGE,null,'admin');
+	},
+
+	/**
+	 * Change handler for when you change the type of a custom field.
+	 * It toggles options / attributes as appropriate.
+	 * @param {event object} e
+	 * @param {widget object} widget
+	 */
+	cf_type_change: function(e,widget) {
+		var root = widget.getRoot();
+		var attributes = widget.getArrayMgr('content').getEntry('attributes['+widget.getValue()+']')||{};
+		root.getWidgetById('cf_values').set_statustext(widget.egw().lang(widget.getArrayMgr('content').getEntry('options['+widget.getValue()+']')||''));
+		jQuery(root.getWidgetById('cf_len').getDOMNode()).toggle(attributes.cf_len && true);
+		jQuery(root.getWidgetById('cf_rows').getDOMNode()).toggle(attributes.cf_rows && true);
+		jQuery(root.getWidgetById('cf_values').getParentDOMNode()).toggle(attributes.cf_values && true);
+	},
+	
+	/**
+	 * Activate none standard SMTP mail accounts for selected users
+	 * 
+	 * @param {type} _selected selected users
+	 * @todo remove under construction message
+	 */
+	emailadminActiveAccounts: function (_action, _selected){
+		
+		var menuaction = 'emailadmin.emailadmin_wizard.ajax_activeAccounts';
+		var accounts = [];
+		var msg1 = egw.lang('%1 accounts being activated',Object.keys(_selected).length);
+		
+		for (var i=0;i< Object.keys(_selected).length;i++)
+		{
+			accounts[i] = {id:_selected[i]['id'].split('::')[1],qouta:"", domain:"", status:_action.id == 'active'?_action.id:''};
+		}
+		var callbackDialog = function (btn){
+			if (btn === et2_dialog.YES_BUTTON)
+			{
+				// long task dialog for de/activation accounts
+				et2_dialog.long_task(function(_val, _resp){
+					if (_val && _resp.type !== 'error')
+					{
+						console.log(_val,_resp);
+					}
+					else
+					{
+
+					}
+				}, msg1, 'Mail Acounts Activation', menuaction, accounts, 'admin');
+			}
+		}
+		// confirmation dialog
+		et2_dialog.show_dialog(callbackDialog, egw.lang('Are you sure you want to %1 mail for selected accounts?', egw.lang(_action.id)), egw.lang('Active Mail Accounts'), {},
+			et2_dialog.BUTTON_YES_NO, et2_dialog.WARNING_MESSAGE, undefined, egw);
 	}
 });
