@@ -9,7 +9,7 @@
  * @author Klaus Leithoff <kl@stylite.de>
  * @author Philip Herbert <philip@knauber.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.calendar_activesync.inc.php 47869 2014-07-23 13:16:24Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -618,7 +618,7 @@ return array();	// temporary disabling meeting requests from calendar
 			'end' => 'endtime',
 		) as $key => $attr)
 		{
-			$event[$key] = egw_time::server2user($message->$attr);
+			if (isset($message->$attr)) $event[$key] = egw_time::server2user($message->$attr);
 		}
 		// copying strings
 		foreach(array(
