@@ -7,12 +7,10 @@
  * @package calendar
  * @subpackage groupdav
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @copyright (c) 2007-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @version $Id: class.calendar_groupdav.inc.php 51575 2015-02-04 11:19:31Z ralfbecker $
+ * @copyright (c) 2007-15 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
+ * @version $Id$
  */
 
-require_once EGW_SERVER_ROOT.'/phpgwapi/inc/horde/lib/core.php';
-
 /**
  * EGroupware: CalDAV / GroupDAV access: calendar handler
  *
@@ -35,7 +33,7 @@ class calendar_groupdav extends groupdav_handler
 	/**
 	 * vCalendar Instance for parsing
 	 *
-	 * @var Horde_iCalendar
+	 * @var Horde_Icalendar
 	 */
 	var $vCalendar;
 
@@ -95,7 +93,7 @@ class calendar_groupdav extends groupdav_handler
 		parent::__construct($app, $groupdav);
 
 		$this->bo = new calendar_boupdate();
-		$this->vCalendar = new Horde_iCalendar;
+		$this->vCalendar = new Horde_Icalendar;
 
 		// since 1.9.003 we allow clients to specify the URL when creating a new event, as specified by CalDAV
 		if (version_compare($GLOBALS['egw_info']['apps']['calendar']['version'], '1.9.003', '>='))
@@ -137,7 +135,7 @@ class calendar_groupdav extends groupdav_handler
 	 * @param array &$options
 	 * @param array &$files
 	 * @param int $user account_id
-	 * @param string $id=''
+	 * @param string $id =''
 	 * @return mixed boolean true on success, false on failure or string with http status (eg. '404 Not Found')
 	 */
 	function propfind($path,&$options,&$files,$user,$id='')
@@ -152,6 +150,14 @@ class calendar_groupdav extends groupdav_handler
 			return $this->free_busy_report($path, $options, $user);
 		}
 
+		if (isset($_GET['download']))
+		{
+			$this->groupdav->propfind_options['props'] = array(array(
+				'xmlns' => groupdav::CALDAV,
+				'name'  => 'calendar-data',
+			));
+		}
+
 		// ToDo: add parameter to only return id & etag
 		$filter = array(
 			'users' => $user,
@@ -159,6 +165,7 @@ class calendar_groupdav extends groupdav_handler
 			'daywise' => false,
 			'date_format' => 'server',
 			'no_total' => true,	// we need no total number of rows (saves extra query)
+			'cfs' => array(),	// return custom-fields, as we use them to store X- attributes
 		);
 		foreach(array(
 			'start' => $GLOBALS['egw_info']['user']['preferences']['groupdav']['calendar-past-limit'],
@@ -246,15 +253,66 @@ class calendar_groupdav extends groupdav_handler
 			// return iterator, calling ourself to return result in chunks
 			$files['files'] = new groupdav_propfind_iterator($this,$path,$filter,$files['files']);
 		}
+		if (isset($_GET['download']))
+		{
+			$this->output_vcalendar($files['files']);
+		}
 		return true;
 	}
 
+	/**
+	 * Download whole calendar as big ics file
+	 *
+	 * @param iterator|array $files
+	 */
+	function output_vcalendar($files)
+	{
+		// todo ETag logic with CTag to not download unchanged calendar again
+		html::content_header('calendar.ics', 'text/calendar');
+
+		$n = 0;
+		foreach($files as $file)
+		{
+			if (!$n++) continue;	// first entry is collection itself
+
+			$icalendar = $file['props']['calendar-data']['val'];
+			if (($start = strpos($icalendar, 'BEGIN:VEVENT')) !== false &&
+				($end = strrpos($icalendar, 'END:VCALENDAR')) !== false)
+			{
+				if ($n === 2)
+				{
+					// skip X-CALENDARSERVER-ACCESS:CONFIDENTIAL, as it is on VCALENDAR not VEVENT level
+					if (($x_calendarserver_access = strpos($icalendar, 'X-CALENDARSERVER-ACCESS:')) !== false)
+					{
+						echo substr($icalendar, 0, $x_calendarserver_access);
+					}
+					// skip timezones, as we would need to collect them from all events
+					// ans most clients understand timezone by reference anyway
+					elseif (($tz = strpos($icalendar, 'BEGIN:VTIMEZONE')) !== false)
+					{
+						echo substr($icalendar, 0, $tz);
+					}
+					else
+					{
+						echo substr($icalendar, 0, $start);
+					}
+				}
+				echo substr($icalendar, $start, $end-$start);
+			}
+		}
+		if ($icalendar && $end)
+		{
+			echo "END:VCALENDAR\n";
+		}
+		common::egw_exit();
+	}
+
 	/**
 	 * Callback for profind interator
 	 *
 	 * @param string $path
 	 * @param array $filter
-	 * @param array|boolean $start=false false=return all or array(start,num)
+	 * @param array|boolean $start =false false=return all or array(start,num)
 	 * @return array with "files" array with values for keys path and props
 	 */
 	function propfind_callback($path,array $filter,$start=false)
@@ -291,6 +349,7 @@ class calendar_groupdav extends groupdav_handler
 					$files[] = array('path' => $path.urldecode($this->get_path($event)));
 					continue;
 				}
+				$schedule_tag = null;
 				$etag = $this->get_etag($event, $schedule_tag);
 
 				//header('X-EGROUPWARE-EVENT-'.$event['id'].': '.$event['title'].': '.date('Y-m-d H:i:s',$event['start']).' - '.date('Y-m-d H:i:s',$event['end']));
@@ -557,7 +616,7 @@ class calendar_groupdav extends groupdav_handler
 	 *
 	 * @param array &$options
 	 * @param int $id
-	 * @param int $user=null account_id
+	 * @param int $user =null account_id
 	 * @return mixed boolean true on success, false on failure or string with http status (eg. '404 Not Found')
 	 */
 	function get(&$options,$id,$user=null)
@@ -570,6 +629,7 @@ class calendar_groupdav extends groupdav_handler
 		$options['data'] = $this->iCal($event, $user, strpos($options['path'], '/inbox/') !== false ? 'REQUEST' : null);
 		$options['mimetype'] = 'text/calendar; charset=utf-8';
 		header('Content-Encoding: identity');
+		$schedule_tag = null;
 		header('ETag: "'.$this->get_etag($event, $schedule_tag).'"');
 		if ($this->use_schedule_tag)
 		{
@@ -584,9 +644,9 @@ class calendar_groupdav extends groupdav_handler
 	 * Taking into account virtual an real exceptions for recuring events
 	 *
 	 * @param array $event
-	 * @param int $user=null account_id of calendar to display
-	 * @param string $method=null eg. 'PUBLISH' for inbox, nothing anywhere else
-	 * @param boolean|array $expand=false true or array with values for 'start', 'end' to expand recurrences
+	 * @param int $user =null account_id of calendar to display
+	 * @param string $method =null eg. 'PUBLISH' for inbox, nothing anywhere else
+	 * @param boolean|array $expand =false true or array with values for 'start', 'end' to expand recurrences
 	 * @return string
 	 */
 	private function iCal(array $event,$user=null, $method=null, $expand=false)
@@ -628,9 +688,9 @@ class calendar_groupdav extends groupdav_handler
 	 * Maybe that should be part of calendar_bo
 	 *
 	 * @param string $uid UID
-	 * @param calendar_bo $bo=null calendar_bo object to reuse for search call
-	 * @param boolean|array $expand=false true or array with values for 'start', 'end' to expand recurrences
-	 * @param int $user=null account_id of calendar to display, to remove master, if current user does not participate in
+	 * @param calendar_bo $bo =null calendar_bo object to reuse for search call
+	 * @param boolean|array $expand =false true or array with values for 'start', 'end' to expand recurrences
+	 * @param int $user =null account_id of calendar to display, to remove master, if current user does not participate in
 	 * @return array
 	 */
 	private static function &get_series($uid,calendar_bo $bo=null, $expand=false, $user=null)
@@ -642,13 +702,18 @@ class calendar_groupdav extends groupdav_handler
 			'filter' => 'owner',  // return all possible entries
 			'daywise' => false,
 			'date_format' => 'server',
+			'cfs' => array(),	// read cfs as we use them to store X- attributes
 		);
 		if (is_array($expand)) $params += $expand;
 
-		$events =& $bo->search($params);
+		if (!($events =& $bo->search($params)))
+		{
+			return array();
+		}
 
 		// find master, which is not always first event, eg. when first event is an exception
 		$master = null;
+		$exceptions = array();
 		foreach($events as $k => &$recurrence)
 		{
 			if ($recurrence['recur_type'])
@@ -660,20 +725,18 @@ class calendar_groupdav extends groupdav_handler
 			}
 		}
 		// if recurring event starts in future behind horizont, nothing will be returned by bo::search()
-		if (!isset($master) && !($master = $bo->read($uid)))
-		{
-			return array();
-		}
+		if (!isset($master)) $master = $bo->read($uid);
+
 		foreach($events as $k => &$recurrence)
 		{
 			//error_log(__FILE__.'['.__LINE__.'] '.__METHOD__."($uid)[$k]:" . array2string($recurrence));
-			if ($recurrence['id'] != $master['id'])	// real exception
+			if (!$master || $recurrence['id'] != $master['id'])	// real exception
 			{
 				// user is NOT participating in this exception
 				if ($user && !self::isParticipant($recurrence, $user))
 				{
 					// if he is NOT in master, delete this exception
-					if (!self::isParticipant($master, $user))
+					if (!$master || !self::isParticipant($master, $user))
 					{
 						unset($events[$k]);
 						continue;
@@ -692,8 +755,17 @@ class calendar_groupdav extends groupdav_handler
 				}
 				continue;	// nothing to change
 			}
+			// alarms are reported on recurrences --> move them to master
+			if ($master)
+			{
+				foreach($recurrence['alarm'] as $alarm)
+				{
+					$master['alarm'][] = $alarm;
+				}
+				$recurrence['alarm'] = array();
+			}
 			// now we need to check if this recurrence is an exception
-			if (!$expand && $master['participants'] == $recurrence['participants'])
+			if (!$expand && $master && $master['participants'] == $recurrence['participants'])
 			{
 				//error_log('NO exception: '.array2string($recurrence));
 				unset($events[$k]);	// no exception --> remove it
@@ -702,12 +774,12 @@ class calendar_groupdav extends groupdav_handler
 			// this is a virtual exception now (no extra event/cal_id in DB)
 			//error_log('virtual exception: '.array2string($recurrence));
 			$recurrence['recurrence'] = $recurrence['start'];
-			$recurrence['reference'] = $master['id'];
+			if ($master) $recurrence['reference'] = $master['id'];
 			$recurrence['recur_type'] = MCAL_RECUR_NONE;	// is set, as this is a copy of the master
 			// not for included exceptions (Lightning): $master['recur_exception'][] = $recurrence['start'];
 		}
 		// only add master if we are not expanding and current user participates in master (and not just some exceptions)
-		if (!$expand && (!$user || self::isParticipant($master, $user)))
+		if (!$expand && $master && (!$user || self::isParticipant($master, $user)))
 		{
 			$events = array_merge(array($master), $events);
 		}
@@ -733,8 +805,8 @@ class calendar_groupdav extends groupdav_handler
 	 *
 	 * @param array &$options
 	 * @param int $id
-	 * @param int $user=null account_id of owner, default null
-	 * @param string $prefix=null user prefix from path (eg. /ralf from /ralf/addressbook)
+	 * @param int $user =null account_id of owner, default null
+	 * @param string $prefix =null user prefix from path (eg. /ralf from /ralf/addressbook)
 	 * @return mixed boolean true on success, false on failure or string with http status (eg. '404 Not Found')
 	 */
 	function put(&$options,$id,$user=null,$prefix=null)
@@ -813,6 +885,7 @@ class calendar_groupdav extends groupdav_handler
 			{
 				$schedule_tag_match = $_SERVER['HTTP_IF_SCHEDULE_TAG_MATCH'];
 				if ($schedule_tag_match[0] == '"') $schedule_tag_match = substr($schedule_tag_match, 1, -1);
+				$schedule_tag = null;
 				$this->get_etag($oldEvent, $schedule_tag);
 
 				if ($schedule_tag_match !== $schedule_tag)
@@ -828,7 +901,9 @@ class calendar_groupdav extends groupdav_handler
 			{
 				$user_and_memberships = $GLOBALS['egw']->accounts->memberships($user, true);
 				$user_and_memberships[] = $user;
-				if (!array_intersect(array_keys($oldEvent['participants']), $user_and_memberships))
+				if (!array_intersect(array_keys($oldEvent['participants']), $user_and_memberships) &&
+					// above can be true, if current user is not in master but just a recurrence
+					(!$oldEvent['recur_type'] || !($series = self::get_series($oldEvent['uid'], $this->bo))))
 				{
 					if ($this->debug) error_log(__METHOD__."(,,$user) user $user is NOT an attendee!");
 					return '403 Forbidden';
@@ -840,7 +915,7 @@ class calendar_groupdav extends groupdav_handler
 					foreach($events as $n => $event)
 					{
 						// for recurrances of event series, we need to read correct recurrence (or if series master is no first event)
-						if ($event['recurrence'] || $n && !$event['recurrence'])
+						if ($event['recurrence'] || $n && !$event['recurrence'] || isset($series))
 						{
 							// first try reading (virtual and real) exceptions
 							if (!isset($series))
@@ -853,7 +928,7 @@ class calendar_groupdav extends groupdav_handler
 								if ($oldEvent['recurrence'] == $event['recurrence']) break;
 							}
 							// if no exception found, check if it might be just a recurrence (no exception)
-							if ($oldEvent['recurrence'] != $event['recurrence'])
+							if ($event['recurrence'] && $oldEvent['recurrence'] != $event['recurrence'])
 							{
 								if (!($oldEvent = $this->bo->read($eventId, $event['recurrence'], true)) ||
 									// virtual exceptions have recurrence=0 and recur_date=recurrence (series master or real exceptions have recurence=0)
@@ -885,7 +960,8 @@ class calendar_groupdav extends groupdav_handler
 						// import alarms, if given and changed
 						if ((array)$event['alarm'] !== (array)$oldEvent['alarm'])
 						{
-							$modified += $this->sync_alarms($oldEvent['id'], (array)$event['alarm'], (array)$oldEvent['alarm'], $user, $event['start']);
+							$event['id'] = $oldEvent['id'];
+							$modified += $handler->sync_alarms($event, (array)$oldEvent['alarm'], $user);
 						}
 					}
 					if (!$modified)	// NO modififictions, or none we understood --> log it and return Ok: "204 No Content"
@@ -953,59 +1029,12 @@ class calendar_groupdav extends groupdav_handler
 		return $retval;
 	}
 
-	/**
-	 * Sync alarms of current user: add alarms added on client and remove the ones removed
-	 *
-	 * @param int $cal_id of event to set alarms
-	 * @param array $alarms
-	 * @param array $old_alarms
-	 * @param int $user account_id of user to create alarm for
-	 * @param int $start start-time of event
-	 * @ToDo store other alarm properties like: ACTION, DESCRIPTION, X-WR-ALARMUID
-	 * @return int number of modified alarms
-	 */
-	private function sync_alarms($cal_id, array $alarms, array $old_alarms, $user, $start)
-	{
-		if ($this->debug) error_log(__METHOD__."($cal_id, ".array2string($alarms).', '.array2string($old_alarms).", $user, $start)");
-		$modified = 0;
-		foreach($alarms as $alarm)
-		{
-			if ($alarm['owner'] != $this->user) continue;	// only import alarms of current user
-
-			// check if alarm is already stored or from other users
-			foreach($old_alarms as $id => $old_alarm)
-			{
-				if ($old_alarm['owner'] != $user || $alarm['offset'] == $old_alarm['offset'])
-				{
-					unset($old_alarms[$id]);	// remove alarms of other user, or already existing alarms
-				}
-			}
-			// alarm not found --> add it
-			if ($alarm['offset'] != $old_alarm['offset'] || $old_alarm['owner'] != $user)
-			{
-				$alarm['owner'] = $user;
-				$alarm['time'] = $start - $alarm['offset'];
-				if ($this->debug) error_log(__METHOD__."() adding new alarm from client ".array2string($alarm));
-				$this->bo->save_alarm($cal_id, $alarm);
-				++$modified;
-			}
-		}
-		// remove all old alarms left from current user
-		foreach($old_alarms as $id => $old_alarm)
-		{
-			if ($this->debug) error_log(__METHOD__."() deleting alarm '$id' deleted on client ".array2string($old_alarm));
-			$this->bo->delete_alarm($id);
-			++$modified;
-		}
-		return $modified;
-	}
-
 	/**
 	 * Handle post request for a schedule entry
 	 *
 	 * @param array &$options
 	 * @param int $id
-	 * @param int $user=null account_id of owner, default null
+	 * @param int $user =null account_id of owner, default null
 	 * @return mixed boolean true on success, false on failure or string with http status (eg. '404 Not Found')
 	 */
 	function post(&$options,$id,$user=null)
@@ -1059,8 +1088,8 @@ class calendar_groupdav extends groupdav_handler
 			$handler = $this->_get_handler();
 			if (($foundEvents = $handler->search($vCalendar, null, false, $charset)))
 			{
-				$eventId = array_shift($foundEvents);
-				list($eventId) = explode(':', $eventId);
+				$id = array_shift($foundEvents);
+				list($eventId) = explode(':', $id);
 
 				if (!($cal_id = $handler->importVCal($vCalendar, $eventId, null,
 					false, 0, $this->groupdav->current_user_principal, $user, $charset)))
@@ -1084,8 +1113,9 @@ class calendar_groupdav extends groupdav_handler
 	 */
 	protected function outbox_freebusy_request($ical, $charset, $user, array &$options)
 	{
-		include_once EGW_SERVER_ROOT.'/phpgwapi/inc/horde/lib/core.php';
-		$vcal = new Horde_iCalendar();
+		unset($options);	// not used, but required by function signature
+
+		$vcal = new Horde_Icalendar();
 		if (!$vcal->parsevCalendar($ical, 'VCALENDAR', $charset))
 		{
 			return '400 Bad request';
@@ -1098,7 +1128,7 @@ class calendar_groupdav extends groupdav_handler
 		$handler->setSupportedFields('groupdav');
 		$handler->calendarOwner = $handler->user = 0;	// to NOT default owner/organizer to something
 		if (!($component = $vcal->getComponent(0)) ||
-			!($event = $handler->vevent2egw($component, $version, $handler->supportedFields, $this->groupdav->current_user_principal, 'Horde_iCalendar_vfreebusy')))
+			!($event = $handler->vevent2egw($component, $version, $handler->supportedFields, $this->groupdav->current_user_principal, 'Horde_Icalendar_Vfreebusy')))
 		{
 			return '400 Bad request';
 		}
@@ -1111,7 +1141,7 @@ class calendar_groupdav extends groupdav_handler
 		$organizer = $component->getAttribute('ORGANIZER');
 		$attendees = (array)$component->getAttribute('ATTENDEE');
 		// X-CALENDARSERVER-MASK-UID specifies to exclude given event from busy-time
-		$mask_uid = $component->getAttribute('X-CALENDARSERVER-MASK-UID');
+		$mask_uid = $component->getAttributeDefault('X-CALENDARSERVER-MASK-UID', null);
 
 		header('Content-type: text/xml; charset=UTF-8');
 
@@ -1121,7 +1151,7 @@ class calendar_groupdav extends groupdav_handler
 		$xml->startDocument('1.0', 'UTF-8');
 		$xml->startElementNs('C', 'schedule-response', groupdav::CALDAV);
 
-		foreach($event['participants'] as $uid => $status)
+		foreach(array_keys($event['participants']) as $uid)
 		{
 			$xml->startElementNs('C', 'response', null);
 
@@ -1158,6 +1188,7 @@ class calendar_groupdav extends groupdav_handler
 	 */
 	function free_busy_report($path,$options,$user)
 	{
+		unset($path);	// unused, but required by function signature
 		if (!$this->bo->check_perms(EGW_ACL_FREEBUSY, 0, $user))
 		{
 			return '403 Forbidden';
@@ -1183,7 +1214,7 @@ class calendar_groupdav extends groupdav_handler
 	 * Reimplemented to add read-free-busy and schedule-deliver privilege
 	 *
 	 * @param string $path path of collection
-	 * @param int $user=null owner of the collection, default current user
+	 * @param int $user =null owner of the collection, default current user
 	 * @return array with privileges
 	 */
 	public function current_user_privileges($path, $user=null)
@@ -1309,7 +1340,7 @@ class calendar_groupdav extends groupdav_handler
 				// check if user is a participant or one of the groups he is a member of --> reject the meeting request
 				$ret = '403 Forbidden';
 				$memberships = $GLOBALS['egw']->accounts->memberships($this->bo->user, true);
-				foreach($event['participants'] as $uid => $status)
+				foreach(array_keys($event['participants']) as $uid)
 				{
 					if ($this->bo->user == $uid || in_array($uid, $memberships))
 					{
@@ -1348,7 +1379,9 @@ class calendar_groupdav extends groupdav_handler
 		$event = $this->bo->read(array($column => $id, 'cal_deleted IS NULL', 'cal_reference=0'), null, true, 'server');
 		if ($event) $event = array_shift($event);	// read with array as 1. param, returns an array of events!
 
-		if (!($retval = $this->bo->check_perms(EGW_ACL_FREEBUSY,$event, 0, 'server')))
+		if (!($retval = $this->bo->check_perms(EGW_ACL_FREEBUSY,$event, 0, 'server')) &&
+			// above can be true, if current user is not in master but just a recurrence
+			(!$event['recur_type'] || !($events = self::get_series($event['uid'], $this->bo))))
 		{
 			if ($this->debug > 0) error_log(__METHOD__."($id) no READ or FREEBUSY rights returning ".array2string($retval));
 			return $retval;
@@ -1394,7 +1427,8 @@ class calendar_groupdav extends groupdav_handler
 	/**
 	 * Get the etag for an entry
 	 *
-	 * @param array|int $event array with event or cal_id
+	 * @param array|int $entry array with event or cal_id
+	 * @param string $schedule_tag =null on return schedule-tag
 	 * @return string|boolean string with etag or false
 	 */
 	function get_etag($entry, &$schedule_tag=null)
@@ -1413,10 +1447,11 @@ class calendar_groupdav extends groupdav_handler
 	 * @param int|array $entry id or array of new created entry
 	 * @param string $path
 	 * @param int|string $retval
-	 * @param boolean $path_attr_is_name=true true: path_attr is ca(l|rd)dav_name, false: id (GroupDAV needs Location header)
+	 * @param boolean $path_attr_is_name =true true: path_attr is ca(l|rd)dav_name, false: id (GroupDAV needs Location header)
 	 */
 	function put_response_headers($entry, $path, $retval, $path_attr_is_name=true)
 	{
+		$schedule_tag = null;
 		$etag = $this->get_etag($entry, $schedule_tag);
 
 		if ($this->use_schedule_tag)
@@ -1446,15 +1481,16 @@ class calendar_groupdav extends groupdav_handler
 	/**
 	 * Add extra properties for calendar collections
 	 *
-	 * @param array $props=array() regular props by the groupdav handler
+	 * @param array $props regular props by the groupdav handler
 	 * @param string $displayname
-	 * @param string $base_uri=null base url of handler
-	 * @param int $user=null account_id of owner of current collection
-	 * @param string $path=null path of the collection
+	 * @param string $base_uri =null base url of handler
+	 * @param int $user =null account_id of owner of current collection
+	 * @param string $path =null path of the collection
 	 * @return array
 	 */
-	public function extra_properties(array $props=array(), $displayname, $base_uri=null, $user=null, $path=null)
+	public function extra_properties(array $props, $displayname, $base_uri=null, $user=null, $path=null)
 	{
+		unset($base_uri);	// unused, but required by function signature
 		if (!isset($props['calendar-description']))
 		{
 			// default calendar description: can be overwritten via PROPPATCH, in which case it's already set
@@ -1526,8 +1562,8 @@ class calendar_groupdav extends groupdav_handler
 	function get_shared()
 	{
 		$shared = array();
-		$calendar_home_set = $GLOBALS['egw_info']['user']['preferences']['groupdav']['calendar-home-set'];
-		$calendar_home_set = $calendar_home_set ? explode(',',$calendar_home_set) : array();
+		$pref = $GLOBALS['egw_info']['user']['preferences']['groupdav']['calendar-home-set'];
+		$calendar_home_set = $pref ? explode(',', $pref) : array();
 		// replace symbolic id's with real nummeric id's
 		foreach(array(
 			'G' => $GLOBALS['egw_info']['user']['account_primary_group'],
