@@ -7,7 +7,7 @@
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright (c) 2004-14 by RalfBecker-At-outdoor-training.de
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.calendar_hooks.inc.php 49588 2014-11-26 13:59:31Z hnategh $
+ * @version $Id$
  */
 
 /**
@@ -45,8 +45,10 @@ class calendar_hooks
 			'mime' => array(
 				'text/calendar' => array(
 					'menuaction' => 'calendar.calendar_uiforms.edit',
-					'mime_id' => 'ical_vfs',
+					'mime_data' => 'ical_data',
+					'mime_url' => 'ical_url',
 					'mime_popup' => '750x400',
+					'mime_target' => '_blank'
 				),
 			),
 			'merge' => true,
@@ -67,70 +69,6 @@ class calendar_hooks
 		return $GLOBALS['egw_info']['server']['calendar_export_limit'];
 	}
 
-	/**
-	 * Draw calendar part of home
-	 */
-	static function home()
-	{
-		if($GLOBALS['egw_info']['user']['preferences']['calendar']['mainscreen_showevents'])
-		{
-			$GLOBALS['egw']->translation->add_app('calendar');
-
-			$save_app_header = $GLOBALS['egw_info']['flags']['app_header'];
-
-			if ($GLOBALS['egw_info']['user']['preferences']['calendar']['mainscreen_showevents'] == 'listview')
-			{
-				if (!file_exists(EGW_SERVER_ROOT.($et_css_file ='/etemplate/templates/'.$GLOBALS['egw_info']['user']['preferences']['common']['template_set'].'/app.css')))
-				{
-					$et_css_file = '/etemplate/templates/default/app.css';
-				}
-				$content =& ExecMethod('calendar.calendar_uilist.home');
-			}
-			else
-			{
-				unset($et_css_file);
-				$content =& ExecMethod('calendar.calendar_uiviews.home');
-			}
-			$portalbox =& CreateObject('phpgwapi.listbox',array(
-				'title'	=> $GLOBALS['egw_info']['flags']['app_header'],
-				'primary'	=> $GLOBALS['egw_info']['theme']['navbar_bg'],
-				'secondary'	=> $GLOBALS['egw_info']['theme']['navbar_bg'],
-				'tertiary'	=> $GLOBALS['egw_info']['theme']['navbar_bg'],
-				'width'	=> '100%',
-				'outerborderwidth'	=> '0',
-				'header_background_image'	=> $GLOBALS['egw']->common->image('phpgwapi/templates/default','bg_filler')
-			));
-			$GLOBALS['egw_info']['flags']['app_header'] = $save_app_header;
-			unset($save_app_header);
-
-			$GLOBALS['portal_order'][] = $app_id = $GLOBALS['egw']->applications->name2id('calendar');
-			foreach(array('up','down','close','question','edit') as $key)
-			{
-				$portalbox->set_controls($key,Array('url' => '/set_box.php', 'app' => $app_id));
-			}
-			$portalbox->data = Array();
-
-			if (!file_exists(EGW_SERVER_ROOT.($css_file ='/calendar/templates/'.$GLOBALS['egw_info']['user']['preferences']['common']['template_set'].'/app.css')))
-			{
-				$css_file = '/calendar/templates/default/app.css';
-			}
-			echo '
-<!-- BEGIN Calendar info -->
-<style type="text/css">
-<!--';
-			if ($et_css_file)	// listview
-			{
-				echo '
-@import url('.$GLOBALS['egw_info']['server']['webserver_url'].$et_css_file.');';
-			}
-			echo '
-@import url('.$GLOBALS['egw_info']['server']['webserver_url'].$css_file.');
--->
-</style>
-'.$portalbox->draw($content)."\n".'<!-- END Calendar info -->'."\n";
-		}
-	}
-
 	/**
 	 * Entries for calendar's admin menu
 	 */
@@ -138,7 +76,7 @@ class calendar_hooks
 	{
 		$file = Array(
 			'Site Configuration' => egw::link('/index.php','menuaction=admin.uiconfig.index&appname=calendar'),
-			'Custom fields' => egw::link('/index.php','menuaction=admin.customfields.edit&appname=calendar'),
+			'Custom fields' => egw::link('/index.php','menuaction=admin.customfields.index&appname=calendar'),
 			'Calendar Holiday Management' => egw::link('/index.php','menuaction=calendar.uiholiday.admin'),
 			'Global Categories' => egw::link('/index.php','menuaction=admin.admin_categories.index&appname=calendar'),
 			'Category ACL' => egw::link('/index.php','menuaction=calendar.calendar_uiforms.cat_acl'),
@@ -232,7 +170,7 @@ class calendar_hooks
 
 		for ($i = 2; $i <= 9; ++$i)
 		{
-			$muliple_weeks[$i.' weeks'] = lang('%1 weeks',$i);
+			$muliple_weeks[$i] = lang('%1 weeks',$i);
 		}
 
 		$intervals = array(
@@ -315,7 +253,7 @@ class calendar_hooks
 				'help'   => 'How many weeks should the multiple week view show?',
 				'xmlrpc' => True,
 				'admin'  => False,
-				'forced'=> 3,
+				'default'=> 2,
 			),
 			'weekdaystarts' => array(
 				'type'   => 'select',
@@ -405,10 +343,7 @@ class calendar_hooks
 				'admin'  => False,
 				'forced' => 'user',
 			),
-			//ATM:Disable the use_time_grid preference
-			//@TODO: the whole use_time_grid preference should be removed
-			// after we decided that is not neccessary to have it at all
-			/*'use_time_grid' => array(
+			'use_time_grid' => array(
 				'type'   => 'select',
 				'label'  => 'Views with fixed time intervals',
 				'name'   => 'use_time_grid',
@@ -417,7 +352,7 @@ class calendar_hooks
 				'xmlrpc' => True,
 				'admin'  => False,
 				'forced' => 'all',
-			),*/
+			),
 			'link_title' => array(
 				'type'   => 'multiselect',
 				'label'  => 'Link title for events to show',
@@ -738,32 +673,6 @@ class calendar_hooks
 			),
 		);
 
-		if ($hook_data['type'] === 'default')
-		{
-			$settings = array_slice($settings, 0, 1, true) + array(
-				'defaultcalendar' => array(
-					'type'  => 'select',
-					'label' => 'Default calendar view',
-					'name'  => 'defaultcalendar',
-					'help'  => 'Only used for first viewing of calendar, afterwards last selected view is used.',
-					'values'	=> array(
-						'day'	=> lang('Dayview'),
-						'day4'	=> lang('four days view'),
-						'week'	=> lang('Weekview'),
-						'weekN'	=> lang('Multiple week view'),
-						'month'	=> lang('Monthview'),
-						'year'	=> lang('Yearview'),
-						'planner' => lang('Planner'),
-						'listview' => lang('listview'),
-					),
-					'xmlrpc' => True,
-					'admin'  => False,
-					'default' => 'week',
-
-				),
-			) + array_slice($settings, 1, count($settings)-1, true);
-		}
-
 		return $settings;
 	}
 
@@ -918,6 +827,19 @@ END:VALARM';
 		unset($data);	// not used, but in function signature for hooks
 		return true;
 	}
+	
+	/**
+	 * Mail integration hook to import mail message contents into a calendar entry
+	 *
+	 * @return string method to be executed for calendar mail integration
+	 */
+	public static function mail_import($args)
+	{
+		return array (
+			'menuaction' => 'calendar.calendar_uiforms.mail_import',
+			'popup' => egw_link::get_registry('calendar', 'edit_popup')
+		);
+	}
 }
 
 // Not part of the class, since config hooks are still using the old style
@@ -941,4 +863,4 @@ function calendar_purge_old($config)
 			$GLOBALS['config_error'] = 'Unable to schedule purge';
 		}
 	}
-}
+}
\ No newline at end of file
