@@ -9,7 +9,7 @@
  * @copyright (c) 2007-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright 2011 Nathan Gray
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.calendar_merge.inc.php 49494 2014-11-19 13:12:17Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -97,6 +97,46 @@ class calendar_merge extends bo_merge
 		$this->query['num_rows'] = -1;
 	}
 
+	/**
+	 * Merges a given document with contact data
+	 *
+	 * Overridden from parent to be able to change a list of events into a range,
+	 * if the target document has no pagerepeat tag.  Otherwise, parent::merge_string()
+	 * would fail because we're trying to merge multiple records with no pagerepeat tag.
+	 *
+	 *
+	 * @param string $content
+	 * @param array $ids array with contact id(s)
+	 * @param string &$err error-message on error
+	 * @param string $mimetype mimetype of complete document, eg. text/*, application/vnd.oasis.opendocument.text, application/rtf
+	 * @param array $fix=null regular expression => replacement pairs eg. to fix garbled placeholders
+	 * @param string $charset=null charset to override default set by mimetype or export charset
+	 * @return string|boolean merged document or false on error
+	 */
+	function merge_string($content,$ids,$err,$mimetype,$fix)
+	{
+		// Handle merging a list of events into a document with range instead of pagerepeat
+		if(strpos($content, '$$pagerepeat') === false && strpos($content, '{{pagerepeat') === false && count($ids) > 1)
+		{
+			// Merging more than one something will fail without pagerepeat
+			if (is_array($ids) && $ids[0]['id'])
+			{
+				// Passed an array of events, to be handled like a date range
+				$events = $ids;
+				$ids = array('start' => PHP_INT_MAX, 'end' => 0);
+				$this->ids = array();
+				foreach($events as $event) {
+					if($event['start'] && egw_time::to($event['start'],'ts') < $ids['start']) $ids['start'] = egw_time::to($event['start'],'ts');
+					if($event['end'] && egw_time::to($event['end'],'ts') > $ids['end']) $ids['end'] = egw_time::to($event['end'],'ts');
+					// Keep ids for future use
+					$this->ids[] = $event['id'];
+				}
+				$ids = array($ids);
+			}
+		}
+		return parent::merge_string($content, $ids, $err, $mimetype,$fix);
+	}
+
 	/**
 	 * Get replacements
 	 *
@@ -133,6 +173,7 @@ class calendar_merge extends bo_merge
 				// Keep ids for future use
 				$this->ids[]  = $event['id'];
 			}
+			$id = array($id);
 		}
 		else
 		{
@@ -179,12 +220,47 @@ class calendar_merge extends bo_merge
 		} else {
 			$event = $id;
 		}
+
+		$record = new calendar_egw_record($event['id']);
+
+		// Convert to human friendly values
+		$types = calendar_egw_record::$types;
+		importexport_export_csv::convert($record, $types, 'calendar');
+
+		$array = $record->get_record_array();
+		foreach($array as $key => $value)
+		{
+			$replacements['$$'.($prefix?$prefix.'/':'').$key.'$$'] = $value;
+		}
+
 		$replacements['$$' . ($prefix ? $prefix . '/' : '') . 'calendar_id'. '$$'] = $event['id'];
 		foreach($this->bo->event2array($event) as $name => $data)
 		{
 			if (substr($name,-4) == 'date') $name = substr($name,0,-4);
 			$replacements['$$' . ($prefix ? $prefix . '/' : '') . 'calendar_'.$name . '$$'] = is_array($data['data']) ? implode(', ',$data['data']) : $data['data'];
 		}
+		// Add seperate lists of participants by type
+		if(strpos($content, 'calendar_participants/')!== false)
+		{
+			$types = array();
+			foreach($this->bo->resources as $resource)
+			{
+				$types[$resource['app']] = array();
+			}
+			foreach($event['participants'] as $uid => $status)
+			{
+				$type = $this->bo->resources[$uid[0]]['app'];
+				if($type == 'home-accounts')
+				{
+					$type = ($GLOBALS['egw']->accounts->get_type($uid) == 'g' ? 'group' : 'account');
+				}
+				$types[$type][] = $this->bo->participant_name($uid);
+			}
+			foreach($types as $t_id => $type)
+			{
+				$replacements['$$'.($prefix ? $prefix . '/' : '') . "calendar_participants/{$t_id}$$"] = implode(', ',$type);
+			}
+		}
 		if(!$replacements['$$'.($prefix ? $prefix . '/' : '') . 'calendar_recur_type$$'])
 		{
 			// Need to set it to '' if not set or previous record may be used
@@ -198,7 +274,7 @@ class calendar_merge extends bo_merge
 				'time' => (date('Ymd',$event['start']) != date('Ymd',$event['end']) ? $GLOBALS['egw_info']['user']['preferences']['common']['dateformat'].' ' : '') . ($GLOBALS['egw_info']['user']['preferences']['common']['timeformat'] == 12 ? 'h:i a' : 'H:i'),
 			) as $name => $format)
 			{
-				$value = date($format,$event[$what]);
+				$value = egw_time::to($event[$what],$format);
 				if ($format == 'l') $value = lang($value);
 				$replacements['$$' .($prefix ? $prefix.'/':'').'calendar_'.$what.$name.'$$'] = $value;
 			}
@@ -213,17 +289,11 @@ class calendar_merge extends bo_merge
 			$replacements['$$'.($prefix?$prefix.'/':'').'calendar_owner/primary_group$$'] = $GLOBALS['egw']->accounts->id2name($GLOBALS['egw']->accounts->id2name($event['owner'],'account_primary_group'));
 		}
 
-		if($content && strpos($content, '$$#') !== 0)
+		if($content && strpos($content, '$$#') !== FALSE)
 		{
 			$this->cf_link_to_expand($event, $content, $replacements);
 		}
 
-		$custom = config::get_customfields('calendar');
-		foreach(array_keys($custom) as $name)
-		{
-			$replacements['$$'.($prefix?$prefix.'/':'').'#'.$name.'$$'] = $event['#'.$name];
-		}
-
 		// Links
 		$replacements += $this->get_all_links('calendar', $event['id'], $prefix, $content);
 
@@ -567,6 +637,18 @@ class calendar_merge extends bo_merge
 			echo '<tr><td>{{#'.$name.'}}</td><td colspan="3">'.$field['label']."</td></tr>\n";
 		}
 
+
+		echo '<tr><td colspan="4"><h3>'.lang('Participants').":</h3></td></tr>";
+		echo '<tr><td>{{calendar_participants/account}}</td><td colspan="3">'.lang('Accounts')."</td></tr>\n";
+		echo '<tr><td>{{calendar_participants/group}}</td><td colspan="3">'.lang('Groups')."</td></tr>\n";
+		foreach($this->bo->resources as $resource)
+		{
+			if($resource['type'])
+			{
+				echo '<tr><td>{{calendar_participants/'.$resource['app'].'}}</td><td colspan="3">'.lang($resource['app'])."</td></tr>\n";
+			}
+		}
+
 		echo '<tr><td colspan="4"><h3>'.lang('Participant table').":</h3></td></tr>";
 		echo '<tr><td colspan="4">{{table/participant}} ... </td></tr>';
 		echo '<tr><td>{{name}}</td><td>'.lang('name').'</td></tr>';
