@@ -5,9 +5,9 @@
  * @link http://www.egroupware.org
  * @package calendar
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @copyright (c) 2004-9 by RalfBecker-At-outdoor-training.de
+ * @copyright (c) 2004-15 by RalfBecker-At-outdoor-training.de
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.calendar_ui.inc.php 47716 2014-07-16 12:18:43Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -110,6 +110,10 @@ class calendar_ui
 	 * @var string $view menuaction of the selected view
 	 */
 	var $view_menuaction;
+	/**
+	 * @var boolean test checkbox checked
+	 */
+	var $test;
 
 	/**
 	 * @var int $first first day of the shown view
@@ -302,6 +306,7 @@ class calendar_ui
 			'planner_days'=> 0,	// full month
 			'view'       => ($this->bo->cal_prefs['defaultcalendar']?$this->bo->cal_prefs['defaultcalendar']:'day'), // use pref, if exists else use the dayview
 			'listview_days'=> '',	// no range
+			'test'       => 'false',
 		) as $state => $default)
 		{
 			if (isset($set_states[$state]))
@@ -508,7 +513,7 @@ class calendar_ui
 		}
 		if($event['participants'][$this->user][0] == 'U')
 		{
-			$icons[] = html::image('calendar','cnr-pending',lang('Needs action'));
+			$icons[] = html::image('calendar','needs-action',lang('Needs action'));
 		}
 		return $icons;
 	}
@@ -747,12 +752,6 @@ class calendar_ui
 
 		$file[++$n] = array('text' => $jscalendar,'no_lang' => True,'link' => False,'icon' => False);
 
-		// set a baseurl for selectboxes, if we are not running inside calendar (eg. prefs or admin)
-		if (substr($_GET['menuaction'],0,9) != 'calendar.')
-		{
-			$baseurl = egw::link('/index.php',array('menuaction'=>'calendar.calendar_uiviews.index'),false);
-		}
-
 		// Category Selection
 		$cat_id = explode(',',$this->cat_id);
 
@@ -765,7 +764,7 @@ class calendar_ui
 			lang('Select a %1',lang('Category')). '"'.($cat_id && count($cat_id) > 1 ? ' multiple=true size=4':''). '>'.
 			'<option value="0">'.lang('All categories').'</option>'.
 				$this->categories->formatted_list('select','all',$cat_id,'True').
-			"</select>\n" . html::image('phpgwapi','attach','','id="calendar_cat_id_multiple"')."
+			"</select>\n" . html::image('phpgwapi','category','','id="calendar_cat_id_multiple"')."
 			<script type=\"text/javascript\" src=\"{$GLOBALS['egw_info']['server']['webserver_url']}/calendar/js/navigation.js\" id=\"calendar-navigation-script\"".
 			" data-link-day-url =\"".htmlspecialchars($link['day']).
 			"\" data-link-week-url=\"".htmlspecialchars($link['week']).
@@ -791,6 +790,13 @@ class calendar_ui
 			}
 			// we no longer exclude non-accounts from the account-selection: it shows all types of participants
 			$accounts = explode(',',$this->owner);
+
+			// Remove '0' - it means current user but will show as #0 with some account selection options
+			$zero_index = array_search('0',$accounts);
+			if($zero_index !== FALSE)
+			{
+				$accounts[$zero_index] = $this->user;
+			}
 			$file[] = array(
 				'text' =>
 				$this->accountsel->selection('owner','uical_select_owner',$accounts,'calendar+',count($accounts) > 1 ? 4 : 1,False,
@@ -828,6 +834,8 @@ class calendar_ui
 			$filter_options .= '<option value="deleted"'.($this->filter == 'deleted' ? ' selected="selected"' : '').' title="'.lang('Show events that have been deleted').'">'.lang('Deleted').'</options>'."\n";
 		}
 		$file[] = $this->_select_box('Filter','filter',$filter_options,'86%');
+		// enable this to get checkbox setting $this->test eg. usable to trigger different code in calendar_so
+		//$file[count($file)-1]['text'] .= html::checkbox('test', $this->test==='true', 'true', 'id="calendar_test"');
 
 		// Merge print
 		if ($GLOBALS['egw_info']['user']['preferences']['calendar']['document_dir'])
@@ -875,7 +883,7 @@ class calendar_ui
 		{
 			$file = Array(
 				'Configuration'=>egw::link('/index.php','menuaction=admin.uiconfig.index&appname=calendar'),
-				'Custom Fields'=>egw::link('/index.php','menuaction=admin.customfields.edit&appname=calendar'),
+				'Custom Fields'=>egw::link('/index.php','menuaction=admin.customfields.index&appname=calendar'),
 				'Holiday Management'=>egw::link('/index.php','menuaction=calendar.uiholiday.admin'),
 				'Global Categories' =>egw::link('/index.php','menuaction=admin.admin_categories.index&appname=calendar'),
 			);
