@@ -7,7 +7,7 @@
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright (c) 2005-13 by RalfBecker-At-outdoor-training.de
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.calendar_uilist.inc.php 49448 2014-11-17 15:55:18Z nathangray $
+ * @version $Id$
  */
 
 /**
@@ -294,7 +294,11 @@ class calendar_uilist extends calendar_ui
 
 		// do we need to query custom fields and which
 		// Check stored preference if selectcols isn't available (ie: first call)
-		$select_cols = explode(',',$params['selectcols'] ? $params['selectcols'] : $GLOBALS['egw_info']['user']['preferences']['calendar']['nextmatch-calendar.list.rows']);
+		$select_cols = $params['selectcols'] ? $params['selectcols'] : $GLOBALS['egw_info']['user']['preferences']['calendar']['nextmatch-calendar.list.rows'];
+		if(!is_array($params['selectcols']))
+		{
+			$select_cols = explode(',',$select_cols);
+		}
 		if (in_array('cfs',$select_cols))
 		{
 			$cfs = array();
@@ -369,6 +373,17 @@ class calendar_uilist extends calendar_ui
 		{
 			$search_params['users'] = explode(',',$this->owner);
 		}
+		if ($params['col_filter'])
+		{
+			$col_filter = array();
+			foreach($params['col_filter'] as $name => $val)
+			{
+				if (!in_array($name, array('participant','row_id')) && (string)$val !== '')
+				{
+					$col_filter[$name] = $val;
+				}
+			}
+		}
 		$rows = $js_integration_data = array();
 		if ($label)
 		{
@@ -377,7 +392,7 @@ class calendar_uilist extends calendar_ui
 			$rows['sel_options']['filter'] = $this->date_filters;
 			$rows['sel_options']['filter'][$params['filter']] = $label;
 		}
-		foreach((array) $this->bo->search($search_params) as $event)
+		foreach((array) $this->bo->search($search_params, !empty($col_filter) ? $col_filter : null) as $event)
 		{
 			if (!$this->bo->check_perms(EGW_ACL_EDIT,$event))
 			{
@@ -395,7 +410,7 @@ class calendar_uilist extends calendar_ui
 			{
 				$event['class'] .= 'rowDeleted ';
 			}
-			
+
 			$event['recure'] = $this->bo->recure2string($event);
 			if ($params['csv_export'])
 			{
@@ -478,6 +493,12 @@ class calendar_uilist extends calendar_ui
 			// set id for grid
 			$event['row_id'] = $event['id'].($event['recur_type'] ? ':'.$event['recur_date'] : '');
 
+			// Format start and end with timezone
+			foreach(array('start','end') as $time)
+			{
+				$event[$time] = egw_time::to($event[$time],'Y-m-d\TH:i:s\Z');
+			}
+
 			$rows[] = $event;
 			unset($app);
 			unset($app_id);
@@ -526,7 +547,7 @@ class calendar_uilist extends calendar_ui
 	 */
 	function action($action,$checked,$use_all,&$success,&$failed,&$action_msg,$session_name,&$msg,$skip_notification=false)
 	{
-		//echo '<p>' . __METHOD__ . "('$action',".print_r($checked,true).','.(int)$use_all.",...)</p>\n";
+		//error_log(__METHOD__."('$action', ".array2string($checked).', all='.(int)$use_all.", ...)");
 		$success = $failed = 0;
 		$msg = null;
 
@@ -560,7 +581,7 @@ class calendar_uilist extends calendar_ui
 			}
 		}
 		// for calendar integration we have to fetch all rows and unset the not selected ones, as we can not filter by id
-		elseif(in_array($action,array('ical','document')))
+		elseif($action == 'document')
 		{
 			$query = is_array($session_name) ? $session_name : egw_session::appsession($session_name,'calendar');
 			@set_time_limit(0);				// switch off the execution time limit, as for big selections it's too small
@@ -576,12 +597,23 @@ class calendar_uilist extends calendar_ui
 		switch($action)
 		{
 			case 'ical':
+				// compile list of unique cal_id's, as iCal should contain whole series, not recurrences
+				// calendar_ical->exportVCal needs to read events again, to get them in server-time
+				$ids = array();
+				foreach($checked as $id)
+				{
+					if (is_array($id)) $id = $id['id'];
+					// get rid of recurrences, doublicate series and calendar-integration events
+					if (($id = (int)$id))
+					{
+						$ids[$id] = $id;
+					}
+				}
 				$boical = new calendar_ical();
-				$ical =& $boical->exportVCal($checked,'2.0','PUBLISH',false);
-				html::content_header($content['file'] ? $content['file'] : 'event.ics','text/calendar',bytes($ical));
+				$ical =& $boical->exportVCal($ids, '2.0', 'PUBLISH');
+				html::content_header('event.ics', 'text/calendar', bytes($ical));
 				echo $ical;
 				common::egw_exit();
-				break;
 
 			case 'document':
 				if (!$settings) $settings = $GLOBALS['egw_info']['user']['preferences']['calendar']['default_document'];
@@ -815,7 +847,7 @@ class calendar_uilist extends calendar_ui
 	 *
 	 * @return array see nextmatch_widget::get_actions()
 	 */
-	private function get_actions()
+	public function get_actions()
 	{
 		$actions = array(
 			'add' => array(
