@@ -6,7 +6,7 @@
  * @author Hadi Nategh	<hn-AT-stylite.de>
  * @copyright (c) 2008-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: app.js 49723 2014-12-05 11:32:08Z hnategh $
+ * @version $Id$
  */
 
 /*egw:uses
@@ -33,16 +33,22 @@ app.classes.calendar = AppJS.extend(
 	init: function()
 	{
 		// make calendar object available, even if not running in top window, as sidebox does
-		if (window.top !== window)
+		if (window.top !== window && !egw(window).is_popup())
 		{
 			// we have to explicitly delete old object or IE11 complains about accessing an already freed script
 			delete window.top.app.calendar;
 			window.top.app.calendar = this;
 		}
-
+		
 		// call parent
 		this._super.apply(this, arguments);
 
+		// Make sure translations are loaded
+		if(!this.egw.is_popup())
+		{
+			this.egw.langRequireApp(this.egw.window, this.appname);
+		}
+
 		//Drag_n_Drop (need to wait for DOM ready to init dnd)
 		jQuery(jQuery.proxy(this.drag_n_drop,this));
 	},
@@ -95,7 +101,6 @@ app.classes.calendar = AppJS.extend(
 			case 'calendar.edit':
 				if (typeof content.data['conflicts'] == 'undefined')
 				{
-					$j(document.getElementById('calendar-edit_calendar-delete_series')).hide();
 					//Check if it's fallback from conflict window or it's from edit window
 					if (content.data['button_was'] != 'freetime')
 					{
@@ -110,10 +115,13 @@ app.classes.calendar = AppJS.extend(
 					}
 					//send Syncronus ajax request to the server to unlock the on close entry
 					//set onbeforeunload with json request to send request when the window gets close by X button
-					window.onbeforeunload = function () {
-						this.egw.json('calendar.calendar_uiforms.ajax_unlock'
-						, [content.data['id'],content.data['lock_token']],null,true,null,null).sendRequest(true);
-					};
+					if (content.data.lock_token)
+					{
+						window.onbeforeunload = function () {
+							this.egw.json('calendar.calendar_uiforms.ajax_unlock',
+							[content.data.id, content.data.lock_token],null,true,null,null).sendRequest(true);
+						};
+					}
 				}
 				this.alarm_custom_date();
 				break;
@@ -181,7 +189,7 @@ app.classes.calendar = AppJS.extend(
 						{
 							// F.F can not handle to style correctly an iframe which is hidden (display:none), therefore we need to
 							// bind a handler to refresh the calendar views after it shows up
-							iframe.one('show',function(){egw_refresh('','calendar');})
+							iframe.one('show',function(){egw_refresh('','calendar');});
 						}
 						else
 						{
@@ -211,6 +219,14 @@ app.classes.calendar = AppJS.extend(
 				return _url.replace(/menuaction=[^&]+/, 'menuaction=calendar.calendar_uilist.listview&ajax=true');
 			}
 		}
+		else if (_url.indexOf('merge=') > 0)
+		{
+			// Just open the merge
+			window.open(_url.replace('&ajax=true',''),'_blank');
+
+			// Tell framework app took care of it
+			return true;
+		}
 		// can not load our own index page, has to be done by framework
 		return false;
 	},
@@ -272,9 +288,12 @@ app.classes.calendar = AppJS.extend(
 						 */
 						start: function(event, ui)
 						{
-							oldInnerHTML = ui.helper[0].innerHTML;
-							oldWidth = ui.helper.width();
-							ui.helper.width(jQuery("#calColumn").width());
+							//Safari 9 needs a delay to get the jquery helper width
+							setTimeout(function(){
+								oldInnerHTML = ui.helper[0].innerHTML;
+								oldWidth = ui.helper.width();
+								ui.helper.width(jQuery("#calColumn").width());
+							},1);
 						}
 					})
 
@@ -703,7 +722,7 @@ app.classes.calendar = AppJS.extend(
 		var eventOwner = _id.substring(_id.lastIndexOf("_C")+2,_id.lastIndexOf(""));
 		var date = this.cal_dnd_tZone_converter(_date);
 
-		if (_eventFlag == 'S')
+		if (_eventFlag === 'S' && eventId.indexOf(':') > 0)
 		{
 			et2_dialog.show_dialog(function(_button_id)
 			{
@@ -787,7 +806,9 @@ app.classes.calendar = AppJS.extend(
 		if (typeof duration != 'undefined' && typeof end != 'undefined')
 		{
 			end.set_disabled(duration.get_value()!=='');
-			if (!end.disabled )
+
+			// Only set end date if not provided, adding seconds fails with DST
+			if (!end.disabled && !content.end)
 			{
 				end.set_value(start.get_value());
 				if (typeof content.duration != 'undefined') end.set_value("+"+content.duration);
@@ -811,7 +832,6 @@ app.classes.calendar = AppJS.extend(
 			{
 				case 'print':
 					this.egw.open_link('calendar.calendar_uiforms.edit&cal_id='+id+'&print=1','_blank','700x700');
-					this.et2._inst.submit();
 					break;
 				case 'mail':
 					this.egw.json('calendar.calendar_uiforms.ajax_custom_mail', [event, !event['id'], false],null,null,null,null).sendRequest();
@@ -861,21 +881,21 @@ app.classes.calendar = AppJS.extend(
 			var buttons = [
 				{
 					button_id: 'keep',
-					statustext:'All exceptions are converted into single events.',
-					text: 'Keep exceptions',
+					title: this.egw.lang('All exceptions are converted into single events.'),
+					text: this.egw.lang('Keep exceptions'),
 					id: 'button[delete_keep_exceptions]',
 					image: 'keep', "default":true
 				},
 				{
 					button_id: 'delete',
-					statustext:'The exceptions are deleted together with the series.',
-					text: 'Delete exceptions',
+					title: this.egw.lang('The exceptions are deleted together with the series.'),
+					text: this.egw.lang('Delete exceptions'),
 					id: 'button[delete_exceptions]',
 					image: 'delete'
 				},
 				{
 					button_id: 'cancel',
-					text: 'Cancel',
+					text: this.egw.lang('Cancel'),
 					id: 'dialog[cancel]',
 					image: 'cancel'
 				}
@@ -888,7 +908,8 @@ app.classes.calendar = AppJS.extend(
 					{
 						if (_button_id != 'dialog[cancel]')
 						{
-							self.et2._inst.submit(_button_id);
+							widget.getRoot().getWidgetById('delete_exceptions').set_value(_button_id == 'button[delete_exceptions]');
+							widget.getInstanceManager().submit('button[delete]');
 							return true;
 						}
 						else
@@ -940,6 +961,43 @@ app.classes.calendar = AppJS.extend(
 		}
 	},
 
+	/**
+	 * In edit popup, search for calendar participants.
+	 * Resources need to have the start & duration (etc.)
+	 * passed along in the query.
+	 *
+	 * @param {Object} request
+	 * @param {et2_link_entry} widget
+	 *
+	 * @returns {boolean} True to continue with the search
+	 */
+	edit_participant_search: function(request, widget)
+	{
+		if(widget.app_select.val() == 'resources')
+		{
+			// Resources search is expecting exec
+			var values = widget.getInstanceManager().getValues(widget.getRoot());
+			if(typeof request.options != 'object' || request.options == null)
+			{
+				request.options = {};
+			}
+			request.options.exec = {
+				start: values.start,
+				end: values.end,
+				duration: values.duration,
+				participants: values.participants,
+				recur_type: values.recur_type,
+				event_id: values.link_to.to_id, // cal_id, if available
+				show_conflict: (egw.preference('defaultresource_sel','calendar') == 'resources_without_conflict') ? '0' : '1'
+			};
+			if(values.whole_day)
+			{
+				request.options.exec.whole_date = true;
+			}
+		}
+		return true;
+	},
+
 	/**
 	 * Handles to select freetime, and replace the selected one on Start,
 	 * and End date&time in edit calendar entry popup.
@@ -979,7 +1037,7 @@ app.classes.calendar = AppJS.extend(
 				alert(this.egw.lang('The original calendar edit popup is closed!'));
 			}
 		}
-		window.close();
+		egw(window).close();
 	},
 
 	/**
@@ -1042,10 +1100,10 @@ app.classes.calendar = AppJS.extend(
 	 */
 	cal_open: function(_action, _senders)
 	{
-		var js_integration_data = this.et2.getArrayMgr('content').data.nm.js_integration_data;
+
+		var js_integration_data = _action.parent.data.nextmatch.options.settings.js_integration_data || this.et2.getArrayMgr('content').data.nm.js_integration_data;
 		var id = _senders[0].id;
 		var matches = id.match(/^(?:calendar::)?([0-9]+):([0-9]+)$/);
-		var backup = _action.data;
 		if (matches)
 		{
 			this.edit_series(matches[1],matches[2]);
@@ -1053,6 +1111,15 @@ app.classes.calendar = AppJS.extend(
 		}
 		matches = id.match(/^([a-z_-]+)([0-9]+)/i);
 		if (matches)
+		{
+			var js_integration_data = _action.parent.data.nextmatch.options.settings.js_integration_data || this.et2.getArrayMgr('content').data.nm.js_integration_data;
+			if(typeof js_integration_data == 'string')
+			{
+				js_integration_data = JSON.parse(js_integration_data);
+			}
+		}
+		matches = id.match(/^calendar::([a-z_-]+)([0-9]+)/i);
+		if (matches && js_integration_data && js_integration_data[matches[1]])
 		{
 			var app = matches[1];
 			_action.data.url = window.egw_webserverUrl+'/index.php?';
@@ -1063,20 +1130,23 @@ app.classes.calendar = AppJS.extend(
 
 			if (js_integration_data[app].edit_popup)
 			{
-				matches = js_integration_data[app].edit_popup.match(/^(.*)x(.*)$/);
-				if (matches)
-				{
-					_action.data.width = matches[1];
-					_action.data.height = matches[2];
-				}
-				else
-				{
-					_action.data.nm_action = 'location';
-				}
+				egw.open_link(_action.data.url,'_blank',js_integration_data[app].edit_popup,app);
+
+				_action.data = backup;	// restore url, width, height, nm_action
+				return;
 			}
 		}
+		else
+		{
+			// Other app integration using link registry
+			var data = egw.dataGetUIDdata(_senders[0].id);
+			if(data && data.data)
+			{
+				return egw.open(data.data.app_id, data.data.app, 'edit');
+			}
+		}
+		// Regular, single event
 		egw.open(id.replace(/^calendar::/g,''),'calendar','edit');
-		_action.data = backup;	// restore url, width, height, nm_action
 	},
 
 	/**
@@ -1246,6 +1316,20 @@ app.classes.calendar = AppJS.extend(
 			jQuery.extend(state, this._super.apply(this, arguments));	// call default implementation
 		}
 
+		// Don't store current user in state to allow admins to create favourites for all
+		// Should make no difference for normal users.
+		if(state.owner == egw.user('account_id'))
+		{
+			// 0 is always the current user, so if an admin creates a default favorite,
+			// it will work for other users too.
+			state.owner = 0;
+		}
+		// Don't store first and last
+		delete state.first;
+		delete state.last;
+		delete state.startdate;
+		delete state.enddate;
+
 		return state;
 	},
 
@@ -1267,18 +1351,18 @@ app.classes.calendar = AppJS.extend(
 			}
 		}
 
+		if (jQuery.isEmptyObject(state)) state.state = {};
+
 		// old calendar state handling on server-side (incl. switching to and from listview)
 		var menuaction = 'calendar.calendar_uiviews.index';
 		if (typeof state.state != 'undefined' && (typeof state.state.view == 'undefined' || state.state.view == 'listview'))
 		{
-			if (state.name)
-			{
-				// 'blank' is the special name for no filters, send that instead of the nice translated name
-				state.state.favorite = jQuery.isEmptyObject(state) || jQuery.isEmptyObject(state.state||state.filter) ? 'blank' : state.name.replace(/[^A-Za-z0-9-_]/g, '_');
-				// set date for "No Filter" (blank) favorite to todays date
-				if (state.state.favorite == 'blank')
-					state.state.date = jQuery.datepicker.formatDate('yymmdd', new Date);
-			}
+			// 'blank' is the special name for no filters, send that instead of the nice translated name
+			state.state.favorite = jQuery.isEmptyObject(state.state||state.filter) ? 'blank' : state.name.replace(/[^A-Za-z0-9-_]/g, '_');
+			// set date for "No Filter" (blank) favorite to todays date
+			if (state.state.favorite == 'blank')
+				state.state.date = jQuery.datepicker.formatDate('yymmdd', new Date);
+
 			menuaction = 'calendar.calendar_uilist.listview';
 			state.state.ajax = 'true';
 			// check if we already use et2 / are in listview
