@@ -6,7 +6,7 @@
  * @package calendar
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: tables_update.inc.php 51307 2015-01-20 13:49:42Z ralfbecker $
+ * @version $Id$
  */
 
 function calendar_v0_9_2to0_9_3update_owner($table, $field)
@@ -694,7 +694,7 @@ function calendar_upgrade0_9_11_001()
 				$recur_type_num = RECUR_YEARLY;
 				break;
 		}
-		$recur_end_use = $GLOBALS['egw_setup']->oProc->f('cal_use_end');
+		$recur_use_end = $GLOBALS['egw_setup']->oProc->f('cal_use_end');
 		$recur_end = $GLOBALS['egw_setup']->oProc->f('cal_end');
 		$recur_interval = $GLOBALS['egw_setup']->oProc->f('cal_frequency');
 		$days = strtoupper($GLOBALS['egw_setup']->oProc->f('cal_days'));
@@ -1458,7 +1458,6 @@ function calendar_upgrade1_0_1_008()
 function calendar_upgrade1_0_1_009()
 {
 	$db2 = clone($GLOBALS['egw_setup']->db);
-	$add_groups = array();
 	$GLOBALS['egw_setup']->db->select('egw_cal','DISTINCT egw_cal.cal_id,cal_groups,cal_recur_date',"cal_groups != ''",__LINE__,__FILE__,
 		False,'','calendar',0,',egw_cal_user WHERE egw_cal.cal_id=egw_cal_user.cal_id');
 	while(($row = $GLOBALS['egw_setup']->db->row(true)))
@@ -1617,7 +1616,7 @@ function calendar_upgrade1_5_002()
 {
 	// update the alarm methods
 	$async = new asyncservice();
-	foreach((array)$async->read('cal:%') as $id => $job)
+	foreach((array)$async->read('cal:%') as $job)
 	{
 		if ($job['method'] == 'calendar.bocalupdate.send_alarm')
 		{
@@ -1655,6 +1654,7 @@ function calendar_upgrade1_6()
 
 	// Search series exception for nearest exception in series master and add that RECURRENCE-ID
 	// as cal_reference (for 1.6.003 and move it to new field cal_recurrence in 1.7.001)
+	$diff = null;
 	foreach($GLOBALS['egw_setup']->db->query('SELECT egw_cal.cal_id,cal_start,recur_exception FROM egw_cal
 		JOIN egw_cal_dates ON egw_cal.cal_id=egw_cal_dates.cal_id
 		JOIN egw_cal_repeats ON cal_reference=egw_cal_repeats.cal_id
@@ -2330,3 +2330,309 @@ ORDER BY master.cal_id DESC", __LINE__, __FILE__, 0, -1, false, egw_db::FETCH_AS
 	}
 	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.1.001';
 }
+
+/**
+ * Modify range_end of recurring events to be end-time of last recurrence (and not just a date)
+ *
+ * This fixes not found last recurrence in day-view.
+ *
+ * @return string
+ */
+function calendar_upgrade14_1_001()
+{
+	foreach($GLOBALS['egw_setup']->db->query(
+"SELECT egw_cal.cal_id AS cal_id,cal_start,cal_end,range_start,range_end,egw_cal_repeats.*,tz_tzid AS tzid
+FROM egw_cal
+JOIN egw_cal_repeats ON egw_cal_repeats.cal_id=egw_cal.cal_id
+JOIN egw_cal_dates ON egw_cal_dates.cal_id=egw_cal.cal_id AND cal_start=range_start
+JOIN egw_cal_timezones ON egw_cal.tz_id=egw_cal_timezones.tz_id
+WHERE range_end IS NOT NULL", __LINE__, __FILE__, 0, -1, false, egw_db::FETCH_ASSOC) as $event)
+	{
+		$event = egw_db::strip_array_keys($event, 'cal_');
+		$event['recur_enddate'] = $event['range_end'];
+		$rrule = calendar_rrule::event2rrule($event, false);
+		$rrule->rewind();
+		$enddate = $rrule->current();
+		do
+		{
+			$rrule->next_no_exception();
+			$occurrence = $rrule->current();
+		}
+		while ($rrule->valid() && ($enddate = $occurrence));
+		$enddate->modify(($event['end'] - $event['start']).' second');
+		if (($range_end = $enddate->format('server')) != $event['range_end'])
+		{
+			$GLOBALS['egw_setup']->db->update('egw_cal',array(
+				'range_end' => $range_end,
+				'cal_etag=cal_etag+1',
+				'cal_modified' => time(),
+			), array(
+				'cal_id' => $event['id'],
+			), __LINE__, __FILE__, 'calendar');
+			//error_log(__FUNCTION__."() #$event[id], start=".date('Y-m-d H:i:s', $event['start']).', end='.date('Y-m-d H:i:s', $event['end']).', range_end='.date('Y-m-d H:i:s', $event['recur_enddate']).' --> '.date('Y-m-d H:i:s', $range_end));
+		}
+	}
+	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.2.002';	// skip 14.2.001 update, as query is fixed now
+}
+
+/**
+ * Partially fix from previous 14.2.001 update "ended" daily recurring events (eg. birthdays)
+ *
+ * Idea is to find time where most yearly recuring events were modified from above update.
+ * Then we set all yearly recuring events modified 1s around that time to unlimited again.
+ *
+ * @return string
+ */
+function calendar_upgrade14_2_001()
+{
+	// get modification time (in seconds) with maximum count of modified yearly recuring events
+	$values = $GLOBALS['egw_setup']->db->query("SELECT cal_modified, COUNT(*)
+FROM egw_cal
+JOIN egw_cal_repeats ON egw_cal.cal_id=egw_cal_repeats.cal_id
+WHERE recur_type=5 AND range_end IS NOT NULL
+GROUP BY cal_modified
+ORDER BY count(*) DESC", __LINE__, __FILE__, 0, 1)->fetch();
+
+	// if maximum is bigger then 3
+	if ($values[1] > 3)
+	{
+		switch($GLOBALS['egw_setup']->db->Type)
+		{
+			case 'mysql':
+				$sql = "UPDATE egw_cal
+JOIN egw_cal_repeats ON egw_cal.cal_id=egw_cal_repeats.cal_id
+SET range_end=NULL
+WHERE recur_type=5 AND range_end IS NOT NULL AND ABS(cal_modified-$values[0])<=1";
+				break;
+			case 'pgsql':
+				$sql = "UPDATE egw_cal
+SET range_end=NULL
+FROM egw_cal AS cal
+JOIN egw_cal_repeats ON cal.cal_id=egw_cal_repeats.cal_id
+WHERE recur_type=5 AND cal.range_end IS NOT NULL AND ABS(cal.cal_modified-$values[0])<=1";
+				break;
+		}
+		if (isset($sql))
+		{
+			$GLOBALS['egw_setup']->db->query($sql, __LINE__, __FILE__);
+
+			if (($num = $GLOBALS['egw_setup']->db->affected_rows()))
+			{
+				error_log(__METHOD__."() removed end-date from $num yearly recuring events again.");
+			}
+		}
+	}
+	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.2.002';
+}
+
+/**
+ * Fix wrong date for 2016 DE holiday Fronleichnam
+ *
+ * @return string
+ */
+function calendar_upgrade14_2_002()
+{
+	$GLOBALS['egw_setup']->db->update('egw_cal_holidays', array(
+		'hol_mday' => 26,
+		'hol_month_num' => 5
+	), array(
+		'hol_locale' => 'DE',
+		'hol_name' => 'Fronleichnam',
+		'hol_occurence' => 2016,
+	), __LINE__, __FILE__, 'calendar');
+
+	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.2.003';
+}
+
+function calendar_upgrade14_2_003()
+{
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal','cal_uid',array(
+		'type' => 'ascii',
+		'precision' => '128',
+		'nullable' => False,
+		'comment' => 'unique id of event(-series)'
+	));
+	// remove all category values, which are not comma-separated nummerical ids
+	// as they might give an SQL error and wont work anyway
+	if ($GLOBALS['egw_setup']->db->Type == 'mysql')
+	{
+		$GLOBALS['egw_setup']->db->query("UPDATE egw_cal SET cal_category='' WHERE cal_category NOT REGEXP '^[0-9,]*$'", __LINE__, __FILE__);
+	}
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal','cal_category',array(
+		'type' => 'ascii',
+		'meta' => 'category',
+		'precision' => '64',
+		'comment' => 'category id(s)'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal','caldav_name',array(
+		'type' => 'ascii',
+		'precision' => '128',
+		'comment' => 'name part of CalDAV URL, if specified by client'
+	));
+
+	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.2.004';
+}
+
+
+function calendar_upgrade14_2_004()
+{
+	/* done by RefreshTable() anyway
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal_user','cal_recur_date',array(
+		'type' => 'int',
+		'meta' => 'timestamp',
+		'precision' => '8',
+		'nullable' => False,
+		'default' => '0'
+	));*/
+	/* done by RefreshTable() anyway
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal_user','cal_user_type',array(
+		'type' => 'ascii',
+		'precision' => '1',
+		'nullable' => False,
+		'default' => 'u',
+		'comment' => 'u=user, g=group, c=contact, r=resource, e=email'
+	));*/
+	/* done by RefreshTable() anyway
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal_user','cal_user_id',array(
+		'type' => 'ascii',
+		'meta' => array(
+			"cal_user_type='u'" => 'account'
+		),
+		'precision' => '128',
+		'nullable' => False,
+		'comment' => 'id or email-address for type=e'
+	));*/
+	/* done by RefreshTable() anyway
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal_user','cal_status',array(
+		'type' => 'ascii',
+		'precision' => '1',
+		'default' => 'A',
+		'comment' => 'U=unknown, A=accepted, R=rejected, T=tentative'
+	));*/
+	/* done by RefreshTable() anyway
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal_user','cal_role',array(
+		'type' => 'ascii',
+		'precision' => '64',
+		'default' => 'REQ-PARTICIPANT',
+		'comment' => 'CHAIR, REQ-PARTICIPANT, OPT-PARTICIPANT, NON-PARTICIPANT, X-CAT-$cat_id'
+	));*/
+	/* done by RefreshTable() anyway
+	$GLOBALS['egw_setup']->oProc->AddColumn('egw_cal_user','cal_user_auto',array(
+		'type' => 'auto',
+		'nullable' => False
+	));*/
+	$GLOBALS['egw_setup']->oProc->RefreshTable('egw_cal_user',array(
+		'fd' => array(
+			'cal_id' => array('type' => 'int','precision' => '4','nullable' => False),
+			'cal_recur_date' => array('type' => 'int','meta' => 'timestamp','precision' => '8','nullable' => False,'default' => '0'),
+			'cal_user_type' => array('type' => 'ascii','precision' => '1','nullable' => False,'default' => 'u','comment' => 'u=user, g=group, c=contact, r=resource, e=email'),
+			'cal_user_id' => array('type' => 'varchar','meta' => array("cal_user_type='u'" => 'account'),'precision' => '128','nullable' => False,'comment' => 'id or email-address for type=e'),
+			'cal_status' => array('type' => 'ascii','precision' => '1','default' => 'A','comment' => 'U=unknown, A=accepted, R=rejected, T=tentative'),
+			'cal_quantity' => array('type' => 'int','precision' => '4','default' => '1','comment' => 'only for certain types (eg. resources)'),
+			'cal_role' => array('type' => 'ascii','precision' => '64','default' => 'REQ-PARTICIPANT','comment' => 'CHAIR, REQ-PARTICIPANT, OPT-PARTICIPANT, NON-PARTICIPANT, X-CAT-$cat_id'),
+			'cal_user_modified' => array('type' => 'timestamp','default' => 'current_timestamp','comment' => 'automatic timestamp of last update'),
+			'cal_user_auto' => array('type' => 'auto','nullable' => False)
+		),
+		'pk' => array('cal_user_auto'),
+		'fk' => array(),
+		'ix' => array('cal_user_modified',array('cal_user_type','cal_user_id')),
+		'uc' => array(array('cal_id','cal_recur_date','cal_user_type','cal_user_id'))
+	));
+
+	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.2.005';
+}
+
+
+function calendar_upgrade14_2_005()
+{
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal_timezones','tz_tzid',array(
+		'type' => 'ascii',
+		'precision' => '128',
+		'nullable' => False
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal_timezones','tz_component',array(
+		'type' => 'ascii',
+		'precision' => '8192',
+		'comment' => 'iCal VTIMEZONE component'
+	));
+
+	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.3';
+}
+
+/**
+ * Store md5 of lowercased raw email-address as cal_user_id to only have a short ascii column for indexes and full rfc822 email in cal_user_attendee
+ *
+ * @return string
+ */
+function calendar_upgrade14_3()
+{
+	$GLOBALS['egw_setup']->oProc->AddColumn('egw_cal_user','cal_user_attendee',array(
+		'type' => 'varchar',
+		'precision' => '255',
+		'comment' => 'email or json object with attr. cn, url, ...'
+	));
+	$email = "TRIM(LOWER(CASE SUBSTR(cal_user_id, -1, 1) WHEN '>' THEN SUBSTR(cal_user_id, 1+".
+		$GLOBALS['egw_setup']->db->strpos("cal_user_id", "'<'").", CHAR_LENGTH(cal_user_id)-".
+		$GLOBALS['egw_setup']->db->strpos("cal_user_id", "'<'")."-1) ELSE cal_user_id END))";
+
+	// delete all but one row, which would give a doublicate key, after above normalising of email addresses
+	// by ordering by status we prever accepted over tentative over unknow over deleted
+	foreach($GLOBALS['egw_setup']->db->select('egw_cal_user', "cal_id,cal_recur_date,$email AS email", array(
+		'cal_user_type' => 'e',
+	), __LINE__, __FILE__, false, "GROUP BY cal_id,cal_recur_date,$email HAVING COUNT(*)>1", 'calendar') as $row)
+	{
+		$n = 0;
+		foreach($GLOBALS['egw_setup']->db->select('egw_cal_user', "*,$email AS email", array(
+			'cal_id' => $row['cal_id'],
+			'cal_recur_date' => $row['cal_recur_date'],
+			'cal_user_type' => 'e',
+			$email.'='.$GLOBALS['egw_setup']->db->quote($row['email']),
+		), __LINE__, __FILE__, false, 'ORDER BY cal_status', 'calendar') as $user)	// order A, T, U, X
+		{
+			if (strpos($user['email'], '@') !== false && !$n++) continue;
+			$GLOBALS['egw_setup']->db->delete('egw_cal_user', array_intersect_key($user, array_flip(array('cal_id','cal_recur_date','cal_user_type','cal_user_id','cal_status'))));
+		}
+	}
+
+	// store only md5 of normalized email to always fit in 32 ascii chars (and allow non-ascii email)
+	$GLOBALS['egw_setup']->db->query(
+		"UPDATE egw_cal_user SET cal_user_attendee=cal_user_id,cal_user_id=MD5($email) WHERE cal_user_type='e'",
+		__LINE__, __FILE__);
+
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_cal_user','cal_user_id',array(
+		'type' => 'ascii',
+		'meta' => array(
+			"cal_user_type='u'" => 'account'
+		),
+		'precision' => '32',
+		'nullable' => False,
+		'comment' => 'id or md5(email-address) for type=e'
+	));
+
+	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.3.001';
+}
+
+/**
+ * Fix NULL in egw_cal_user.cal_user_attendee for recurrences by using value from master (cal_recur_date=0)
+ *
+ * @return string
+ */
+function calendar_upgrade14_3_001()
+{
+	foreach($GLOBALS['egw_setup']->db->query("SELECT egw_cal_user.cal_id AS cal_id,egw_cal_user.cal_user_id AS cal_user_id,egw_cal_user.cal_user_attendee AS cal_user_attendee
+FROM egw_cal_user
+JOIN egw_cal_user euser ON euser.cal_id=egw_cal_user.cal_id AND euser.cal_user_type='e' AND euser.cal_user_id=egw_cal_user.cal_user_id AND euser.cal_user_attendee IS NULL
+WHERE egw_cal_user.cal_user_type='e' AND egw_cal_user.cal_user_attendee IS NOT NULL
+GROUP BY egw_cal_user.cal_id,egw_cal_user.cal_user_id,egw_cal_user.cal_user_attendee", __LINE__, __FILE__) as $row)
+	{
+		$GLOBALS['egw_setup']->db->update('egw_cal_user', array(
+			'cal_user_attendee' => $row['cal_user_attendee'],
+		), array(
+			'cal_id' => $row['cal_id'],
+			'cal_user_type' => 'e',
+			'cal_user_id' => $row['cal_user_id'],
+			'cal_user_attendee IS NULL'
+		), __LINE__, __FILE__, 'calendar');
+	}
+	return $GLOBALS['setup_info']['calendar']['currentver'] = '14.3.002';
+}
