@@ -6,7 +6,7 @@
  * @link http://www.egroupware.org
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
  * @author RalfBecker@outdoor-training.de
- * @version $Id: post_install.php 51172 2015-01-09 11:01:10Z ralfbecker $
+ * @version $Id$
  */
 
 if (php_sapi_name() !== 'cli')	// security precaution: forbit calling post_install as web-page
@@ -16,7 +16,6 @@ if (php_sapi_name() !== 'cli')	// security precaution: forbit calling post_insta
 $verbose = false;
 $config = array(
 	'php'         => '/usr/bin/php',
-	'pear'        => '/usr/bin/pear',
 	'source_dir'  => '/usr/share/egroupware',
 	'data_dir'    => '/var/lib/egroupware',
 	'header'      => '$data_dir/header.inc.php',	// symlinked to source_dir by rpm
@@ -57,7 +56,7 @@ $config = array(
 	'ldap_search_filter' => '(uid=%user)',
 	'ldap_group_context' => 'ou=groups,$base',
 	'ldap_encryption_type' => '',
-	'sambaadmin/sambaSID'=> '',	// SID for sambaadmin
+	'sambaadmin/sambasid'=> '',	// SID for sambaadmin
 	'mailserver'    => '',
 	'smtpserver'    => 'localhost,25',
 	'smtp'          => '',	// see setup-cli.php --help config
@@ -70,7 +69,7 @@ $config = array(
 );
 
 // read language from LANG enviroment variable
-if (($lang = isset($_ENV['LANG']) ? $_ENV['LANG'] : $_SERVER['LANG']))
+if (($lang = isset($_ENV['LANG']) ? $_ENV['LANG'] : (isset($_SERVER['LANG']) ? $_SERVER['LANG'] : null)))
 {
 	@list($lang,$nat) = preg_split('/[_.]/',$lang);
 	if (in_array($lang.'-'.strtolower($nat),array('es-es','pt-br','zh-tw')))
@@ -93,7 +92,7 @@ function set_distro_defaults($distro=null)
 	{
 		$distro = file_exists('/etc/SuSE-release') ? 'suse' :
 			(file_exists('/etc/mandriva-release') ? 'mandriva' :
-			(file_exists('/etc/lsb-release') && preg_match('/^DISTRIB_ID="Univention"$/mi',
+			(file_exists('/etc/lsb-release') && preg_match('/^DISTRIB_ID="?Univention"?$/mi',
 				file_get_contents('/etc/lsb-release')) ? 'univention' :
 			(file_exists('/etc/debian_version') ? 'debian' : 'rh')));
 	}
@@ -102,7 +101,6 @@ function set_distro_defaults($distro=null)
 		case 'suse':
 			// openSUSE 12.1+ no longer uses php5
 			if (file_exists('/usr/bin/php5')) $config['php'] = '/usr/bin/php5';
-			if (file_exists('/usr/bin/pear5')) $config['pear'] = '/usr/bin/pear5';
 			$config['start_db'] = '/sbin/service mysql';
 			$config['autostart_db'] = '/sbin/chkconfig --level 345 mysql on';
 			$config['start_webserver'] = '/sbin/service apache2';
@@ -261,9 +259,6 @@ foreach(array('php','source_dir','data_dir','setup-cli') as $name)
 // fix important php.ini and conf.d/*.ini settings
 check_fix_php_apc_ini();
 
-// install/upgrade required pear packages
-check_install_pear_packages();
-
 $setup_cli = $config['php'].' -d memory_limit=256M '.$config['setup-cli'];
 
 if (!file_exists($config['header']) || filesize($config['header']) < 200)	// default header redirecting to setup is 147 bytes
@@ -305,7 +300,7 @@ if (!file_exists($config['header']) || filesize($config['header']) < 200)	// def
 		foreach(array(
 			'domain','ldap_suffix','ldap_host','ldap_admin','ldap_admin_pw',	// non-egw params: only used for create
 			'ldap_base','ldap_root_dn','ldap_root_pw','ldap_context','ldap_search_filter','ldap_group_context',	// egw params
-			'ldap_encryption_type', 'sambaadmin/sambaSID',
+			'ldap_encryption_type', 'sambaadmin/sambasid',
 		) as $name)
 		{
 			if (strpos($value=$config[$name],'$') !== false)
@@ -406,7 +401,10 @@ else
 	// get user from header and replace password, as we dont know it
 	$old_password = patch_header($config['header'],$config['config_user'],$config['config_passwd']);
 	// register a shutdown function to put old password back in any case
-	register_shutdown_function('patch_header',$config['header'],$config['config_user'],$old_password);
+	register_shutdown_function(function() use (&$config, $old_password)
+	{
+		patch_header($config['header'], $config['config_user'], $old_password);
+	});
 
 	// update egroupware
 	$setup_update = $setup_cli.' --update '.escapeshellarg('all,'.$config['config_user'].','.$config['config_passwd'].',,'.$config['install-update-app']);
@@ -578,152 +576,6 @@ function usage($error=null)
 	exit(0);
 }
 
-/**
- * Get installed pear packages, optional from a certain channel
- *
- * @global type $config
- * @param string $channel=''
- * @return null|array with package => version
- */
-function pear_list($channel='')
-{
-	global $config;
-
-	$out = $ret = null;
-	exec($config['pear'].' list'.($channel?' -c '.$channel:''),$out,$ret);
-	if ($channel && $ret == 1)
-	{
-		return null;
-	}
-	if ($ret)
-	{
-		echo "Error running pear command ($config[pear])!\n";
-		exit(95);
-	}
-	$packages_installed = array();
-	foreach($out as $line)
-	{
-		$matches = null;
-		if (preg_match('/^([a-z0-9_]+)\s+([0-9.]+[a-z0-9]*)\s+([a-z]+)/i',$line,$matches))
-		{
-			$packages_installed[($channel?$channel.'/':'').$matches[1]] = $matches[2];
-		}
-	}
-	return $packages_installed;
-}
-
-/**
- * Check if required PEAR packges are installed and install them if not, update pear packages with to low version
- */
-function check_install_pear_packages()
-{
-	global $config;
-	$packages_installed = pear_list();
-
-	// some setup files use autoloader
-	define('EGW_SERVER_ROOT', dirname(dirname(__DIR__)));
-	define('EGW_INCLUDE_ROOT', EGW_SERVER_ROOT);
-	define('EGW_API_INC', EGW_SERVER_ROOT.'/phpgwapi/inc');
-	include_once(EGW_API_INC.'/common_functions.inc.php');
-
-	// read required packages from apps
-	$packages = array('PEAR' => true, 'HTTP_WebDAV_Server' => '999.egw-pear');	// pear must be the first, to run it's update first!
-	$channels = array();
-	$egw_pear_packages = array();
-	$setup_info = array();
-	foreach(scandir($config['source_dir']) as $app)
-	{
-		if (is_dir($dir=$config['source_dir'].'/'.$app) && file_exists($file=$dir.'/setup/setup.inc.php')) include $file;
-	}
-	foreach($setup_info as $app => $data)
-	{
-		if (isset($data['check_install']))
-		{
-			foreach($data['check_install'] as $package => $args)
-			{
-				if ($args['func'] == 'pear_check')
-				{
-					if (!$package) $package = 'PEAR';
-					// if package is prefixed with a channel, list or discover it first
-					if (strpos($package, '/'))
-					{
-						list($channel) = explode('/', $package);
-						if (!in_array($channel, $channels))
-						{
-							if (($channel_packages = pear_list($channel)))
-							{
-								$packages_installed += $channel_packages;
-							}
-							else
-							{
-								$discover_cmd = $config['pear'].' channel-discover '.$channel;
-								echo "$discover_cmd\n";	system($discover_cmd);
-							}
-							$channels[] = $channel;
-						}
-					}
-					// only overwrite lower version or no version
-					if (!isset($packages[$package]) || $packages[$package] === true || isset($args['version']) && version_compare($args['version'],$packages[$package],'>'))
-					{
-						$packages[$package] = isset($args['version']) ? $args['version'] : true;
-					}
-				}
-			}
-		}
-		if ($app == 'egw-pear')
-		{
-			$egw_pear_packages['HTTP_WebDAV_Server'] = $egw_pear_packages['Net_IMAP'] = '999.egw-pear';
-		}
-	}
-	//echo 'Installed: '; print_r($packages_installed);
-	//echo 'egw-pear: '; print_r($egw_pear_packages);
-	//echo 'Required: '; print_r($packages);
-	$to_install = array_diff(array_keys($packages),array_keys($packages_installed),array_keys($egw_pear_packages));
-
-	$need_upgrade = array();
-	foreach($packages as $package => $version)
-	{
-		if ($version !== true && $version !== '999.egw-pear' && isset($packages_installed[$package]) &&
-			version_compare($version, $packages_installed[$package], '>'))
-		{
-			$need_upgrade[] = $package;
-		}
-	}
-	//echo 'Need upgrade: '; print_r($need_upgrade);
-	//echo 'To install: '; print_r($to_install);
-	if (($to_install || $need_upgrade))
-	{
-		if (getmyuid())
-		{
-			echo "You need to run as user root to be able to install/upgrade required PEAR packages!\n";
-		}
-		else
-		{
-			echo "Install/upgrade required PEAR packages:\n";
-			// need to run upgrades first, they might be required for install!
-			if ($need_upgrade)
-			{
-				if (in_array('PEAR',$need_upgrade))	// updating pear itself can be very tricky, this is what's needed for stock RHEL pear
-				{
-					$cmd = $config['pear'].' channel-update pear.php.net';
-					echo "$cmd\n";	system($cmd);
-					$cmd = $config['pear'].' upgrade --force Console_Getopt Archive_Tar';
-					echo "$cmd\n";	system($cmd);
-				}
-				$cmd = $config['pear'].' upgrade '.implode(' ',$need_upgrade);
-				echo "$cmd\n";	system($cmd);
-			}
-			if ($to_install)
-			{
-				$cmd = $config['pear'].' install '.implode(' ', $to_install);
-				echo "$cmd\n";	system($cmd);
-			}
-		}
-	}
-}
-
-function lang() {}	// required to be able to include */setup/setup.inc.php files
-
 /**
  * fix egw_cache perms evtl. created by root, stoping webserver from accessing it
  */
@@ -763,6 +615,14 @@ function set_univention_defaults()
 	set_distro_defaults('debian');
 	$config['distro'] = 'univention';
 
+	// set lang from ucr locale, as cloud-config at least never has anything but EN set in enviroment
+	@list($lang,$nat) = preg_split('/[_.]/', _ucr_get('locale/default'));
+	if (in_array($lang.'-'.strtolower($nat),array('es-es','pt-br','zh-tw')))
+	{
+		$lang .= '-'.strtolower($nat);
+	}
+	$config['lang'] = $lang;
+
 	// mysql settings
 	$config['db_root_pw'] = _ucr_secret('mysql');
 
@@ -787,7 +647,7 @@ function set_univention_defaults()
 		$config['account-auth'] = 'univention,ldap';
 
 		// set sambaadmin sambaSID
-		$config['sambaadmin/sambaSID'] = exec('/usr/bin/univention-ldapsearch -x "(objectclass=sambadomain)" sambaSID|sed -n "s/sambaSID: \(.*\)/\1/p"');
+		$config['sambaadmin/sambasid'] = exec('/usr/bin/univention-ldapsearch -x "(objectclass=sambadomain)" sambaSID|sed -n "s/sambaSID: \(.*\)/\1/p"');
 
 		// mailserver, see setup-cli.php --help config
 		if (($mailserver = exec('/usr/bin/univention-ldapsearch -x "(univentionAppID=mailserver_*)" univentionAppInstalledOnServer|sed -n "s/univentionAppInstalledOnServer: \(.*\)/\1/p"')) &&
@@ -800,15 +660,27 @@ function set_univention_defaults()
 			$config['smtpserver'] = "$mailserver,465,,,yes,tls,no,yes";
 			$config['smtp'] = ',emailadmin_smtp_univention';
 			$config['mailserver'] = "$mailserver,993,$domain,email,tls";
-			$config['imap'] = /*'cyrus,'._ucr_secret('cyrus')*/','.',emailadmin_imap_cyrus';
+			if (_ucr_get('mail/dovecot') == 'yes')
+			{
+				$config['imap'] = /*'cyrus,'._ucr_secret('cyrus')*/','.',emailadmin_imap_dovecot';
+				// default with sieve port to 4190, as config is only available on host mailserver app is installed
+				if (!($sieve_port = _ucr_get('mail/dovecot/sieve/port'))) $sieve_port = 4190;
+			}
+			else
+			{
+				$config['imap'] = /*'cyrus,'._ucr_secret('cyrus')*/','.',emailadmin_imap_cyrus';
+				// default with sieve port to 4190, as config is only available on host mailserver app is installed
+				if (!($sieve_port = _ucr_get('mail/cyrus/sieve/port'))) $sieve_port = 4190;
+			}
 			// set folders so mail creates them on first login, UCS does not automatic
 			$config['folder'] = 'INBOX/Sent,INBOX/Trash,INBOX/Drafts,INBOX/Templates,INBOX/Spam';
-			// default with sieve port to 4190, as config is only available on host mailserver app is installed
-			if (!($sieve_port = _ucr_get('mail/cyrus/sieve/port'))) $sieve_port = 4190;
 			$config['sieve'] = "$mailserver,$sieve_port,starttls";
 			// set an email address for sysop user so mail works right away
 			$config['admin_email'] = '$admin_user@'.$domain;
 		}
+		# add directory of univention-directory-manager and it's sysmlink target to open_basedir
+		system("/bin/sed -i 's|/usr/bin|/usr/bin:/usr/sbin:/usr/share/univention-directory-manager-tools|' /etc/egroupware/apache.conf");
+
 	}
 }
 
@@ -858,7 +730,7 @@ function _ucr_secret($name)
  */
 function check_fix_php_apc_ini()
 {
-	if (extension_loaded('apc'))
+	if (extension_loaded('apc') || extension_loaded('apcu'))
 	{
 		$shm_size = ini_get('apc.shm_size');
 		$shm_segments = ini_get('apc.shm_segments');
@@ -875,7 +747,7 @@ function check_fix_php_apc_ini()
 			phpinfo();
 			$phpinfo = ob_get_clean();
 			$matches = null;
-			if (preg_match('#(/[a-z0-5./-]+apcu?.ini)(,| |$)#mi', $phpinfo, $matches) &&
+			if (preg_match('#(/[a-z0-9./-]+apcu?.ini)(,| |$)#mi', $phpinfo, $matches) &&
 				file_exists($path = $matches[1]) && ($apc_ini = file_get_contents($path)))
 			{
 				$new_shm_size = 128 / $shm_segments;
