@@ -6,9 +6,9 @@
  * @package emailadmin
  * @author Ralf Becker <rb@stylite.de>
  * @author Stylite AG <info@stylite.de>
- * @copyright (c) 2013-14 by Ralf Becker <rb@stylite.de>
+ * @copyright (c) 2013-16 by Ralf Becker <rb@stylite.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.emailadmin_account.inc.php 49750 2014-12-08 16:23:23Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -55,10 +55,11 @@
  * @property-read string $acc_imap_logintype how to construct login-name standard, vmailmgr, admin, uidNumber
  * @property-read string $acc_domain domain name
  * @property-read boolean $acc_imap_administration enable administration
- * @property-read string $acc_admin_username
- * @property-read string $acc_admin_password
+ * @property-read string $acc_imap_admin_username
+ * @property-read string $acc_imap_admin_password
  * @property-read boolean $acc_further_identities are non-admin users allowed to create further identities
  * @property-read boolean $acc_user_editable are non-admin users allowed to edit this account, if it is for them
+ * @property-read boolean $acc_user_forward are non-admin users allowed change forwards
  * @property-read int $acc_modified timestamp of last modification
  * @property-read int $acc_modifier account_id of last modifier
  * @property-read int $ident_id standard identity
@@ -483,7 +484,7 @@ class emailadmin_account implements ArrayAccess
 		if (!isset($this->smtpTransport))
 		{
 			$secure = false;
-			switch($this->acc_imap_ssl & ~self::SSL_VERIFY)
+			switch($this->acc_smtp_ssl & ~self::SSL_VERIFY)
 			{
 				case self::SSL_STARTTLS:
 					$secure = 'tls';	// Horde uses 'tls' for STARTTLS, not ssl connection with tls version >= 1 and no sslv2/3
@@ -704,7 +705,7 @@ class emailadmin_account implements ArrayAccess
 				$err = null;
 				$value = $merge->merge_string($value,
 					(array)accounts::id2name($account_id, 'person_id'),
-					$err, $name == 'ident_signature' ? 'text/html' : 'text/plain');
+					$err, $name == 'ident_signature' ? 'text/html' : 'text/plain', null, 'utf-8');
 			}
 		}
 		//error_log(__METHOD__."(".array2string($identity).") returning ".array2string($to_replace));
@@ -1069,7 +1070,7 @@ class emailadmin_account implements ArrayAccess
 	 */
 	protected static function db2data(array $data)
 	{
-		foreach(array('acc_sieve_enabled','acc_further_identities','acc_user_editable','acc_smtp_auth_session') as $name)
+		foreach(array('acc_sieve_enabled','acc_further_identities','acc_user_editable','acc_smtp_auth_session','acc_user_forward') as $name)
 		{
 			if (isset($data[$name]))
 			{
@@ -1164,6 +1165,13 @@ class emailadmin_account implements ArrayAccess
 		// add imap credentials
 		$cred_type = $data['acc_imap_username'] == $data['acc_smtp_username'] &&
 			$data['acc_imap_password'] == $data['acc_smtp_password'] ? 3 : 1;
+ 		// if both passwords are unavailable, they seem identical, do NOT store them together, as they are not!
+		if ($cred_type == 3 && $data['acc_imap_password'] == emailadmin_credentials::UNAVAILABLE &&
+			$data['acc_imap_password'] == emailadmin_credentials::UNAVAILABLE &&
+			$data['acc_imap_cred_id'] != $data['acc_smtp_cred_id'])
+		{
+			$cred_type = 1;
+		}
 		emailadmin_credentials::write($data['acc_id'], $data['acc_imap_username'], $data['acc_imap_password'],
 			$cred_type, $valid_for, $data['acc_imap_cred_id']);
 		// add smtp credentials if necessary and different from imap
@@ -1173,6 +1181,11 @@ class emailadmin_account implements ArrayAccess
 				2, $valid_for, $data['acc_smtp_cred_id'] != $data['acc_imap_cred_id'] ?
 					$data['acc_smtp_cred_id'] : null);
 		}
+		// delete evtl. existing SMTP credentials, after storing IMAP&SMTP together now
+		elseif ($data['acc_smtp_cred_id'])
+		{
+			emailadmin_credentials::delete($data['acc_id'], $valid_for, emailadmin_credentials::SMTP, true);
+		}
 		// store or delete admin credentials
 		if ($data['acc_imap_admin_username'] && $data['acc_imap_admin_password'])
 		{
@@ -1211,7 +1224,7 @@ class emailadmin_account implements ArrayAccess
 	 *
 	 * @param array|emailadmin_account $account
 	 * @param int $account_id =null
-	 * @return boolean
+	 * @return int account_id for whom credentials are valid or 0 for all
 	 */
 	protected static function credentials_valid_for($account, $account_id=null)
 	{
@@ -1300,11 +1313,14 @@ class emailadmin_account implements ArrayAccess
 		if ($only_current_user)
 		{
 			$account_id = $only_current_user === true ? $GLOBALS['egw_info']['user']['account_id'] : $only_current_user;
-			if (!is_numeric($account_id))
+			// no account_id happens eg. for notifications during login
+			if ($account_id && !is_numeric($account_id))
 			{
 				throw new egw_exception_wrong_parameter(__METHOD__."(".array2string($only_current_user).") is NO valid account_id");
 			}
-			$where[] = self::$db->expression(self::VALID_TABLE, self::VALID_TABLE.'.', array('account_id' => self::memberships($account_id)));
+			$where[] = self::$db->expression(self::VALID_TABLE, self::VALID_TABLE.'.', array(
+				'account_id' => $account_id ? self::memberships($account_id) : 0
+			));
 		}
 		if (empty($order_by) || !preg_match('/^[a-z_]+ (ASC|DESC)$/i', $order_by))
 		{
@@ -1398,9 +1414,10 @@ class emailadmin_account implements ArrayAccess
 	 *
 	 * @param boolean $smtp =false false: usable for IMAP, true: usable for SMTP
 	 * @param boolean $return_id =false true: return acc_id, false return account object
+	 * @param boolean $log_no_default =true true: error_log if no default found, false be silent
 	 * @return emailadmin_account|null
 	 */
-	static function get_default($smtp=false, $return_id=false)
+	static function get_default($smtp=false, $return_id=false, $log_no_default=true)
 	{
 		try
 		{
@@ -1410,7 +1427,8 @@ class emailadmin_account implements ArrayAccess
 				{
 					if (!$params['acc_smtp_host'] || !$params['acc_smtp_port']) continue;
 					// check requirement of session, which is not available in async service!
-					if (isset($GLOBALS['egw_info']['flags']['async-service']))
+					if (isset($GLOBALS['egw_info']['flags']['async-service']) ||
+						empty($GLOBALS['egw_info']['user']['account_id']))	// happens during login when notifying about blocked accounts
 					{
 						if ($params['acc_smtp_auth_session']) continue;
 						// may fail because of smtp only profile, or no session password, etc
@@ -1439,7 +1457,7 @@ class emailadmin_account implements ArrayAccess
 		}
 		catch (Exception $e)
 		{
-			error_log(__METHOD__.__LINE__.' Error no Default available.'.$e->getMessage());
+			if ($log_no_default) error_log(__METHOD__.__LINE__.' Error no Default available.'.$e->getMessage());
 		}
 		return null;
 	}
@@ -1498,7 +1516,13 @@ class emailadmin_account implements ArrayAccess
 						$account = array_merge($account, emailadmin_credentials::from_session($account));
 					}
 				}
-				if (empty($account['ident_email']) && !empty($account['acc_imap_username']))
+				// fill an empty ident_realname or ident_email of current user with data from user account
+				if ($replace_placeholders && (!isset($account_id) || $account_id == $GLOBALS['egw_info']['user']['acount_id']))
+				{
+					if (empty($account['ident_realname'])) $account['ident_realname'] = $GLOBALS['egw_info']['user']['account_fullname'];
+					if (empty($account['ident_email'])) $account['ident_email'] = $GLOBALS['egw_info']['user']['account_email'];
+				}
+				if (empty($account['ident_email']) && !empty($account['acc_imap_username']) && strpos($account['acc_imap_username'], '@') !== false)
 				{
 					$account['ident_email'] = $account['acc_imap_username'];
 				}
