@@ -7,7 +7,7 @@
  * @author Ralf Becker <rb@stylite.de>
  * @author Stylite AG <info@stylite.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.emailadmin_imap.inc.php 51079 2014-12-18 13:42:27Z leithoff $
+ * @version $Id$
  */
 
 require_once EGW_INCLUDE_ROOT.'/emailadmin/inc/class.defaultimap.inc.php';
@@ -189,6 +189,23 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 				)),
 			);
 		}
+		// switch to allow to disable some capabilites known to be troublesome
+		switch (strtolower(trim($this->params['acc_imap_host'])))
+		{
+			case 'imap.yandex.ru':
+			case 'imap.yandex.com':
+				// imap.yandex.com - reports BINARY (server side decoding) but does not decode but
+				// returns undecoded bodyParts AND reports an encoding for the returned part.
+				// expected behavior would be: if server side decoding succeeds , horde should
+				// either report 7bit or 8bit when calling getBodyPartDecode. if it fails or BINARY
+				// is not supported NULL is expected on getBodyPartDecode
+				// yandex.com does not succeed in decoding but getBodyPartDecode is reported as 7bit/8bit
+				// as we have no way to tell this apart we ignore BINARY this affects
+				// Horde_Imap_Client_Fetch_Query::bodyPart for its fetch parameter decode=true is ignored
+				// (other functionality depending on BINARY is, of cause, affected too)
+				$parent_params['capability_ignore']= array_merge((array)$parent_params['capability_ignore'],array('BINARY'));
+				break;
+		}
 		parent::__construct($parent_params);
 	}
 
@@ -197,7 +214,7 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 	 *
 	 * Plugins can overwrite it to eg. construct a special admin user name
 	 *
-	 * @param string $_username =null create an admin connection for given user or $this->acc_imap_username
+	 * @param string $_username =true create an admin connection for given user or $this->acc_imap_username
 	 */
 	function adminConnection($_username=true)
 	{
@@ -505,9 +522,10 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 	 * getStatus
 	 *
 	 * @param string $mailbox
+	 * @param ignoreStatusCache bool ignore the cache used for counters
 	 * @return array with counters
 	 */
-	function getStatus($mailbox)
+	function getStatus($mailbox, $ignoreStatusCache=false)
 	{
 		$mailboxes = $this->listMailboxes($mailbox,Horde_Imap_Client::MBOX_ALL,array(
 				'attributes'=>true,
@@ -516,6 +534,9 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 				'special_use'=>true,
 			));
 
+		$flags = Horde_Imap_Client::STATUS_ALL;
+		if ($ignoreStatusCache) $flags |= Horde_Imap_Client::STATUS_FORCE_REFRESH;
+
 		$mboxes = new Horde_Imap_Client_Mailbox_List($mailboxes);
 		//error_log(__METHOD__.__LINE__.array2string($mboxes->count()));
 		foreach ($mboxes->getIterator() as $k =>$box)
@@ -524,12 +545,12 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 			{
 				if (stripos(array2string($box['attributes']),'\noselect')=== false)
 				{
-					$status = $this->status($k);
+					$status = $this->status($k, $flags);
 					foreach ($status as $key => $v)
 					{
 						$_status[strtoupper($key)]=$v;
 					}
-					$_status['HIERACHY_DELIMITER'] = $_status['delimiter'] = $box['delimiter'];//$this->getDelimiter('user');
+					$_status['HIERACHY_DELIMITER'] = $_status['delimiter'] = ($box['delimiter']?$box['delimiter']:$this->getDelimiter('personal'));
 					$_status['ATTRIBUTES'] = $box['attributes'];
 					//error_log(__METHOD__.__LINE__.$k.'->'.array2string($_status));
 					return $_status;
@@ -600,7 +621,7 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 		foreach ((array)$mailboxes as $k =>$box)
 		{
 			//error_log(__METHOD__.__LINE__.' Box:'.$k.'->'.array2string($box));
-			$ret[$k]=array('MAILBOX'=>$k,'ATTRIBUTES'=>$box['attributes'],'delimiter'=>$box['delimiter'],'SUBSCRIBED'=>true);
+			$ret[$k]=array('MAILBOX'=>$k,'ATTRIBUTES'=>$box['attributes'],'delimiter'=>($box['delimiter']?$box['delimiter']:$this->getDelimiter('personal')),'SUBSCRIBED'=>true);
 		}
 		// for unknown reasons on ALL, UNSUBSCRIBED are not returned
 		//always fetch unsubscribed, think about only fetching it when $options['attributes'] is set
@@ -616,7 +637,7 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 				//error_log(__METHOD__.__LINE__.' Box:'.$k.' already In?'.array_key_exists($k,$boxexists).'->'.array2string($box));
 				if(!array_key_exists($k,$ret))
 				{
-					$ret[$k]=array('MAILBOX'=>$k,'ATTRIBUTES'=>$box['attributes'],'delimiter'=>$box['delimiter'],'SUBSCRIBED'=>false);
+					$ret[$k]=array('MAILBOX'=>$k,'ATTRIBUTES'=>$box['attributes'],'delimiter'=>($box['delimiter']?$box['delimiter']:$this->getDelimiter('personal')),'SUBSCRIBED'=>false);
 				}
 				else
 				{
@@ -689,7 +710,7 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 			}
 			else
 			{
-				$ret[$k]=array('MAILBOX'=>$k,'ATTRIBUTES'=>$box['attributes'],'delimiter'=>$box['delimiter'],'SUBSCRIBED'=>true);
+				$ret[$k]=array('MAILBOX'=>$k,'ATTRIBUTES'=>$box['attributes'],'delimiter'=>($box['delimiter']?$box['delimiter']:$this->getDelimiter('personal')),'SUBSCRIBED'=>true);
 			}
 		}
 		return $ret;
@@ -946,6 +967,7 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 		static $nameSpace=null;
 		$foldersNameSpace = array();
 		$delimiter = $this->getDelimiter();
+		if (empty($delimiter)) $delimiter='/';
 		if (is_null($nameSpace)) $nameSpace = $this->getNameSpaceArray();
 		if (is_array($nameSpace)) {
 			foreach($nameSpace as $type => $singleNameSpaceArray)
@@ -1004,7 +1026,7 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 		$namespaces = $this->getNamespaces();
 		foreach ($namespaces as $nsp)
 		{
-			if ($nsp['type']==$type) return $nsp['delimiter'];
+			if ($nsp['type']==$type && $nsp['delimiter']) return $nsp['delimiter'];
 		}
 		return "/";
 	}
@@ -1122,7 +1144,7 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 			$username .= '@'. $this->domainName;
 		}
 
-		$mailboxString = $nameSpaces['others'][0]['name'] . $username . (!empty($_folderName) ? $nameSpaces['others'][0]['delimiter'] . $_folderName : '');
+		$mailboxString = $nameSpaces['others'][0]['name'] . $username . (!empty($_folderName) ? ($nameSpaces['others'][0]['delimiter']?$nameSpaces['others'][0]['delimiter']:'/') . $_folderName : '');
 
 		return $mailboxString;
 	}
@@ -1151,7 +1173,7 @@ class emailadmin_imap extends Horde_Imap_Client_Socket implements defaultimap
 					'name' => $data['name'],
 					'prefix' => $data['name'],
 					'prefix_present' => !empty($data['name']),
-					'delimiter' => $data['delimiter'],
+					'delimiter' => ($data['delimiter']?$data['delimiter']:'/'),
 				);
 			}
 		}
