@@ -10,7 +10,7 @@
  * @author Hadi Nategh	<hn@stylite.de>
  * @copyright 2002 by Stephen Grier <stephengrier@users.sourceforge.net>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.emailadmin_script.inc.php 47268 2014-06-16 12:45:23Z hnategh $
+ * @version $Id$
  */
 
 /**
@@ -92,22 +92,19 @@ class emailadmin_script {
 			return false;
 		}
 
-		// if script doesn't yet exist, nothing to retrieve.
-		// safe to write to this script file.
-	#LK	if (!AppSession::scriptExists($this->name)) {
-	#LK		$this->so = true;
-	#LK		return true;
-	#LK	}
+		// If the called script name is not exist then create it
+		// otherwise we might get error due to non existance script
+		if (!in_array($this->name, $connection->listScripts()))
+		{
+			$this->updateScript($connection);
+		}
 
-		#print "<br><br><br><br>get Script ". $this->name ."<bR>";
-
-		if(PEAR::isError($script = $connection->getScript($this->name))) {
+		if(self::isError($script = $connection->getScript($this->name))) {
 			if ($this->debug) error_log(__CLASS__.'::'.__METHOD__.": error retrieving script: ".$script->getMessage());
 			return $script;
 		}
-
 		#print "<br>AAA: Script is ". htmlentities($script) ."<br>";
-		$lines = array();
+
 		$lines = preg_split("/\n/",$script); //,PREG_SPLIT_NO_EMPTY);
 
 		$rules = array();
@@ -125,8 +122,8 @@ class emailadmin_script {
 
 		/* next line should be the recognised encoded head. if not, the script
 		 * is of an unrecognised format, and we should not overwrite it. */
-		$line = array_shift($lines);
-		if (!preg_match("/^# ?Mail(.*)rules for/", $line)){
+		$line1 = array_shift($lines);
+		if (!preg_match("/^# ?Mail(.*)rules for/", $line1)){
 				$this->errstr = 'retrieveRules: encoding not recognised';
 				$this->so = false;
 				if ($this->debug) error_log(__CLASS__.'::'.__METHOD__.": encoding not recognised");
@@ -136,7 +133,9 @@ class emailadmin_script {
 
 		$line = array_shift($lines);
 
-		while (isset($line)){
+		while (isset($line))
+		{
+			$matches = null;
 			if (preg_match("/^ *#(#PSEUDO|rule|vacation|mode|notify)/i",$line,$matches)){
 				$line = rtrim($line);
 				switch ($matches[1]){
@@ -182,9 +181,7 @@ class emailadmin_script {
 						if (preg_match("/^ *#vacation&&(.*)&&(.*)&&(.*)&&(.*)&&(.*)/i",$line,$bits) ||
 							preg_match("/^ *#vacation&&(.*)&&(.*)&&(.*)&&(.*)/i",$line,$bits)) {
 							$vacation['days'] = $bits[1];
-							$vaddresslist = $bits[2];
-							$vaddresslist = preg_replace("/\"|\s/","",$vaddresslist);
-							$vaddresses = array();
+							$vaddresslist = preg_replace("/\"|\s/","",$bits[2]);
 							$vaddresses = preg_split("/,/",$vaddresslist);
 							$vacation['text'] = $bits[3];
 
@@ -229,6 +226,7 @@ class emailadmin_script {
 		$this->script = $script;
 		$this->rules = $rules;
 		$this->vacation = $vacation;
+		if (!(in_array('vacation',$connection->_capability['extensions'])|| in_array('VACATION', $connection->_capability['extensions']))) $this->vacation = false;
 		$this->emailNotification = $emailNotification; // Added email notifications
 		if ($this->debug) error_log(__CLASS__.'::'.__METHOD__.": Script succesful retrieved: ".print_r($vacation,true));
 
@@ -236,19 +234,25 @@ class emailadmin_script {
 	}
 
 
-	// update and save sieve script
-	function updateScript ($connection)
+	/**
+	 * update and save sieve script
+	 *
+	 * @param emailadmin_imap $connection
+	 * @param boolean $utf7imap_fileinto =false true: encode foldernames with utf7imap, default utf8
+	 */
+	function updateScript ($connection, $utf7imap_fileinto=false)
 	{
 		#global $_SESSION,$default,$sieve;
 		global $default,$sieve;
 
 		$activerules = 0;
 		$regexused = 0;
+		$regexsupported = true;
 		$rejectused = 0;
 		$vacation_active = false;
 
 		$username	= $GLOBALS['egw_info']['user']['account_lid'];
-		$version	= $GLOBALS['egw_info']['apps']['felamimail']['version'];
+		$version	= $GLOBALS['egw_info']['apps']['mail']['version'];
 
 		//include "$default->lib_dir/version.php";
 
@@ -266,11 +270,14 @@ class emailadmin_script {
 #LK		}
 
 		// lets generate the main body of the script from our rules
-
+		//error_log(__METHOD__.__LINE__.array2string($connection->_capability));
 		$enotify = $variables= $supportsbody = false;
 		if (in_array('enotify',$connection->_capability['extensions'])|| in_array('ENOTIFY', $connection->_capability['extensions'])) $enotify = true;
 		if (in_array('variables',$connection->_capability['extensions'])|| in_array('VARIABLES', $connection->_capability['extensions'])) $variables = true;
 		if (in_array('body', $connection->_capability['extensions']) || in_array('BODY', $connection->_capability['extensions'])) $supportsbody = true;
+		if (!(in_array('vacation',$connection->_capability['extensions'])|| in_array('VACATION', $connection->_capability['extensions']))) $this->vacation = false;
+		if (!(in_array('regex',$connection->_capability['extensions'])|| in_array('REGEX', $connection->_capability['extensions']))) $regexsupported = false;
+
 		$newscriptbody = "";
 		$continue = 1;
 
@@ -381,7 +388,8 @@ class emailadmin_script {
 				if (!$rule['unconditional']) $newruletext .= ") {\n\t";
 
 				if (preg_match("/folder/i",$rule['action'])) {
-						$newruletext .= "fileinto \"" . $rule['action_arg'] . "\";";
+						$newruletext .= "fileinto \"" . ($utf7imap_fileinto ?
+							translation::convert($rule['action_arg'],'utf-8', 'utf7-imap') : $rule['action_arg']) . "\";";
 				}
 				if (preg_match("/reject/i",$rule['action'])) {
 						$newruletext .= "reject text: \n" . $rule['action_arg'] . "\n.\n;";
@@ -428,7 +436,7 @@ class emailadmin_script {
 				$defaultaddr = $sieve->user . '@' . $sieve->maildomain;
 				array_push($vacation['addresses'],$defaultaddr);
 			}
-			if ($vacation['status'] == 'on' || $vacation['status'] == 'by_date' &&
+			if (($vacation['status'] == 'on' && strlen(trim($vacation['text']))>0)|| $vacation['status'] == 'by_date' &&
 				$vacation['start_date'] <= time() && time() < $vacation['end_date']+24*3600)	// +24*3600 to include the end_date day
 			{
 				if (trim($vacation['forwards'])) {
@@ -443,7 +451,21 @@ class emailadmin_script {
 					$newscriptbody .= "\tkeep;\n}\n";
 				}
 				$vacation_active = true;
-				$newscriptbody .= "if header :contains ".'"X-Spam-Status" '.'"YES"'."{\n\tstop;\n}\n"; //stop vacation reply if it is spam
+				if ($vacation['text'])
+				{
+					if ($regexsupported)
+					{
+						$newscriptbody .= "if header :regex ".'"X-Spam-Status" '.'"\\\\bYES\\\\b"'."{\n\tstop;\n}\n"; //stop vacation reply if it is spam
+						$regexused = 1;
+					}
+					else
+					{
+						// if there are no regex'es supported use a different Anti-Spam Rule: if X-Spam-Status holds
+						// additional spamscore information (e.g. BAYES) this rule may prevent Vacation notification
+						// TODO: refine rule without using regex
+						$newscriptbody .= "if header :contains ".'"X-Spam-Status" '.'"YES"'."{\n\tstop;\n}\n"; //stop vacation reply if it is spam
+					}
+				}
 				$newscriptbody .= "vacation :days " . $vacation['days'] . " :addresses [";
 				$first = 1;
 				foreach ($vacation['addresses'] as $vaddress) {
@@ -515,12 +537,12 @@ class emailadmin_script {
 
 		$newscripthead = "";
 		$newscripthead .= "#Mail filter rules for " . $username . "\n";
-		$newscripthead .= '#Generated by ' . $username . ' using FeLaMiMail ' . $version . ' ' . date($default->script_date_format);
+		$newscripthead .= '#Generated by ' . $username . ' using Mail ' . $version . ' ' . date($default->script_date_format);
 		$newscripthead .= "\n";
 
 		if ($activerules) {
 			$newscripthead .= "require [\"fileinto\"";
-			if ($regexused) $newscripthead .= ",\"regex\"";
+			if ($regexsupported && $regexused) $newscripthead .= ",\"regex\"";
 			if ($rejectused) $newscripthead .= ",\"reject\"";
 			if ($this->vacation && $vacation_active) {
 				$newscripthead .= ",\"vacation\"";
@@ -530,9 +552,25 @@ class emailadmin_script {
 			$newscripthead .= "];\n\n";
 		} else {
 			// no active rules, but might still have an active vacation rule
+			$closeRequired = false;
 			if ($this->vacation && $vacation_active)
-				$newscripthead .= "require [\"vacation\"];\n\n";
-			if ($this->emailNotification && $this->emailNotification['status'] == 'on') $newscripthead .= "require [\"".($enotify?'e':'')."notify\"".($variables?',"variables"':'')."];\n\n"; // Added email notifications
+			{
+				$newscripthead .= "require [\"vacation\"";
+				if ($regexsupported && $regexused) $newscripthead .= ",\"regex\"";
+				$closeRequired=true;
+			}
+			if ($this->emailNotification && $this->emailNotification['status'] == 'on')
+			{
+				if ($this->vacation && $vacation_active)
+				{
+					$newscripthead .= ",\"".($enotify?'e':'')."notify\"".($variables?',"variables"':'')."];\n\n"; // Added email notifications
+				}
+				else
+				{
+					$newscripthead .= "require [\"".($enotify?'e':'')."notify\"".($variables?',"variables"':'')."];\n\n"; // Added email notifications
+				}
+			}
+			if ($closeRequired) $newscripthead .= "];\n\n";
 		}
 
 		// generate the encoded script foot
@@ -587,11 +625,11 @@ class emailadmin_script {
 		$this->script = $newscript;
 		//error_log(__METHOD__.__LINE__.array2string($newscript));
 		//print "<pre>$newscript</pre>"; exit;
-		$scriptfile = $this->name;
 		//print "<hr><pre>".htmlentities($newscript)."</pre><hr>";
 		$ret = $connection->installScript($this->name, $newscript, true);
-		if (!$ret || PEAR::isError($ret)) {
-			$this->errstr = 'updateScript: putscript failed: ' . (PEAR::isError($ret)?$ret->message:$connection->errstr);
+		if (!$ret || self::isError($ret)) {
+			$this->errstr = 'updateScript: putscript failed: ' . (self::isError($ret)?$ret->message:$connection->errstr);
+			if ($regexused&&!$regexsupported) $this->errstr .= " REGEX is not an supported CAPABILITY";
 			error_log(__METHOD__.__LINE__.' # Error: ->'.$this->errstr);
 			error_log(__METHOD__.__LINE__.' # ScriptName:'.$this->name.' Script:'.$newscript);
 			error_log(__METHOD__.__LINE__.' # Instance='.$GLOBALS['egw_info']['user']['domain'].', User='.$GLOBALS['egw_info']['user']['account_lid']);
@@ -600,4 +638,25 @@ class emailadmin_script {
 
 		return true;
 	}
+
+	/**
+	 * Tell whether a value is a PEAR error.
+	 *
+	 * Implemented here to get arround: PHP Deprecated:  Non-static method self::isError() should not be called statically
+	 *
+	 * @param   mixed $data   the value to test
+	 * @param   int   $code   if $data is an error object, return true
+	 *                        only if $code is a string and
+	 *                        $obj->getMessage() == $code or
+	 *                        $code is an integer and $obj->getCode() == $code
+	 * @access  public
+	 * @return  bool    true if parameter is an error
+	 */
+	protected static function isError($data, $code = null)
+	{
+		static $pear=null;
+		if (!isset($pear)) $pear = new PEAR();
+
+		return $pear->isError($data, $code);
+	}
 }
