@@ -7,7 +7,7 @@
  * @author Ralf Becker <rb@stylite.de>
  * @copyright (c) 2013-14 by Ralf Becker <rb@stylite.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.emailadmin_wizard.inc.php 49519 2014-11-20 12:47:58Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -17,6 +17,7 @@
  * 1. query Mozilla ISPDB for domain from email (perfering SSL over STARTTLS over insecure connection)
  * 2. guessing and verifying in DNS server-names based on domain from email:
  *	- (imap|smtp).$domain, mail.$domain
+ *  - MX is *.mail.protection.outlook.com use (outlook|smtp).office365.com
  *  - MX for $domain
  *  - replace host in MX with (imap|smtp) or mail
  */
@@ -29,7 +30,7 @@ class emailadmin_wizard
 	/**
 	 * Connection timeout in seconds used in autoconfig, can and should be really short!
 	 */
-	const TIMEOUT = 2;
+	const TIMEOUT = 3;
 	/**
 	 * Prefix for callback names
 	 *
@@ -73,6 +74,7 @@ class emailadmin_wizard
 	public $public_functions = array(
 		'add' => true,
 		'edit' => true,
+		'ajax_activeAccounts' => true
 	);
 
 	/**
@@ -131,7 +133,7 @@ class emailadmin_wizard
 	 *
 	 * @var array
 	 */
-	public static $no_sieve_blacklist = array('gmail.com', 'googlemail.com');
+	public static $no_sieve_blacklist = array('gmail.com', 'googlemail.com', 'outlook.office365.com');
 
 	/**
 	 * Is current use a mail administrator / has run rights for EMailAdmin
@@ -849,6 +851,23 @@ class emailadmin_wizard
 				if ($content['accounts'])
 				{
 					list($content['acc_id']) = each($content['accounts']);
+					//error_log(__METHOD__.__LINE__.'.'.array2string($content['acc_id']));
+					// test if the "to be selected" acccount is imap or not
+					if (count($content['accounts'])>1 && emailadmin_account::is_multiple($content['acc_id']))
+					{
+						try {
+							$account = emailadmin_account::read($content['acc_id'], $content['called_for']);
+							//try to select the first account that is of type imap
+							if (!$account->is_imap())
+							{
+								list($content['acc_id']) = each($content['accounts']);
+								//error_log(__METHOD__.__LINE__.'.'.array2string($content['acc_id']));
+							}
+						}
+						catch(egw_exception_not_found $e) {
+							if (self::$debug) _egw_log_exception($e);
+						}
+					}
 				}
 				if (!$content['accounts'])	// no email account, call wizard
 				{
@@ -1066,12 +1085,20 @@ class emailadmin_wizard
 									$msg = lang('Notification folders updated.');
 								}
 							}
+							if ($content['acc_user_forward'] && !empty($content['acc_smtp_type']) && $content['acc_smtp_type'] != 'emailadmin_smtp')
+							{
+								$account = new emailadmin_account($content);
+								$account->smtpServer()->saveSMTPForwarding($content['called_for'] ?
+									$content['called_for'] : $GLOBALS['egw_info']['user']['account_id'],
+									$content['mailForwardingAddress'],
+									$content['forwardOnly'] ? null : 'yes');
+							}
 						}
 					}
 					catch (Horde_Imap_Client_Exception $e)
 					{
 						_egw_log_exception($e);
-						$tpl->set_validation_error('acc_imap_admin_username', $msg=lang($e->getMessage()));
+						$tpl->set_validation_error('acc_imap_admin_username', $msg=lang($e->getMessage()).($e->details?', '.lang($e->details):''));
 						$msg_type = 'error';
 						$content['tabs'] = 'emailadmin.account.imap';	// should happen automatic
 						break;
@@ -1168,6 +1195,7 @@ class emailadmin_wizard
 				}
 			}
 		}
+
 		$sel_options['acc_imap_type'] = emailadmin_base::getIMAPServerTypes(false);
 		$sel_options['acc_smtp_type'] = emailadmin_base::getSMTPServerTypes(false);
 		$sel_options['acc_imap_logintype'] = self::$login_types;
@@ -1227,7 +1255,16 @@ class emailadmin_wizard
 			is_a($content['acc_smtp_type'], 'emailadmin_smtp_ldap', true))
 		{
 			$content['no_forward_available'] = !constant($content['acc_smtp_type'].'::FORWARD_ATTR');
-			$readonlys['deliveryMode'] = !constant($content['acc_smtp_type'].'::FORWARD_ONLY_ATTR');
+			if (!constant($content['acc_smtp_type'].'::FORWARD_ONLY_ATTR'))
+			{
+				$readonlys['deliveryMode'] = true;
+			}
+		}
+
+		// account allows users to change forwards
+		if (!$edit_access && !$readonlys['tabs']['emailadmin.account.aliases'] && $content['acc_user_forward'])
+		{
+			$readonlys['mailForwardingAddress'] = false;
 		}
 
 		// allow imap classes to disable certain tabs or fields
@@ -1407,6 +1444,11 @@ class emailadmin_wizard
 		if (($dns = dns_get_record($domain, DNS_MX)))
 		{
 			//error_log(__METHOD__."('$email') dns_get_record('$domain', DNS_MX) returned ".array2string($dns));
+			// hosts for office365 are outlook|smpt.office365.com for MX *.mail.protection.outlook.com
+			if (substr($dns[0]['target'], -28) == '.mail.protection.outlook.com')
+			{
+				$hosts[($type == 'imap' ? 'outlook' : 'smtp').'.office365.com'] = true;
+			}
 			$hosts[preg_replace('/^[^.]+/', $type, $dns[0]['target'])] = true;
 			$hosts[preg_replace('/^[^.]+/', 'mail', $dns[0]['target'])] = true;
 			if ($type == 'smtp') $hosts[preg_replace('/^[^.]+/', 'send', $dns[0]['target'])] = true;
@@ -1421,4 +1463,67 @@ class emailadmin_wizard
 		//error_log(__METHOD__."('$email') returning ".array2string($hosts));
 		return $hosts;
 	}
+	
+	/**
+	 * Set mail account status wheter to 'active' or '' (inactive)
+	 *
+	 * @param array $_data account an array of data called via long task running dialog
+	 *	$_data:array (
+	 *		id => account_id,
+	 *		qouta => quotaLimit,
+	 *		domain => mailLocalAddress,
+	 *		status => mail activation status('active'|'')
+	 *	)
+	 * @return json response
+	 */
+	public function ajax_activeAccounts($_data)
+	{
+		if (!$this->is_admin) die('no rights to be here!');
+		$response = egw_json_response::get();
+$msg = 'The response is empty';
+		if (($account = $GLOBALS['egw']->accounts->read($_data['id'])))
+		{
+			if ($_data['quota'] !== '' || $_data['accountStatus'] !== ''
+				|| strpos($_data['domain'], '.'))
+			{
+				$emailadmin = emailadmin_account::get_default();
+				if (!emailadmin_account::is_multiple($emailadmin))
+				{
+					$msg = lang('No default account found!');
+					return $response->data($msg);
+				}
+
+				$ea_account = emailadmin_account::read($emailadmin->acc_id, $_data['id']);				
+				if (($userData = $ea_account->getUserData ()))
+				{
+					$userData = array(
+						'acc_smtp_type' => $ea_account->acc_smtp_type,
+						'accountStatus' => $_data['status'],
+						'quotaLimit' => $_data['qouta']? $_data['qouta']: $userData['qoutaLimit'],
+						'mailLocalAddress' => $userData['mailLocalAddress']
+					);
+					
+					if (strpos($_data['domain'], '.') !== false)
+					{
+						$userData['mailLocalAddress'] = preg_replace('/@'.preg_quote($ea_account->acc_domain).'$/', '@'.$_data['domain'], $userData['mailLocalAddress']);
+
+						foreach($userData['mailAlternateAddress'] as &$alias)
+						{
+							$alias = preg_replace('/@'.preg_quote($ea_account->acc_domain).'$/', '@'.$_data['domain'], $alias);
+						}
+					}
+					// fullfill the saveUserData requirements
+					$userData += $ea_account->params;
+					$ea_account->saveUserData($_data['id'], $userData);
+					$msg = '#'.$_data['id'].' '.$account['account_fullname']. ' '.($userData['accountStatus'] == 'active'? lang('activated'):lang('deactivated'));
+				}
+				else
+				{
+					$msg .= lang('No profile defined for user %1', '#'.$_data['id'].' '.$account['account_fullname']."\n");
+					
+				}
+			}
+		}
+		$response->data($msg);
+	}
 }
