@@ -9,7 +9,7 @@
  * @package emailadmin
  * @subpackage setup
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: tables_update.inc.php 47916 2014-07-25 12:12:30Z ralfbecker $
+ * @version $Id$
  */
 
 function emailadmin_upgrade0_0_3()
@@ -882,3 +882,83 @@ function emailadmin_upgrade1_9_019()
 {
 	return $GLOBALS['setup_info']['emailadmin']['currentver'] = '14.1';
 }
+
+
+function emailadmin_upgrade14_1()
+{
+	$GLOBALS['egw_setup']->oProc->RefreshTable('egw_ea_valid',array(
+		'fd' => array(
+			'acc_id' => array('type' => 'int','precision' => '4','nullable' => False),
+			'account_id' => array('type' => 'int','meta' => 'account','precision' => '4','nullable' => False)
+		),
+		'pk' => array(),
+		'fk' => array(),
+		'ix' => array(array('account_id','acc_id')),
+		'uc' => array(array('acc_id','account_id'))
+	));
+
+	return $GLOBALS['setup_info']['emailadmin']['currentver'] = '14.2';
+}
+
+function emailadmin_upgrade14_2()
+{
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_mailaccounts','mail_value',array(
+		'type' => 'ascii',
+		'precision' => '128',
+		'nullable' => False,
+		'comment' => 'the value (that should be) corresponding to the mail_type'
+	));
+
+	return $GLOBALS['setup_info']['emailadmin']['currentver'] = '14.2.001';
+}
+
+
+function emailadmin_upgrade14_2_001()
+{
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_ea_accounts','acc_imap_host',array(
+		'type' => 'ascii',
+		'precision' => '128',
+		'nullable' => False,
+		'comment' => 'imap hostname'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_ea_accounts','acc_sieve_host',array(
+		'type' => 'ascii',
+		'precision' => '128',
+		'comment' => 'sieve host, default imap_host'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_ea_accounts','acc_smtp_type',array(
+		'type' => 'ascii',
+		'precision' => '32',
+		'default' => 'emailadmin_smtp',
+		'comment' => 'smtp class to use'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_ea_accounts','acc_imap_type',array(
+		'type' => 'ascii',
+		'precision' => '32',
+		'default' => 'emailadmin_imap',
+		'comment' => 'imap class to use'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_ea_accounts','acc_imap_logintype',array(
+		'type' => 'ascii',
+		'precision' => '20',
+		'comment' => 'standard, vmailmgr, admin, uidNumber'
+	));
+
+	return $GLOBALS['setup_info']['emailadmin']['currentver'] = '14.3';
+}
+
+/**
+ * New flag to allow non-emailadmins (users) to define (add, edit or remove) forwards
+ *
+ * @return string
+ */
+function emailadmin_upgrade14_3()
+{
+	$GLOBALS['egw_setup']->oProc->AddColumn('egw_ea_accounts','acc_user_forward',array(
+		'type' => 'bool',
+		'default' => '0',
+		'comment' => 'allow user to define forwards'
+	));
+
+	return $GLOBALS['setup_info']['emailadmin']['currentver'] = '14.3.001';
+}
