@@ -7,7 +7,7 @@
  * @package addressbook
  * @copyright (c) 2007-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.bo_merge.inc.php 49714 2014-12-05 10:44:00Z hnategh $
+ * @version $Id$
  */
 
 /**
@@ -191,17 +191,18 @@ abstract class bo_merge
 	 * Return replacements for a contact
 	 *
 	 * @param int|string|array $contact contact-array or id
-	 * @param string $prefix='' prefix like eg. 'user'
+	 * @param string $prefix ='' prefix like eg. 'user'
+	 * @param boolean $ignore_acl =false true: no acl check
 	 * @return array
 	 */
-	public function contact_replacements($contact,$prefix='')
+	public function contact_replacements($contact,$prefix='',$ignore_acl=false)
 	{
 		if (!is_array($contact))
 		{
-			$contact = $this->contacts->read($contact);
+			$contact = $this->contacts->read($contact, $ignore_acl);
 		}
 		if (!is_array($contact)) return array();
-
+		
 		$replacements = array();
 		foreach(array_keys($this->contacts->contact_fields) as $name)
 		{
@@ -258,7 +259,8 @@ abstract class bo_merge
 			if ($name != 'photo') $replacements['$$'.($prefix ? $prefix.'/':'').$name.'$$'] = $value;
 		}
 		// set custom fields, should probably go to a general method all apps can use
-		foreach($this->contacts->customfields as $name => $field)
+		// need to load all cfs for $ignore_acl=true
+		foreach($ignore_acl ? egw_customfields::get('addressbook', true) : $this->contacts->customfields as $name => $field)
 		{
 			$name = '#'.$name;
 			$replacements['$$'.($prefix ? $prefix.'/':'').$name.'$$'] = egw_customfields::format($field, (string)$contact[$name]);
@@ -359,7 +361,7 @@ abstract class bo_merge
 	protected function get_all_links($app, $id, $prefix, &$content)
 	{
 		$array = array();
-		$pattern = '@\$(link|links|attachments|links_attachments)\/?(title|href|link)?\/?([a-z]*)\$@';
+		$pattern = '@\$(links_attachments|links|attachments|link)\/?(title|href|link)?\/?([a-z]*)\$@';
 		static $link_cache;
 		if(preg_match_all($pattern, $content, $matches))
 		{
@@ -401,7 +403,8 @@ abstract class bo_merge
 						$array[($prefix?$prefix.'/':'').$placeholder] = $title;
 						break;
 					case 'links':
-						$array[($prefix?$prefix.'/':'').$placeholder] = $this->get_links($app, $id, '!'.egw_link::VFS_APPNAME, array(),$matches[2][$i]);
+						$link_app = $matches[3][$i] ? $matches[3][$i] :  '!'.egw_link::VFS_APPNAME;
+						$array[($prefix?$prefix.'/':'').$placeholder] = $this->get_links($app, $id, $link_app, array(),$matches[2][$i]);
 						break;
 					case 'attachments':
 						$array[($prefix?$prefix.'/':'').$placeholder] = $this->get_links($app, $id, egw_link::VFS_APPNAME,array(),$matches[2][$i]);
@@ -746,6 +749,36 @@ abstract class bo_merge
 			return false;
 		}
 		if ($this->report_memory_usage) error_log(__METHOD__."(count(ids)=".count($ids).") strlen(contentrepeat)=".strlen($contentrepeat).', strlen(labelrepeat)='.strlen($Labelrepeat));
+
+		if ($contentrepeat)
+		{
+			$content_stream = fopen('php://temp','r+');
+			fwrite($content_stream, $contentstart);
+			$joiner = '';
+			switch($mimetype)
+			{
+				case 'application/rtf':
+				case 'text/rtf':
+					$joiner = '\\par \\page\\pard\\plain';
+					break;
+				case 'application/vnd.oasis.opendocument.text':
+				case 'application/vnd.oasis.opendocument.spreadsheet':
+				case 'application/xml':
+				case 'text/html':
+					$joiner = '';
+					break;
+				case 'application/vnd.openxmlformats-officedocument.wordprocessingml.document':
+				case 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':
+					$joiner = '<w:br w:type="page" />';
+					break;
+				case 'text/plain':
+					$joiner = "\r\n";
+					break;
+				default:
+					$err = lang('%1 not implemented for %2!','$$pagerepeat$$',$mimetype);
+					return false;
+			}
+		}
 		foreach ((array)$ids as $n => $id)
 		{
 			if ($contentrepeat) $content = $contentrepeat;   //content to repeat
@@ -807,7 +840,14 @@ abstract class bo_merge
 			{
 				$content = preg_replace('/\$\$[a-z0-9_\/]+\$\$/i','',$content);
 			}
-			if ($contentrepeat) $contentrep[is_array($id) ? implode(':',$id) : $id] = $content;
+			if ($contentrepeat)
+			{
+				fwrite($content_stream, ($n == 0 ? '' : $joiner) . $content);
+			}
+			if($lableprint)
+			{
+				$contentrep[is_array($id) ? implode(':',$id) : $id] = $content;
+			}
 		}
 		if ($Labelrepeat)
 		{
@@ -848,24 +888,9 @@ abstract class bo_merge
 
 		if ($contentrepeat)
 		{
-			switch($mimetype)
-			{
-				case 'application/rtf':
-				case 'text/rtf':
-					return $contentstart.implode('\\par \\page\\pard\\plain',$contentrep).$contentend;
-				case 'application/vnd.oasis.opendocument.text':
-				case 'application/vnd.oasis.opendocument.spreadsheet':
-				case 'application/xml':
-				case 'text/html':
-					return $contentstart.implode('',$contentrep).$contentend;
-				case 'application/vnd.openxmlformats-officedocument.wordprocessingml.document':
-				case 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':
-					return $contentstart.implode('<w:br w:type="page" />',$contentrep).$contentend;
-				case 'text/plain':
-					return $contentstart.implode("\r\n",$contentrep).$contentend;
-			}
-			$err = lang('%1 not implemented for %2!','$$pagerepeat$$',$mimetype);
-			return false;
+			fwrite($content_stream, $contentend);
+			rewind($content_stream);
+			return stream_get_contents($content_stream);
 		}
 		if ($this->report_memory_usage) error_log(__METHOD__."() returning ".egw_vfs::hsize(memory_get_peak_usage(true)));
 
@@ -927,7 +952,8 @@ abstract class bo_merge
 			{
 				if($value = $replacements['$$'.$field.'$$'])
 				{
-					$replacements['$$'.$field.'/date$$'] = egw_time::to($value,true);
+					$time = egw_time::createFromFormat('+'.egw_time::$user_dateformat.' '.egw_time::$user_timeformat.'*', $value);
+					$replacements['$$'.$field.'/date$$'] = $time ? $time->format(egw_time::$user_dateformat)  : '';
 				}
 			}
 		}
@@ -946,7 +972,7 @@ abstract class bo_merge
 					case 'text/html':
 						$replace_tags = array(
 							'<b>','<strong>','<i>','<em>','<u>','<span>','<ol>','<ul>','<li>',
-							'<table>','<tr>','<td>','<a>',
+							'<table>','<tr>','<td>','<a>','<style>',
 						);
 						break;
 					case 'application/vnd.oasis.opendocument.text':		// open office
@@ -976,7 +1002,8 @@ abstract class bo_merge
 					continue;
 				}
 				// decode html entities back to utf-8
-				if (is_string($value) && (strpos($value,'&') !== false))
+
+				if (is_string($value) && (strpos($value,'&') !== false) && $this->parse_html_styles)
 				{
 					$value = html_entity_decode($value,ENT_QUOTES,$charset);
 
@@ -986,8 +1013,13 @@ abstract class bo_merge
 						$value = preg_replace('/&[^; ]+;/','',$value);
 					}
 				}
-				// remove all html tags, evtl. included
-				if (is_string($value) && (strpos($value,'<') !== false))
+
+				if(!$this->parse_html_styles)
+				{
+					// Encode special chars so they don't break the file
+					$value = htmlspecialchars($value,ENT_NOQUOTES);
+				}
+				else if (is_string($value) && (strpos($value,'<') !== false))
 				{
 					// Clean HTML, if it's being kept
 					if($replace_tags && extension_loaded('tidy')) {
@@ -1005,7 +1037,7 @@ abstract class bo_merge
 						}
 					}
 					// replace </p> and <br /> with CRLF (remove <p> and CRLF)
-					$value = str_replace(array("\r","\n",'<p>','</p>','<br />'),array('','','',"\r\n","\r\n"),$value);
+					$value = str_replace(array("\r","\n",'<p>','</p>','<div>','</div>','<br />'),array('','','',"\r\n",'',"\r\n","\r\n"),$value);
 					$value = strip_tags($value,implode('',$replace_tags));
 
 					// Change <tag>...\r\n</tag> to <tag>...</tag>\r\n or simplistic line break below will mangle it
@@ -1076,7 +1108,15 @@ abstract class bo_merge
 			}
 			// now decode &, < and >, which need to be encoded as entities in xml
 			// Check for encoded >< getting double-encoded
-			$replacements = str_replace(array('&',"\r","\n",'&amp;lt;','&amp;gt;'),array('&amp;','',$break,'&lt;','&gt;'),$replacements);
+			if($this->parse_html_styles)
+			{
+				$replacements = str_replace(array('&',"\r","\n",'&amp;lt;','&amp;gt;'),array('&amp;','',$break,'&lt;','&gt;'),$replacements);
+			}
+			else
+			{
+				// Need to at least handle new lines, or it'll be run together on one line
+				$replacements = str_replace(array("\r","\n"),array('',$break),$replacements);
+			}
 		}
 		return str_replace(array_keys($replacements),array_values($replacements),$content);
 	}
@@ -1226,6 +1266,9 @@ abstract class bo_merge
 		switch($mimetype)
 		{
 			case 'application/vnd.oasis.opendocument.spreadsheet':		// open office calc
+				// Removing these forces calc to respect our set value-type
+				$content = str_ireplace('calcext:value-type="string"','',$content);
+
 				$format = '/<table:table-cell([^>]+?)office:value-type="[^"]+"([^>]*?)>.?<([a-z].*?)[^>]*>\$\$('.implode('|',$names).')\$\$<\/\3>.?<\/table:table-cell>/s';
 				$replacement = '<table:table-cell$1office:value-type="date" office:date-value="\$\$$4\$\$"$2><$3>\$\$$4\$\$</$3></table:table-cell>';
 				break;
@@ -1327,19 +1370,19 @@ abstract class bo_merge
 		if (strpos($content,'$$IF') !== false)
 		{	//Example use to use: $$IF n_prefix~Herr~Sehr geehrter~Sehr geehrte$$
 			$this->replacements =& $replacements;
-			$content = preg_replace_callback('/\$\$IF ([0-9a-z_\/-]+)~(.*)~(.*)~(.*)\$\$/imU',Array($this,'replace_callback'),$content);
+			$content = preg_replace_callback('/\$\$IF ([#0-9a-z_\/-]+)~(.*)~(.*)~(.*)\$\$/imU',Array($this,'replace_callback'),$content);
 			unset($this->replacements);
 		}
 		if (strpos($content,'$$NELF') !== false)
 		{	//Example: $$NEPBR org_unit$$ sets a LF and value of org_unit, only if there is a value
 			$this->replacements =& $replacements;
-			$content = preg_replace_callback('/\$\$NELF ([0-9a-z_\/-]+)\$\$/imU',Array($this,'replace_callback'),$content);
+			$content = preg_replace_callback('/\$\$NELF ([#0-9a-z_\/-]+)\$\$/imU',Array($this,'replace_callback'),$content);
 			unset($this->replacements);
 		}
 		if (strpos($content,'$$NENVLF') !== false)
 		{	//Example: $$NEPBRNV org_unit$$ sets only a LF if there is a value for org_units, but did not add any value
 			$this->replacements =& $replacements;
-			$content = preg_replace_callback('/\$\$NENVLF ([0-9a-z_\/-]+)\$\$/imU',Array($this,'replace_callback'),$content);
+			$content = preg_replace_callback('/\$\$NENVLF ([#0-9a-z_\/-]+)\$\$/imU',Array($this,'replace_callback'),$content);
 			unset($this->replacements);
 		}
 		if (strpos($content,'$$LETTERPREFIX$$') !== false)
@@ -1350,7 +1393,7 @@ abstract class bo_merge
 		if (strpos($content,'$$LETTERPREFIXCUSTOM') !== false)
 		{	//Example use to use for a custom Letter Prefix: $$LETTERPREFIX n_prefix title n_family$$
 			$this->replacements =& $replacements;
-			$content = preg_replace_callback('/\$\$LETTERPREFIXCUSTOM ([0-9a-z_-]+)(.*)\$\$/imU',Array($this,'replace_callback'),$content);
+			$content = preg_replace_callback('/\$\$LETTERPREFIXCUSTOM ([#0-9a-z_-]+)(.*)\$\$/imU',Array($this,'replace_callback'),$content);
 			unset($this->replacements);
 		}
 		return $content;
@@ -1366,7 +1409,13 @@ abstract class bo_merge
 	{
 		if (array_key_exists('$$'.$param[4].'$$',$this->replacements)) $param[4] = $this->replacements['$$'.$param[4].'$$'];
 		if (array_key_exists('$$'.$param[3].'$$',$this->replacements)) $param[3] = $this->replacements['$$'.$param[3].'$$'];
-		$replace = preg_match('/'.preg_quote($param[2], '/').'/',$this->replacements['$$'.$param[1].'$$']) ? $param[3] : $param[4];
+		
+		$pattern = '/'.preg_quote($param[2], '/').'/';
+		if (strpos($param[0],'$$IF') === 0 && (trim($param[2]) == "EMPTY" || $param[2] === ''))
+		{
+			$pattern = '/^$/';
+		}
+		$replace = preg_match($pattern,$this->replacements['$$'.$param[1].'$$']) ? $param[3] : $param[4];
 		switch($this->mimetype)
 		{
 			case 'application/vnd.oasis.opendocument.text':		// open office
@@ -1877,7 +1926,7 @@ abstract class bo_merge
 		$action['nm_action'] = 'long_task';
 		$action['popup'] = egw_link::get_registry('mail', 'edit_popup');
 		$action['message'] = lang('insert in %1',egw_vfs::decodePath($file['name']));
-		$action['menuaction'] = 'mail.mail_compose.ajax_merge&document='.$file['path'];
+		$action['menuaction'] = 'mail.mail_compose.ajax_merge&document='.$file['path'].'&merge='. get_called_class();
 	}
 
 	/**
