@@ -8,7 +8,7 @@
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
  * @package etemplate
  * @subpackage extensions
- * @version $Id: class.customfields_widget.inc.php 49494 2014-11-19 13:12:17Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -45,6 +45,7 @@ class customfields_widget
 	 */
 	var $cf_types = array(
 		'text'     => 'Text',
+		'int'      => 'Integer',
 		'float'    => 'Float',
 		'label'    => 'Label',
 		'select'   => 'Selectbox',
@@ -290,7 +291,7 @@ class customfields_widget
 						// and used as such.  All unknown values will be used for selection, not passed through to the query
 						if (isset($field['values']['@']))
 						{
-							$options['values'] = $this->_get_options_from_file($field['values']['@']);
+							$options['values'] = egw_customfields::get_options_from_file($field['values']['@']);
 							unset($field['values']['@']);
 						} else {
 							$options['values'] = array_diff_key($field['values'], array_flip(ajax_select_widget::$known_options));
@@ -310,7 +311,7 @@ class customfields_widget
 						$showthis = '#a#l#l#';
 						if (count($field['values']) == 1 && isset($field['values']['@']))
 						{
-							$field['values'] = $this->_get_options_from_file($field['values']['@']);
+							$field['values'] = egw_customfields::get_options_from_file($field['values']['@']);
 						}
 						if($this->advanced_search && $field['rows'] <= 1) $field['values'][''] = lang('doesn\'t matter');
 						if ($readonly)
@@ -333,6 +334,7 @@ class customfields_widget
 						}
 						break;
 					case 'float':
+					case 'int':
 						$known_options = array('min'=>null, 'max'=>null,'size' => $field['len'], 'precision' => null);
 						$options = array_merge($known_options, $field['values']);
 						$input =& boetemplate::empty_cell($field['type'],$this->prefix.$lname,array(
