@@ -8,7 +8,7 @@
  * @link http://www.egroupware.org
  * @author Ralf Becker <RalfBecker@outdoor-training.de>
  * @copyright 2002-13 by RalfBecker@outdoor-training.de
- * @version $Id: class.etemplate_new.inc.php 49702 2014-12-04 15:52:15Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -95,17 +95,49 @@ class etemplate_new extends etemplate_widget_template
 	 *	-1 = first time return html, after use 0 (echo html incl. navbar), eg. for home
 	 *	 2 = echo without navbar (eg. for popups)
 	 *	 3 = return eGW independent html site
+	 *	 4 = json response
 	 * @param string $ignore_validation if not empty regular expression for validation-errors to ignore
 	 * @param array $changes change made in the last call if looping, only used internaly by process_exec
 	 * @return string html for $output_mode == 1, else nothing
 	 */
 	function exec($method,array $content,array $sel_options=null,array $readonlys=null,array $preserv=null,$output_mode=0,$ignore_validation='',array $changes=null)
 	{
+		$hook_data = $GLOBALS['egw']->hooks->process(
+			array('hook_location'   => 'etemplate2_before_exec') +
+			array('location_name'   => $this->name) +
+			array('location_object' => &$this) +
+			$content
+		);
+
+		foreach($hook_data as $extras) {
+			if (!$extras) continue;
+
+			foreach(isset($extras[0]) ? $extras : array($extras) as $extra) {
+				if ($extra['data'] && is_array($extra['data'])) {
+					$content = array_merge($content, $extra['data']);
+				}
+
+				if ($extra['preserve'] && is_array($extra['preserve'])) {
+					$preserv = array_merge($preserv, $extra['preserve']);
+				}
+
+				if ($extra['readonlys'] && is_array($extra['readonlys'])) {
+					$readonlys = array_merge($readonlys, $extra['readonlys']);
+				}
+			}
+		}
+		unset($hook_data);
+
 		// Include the etemplate2 javascript code
 		egw_framework::validate_file('.', 'etemplate2', 'etemplate');
 
 		if (!$this->rel_path) throw new egw_exception_assertion_failed("No (valid) template '$this->name' found!");
 
+		if ($output_mode == 4)
+		{
+			$output_mode = 0;
+			self::$response = egw_json_response::get();
+		}
 		self::$request->output_mode = $output_mode;	// let extensions "know" they are run eg. in a popup
 		self::$request->content = self::$cont = $content;
 		self::$request->changes = $changes;
@@ -189,11 +221,19 @@ class etemplate_new extends etemplate_widget_template
 			{
 				egw_framework::validate_file('.','app',$app,false);
 			}
+			// Category styles
+			categories::css($app);
+
+			// set action attribute for autocomplete form tag
+			// as firefox complains on about:balnk action, thus we have to literaly submit the form to a blank html
+			$form_action = "about:blank";
+			if (html::$user_agent == 'firefox') $form_action = $GLOBALS['egw_info']['server']['webserver_url'].'/etemplate/empty.html';
 
 			// check if we are in an ajax-exec call from jdots template (or future other tabbed templates)
 			if (isset($GLOBALS['egw']->framework->response))
 			{
-				$content = '<div id="'.$dom_id.'" class="et2_container"></div>';
+				$content = '<form target="egw_iframe_autocomplete_helper" action="'.$form_action.'" id="'.$dom_id.'" class="et2_container"></form>'."\n".
+					'<iframe name="egw_iframe_autocomplete_helper" style="width:0;height:0;position: absolute;visibility:hidden;"></iframe>';
 				// add server-side page-generation times
 				if($GLOBALS['egw_info']['user']['preferences']['common']['show_generation_time'])
 				{
@@ -207,9 +247,12 @@ class etemplate_new extends etemplate_widget_template
 				return;
 			}
 			// let framework know, if we are a popup or not ('popup' not true, which is allways used by index.php!)
-			$GLOBALS['egw_info']['flags']['nonavbar'] = $output_mode == 2 ? 'popup' : false;
+			if (!isset($GLOBALS['egw_info']['flags']['nonavbar']) || is_bool($GLOBALS['egw_info']['flags']['nonavbar']))
+			{
+				$GLOBALS['egw_info']['flags']['nonavbar'] = $output_mode == 2 ? 'popup' : false;
+			}
 			echo $GLOBALS['egw']->framework->header();
-			if ($output_mode != 2)
+			if ($output_mode != 2 && !$GLOBALS['egw_info']['flags']['nonavbar'])
 			{
 				parse_navbar();
 			}
@@ -222,7 +265,8 @@ class etemplate_new extends etemplate_widget_template
 			{
 				$load_array['response'] = egw_json_response::get()->returnResult();
 			}
-			echo '<div id="'.$dom_id.'" class="et2_container" data-etemplate="'.html::htmlspecialchars(egw_json_response::json_encode($load_array), true).'"></div>';
+			echo '<form target="egw_iframe_autocomplete_helper" action="'.$form_action.'" id="'.$dom_id.'" class="et2_container" data-etemplate="'.html::htmlspecialchars(egw_json_response::json_encode($load_array), true).'"></form>'."\n".
+				'<iframe name="egw_iframe_autocomplete_helper" style="width:0;height:0;position: absolute;visibility:hidden;"></iframe>';
 
 			if ($output_mode == 2)
 			{
@@ -244,6 +288,7 @@ class etemplate_new extends etemplate_widget_template
 	{
 		foreach($sel_options as &$options)
 		{
+			if (!is_array($options)||empty($options)) continue;
 			foreach($options as $key => $value)
 			{
 				if (is_numeric($key) && (!is_array($value) || !isset($value['value'])))
@@ -307,10 +352,61 @@ class etemplate_new extends etemplate_widget_template
 		// tell request call to remove request, if it is not modified eg. by call to exec in callback
 		self::$request->remove_if_not_modified();
 
+		$hook_data = $GLOBALS['egw']->hooks->process(
+			array(
+				'hook_location'   => 'etemplate2_before_process',
+				'location_name'   => $template->id) +
+				self::complete_array_merge(self::$request->preserv, $validated)
+			);
+
+		foreach($hook_data as $extras) {
+			if (!$extras) continue;
+
+			foreach(isset($extras[0]) ? $extras : array($extras) as $extra) {
+				if ($extra['data'] && is_array($extra['data'])) {
+					$validated = array_merge($validated, $extra['data']);
+				}
+			}
+		}
+		unset($hook_data);
+
 		//error_log(__METHOD__."(,".array2string($content).')');
 		//error_log(' validated='.array2string($validated));
 		$content = ExecMethod(self::$request->method, self::complete_array_merge(self::$request->preserv, $validated));
 
+		$tcontent = array();
+
+		if( is_array($content) ) {
+			$tcontent = $content;
+		}
+		else {
+			$tcontent = self::complete_array_merge(
+				self::$request->preserv,
+				$validated
+				);
+		}
+
+		$hook_data = $GLOBALS['egw']->hooks->process(
+			array(
+				'hook_location'   => 'etemplate2_after_process',
+				'location_name'   => $template->id) +
+				$tcontent
+			);
+		unset($tcontent);
+
+		if( is_array($content) ) {
+			foreach($hook_data as $extras) {
+				if (!$extras) continue;
+
+				foreach(isset($extras[0]) ? $extras : array($extras) as $extra) {
+					if ($extra['data'] && is_array($extra['data'])) {
+						$content = array_merge($content, $extra['data']);
+					}
+				}
+			}
+		}
+		unset($hook_data);
+
 		if (isset($GLOBALS['egw_info']['flags']['java_script']))
 		{
 			// Strip out any script tags
@@ -350,7 +446,7 @@ class etemplate_new extends etemplate_widget_template
 			error_log("Old etemplate submitted");
 			return ExecMethod('etemplate.etemplate.process_exec');
 		}
-		error_log(__METHOD__."(".array2string($content).")");
+		//error_log(__METHOD__."(".array2string($content).")");
 
 		self::$request = etemplate_request::read($_POST['etemplate_exec_id']);
 
@@ -369,8 +465,8 @@ class etemplate_new extends etemplate_widget_template
 			error_log(__METHOD__."(,".array2string($content).') validation_errors='.array2string(self::$validation_errors));
 			exit;
 		}
-		error_log(__METHOD__."(,".array2string($content).')');
-		error_log(' validated='.array2string($validated));
+		//error_log(__METHOD__."(,".array2string($content).')');
+		//error_log(' validated='.array2string($validated));
 
 		return ExecMethod(self::$request->method, self::complete_array_merge(self::$request->preserv, $validated));
 	}
@@ -506,7 +602,7 @@ class etemplate_new extends etemplate_widget_template
 			{
 				if (!is_array($v) || !isset($old[$k]) || 	// no array or a new array
 					isset($v[0]) && !is_array($v[0]) && isset($v[count($v)-1])	|| // or no associative array, eg. selecting multiple accounts
-					is_array($v) && count($v) == 0 && is_array($old[$k])) // Empty array replacing non-empty
+					is_array($v) && count($v) == 0)			// Empty array replacing non-empty
 				{
 					$old[$k] = $v;
 				}
@@ -617,15 +713,18 @@ foreach($files as $filename)
 $widgets = $GLOBALS['egw']->hooks->process('etemplate2_register_widgets');
 foreach($widgets as $app => $list)
 {
-	foreach($list as $class)
+	if (is_array($list))
 	{
-		try
+		foreach($list as $class)
 		{
-			__autoload($class);
-		}
-		catch(Exception $e)
-		{
-			error_log($e->getMessage());
+			try
+			{
+				class_exists($class);	// trigger autoloader
+			}
+			catch(Exception $e)
+			{
+				error_log($e->getMessage());
+			}
 		}
 	}
 }
