@@ -8,7 +8,7 @@
  * @link http://www.egroupware.org
  * @author Ralf Becker <RalfBecker@outdoor-training.de>
  * @copyright 2002-14 by RalfBecker@outdoor-training.de
- * @version $Id: class.etemplate_widget.inc.php 49489 2014-11-19 09:23:39Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -310,6 +310,12 @@ class etemplate_widget
 				{
 					$class_name = self::$widget_registry[$basetype];
 				}
+				// Look for old widgets that were adapted but not renamed
+				else if (class_exists($class_name = $basetype.'_widget') && in_array('etemplate_widget', class_parents($class_name)))
+				{
+					// Side-effects set $class_name
+					//error_log("Ported old widget: $class_name");
+				}
 				else
 				{
 					// Fall back to widget class, we can not ignore it, as the widget may contain other widgets
@@ -711,6 +717,22 @@ class etemplate_widget
 		return $pos;
 	}
 
+	/**
+	 * return a reference to $arr[$idx]
+	 *
+	 * This works for non-trival indexes like 'a[b][c]' too: it returns &$arr[a][b][c]
+	 * $sub = get_array($arr,'a[b]'); $sub = 'c'; is equivalent to $arr['a']['b'] = 'c';
+	 *
+	 * @param array& $_arr the array to search, referenz as a referenz gets returned
+	 * @param string $_idx the index, may contain sub-indices like a[b], see example below
+	 * @param mixed $_value value to set
+	 */
+	static function set_array(&$_arr, $_idx, $_value)
+	{
+		$ref =& self::get_array($_arr, $_idx, true);
+		if (true) $ref = $_value;
+	}
+
 	/**
 	 * Checks if a widget is readonly:
 	 * - readonly attribute set
@@ -845,7 +867,7 @@ class etemplate_widget
 				'value' => $val
 			));
 			// Don't delete it
-			self::$request->data_modified = true;
+			self::$request->unset_to_process('');
 			//error_log(__METHOD__."('$name', '$attr', ...) ".function_backtrace());
 		}
 		if (isset($this)) $this->attrs[$attr] = $val;
@@ -888,7 +910,8 @@ class etemplate_widget_box extends etemplate_widget
 	/**
 	 * Run a given method on all children
 	 *
-	 * Reimplemented because grids and boxes can have an own namespace
+	 * Reimplemented because grids and boxes can have an own namespace.
+	 * GroupBox has no namespace!
 	 *
 	 * @param string $method_name
 	 * @param array $params =array('') parameter(s) first parameter has to be cname!
@@ -901,7 +924,7 @@ class etemplate_widget_box extends etemplate_widget
 		$old_cname = $params[0];
 		$old_expand = $params[1];
 
-		if ($this->id) $cname = self::form_name($cname, $this->id, $params[1]);
+		if ($this->id && $this->type != 'groupbox') $cname = self::form_name($cname, $this->id, $params[1]);
 		if ($expand['cname'] !== $cname && $cname)
 		{
 			$expand['cont'] =& self::get_array(self::$request->content, $cname);
