@@ -8,7 +8,7 @@
  * @link http://www.egroupware.org
  * @author Ralf Becker <RalfBecker@outdoor-training.de>
  * @copyright 2002-14 by RalfBecker@outdoor-training.de
- * @version $Id: class.etemplate_widget_menupopup.inc.php 51065 2014-12-17 13:58:33Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -23,6 +23,29 @@ class etemplate_widget_menupopup extends etemplate_widget
 	 */
 	const SEARCH_ROW_LIMIT = PHP_INT_MAX; // Automatic disabled, only explicit
 
+	/**
+	 * These types are either set or cached on the client side, so we don't send
+	 * their options unless asked via AJAX
+	 */
+	public static $cached_types = array(
+		'select-account',
+		'select-app',
+		'select-bool',
+		'select-country',
+		// DOW needs some server-side pre-processing to unpack the options,
+		// so can't be skipped.
+		//'select-dow',
+		'select-number',
+		'select-priority',
+		'select-percent',
+		'select-year',
+		'select-month',
+		'select-day',
+		'select-hour',
+		'select-lang',
+		'select-timezone'
+	);
+
 	/**
 	 * @var array
 	 */
@@ -42,6 +65,19 @@ class etemplate_widget_menupopup extends etemplate_widget
 		12 => 'December'
 	);
 
+	/**
+	 * Constructor
+	 *
+	 * @param string|XMLReader $xml string with xml or XMLReader positioned on the element to construct
+	 * @throws egw_exception_wrong_parameter
+	 */
+	public function __construct($xml = '')
+	{
+		if($xml) {
+			parent::__construct($xml);
+		}
+	}
+
 	/**
 	 * Parse and set extra attributes from xml in template object
 	 *
@@ -92,6 +128,11 @@ class etemplate_widget_menupopup extends etemplate_widget
 			$value = $value_in = self::get_array($content, $form_name);
 
 			$allowed = self::selOptions($form_name, true);	// true = return array of option-values
+			$type_options = self::typeOptions($this,
+					// typeOptions thinks # of rows is the first thing in options
+					($this->attrs['rows'] && strpos($this->attrs['options'], $this->attrs['rows']) !== 0 ? $this->attrs['rows'].','.$this->attrs['options'] : $this->attrs['options']));
+			$allowed = array_merge($allowed,array_keys($type_options));
+			
 			if (!$this->attrs['multiple'] || !($this->attrs['options'] > 1)) $allowed[] = '';
 
 			foreach((array) $value as $val)
@@ -123,6 +164,13 @@ class etemplate_widget_menupopup extends etemplate_widget
 						}
 						break;
 
+					case 'select-timezone':
+						if (!calendar_timezones::tz2id($val))
+						{
+							self::set_validation_error($form_name, lang("'%1' is NOT a valid timezone!", $val));
+						}
+						break;
+
 					default:
 						if(!in_array($val, $allowed))
 						{
@@ -174,9 +222,11 @@ class etemplate_widget_menupopup extends etemplate_widget
 						}
 					}
 			}
-			$valid =& self::get_array($validated, $form_name, true);
-			if (true) $valid = $value;
-			//error_log(__METHOD__."() $form_name: ".array2string($value_in).' --> '.array2string($value).', allowed='.array2string($allowed));
+			if (isset($value))
+			{
+				self::set_array($validated, $form_name, $value);
+				//error_log(__METHOD__."() $form_name: ".array2string($value_in).' --> '.array2string($value).', allowed='.array2string($allowed));
+			}
 		}
 		else
 		{
@@ -208,13 +258,18 @@ class etemplate_widget_menupopup extends etemplate_widget
 		{
 			$form_name = $matches[1];
 		}
+		// happens with autorepeated grids: this->id='some[$row_cont[thing]][else]' --> just use 'else'
+		elseif (preg_match('/\$row.*\[([^]]+)\]$/', $this->id, $matches))
+		{
+			$form_name = $matches[1];
+		}
 		else
 		{
 			$form_name = self::form_name($cname, $this->id, $expand);
 		}
 		if (!is_array(self::$request->sel_options[$form_name])) self::$request->sel_options[$form_name] = array();
 		$type = $this->attrs['type'] ? $this->attrs['type'] : $this->type;
-		if ($type != 'select')
+		if ($type != 'select' && $type != 'menupopup')
 		{
 			// Check selection preference, we may be able to skip reading some data
 			$select_pref = $GLOBALS['egw_info']['user']['preferences']['common']['account_selection'];
@@ -224,20 +279,31 @@ class etemplate_widget_menupopup extends etemplate_widget
 				unset(self::$request->content[$this->id]);
 				$this->attrs['readonly'] = true;
 			}
-
-			// += to keep further options set by app code
-			$no_lang = null;
-			self::$request->sel_options[$form_name] += self::typeOptions($this,
-				// typeOptions thinks # of rows is the first thing in options
-				($this->attrs['rows'] && strpos($this->attrs['options'], $this->attrs['rows']) !== 0 ? $this->attrs['rows'].','.$this->attrs['options'] : $this->attrs['options']),
-				$no_lang, $this->attrs['readonly'], self::get_array(self::$request->content, $form_name), $form_name);
-			// need to run that here manually, automatic run through etemplate_new::exec() already happend
-			self::fix_encoded_options(self::$request->sel_options[$form_name]);
-
-			// if no_lang was modified, forward modification to the client
-			if ($no_lang != $this->attr['no_lang'])
+			if(!in_array($this->attrs['type'], self::$cached_types))
 			{
-				self::setElementAttribute($form_name, 'no_lang', $no_lang);
+				// adding type specific options here, while keep further options set by app code
+				// we need to make sure to run only once for auto-repeated rows, because
+				// array_merge used to keep options from app would otherwise add
+				// type-specific ones multiple time (and of cause better performance)
+				$no_lang = null;
+				static $form_names_done = array();
+				if (!isset($form_names_done[$form_name]) &&
+					($type_options = self::typeOptions($this,
+					// typeOptions thinks # of rows is the first thing in options
+					($this->attrs['rows'] && strpos($this->attrs['options'], $this->attrs['rows']) !== 0 ? $this->attrs['rows'].','.$this->attrs['options'] : $this->attrs['options']),
+					$no_lang, $this->attrs['readonly'], self::get_array(self::$request->content, $form_name), $form_name)))
+				{
+					self::fix_encoded_options($type_options);
+
+					self::$request->sel_options[$form_name] = array_merge(self::$request->sel_options[$form_name], $type_options);
+
+					// if no_lang was modified, forward modification to the client
+					if ($no_lang != $this->attr['no_lang'])
+					{
+						self::setElementAttribute($form_name, 'no_lang', $no_lang);
+					}
+				}
+				$form_names_done[$form_name] = true;
 			}
 		}
 
@@ -245,6 +311,12 @@ class etemplate_widget_menupopup extends etemplate_widget
 		$options = (isset(self::$request->sel_options[$form_name]) ? $form_name : $this->id);
 		if(is_array(self::$request->sel_options[$options]))
 		{
+			if(in_array($this->attrs['type'], self::$cached_types) && !isset($form_names_done[$options]))
+			{
+				// Fix any custom options from application
+				self::fix_encoded_options(self::$request->sel_options[$options],true);
+				$form_names_done[$options] = true;
+			}
 			// Turn on search, if there's a lot of rows (unless explicitly set)
 			if(!array_key_exists('search',$this->attrs) && count(self::$request->sel_options[$options]) >= self::SEARCH_ROW_LIMIT)
 			{
@@ -269,28 +341,22 @@ class etemplate_widget_menupopup extends etemplate_widget
 	{
 		$backup_options = $options;
 
+		$values = array();
 		foreach($options as $value => &$label)
 		{
 			// Of course once we re-index the options, we can't detect duplicates
 			// so check here, as we re-index
 			// Duplicates might happen if app programmer isn't paying attention and
 			// either uses the same ID in the template, or adds the options twice
-			if(is_numeric($value) && (!is_array($label) || is_array($label) && !array_key_exists('value',$label)))
+			$check_value = (string)(is_array($label) && array_key_exists('value', $label) ? $label['value'] : $value);
+			if (isset($values[$check_value]))
 			{
-				$check_value = (string)(is_array($label) && array_key_exists('value', $label) ? $label['value'] : $value);
-				if((string)$value === $check_value)
-				{
-					foreach($options as $key => $existing)
-					{
-						if(is_array($existing) && isset($existing['value']) && (string)$existing['value'] === $check_value && $key != $value)
-						{
-							unset($options[$value]);
-							continue 2;
-						}
-					}
-				}
+				unset($options[$value]);
+				continue;
 			}
-			if (is_null($use_array_of_objects) && is_numeric($value))
+			$values[$check_value] = $label;
+
+			if (is_null($use_array_of_objects) && is_numeric($value) && (!is_array($label) || !isset($label['value'])))
 			{
 				$options = $backup_options;
 				return self::fix_encoded_options($options, true);
@@ -414,15 +480,16 @@ class etemplate_widget_menupopup extends etemplate_widget
 		{
 			$widget = $widget_type;
 			$widget_type = $widget->attrs['type'] ? $widget->attrs['type'] : $widget->type;
-			// Legacy / static support
-			// Have to do this explicitly, since legacy options is not defined on class level
-			$legacy_options = explode(',',$_legacy_options);
-			foreach($legacy_options as &$field)
-			{
-				$field = self::expand_name($field, 0, 0,'','',self::$cont);
-			}
-			list($rows,$type,$type2,$type3,$type4,$type5) = $legacy_options;
 		}
+		// Legacy / static support
+		// Have to do this explicitly, since legacy options is not defined on class level
+		$legacy_options = explode(',',$_legacy_options);
+		foreach($legacy_options as &$field)
+		{
+			$field = self::expand_name($field, 0, 0,'','',self::$cont);
+		}
+		
+		list($rows,$type,$type2,$type3,$type4,$type5) = $legacy_options;
 		$no_lang = false;
 		$options = array();
 		switch ($widget_type)
@@ -449,14 +516,6 @@ class etemplate_widget_menupopup extends etemplate_widget
 				$options = array(0 => 'no',1 => 'yes');
 				break;
 
-			case 'select-access':
-				$options = array(
-					'private' => 'Private',
-					'public' => 'Global public',
-					'group' => 'Group public'
-				);
-				break;
-
 			case 'select-country':	// #Row|Extralabel,1=use country name, 0=use 2 letter-code,custom country field name
 				if($type == 0 && $type2)
 				{
@@ -773,6 +832,23 @@ class etemplate_widget_menupopup extends etemplate_widget
 		}
 		return $info;
 	}
+
+	/**
+	 * Some select options are fairly static, but can only be generated on the server
+	 * so we generate them here, then cache them client-side
+	 *
+	 * @param string $type
+	 * @param Array|String $attributes
+	 * @param string $value Optional current value, to make sure it's included
+	 */
+	public static function ajax_get_options($type, $attributes, $value = null)
+	{
+		$no_lang = false;
+		$options = self::typeOptions($type, $attributes,$no_lang,false,$value);
+		self::fix_encoded_options($options,true);
+		$response = egw_json_response::get();
+		$response->data($options);
+	}
 }
 
 etemplate_widget::registerWidget('etemplate_widget_menupopup', array('selectbox','listbox','select','menupopup'));
