@@ -8,7 +8,7 @@
  * @link http://www.egroupware.org
  * @author Ralf Becker <RalfBecker@outdoor-training.de>
  * @copyright 2002-11 by RalfBecker@outdoor-training.de
- * @version $Id: class.etemplate_widget_nextmatch.inc.php 48971 2014-10-09 08:39:24Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -122,6 +122,10 @@ class etemplate_widget_nextmatch extends etemplate_widget
 		$send_value = $value;
 
 		list($app) = explode('.',$value['get_rows']);
+		if(!$GLOBALS['egw_info']['apps'][$app])
+		{
+			list($app) = explode('.',$this->attrs['template']);
+		}
 
 		// Check for a favorite in URL
 		if($_GET['favorite'] && $value['favorites'])
@@ -140,16 +144,21 @@ class etemplate_widget_nextmatch extends etemplate_widget
 				}
 				unset($send_value['col_filter']);
 			}
+			// Old type
 			if($favorite && $favorite['filter'])
 			{
-				$send_value = array_merge($value, $favorite['filter']);
+				$favorite['state'] = $favorite['filter'];
+			}
+			if($favorite && $favorite['state'])
+			{
+				$send_value = array_merge($value, $favorite['state']);
 
 				// Ajax call can handle the saved sort here, but this can't
-				if($favorite['filter']['sort'])
+				if($favorite['state']['sort'])
 				{
 					unset($send_value['sort']);
-					$send_value['order'] = $favorite['filter']['sort']['id'];
-					$send_value['sort'] = $favorite['filter']['sort']['asc'] ? 'ASC' : 'DESC';
+					$send_value['order'] = $favorite['state']['sort']['id'];
+					$send_value['sort'] = $favorite['state']['sort']['asc'] ? 'ASC' : 'DESC';
 				}
 			}
 		}
@@ -648,19 +657,12 @@ class etemplate_widget_nextmatch extends etemplate_widget
 			{
 				if($n == 'sel_options')
 				{
-					foreach($row as &$options)
+					foreach($row as $name => &$options)
 					{
-						foreach($options as &$label)
-						{
-							if(!is_array($label))
-							{
-								$label = html_entity_decode($label, ENT_NOQUOTES,'utf-8');
-							}
-							else if ($label['label'])
-							{
-								$label['label'] = html_entity_decode($label['label'], ENT_NOQUOTES,'utf-8');
-							}
-						}
+						// remember newly set options for validation of nextmatch filters
+						self::$request->sel_options[$name] = $options;
+
+						etemplate_widget_menupopup::fix_encoded_options($options, true);
 					}
 				}
 				$rows[$n] = $row;
@@ -774,6 +776,7 @@ class etemplate_widget_nextmatch extends etemplate_widget
 	 * - string 'confirm' confirmation message
 	 * - string 'confirm_multiple' confirmation message for multiple selected, defaults to 'confirm'
 	 * - boolean 'postSubmit' eg. downloads need a submit via POST request not our regular Ajax submit, only works with nm_action=submit!
+	 * - string 'hint' tooltip on menu item
 	 *
 	 * @param array $actions id indexed array of actions / array with valus for keys: 'iconUrl', 'caption', 'onExecute', ...
 	 * @param string $template_name ='' name of the template, used as default for app name of images
@@ -1039,30 +1042,46 @@ class etemplate_widget_nextmatch extends etemplate_widget
 
 
 		// Save current column settings as default, clear, or force (admins only)
-		if($GLOBALS['egw_info']['user']['apps']['admin'] && $app)
+		if($GLOBALS['egw_info']['user']['apps']['admin'] && $app && $value['selectcols'])
 		{
 			$pref_name = 'nextmatch-' . (isset($content_value['columnselection_pref']) ? $content_value['columnselection_pref'] : $this->attrs['template']);
 			$refresh_pref_name = $pref_name.'-autorefresh';
-			$pref_level = $value['nm_col_preference'] == 'force' ? 'forced' : 'default';
+			switch($value['nm_col_preference']) {
+				case 'force':
+					$pref_level = 'forced';
+					break;
+				case 'reset':
+				case 'default':
+					$pref_level = 'default';
+					break;
+				default:
+					$pref_level = 'user';
+			}
 
 			// Clear forced pref before setting default
 			if($pref_level != 'forced')
 			{
 				$GLOBALS['egw']->preferences->delete($app,$pref_name,'forced');
 				$GLOBALS['egw']->preferences->delete($app,$refresh_pref_name,'forced');
+				$GLOBALS['egw']->preferences->delete($app,$pref_name.'-size','forced');
+				$GLOBALS['egw']->preferences->delete($app,$pref_name.'-lettersearch','forced');
 				$GLOBALS['egw']->preferences->save_repository(true,'forced');
 			}
 
 			// Set columns + refresh as default for all users
 			// Columns included in submit, preference might not be updated yet
 			$cols = $value['selectcols'];
-			$GLOBALS['egw']->preferences->read();
+			$GLOBALS['egw']->preferences->read_repository(true);
 			$GLOBALS['egw']->preferences->add($app,$pref_name,is_array($cols) ? implode(',',$cols) : $cols, $pref_level);
 
 			// Autorefresh
 			$refresh = $value['nm_autorefresh'];
 			$GLOBALS['egw']->preferences->add($app,$refresh_pref_name,(int)$refresh,$pref_level);
 
+			// Lettersearch
+			$lettersearch = is_array($cols) && in_array('lettersearch', $cols);
+			$GLOBALS['egw']->preferences->add($app,$pref_name.'-lettersearch',(int)$lettersearch,$pref_level);
+
 			$GLOBALS['egw']->preferences->save_repository(true,$pref_level);
 			$GLOBALS['egw']->preferences->read(true);
 
@@ -1071,6 +1090,7 @@ class etemplate_widget_nextmatch extends etemplate_widget
 				// Clear column + refresh preference so users go back to default
 				$GLOBALS['egw']->preferences->delete_preference($app,$pref_name);
 				$GLOBALS['egw']->preferences->delete_preference($app,$pref_name.'-size');
+				$GLOBALS['egw']->preferences->delete_preference($app,$pref_name.'-lettersearch');
 				$GLOBALS['egw']->preferences->delete_preference($app,$refresh_pref_name);
 			}
 		}
@@ -1193,6 +1213,10 @@ class etemplate_widget_nextmatch_customfilter extends etemplate_widget_transform
 				list($type) = explode('-',$this->attrs['type']);
 				if($type == 'select')
 				{
+					if(in_array($this->attrs['type'], etemplate_widget_menupopup::$cached_types))
+					{
+						$widget_type = $this->attrs['type'];
+					}
 					$this->attrs['type'] = 'nextmatch-filterheader';
 				}
 				self::$transformation['type'] = $this->attrs['type'];
@@ -1203,6 +1227,11 @@ class etemplate_widget_nextmatch_customfilter extends etemplate_widget_transform
 
 		parent::beforeSendToClient($cname, $expand);
 		$this->setElementAttribute($form_name, 'type', $this->attrs['type']);
+		if($widget_type)
+		{
+			$this->setElementAttribute($form_name, 'widget_type', $widget_type);
+		}
+		parent::beforeSendToClient($cname, $expand);
 	}
 }
 
