@@ -8,7 +8,7 @@
  * @subpackage extensions
  * @link http://www.egroupware.org
  * @author Ralf Becker <RalfBecker@outdoor-training.de>
- * @version $Id: class.nextmatch_widget.inc.php 46904 2014-05-12 22:16:05Z nathangray $
+ * @version $Id$
  */
 
 /**
@@ -634,7 +634,7 @@ class nextmatch_widget
 			'add'  => 'new',
 			'new'  => 'new',
 			'delete' => 'delete',
-			'cat'  => 'attach',		// add as category icon to api
+			'cat'  => 'category',		// add as category icon to api
 			'document' => 'etemplate/merge',
 			'print'=> 'print',
 			'copy' => 'copy',
