@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel
  * @copyright Stylite 2011
- * @version $Id: et2_core_DOMWidget.js 48841 2014-10-01 15:37:01Z nathangray $
+ * @version $Id$
  */
 
 "use strict";
@@ -75,6 +75,24 @@ var et2_DOMWidget = et2_widget.extend(et2_IDOMNode,
 			type: "js",
 			default: et2_no_init,
 			description: "Set default onExecute javascript method for action not specifying their own"
+		},
+		resize_ratio: {
+			name: "Resize height of the widget on callback resize",
+			type:"string",
+			default: '',
+			description: "Allow Resize height of the widget based on exess height and given ratio"
+		},
+		data: {
+			name: "comma-separated name:value pairs set as data attributes on DOM node",
+			type: "string",
+			default: '',
+			description: 'data="mime:${row}[mime]" would generate data-mime="..." in DOM, eg. to use it in CSS on a parent'
+		},
+		background: {
+			name: "Add background image",
+			type: "string",
+			default:'',
+			description: "Sets background image, left, right and scale on DOM",
 		}
 	},
 
@@ -120,7 +138,7 @@ var et2_DOMWidget = et2_widget.extend(et2_IDOMNode,
 			this._actionManager.remove();
 			this._actionManager = null;
 		}
-		
+
 		if (this._surroundingsMgr)
 		{
 			this._surroundingsMgr.free();
@@ -292,6 +310,9 @@ var et2_DOMWidget = et2_widget.extend(et2_IDOMNode,
 			var idx = 0;
 			var children = this._parent.getChildren();
 
+			if(children && children.indexOf) return children.indexOf(this);
+
+			egw.debug('warn', 'No Array.indexOf(), falling back to looping. ');
 			for (var i = 0; i < children.length; i++)
 			{
 				if (children[i] == this)
@@ -395,6 +416,36 @@ var et2_DOMWidget = et2_widget.extend(et2_IDOMNode,
 		}
 	},
 
+	set_data: function(_value)
+	{
+		var node = this.getDOMNode(this);
+		if (node && _value)
+		{
+			var pairs = _value.split(/,/g);
+			for(var i=0; i < pairs.length; ++i)
+			{
+				var name_value = pairs[i].split(':');
+				$j(node).attr('data-'+name_value[0], name_value[1]);
+			}
+		}
+	},
+	
+	set_background: function(_value)
+	{
+		var node = this.getDOMNode(this);
+		var values = '';
+		if (_value && node)
+		{
+			values = _value.split(',');
+			jQuery(node).css({
+				"background-image":'url("'+values[0]+'")',
+				"background-position-x":values[1],
+				"background-position-y":values[2],
+				"background-scale":values[3]
+			});
+		}
+	},
+	
 	/**
 	 * Set Actions on the widget
 	 *
@@ -495,18 +546,23 @@ var et2_DOMWidget = et2_widget.extend(et2_IDOMNode,
 			// objects
 			widget_object = objectManager.insertObject(false, new egwActionObject(
 				this.id, objectManager, new et2_action_object_impl(this),
-				objectManager.manager.getActionById(this.id) || objectManager.manager
+				this._actionManager || objectManager.manager.getActionById(this.id) || objectManager.manager
 			));
 		}
+		else
+		{
+			widget_object.setAOI(new et2_action_object_impl(this, this.getDOMNode()));
+		}
 
 		// Delete all old objects
 		widget_object.clear();
+		widget_object.unregisterActions();
 
 		// Go over the widget & add links - this is where we decide which actions are
 		// 'allowed' for this widget at this time
 		var action_links = this._get_action_links(actions);
 		widget_object.updateActionLinks(action_links);
-	}
+		}
 });
 
 /**
