@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel
  * @copyright Stylite 2011
- * @version $Id: et2_core_common.js 47323 2014-06-23 08:10:40Z ralfbecker $
+ * @version $Id$
  */
 
 "use strict";
@@ -30,7 +30,7 @@ if (typeof Array.prototype.indexOf == "undefined")
 /**
  * Array with all types supported by the et2_checkType function.
  */
-var et2_validTypes = ["boolean", "string", "html", "float", "integer", "any", "js", "dimension"];
+var et2_validTypes = ["boolean", "string", "rawstring", "html", "float", "integer", "any", "js", "dimension"];
 
 /**
  * Object whith default values for the above types. Do not specify array or
@@ -40,6 +40,7 @@ var et2_validTypes = ["boolean", "string", "html", "float", "integer", "any", "j
 var et2_typeDefaults = {
 	"boolean": false,
 	"string": "",
+	"rawstring": "",	// no html-entity decoding
 	"html": "",
 	"js": null,
 	"float": 0.0,
@@ -153,7 +154,7 @@ function et2_checkType(_val, _type, _attr, _widget)
 	}
 
 	// Check whether the given value is of the type "string"
-	if (_type == "string" || _type == "html")
+	if (_type == "string" || _type == "html" || _type == "rawstring")
 	{
 		if (typeof _val == "number")	// as php is a bit vague here, silently convert to a string
 		{
@@ -162,7 +163,7 @@ function et2_checkType(_val, _type, _attr, _widget)
 
 		if (typeof _val == "string")
 		{
-			return _type == "html" ? _val : html_entity_decode(_val);
+			return _type == "string" ? html_entity_decode(_val) : _val;
 		}
 
 		// Handle some less common possibilities
@@ -230,10 +231,11 @@ function et2_checkType(_val, _type, _attr, _widget)
 	// Javascript
 	if (_type == "js")
 	{
-		if (typeof _val == "function")
+		if (typeof _val == "function" || typeof _val == "undefined")
 		{
 			return _val;
 		}
+		if (_val) _val = _val.replace(/window\.close\(\)/g, 'egw(window).close()');
 
 		// Check to see if it's a string in app.appname.function format, and wrap it in
 		// a closure to make sure context is preserved
@@ -260,7 +262,7 @@ function et2_checkType(_val, _type, _attr, _widget)
 			}
 		}
 
-		if (typeof _val == "string")
+		if (!_val || typeof _val == "string")
 		{
 			return _val;	// get compiled later in widgets own initAttributes, as widget is not yet initialised
 		}
@@ -777,4 +779,4 @@ function et2_rangeSubstract(_ar1, _ar2)
 function html_entity_decode(_str)
 {
 	return _str && _str.indexOf('&') != -1 ? jQuery('<span>'+_str+'</span>').text() : _str;
-}
\ No newline at end of file
+}
