@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel
  * @copyright Stylite 2011
- * @version $Id: et2_core_widget.js 47112 2014-05-27 20:16:02Z nathangray $
+ * @version $Id$
  */
 
 "use strict";
@@ -393,6 +393,7 @@ var et2_widget = ClassWithAttributes.extend(
 		{
 			return this;
 		}
+		if(!this._children) return null;
 
 		for (var i = 0; i < this._children.length; i++)
 		{
@@ -503,8 +504,8 @@ var et2_widget = ClassWithAttributes.extend(
 					var mod = this.getArrayMgr("modifications").getEntry(_target.id);
 					if(typeof mod.options != "undefined") attrValue = _attrsObj[i].value = mod.options;
 				}
-				// Check for entire legacy options passed in content
-				if(attrValue.charAt(0) == '@' && attrValue.indexOf(',') == -1)
+				// expand legacyOptions with content
+				if(attrValue.charAt(0) == '@' || attrValue.indexOf('$') != -1)
 				{
 					attrValue = mgr.expandName(attrValue);
 				}
@@ -951,7 +952,7 @@ var et2_widget = ClassWithAttributes.extend(
 	 * @param {string} _part name of array mgr to return
 	 */
 	getArrayMgr: function(_part) {
-		if (typeof this._mgrs[_part] != "undefined")
+		if (this._mgrs && typeof this._mgrs[_part] != "undefined")
 		{
 			return this._mgrs[_part];
 		}
