@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel
  * @copyright Stylite 2011-2012
- * @version $Id: et2_dataview_controller.js 48691 2014-09-22 22:29:09Z nathangray $
+ * @version $Id$
  */
 
 /*egw:uses
@@ -352,7 +352,7 @@ var et2_dataview_controller = Class.extend({
 		if (!_entry.row)
 		{
 			createdRow = true;
-			_entry.row = new et2_dataview_row(this._grid);
+			_entry.row = this._createRow(ctx);
 			_entry.row.setDestroyCallback(this._destroyCallback, ctx);
 		}
 
@@ -388,6 +388,17 @@ var et2_dataview_controller = Class.extend({
 		return this.hasData;
 	},
 
+
+	/**
+	 * Create a new row.
+	 * 
+	 * @param {type} ctx
+	 * @returns {et2_dataview_container}
+	 */
+	_createRow: function(ctx) {
+		return new et2_dataview_row(this._grid);
+	},
+
 	/**
 	 * Function which gets called by the grid when data is requested.
 	 *
@@ -678,7 +689,7 @@ var et2_dataview_controller = Class.extend({
 
 		// Unregister the data callback
 		this.self._dataProvider.dataUnregisterUID(this.entry.uid,
-				this.self._dataCallback, null);
+				this.self._dataCallback, this);
 	},
 
 	/**
