@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Nathan Gray
  * @copyright Nathan Gray 2011
- * @version $Id: et2_extension_customfields.js 49810 2014-12-10 14:39:48Z ralfbecker $
+ * @version $Id$
  */
 
 "use strict";
@@ -53,7 +53,7 @@ var et2_customfields_list = et2_valueWidget.extend([et2_IDetachedDOM, et2_IInput
 		}
 	},
 
-	legacyOptions: ["type_filter","private"], // Field restriction & private done server-side
+	legacyOptions: ["type_filter","private", "fields"], // Field restriction & private done server-side
 
 	prefix: '#',
 
@@ -189,7 +189,8 @@ var et2_customfields_list = et2_valueWidget.extend([et2_IDetachedDOM, et2_IInput
 			{
 
 				var row = jQuery(document.createElement("tr"))
-					.appendTo(this.tbody);
+					.appendTo(this.tbody)
+					.addClass(this.id+'_'+id);
 				var cf = jQuery(document.createElement("td"))
 					.appendTo(row);
 				if(!field.type) field.type = 'text";'
@@ -199,8 +200,7 @@ var et2_customfields_list = et2_valueWidget.extend([et2_IDetachedDOM, et2_IInput
 					'id': 		id,
 					'statustext':	field.help,
 					'needed':	field.needed,
-					'readonly':	this.options.readonly ||
-						this.getArrayMgr("readonlys").isReadOnly(id),
+					'readonly':	this.getArrayMgr("readonlys").isReadOnly(id, null, this.options.readonly),
 					'value':	this.options.value[this.prefix+field_name]
 				};
 				// Can't have a required readonly, it will warn & be removed later, so avoid the warning
@@ -232,6 +232,16 @@ var et2_customfields_list = et2_valueWidget.extend([et2_IDetachedDOM, et2_IInput
 				}
 				this.rows[id] = cf[0];
 
+				// Set any additional attributes set in options, but not for widgets that pass actual options
+				if(['select','radio','radiogroup','checkbox','button'].indexOf(field.type) == -1 && !jQuery.isEmptyObject(field.values))
+				{
+					var w = et2_registry[attrs.type ? attrs.type : field.type];
+					for(var attr_name in field.values)
+					{
+						if (typeof w.prototype.attributes[attr_name] != "undefined")
+						attrs[attr_name] = field.values[attr_name];
+					}
+				}
 				// Create widget
 				var widget = this.widgets[field_name] = et2_createWidget(attrs.type ? attrs.type : field.type, attrs, this);
 			}
@@ -290,11 +300,18 @@ var et2_customfields_list = et2_valueWidget.extend([et2_IDetachedDOM, et2_IInput
 				_attrs["value"] = {};
 				if (val !== null)
 				{
-					// Only set the values that match desired custom fields
-					for(var key in val)
+					if(this.id.indexOf(this.prefix) === 0 && typeof data.fields != 'undefined' && data.fields[this.id.replace(this.prefix,'')] === true)
 					{
-						if(key.indexOf(this.prefix) == 0) {
-							_attrs["value"][key] = val[key];
+						_attrs['value'][this.id] = val;
+					}
+					else
+					{
+						// Only set the values that match desired custom fields
+						for(var key in val)
+						{
+							if(key.indexOf(this.prefix) == 0) {
+								_attrs["value"][key] = val[key];
+							}
 						}
 					}
 					//_attrs["value"] = val;
@@ -419,6 +436,20 @@ var et2_customfields_list = et2_valueWidget.extend([et2_IDetachedDOM, et2_IInput
 		}
 		return true;
 	},
+	_setup_ajax_select: function(field_name, field, attrs) {
+		var attributes = ['get_rows','get_title','id_field','template'];
+		if(field.values)
+		{
+			for(var i = 0; i < attributes.length; i++)
+			{
+				if(typeof field.values[attributes[i]] !== 'undefined')
+				{
+					attrs[attributes[i]] = field.values[attributes[i]];
+				}
+			}
+		}
+		return true;
+	},
 	_setup_float: function(field_name, field, attrs) {
 		// No label on the widget itself
 		delete(attrs.label);
@@ -441,7 +472,11 @@ var et2_customfields_list = et2_valueWidget.extend([et2_IDetachedDOM, et2_IInput
 
 		return true;
 	},
-	 _setup_date: function(field_name, field, attrs) {
+	_setup_select_account: function(field_name, field, attrs) {
+		attrs.empty_label = egw.lang('Select');
+		return this._setup_select(field_name, field, attrs);
+	},
+	_setup_date: function(field_name, field, attrs) {
 		attrs.data_format = 'Y-m-d';
 		return true;
 	},
