@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel
  * @copyright Stylite 2011
- * @version $Id: et2_extension_nextmatch.js 51117 2014-12-19 14:42:58Z hnategh $
+ * @version $Id$
  */
 
 "use strict";
@@ -68,7 +68,7 @@ var et2_INextmatchSortable = new Interface({
  *
  * @augments et2_DOMWidget
  */
-var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
+var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput, et2_IPrint],
 {
 	attributes: {
 		// These normally set in settings, but broken out into attributes to allow run-time changes
@@ -113,7 +113,12 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 			"description": "Hide the second filter",
 			"default": et2_no_init
 		},
-
+		"view": {
+			"name": "View",
+			"type": "string",
+			"description": "Display entries as either 'row' or 'tile'.  A matching template must also be set after changing this.",
+			"default": et2_no_init
+		},
 		"onselect": {
 			"name": "onselect",
 			"type": "js",
@@ -139,6 +144,10 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 
 	columns: [],
 
+	// Current view, either row or tile.  We store it here as controllers are
+	// recreated when the template changes.
+	view: 'row',
+
 	/**
 	 * Constructor
 	 *
@@ -148,6 +157,9 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 		this._super.apply(this, arguments);
 		this.activeFilters = {col_filter:{}};
 
+		// Directly set current col_filters from settings
+		jQuery.extend(this.activeFilters.col_filter, this.options.settings.col_filter);
+
 		/*
 		Process selected custom fields here, so that the settings are correctly
 		set before the row template is parsed
@@ -267,8 +279,8 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 			{
 				var self = this;
 				// Register a handler
-				$j('table.egwGridView_grid',this.div)
-					.on('dragenter','tr',function(e) {
+				$j(this.div)
+					.on('dragenter','.egwGridView_grid tr',function(e) {
 						// Figure out _which_ row
 						var row = self.controller.getRowByNode(this);
 
@@ -285,17 +297,31 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 						}
 						return false;
 					})
-					.on('dragexit','tr', function(e) {
+					.on('dragexit','.egwGridView_grid tr', function(e) {
 						self.controller._selectionMgr.setFocused();
 					})
-					.on('dragover','tr',false).attr("dropzone","copy")
+					.on('dragover','.egwGridView_grid tr',false).attr("dropzone","copy")
 
-					.on('drop', 'tr',function(e) {
+					.on('drop', '.egwGridView_grid tr',function(e) {
 						self.handle_drop(e,this);
 						return false;
 					});
 			}
 		}
+		// stop invalidation in no visible tabs
+		$j(this.getInstanceManager().DOMContainer.parentNode).on('hide.et2_nextmatch', jQuery.proxy(function(e) {
+			if(this.controller && this.controller._grid)
+			{
+				this.controller._grid.doInvalidate = false;
+			}
+		},this));
+		$j(this.getInstanceManager().DOMContainer.parentNode).on('show.et2_nextmatch', jQuery.proxy(function(e) {
+			if(this.controller && this.controller._grid)
+			{
+				this.controller._grid.doInvalidate = true;
+			}
+		},this));
+
 		return true;
 	},
 
@@ -392,6 +418,11 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 	 */
 	applyFilters: function(_set) {
 		var changed = false;
+		var keep_selection = false;
+
+		// Avoid loops cause by change events
+		if(this.update_in_progress) return;
+		this.update_in_progress = true;
 
 		// Cleared explicitly
 		if(typeof _set != 'undefined' && jQuery.isEmptyObject(_set))
@@ -440,6 +471,17 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 						}
 					}
 				}
+				else if (s === 'selected')
+				{
+					changed = true;
+					keep_selection = true;
+					this.controller._selectionMgr.resetSelection();
+					for(var i in _set.selected)
+					{
+						this.controller._selectionMgr.setSelected(_set.selected[i].indexOf('::') > 0 ? _set.selected[i] : this.controller.dataStorePrefix + '::'+_set.selected[i],true);
+					}
+					delete _set.selected;
+				}
 				else if (this.activeFilters[s] !== _set[s])
 				{
 					this.activeFilters[s] = _set[s];
@@ -451,7 +493,7 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 		this.egw().debug("info", "Changing nextmatch filters to ", this.activeFilters);
 
 		// Keep the selection after applying filters, but only if unchanged
-		if(!changed)
+		if(!changed || keep_selection)
 		{
 			this.controller.keepSelection();
 		}
@@ -484,6 +526,21 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 
 		// Trigger an update
 		this.controller.update(true);
+
+		if(changed)
+		{
+			// Highlight matching favorite in sidebox
+			if(this.getInstanceManager().app)
+			{
+				var app = this.getInstanceManager().app;
+				if(window.app[app] && window.app[app].highlight_favorite)
+				{
+					window.app[app].highlight_favorite();
+				}
+			}
+		}
+
+		this.update_in_progress = false;
 	},
 
 	/**
@@ -937,7 +994,7 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 		this.egw().set_preference(app, pref, colDisplay.join(","),
 			// Use callback after the preference gets set to trigger refresh, in case app
 			// isn't looking at selectcols and just uses preference
-			cf_added ? jQuery.proxy(function() {this.controller.update(true);}, this):null
+			cf_added ? jQuery.proxy(function() {if(this.controller) this.controller.update(true);}, this):null
 		);
 
 		// Save adjusted column sizes
@@ -1109,6 +1166,9 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 		}
 		this.controller.setPrefix(this.options.settings.dataStorePrefix);
 
+		// Set the view
+		this.controller._view = this.view;
+
 		// Load the initial order
 		/*this.controller.loadInitialOrder(this._getInitialOrder(
 			this.options.settings.rows, this.options.settings.row_id
@@ -1232,6 +1292,12 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 			// Custom fields get listed separately
 			if(widget.instanceOf(et2_nextmatch_customfields))
 			{
+				if(jQuery.isEmptyObject(widget.customfields))
+				{
+					// No customfields defined, don't show column
+					delete(columns[col.id]);
+					continue;
+				}
 				for(var field_name in widget.customfields)
 				{
 					columns[widget.prefix+field_name] = " - "+widget.customfields[field_name].label;
@@ -1355,11 +1421,13 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 				// Auto refresh
 				self._set_autorefresh(autoRefresh.get_value());
 
-				// Set default or clear forced?
-				if(defaultCheck.get_value())
+				// Set default or clear forced
+				if(show_letters)
 				{
-					self.getInstanceManager().submit();
+					self.activeFilters.selectcols.push('lettersearch');
 				}
+				self.getInstanceManager().submit();
+
 				self.selectPopup = null;
 			};
 
@@ -1397,6 +1465,71 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 			.css("left", s_position.left + this.div.width() - this.selectPopup.width());
 	},
 
+	/**
+	 * Set the currently displayed columns, without updating user's preference
+	 *
+	 * @param {string[]} column_list List of column names
+	 * @param {boolean} trigger_update=false - explicitly trigger an update
+	 */
+	set_columns: function(column_list, trigger_update)
+	{
+		var columnMgr = this.dataview.getColumnMgr();
+		var visibility = {};
+
+		// Initialize to false
+		for (var i = 0; i < columnMgr.columns.length; i++)
+		{
+			var col = columnMgr.columns[i];
+			if(col.caption && col.visibility != ET2_COL_VISIBILITY_ALWAYS_NOSELECT )
+			{
+				visibility[col.id] = {visible: false};
+			}
+		}
+		for(var i = 0; i < this.columns.length; i++)
+		{
+
+			var widget = this.columns[i].widget;
+			var colName = this._getColumnName(widget);
+			if(column_list.indexOf(colName) !== -1)
+			{
+				visibility[columnMgr.columns[i].id].visible = true;
+			}
+			// Custom fields are listed seperately in column list, but are only 1 column
+			if(widget && widget.instanceOf(et2_nextmatch_customfields)) {
+
+				// Just the ID for server side, not the whole nm name - some apps use it to skip custom fields
+				colName = widget.id;
+				if(column_list.indexOf(colName) !== -1)
+				{
+					visibility[columnMgr.columns[i].id].visible = true;
+				}
+
+				var cf = this.columns[i].widget.options.customfields;
+				var visible = this.columns[i].widget.options.fields;
+
+				// Turn off all custom fields
+				for(var field_name in cf)
+				{
+					visible[field_name] = false;
+				}
+				// Turn on selected custom fields - start from 0 in case they're not in order
+				for(var j = 0; j < column_list.length; j++)
+				{
+					if(column_list[j].indexOf(et2_customfields_list.prototype.prefix) != 0) continue;
+					visible[column_list[j].substring(1)] = true;
+				}
+				widget.set_visible(visible);
+			}
+		}
+		columnMgr.setColumnVisibilitySet(visibility);
+
+		// We don't want to update user's preference, so directly update
+		this.dataview._updateColumns();
+
+		// Allow column widgets a chance to resize
+		this.iterateOver(function(widget) {widget.resize();}, this, et2_IResizeable);
+	},
+
 	/**
 	 * Set the letter search preference, and update the UI
 	 *
@@ -1498,6 +1631,18 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 			this.rowProvider.free();
 			this.controller.free();
 
+			// Free any children from previous template
+			// They may get left behind because of how detached nodes are processed
+			// We don't use iterateOver because it checks sub-children
+			for(var i = this._children.length-1; i >=0 ; i--)
+			{
+				var _node = this._children[i];
+				if(_node != this.header) {
+					this.removeChild(_node);
+					_node.destroy();
+				}
+			}
+
 			// Clear this setting if it's the same as the template, or
 			// the columns will not be loaded
 			if(this.template == this.options.settings.columnselection_pref)
@@ -1645,12 +1790,37 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 	set_actions: function(actions) {
 		if(actions != this.options.actions && this.controller != null && this.controller._actionManager)
 		{
+			for(var i = this.controller._actionManager.children.length - 1; i >= 0; i--)
+			{
+				this.controller._actionManager.children[i].remove();
+			}
 			this.options.actions = actions;
 			this.options.settings.action_links = this.controller._actionLinks = this._get_action_links(actions);
+
 			this.controller._initActions(actions);
 		}
 	},
 
+	/**
+	 * Switch view between row and tile.
+	 * This should be followed by a call to change the template to match, which
+	 * will cause a reload of the grid using the new settings.
+	 *
+	 * @param {string} view Either 'tile' or 'row'
+	 */
+	set_view: function(view)
+	{
+		// Restrict to the only 2 accepted values
+		if(view == 'tile')
+		{
+			this.view = 'tile';
+		}
+		else
+		{
+			this.view = 'row';
+		}
+	},
+
 	/**
 	 * Set a different / additional handler for dropped files.
 	 *
@@ -1660,9 +1830,9 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 	 *
 	 * @param {String|Function} handler
 	 */
-	 set_onfiledrop: function(handler) {
+	set_onfiledrop: function(handler) {
 		this.options.onfiledrop = handler;
-	 },
+	},
 
 	/**
 	 * Handle drops of files by linking to the row, if possible.
@@ -1675,7 +1845,7 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 	 * @param {object} event
 	 * @param {object} target
 	 */
-	 handle_drop: function(event, target) {
+	handle_drop: function(event, target) {
 		// Check to see if we can handle the link
 		// First, find the UID
 		var row = this.controller.getRowByNode(target);
@@ -1750,7 +1920,7 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 
 		// Upload and link - this triggers the upload, which triggers the link, which triggers the cleanup and refresh
 		link.file_upload.set_value(files);
-	 },
+	},
 
 	getDOMNode: function(_sender) {
 		if (_sender == this)
@@ -1804,6 +1974,189 @@ var et2_nextmatch = et2_DOMWidget.extend([et2_IResizeable, et2_IInput],
 	set_value: function(_value)
 	{
 		this.value = _value;
+	},
+
+	// Printing
+	/**
+	 * Prepare for printing
+	 *
+	 * We check for un-loaded rows, and ask the user what they want to do about them.
+	 * If they want to print them all, we ask the server and print when they're loaded.
+	 */
+	beforePrint: function() {
+		// Add the class, if needed
+		this.div.addClass('print');
+
+		// Trigger resize, so we can fit on a page
+		this.dynheight.outerNode.css('max-width',this.div.css('max-width'));
+		this.resize();
+		// Reset height to auto (after width resize) so there's no restrictions
+		this.dynheight.innerNode.css('height', 'auto');
+
+		// Check for rows that aren't loaded yet, or lots of rows
+		var range = this.controller._grid.getIndexRange();
+		this.old_height = this.controller._grid._scrollHeight;
+		var loaded_count = range.bottom - range.top +1;
+		var total = this.controller._grid.getTotalCount();
+		if(loaded_count != total ||
+			this.controller._grid.getTotalCount() > 100)
+		{
+			// Defer the printing
+			var defer = jQuery.Deferred();
+
+			// Something not in the grid, lets ask
+			et2_dialog.show_prompt(jQuery.proxy(function(button, value) {
+					if(button == 'dialog[cancel]') {
+						// Give dialog a chance to close, or it will be in the print
+						window.setTimeout(function() {defer.reject();}, 0);
+						return;
+					}
+					value = parseInt(value);
+					if(value > total)
+					{
+						value = total;
+					}
+
+					// If they want the whole thing, treat it as all
+					if(button == 'dialog[ok]' && value == this.controller._grid.getTotalCount())
+					{
+						button = 'dialog[all]';
+						// Add the class, gives more reliable sizing
+						this.div.addClass('print');
+						// Show it all
+						$j('.egwGridView_scrollarea',this.div).css('height','auto');
+					}
+					// We need more rows
+					if(button == 'dialog[all]' || value > loaded_count)
+					{
+						var count = 0;
+						var fetchedCount = 0;
+						var cancel = false;
+						var nm = this;
+						var dialog = et2_dialog.show_dialog(
+							// Abort the long task if they canceled the data load
+							function() {count = total; cancel=true;window.setTimeout(function() {defer.reject();},0);},
+							egw.lang('Loading'), egw.lang('please wait...'),{},[
+								{"button_id": et2_dialog.CANCEL_BUTTON,"text": 'cancel',id: 'dialog[cancel]',image: 'cancel'}
+							]
+						);
+
+						// dataFetch() is asyncronous, so all these requests just get fired off...
+						// 200 rows chosen arbitrarily to reduce requests.
+						do {
+							var ctx = {
+								"self": this.controller,
+								"start": count,
+								"count": Math.min(value,200),
+								"lastModification": this.controller._lastModification
+							};
+							if(nm.controller.dataStorePrefix)
+							{
+								ctx.prefix = nm.controller.dataStorePrefix;
+							}
+							nm.controller.dataFetch({start:count, num_rows: Math.min(value,200)}, function(data)  {
+								// Keep track
+								if(data && data.order)
+								{
+									fetchedCount += data.order.length;
+								}
+								nm.controller._fetchCallback.apply(this, arguments);
+
+								if(fetchedCount >= value)
+								{
+									if(cancel)
+									{
+										dialog.destroy();
+										defer.reject();
+										return;
+									}
+									// Use CSS to hide all but the requested rows
+									// Prevents us from showing more than requested, if actual height was less than average
+									nm.print_row_selector = ".egwGridView_grid > tbody > tr:not(:nth-child(-n+"+value+"))";
+									egw.css(nm.print_row_selector, 'display: none');
+
+									// No scrollbar in print view
+									$j('.egwGridView_scrollarea',this.div).css('overflow-y','hidden');
+									// Show it all
+									$j('.egwGridView_scrollarea',this.div).css('height','auto');
+
+									// Grid needs to redraw before it can be printed, so wait
+									window.setTimeout(jQuery.proxy(function() {
+										dialog.destroy();
+
+										// Should be OK to print now
+										defer.resolve();
+									},nm),ET2_GRID_INVALIDATE_TIMEOUT);
+
+								}
+
+							},ctx);
+							count += 200;
+						} while (count < value)
+						nm.controller._grid.setScrollHeight(nm.controller._grid.getAverageHeight() * (value+1));
+					}
+					else
+					{
+						// Don't need more rows, limit to requested and finish
+
+						// Show it all
+						$j('.egwGridView_scrollarea',this.div).css('height','auto');
+
+						// Use CSS to hide all but the requested rows
+						// Prevents us from showing more than requested, if actual height was less than average
+						this.print_row_selector = ".egwGridView_grid > tbody > tr:not(:nth-child(-n+"+value+"))";
+						egw.css(this.print_row_selector, 'display: none');
+
+						// No scrollbar in print view
+						$j('.egwGridView_scrollarea',this.div).css('overflow-y','hidden');
+						// Give dialog a chance to close, or it will be in the print
+						window.setTimeout(function() {defer.resolve();}, 0);
+					}
+				},this),
+				egw.lang('How many rows to print'), egw.lang('Print'),
+				Math.min(100, total),
+				[
+					{"button_id": 1,"text": egw.lang('Ok'), id: 'dialog[ok]', image: 'check', "default":true},
+					// Nice for small lists, kills server for large lists
+					//{"button_id": 2,"text": egw.lang('All'), id: 'dialog[all]', image: ''},
+					{"button_id": 0,"text": egw.lang('Cancel'), id: 'dialog[cancel]', image: 'cancel'}
+				]
+			);
+			return defer;
+		}
+		else
+		{
+			// Show all rows
+			this.dynheight.innerNode.css('height', 'auto');
+			$j('.egwGridView_scrollarea',this.div).css('height','auto');
+		}
+		// Don't return anything, just work normally
+	},
+
+	/**
+	 * Try to clean up the mess we made getting ready for printing
+	 * in beforePrint()
+	 */
+	afterPrint: function() {
+
+		this.div.removeClass('print');
+
+		// Put scrollbar back
+		$j('.egwGridView_scrollarea',this.div).css('overflow-y','');
+
+		// Correct size of grid, and trigger resize to fix it
+		this.controller._grid.setScrollHeight(this.old_height);
+		delete this.old_height;
+
+		// Remove CSS rule hiding extra rows
+		if(this.print_row_selector)
+		{
+			egw.css(this.print_row_selector, false);
+			delete this.print_row_selector;
+		}
+
+		this.dynheight.outerNode.css('max-width','inherit');
+		this.resize();
 	}
 });
 et2_register_widget(et2_nextmatch, ["nextmatch"]);
@@ -1925,7 +2278,14 @@ var et2_nextmatch_header_bar = et2_DOMWidget.extend(et2_INextmatchHeader,
 			.on("keypress", function(event) {
 				if(event.which == 13)
 				{
-					self.nextmatch.applyFilters({search: self.search.getValue()});
+					event.preventDefault();
+					self.getInstanceManager().autocomplete_fixer();
+					// Use a timeout to make sure we get the autocomplete value,
+					// if one was chosen, instead of what was actually typed.
+					// Chrome doesn't need this, but FF does.
+					window.setTimeout(function() {
+						self.nextmatch.applyFilters({search: self.search.getValue()});
+					},0);
 				}
 			});
 		// Firefox treats search differently.  Add in the clear button.
@@ -1955,7 +2315,7 @@ var et2_nextmatch_header_bar = et2_DOMWidget.extend(et2_INextmatchHeader,
 		// Add category
 		if(!settings.no_cat) {
 			settings.cat_id_label = egw.lang("Category");
-			this.category = this._build_select('cat_id', 'select-cat', settings.cat_id, true);
+			this.category = this._build_select('cat_id', 'select-cat', settings.cat_id, settings.cat_is_select !== true);
 		}
 
 		// Filter 1
@@ -2146,7 +2506,7 @@ var et2_nextmatch_header_bar = et2_DOMWidget.extend(et2_INextmatchHeader,
 		}
 
 		// Legacy: Add in 'All' option for cat_id, if not provided.
-		if(name == 'cat_id' && options != null && typeof options[''] == 'undefined' && typeof options[0] == 'undefined')
+		if(name == 'cat_id' && options != null && (typeof options[''] == 'undefined' && typeof options[0] != 'undefined' && options[0].value != ''))
 		{
 			widget_options.empty_label = this.egw().lang('All');
 			this.egw().debug('warn', 'Nextmatch category filter had no "All" option.  Added, but you should fix that.');
@@ -2538,6 +2898,7 @@ var et2_nextmatch_customfields = et2_customfields_list.extend(et2_INextmatchHead
 		}
 		var columnMgr = this.nextmatch.dataview.getColumnMgr();
 		var nm_column = null;
+		var set_fields = {};
 		for(var i = 0; i < this.nextmatch.columns.length; i++)
 		{
 			if(this.nextmatch.columns[i].widget == this)
@@ -2570,14 +2931,21 @@ var et2_nextmatch_customfields = et2_customfields_list.extend(et2_INextmatchHead
 
 			// Create widget by type
 			var widget = null;
-
-			if(field.type == 'select')
+			if(field.type == 'select' || field.type == 'select-account')
 			{
-				widget = et2_createWidget("nextmatch-filterheader", {
-					id: cf_id,
-					label: field.label,
-					select_options: field.values
-				}, this);
+				if(field.values && typeof field.values[''] !== 'undefined')
+				{
+					delete(field.values['']);
+				}
+				widget = et2_createWidget(
+					field.type == 'select-account' ? 'nextmatch-accountfilter' : "nextmatch-filterheader",
+					{
+						id: cf_id,
+						empty_label: field.label,
+						select_options: field.values
+					},
+					this
+				);
 			}
 			else if (apps[field.type])
 			{
@@ -2601,7 +2969,13 @@ var et2_nextmatch_customfields = et2_customfields_list.extend(et2_INextmatchHead
 			{
 				cf.hide();
 			}
+			else if (jQuery.isEmptyObject(this.options.fields))
+			{
+				// If we're showing it make sure it's set, but only after
+				set_fields[field_name] = true;
+			}
 		}
+		jQuery.extend(this.options.fields, set_fields);
 	},
 
 	/**
@@ -2791,9 +3165,9 @@ var et2_nextmatch_filterheader = et2_selectbox.extend([et2_INextmatchHeader, et2
 		this.nextmatch = _nextmatch;
 
 		// Set current filter value from nextmatch settings
-		if(this.nextmatch.options.settings.col_filter && typeof this.nextmatch.options.settings.col_filter[this.id] != "undefined")
+		if(this.nextmatch.activeFilters.col_filter && typeof this.nextmatch.activeFilters.col_filter[this.id] != "undefined")
 		{
-			this.set_value(this.nextmatch.options.settings.col_filter[this.id]);
+			this.set_value(this.nextmatch.activeFilters.col_filter[this.id]);
 
 			// Make sure it's set in the nextmatch
 			_nextmatch.activeFilters.col_filter[this.id] = this.getValue();
@@ -2839,12 +3213,6 @@ var et2_nextmatch_accountfilterheader = et2_selectAccount.extend([et2_INextmatch
 
 	},
 
-	set_select_options: function(_options) {
-		// Tell framework to ignore, or it will reset it to ''/empty when it does loadingFinished()
-		this.attributes.select_options.ignore = true;
-		this._super.apply(this, arguments);
-	},
-
 	/**
 	 * Set nextmatch is the function which has to be implemented for the
 	 * et2_INextmatchHeader interface.
@@ -2855,9 +3223,9 @@ var et2_nextmatch_accountfilterheader = et2_selectAccount.extend([et2_INextmatch
 		this.nextmatch = _nextmatch;
 
 		// Set current filter value from nextmatch settings
-		if(this.nextmatch.options.settings.col_filter && this.nextmatch.options.settings.col_filter[this.id])
+		if(this.nextmatch.activeFilters.col_filter && this.nextmatch.activeFilters.col_filter[this.id])
 		{
-			this.set_value(this.nextmatch.options.settings.col_filter[this.id]);
+			this.set_value(this.nextmatch.activeFilters.col_filter[this.id]);
 		}
 	},
 	// Make sure selectbox is not longer than the column
@@ -2971,12 +3339,14 @@ var et2_nextmatch_customfilter = et2_nextmatch_filterheader.extend(
 		"widget_type": {
 			"name": "Actual type",
 			"type": "string",
-			"description": "The actual type of widget you should use"
+			"description": "The actual type of widget you should use",
+			"no_lang": 1
 		},
 		"widget_options": {
 			"name": "Actual options",
 			"type": "any",
 			"description": "The options for the actual widget",
+			"no_lang": 1,
 			"default": {}
 		}
 	},
@@ -2992,7 +3362,6 @@ var et2_nextmatch_customfilter = et2_nextmatch_filterheader.extend(
 	 * @memberOf et2_nextmatch_customfilter
 	 */
 	init: function(_parent, _attrs) {
-		this._super.apply(this, arguments);
 
 		switch(_attrs.widget_type)
 		{
@@ -3000,11 +3369,29 @@ var et2_nextmatch_customfilter = et2_nextmatch_filterheader.extend(
 				_attrs.type = 'nextmatch-entryheader';
 				break;
 			default:
-				_attrs.type = _attrs.widget_type;
+				if(_attrs.widget_type.indexOf('select') === 0)
+				{
+					_attrs.type = 'nextmatch-filterheader';
+				}
+				else
+				{
+					_attrs.type = _attrs.widget_type;
+				}
 		}
-		// Avoid warning about non-existant attribute
-		delete(_attrs.widget_type);
+		jQuery.extend(_attrs.widget_options,{id: this.id});
+
+		_attrs.id = '';
+		this._super.apply(this, arguments);
 		this.real_node = et2_createWidget(_attrs.type, _attrs.widget_options, this._parent);
+		var select_options = [];
+		var correct_type = _attrs.type;
+		this.real_node._type = _attrs.widget_type;
+		et2_selectbox.find_select_options(this.real_node, select_options, _attrs);
+		this.real_node._type = correct_type;
+		if(typeof this.real_node.set_select_options === 'function')
+		{
+			this.real_node.set_select_options(select_options);
+		}
 	},
 
 	// Just pass the real DOM node through, in case anybody asks
@@ -3014,7 +3401,7 @@ var et2_nextmatch_customfilter = et2_nextmatch_filterheader.extend(
 
 	// Also need to pass through real children
 	getChildren: function() {
-		return this.real_node.getChildren();
+		return this.real_node.getChildren() || [];
 	},
 	setNextmatch: function(_nextmatch)
 	{
