@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel
  * @copyright Stylite 2012
- * @version $Id: et2_extension_nextmatch_controller.js 49674 2014-12-02 22:15:44Z nathangray $
+ * @version $Id$
  */
 
 "use strict";
@@ -19,6 +19,7 @@
 	et2_dataview_view_row;
 	et2_dataview_controller;
 	et2_dataview_interfaces;
+	et2_dataview_view_tile;
 
 	et2_extension_nextmatch_actions; // Contains nm_action
 
@@ -30,6 +31,10 @@
  */
 var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 {
+	// Display constants
+	VIEW_ROW: 'row',
+	VIEW_TILE: 'tile',
+
 	/**
 	 * Initializes the nextmatch controller.
 	 *
@@ -94,6 +99,8 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 		// dataUnregisterUID
 		this.dataUnregisterUID = _egw.dataUnregisterUID;
 
+		// Default to rows
+		this._view = et2_nextmatch_controller.prototype.VIEW_ROW;
 	},
 
 	destroy: function () {
@@ -190,6 +197,34 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 
 	/** -- PRIVATE FUNCTIONS -- **/
 
+	/**
+	 * Create a new row, either normal or tiled
+	 *
+	 * @param {type} ctx
+	 * @returns {et2_dataview_container}
+	 */
+	_createRow: function(ctx) {
+		switch(this._view)
+		{
+			case et2_nextmatch_controller.prototype.VIEW_TILE:
+				var row = new et2_dataview_tile(this._grid);
+				// Try to overcome chrome rendering issue where float is not
+				// applied properly, leading to incomplete rows
+				window.setTimeout(function() {
+					if(!row.tr) return;
+					row.tr.css('float','none');
+					window.setTimeout(function() {
+						if(!row.tr) return;
+						row.tr.css('float','left');
+					},50);
+				},100);
+				return row;
+			case et2_nextmatch_controller.prototype.VIEW_ROW:
+			default:
+				return new et2_dataview_row(this._grid);
+		}
+	},
+
 	/**
 	 * Initializes the action and the object manager.
 	 */
@@ -245,11 +280,47 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 		{
 			this._objectManager = gom.addObject(
 				new egwActionObjectManager(uid, this._actionManager));
+
+			this._objectManager.handleKeyPress = function(_keyCode, _shift, _ctrl, _alt) {
+				for(var i = 0; i < self._actionManager.children.length; i++)
+				{
+					if(typeof self._actionManager.children[i].shortcut === 'object' &&
+						self._actionManager.children[i].shortcut &&
+						_keyCode == self._actionManager.children[i].shortcut.keyCode)
+					{
+						return this.executeActionImplementation(
+							{
+								"keyEvent": {
+									"keyCode": _keyCode,
+									"shift": _shift,
+									"ctrl": _ctrl,
+									"alt": _alt
+								}
+							}, "popup", EGW_AO_EXEC_SELECTED);
+					}
+				}
+				return egwActionObject.prototype.handleKeyPress.call(this, _keyCode,_shift,_ctrl,_alt);
+			}
+			
 		}
 		this._objectManager.flags = this._objectManager.flags
 				| EGW_AO_FLAG_DEFAULT_FOCUS | EGW_AO_FLAG_IS_CONTAINER;
 
 		this._init_links_dnd(this._actionManager);
+
+		if(this._selectionMgr)
+		{
+			// Need to update the action links for every registered row too
+			for (var uid in this._selectionMgr._registeredRows)
+			{
+				// Get the corresponding entry from the registered rows array
+				var entry = this._selectionMgr._getRegisteredRowsEntry(uid);
+				if(entry.ao)
+				{
+					entry.ao.updateActionLinks(this._actionLinks);
+				}
+			}
+		}
 	},
 
 	/**
@@ -261,7 +332,23 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 
 		var drop_action = mgr.getActionById('egw_link_drop');
 		var drag_action = mgr.getActionById('egw_link_drag');
-
+		var drop_cancel = mgr.getActionById('egw_cancel_drop');
+		
+		if(!this._actionLinks)
+		{
+			this._actionLinks = [];
+		}
+		
+		if (!drop_cancel)
+		{
+			// Create a generic cancel action in order to cancel drop action
+			// applied for all apps plus file and link action.
+			drop_cancel = mgr.addAction('drop', 'egw_cancel_drop', this.egw.lang('Cancel'), egw.image('cancel'), function(){},true);
+			drop_cancel.set_group('99');
+			drop_cancel.acceptedTypes = drop_cancel.acceptedTypes.concat(Object.keys(egw.user('apps')).concat(['link', 'file']));
+			this._actionLinks.push (drop_cancel.id);
+		}
+		
 		// Check if this app supports linking
 		if(!egw.link_get_registry(this.dataStorePrefix || this.egw.appName, 'query') ||
 			egw.link_get_registry(this.dataStorePrefix || this.egw.appName, 'title'))
@@ -289,7 +376,7 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 		if(drop_action == null)
 		{
 			// Create the drop action that links entries
-			drop_action = mgr.addAction('drop', 'egw_link_drop', this.egw.lang('link'), 'link', function(action, source, dropped) {
+			drop_action = mgr.addAction('drop', 'egw_link_drop', this.egw.lang('Create link'), egw.image('link'), function(action, source, dropped) {
 				// Extract link IDs
 				var links = [];
 				var id = '';
@@ -310,6 +397,11 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 					function(result) {
 						if(result)
 						{
+							for (var i=0; i < this._objectManager.selectedChildren.length; i++)
+							{
+								this._widget.refresh(this._objectManager.selectedChildren[i].id,'update');
+							}
+							this._widget.egw().message('Linked');
 							// Update the target to show the liks
 							this._widget.refresh(dropped.id,'update');
 						}
@@ -318,7 +410,7 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 					true,
 					self
 				).sendRequest();
-				
+
 			},true);
 		}
 		if(this._actionLinks.indexOf(drop_action.id) < 0)
@@ -372,7 +464,7 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 
 		this._init_links_dnd(this._actionManager);
 	},
-	
+
 	/**
 	 * Overwrites the inherited _destroyCallback function in order to be able
 	 * to free the "rowWidget".
@@ -432,7 +524,8 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 		// No UID, so return a filtered list of actions that doesn't need a UID
 		var links = [];
 		try {
-			links = typeof this._widget.options.settings.placeholder_actions != 'undefined' ? this._widget.options.settings.placeholder_actions : ["add"];
+			links = typeof this._widget.options.settings.placeholder_actions != 'undefined' ?
+				this._widget.options.settings.placeholder_actions : (this._widget.options.add ? ["add"] : []);
 		} catch (e) {
 		}
 
@@ -510,17 +603,29 @@ var et2_nextmatch_controller = et2_dataview_controller.extend(et2_IDataProvider,
 			{
 				this.self._selectionMgr.selectAll();
 			}
-			for(var i = 0; i < this.self.kept_selection.ids.length || 0; i++)
+			for(var i = (this.self.kept_selection.ids.length || 1)-1; i >= 0; i--)
 			{
-				this.self._selectionMgr.setSelected(this.self.kept_selection.ids[i],true);
+				// Only keep the selected if they came back in the fetch
+				if(_response.order.indexOf(this.self.kept_selection.ids[i]) >= 0)
+				{
+					this.self._selectionMgr.setSelected(this.self.kept_selection.ids[i],true);
+					this.self.kept_selection.ids.splice(i,1);
+				}
+				else
+				{
+					this.self.kept_selection.ids.splice(i,1);
+				}
 			}
-			if(this.self.kept_focus)
+			if(this.self.kept_focus && _response.order.indexOf(this.self.kept_focus) >= 0)
 			{
 				this.self._selectionMgr.setFocused(this.self.kept_focus,true);
 			}
 			// Re-expanding rows handled in et2_extension_nextmatch_rowProvider
 			// Expansions might still be valid, so we don't clear them
-			this.self.kept_selection = null;
+			if(this.self.kept_selection != null && typeof this.self.kept_selection.ids != 'undefined' && this.self.kept_selection.ids.length == 0)
+			{
+				this.self.kept_selection = null;
+			}
 			this.self.kept_focus = null;
 		}
 	},
