@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Nathan Gray
  * @copyright Nathan Gray 2011
- * @version $Id: et2_widget_date.js 49716 2014-12-05 11:07:08Z hnategh $
+ * @version $Id$
  */
 
 "use strict";
@@ -37,11 +37,41 @@ var et2_date = et2_inputWidget.extend(
 		"type": {
 			"ignore": false
 		},
+		"blur": {
+			"name": "Placeholder",
+			"type": "string",
+			"default": "",
+			"description": "This text get displayed if an input-field is empty and does not have the input-focus (blur). It can be used to show a default value or a kind of help-text."
+		},
 		"data_format": {
 			"ignore": true,
 			"description": "Date/Time format. Can be set as an options to date widget",
 			"default": ''
-		}
+		},
+		year_range: {
+			name: "Year range",
+			type: "string",
+			default: "c-10:c+10",
+			description: "The range of years displayed in the year drop-down: either relative to today's year (\"-nn:+nn\"), relative to the currently selected year (\"c-nn:c+nn\"), absolute (\"nnnn:nnnn\"), or combinations of these formats (\"nnnn:-nn\"). Note that this option only affects what appears in the drop-down, to restrict which dates may be selected use the min and/or max options."
+		},
+		min: {
+			"name": "Minimum",
+			"type": "any",
+			"default": et2_no_init,
+			"description": 'Minimum allowed date.  Multiple types supported:\
+Date: A date object containing the minimum date.\
+Number: A number of days from today. For example 2 represents two days from today and -1 represents yesterday.\
+String: A string in the user\'s date format, or a relative date. Relative dates must contain value and period pairs; valid periods are "y" for years, "m" for months, "w" for weeks, and "d" for days. For example, "+1m +7d" represents one month and seven days from today.'
+		},
+		max: {
+			"name": "Maximum",
+			"type": "any",
+			"default": et2_no_init,
+			"description": 'Maximum allowed date.   Multiple types supported:\
+Date: A date object containing the maximum date.\
+Number: A number of days from today. For example 2 represents two days from today and -1 represents yesterday.\
+String: A string in the user\'s date format, or a relative date. Relative dates must contain value and period pairs; valid periods are "y" for years, "m" for months, "w" for weeks, and "d" for days. For example, "+1m +7d" represents one month and seven days from today.'
+		},
 	},
 
 	legacyOptions: ["data_format"],
@@ -68,6 +98,7 @@ var et2_date = et2_inputWidget.extend(
 		this.span = $j(document.createElement("span")).addClass("et2_date");
 
 		this.input_date = $j(document.createElement("input"));
+		if (this.options.blur) this.input_date.attr('placeholder', this.egw().lang(this.options.blur));
 		this.input_date.addClass("et2_date").attr("type", "text")
 			.attr("size", 7)	// strlen("10:00pm")=7
 			.appendTo(this.span);
@@ -84,6 +115,28 @@ var et2_date = et2_inputWidget.extend(
 			this.input_date.addClass("et2_time");
 			this.egw().time(this.input_date);
 		}
+
+		// Avoid collision of datepicker dialog with input field
+		this.input_date.datepicker('option', 'beforeShow', function(input, inst){
+			var cal = inst.dpDiv;
+			setTimeout(function () {
+				var $input = jQuery(input);
+				var inputOffset = $input.offset();
+				// position the datepicker in freespace zone
+				// avoid datepicker calendar collision with input field
+				if (cal.height() + inputOffset.top > window.innerHeight)
+				{
+					cal.position({
+						my: "left center",
+						at: 'right bottom',
+						collision: 'flip fit',
+						of: input,
+					});
+				}
+				
+			},0);
+		});
+		
 		// Update internal value when changed
 		var self = this;
 		this.input_date.bind('change', function(e){
@@ -106,6 +159,20 @@ var et2_date = et2_inputWidget.extend(
 		}
 	},
 
+	/**
+	 * Dynamic disable or enable datepicker
+	 *
+	 * @param {boolean} _ro
+	 */
+	set_readonly: function(_ro)
+	{
+		if (this.input_date && !this.input_date.attr('disabled') != !_ro)
+		{
+			this.input_date.attr('disabled', !!_ro)
+				.datepicker('option', 'disabled', !!_ro);
+		}
+	},
+
 	/**
 	 * Set (full) year of current date
 	 *
@@ -213,6 +280,106 @@ var et2_date = et2_inputWidget.extend(
 		return this.input_date.val() == "" ? null : this.date.getTime();
 	},
 
+	/**
+	 * The range of years displayed in the year drop-down: either relative
+	 * to today's year ("-nn:+nn"), relative to the currently selected year
+	 * ("c-nn:c+nn"), absolute ("nnnn:nnnn"), or combinations of these formats
+	 * ("nnnn:-nn"). Note that this option only affects what appears in the
+	 * drop-down, to restrict which dates may be selected use the min_date
+	 * and/or max_date options.
+	 * @param {string} _value
+	 */
+	set_year_range: function(_value)
+	{
+		if(this.input_date && this._type == 'date')
+		{
+			this.input_date.datepicker('option','yearRange',_value);
+		}
+		this.options.year_range = _value;
+	},
+
+	/**
+	 * Set the minimum allowed date
+	 *
+	 * The minimum selectable date. When set to null, there is no minimum.
+	 *	Multiple types supported:
+	 *	Date: A date object containing the minimum date.
+	 *	Number: A number of days from today. For example 2 represents two days
+	 *		from today and -1 represents yesterday.
+	 *	String: A string in the format defined by the dateFormat option, or a
+	 *		relative date. Relative dates must contain value and period pairs;
+	 *		valid periods are "y" for years, "m" for months, "w" for weeks, and
+	 *		"d" for days. For example, "+1m +7d" represents one month and seven
+	 *		days from today.
+	 * @param {Date|Number|String} _value
+	 */
+	set_min: function(_value) {
+		if(this.input_date)
+		{
+			if (this.is_mobile)
+			{
+				this.input_date.attr('min', this._relativeDate(_value));
+			}
+			else
+			{
+				// Check for full timestamp
+				if(typeof _value == 'string' && _value.match(/(\d{4})-(\d{2})-(\d{2})T(\d{2})\:(\d{2})\:(\d{2})(?:\.\d{3})?(?:Z|[+-](\d{2})\:(\d{2}))/))
+				{
+					_value = new Date(_value);
+					// Add timezone offset back in, or formatDate will lose those hours
+					var formatDate = new Date(_value.valueOf() + this.date.getTimezoneOffset() * 60 * 1000);
+					if(this._type == 'date')
+					{
+						_value = jQuery.datepicker.formatDate(this.input_date.datepicker('option', 'dateFormat'), formatDate);
+					}
+				}
+				this.input_date.datepicker('option','minDate',_value);
+			}
+		}
+		this.options.min = _value;
+	},
+
+	/**
+	 * Set the maximum allowed date
+	 *
+	 * The maximum selectable date. When set to null, there is no maximum.
+	 *	Multiple types supported:
+	 *	Date: A date object containing the maximum date.
+	 *	Number: A number of days from today. For example 2 represents two days
+	 *		from today and -1 represents yesterday.
+	 *	String: A string in the format defined by the dateFormat option, or a
+	 *		relative date. Relative dates must contain value and period pairs;
+	 *		valid periods are "y" for years, "m" for months, "w" for weeks, and
+	 *		"d" for days. For example, "+1m +7d" represents one month and seven
+	 *		days from today.
+	 * @param {Date|Number|String} _value
+	 */
+	set_max: function(_value) {
+		if(this.input_date)
+		{
+			if (this.is_mobile)
+			{
+				this.input_date.attr('max', this._relativeDate(_value));
+			}
+			else
+			{
+				// Check for full timestamp
+				if(typeof _value == 'string' && _value.match(/(\d{4})-(\d{2})-(\d{2})T(\d{2})\:(\d{2})\:(\d{2})(?:\.\d{3})?(?:Z|[+-](\d{2})\:(\d{2}))/))
+				{
+					_value = new Date(_value);
+					// Add timezone offset back in, or formatDate will lose those hours
+					var formatDate = new Date(_value.valueOf() + this.date.getTimezoneOffset() * 60 * 1000);
+					if(this._type == 'date')
+					{
+						_value = jQuery.datepicker.formatDate(this.input_date.datepicker('option', 'dateFormat'), formatDate);
+					}
+				}
+				this.input_date.datepicker('option','maxDate',_value);
+			}
+		}
+		this.options.max = _value;
+	},
+	
 	/**
 	 * Setting date
 	 *
@@ -393,7 +560,7 @@ var et2_date = et2_inputWidget.extend(
 
 		// Convert to timestamp - no seconds
 		this.date.setSeconds(0,0);
-		return this.date?this.date.toJSON().replace(/\.\d{3}Z$/, 'Z'):this.date;
+		return (this.date && typeof this.date.toJSON != 'undefined' && this.date.toJSON())?this.date.toJSON().replace(/\.\d{3}Z$/, 'Z'):this.date;
 	}
 });
 et2_register_widget(et2_date, ["date", "date-time", "date-timeonly"]);
@@ -472,7 +639,7 @@ var et2_date_duration = et2_date.extend(
 
 		// Get translations
 		this.time_formats = {
-			"d": this.options.short_labels ? this.egw().lang("m") : this.egw().lang("Days"),
+			"d": this.options.short_labels ? this.egw().lang("d") : this.egw().lang("Days"),
 			"h": this.options.short_labels ? this.egw().lang("h") : this.egw().lang("Hours"),
 			"m": this.options.short_labels ? this.egw().lang("m") : this.egw().lang("Minutes")
 		},
@@ -568,7 +735,7 @@ var et2_date_duration = et2_date.extend(
 			}
 			else
 			{
-				this.format.text(this.time_formats[display.unit]);
+				this.format.text(display.unit ? this.time_formats[display.unit] : '');
 			}
 		}
 	},
@@ -640,6 +807,9 @@ var et2_date_duration = et2_date.extend(
 				value *= 60;
 				break;
 		}
+		// Minutes should be an integer.  Floating point math.
+		value = Math.round(value);
+		
 		switch(this.options.data_format)
 		{
 			case 'd':
@@ -746,7 +916,10 @@ var et2_date_ro = et2_valueWidget.extend([et2_IDetachedDOM],
 		"data_format": {
 			"ignore": true,
 			"description": "Format data is in.  This is not used client-side because it's always a timestamp client side."
-		}
+		},
+		min: {ignore: true},
+		max: {ignore: true},
+		year_range: {ignore: true}
 	},
 
 	legacyOptions: ["data_format"],
@@ -821,12 +994,17 @@ var et2_date_ro = et2_valueWidget.extend([et2_IDetachedDOM],
 		var display = this.date.toString();
 
 		switch(this._type) {
+			case "time_or_date":
 			case "date-time_today":
 				// Today - just the time
 				if(date('Y-m-d', this.date) == date('Y-m-d'))
 				{
 					display = date(this.egw().preference('timeformat') == '24' ? 'H:i' : 'g:i a', this.date);
 				}
+				else if (this._type === "time_or_date")
+				{
+					display = date(this.egw().preference('dateformat'), this.date);
+				}
 				// Before today - date and time
 				else
 				{
@@ -937,39 +1115,293 @@ var et2_date_ro = et2_valueWidget.extend([et2_IDetachedDOM],
 		}
 	}
 });
-et2_register_widget(et2_date_ro, ["date_ro", "date-time_ro", "date-since", "date-time_today"]);
+et2_register_widget(et2_date_ro, ["date_ro", "date-time_ro", "date-since", "date-time_today", "time_or_date", "date-timeonly_ro"]);
+
 
 /**
- * @augments et2_date_ro
+ * Widget for selecting a date range
+ *
+ * @augments et2_inputWidget
  */
-var et2_date_timeonly_ro = et2_date_ro.extend(
-{
+var et2_date_range = et2_inputWidget.extend({
 	attributes: {
-		"value": {
-			"type": "string"
+		value: {
+			"type": "any",
+			"description": "An object with keys 'from' and 'to' for absolute ranges, or a relative range string"
+		},
+		relative: {
+			name: 'Relative',
+			type: 'boolean',
+			description: 'Is the date range relative (this week) or absolute (2016-02-15 - 2016-02-21).  This will affect the value returned.'
 		}
 	},
+
 	/**
-	 * Construtor
+	 * Constructor
 	 *
-	 * @param _value
-	 * @memberOf et2_date_timeonly_ro
+	 * @memberOf et2_number
 	 */
-	set_value: function(_value) {
-		if(this.egw().preference("timeformat") == "12" && _value.indexOf(":") > 0) {
-			var parts = _value.split(":");
-			if(parts[0] >= 12) {
-				this.span.text((parts[0] == "12" ? "12" : parseInt(parts[0])-12)+":"+parts[1]+" pm");
-			}
-			else
-			{
-				this.span.text(_value + " am");
-			}
+	init: function init() {
+		this._super.apply(this, arguments);
+
+		this.div = jQuery(document.createElement('div'))
+			.attr({	class:'et2_date_range'});
+
+		this.from = null;
+		this.to = null;
+		this.select = null;
+
+		// Set domid
+		this.set_id(this.id);
+
+		this.setDOMNode(this.div[0]);
+		this._createWidget();
+
+		this.set_relative(this.options.relative || false)
+	},
+
+	_createWidget: function createInputWidget() {
+		var widget = this;
+
+		this.from = et2_createWidget('date',{
+			id: this.id+'[from]',
+			blur: egw.lang('From'),
+			onchange: function() { widget.to.set_min(widget.from.getValue()); }
+		},this);
+		this.to = et2_createWidget('date',{
+			id: this.id+'[to]',
+			blur: egw.lang('To'),
+			onchange: function() {widget.from.set_max(widget.to.getValue()); }
+		},this);
+		this.select = et2_createWidget('select',{
+			id: this.id+'[relative]',
+			select_options: et2_date_range.relative_dates,
+			empty_label: this.options.blur || 'All'
+		},this);
+		this.select.loadingFinished();
+	},
+
+	/**
+	 * Function which allows iterating over the complete widget tree.
+	 * Overridden here to avoid problems with children when getting value
+	 *
+	 * @param _callback is the function which should be called for each widget
+	 * @param _context is the context in which the function should be executed
+	 * @param _type is an optional parameter which specifies a class/interface
+	 * 	the elements have to be instanceOf.
+	 */
+	iterateOver: function(_callback, _context, _type) {
+		if (typeof _type == "undefined")
+		{
+			_type = et2_widget;
+		}
+
+		if (this.isInTree() && this.instanceOf(_type))
+		{
+			_callback.call(_context, this);
+		}
+	},
+
+	/**
+	 * Toggles relative or absolute dates
+	 * 
+	 * @param {boolean} _value
+	 */
+	set_relative: function set_relative(_value)
+	{
+		this.options.relative = _value;
+		if(this.options.relative)
+		{
+			$j(this.from.getDOMNode()).hide();
+			$j(this.to.getDOMNode()).hide();
 		}
 		else
 		{
-			this.span.text(_value);
+			$j(this.select.getDOMNode()).hide();
+		}
+	},
+
+	set_value: function set_value(value)
+	{
+		if(!value || typeof value == 'null')
+		{
+			this.select.set_value('');
+			this.from.set_value(null);
+			this.to.set_value(null);
+		}
+
+		// Relative
+		if(value && typeof value === 'string')
+		{
+			this._set_relative_value(value);
+
+		}
+		else if(value && typeof value.from === 'undefined' && value[0])
+		{
+			value = {
+				from: value[0],
+				to: value[1] || new Date().valueOf()/1000
+			}
+		}
+		else if (value && value.from && value.to)
+		{
+			this.from.set_value(value.from);
+			this.to.set_value(value.to);
+		}
+	},
+
+	getValue: function getValue()
+	{
+		return this.options.relative ?
+			this.select.getValue() :
+			{ from: this.from.getValue(), to: this.to.getValue() }
+	},
+
+	_set_relative_value: function(_value)
+	{
+		if(this.options.relative)
+		{
+			$j(this.select.getDOMNode()).show();
+		}
+		// Show description
+		this.select.set_value(_value);
+
+		var now = new Date();
+		now.setUTCMinutes(-now.getTimezoneOffset());
+		now.setUTCHours(0);
+		now.setUTCSeconds(0);
+
+
+		// Use strings to avoid references
+		this.from.set_value(now.toJSON());
+		this.to.set_value(now.toJSON());
+
+		var relative = null;
+		for(var index in et2_date_range.relative_dates)
+		{
+			if(et2_date_range.relative_dates[index].value === _value)
+			{
+				relative = et2_date_range.relative_dates[index];
+				break;
+			}
+		}
+		if(relative)
+		{
+			var dates = ["from","to"]
+			var value = now.toJSON();
+			for(var i = 0; i < dates.length; i++)
+			{
+				var date = dates[i];
+				if(typeof relative[date] == "function")
+				{
+					value = relative[date](new Date(value))
+				}
+				else
+				{
+					value = this[date]._relativeDate(relative[date]);
+				}
+				this[date].set_value(value);
+			}
 		}
 	}
 });
-et2_register_widget(et2_date_timeonly_ro, ["date-timeonly_ro"]);
+et2_register_widget(et2_date_range, ["date-range"]);
+// Static part of the date range class
+jQuery.extend(et2_date_range,
+{
+	// Class Constants
+	relative_dates: [
+		// Start and end are relative offsets, see et2_date.set_min()
+		// or Date objects
+		{
+			value: 'Today',
+			label: 'Today',
+			from: '',
+			to: '+1d'
+		},
+		{
+			label: 'Yesterday',
+			value: 'Yesterday',
+			from: '-1d',
+			to: ''
+		},
+		{
+			label: 'This week',
+			value: 'This week',
+			from: function(date) {return egw.week_start(date);},
+			to: function(date) {
+				date.setUTCDate(date.getUTCDate() + 6);
+				return date;
+			}
+		},
+		{
+			label: 'Last week',
+			value: 'Last week',
+			from: function(date) {
+				var d = egw.week_start(date);
+				d.setUTCDate(d.getUTCDate() - 7);
+				return d;
+			},
+			to: function(date) {
+				date.setUTCDate(date.getUTCDate() + 6);
+				return date;
+			}
+		},
+		{
+			label: 'This month',
+			value: 'This month',
+			from: '',
+			to: '+1m'
+		},
+		{
+			label: 'Last month',
+			value: 'Last month',
+			from: '-1m',
+			to: ''
+		},
+		{
+			label: 'Last 3 months',
+			value: 'Last 3 months',
+			from: '-3m',
+			to: ''
+		},
+		/*
+		'This quarter'=> array(0,0,0,0,  0,0,0,0),      // Just a marker, needs special handling
+		'Last quarter'=> array(0,-4,0,0, 0,-4,0,0),     // Just a marker
+		*/
+		{
+			label: 'This year',
+			value: 'This year',
+			from: function(d) {
+				d.setUTCMonth(0);
+				d.setUTCDate(1);
+				return d;
+			},
+			to: function(d) {
+				d.setUTCMonth(11);
+				d.setUTCDate(31);
+				return d
+			}
+		},
+		{
+			label: 'Last year',
+			value: 'Last year',
+			from: function(d) {
+				d.setUTCMonth(0);
+				d.setUTCDate(1);
+				d.setUTCYear(d.getUTCYear() - 1)
+				return d;
+			},
+			to: function(d) {
+				d.setUTCMonth(11);
+				d.setUTCDate(31);
+				d.setUTCYear(d.getUTCYear() - 1)
+				return d
+			}
+		},
+		/* Still needed?
+		'2 years ago' => array(-2,0,0,0, -1,0,0,0),
+		'3 years ago' => array(-3,0,0,0, -2,0,0,0),
+		*/
+	]
+});
