@@ -33,7 +33,7 @@
  * @see http://docs.dhtmlx.com/gantt/index.html
  * @augments et2_valueWidget
  */
-var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
+var et2_gantt = et2_inputWidget.extend([et2_IResizeable,et2_IInput],
 {
 	// Filters are inside gantt namespace
 	createNamespace: true,
@@ -57,7 +57,18 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 			"default": "minute",
 			"description": "The unit for task duration values.  One of minute, hour, week, year."
 		},
-		value: {type: 'any'}
+		columns: {
+			name: "Columns",
+			type: "any",
+			default: [
+				{name: "text", label: egw.lang('Title'), tree: true, width: '*'}
+			],
+			description: "Columns for the grid portion of the gantt chart.  An array of objects with keys name, label, etc.  See http://docs.dhtmlx.com/gantt/api__gantt_columns_config.html"
+		},
+		value: {type: 'any'},
+		needed: {ignore: true},
+		onfocus: {ignore: true},
+		tabindex: {ignore: true}
 	},
 	
 	// Common configuration for Egroupware/eTemplate
@@ -71,8 +82,10 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 		duration_step: 1,
 
 		show_progress: true,
-		order_branch: true,
+		order_branch: false,
 		min_column_width: 30,
+		min_grid_column_width: 30,
+		task_height: 25,
 		fit_tasks: true,
 		autosize: '',
 		// Date rounding happens either way, but this way it rounds to the displayed grid resolution
@@ -173,6 +186,10 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 		{
 			this.set_value(this.options.value);
 		}
+		if(this.options.columns)
+		{
+			this.set_columns(this.options.columns);
+		}
 		
 		// Update start & end dates with chart values for consistency
 		if(start_date && this.options.value.data && this.options.value.data.length)
@@ -240,6 +257,83 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 		}
 	},
 
+	/**
+	 * Set the columns for the grid (left) portion
+	 *
+	 * @param {Object[]} columns - A list of columns
+	 * @param {string} columns[].name The column's ID
+	 * @param {string} columns[].label The title for the column header
+	 * @param {string} columns[].width Width of the column
+	 *
+	 * @see http://docs.dhtmlx.com/gantt/api__gantt_columns_config.html for full options
+	 */
+	set_columns: function(columns)
+	{
+		this.gantt_config.columns = columns;
+
+		var displayed_columns = [];
+		var gantt_widget = this;
+
+		// Make sure there's enough room for them all
+		var width = 0;
+		for(var col in columns)
+		{
+			// Preserve original width, gantt will resize column to fit
+			if(!columns[col]._width)
+			{
+				columns[col]._width = columns[col].width;
+			}
+			columns[col].width = columns[col]._width;
+			if(!columns[col].template)
+			{
+				// Use an et2 widget to render the column value, if one was provided
+				// otherwise, just display the value
+				columns[col].template = function(task) {
+					var value = typeof task[this.name] == 'undefined'||task[this.name] == null ? '':task[this.name];
+
+					// No value, but there's a project title.  Try reading the project value.
+					if(!value && this.name.indexOf('pe_') == 0 && task.pm_title)
+					{
+						var pm_col = this.name.replace('pe_','pm_');
+						value = typeof task[pm_col] == 'undefined' || task[pm_col] == null ? '':task[pm_col];
+					}
+					if(this.widget && typeof this.widget == 'string')
+					{
+						var attrs = jQuery.extend({readonly:true}, this.widget_attributes||{});
+						this.widget = et2_createWidget(this.widget, attrs, gantt_widget);
+					}
+					if (this.widget)
+					{
+						this.widget.set_value(value);
+						value = $j(this.widget.getDOMNode()).html();
+					}
+					return '<div class="gantt_column_'+this.name+'">' + value + '</div>';
+				};
+			}
+
+			// Actual hiding is available in the pro version of gantt chart
+			if(!columns[col].hide)
+			{
+				displayed_columns.push(columns[col]);
+				width += parseInt(columns[col]._width) || 0;
+			}
+
+		}
+		// Add in add column
+		displayed_columns.push({name: 'add', width: 26});
+		width += 26;
+
+		if(width != this.gantt_config.grid_width || typeof this.gantt_config.grid_width == 'undefined')
+		{
+			this.gantt_config.grid_width = Math.min(Math.max(200, width), this.htmlNode.width());
+		}
+
+		if(this.gantt == null) return;
+		this.gantt.config.columns = displayed_columns;
+		this.gantt.config.grid_width = this.gantt_config.grid_width;
+		this.gantt.render();
+	},
+
 	/**
 	 * Sets the data to be displayed in the gantt chart.
 	 *
@@ -298,7 +392,8 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 			this.config.start_date = value.start_date || null;
 			this.config.end_date = value.end_date || null;
 			this.parse(safe_value);
-
+			
+			gantt_widget._apply_sort();
 			gantt_widget.gantt_loading = false;
 			// Once we force the start / end date (below), gantt won't recalculate
 			// them if the user clears the date, so we store them and use them
@@ -415,6 +510,7 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 						[_task_ids[i],value,task.parent||false],
 						function(data) {
 							this.gantt.parse(data);
+							this._apply_sort();
 							this.gantt.hideCover();
 						},
 						this,true,this
@@ -428,6 +524,7 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 					if(data = this.egw().dataGetUIDdata(_task_ids[i]) && data.data)
 					{
 						this.gantt.parse(data.data);
+						this._apply_sort();
 					}
 					else
 					{
@@ -591,6 +688,87 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 		return level;
 	},
 
+	/**
+	 * Apply user's sort preference
+	 */
+	_apply_sort: function()
+	{
+		switch(egw.preference('gantt_pm_elementbars_order','projectmanager'))
+		{
+			case "pe_start":
+			case "pe_start,pe_end":
+				this.gantt.sort('start_date',false);
+				break;
+			case "pe_end":
+				this.gantt.sort('end_date',false);
+				break;
+			case 'pe_title':
+				this.gantt.sort('pe_title',false);
+				break;
+		}
+	},
+
+	/**
+	 * Exports the gantt chart to an external service that generates a file.
+	 *
+	 * @param {string} to One of PDF or PNG
+	 * @returns {undefined}
+	 */
+	_export: function(to) {
+		var w = egw.open_link(egw.link('/etemplate/gantt_print.php'),'_blank','400x400');
+		var self = this;
+		jQuery(w).load(function() {
+			w.egw_LAB.wait(function() {
+				w.gantt = jQuery.extend(true,{},self.gantt);
+
+				// Loading it twice breaks export
+				if(typeof w.gantt.exportToPNG == 'undefined')
+				{
+					w.egw_LAB.script("https://export.dhtmlx.com/gantt/api.js");
+				}
+				w.egw_LAB.wait(function() {
+					$j(w.gantt.$container).parent().clone().appendTo(w.document.body);
+					// Custom CSS - just send it all
+					var css = '';
+					$j("link[type='text/css']").each(function() {css += this.outerHTML;});
+
+					var options = {
+						name: (w.gantt.getTask(w.gantt._order[0]).text||'gantt').replace(/ /g,'_')+'.'+to.toLowerCase(),
+						header: css + egw.config('site_title','phpgwapi'),
+						footer: $j('#egw_fw_footer',w.opener).html(),
+						// Doesn't work, export never happens:
+						// callback: function() {w.setTimeout(function() {w.close();}, 5000);}
+					};
+					console.log(options);
+					switch(to)
+					{
+						case 'PNG':
+							w.gantt.exportToPNG(options);
+							break;
+						case 'PDF':
+							w.gantt.exportToPDF(options);
+					}
+					w.setTimeout(function() {w.close();}, 5000);
+				});
+			});
+		});
+	},
+
+
+	/**
+	 * Exports the gantt chart do dhtmlx's external service, and makes a PDF
+	 */
+	exportToPDF: function() {
+		this._export('PDF');
+	},
+
+	/**
+	 * Exports the gantt chart do dhtmlx's external service, and makes a PNG
+	 */
+	exportToPNG: function() {
+		this._export('PNG');
+	},
+
 	/**
 	 * Bind all the internal gantt events for nice widget actions
 	 */
@@ -612,11 +790,9 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 			// Some crazy stuff make sure timing is OK to scroll after re-render
 			// TODO: Make this more consistently go to where you click
 			var id = gantt_widget.gantt.attachEvent("onGanttRender", function() {
-				console.log('Render');
 				gantt_widget.gantt.detachEvent(id);
 				gantt_widget.gantt.scrollTo(parseInt($j('.gantt_task_scale',gantt_widget.gantt_node).width() *current_position),0);
 				window.setTimeout(function() {
-					console.log("Scroll to");
 					gantt_widget.gantt.scrollTo(parseInt($j('.gantt_task_scale',gantt_widget.gantt_node).width() *current_position),0);
 				},100);
 			});
@@ -641,7 +817,14 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 			*/
 		});
 
+		this.gantt.attachEvent("onGridHeaderClick", function(column_name, e) {
+			if(column_name === "add")
+			{
+				gantt_widget._column_selection(e);
+			}
+		});
 		this.gantt.attachEvent("onContextMenu",function(taskId, linkId, e) {
+			if(gantt_widget.options.readonly) return false;
 			if(taskId)
 			{
 				gantt_widget._link_task(taskId);
@@ -662,6 +845,16 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 
 		// Update server after dragging a task
 		this.gantt.attachEvent("onAfterTaskDrag", function(id, mode, e) {
+			if(gantt_widget.options.readonly) return false;
+
+			// Round to nearest 10%
+			var task = this.getTask(id);
+			if(mode==="progress")
+			{
+				task.progress = Math.round(task.progress * 10) / 10;
+				this.updateTask(task.id);
+			}
+
 			var task = jQuery.extend({},this.getTask(id));
 
 			// Gantt chart deals with dates as Date objects, format as server likes
@@ -671,16 +864,23 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 			
 			var value = gantt_widget.getInstanceManager().getValues(gantt_widget.getInstanceManager().widgetContainer);
 
-			if(gantt_widget.options.ajax_update)
+			var set = true;
+			if(gantt_widget.options.onchange)
+			{
+				e.data = {task: task, mode: mode, value: value};
+				set = gantt_widget.change(e, gantt_widget);
+			}
+			if(gantt_widget.options.ajax_update && set)
 			{
 				var request = gantt_widget.egw().json(gantt_widget.options.ajax_update,
-					[task,value]
+					[task, mode, value]
 				).sendRequest(true);
 			}
 		});
 
 		// Update server for links
 		var link_update = function(id, link) {
+			if(gantt_widget.options.readonly) return false;
 			if(gantt_widget.options.ajax_update)
 			{
 				link.parent = this.getTask(link.source).parent;
@@ -764,6 +964,7 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 	_bindChildren: function() {
 		var gantt_widget = this;
 		this.iterateOver(function(_widget){
+			if(_widget.instanceOf(et2_gantt)) return;
 			// Existing change function
 			var widget_change = _widget.change;
 
@@ -803,6 +1004,91 @@ var et2_gantt = et2_valueWidget.extend([et2_IResizeable,et2_IInput],
 		}, this, et2_inputWidget);
 	},
 
+	/**
+	 * Start UI for selecting among defined columns
+	 */
+	_column_selection: function(e)
+	{
+		var self = this;
+		var columns = [];
+		var columns_selected = [];
+		for (var i = 0; i < this.gantt_config.columns.length; i++)
+		{
+			var col = this.gantt_config.columns[i];
+			columns.push({
+				value: col.name,
+				label: col.label
+			})
+			if(!col.hide)
+			{
+				columns_selected.push(col.name);
+			}
+		}
+
+		// Build the popup
+		if(!this.selectPopup)
+		{
+			var select = et2_createWidget("select", {
+				multiple: true,
+				rows: 8,
+				empty_label:this.egw().lang("select columns"),
+				selected_first: false
+			}, this);
+			select.set_select_options(columns);
+			select.set_value(columns_selected);
+
+			var okButton = et2_createWidget("buttononly", {}, this);
+			okButton.set_label(this.egw().lang("ok"));
+			okButton.onclick = function() {
+				// Update columns
+				var value = select.getValue() || [];
+				for (var i = 0; i < columns.length; i++)
+				{
+					self.gantt_config.columns[i].hide = value.indexOf(columns[i].value) < 0 ;
+				}
+				self.set_columns(self.gantt_config.columns);
+
+				// Update Implicit preference
+				this.egw().set_preference(self.getInstanceManager().app, 'gantt_columns_' + self.id, value);
+
+				// Hide popup
+				self.selectPopup.toggle();
+				self.selectPopup.remove();
+				self.selectPopup = null;
+				$j('body').off('click.gantt');
+			};
+
+			var cancelButton = et2_createWidget("buttononly", {}, this);
+			cancelButton.set_label(this.egw().lang("cancel"));
+			cancelButton.onclick = function() {
+				self.selectPopup.toggle();
+				self.selectPopup.remove();
+				self.selectPopup = null;
+				$j('body').off('click.gantt');
+			};
+
+			// Create and add popup
+			this.selectPopup = jQuery(document.createElement("div"))
+				.addClass("colselection ui-dialog ui-widget-content")
+				.append(select.getDOMNode())
+				.append(okButton.getDOMNode())
+				.append(cancelButton.getDOMNode())
+				.appendTo(this.getInstanceManager().DOMContainer);
+			// Bind so if you click elsewhere, it closes
+			window.setTimeout(function() {$j(document).one('mouseup.gantt', function(e){
+				if(!self.selectPopup.is(e.target) && self.selectPopup.has(e.target).length === 0)
+				{
+					cancelButton.onclick();
+				}
+			});},1);
+		}
+		else
+		{
+			this.selectPopup.toggle();
+		}
+		this.selectPopup.position({my:'right top', at:'right bottom', of: e.target});
+	},
+
 	/**
 	 * Link the actions to the DOM nodes / widget bits.
 	 * Overridden to make the gantt chart a container, so it can't be selected.
@@ -950,6 +1236,31 @@ $j(function() {
 		}
 	};
 
+	gantt.templates.leftside_text = function(start, end, task) {
+		var text = '';
+		if(task.planned_start)
+		{
+			if(typeof task.planned_start == 'string') task.planned_start = gantt.date.parseDate(task.planned_start, "xml_date");
+			var p_start = gantt.posFromDate(task.planned_start) - gantt.posFromDate(start);
+			text = "<div class='gantt_task_line gantt_task_planned' style='width:"+Math.abs(p_start)+"px; right:"+(p_start > 0 ? -p_start : 0)+"px;'><span>"
+				+ gantt.date.date_to_str(gantt.config.api_date)(task.planned_start)
+				+ "</span></div>";
+		}
+		return text;
+	};
+	gantt.templates.rightside_text = function(start, end, task) {
+		var text = '';
+		if(task.planned_end)
+		{
+			if(typeof task.planned_end == 'string') task.planned_end = gantt.date.parseDate(task.planned_end, "xml_date");
+			var p_end = gantt.posFromDate(task.planned_end) - gantt.posFromDate(end);
+			text = "<div class='gantt_task_line gantt_task_planned' style='left:"+(p_end > 0 ? 0 : p_end) +"px; width:"+Math.abs(p_end)+"px'><span>"
+				+ gantt.date.date_to_str(gantt.config.api_date)(task.planned_end)
+				+ "</span></div>";
+		}
+		return text;
+	};
+
 	// Link styling
 	gantt.templates.link_class = function(link) {
 		var link_class = '';
