@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel
  * @copyright Stylite 2011
- * @version $Id: et2_widget_grid.js 48204 2014-08-22 12:32:49Z ralfbecker $
+ * @version $Id$
  */
 
 "use strict";
@@ -26,7 +26,7 @@
  *
  * @augments et2_DOMWidget
  */
-var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
+var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned, et2_IResizeable],
 {
 	createNamespace: true,
 
@@ -62,10 +62,14 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 	 */
 	init: function() {
 		// Create the table body and the table
-		this.tbody = $j(document.createElement("tbody"));
 		this.table = $j(document.createElement("table"))
 			.addClass("et2_grid");
-		this.table.append(this.tbody);
+		this.thead = $j(document.createElement("thead"))
+			.appendTo(this.table);
+		this.tfoot = $j(document.createElement("tfoot"))
+			.appendTo(this.table);
+		this.tbody = $j(document.createElement("tbody"))
+			.appendTo(this.table);
 
 		// Call the parent constructor
 		this._super.apply(this, arguments);
@@ -180,7 +184,7 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 				colDataEntry["class"] = et2_readAttrWithDefault(node, "class", "");
 				colDataEntry["align"] = et2_readAttrWithDefault(node, "align", "");
 				colDataEntry["span"] = et2_readAttrWithDefault(node, "span", "1");
-				
+
 				// Keep any others attributes set, there's no 'column' widget
 				for(var i in node.attributes)
 				{
@@ -212,6 +216,7 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 				rowDataEntry["class"] = et2_readAttrWithDefault(node, "class", "");
 				rowDataEntry["valign"] = et2_readAttrWithDefault(node, "valign", "");
 				rowDataEntry["span"] = et2_readAttrWithDefault(node, "span", "1");
+				rowDataEntry["part"] = et2_readAttrWithDefault(node, "part", "body");
 
 				var id = et2_readAttrWithDefault(node, "id", "");
 				if(id)
@@ -283,7 +288,7 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 								var ident = content.expandName(value);
 								// expandName() handles index into content (@), but we have to look up
 								// regular values
-								if(value.indexOf('@') < 0)
+								if(value[0] != '@')
 								{
 									// Returns null if there isn't an actual value
 									ident = content.getEntry(ident,false,true);
@@ -293,7 +298,7 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 									rowData[rowIndex] = jQuery.extend({}, rowDataEntry);
 									content.perspectiveData.row = ++rowIndex;
 									ident = content.expandName(value);
-									if(value.indexOf('@') < 0)
+									if(value[0] != '@')
 									{
 										// Returns null if there isn't an actual value
 										ident = content.getEntry(ident,false,true);
@@ -645,7 +650,23 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 		for (var y = 0; y < h; y++)
 		{
 			var row = _cells[y];
-			var tr = $j(document.createElement("tr")).appendTo(this.tbody)
+			var parent = this.tbody;
+			switch(this.rowData[y]["part"])
+			{
+				case 'header':
+					if (!this.tbody.children().length && !this.tfoot.children().length)
+					{
+						parent = this.thead;
+					}
+					break;
+				case 'footer':
+					if (!this.tbody.children().length)
+					{
+						parent = this.tfoot;
+					}
+					break;
+			}
+			var tr = $j(document.createElement("tr")).appendTo(parent)
 				.addClass(this.rowData[y]["class"]);
 
 			if (this.rowData[y].disabled)
@@ -838,6 +859,8 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 			}
 		}
 		this.managementArray = [];
+		this.thead.empty();
+		this.tfoot.empty();
 		this.tbody.empty();
 
 		// Update array managers
@@ -886,10 +909,11 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 			items: "tr:not(.th)",
 			distance: 15,
 			stop: function(event, ui) {
-				self.egw().jsonq(sortable,[self.tbody.sortable("toArray"), self.id],
+				self.egw(window).json(sortable,[self.tbody.sortable("toArray"), self.id],
 					null,
-					self
-				);
+					self,
+					true
+				).sendRequest();
 			}
 		});
 	},
@@ -909,7 +933,7 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 			// objects
 			widget_object = objectManager.insertObject(false, new egwActionObject(
 				this.id, objectManager, new et2_action_object_impl(this),
-				objectManager.manager.getActionById(this.id) || objectManager.manager
+				this._actionManager || objectManager.manager.getActionById(this.id) || objectManager.manager
 			));
 		}
 
@@ -981,6 +1005,69 @@ var et2_grid = et2_DOMWidget.extend([et2_IDetachedDOM, et2_IAligned],
 			}
 		}
 		return ids.join('_');
+	},
+
+	resize: function (_height)
+	{
+		if (typeof this.options != 'undefined' && _height
+				&& typeof this.options.resize_ratio != 'undefined' && this.options.resize_ratio)
+		{
+			// apply the ratio
+			_height = (this.options.resize_ratio != '')? _height * this.options.resize_ratio: _height;
+			if (_height != 0)
+			{
+				if (this.wrapper)
+				{
+					this.wrapper.height(this.wrapper.height() + _height);
+				}
+				else
+				{
+					this.table.height(this.table.height() + _height );
+				}
+			}
+
+		}
+	},
+
+	/**
+	 * Get a dummy row object containing all widget of a row
+	 *
+	 * This is only a temp. solution until rows are implemented as eT2 containers and
+	 * _sender.getParent() will return a real row container.
+	 *
+	 * @param {et2_widget} _sender
+	 * @returns {Array|undefined}
+	 */
+	getRow: function(_sender)
+	{
+		if (!_sender || !this.cells) return;
+
+		for(var r=0; r < this.cells.length; ++r)
+		{
+			var row = this.cells[r];
+			for(var c=0; c < row.length; ++c)
+			{
+				if (!row[c].widget) continue;
+
+				var found = row[c].widget === _sender;
+				if (!found) row[c].widget.iterateOver(function(_widget) {if (_widget === _sender) found = true;});
+				if (found)
+				{
+					// return a fake row object allowing to iterate over it's children
+					var row_obj = new et2_widget(this, {});
+					for(var c=0; c < row.length; ++c)
+					{
+						if (row[c].widget) row_obj._children.push(row[c].widget);
+					}
+					row_obj.isInTree = jQuery.proxy(this.isInTree, this);
+					// we must not free the children!
+					row_obj.destroy = function(){
+						delete row_obj._children;
+					};
+					return row_obj;
+				}
+			}
+		}
 	}
 });
 et2_register_widget(et2_grid, ["grid"]);
\ No newline at end of file
