@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Nathan Gray
  * @copyright 2011 Nathan Gray
- * @version $Id: et2_widget_link.js 51320 2015-01-20 17:37:04Z ralfbecker $
+ * @version $Id$
  */
 
 "use strict";
@@ -89,17 +89,23 @@ var et2_link_to = et2_inputWidget.extend(
 		this.link_entry = null;
 		this.file_upload = null;
 
-		this.createInputWidget();
+		if (!this.options.readonly) this.createInputWidget();
 	},
 
 	destroy: function() {
 		this.link_button = null;
 		this.status_span = null;
 
-		this.link_entry.destroy();
-		this.link_entry = null;
-		this.file_upload.destroy();
-		this.file_upload = null;
+		if(this.link_entry)
+		{
+			this.link_entry.destroy();
+			this.link_entry = null;
+		}
+		if(this.file_upload)
+		{
+			this.file_upload.destroy();
+			this.file_upload = null;
+		}
 		this.div = null;
 
 		this._super.apply(this, arguments);
@@ -522,7 +528,7 @@ var et2_link_entry = et2_inputWidget.extend(
 			"name": "Query callback",
 			"type": "js",
 			"default": et2_no_init,
-			"description": "Callback before query to server.  Must return true, or false to abort query."
+			"description": "Callback before query to server.  It will be passed the request & et2_link_entry objects.  Must return true, or false to abort query."
 		},
 		"select": {
 			"name": "Select callback",
@@ -582,7 +588,6 @@ var et2_link_entry = et2_inputWidget.extend(
 
 		// Application selection
 		this.app_select = $j(document.createElement("select")).appendTo(this.div)
-			.val(this.options.value.app||'')
 			.change(function(e) {
 				// Clear cache when app changes
 				self.cache = {};
@@ -607,6 +612,11 @@ var et2_link_entry = et2_inputWidget.extend(
 			this.app_select.hide();
 			this.div.addClass("no_app");
 		}
+		else
+		{
+			// Now that options are in, set to last used app
+			this.app_select.val(this.options.value.app||'');
+		}
 
 		// Search input
 		this.search = $j(document.createElement("input"))
@@ -685,7 +695,7 @@ var et2_link_entry = et2_inputWidget.extend(
 				self.search.autocomplete("option","minLength", 0);
 				self.search.autocomplete("search");
 				self.search.autocomplete("option","minLength", self.minimum_characters);
-				e.stopPropagation();
+				return false;
 			}
 		});
 
@@ -742,7 +752,7 @@ var et2_link_entry = et2_inputWidget.extend(
 		else
 		{
 			_attrs["select_options"] = this.egw().link_app_list('query');
-			delete _attrs["select_options"]["addressbook-email"];
+			if (typeof _attrs["select_options"]["addressbook-email"] != 'undefined') delete _attrs["select_options"]["addressbook-email"];
 		}
 
 		// Check whether the options entry was found, if not read it from the
@@ -811,7 +821,7 @@ var et2_link_entry = et2_inputWidget.extend(
 			this.clear.hide();
 			this.options.value = _value = {'id':null};
 		}
-		if(!_value.app) _value.app = this.options.only_app;
+		if(!_value.app) _value.app = this.options.only_app || this.app_select.val();
 
 		if(_value.id) {
 			// Remove specific display and revert to CSS file
@@ -917,7 +927,7 @@ var et2_link_entry = et2_inputWidget.extend(
 		// Allow hook / tie in
 		if(this.options.query && typeof this.options.query == 'function')
 		{
-			if(!this.options.query(request, response)) return false;
+			if(!this.options.query(request, this)) return false;
 		}
 
 		if((typeof request.no_cache == 'undefined' && !request.no_cache) && request.term in this.cache) {
@@ -1143,8 +1153,9 @@ var	et2_link = et2_valueWidget.extend([et2_IDetachedDOM],
 		if(_value.id && _value.app)
 		{
 			this.link.addClass("et2_link");
-			this.link.click( function(){
+			this.link.click( function(e){
 				self.egw().open(_value, "", "view",null,_value.app,_value.app);
+				e.stopImmediatePropagation();
 			});
 		}
 		else
@@ -1232,7 +1243,7 @@ et2_register_widget(et2_link, ["link", "link-entry_ro"]);
  *
  * @augments et2_valueWidget
  */
-var et2_link_string = et2_valueWidget.extend([et2_IDetachedDOM],
+var et2_link_string = expose(et2_valueWidget.extend([et2_IDetachedDOM],
 {
 	attributes: {
 		"application": {
@@ -1256,7 +1267,13 @@ var et2_link_string = et2_valueWidget.extend([et2_IDetachedDOM],
 			"type": "string",
 			"default":"",
 			"description": "Sub-type key to list only entries of that type"
-		}
+		},
+		"expose_view":{
+			name: "Expose view",
+			type: "boolean",
+			default: true,
+			description: "Clicking on description with href value would popup an expose view, and will show content referenced by href."
+		},
 	},
 
 	/**
@@ -1334,13 +1351,54 @@ var et2_link_string = et2_valueWidget.extend([et2_IDetachedDOM],
 		this.egw().jsonq(this.egw().getAppName()+'.etemplate_widget_link.ajax_link_list', [_value], this.set_value, this);
 		return;
 	},
-
+	/**
+	 * Function to get media content to feed the expose
+	 * @param {type} _value
+	 * @returns {Array|Array.getMedia.mediaContent}
+	 */
+	getMedia: function (_value)
+	{
+		var base_url = egw.webserverUrl.match(/^\//,'ig')?egw(window).window.location.origin + egw.webserverUrl : egw.webserverUrl;
+		var mediaContent = [];
+		if (_value && typeof _value.type !='undefined' && _value.type.match(/video\//,'ig'))
+		{
+			mediaContent = [{
+					title: _value.id,
+					type: _value.type,
+					poster:'', // TODO: Should be changed by correct video thumbnail later
+					href: base_url + egw().mime_open(_value),
+					download_href: base_url + egw().mime_open(_value) + '?download',
+				}];
+		}
+		else if(_value)
+		{
+			mediaContent = [{
+				title: _value.id,
+				href: base_url + egw().mime_open(_value).url,
+				download_href: base_url + egw().mime_open(_value).url + '?download',
+				type: _value.type,
+			}];
+		}
+		if (mediaContent[0].href && mediaContent[0].href.match(/\/webdav.php/,'ig')) mediaContent[0]["download_href"] = mediaContent[0].href + '?download';
+		return mediaContent;
+	},
 	_add_link: function(_link_data) {
 		var self = this;
 		var link = $j(document.createElement("li"))
 			.appendTo(this.list)
 			.addClass("et2_link loading")
-			.click( function(){self.egw().open(_link_data, "", "view",null,_link_data.app,_link_data.app);});
+			.click( function(e){
+				if (self.options.expose_view && typeof _link_data.type !='undefined' 
+					&& _link_data.type.match(self.mime_regexp,'ig'))
+				{
+					self._init_blueimp_gallery(e, _link_data);
+				}
+				else
+				{
+					self.egw().open(_link_data, "", "view",null,_link_data.app,_link_data.app);
+				}
+				e.stopImmediatePropagation();
+			});
 
 		if(_link_data.title) link.text(_link_data.title);
 
@@ -1415,7 +1473,7 @@ var et2_link_string = et2_valueWidget.extend([et2_IDetachedDOM],
 			this._labelContainer.contents().not(this.list).remove();
 		}
 	}
-});
+}));
 et2_register_widget(et2_link_string, ["link-string"]);
 
 /**
@@ -1431,6 +1489,18 @@ var et2_link_list = et2_link_string.extend(
 			"type": "boolean",
 			"default": false,
 			"description": "Show links that are marked as deleted, being held for purge"
+		},
+		"onchange": {
+			"name": "onchange",
+			"type": "js",
+			"default": et2_no_init,
+			"description": "JS code which is executed when the links change."
+		},
+		readonly: {
+			name: "readonly",
+			type: "boolean",
+			"default": false,
+			description: "Does NOT allow user to enter data, just displays existing data"
 		}
 	},
 
@@ -1444,7 +1514,7 @@ var et2_link_list = et2_link_string.extend(
 
 		this.list = $j(document.createElement("table"))
 			.addClass("et2_link_list");
-		if(this.options['class']) this.node.addClass(this.options['class']);
+		if(this.options['class']) this.list.addClass(this.options['class']);
 		this.setDOMNode(this.list[0]);
 
 		// Set up context menu
@@ -1503,7 +1573,8 @@ var et2_link_list = et2_link_string.extend(
 			var link_data = self.context.data;
 			if(link_data.app == 'file')
 			{
-				var url = self.egw().mime_open(link_data);
+				// File info is always the same
+				var url = '/apps/'+link_data.app2+'/'+link_data.id2+'/'+decodeURIComponent(link_data.id);
 				if(typeof url == 'string' && url.indexOf('webdav.php'))
 				{
 					// URL is url to file in webdav, so get rid of that part
@@ -1522,9 +1593,29 @@ var et2_link_list = et2_link_string.extend(
 			// Download file
 			if(link_data.download_url)
 			{
-				window.location = self.egw().link(link_data.download_url,"download");
-				return;
+				var url = link_data.download_url;
+				if (url[0] == '/') url = egw.link(url);
+
+				var a = document.createElement('a');
+				if(typeof a.download == "undefined")
+				{
+					window.location = url+"?download";
+					return false;
+				}
+				
+				// Multiple file download for those that support it
+				a = $j(a)
+					.prop('href', url)
+					.prop('download', link_data.title || "")
+					.appendTo(self.getInstanceManager().DOMContainer);
+
+				var evt = document.createEvent('MouseEvent');
+				evt.initMouseEvent('click', true, true, window, 1, 0, 0, 0, 0, false, false, false, false, 0, null);
+				a[0].dispatchEvent(evt);
+				a.remove();
+				return false;
 			}
+			
 			self.egw().open(link_data, "", "view",'download',link_data.target ? link_data.target : link_data.app,link_data.app);
 		});
 		this.context.addItem("zip", this.egw().lang("Save as Zip"), this.egw().image('save_zip'), function(menu_item) {
@@ -1544,7 +1635,7 @@ var et2_link_list = et2_link_string.extend(
 			var link_id = isNaN(self.context.data.link_id) ? self.context.data : self.context.data.link_id;
 			var row = jQuery('#link_'+(self.context.data.dom_id ? self.context.data.dom_id : self.context.data.link_id), self.list);
 			et2_dialog.show_dialog(
-				function(button) { debugger; if(button == et2_dialog.YES_BUTTON) self._delete_link(link_id,row);},
+				function(button) { if(button == et2_dialog.YES_BUTTON) self._delete_link(link_id,row);},
 				egw.lang('Delete link?')
 			);
 		});
@@ -1569,42 +1660,54 @@ var et2_link_list = et2_link_string.extend(
 	{
 		this.list.empty();
 		// Handle server passed a list of links that aren't ready yet
-		if(_value && typeof _value == "object" && _value.to_id && typeof _value.to_id == "object")
+		if(_value && typeof _value == "object")
 		{
-			for(var id in _value.to_id)
+			var list = [];
+			if(_value.to_id && typeof _value.to_id == "object")
 			{
-				var link = _value.to_id[id];
-				if(link.app)
+				list = _value.to_id;
+			}
+			else if (_value.length)
+			{
+				list = _value;
+			}
+			if(list.length > 0)
+			{
+				for(var id in list)
 				{
-					// Temp IDs can cause problems since the ID includes the file name or :
-					if(link.link_id && typeof link.link_id != 'number')
+					var link = list[id];
+					if(link.app)
 					{
-						link.dom_id = 'temp_'+egw.uid();
-					}
-					// Icon should be in registry
-					if(typeof link.icon == 'undefined')
-					{
-						link.icon = egw.link_get_registry(link.app,'icon');
-						// No icon, try by mime type - different place for un-saved entries
-						if(link.icon == false && link.id.type)
+						// Temp IDs can cause problems since the ID includes the file name or :
+						if(link.link_id && typeof link.link_id != 'number')
 						{
-							// Triggers icon by mime type, not thumbnail or app
-							link.type = link.id.type;
-							link.icon = true;
+							link.dom_id = 'temp_'+egw.uid();
 						}
+						// Icon should be in registry
+						if(!link.icon)
+						{
+							link.icon = egw.link_get_registry(link.app,'icon');
+							// No icon, try by mime type - different place for un-saved entries
+							if(link.icon == false && link.id.type)
+							{
+								// Triggers icon by mime type, not thumbnail or app
+								link.type = link.id.type;
+								link.icon = true;
+							}
+						}
+						// Special handling for file - if not existing, we can't ask for title
+						if(typeof link.id =='object' && !link.title)
+						{
+							link.title = link.id.name || '';
+						}
+						this._add_link(link);
 					}
-					// Special handling for file - if not existing, we can't ask for title
-					if(link.app == 'file' && typeof link.title == 'undefined')
-					{
-						link.title = link.id.name || '';
-					}
-					this._add_link(link);
 				}
 			}
-		}
-		else
-		{
-			this._super.apply(this,arguments);
+			else
+			{
+				this._super.apply(this,arguments);
+			}
 		}
 	},
 
@@ -1613,6 +1716,13 @@ var et2_link_list = et2_link_string.extend(
 			.attr("id", "link_"+(_link_data.dom_id ? _link_data.dom_id : (typeof _link_data.link_id == "string" ? _link_data.link_id.replace(/[:\.]/g,'_'):_link_data.link_id ||_link_data.id)))
 			.attr("draggable", _link_data.app == 'file' ? "true" : "")
 			.appendTo(this.list);
+		if(!_link_data.link_id)
+		{
+			for(var k in _link_data)
+			{
+				row[0].dataset[k] = _link_data[k];
+			}
+		}
 
 		// Icon
 		var icon = $j(document.createElement("td"))
@@ -1622,28 +1732,52 @@ var et2_link_list = et2_link_string.extend(
 		{
 			var icon_widget = et2_createWidget("image");
 			var src = '';
+			// Creat a mime widget if the link has type
 			if(_link_data.type)
 			{
 				// VFS - file
-				src = this.egw().mime_icon(_link_data.type, _link_data.icon);
+				var vfs_widget = et2_createWidget('vfs-mime');
+				vfs_widget.set_value({
+					download_url:_link_data.download_url,
+					name:_link_data.title,
+					mime:_link_data.type,
+					path:_link_data.icon
+				});
+				icon.append(vfs_widget.getDOMNode());
 			}
 			else
 			{
 				src = this.egw().image(_link_data.icon);
+				if(src)	icon_widget.set_src(src);
+				icon.append(icon_widget.getDOMNode());
 			}
-			if(src)	icon_widget.set_src(src);
-			icon.append(icon_widget.getDOMNode());
 		}
 
 		var columns = ['title','remark'];
 
 		var self = this;
 		for(var i = 0; i < columns.length; i++) {
-			$j(document.createElement("td"))
+			var $td = $j(document.createElement("td"))
 				.appendTo(row)
 				.addClass(columns[i])
-				.click( function(){self.egw().open(_link_data, "", "view",null,_link_data.target ? _link_data.target : _link_data.app,_link_data.app);})
 				.text(_link_data[columns[i]] ? _link_data[columns[i]]+"" : "");
+			
+			//Bind the click handler if there is download_url	
+			if (_link_data && (typeof _link_data.download_url != 'undefined' || _link_data.app !='egw-data'))
+			{
+				$td.click( function(){
+					// Check if the link entry is mime with media type, in order to open it in expose view
+					if (typeof _link_data.type != 'undefined' && _link_data.type.match(self.mime_regexp,'ig'))
+					{
+						var $vfs_img_node = jQuery(this).parent().find('.vfsMimeIcon');
+						if ($vfs_img_node.length > 0) $vfs_img_node.click();
+					}
+					else
+					{
+						self.egw().open(_link_data, "", "view",null,_link_data.target ? _link_data.target : _link_data.app,_link_data.app);
+					}
+				});	
+			}
 		}
 
 		if (typeof _link_data.title == 'undefined')
@@ -1667,35 +1801,42 @@ var et2_link_list = et2_link_string.extend(
 		*/
 
 		// Delete
-		var delete_button = $j(document.createElement("td"))
-			.appendTo(row);
-		$j("<div />")
-			.appendTo(delete_button)
-			// We don't use ui-icon because it assigns a bg image
-			.addClass("delete icon")
-			.bind( 'click', function() {
-				et2_dialog.show_dialog(
-					function(button) {
-						if(button == et2_dialog.YES_BUTTON)
-						{
-							self._delete_link(
-								self.value && typeof self.value.to_id != 'object' && _link_data.link_id ? _link_data.link_id:_link_data,
-								row
-							);
-						}
-					},
-					egw.lang('Delete link?')
-				);
-			});
-
+		// build delete button if the link is not readonly
+		if (!this.options.readonly)
+		{
+			var delete_button = $j(document.createElement("td"))
+				.appendTo(row);
+			$j("<div />")
+				.appendTo(delete_button)
+				// We don't use ui-icon because it assigns a bg image
+				.addClass("delete icon")
+				.bind( 'click', function() {
+					et2_dialog.show_dialog(
+						function(button) {
+							if(button == et2_dialog.YES_BUTTON)
+							{
+								self._delete_link(
+									self.value && typeof self.value.to_id != 'object' && _link_data.link_id ? _link_data.link_id:_link_data,
+									row
+								);
+							}
+						},
+						egw.lang('Delete link?')
+					);
+				});
+		}
 		// Context menu
 		row.bind("contextmenu", function(e) {
+			// Comment only available if link_id is there and not readonly
+			self.context.getItem("comment").set_enabled(typeof _link_data.link_id != 'undefined' && !self.options.readonly);
 			// File info only available for existing files
 			self.context.getItem("file_info").set_enabled(typeof _link_data.id != 'object' && _link_data.app == 'file');
 			self.context.getItem("save").set_enabled(typeof _link_data.id != 'object' && _link_data.app == 'file');
 			// Zip download only offered if there are at least 2 files
 			self.context.getItem("zip").set_enabled($j('[id^="link_-"]',this.list).length >= 2);
-
+			// Show delete item only if the widget is not readonly
+			self.context.getItem("delete").set_enabled(!self.options.readonly);
+			
 			self.context.data = _link_data;
 			self.context.showAt(e.pageX, e.pageY, true);
 			e.preventDefault();
@@ -1767,6 +1908,10 @@ var et2_link_list = et2_link_string.extend(
 			delete_button.removeClass("delete").addClass("loading");
 			row.off();
 		}
+		if(this.onchange)
+		{
+			this.onchange(this,link_id,row);
+		}
 		if(typeof link_id != "object")
 		{
 			egw.json(this.egw().getAppName()+".etemplate_widget_link.ajax_delete.etemplate", [link_id],
