@@ -8,7 +8,7 @@
  * @author Nathan Gray
  * @author Andreas Stöckel
  * @copyright Nathan Gray 2011
- * @version $Id: et2_widget_selectbox.js 49762 2014-12-08 20:02:38Z nathangray $
+ * @version $Id$
  */
 
 "use strict";
@@ -114,7 +114,7 @@ var et2_selectbox = et2_inputWidget.extend(
 		{
 			if(isNaN(this.options.rows))
 			{
-				this.options.empty_label = this.options.rows;
+				this.options.empty_label = this.egw().lang(this.options.rows);
 				this.options.rows = 1;
 			}
 			else
@@ -170,7 +170,7 @@ var et2_selectbox = et2_inputWidget.extend(
 			return;
 		}
 
-		var sel_options = et2_selectbox.find_select_options(this, _attrs['select_options']);
+		var sel_options = et2_selectbox.find_select_options(this, _attrs['select_options'], _attrs);
 		if(!jQuery.isEmptyObject(sel_options))
 		{
 			_attrs['select_options'] = sel_options;
@@ -295,13 +295,15 @@ var et2_selectbox = et2_inputWidget.extend(
 			if(option_data.icon)
 			{
 				var img = this.egw().image(option_data.icon);
-				jQuery(document.createElement("img"))
+				jQuery(document.createElement(img ? "img" : "div"))
 					.attr("src", img)
+					.addClass('cat_icon cat_' + _value)
 					.appendTo(label);
 			}
 			if(option_data.color)
 			{
-				label.css("background-color",option_data.color);
+				label.css("background-color",option_data.color)
+					.addClass('cat_' + _value);
 			}
 		}
 		label.append(jQuery("<span>"+_label+"</span>"));
@@ -436,7 +438,7 @@ var et2_selectbox = et2_inputWidget.extend(
 	set_value: function(_value)
 	{
 		if (typeof _value == "number") _value = ""+_value;	// convert to string for consitent matching
-		if(typeof _value == "string" && this.options.multiple && _value.match(this._is_multiple_regexp) !== null)
+		if(typeof _value == "string" && (this.options.multiple || this.options.expand_multiple_rows) && _value.match(this._is_multiple_regexp) !== null)
 		{
 			_value = _value.split(',');
 		}
@@ -447,6 +449,7 @@ var et2_selectbox = et2_inputWidget.extend(
 			// No options set yet
 			this.set_select_options(this.options.select_options);
 		}
+		this._oldValue = this.value;
 		if(this.input !== null && (this.options.tags || this.options.search))
 		{
 			// Value must be a real Array, not an object
@@ -459,11 +462,12 @@ var et2_selectbox = et2_inputWidget.extend(
 		{
 			return this.set_multi_value(_value);
 		}
-		if(typeof _value != 'string' && jQuery(this.value).not(_value).length == 0 && jQuery(_value).not(this.value).length == 0)
+		// Auto-expand multiple if not yet turned on, and value has multiple
+		if(this.options.expand_multiple_rows && !this.options.multiple && jQuery.isArray(_value) && _value.length > 1)
 		{
-			// Unchanged
-			if(_value == this.value) return;
+			this.set_multiple(true, this.options.expand_multiple_rows);
 		}
+
 		jQuery("option",this.input).prop("selected", false);
 		if(typeof _value == "array")
 		{
@@ -483,7 +487,9 @@ var et2_selectbox = et2_inputWidget.extend(
 		{
 			if(_value && jQuery("option[value='"+_value+"']", this.input).prop("selected", true).length == 0)
 			{
-				if(this.options.select_options[_value])
+				if(this.options.select_options[_value] ||
+					this.options.select_options.filter && 
+					this.options.select_options.filter(function(value) {return value == _value;}))
 				{
 					// Options not set yet? Do that now, which will try again.
 					return this.set_select_options(this.options.select_options);
@@ -494,13 +500,18 @@ var et2_selectbox = et2_inputWidget.extend(
 				}
 				else
 				{
-					this.egw().debug("warn", "Tried to set value '%s' that isn't an option", _value, this);
+					var debug_value = _value;
+					if(debug_value === null) debug_value == 'NULL';
+					this.egw().debug("warn", "Tried to set value '%s' that isn't an option", debug_value, this);
 				}
 				return;
 			}
 		}
-
 		this.value = _value;
+		if(this.isAttached() && this._oldValue !== et2_no_init && this._oldValue !== _value)
+		{
+			this.input.change();
+		}
 	},
 
 	set_multi_value: function(_value) {
@@ -523,7 +534,9 @@ var et2_selectbox = et2_inputWidget.extend(
 		{
 			if(jQuery("input[value='"+_value+"']", this.multiOptions).prop("checked", true).length == 0)
 			{
-				this.egw().debug("warn", "Tried to set value that isn't an option", this, _value);
+				var debug_value = _value;
+				if(debug_value === null) debug_value == 'NULL';
+				this.egw().debug("warn", "Tried to set value '%s' that isn't an option", debug_value, this);
 			}
 		}
 
@@ -545,7 +558,7 @@ var et2_selectbox = et2_inputWidget.extend(
 		this.options.expand_multiple_rows = _rows;
 
 		var surroundings = this.getSurroundings();
-		if(_rows <= 1 )
+		if(_rows <= 1 && this.expand_button )
 		{
 			// Remove
 			surroundings.removeDOMNode(this.expand_button.get(0));
@@ -649,7 +662,7 @@ var et2_selectbox = et2_inputWidget.extend(
 			// Translate the options
 			if(!this.options.no_lang)
 			{
-				if (typeof _options[key] === 'object')
+				if (typeof _options[key] === 'object' && _options[key] !== null)
 				{
 					if(_options[key]["label"]) _options[key]["label"] = this.egw().lang(_options[key]["label"]);
 					if(_options[key]["title"]) _options[key]["title"] = this.egw().lang(_options[key]["title"]);
@@ -660,7 +673,7 @@ var et2_selectbox = et2_inputWidget.extend(
 				}
 			}
 
-			if (typeof _options[key] === 'object' && _options[key] != null)
+			if (typeof _options[key] === 'object' && _options[key] !== null)
 			{
 				// Optgroup
 				if(typeof _options[key]["label"] == 'undefined' && typeof _options[key]["title"] == "undefined")
@@ -677,7 +690,7 @@ var et2_selectbox = et2_inputWidget.extend(
 
 					for(var sub in _options[key])
 					{
-						if (typeof _options[key][sub] === 'object')
+						if (typeof _options[key][sub] === 'object' && _options[key][sub] !== null)
 						{
 							this._appendOptionElement(sub,
 								_options[key][sub]["label"] ? _options[key][sub]["label"] : "",
@@ -721,11 +734,12 @@ var et2_selectbox = et2_inputWidget.extend(
 			var value = [];
 			jQuery("input:checked",this.multiOptions).each(function(){value.push(this.value);});
 			// we need to return null for no value instead of empty array, which gets overwritten by preserved value on server-side
-			this.value = value.length > 0 ? value : null;
+			this.value = value;
 		}
 		else
 		{
 			this.value = this._super.apply(this, arguments);
+			if (this.value === null) this.value = [];	// do NOT return null, as it does not get transmitted to server
 		}
 		return this.value;
 	},
@@ -744,7 +758,7 @@ var et2_selectbox = et2_inputWidget.extend(
 	}
 });
 et2_register_widget(et2_selectbox, ["menupopup", "listbox", "select", "select-cat",
-	"select-percent", 'select-priority', 'select-access',
+	"select-percent", 'select-priority',
 	'select-country', 'select-state', 'select-year', 'select-month',
 	'select-day', 'select-dow', 'select-hour', 'select-number', 'select-app',
 	'select-lang', 'select-bool', 'select-timezone' ]);
@@ -752,19 +766,40 @@ et2_register_widget(et2_selectbox, ["menupopup", "listbox", "select", "select-ca
 // Static class stuff
 jQuery.extend(et2_selectbox,
 {
+	type_cache: {},
+
 	/**
 	 * Find the select options for a widget, out of the many places they could be.
 	 * @param {et2_widget} widget to check for.  Should be some sort of select widget.
 	 * @param {object} attr_options Select options in attributes array
+	 * @param {object} attrs Widget attributes
 	 * @return {object} Select options, or empty object
 	 */
-	find_select_options: function(widget, attr_options)
+	find_select_options: function(widget, attr_options, attrs)
 	{
 		var name_parts = widget.id.replace(/&#x5B;/g,'[').replace(/]|&#x5D;/g,'').split('[');
 
+		var type_options = [];
 		var content_options = {};
 
-		// Try to find the options inside the "sel-options" array
+		// First check type, there may be static options.  There's some special handling
+		// for filterheaders, which have the wrong type.
+		var type = widget.instanceOf(et2_nextmatch_filterheader) ? attrs.widget_type || '' : widget._type;
+		var type_function = type.replace('select-','').replace('_ro','')+'_options';
+		if(typeof this[type_function] == 'function')
+		{
+			var old_type = widget._type;
+			widget._type = type;
+			if(typeof attrs.other == 'string')
+			{
+				attrs.other = attrs.other.split(',');
+			}
+			// Copy, to avoid accidental modification
+			jQuery.extend(true, type_options, this[type_function].call(this, widget, attrs));
+			widget._type = old_type;
+		}
+
+		// Try to find the options inside the "sel-options"
 		if(widget.getArrayMgr("sel_options"))
 		{
 			// Try first according to ID
@@ -815,20 +850,17 @@ jQuery.extend(et2_selectbox,
 
 			// Maybe in a row, and options got stuck in ${row} instead of top level
 			// not sure this code is still needed, as server-side no longer creates ${row} or {$row} for select-options
-			if(!content_options || content_options.length == 0)
+			var row_stuck = ['${row}','{$row}'];
+			for(var i = 0; i < row_stuck.length && (!content_options || content_options.length == 0); i++)
 			{
-				var row_stuck = ['${row}','{$row}'];
-				for(var i = 0; i < row_stuck.length; i++)
+				// perspectiveData.row in nm, data["${row}"] in an auto-repeat grid
+				if(widget.getArrayMgr("sel_options").perspectiveData.row || widget.getArrayMgr("sel_options").data[row_stuck[i]])
 				{
-					// perspectiveData.row in nm, data["${row}"] in an auto-repeat grid
-					if(widget.getArrayMgr("sel_options").perspectiveData.row || widget.getArrayMgr("sel_options").data[row_stuck[i]])
+					var row_id = widget.id.replace(/[0-9]+/,row_stuck[i]);
+					content_options = widget.getArrayMgr("sel_options").getEntry(row_id);
+					if(!content_options || content_options.length == 0)
 					{
-						var row_id = widget.id.replace(/[0-9]+/,row_stuck[i]);
-						content_options = widget.getArrayMgr("sel_options").getEntry(row_id);
-						if(!content_options || content_options.length == 0)
-						{
-							content_options = widget.getArrayMgr("sel_options").getEntry(row_stuck[i] + '[' + widget.id + ']');
-						}
+						content_options = widget.getArrayMgr("sel_options").getEntry(row_stuck[i] + '[' + widget.id + ']');
 					}
 				}
 			}
@@ -858,7 +890,272 @@ jQuery.extend(et2_selectbox,
 		{
 			content_options = {};
 		}
+		
+		// Include type options, preferring any content options
+		if(type_options.length || !jQuery.isEmptyObject(type_options))
+		{
+			for(var i in content_options)
+			{
+				var value = typeof content_options[i] == 'object' && typeof content_options[i].value !== 'undefined' ? content_options[i].value : i;
+				var added = false;
+
+				// Override any existing
+				for(var j in type_options)
+				{
+					if(type_options[j].value === value)
+					{
+						added = true;
+						type_options[j] = content_options[i];
+						break;
+					}
+				}
+				if(!added)
+				{
+					type_options.splice(i,0,content_options[i]);
+				}
+			}
+			content_options = type_options;
+		}
 		return content_options;
+	},
+	
+	/**
+	 * Some static options, no need to transfer them over and over.
+	 * We still need the same thing on the server side to validate, so they
+	 * have to match.  See etemplate_widget_menupopup::typeOptions()
+	 * The type specific legacy options wind up in attrs.other.
+	 */
+	priority_options: function(widget) {
+		return [
+			{value: 1, label: 'low'},
+			{value: 2, label: 'normal'},
+			{value: 3, label: 'high'},
+			{value: 0, label: 'undefined'}
+		];
+	},
+	bool_options: function(widget) {
+		return [
+			{value: 0, label: 'no'},
+			{value: 1, label: 'yes'}
+		];
+	},
+	month_options: function(widget) {
+		return [
+			{value: 1, label:'January'},
+			{value: 2, label:'February'},
+			{value: 3, label:'March'},
+			{value: 4, label:'April'},
+			{value: 5, label:'May'},
+			{value: 6, label:'June'},
+			{value: 7, label:'July'},
+			{value: 8, label:'August'},
+			{value: 9, label:'September'},
+			{value: 10, label:'October'},
+			{value: 11, label:'November'},
+			{value: 12, label:'December'}
+		];
+	},
+	number_options: function(widget, attrs) {
+		if(typeof attrs.other != 'object')
+		{
+			attrs.other = [];
+		}
+		var options = [];
+		var min = typeof(attrs.other[0]) == 'undefined' ? 1 : parseInt(attrs.other[0]);
+		var max = typeof(attrs.other[1]) == 'undefined' ? 10: parseInt(attrs.other[1]);
+		var interval = typeof(attrs.other[2]) == 'undefined' ? 1: parseInt(attrs.other[2]);
+		var format = '%d';
+
+		// leading zero specified in interval
+		if (attrs.other[2] && attrs.other[2][0] == '0')
+		{
+			format = '%0'+(''+interval).length+'d';
+		}
+		// Suffix
+		if(attrs.other[3])
+		{
+			format += widget.egw().lang(attrs.other[3]);
+		}
+
+		// Avoid infinite loop if interval is the wrong direction
+		if ((min <= max) != (interval > 0))
+		{
+			interval = -interval;
+		}
+
+		for (var i=0, n=min; n <= max && i <= 100; n += interval,++i)
+		{
+			options.push({value: n, label: sprintf(format,n)});
+		}
+		return options;
+	},
+	percent_options: function(widget, attrs)
+	{
+		if(typeof attrs.other != 'object')
+		{
+			attrs.other = [];
+		}
+		attrs.other[0] = 0;
+		attrs.other[1] = 100;
+		attrs.other[2] = typeof(attrs.other[2]) == 'undefined' ? 10 : parseInt(attrs.other[2]);
+		attrs.other[3] = '%%';
+		return this.number_options(widget,attrs);
+	},
+	year_options: function(widget, attrs)
+	{
+		if(typeof attrs.other != 'object')
+		{
+			attrs.other = [];
+		}
+		var t = new Date();
+		attrs.other[0] = t.getFullYear() - (typeof(attrs.other[0]) == 'undefined' ? 3 : parseInt(attrs.other[0]));
+		attrs.other[1] = t.getFullYear() + (typeof(attrs.other[1]) == 'undefined' ? 2 : parseInt(attrs.other[1]));
+		attrs.other[2] = typeof(attrs.other[2]) == 'undefined' ? 1 : parseInt(attrs.other[2]);
+		return this.number_options(widget,attrs);
+	},
+	day_options: function(widget, attrs)
+	{
+		attrs.other = [1,31,1];
+		return this.number_options(widget,attrs);
+	},
+	hour_options: function(widget, attrs)
+	{
+		var options = [];
+		var timeformat = egw.preference('common','timeformat');
+		for (var h = 0; h <= 23; ++h)
+		{
+			options.push({
+				value: h,
+				label: timeformat == 12 ?
+					(( 12 ? h % 12 : 12)+' '+(h < 12 ? egw.lang('am') : egw.lang('pm'))) :
+					sprintf('%02d',h)
+			});
+		}
+		return options;
+	},
+	app_options: function(widget,attrs) {
+		var options = ','+(attrs.other||[]).join(',');
+		return this.cached_server_side_options(widget, options, attrs);
+	},
+	cat_options: function(widget, attrs) {
+		// Add in application, if not there
+		if(typeof attrs.other == 'undefined')
+		{
+			attrs.other = new Array(4);
+		}
+		if(typeof attrs.other[3] == 'undefined')
+		{
+			attrs.other[3] = attrs.application || widget.getInstanceManager().app;
+		}
+		var options =(attrs.other||[]).join(',');
+		return this.cached_server_side_options(widget, options, attrs);
+	},
+	country_options: function(widget, attrs) {
+		var options = ',';
+		return this.cached_server_side_options(widget, options, attrs);
+	},
+	dow_options: function(widget,attrs) {
+		var options = ','+(attrs.other||[]).join(',');
+		return this.cached_server_side_options(widget, options, attrs);
+	},
+	lang_options: function(widget,attrs) {
+		var options = ','+(attrs.other||[]).join(',');
+		return this.cached_server_side_options(widget, options, attrs);
+	},
+	timezone_options: function(widget,attrs) {
+		var options = ','+(attrs.other||[]).join(',');
+		return this.cached_server_side_options(widget, options, attrs);
+	},
+	/**
+	 * Some options change, or are too complicated to have twice, so we get the
+	 * options from the server once, then keep them to use if they're needed again.
+	 * We use the options string to keep the different possibilites (eg. categories
+	 * for different apps) seperate.
+	 * 
+	 * @param {et2_selectbox} widget Selectbox we're looking at
+	 * @param {string} options_string
+	 * @param {Object} attrs Widget attributes (not yet fully set)
+	 * @returns {Object} Array of options, or empty and they'll get filled in later
+	 */
+	cached_server_side_options: function(widget, options_string, attrs)
+	{
+		// normalize options by removing trailing commas
+		options_string = options_string.replace(/,+$/, '');
+		
+		var cache_id = widget._type+'_'+options_string;
+		var cache = egw.window.et2_selectbox.type_cache[cache_id];
+
+		// Options for a selectbox in a nextmatch must be returned now, as the
+		// widget we have is not enough to set the options later.
+		var in_nextmatch = false;
+		if(typeof cache === 'undefined' || typeof cache.length === 'undefined')
+		{
+			var parent = widget._parent;
+			while(parent && !in_nextmatch)
+			{
+				in_nextmatch =(parent && parent._type && parent._type === 'nextmatch')
+				parent = parent._parent;
+			}
+		}
+		if (typeof cache == 'undefined' || in_nextmatch)
+		{
+			// Fetch with json instead of jsonq because there may be more than
+			// one widget listening for the response by the time it gets back,
+			// and we can't do that when it's queued.
+			var req = egw.json(
+				widget.getInstanceManager().app+'.etemplate_widget_menupopup.ajax_get_options.etemplate',
+				[widget._type,options_string,attrs.value]
+			).sendRequest(!in_nextmatch);
+			if(typeof cache === 'undefined')
+			{
+				egw.window.et2_selectbox.type_cache[cache_id] = req;
+			}
+			cache = req;
+		}
+		if(typeof cache.done == 'function')
+		{
+			// pending, wait for it
+			cache.done(jQuery.proxy(function(response) {
+				egw.window.et2_selectbox.type_cache[this.cache_id] = response.response[0].data||undefined;
+				// Set select_options in attributes in case we get a resonse before
+				// the widget is finished loading (otherwise it will re-set to {})
+				attrs.select_options = egw.window.et2_selectbox.type_cache[this.cache_id];
+
+				egw.window.setTimeout(jQuery.proxy(function() {
+					// Avoid errors if widget is destroyed before the timeout
+					if(this.widget && typeof this.widget.id !== 'undefined')
+					{
+						this.widget.set_select_options(et2_selectbox.find_select_options(this.widget,{}, this.widget.options));
+					}
+				},this),1);
+			},{widget:widget,cache_id:cache_id}));
+			return [];
+		}
+		else
+		{
+			// Check that the value is in there
+			// Make sure we are not requesting server for an empty value option or 
+			// other widgets but select-timezone as server won't find anything and
+			// it will fall into an infinitive loop, e.g. select-cat widget.
+			if(attrs.value && attrs.value != "" && attrs.value != "0" && attrs.type == "select-timezone")
+			{
+				var missing_option = true;
+				for(var i = 0; i < cache.length && missing_option; i++)
+				{
+					if(cache[i].value == attrs.value)
+					{
+						missing_option = false;
+					}
+				}
+				// Try again - ask the server with the current value this time
+				if(missing_option)
+				{
+					delete egw.window.et2_selectbox.type_cache[cache_id];
+					return this.cached_server_side_options(widget, options_string, attrs);
+				}
+			}
+			return cache;
+		}
 	}
 });
 
@@ -939,6 +1236,11 @@ var et2_selectbox_ro = et2_selectbox.extend([et2_IDetachedDOM],
 		{
 			_value = _value.match(this._is_multiple_regexp) !== null ? _value.split(',') : [_value];
 		}
+		// need to handle numerical values too
+		else if (typeof _value == 'number')
+		{
+			_value = [_value];
+		}
 		this.span.empty();
 
 		if(_value)
@@ -955,6 +1257,7 @@ var et2_selectbox_ro = et2_selectbox.extend([et2_IDetachedDOM],
 					{
 						this.span.text(label);
 						if (typeof option == 'object' && option.title) this.set_statustext(option.title);
+						break;
 					}
 					else
 					{
@@ -962,10 +1265,15 @@ var et2_selectbox_ro = et2_selectbox.extend([et2_IDetachedDOM],
 							.text(label)
 							.attr('data-value', _value[i])
 							.appendTo(this.span);
+						break;
 					}
 				}
 			}
 		}
+		else if (this.options.empty_label)
+		{
+			this.span.text(this.options.empty_label);
+		}
 	},
 
 	/**
