@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Nathan Gray
  * @copyright Nathan Gray 2013
- * @version $Id: et2_widget_taglist.js 51090 2014-12-19 09:28:09Z hnategh $
+ * @version $Id$
  */
 
 "use strict";
@@ -39,7 +39,7 @@ var et2_taglist = et2_selectbox.extend(
 		"select_options": {
 			"type": "any",
 			"name": "Select options",
-			"default": null, //[{id: "a", label: "Alpha"},{id:"b", label: "Beta"}],
+			"default": {}, //[{id: "a", label: "Alpha"},{id:"b", label: "Beta"}],
 			"description": "Internaly used to hold the select options."
 		},
 
@@ -97,7 +97,7 @@ var et2_taglist = et2_selectbox.extend(
 			"description": "The maximum number of items the user can select if multiple selection is allowed."
 		},
 		// Selectbox attributes that are not applicable
-		"multiple": { ignore: true},
+		"multiple": { default: true},
 		"rows": { ignore: true},
 		"tags": { ignore: true},
 		useCommaKey: {
@@ -183,12 +183,14 @@ var et2_taglist = et2_selectbox.extend(
 		// Display / hide a loading icon while fetching
 		$j(this.taglist)
 			.on("beforeload", function() {this.container.prepend('<div class="ui-icon loading"/>');})
-			.on("load", function() {$j('.loading',this.container).remove();});
+			.on("load", function() {$j('.loading',this.container).remove();})
+		// Keep focus when selecting from the list
+			.on("selectionchange", function() { $j('input',this.container).focus();});
 
 		// Unbind change handler of widget's ancestor to stop it from bubbling
 		// taglist has its own onchange
 		$j(this.getDOMNode()).unbind('change.et2_inputWidget');
-		
+
 		// onChange
 		if(this.options.onchange && typeof this.options.onchange == 'function')
 		{
@@ -309,44 +311,74 @@ var et2_taglist = et2_selectbox.extend(
 	 */
 	set_value: function(value)
 	{
-		this.options.value = value;
-		if(this.taglist == null) return;
+		if (value === '' || value === null)
+		{
+			value = [];
+		}
+		else if (typeof value === 'string' && this.options.multiple)
+		{
+			value = value.split(',');
+		}
 
-		this.taglist.clear(true);
-		if(!value) return;
+		var values = jQuery.isArray(value) ? jQuery.extend([],value) : [value];
 
-		var values = jQuery.isArray(value) ? value : [value];
+		if(!value && this.taglist)
+		{
+			this.taglist.clear(true);
+			return;
+		}
+
+		var result = [];
 		for(var i=0; i < values.length; ++i)
 		{
 			var v = values[i];
 			if (v && typeof v == 'object' && typeof v.id != 'undefined' && typeof v.label != 'undefined')
 			{
-				// alread in correct format
+				// already in correct format
 			}
-			else if (this.options.select_options && typeof this.options.select_options[v] == 'undefined' || typeof v == 'string')
+			else if (this.options.select_options &&
+				// Check options
+				(result = jQuery.grep(this.options.select_options, function(e) {
+					return e.id == v;
+				})) && result.length
+			)
 			{
 				// Options should have been provided, but they weren't
 				// This can happen for ajax source with an existing value
-				if(this.options.select_options == null)
+				if(this.options.select_options === null)
 				{
-					this.options.select_options = {};
+					this.options.select_options = [];
 				}
-				values[i] = {
+				values[i] = result[0] ? result[0] : {
 					id: v,
 					label: v
 				};
 			}
-			else(this.options.select_options)
+			else if (this.taglist &&
+				// Check current selection to avoid going back to server
+				(result = jQuery.grep(this.taglist.getSelection(), function(e) {
+					return e.id == v;
+				})) && result.length)
 			{
-				if (typeof values[i].id == 'undefined')
+				values[i] = result[0] ? result[0] : {
+					id: v,
+					label: v
+				};
+			}
+			else
+			{
+				if (typeof values[i].id === 'undefined')
 				{
 					values[i] = {
 						id: v,
-						label: this.options.select_options[v]
+						label: v
 					};
 				}
 			}
 		}
+		this.options.value = values;
+
+		if(this.taglist === null) return;
 
 		this.taglist.addToSelection(values);
 	},
@@ -556,10 +588,13 @@ var et2_taglist_email = et2_taglist.extend(
 						var taglist = this.taglist;
 						window.setTimeout(function()
 						{
+							var items = [];
 							for(var i=0; i < parts.length; ++i)
 							{
-								taglist.addToSelection({id: parts[i], label: parts[i]});
+								items.push({id: parts[i], label: parts[i]});
+
 							}
+							taglist.addToSelection(items);
 						}, 10);
 					}
 				}
@@ -569,6 +604,7 @@ var et2_taglist_email = et2_taglist.extend(
 		var label = jQuery('<span>').text(item.label);
 		if (item.class) label.addClass(item.class);
 		if (typeof item.title != 'undefined') label.attr('title', item.title);
+		if (typeof item.data != 'undefined') label.attr('data', item.data);
 		if (!valid) label.addClass('ui-state-error');
 
 		return label;
@@ -606,4 +642,4 @@ et2_register_widget(et2_taglist_ro, ["taglist_ro","taglist_email_ro", "taglist_a
 
 // Require css
 // TODO: merge into etemplate2.css with all other widgets when done
-if(typeof egw != 'undefined') egw(window).includeCSS(egw.webserverUrl + "/phpgwapi/js/jquery/magicsuggest/magicsuggest.css");
+if(typeof egw == 'function') egw(window).includeCSS(egw.webserverUrl + "/phpgwapi/js/jquery/magicsuggest/magicsuggest.css");
