@@ -9,7 +9,7 @@
  * @author Nathan Gray
  * @author Ralf Becker
  * @copyright Nathan Gray 2011
- * @version $Id: et2_widget_tree.js 48091 2014-08-15 11:25:04Z ralfbecker $
+ * @version $Id$
  */
 
 "use strict";
@@ -101,6 +101,18 @@ var et2_tree = et2_inputWidget.extend(
 			"type": "string",
 			"default": "",
 			"description": "comma-separated names of icons for a leaf, closed and opend folder (default: leaf.gif,folderClosed.gif,folderOpen.gif), images with extension get loaded from image_path, just 'image' or 'appname/image' are allowed too"
+		},
+		"multimarking": {
+			"name": "multimarking",
+			"type": "any",
+			"default": false,
+			"description": "Allow marking multiple nodes, default is false which means disabled multiselection, true or 'strict' activates it and 'strict' makes it strick to only same level marking"
+		},
+		highlighting:{
+			"name": "highlighting",
+			"type": "boolean",
+			"default": false,
+			"description": "Add highlighting class on hovered over item, highlighting is disabled by default"
 		}
 	},
 
@@ -203,6 +215,12 @@ var et2_tree = et2_inputWidget.extend(
 		if (widget.options.autoloading)
 		{
 			var url = widget.options.autoloading;
+			
+			//Set escaping mode to utf8, as url in 
+			//autoloading needs to be utf8 encoded.
+			//For instance item id with umlaut.
+			widget.input.setEscapingMode('utf8');
+			
 			if (url.charAt(0) != '/' && url.substr(0,4) != 'http')
 			{
 				url = '/json.php?menuaction='+url;
@@ -212,6 +230,13 @@ var et2_tree = et2_inputWidget.extend(
 			widget.input.setXMLAutoLoading(egw.link(url));
 			widget.input.setDataMode('JSON');
 		}
+
+		if (widget.options.multimarking)
+		{
+			widget.input.enableMultiselection(!!widget.options.multimarking, widget.options.multimarking === 'strict');
+		}
+		// Enable/Disable highlighting
+		widget.input.enableHighlighting(widget.options.highlighting?true:false);
 	},
 
 	/**
@@ -374,7 +399,10 @@ var et2_tree = et2_inputWidget.extend(
 			for(var i = 0; i < this.value.length; i++)
 			{
 				this.input.setCheck(this.value[i], true);
-				this.input.openItem(this.value[i]);
+				// autoloading openning needs to be absolutely based on user interaction
+				// or open flag in folder structure, therefore, We should 
+				// not force it to open the node
+				if (!this.options.autoloading) this.input.openItem(this.value[i]);
 			}
 		}
 		else
@@ -439,7 +467,30 @@ var et2_tree = et2_inputWidget.extend(
 	 */
 	getValue: function() {
 		if(this.input == null) return null;
-		return this.options.multiple ? this.input.getAllChecked().split(this.input.dlmtr) : this.input.getSelectedItemId();
+		if (this.options.multiple)
+		{
+			var allChecked = this.input.getAllChecked().split(this.input.dlmtr);
+			var allUnchecked = this.input.getAllUnchecked().split(this.input.dlmtr);
+			if (this.options.autoloading)
+			{
+				
+				var res = {};
+				for (var i=0;i<allChecked.length;i++)
+				{
+					res[allChecked[i]]= {value:true};
+				}
+				for (var i=0;i<allUnchecked.length;i++)
+				{
+					res[allUnchecked[i]]= {value:false};
+				}
+				return res;
+			}
+			else
+			{
+				return allChecked;
+			}
+		}
+		return this.input.getSelectedItemId();
 	},
 
 	/**
