@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Nathan Gray
  * @copyright Nathan Gray 2012
- * @version $Id: et2_widget_vfs.js 47388 2014-06-25 16:26:59Z hnategh $
+ * @version $Id$
 */
 
 "use strict";
@@ -18,6 +18,7 @@
 	et2_core_valueWidget;
 	et2_widget_description;
 	et2_widget_file;
+	/etemplate/js/expose.js;
 */
 
 /**
@@ -61,7 +62,11 @@ var et2_vfs = et2_valueWidget.extend([et2_IDetachedDOM],
 	set_value: function(_value) {
 		if (typeof _value !== 'object')
 		{
-			this.egw().debug("warn", "%s only has path, needs full array", this.id, _value);
+			// Only warn if it's an actual value, just blank for falsy values
+			if(_value)
+			{
+				this.egw().debug("warn", "%s only has path, needs full array", this.id, _value);
+			}
 			this.span.empty().text(_value);
 			return;
 		}
@@ -194,7 +199,7 @@ var et2_vfsName = et2_textbox.extend(
 		this._super.apply(this,[_value]);
 	},
 	getValue: function() {
-		return egw.encodePath(this._super.apply(this));
+		return egw.encodePath(this._super.apply(this)||'');
 	}
 });
 et2_register_widget(et2_vfsName, ["vfs-name"]);
@@ -245,7 +250,7 @@ et2_register_widget(et2_vfsName_ro, ["vfs-name_ro"]);
  *
  * @augments et2_valueWidget
  */
-var et2_vfsMime = et2_valueWidget.extend([et2_IDetachedDOM],
+var et2_vfsMime = expose(et2_valueWidget.extend([et2_IDetachedDOM],
 {
 	attributes: {
 		"value": {
@@ -257,7 +262,26 @@ var et2_vfsMime = et2_valueWidget.extend([et2_IDetachedDOM],
 			"type": "integer",
 			"description": "Size of icon / thumbnail, in pixels",
 			"default": et2_no_init
+		},
+		"expose_callback":{
+			"name": "expose_callback",
+			"type": "js",
+			"default": et2_no_init,
+			"description": "JS code which is executed when expose slides."
+		},
+		expose_view: {
+			name: "Expose view",
+			type: "boolean",
+			default: true,
+			description: "Clicking on an image would popup an expose view"
+		},
+		thumb_mime_size:{
+			name: "Image thumbnail size",
+			type: "string",
+			default:"",
+			description:" Size of thumbnail in pixel for specified mime type with syntax of: mime_type(s),size (eg. image,video,128)"
 		}
+
 	},
 
 	legacyOptions:["size"],
@@ -275,6 +299,65 @@ var et2_vfsMime = et2_valueWidget.extend([et2_IDetachedDOM],
 		this.iconOverlayContainer.append(this.image);
 		this.setDOMNode(this.iconOverlayContainer[0]);
 	},
+
+	/**
+	 * Handler for expose slide action, from expose
+	 * Returns data needed for the given index, or false to let expose handle it
+	 *
+	 * @param {Gallery} gallery
+	 * @param {integer} index
+	 * @param {DOMNode} slide
+	 * @return {Array} array of objects consist of media contnet
+	 */
+	expose_onslide: function(gallery, index, slide)
+	{
+		var content = false;
+		if (this.options.expose_callback && typeof this.options.expose_callback == 'function')
+		{
+			//Call the callback to load more items
+			content = this.options.expose_callback.call(this,[gallery, index]);
+			if (content) this.add(content);
+		}
+		return content;
+	},
+
+	/**
+	 * Function to get media content to feed the expose
+	 *
+	 * @param {type} _value
+	 * @returns {Array} return an array of object consists of media content
+	 */
+	getMedia: function (_value)
+	{
+		var base_url = egw.webserverUrl.match(/^\//,'ig')?egw(window).window.location.origin + egw.webserverUrl:egw.webserverUrl;
+		var mediaContent = [{
+			title: _value.name,
+			type: _value.mime,
+			href: _value.download_url
+		}];
+		// check if download_url is not already an url (some stream-wrappers allow to specify that!)
+		if (_value.download_url && (_value.download_url[0] == '/' || _value.download_url.substr(0, 4) != 'http'))
+		{
+			mediaContent[0].href = base_url + _value.download_url;
+
+			if (mediaContent[0].href && mediaContent[0].href.match(/\/webdav.php/,'ig'))
+			{
+				mediaContent[0].download_href = mediaContent[0].href + '?download';
+			}
+		}
+		if (_value && _value.mime && _value.mime.match(/video\//,'ig'))
+		{
+			mediaContent[0].thumbnail = this.egw().mime_icon(_value.mime, _value.path, undefined, _value.mtime);
+		}
+		else
+		{
+			mediaContent[0].thumbnail = _value.path && _value.mime ?
+				this.egw().mime_icon(_value.mime, _value.path, undefined, _value.mtime) :
+				this.image.attr('src')+ '&thheight=128';
+		}
+		return mediaContent;
+	},
+
 	set_value: function(_value) {
 		if (typeof _value !== 'object')
 		{
@@ -291,6 +374,16 @@ var et2_vfsMime = et2_valueWidget.extend([et2_IDetachedDOM],
 				{
 					src += "&thsize="+this.options.size;
 				}
+				else if (this.options.thumb_mime_size)
+				{
+					var mime_size = this.options.thumb_mime_size.split(',');
+					var mime_regex = RegExp(_value.mime.split('/')[0]);
+					if (typeof mime_size != 'undefined' && jQuery.isArray(mime_size)
+							&& !isNaN(mime_size[mime_size.length-1]) && isNaN(mime_size[0]) && this.options.thumb_mime_size.match(mime_regex[0], 'ig'))
+					{
+						src += "&thsize=" + mime_size[mime_size.length-1];
+					}
+				}
 				this.image.css("max-width", "100%");
 			}
 			this.image.attr("src", src);
@@ -323,12 +416,13 @@ var et2_vfsMime = et2_valueWidget.extend([et2_IDetachedDOM],
 		_attrs.push("value", "class");
 	},
 	getDetachedNodes: function() {
-		return [this.iconOverlayContainer[0]];
+		return [this.node, this.iconOverlayContainer[0], this.image[0]];
 	},
 
 	setDetachedAttributes: function(_nodes, _values) {
-		this.iconOverlayContainer = jQuery(_nodes[0]);
-		this.image = jQuery(_nodes[0].children[0]);
+		this.iconOverlayContainer = jQuery(_nodes[1]);
+		this.image = jQuery(_nodes[2]);
+		this.node = _nodes[0];
 		this.overlayContainer = _nodes[0].children[1];
 		if(typeof _values['class'] != "undefined") {
 			this.image.addClass(_values['class']);
@@ -337,7 +431,7 @@ var et2_vfsMime = et2_valueWidget.extend([et2_IDetachedDOM],
 			this.set_value(_values['value']);
 		}
 	}
-});
+}));
 et2_register_widget(et2_vfsMime, ["vfs-mime"]);
 
 /**
@@ -619,7 +713,7 @@ var et2_vfsSelect = et2_inputWidget.extend(
 		"value": {
 			"type": "any", // Object
 			"description": "Array of paths (strings)"
-		}, 
+		},
 		"button_caption":{
 			name: "button caption",
 			type: "string",
@@ -642,12 +736,12 @@ var et2_vfsSelect = et2_inputWidget.extend(
 
 		// Allow no child widgets
 		this.supportedWidgetClasses = [];
-		
+
 		this.button = $j(document.createElement("button"))
 			.attr("title", this.egw().lang("Select file(s) from VFS"))
 			.addClass("et2_button et2_button_text et2_vfs_btn")
 			.css("background-image","url("+this.egw().image("filemanager/navbar")+")");
-			
+
 		if (this.options.button_caption != "")
 		{
 			this.button.text(this.options.button_caption);
@@ -679,27 +773,31 @@ var et2_vfsSelect = et2_inputWidget.extend(
 		};
 
 		// Open the filemanager select in a popup
-		var popup = this.egw().open_link(
+		var popup = this.egw(window).open_link(
 			this.egw().link('/index.php', attrs),
 			'link_existing',
-			'640x580'
+			'680x400'
 		);
 		if(popup)
 		{
+			// Safari and IE lose reference to global variables after window close
+			// Try to get updated data before window is closed then later we trigger
+			// change event on widget
+			self.egw().window.setTimeout(function(){
+				jQuery(popup).bind('unload',function(){
+					// Set selected files to widget
+					self.value = this.selected_files;
+
+					// Update path to where the user wound up]
+					if (typeof this.etemplate2 !='undefined') self.options.path = this.etemplate2.getByApplication('filemanager')[0].widgetContainer.getArrayMgr("content").getEntry('path');
+				});
+			},1000);
+
 			// Update on close doesn't always (ever, in chrome) work, so poll
 			var poll = self.egw().window.setInterval(
 				function() {
 					if(popup.closed) {
 						self.egw().window.clearInterval(poll);
-
-						// Update path to where the user wound up
-						var path = popup.etemplate2.getByApplication('filemanager')[0].widgetContainer.getArrayMgr("content").getEntry('path') || '';
-						self.options.path = path;
-
-						// Get the selected files
-						var files = popup.selected_files || [];
-						self.value = files;
-
 						// Fire a change event so any handlers run
 						$j(self.node).change();
 					}
@@ -733,17 +831,17 @@ var et2_vfsSelect = et2_inputWidget.extend(
 	{
 		this.options.button_label = label;
 	},
-	
+
 	/**
 	 * Set the caption for vfs-select button
-	 * 
+	 *
 	 * @param {string} caption string value as a caption
 	 */
 	set_button_caption: function (caption)
 	{
 		this.options.button_caption = caption;
 	},
-	
+
 	/**
 	 * Set the ID passed to the server side callback
 	 *
