@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel
  * @copyright Stylite 2011
- * @version $Id: etemplate2.js 49710 2014-12-05 10:28:01Z hnategh $
+ * @version $Id$
  */
 
 /*egw:uses
@@ -21,6 +21,7 @@
 	et2_widget_button;
 	et2_widget_color;
 	et2_widget_description;
+	et2_widget_entry;
 	et2_widget_textbox;
 	et2_widget_number;
 	et2_widget_url;
@@ -53,6 +54,7 @@
 	et2_widget_vfs;
 	et2_widget_video;
 	et2_widget_itempicker;
+	et2_widget_script;
 
 	et2_extension_nextmatch;
 	et2_extension_customfields;
@@ -99,20 +101,77 @@ function etemplate2(_container, _menuaction)
 }
 
 // List of templates (XML) that are known, not always used.  Indexed by id.
-etemplate2.prototype.templates = {};
-
+// We share list of templates with iframes and popups
+try {
+	if (opener && opener.etemplate2)
+	{
+		etemplate2.prototype.templates = opener.etemplate2.prototype.templates;
+	}
+	else if (top.etemplate2)
+	{
+		etemplate2.prototype.templates = top.etemplate2.prototype.templates;
+	}
+}
+catch (e) {
+	// catch security exception if opener is from a different domain
+	console.log('Security exception accessing etemplate2.prototype of opener or top!');
+}
+if (typeof etemplate2.prototype.templates == "undefined")
+{
+	etemplate2.prototype.templates = {};
+}
 
 /**
  * Calls the resize event of all widgets
+ *
+ * @param {jQuery.event} e
  */
-etemplate2.prototype.resize = function()
+etemplate2.prototype.resize = function(e)
 {
-	if (this.widgetContainer)
+	var event = e;
+	var self = this;
+	var excess_height = false;
+
+	// Check if the framework has an specific excess height calculation
+	if (typeof window.framework != 'undefined' && typeof window.framework.get_wExcessHeight != 'undefined')
+	{
+		excess_height = window.framework.get_wExcessHeight(window);
+	}
+
+	//@TODO implement getaccess height for other framework and remove
+	if (typeof event != 'undefined' && event.type == 'resize')
+	{
+		setTimeout(function(){
+			if (self.widgetContainer)
+			{
+				var appHeader = $j('#divAppboxHeader');
+
+				//Calculate the excess height
+				excess_height = egw(window).is_popup()? $j(window).height() - $j('.et2_container').height() - appHeader.outerHeight()+11: false;
+				// Recalculate excess height if the appheader is shown
+				if (appHeader.length > 0 && appHeader.is(':visible')) excess_height -= appHeader.outerHeight()-9;
+
+				// Do not resize if the template height is bigger than screen available height
+				// For templates which have sub templates and they are bigger than screenHeight
+				if(screen.availHeight < $j('.et2_container').height()) excess_height = 0;
+
+				// Call the "resize" event of all functions which implement the
+				// "IResizeable" interface
+				self.widgetContainer.iterateOver(function(_widget) {
+
+					if (excess_height != 0 || excess_height === false) _widget.resize(excess_height);
+				}, self, et2_IResizeable);
+			}
+		},100);
+	}
+	// Initial resize needs to be resized immediately (for instance for nextmatch resize)
+	else if(this.widgetContainer)
 	{
 		// Call the "resize" event of all functions which implement the
 		// "IResizeable" interface
 		this.widgetContainer.iterateOver(function(_widget) {
-			_widget.resize();
+
+			_widget.resize(excess_height);
 		}, this, et2_IResizeable);
 	}
 };
@@ -272,6 +331,18 @@ etemplate2.prototype.download = function(_url)
 etemplate2.prototype.load = function(_name, _url, _data, _callback)
 {
 	this.name = _name;	// store top-level template name to have it available in widgets
+	// store template base url, in case initial template is loaded via webdav, to use that for further loads too
+	// need to split off domain first, as it could contain app-name part of template eg. stylite.report.xet and https://my.stylite.de/egw/...
+	if (_url && _url[0] != '/')
+	{
+		this.template_base_url = _url.match(/https?:\/\/[^/]+/).shift();
+		_url = _url.split(this.template_base_url)[1];
+	}
+	else
+	{
+		this.template_base_url = '';
+	}
+	this.template_base_url += _url.split(_name.split('.').shift())[0];
 
 	egw().debug("info", "Loaded data", _data);
 	var currentapp = this.app = _data.currentapp || window.egw_appName;
@@ -326,6 +397,12 @@ etemplate2.prototype.load = function(_name, _url, _data, _callback)
 		{
 			egw.debug("warn", "Did not load '%s' JS object",appname);
 		}
+		// If etemplate current app does not match app owning the template,
+		// initialize the current app too
+		if (typeof app[this.app] !== 'object' && typeof app.classes[this.app] == 'function')
+		{
+			app[this.app] = new app.classes[this.app]();
+		}
 		if(typeof app[appname] == "object")
 		{
 			app_callback = function(_et2, _name) {
@@ -383,7 +460,7 @@ etemplate2.prototype.load = function(_name, _url, _data, _callback)
 			this.widgetContainer.loadingFinished(deferred);
 
 			// Connect to the window resize event
-			$j(window).on("resize."+this.uniqueId, this, function(e) {e.data.resize();});
+			$j(window).on("resize."+this.uniqueId, this, function(e) {e.data.resize(e);});
 
 			// Insert the document fragment to the DOM Container
 			this.DOMContainer.appendChild(frag);
@@ -406,22 +483,31 @@ etemplate2.prototype.load = function(_name, _url, _data, _callback)
 			jQuery.when.apply(jQuery, deferred).done(jQuery.proxy(function() {
 				egw.debug("log", "Finished loading %s, triggering load event", _name);
 
+				if (typeof window.framework != 'undefined' && typeof window.framework.et2_loadingFinished != 'undefined')
+				{
+					//Call loading finished method of the framework with local window
+					window.framework.et2_loadingFinished(egw(window).window);
+				}
 				// Trigger the "resize" event
 				this.resize();
 
 				// Automatically set focus to first visible input for popups
 				if(this.widgetContainer._egw.is_popup() && $j('[autofocus]',this.DOMContainer).focus().length == 0)
 				{
-					$j('input:visible',this.DOMContainer)
+					var $input = $j('input:visible',this.DOMContainer)
 						// Date fields open the calendar popup on focus
 						.not('.et2_date')
 						.filter(function() {
 						// Skip inputs that are out of tab ordering
 						$this = $j(this);
 						return !$this.attr('tabindex') || $this.attr('tabIndex')>=0;
-					}).first().focus();
-				};
-				
+					}).first();
+
+					// mobile device, focus only if the field is empty (usually means new entry)
+					// should focus always for non-mobile one
+					if (egwIsMobile() && $input.val() == "" || !egwIsMobile()) $input.focus();
+				}
+
 				// Tell others about it
 				if(typeof _callback == "function")
 				{
@@ -431,6 +517,12 @@ etemplate2.prototype.load = function(_name, _url, _data, _callback)
 				{
 					app_callback.call(window,this,_name);
 				}
+				if(appname != this.app && typeof window.app[this.app] == "object")
+				{
+					// Loaded a template from a different application?
+					// Let the application that loaded it know too
+					window.app[this.app].et2_ready(this, this.name);
+				}
 
 				$j(this.DOMContainer).trigger('load', this);
 
@@ -453,33 +545,45 @@ etemplate2.prototype.load = function(_name, _url, _data, _callback)
 
 
 		// Load & process
-		if(!this.templates[_name])
-		{
-			// Asynchronously load the XET file
-			et2_loadXMLFromURL(_url, function(_xmldoc) {
+		try {
+			if (this.templates[_name])
+			{
+				// Set array managers first, or errors will happen
+				this.widgetContainer.setArrayMgrs(this._createArrayManagers(_data));
 
-				// Scan for templates and store them
-				for(var i = 0; i < _xmldoc.childNodes.length; i++) {
-					var template = _xmldoc.childNodes[i];
-					if(template.nodeName.toLowerCase() != "template") continue;
-					this.templates[template.getAttribute("id")] = template;
-					if(!_name) this.name = template.getAttribute("id");
-				}
+				// Already have it
 				_load.apply(this,[]);
-			}, this);
-
-			// Split the given data into array manager objects and pass those to the
-			// widget container - do this here because file is loaded async
-			this.widgetContainer.setArrayMgrs(this._createArrayManagers(_data));
+				return;
+			}
 		}
-		else
-		{
-			// Set array managers first, or errors will happen
-			this.widgetContainer.setArrayMgrs(this._createArrayManagers(_data));
+		catch (e) {
+			// wired security exception in IE denying access to template cache in opener
+			if (e.message == 'Permission denied')
+			{
+				etemplate2.prototype.templates = {};
+			}
+			// other error eg. in app.js et2_ready or event handlers --> rethrow it
+			else
+			{
+				throw e;
+			}
+		}
+		// Asynchronously load the XET file
+		et2_loadXMLFromURL(_url, function(_xmldoc) {
 
-			// Already have it
+			// Scan for templates and store them
+			for(var i = 0; i < _xmldoc.childNodes.length; i++) {
+				var template = _xmldoc.childNodes[i];
+				if(template.nodeName.toLowerCase() != "template") continue;
+				this.templates[template.getAttribute("id")] = template;
+				if(!_name) this.name = template.getAttribute("id");
+			}
 			_load.apply(this,[]);
-		}
+		}, this);
+
+		// Split the given data into array manager objects and pass those to the
+		// widget container - do this here because file is loaded async
+		this.widgetContainer.setArrayMgrs(this._createArrayManagers(_data));
 	}, this);
 };
 
@@ -501,29 +605,68 @@ etemplate2.prototype.isDirty = function()
 	return dirty;
 };
 
+/**
+ * Submit the et2_container form to a blank iframe in order to activate browser autocomplete
+ */
+etemplate2.prototype.autocomplete_fixer = function ()
+{
+	var self = this;
+	var form = self.DOMContainer;
+
+	// Safari always do the autofill for password field regardless of autocomplete = off
+	// and since there's no other way to switch the autocomplete of, we should switch the
+	// form autocomplete off (e.g. compose dialog, attachment password field)
+	if (navigator.userAgent.match(/safari/i) && !navigator.userAgent.match(/chrome/i)
+			&& jQuery('input[type="password"]').length > 0)
+	{
+		return;
+	}
+
+	if (form)
+	{
+		// Stop submit propagation in order to not fire other possible submit events
+		// for instance, CKEditor has its own submit event handler which we do not want to
+		// fire that on submit
+		form.onsubmit = function(e){e.stopPropagation();};
+
+		// Firefox give a security warning when transmitting to "about:blank" from a https site
+		// we work around that by giving existing etemplate/empty.html url
+		// Safari shows same warning, thought Chrome userAgent also includes Safari
+		if (navigator.userAgent.match(/(firefox|safari|iceweasel)/i) && !navigator.userAgent.match(/chrome/i))
+		{
+			jQuery(form).attr({action: egw.webserverUrl+'/etemplate/empty.html',method:'post'});
+		}
+		// need to trigger submit because submit() would not trigger onsubmit event
+		// since the submit does not get fired directly via user interaction.
+		jQuery(form).trigger('submit');
+	}
+};
+
 /**
  * Submit form via ajax
  *
  * @param {(et2_button|string)} button button widget or string with id
  * @param {boolean} async true: do an asynchronious submit, default is synchronious
  * @param {boolean} no_validation - Do not do individual widget validation, just submit their current values
+ * @param {et2_widget|undefined} _container container to submit, default whole template
  * @return {boolean} true if submit was send, false if eg. validation stoped submit
  */
-etemplate2.prototype.submit = function(button, async, no_validation)
+etemplate2.prototype.submit = function(button, async, no_validation, _container)
 {
 	if(typeof no_validation == 'undefined')
 	{
 		no_validation = false;
 	}
+	var container = _container || this.widgetContainer;
 
 	// Get the form values
-	var values = this.getValues(this.widgetContainer);
+	var values = this.getValues(container);
 
 	// Trigger the submit event
 	var canSubmit = true;
 	if(!no_validation)
 	{
-		this.widgetContainer.iterateOver(function(_widget) {
+		container.iterateOver(function(_widget) {
 			if (_widget.submit(values) === false)
 			{
 				canSubmit = false;
@@ -581,6 +724,10 @@ etemplate2.prototype.submit = function(button, async, no_validation)
 		// Create the request object
 		if (this.menuaction)
 		{
+
+			//Autocomplete
+			this.autocomplete_fixer();
+
 			// unbind our session-destroy handler, as we are submitting
 			this.unbind_unload();
 
@@ -802,6 +949,37 @@ etemplate2.app_refresh = function(_msg, _app, _id, _type)
 	return refresh_done;
 };
 
+/**
+ * "Intelligently" print a given etemplate
+ *
+ * Mostly, we let the nextmatch change how many rows it's showing, so you don't
+ * get just one printed page.
+ *
+ * @return {Deferred[]} A list of Deferred objects that must complete before
+ *  actual printing can begin.
+ */
+etemplate2.prototype.print = function()
+{
+	// Sometimes changes take time
+	var deferred = [];
+
+	// Skip hidden etemplates
+	if(!jQuery(this.DOMContainer).filter(':visible').length) return [];
+
+	// Allow any widget to change for printing
+	this.widgetContainer.iterateOver(function(_widget) {
+		// Skip widgets from a different etemplate (home)
+		if(_widget.getInstanceManager() != this) return;
+		var result = _widget.beforePrint();
+		if (typeof result == "object" && result.done)
+		{
+			deferred.push(result);
+		}
+	},this,et2_IPrint);
+
+	return deferred;
+};
+
 // Some static things to make getting into widget context a little easier //
 
 /**
@@ -918,7 +1096,7 @@ function etemplate2_handle_load(_type, _response)
 		{
 			alert(data['window-close']);
 		}
-		window.close();
+		egw.close();
 		return true;
 	}
 
