@@ -6,16 +6,16 @@
  * @link http://www.egroupware.org
  * @package filemanager
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @copyright (c) 2007-10 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
+ * @copyright (c) 2007-16 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: cli.php 49162 2014-10-23 13:39:21Z ralfbecker $
+ * @version $Id$
  *
  * @todo --domain does NOT work with --user root_* for domains other then the first in header.inc.php
  */
 
 chdir(dirname(__FILE__));	// to enable our relative pathes to work
 
-error_reporting(error_reporting() & ~E_NOTICE);
+error_reporting(error_reporting() & ~E_NOTICE & ~E_DEPRECATED);
 
 if (php_sapi_name() !== 'cli')	// security precaution: forbit calling filemanager/cli.php as web-page
 {
@@ -34,8 +34,7 @@ function user_pass_from_argv(&$account)
 	//print_r($account);
 	if (!($sessionid = $GLOBALS['egw']->session->create($account)))
 	{
-		echo "Wrong user-account or -password !!!\n\n";
-		usage('',1);
+		usage("Wrong username or -password!");
 	}
 	return $sessionid;
 }
@@ -43,11 +42,14 @@ function user_pass_from_argv(&$account)
 /**
  * Give a usage message and exit
  *
- * @param string $action=null
- * @param int $ret=0 exit-code
+ * @param string $error_msg ='' error-message to be printed in front of usage
  */
-function usage($action=null,$ret=0)
+function usage($error_msg='')
 {
+	if ($error_msg)
+	{
+		echo "$error_msg\n\n";
+	}
 	$cmd = basename(__FILE__);
 	echo "Usage:\t$cmd ls [-r|--recursive|-l|--long|-i|--inode] URL [URL2 ...]\n";
 	echo "\t$cmd cat URL [URL2 ...]\n";
@@ -60,7 +62,7 @@ function usage($action=null,$ret=0)
 	echo "\t$cmd chmod [-r|--recursive] [ugoa]*[+-=][rwx]+,... URL [URL2 ...]\n";
 	echo "\t$cmd chown [-r|--recursive] user URL [URL2 ...]\n";
 	echo "\t$cmd chgrp [-r|--recursive] group URL [URL2 ...]\n";
-	echo "\t$cmd find URL [URL2 ...] [-type (d|f)][-depth][-mindepth n][-maxdepth n][-mime type[/sub]][-name pattern][-path pattern][-uid id][-user name][-nouser][-gid id][-group name][-nogroup][-size N][-cmin N][-ctime N][-mmin N][-mtime N] (N: +n --> >n, -n --> <n, n --> =n) [-limit N[,n]][-order (name|size|...)][-sort (ASC|DESC)]\n";
+	echo "\t$cmd find URL [URL2 ...] [-type (d|f)][-depth][-mindepth n][-maxdepth n][-mime type[/sub]][-name pattern][-path pattern][-uid id][-user name][-nouser][-gid id][-group name][-nogroup][-size N][-cmin N][-ctime N][-mmin N][-mtime N] (N: +n --> >n, -n --> <n, n --> =n) [-limit N[,n]][-order (name|size|...)][-sort (ASC|DESC)][-hidden][-show-deleted][-(name|name-preg|path|path-preg) S]\n";
 	echo "\t$cmd mount URL [path] (without path prints out the mounts)\n";
 	echo "\t$cmd umount [-a|--all (restores default mounts)] URL|path\n";
 	echo "\t$cmd eacl URL [rwx-] [user or group]\n";
@@ -75,21 +77,21 @@ function usage($action=null,$ret=0)
 	exit;
 }
 $long = $numeric = $recursive = $perms = $all = $inode = false;
-$argv = $_SERVER['argv'];
-$cmd = basename(array_shift($argv),'.php');
-if ($argv[0][0] != '-' && $argv[0][0] != '/' && strpos($argv[0],'://') === false)
+$args = $_SERVER['argv'];
+$cmd = basename(array_shift($args),'.php');
+if ($args[0][0] != '-' && $args[0][0] != '/' && strpos($args[0],'://') === false)
 {
-	$cmd = array_shift($argv);
+	$cmd = array_shift($args);
 }
 
-if (!$argv) $argv = array('-h');
+if (!$args) $args = array('-h');
 
-$args = $find_options = array();
-while(!is_null($option = array_shift($argv)))
+$argv = $find_options = array();
+while(!is_null($option = array_shift($args)))
 {
 	if ($option == '-' || $option[0] != '-')	// no option --> argument
 	{
-		$args[] = $option;
+		$argv[] = $option;
 		continue;
 	}
 
@@ -100,17 +102,18 @@ while(!is_null($option = array_shift($argv)))
 			{
 				if (!in_array($option,array('-type','-depth','-mindepth','-maxdepth','-name','-path',
 					'-uid','-user','-nouser','-gid','-group','-nogroup','-mime',
-					'-empty','-size','-cmin','-ctime','-mmin','-mtime','-limit','-order','-sort')))
+					'-empty','-size','-cmin','-ctime','-mmin','-mtime','-limit','-order','-sort',
+					'-hidden','-show-deleted','-name-preg','-path','-path-preg')))
 				{
-					usage();
+					usage("Unknown find option '$option'!");
 				}
-				if (in_array($option,array('-empty','-depth','-nouser','-nogroup')))
+				if (in_array($option,array('-empty','-depth','-nouser','-nogroup','-hidden','-show-deleted')))
 				{
 					$find_options[substr($option,1)] = true;
 				}
 				else
 				{
-					$find_options[substr($option,1)] = array_shift($argv);
+					$find_options[str_replace('-','_',substr($option,1))] = array_shift($args);
 				}
 				break;
 			}
@@ -119,7 +122,7 @@ while(!is_null($option = array_shift($argv)))
 			{
 				for($i = 1; $i < strlen($option); ++$i)
 				{
-					array_unshift($argv,'-'.$option[$i]);
+					array_unshift($args,'-'.$option[$i]);
 				}
 				break;
 			}
@@ -154,7 +157,7 @@ while(!is_null($option = array_shift($argv)))
 			break;
 
 		case '-d': case '--date':
-			$time = strtotime(array_shift($argv));
+			$time = strtotime(array_shift($args));
 			break;
 
 		case '-a': case '--all':
@@ -162,14 +165,14 @@ while(!is_null($option = array_shift($argv)))
 			break;
 
 		case '--user':
-			$user = array_shift($argv);
+			$user = array_shift($args);
 			break;
 		case '--password':
 		case '--passwd':
-			$passwd = array_shift($argv);
+			$passwd = array_shift($args);
 			break;
 		case '--domain':
-			$domain = array_shift($argv);
+			$domain = array_shift($args);
 			break;
 	}
 }
@@ -177,7 +180,6 @@ if ($user && $passwd)
 {
 	load_egw($user,$passwd,$domain ? $domain : 'default');
 }
-$argv = $args;
 $argc = count($argv);
 
 switch($cmd)
@@ -185,7 +187,7 @@ switch($cmd)
 	case 'umount':
 		if ($argc != 1 && !$all)
 		{
-			usage();
+			usage('Wrong number of parameters!');
 		}
 		if (($url = $argv[0])) load_wrapper($url);
 		if(!egw_vfs::$is_root)
@@ -209,7 +211,7 @@ switch($cmd)
 	case 'mount':
 		if ($argc > 2)
 		{
-			usage();
+			usage('Wrong number of parameters!');
 		}
 		load_wrapper($url=$argv[0]);
 
@@ -252,7 +254,7 @@ switch($cmd)
 		break;
 
 	case 'rename':
-		if (count($argv) != 2) usage(null,3);
+		if (count($argv) != 2) usage('Wrong number of parameters!');
 		load_wrapper($argv[0]);
 		load_wrapper($argv[1]);
 		rename($argv[0],$argv[1]);
@@ -422,7 +424,7 @@ switch($cmd)
 							echo "\n$name:\n";
 						}
 						// separate evtl. query part, to re-add it after the file-name
-						$query = '';
+						unset($query);
 						list($url,$query) = explode('?',$url,2);
 						if ($query) $query = '?'.$query;
 
@@ -469,7 +471,7 @@ switch($cmd)
  */
 function load_wrapper($url)
 {
-	$scheme = egw_vfs::parse_url($url,PHP_URL_SCHEME);
+	$scheme = parse_url($url,PHP_URL_SCHEME);
 
 	if (!in_array($scheme,stream_get_wrappers()))
 	{
@@ -484,12 +486,12 @@ function load_wrapper($url)
 			default:
 				if (!isset($GLOBALS['egw']) && !in_array($scheme,array('smb','imap')))
 				{
-					load_egw(egw_vfs::parse_url($url,PHP_URL_USER),egw_vfs::parse_url($url,PHP_URL_PASS),egw_vfs::parse_url($url,PHP_URL_HOST));
+					load_egw(parse_url($url,PHP_URL_USER), parse_url($url,PHP_URL_PASS), parse_url($url,PHP_URL_HOST));
 				}
 				// get eGW's __autoload() function
 				include_once(EGW_API_INC.'/common_functions.inc.php');
 
-				if (!class_exists(str_replace('.','_',$scheme).'_stream_wrapper'))
+				if (!egw_vfs::load_wrapper($scheme))
 				{
 					die("Unknown scheme '$scheme' in $url !!!\n\n");
 				}
@@ -538,10 +540,12 @@ function load_egw($user,$passwd,$domain='default')
 		$GLOBALS['egw_info']['flags']['currentapp'] = 'login';
 		include('../header.inc.php');
 
-		if ($user == 'root_'.$GLOBALS['egw_info']['server']['header_admin_user'] &&
-			_check_pw($GLOBALS['egw_info']['server']['header_admin_password'],$passwd) ||
-			$user == 'root_'.$GLOBALS['egw_domain'][$domain]['config_user'] &&
-			_check_pw($GLOBALS['egw_domain'][$domain]['config_passwd'],$passwd))
+		if (setup::check_auth($user, $passwd,
+				'root_'.$GLOBALS['egw_info']['server']['header_admin_user'],
+				$GLOBALS['egw_info']['server']['header_admin_password']) ||
+			setup::check_auth($user, $passwd,
+				'root_'.$GLOBALS['egw_domain'][$domain]['config_user'],
+				$GLOBALS['egw_domain'][$domain]['config_passwd']))
 		{
 			echo "\nRoot access granted!\n";
 			egw_vfs::$is_root = true;
@@ -560,23 +564,6 @@ function load_egw($user,$passwd,$domain='default')
 	}
 }
 
-/**
- * Check password against a md5 hash or cleartext password
- *
- * @param string $hash_or_cleartext
- * @param string $pw
- * @return boolean
- */
-function _check_pw($hash_or_cleartext,$pw)
-{
-	//echo "_check_pw($hash_or_cleartext,$pw) md5=".md5($pw)."\n";
-	if (preg_match('/^[0-9a-f]{32}$/',$hash_or_cleartext))
-	{
-		return $hash_or_cleartext == md5($pw);
-	}
-	return $hash_or_cleartext == $pw;
-}
-
 /**
  * Set, delete or show the extended acl for a given path
  *
@@ -588,7 +575,7 @@ function do_eacl(array $argv)
 
 	if ($argc < 1 || $argc > 3)
 	{
-		usage();
+		usage('Wrong number of parameters!');
 	}
 	load_wrapper($url = $argv[0]);
 	if (!class_exists('egw_vfs'))
@@ -634,10 +621,10 @@ function do_eacl(array $argv)
  * Give the stats for one file
  *
  * @param string $url
- * @param boolean $long=false true=long listing with owner,group,size,perms, default false only filename
- * @param boolean $numeric=false true=give numeric uid&gid, else resolve the id to a name
- * @param boolean $full_path=false true=give full path instead of just filename
- * @param boolean $inode=false true=display inode (sqlfs id)
+ * @param boolean $long =false true=long listing with owner,group,size,perms, default false only filename
+ * @param boolean $numeric =false true=give numeric uid&gid, else resolve the id to a name
+ * @param boolean $full_path =false true=give full path instead of just filename
+ * @param boolean $inode =false true=display inode (sqlfs id)
  */
 function do_stat($url,$long=false,$numeric=false,$full_path=false,$inode=false)
 {
@@ -724,8 +711,9 @@ function do_cp($argv,$recursive=false,$perms=false)
 
 	if (count($argv) > 1 && $to_exists && !is_dir($to))
 	{
-		usage(null,4);
+		usage("No such directory '$to'!");
 	}
+	$anz_dirs = $anz_files = 0;
 	foreach($argv as $from)
 	{
 		if (is_dir($from) && (!file_exists($to) || is_dir($to)) && $recursive && class_exists('egw_vfs'))
@@ -755,11 +743,11 @@ function do_cp($argv,$recursive=false,$perms=false)
 	}
 }
 
-function _cp($from,$to,$verbose=false,$perms=false)
+function _cp($from,$to,$verbose=false)
 {
 	load_wrapper($from);
 
-	if (is_dir($to) || !file_exists($to) && is_dir($from) && $mkdir)
+	if (is_dir($to))
 	{
 		$path = egw_vfs::parse_url($from,PHP_URL_PATH);
 		if (is_dir($to))
@@ -829,9 +817,17 @@ function do_lntree($from,$to)
 	if ($from[0] == '/') $from = 'sqlfs://default'.$from;
 	load_wrapper($from);
 
-	if (!file_exists($from) || $to[0] != '/' || file_exists($to) || !is_writable(dirname($to)))
+	if (!file_exists($from))
 	{
-		usage(null,4);
+		usage("Directory '$from' does NOT exist!");
+	}
+	elseif ($to[0] != '/' || file_exists($to))
+	{
+		usage("Directory '$to' does not exist!");
+	}
+	elseif (!is_writable(dirname($to)))
+	{
+		usage("Directory '$to' is not writable!");
 	}
 	egw_vfs::find($from, array(
 		'url' => true,
