@@ -5,9 +5,9 @@
  * @link http://www.egroupware.org
  * @package filemanager
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @copyright (c) 2008-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
+ * @copyright (c) 2008-16 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.filemanager_ui.inc.php 49548 2014-11-21 19:36:53Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -58,8 +58,8 @@ class filemanager_ui
 			egw_vfs::$is_root = true;
 		}
 
-		self::init_views();
-		self::$merge_prop_namespace = egw_vfs::DEFAULT_PROP_NAMESPACE.$GLOBALS['egw_info']['flags']['currentapp'];
+		static::init_views();
+		static::$merge_prop_namespace = egw_vfs::DEFAULT_PROP_NAMESPACE.$GLOBALS['egw_info']['flags']['currentapp'];
 	}
 
 	/**
@@ -69,21 +69,21 @@ class filemanager_ui
 	 */
 	public static function init_views()
 	{
-		if (!self::$views_init)
+		if (!static::$views_init)
 		{
 			// translate our labels
-			foreach(self::$views as $method => &$label)
+			foreach(static::$views as &$label)
 			{
 				$label = lang($label);
 			}
 			// search for plugins with additional filemanager views
-			foreach($GLOBALS['egw']->hooks->process('filemanager_views') as $app => $views)
+			foreach($GLOBALS['egw']->hooks->process('filemanager_views') as $views)
 			{
-				if (is_array($views)) self::$views += $views;
+				if (is_array($views)) static::$views += $views;
 			}
-			self::$views_init = true;
+			static::$views_init = true;
 		}
-		return self::$views;
+		return static::$views;
 	}
 
 	/**
@@ -98,10 +98,10 @@ class filemanager_ui
 		{
 			$view = $_GET['view'];
 		}
-		if (!isset(self::$views[$view]))
+		if (!isset(static::$views[$view]))
 		{
-			reset(self::$views);
-			$view = key(self::$views);
+			reset(static::$views);
+			$view = key(static::$views);
 		}
 		return $view;
 	}
@@ -111,7 +111,7 @@ class filemanager_ui
 	 *
 	 * @return array
 	 */
-	private static function get_actions()
+	public static function get_actions()
 	{
 		$actions = array(
 			'open' => array(
@@ -144,27 +144,18 @@ class filemanager_ui
 				'allowOnMultiple' => false,
 				'onExecute' => 'javaScript:app.filemanager.editprefs',
 			),
+			'mkdir' => array(
+				'caption' => lang('Create directory'),
+				'icon' => 'filemanager/button_createdir',
+				'group' => $group,
+				'allowOnMultiple' => false,
+				'onExecute' => 'javaScript:app.filemanager.createdir'
+			),
 			'mail' => array(
 				'caption' => lang('Mail files'),
 				'icon' => 'filemanager/mail_post_to',
 				'group' => $group,
-				'onExecute' => 'javaScript:app.filemanager.mail',
-			),
-			'copy' => array(
-				'caption' => lang('Copy'),
-				'group' => ++$group,
-				'onExecute' => 'javaScript:app.filemanager.clipboard',
-			),
-			'add' => array(
-				'caption' => lang('Add to clipboard'),
-				'group' => $group,
-				'icon' => 'copy',
-				'onExecute' => 'javaScript:app.filemanager.clipboard',
-			),
-			'cut' => array(
-				'caption' => lang('Cut'),
-				'group' => $group,
-				'onExecute' => 'javaScript:app.filemanager.clipboard',
+				'children' => array(),
 			),
 			'documents' => filemanager_merge::document_action(
 				$GLOBALS['egw_info']['user']['preferences']['filemanager']['document_dir'],
@@ -186,7 +177,8 @@ class filemanager_ui
 			'file_drop_mail' => array(
 				'type' => 'drop',
 				'acceptedTypes' => 'mail',
-				'onExecute' => 'javaScript:app.filemanager.drop'
+				'onExecute' => 'javaScript:app.filemanager.drop',
+				'hideOnDisabled' => true
 			),
 			'file_drop_move' => array(
 				'icon' => 'stylite/move',
@@ -202,12 +194,30 @@ class filemanager_ui
 				'caption' => lang('Copy into folder'),
 				'type' => 'drop',
 				'onExecute' => 'javaScript:app.filemanager.drop'
+			),
+			'file_drop_symlink' => array(
+				'icon' => 'linkpaste',
+				'acceptedTypes' => 'file',
+				'caption' => lang('Link into folder'),
+				'type' => 'drop',
+				'onExecute' => 'javaScript:app.filemanager.drop'
 			)
 		);
 		if (!isset($GLOBALS['egw_info']['user']['apps']['mail']))
 		{
 			unset($actions['mail']);
 		}
+		else
+		{
+			foreach(egw_sharing::$modes as $mode => $data)
+			{
+				$actions['mail']['children']['mail_'.$mode] = array(
+					'caption' => $data['label'],
+					'hint' => $data['title'],
+					'onExecute' => 'javaScript:app.filemanager.mail',
+				);
+			}
+		}
 		return $actions;
 	}
 
@@ -227,7 +237,7 @@ class filemanager_ui
 		switch($scope)
 		{
 			case 'self':
-				$props = egw_vfs::propfind($path, self::$merge_prop_namespace);
+				$props = egw_vfs::propfind($path, static::$merge_prop_namespace);
 				$app = empty($props) ? null : $props[0]['val'];
 				break;
 			case 'parents':
@@ -235,7 +245,7 @@ class filemanager_ui
 				$currentpath = $path;
 				while($dir = egw_vfs::dirname($currentpath))
 				{
-					$props = egw_vfs::propfind($dir, self::$merge_prop_namespace);
+					$props = egw_vfs::propfind($dir, static::$merge_prop_namespace);
 					if(!empty($props))
 					{
 						// found prop in parent directory
@@ -277,21 +287,28 @@ class filemanager_ui
 					'default_cols'   => '!comment,ctime',	// I  columns to use if there's no user or default pref (! as first char uses all but the named columns), default all columns
 					'csv_fields'     =>	false, // I  false=disable csv export, true or unset=enable it with auto-detected fieldnames,
 									//or array with name=>label or name=>array('label'=>label,'type'=>type) pairs (type is a eT widget-type)
-					'actions'        => self::get_actions(),
+					'actions'        => static::get_actions(),
 					'row_id'         => 'path',
 					'row_modified'   => 'mtime',
 					'parent_id'      => 'dir',
 					'is_parent'      => 'mime',
 					'is_parent_value'=> egw_vfs::DIR_MIME_TYPE,
 					'header_left'    => 'filemanager.index.header_left',
-					'favorites'      => true
+					'favorites'      => true,
+					'placeholder_actions' => array('mkdir','file_drop_mail','file_drop_move','file_drop_copy','file_drop_symlink')
 				);
-				$content['nm']['path'] = self::get_home_dir();
+				$content['nm']['path'] = static::get_home_dir();
 			}
-			$content['nm']['home_dir'] = self::get_home_dir();
+			$content['nm']['home_dir'] = static::get_home_dir();
+			$content['nm']['view'] = $GLOBALS['egw_info']['user']['preferences']['filemanager']['nm_view'];
 
 			if (isset($_GET['msg'])) $msg = $_GET['msg'];
 
+			// Blank favorite set via GET needs special handling for path
+			if (isset($_GET['favorite']) && $_GET['favorite'] == 'blank')
+			{
+				$content['nm']['path'] = static::get_home_dir();
+			}
 			// switch to projectmanager folders
 			if (isset($_GET['pm_id']))
 			{
@@ -305,7 +322,7 @@ class filemanager_ui
 						$path = egw_vfs::dirname($content['nm']['path']);
 						break;
 					case '~':
-						$path = self::get_home_dir();
+						$path = static::get_home_dir();
 						break;
 				}
 				if ($path[0] == '/' && egw_vfs::stat($path,true) && egw_vfs::is_dir($path) && egw_vfs::check_access($path,egw_vfs::READABLE))
@@ -322,7 +339,7 @@ class filemanager_ui
 				if (!$content['nm']['filter']) $content['nm']['filter'] = '';
 			}
 		}
-		$view = self::get_view();
+		$view = static::get_view();
 
 		if (strpos($view,'::') !== false && version_compare(PHP_VERSION,'5.3.0','<'))
 		{
@@ -377,7 +394,7 @@ class filemanager_ui
 		{
 			if ($content['nm']['action'])
 			{
-				$msg = self::action($content['nm']['action'],$content['nm']['selected'],$content['nm']['path']);
+				$msg = static::action($content['nm']['action'],$content['nm']['selected'],$content['nm']['path']);
 				if($msg) egw_framework::message($msg);
 
 				// clean up after action
@@ -391,7 +408,7 @@ class filemanager_ui
 			}
 			elseif($content['nm']['rows']['delete'])
 			{
-				$msg = self::action('delete',array_keys($content['nm']['rows']['delete']),$content['nm']['path']);
+				$msg = static::action('delete',array_keys($content['nm']['rows']['delete']),$content['nm']['path']);
 				if($msg) egw_framework::message($msg);
 
 				// clean up after action
@@ -466,6 +483,21 @@ class filemanager_ui
 			'5' => 'Files from links',
 			'0'  => 'Files from subdirectories',
 		);
+		// sharing has no divAppbox, we need to set popupMainDiv instead, to be able to drop files everywhere
+		if (substr($_SERVER['SCRIPT_FILENAME'], -10) == '/share.php')
+		{
+			$tpl->setElementAttribute('nm[buttons][upload]', 'drop_target', 'popupMainDiv');
+		}
+		// Set view button to match current settings
+		if($content['nm']['view'] == 'tile')
+		{
+			$tpl->setElementAttribute('nm[buttons][button][change_view]','label',lang('List view'));
+			$tpl->setElementAttribute('nm[buttons][button][change_view]','image','list_row');
+
+		}
+		// if initial load is done via GET request (idots template or share.php)
+		// get_rows cant call app.filemanager.set_readonly, so we need to do that here
+		$content['initial_path_readonly'] = !egw_vfs::is_writable($content['nm']['path']);
 
 		$tpl->exec('filemanager.filemanager_ui.index',$content,$sel_options,$readonlys,array('nm' => $content['nm']));
 	}
@@ -485,6 +517,10 @@ class filemanager_ui
 		{
 			$start = $path;
 		}
+		elseif (!egw_vfs::is_dir($start) && egw_vfs::check_access($start, egw_vfs::READABLE))
+		{
+			$start = '/';
+		}
 		return $start;
 	}
 
@@ -499,7 +535,7 @@ class filemanager_ui
 	 * @param int &$files=null on return number of files deleted
 	 * @return string success or failure message displayed to the user
 	 */
-	static private function action($action,$selected,$dir=null,&$errs=null,&$files=null,&$dirs=null)
+	static public function action($action,$selected,$dir=null,&$errs=null,&$files=null,&$dirs=null)
 	{
 		if (!count($selected))
 		{
@@ -509,12 +545,11 @@ class filemanager_ui
 
 		switch($action)
 		{
-			case 'mail':
-				throw new egw_exception_assertion_failed('Implemented on clientside!');
 
 			case 'delete':
-				return self::do_delete($selected,$errs,$files,$dirs);
+				return static::do_delete($selected,$errs,$files,$dirs);
 
+			case 'mail':
 			case 'copy':
 				foreach($selected as $path)
 				{
@@ -652,7 +687,7 @@ class filemanager_ui
 						$document_merge = new filemanager_merge(egw_vfs::decodePath($dir));
 						$msg = $document_merge->download($settings, $selected, '', $GLOBALS['egw_info']['user']['preferences']['filemanager']['document_dir']);
 						if($msg) return $msg;
-						$failed = count($selected);
+						$errs = count($selected);
 						return false;
 				}
 		}
@@ -702,7 +737,8 @@ class filemanager_ui
 			// now we use find to loop through all files and dirs: (selected only contains dirs now)
 			// - depth=true to get first the files and then the dir containing it
 			// - hidden=true to also return hidden files (eg. Thumbs.db), as we cant delete non-empty dirs
-			foreach(egw_vfs::find($selected,array('depth'=>true,'hidden'=>true)) as $path)
+			// - show-deleted=false to not (finally) deleted versioned files
+			foreach(egw_vfs::find($selected,array('depth'=>true,'hidden'=>true,'show-deleted'=>false)) as $path)
 			{
 				if (($is_dir = egw_vfs::is_dir($path) && !egw_vfs::is_link($path)) && egw_vfs::rmdir($path,0))
 				{
@@ -734,22 +770,28 @@ class filemanager_ui
 	 *
 	 * @param array $query
 	 * @param array &$rows
-	 * @param array &$readonlys
 	 */
-	function get_rows($query,&$rows,&$readonlys)
+	function get_rows(&$query, &$rows)
 	{
-		// show projectmanager sidebox for projectmanager path
-		if (substr($query['path'],0,20) == '/apps/projectmanager' && isset($GLOBALS['egw_info']['user']['apps']['projectmanager']))
-		{
-			$GLOBALS['egw_info']['flags']['currentapp'] = 'projectmanager';
-		}
 		// do NOT store query, if hierarchical data / children are requested
 		if (!$query['csv_export'])
 		{
 			egw_session::appsession('index','filemanager',$query);
 		}
-		if(!$query['path']) $query['path'] = self::get_home_dir();
+		if(!$query['path']) $query['path'] = static::get_home_dir();
 
+		// Change template to match selected view
+		if($query['view'])
+		{
+			$query['template'] = ($query['view'] == 'row' ? 'filemanager.index.rows' : 'filemanager.tile');
+
+			// Store as preference but only for index, not home
+			if($query['get_rows'] == 'filemanager.filemanager_ui.get_rows')
+			{
+				$GLOBALS['egw']->preferences->add('filemanager','nm_view',$query['view']);
+				$GLOBALS['egw']->preferences->save_repository();
+			}
+		}
 		// be tolerant with (in previous versions) not correct urlencoded pathes
 		if (!egw_vfs::stat($query['path'],true) && egw_vfs::stat(urldecode($query['path'])))
 		{
@@ -757,13 +799,19 @@ class filemanager_ui
 		}
 		if (!egw_vfs::stat($query['path'],true) || !egw_vfs::is_dir($query['path']) || !egw_vfs::check_access($query['path'],egw_vfs::READABLE))
 		{
-			// we will leave here, since we are not allowed, or the location does not exist. Index must handle that, and give
-			// an appropriate message
-			egw::redirect_link('/index.php',array('menuaction'=>'filemanager.filemanager_ui.index',
-				'path' => self::get_home_dir(),
-				'msg' => lang('The requested path %1 is not available.',egw_vfs::decodePath($query['path'])),
-				'ajax' => 'true'
-			));
+			// only redirect, if it would be to some other location, gives redirect-loop otherwise
+			if ($query['path'] != ($path = static::get_home_dir()))
+			{
+				// we will leave here, since we are not allowed, or the location does not exist. Index must handle that, and give
+				// an appropriate message
+				egw::redirect_link('/index.php',array('menuaction'=>'filemanager.filemanager_ui.index',
+					'path' => $path,
+					'msg' => lang('The requested path %1 is not available.',egw_vfs::decodePath($query['path'])),
+					'ajax' => 'true'
+				));
+			}
+			$rows = array();
+			return 0;
 		}
 		$rows = $dir_is_writable = array();
 		if($query['searchletter'] && !empty($query['search']))
@@ -789,7 +837,8 @@ class filemanager_ui
 
 		$maxdepth = $filter && $filter != 4 ? (int)(boolean)$filter : null;
 		if($filter == 5) $maxdepth = 2;
-		foreach(egw_vfs::find(!empty($query['col_filter']['dir']) ? $query['col_filter']['dir'] : $query['path'],array(
+		$n = 0;
+		$vfs_options = array(
 			'mindepth' => 1,
 			'maxdepth' => $maxdepth,
 			'dirsontop' => $filter <= 1,
@@ -800,7 +849,12 @@ class filemanager_ui
 			'name_preg' => $namefilter,
 			'hidden' => $filter == 3,
 			'follow' => $filter == 5,
-		),true) as $path => $row)
+		);
+		if($query['col_filter']['mime'])
+		{
+			$vfs_options['mime'] = $query['col_filter']['mime'];
+		}
+		foreach(egw_vfs::find(!empty($query['col_filter']['dir']) ? $query['col_filter']['dir'] : $query['path'],$vfs_options,true) as $path => $row)
 		{
 			//echo $path; _debug_array($row);
 
@@ -814,12 +868,19 @@ class filemanager_ui
 				$row['class'] = 'isDir';
 			}
 			$row['download_url'] = egw_vfs::download_url($path);
+			$row['gid'] = -abs($row['gid']);	// gid are positive, but we use negagive account_id for groups internal
 
 			$rows[++$n] = $row;
 			$path2n[$path] = $n;
 		}
 		// query comments and cf's for the displayed rows
 		$cols_to_show = explode(',',$GLOBALS['egw_info']['user']['preferences']['filemanager']['nextmatch-filemanager.index.rows']);
+
+		// Always include comment in tiles
+		if($query['view'] == 'tile')
+		{
+			$cols_to_show[] = 'comment';
+		}
 		$all_cfs = in_array('customfields',$cols_to_show) && $cols_to_show[count($cols_to_show)-1][0] != '#';
 		if ($path2n && (in_array('comment',$cols_to_show) || in_array('customfields',$cols_to_show)) &&
 			($path2props = egw_vfs::propfind(array_keys($path2n))))
@@ -943,6 +1004,7 @@ class filemanager_ui
 			if (in_array($button,array('save','apply')))
 			{
 				$props = array();
+				$perm_changed = $perm_failed = 0;
 				foreach($content['old'] as $name => $old_value)
 				{
 					if (isset($content[$name]) && ($old_value != $content[$name] ||
@@ -986,7 +1048,7 @@ class filemanager_ui
 						{
 							$mergeprop = array(
 								array(
-									'ns'	=> self::$merge_prop_namespace,
+									'ns'	=> static::$merge_prop_namespace,
 									'name'	=> 'mergeapp',
 									'val'	=> (!empty($content[$name]) ? $content[$name] : null),
 								),
@@ -1005,7 +1067,7 @@ class filemanager_ui
 						{
 							static $name2cmd = array('uid' => 'chown','gid' => 'chgrp','perms' => 'chmod');
 							$cmd = array('egw_vfs',$name2cmd[$name]);
-							$value = $name == 'perms' ? self::perms2mode($content['perms']) : $content[$name];
+							$value = $name == 'perms' ? static::perms2mode($content['perms']) : $content[$name];
 							if ($content['modify_subs'])
 							{
 								if ($name == 'perms')
@@ -1073,7 +1135,7 @@ class filemanager_ui
 				if ($content['eacl']['delete'])
 				{
 					list($ino_owner) = each($content['eacl']['delete']);
-					list($ino,$owner) = explode('-',$ino_owner,2);	// $owner is a group and starts with a minus!
+					list(, $owner) = explode('-',$ino_owner,2);	// $owner is a group and starts with a minus!
 					$msg .= egw_vfs::eacl($path,null,$owner) ? lang('ACL deleted.') : lang('Error deleting the ACL entry!');
 				}
 				elseif ($button == 'eacl')
@@ -1162,8 +1224,8 @@ class filemanager_ui
 		}
 
 		// mergeapp
-		$content['mergeapp'] = self::get_mergeapp($path, 'self');
-		$content['mergeapp_parent'] = self::get_mergeapp($path, 'parents');
+		$content['mergeapp'] = static::get_mergeapp($path, 'self');
+		$content['mergeapp_parent'] = static::get_mergeapp($path, 'parents');
 		if(!empty($content['mergeapp']))
 		{
 			$content['mergeapp_effective'] = $content['mergeapp'];
@@ -1222,7 +1284,7 @@ class filemanager_ui
 			$tpl->setElementAttribute('tabs', 'add_tabs', true);
 
 			$tabs =& $tpl->getElementAttribute('tabs','tabs');
-			$tabs = array();
+			if (true) $tabs = array();
 
 			foreach(isset($extra_tabs[0]) ? $extra_tabs : array($extra_tabs) as $extra_tab)
 			{
@@ -1256,7 +1318,7 @@ class filemanager_ui
 	 * @param string $action eg. 'delete', ...
 	 * @param array $selected selected path(s)
 	 * @param string $dir=null current directory
-	 * @see self::action()
+	 * @see static::action()
 	 */
 	public static function ajax_action($action, $selected, $dir=null, $props=null)
 	{
@@ -1341,7 +1403,7 @@ class filemanager_ui
 			// Upload, then link
 			case 'link':
 				// First upload
-				$arr = self::ajax_action('upload', $selected, $dir, $props);
+				$arr = static::ajax_action('upload', $selected, $dir, $props);
 				$app_dir = egw_link::vfs_path($props['entry']['app'],$props['entry']['id'],'',true);
 
 				foreach($arr['uploaded'] as $file)
@@ -1358,7 +1420,7 @@ class filemanager_ui
 				return;
 
 			default:
-				$arr['msg'] = self::action($action, $selected, $dir, $arr['errs'], $arr['dirs'], $arr['files']);
+				$arr['msg'] = static::action($action, $selected, $dir, $arr['errs'], $arr['dirs'], $arr['files']);
 		}
 		$response->data($arr);
 		//error_log(__METHOD__."('$action',".array2string($selected).') returning '.array2string($arr));
