@@ -6,7 +6,7 @@
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright (c) 2008-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: app.js 51081 2014-12-18 13:47:04Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -18,21 +18,13 @@ app.classes.filemanager = AppJS.extend(
 {
 	appname: 'filemanager',
 	/**
-	 * path widget
+	 * path widget, by template
 	 */
-	path_widget: null,
-	/**
-	 * Array of pathes from files in clipboard
-	 */
-	clipboard_files: [],
+	path_widget: {},
 	/**
 	 * Are files cut into clipboard - need to be deleted at source on paste
 	 */
 	clipboard_is_cut: false,
-	/**
-	 * Key for storing clipboard in browser localstorage
-	 */
-	clipboard_localstore_key: 'filemanager.clipboard',
 
 	/**
 	 * Constructor
@@ -43,6 +35,17 @@ app.classes.filemanager = AppJS.extend(
 	{
 		// call parent
 		this._super.apply(this, arguments);
+
+		// Loading filemanager in its tab and home causes us problems with
+		// unwanted destruction, so we check for already existing path widgets
+		var lists = etemplate2.getByApplication('home');
+		for(var i = 0; i < lists.length; i++)
+		{
+			if(lists[i].app == 'filemanager' && lists[i].widgetContainer.getWidgetById('path'))
+			{
+				this.path_widget[lists[i].uniqueId] = lists[i].widgetContainer.getWidgetById('path');
+			}
+		}
 	},
 
 	/**
@@ -61,13 +64,22 @@ app.classes.filemanager = AppJS.extend(
 	 * make sure to clean it up in destroy().
 	 *
 	 * @param et2 etemplate2 Newly ready object
+	 * @param {string} name template name
 	 */
-	et2_ready: function(et2)
+	et2_ready: function(et2,name)
 	{
 		// call parent
 		this._super.apply(this, arguments);
 
-		this.path_widget = this.et2.getWidgetById('path');
+		var path_widget = this.et2.getWidgetById('path');
+		if(path_widget)	// do NOT set not found path-widgets, as uploads works on first one only!
+		{
+			this.path_widget[et2.DOMContainer.id] = path_widget;
+			// Bind to removal to remove from list
+			$j(et2.DOMContainer).on('clear', function(e) {
+				if (app.filemanager && app.filemanager.path_widget) delete app.filemanager.path_widget[e.target.id];
+			});
+		}
 
 		if(this.et2.getWidgetById('nm'))
 		{
@@ -77,12 +89,13 @@ app.classes.filemanager = AppJS.extend(
 		}
 
 		// get clipboard from browser localstore and update button tooltips
-		if (typeof window.localStorage != 'undefined' && typeof window.localStorage[this.clipboard_localstore_key] != 'undefined')
-		{
-			this.clipboard_files = JSON.parse(window.localStorage[this.clipboard_localstore_key]);
-		}
 		this.clipboard_tooltips();
 
+		// calling set_readonly for initial path
+		if (this.et2.getArrayMgr('content').getEntry('initial_path_readonly'))
+		{
+			this.readonly = [this.et2.getArrayMgr('content').getEntry('nm[path]'), true];
+		}
 		if (typeof this.readonly != 'undefined')
 		{
 			this.set_readonly.apply(this, this.readonly);
@@ -90,6 +103,64 @@ app.classes.filemanager = AppJS.extend(
 		}
 	},
 
+	/**
+	 * Set the application's state to the given state.
+	 *
+	 * Extended from parent to also handle view
+	 *
+	 *
+	 * @param {{name: string, state: object}|string} state Object (or JSON string) for a state.
+	 *	Only state is required, and its contents are application specific.
+	 *
+	 * @return {boolean} false - Returns false to stop event propagation
+	 */
+	setState: function(state)
+	{
+		// State should be an object, not a string, but we'll parse
+		if(typeof state == "string")
+		{
+			if(state.indexOf('{') != -1 || state =='null')
+			{
+				state = JSON.parse(state);
+			}
+		}
+		var result = this._super.call(this, state, 'filemanager.index');
+
+		// This has to happen after the parent, changing to tile recreates
+		// nm controller
+		if(typeof state == "object" && state.state && state.state.view)
+		{
+			var et2 = etemplate2.getById('filemanager-index');
+			if(et2)
+			{
+				this.et2 = et2.widgetContainer;
+				this.change_view(state.state.view);
+			}
+		}
+		return result;
+	},
+
+	/**
+	 * Retrieve the current state of the application for future restoration
+	 *
+	 * Extended from parent to also set view
+	 *
+	 * @return {object} Application specific map representing the current state
+	 */
+	getState: function()
+	{
+		var state = this._super.apply(this);
+
+		var et2 = etemplate2.getById('filemanager-index');
+		if(et2)
+		{
+			var nm = et2.widgetContainer.getWidgetById('nm');
+			state.view = nm.view;
+		}
+
+		return state;
+	},
+
 	/**
 	 * Regexp to convert id to a path, use this.id2path(_id)
 	 */
@@ -108,7 +179,7 @@ app.classes.filemanager = AppJS.extend(
 	/**
 	 * Convert array of elems to array of paths
 	 *
-	 * @param {array} _elems selected items from actions
+	 * @param {egwActionObject[]} _elems selected items from actions
 	 * @return array
 	 */
 	_elems2paths: function(_elems)
@@ -116,7 +187,9 @@ app.classes.filemanager = AppJS.extend(
 		var paths = [];
 		for (var i = 0; i < _elems.length; i++)
 		{
-			paths.push(this.id2path(_elems[i].id));
+			// If selected has no id, try parent.  This happens for the placeholder row
+			// in empty directories.
+			paths.push(_elems[i].id? this.id2path(_elems[i].id) : _elems[i]._context._parentId);
 		}
 		return paths;
 	},
@@ -148,22 +221,28 @@ app.classes.filemanager = AppJS.extend(
 	/**
 	 * Get current working directory
 	 *
+	 * @param {string} etemplate_name
 	 * @return string
 	 */
-	get_path: function()
+	get_path: function(etemplate_name)
 	{
-		return this.path_widget ? this.path_widget.get_value() : null;
+		if(!etemplate_name || typeof this.path_widget[etemplate_name] == 'undefined')
+		{
+			for(etemplate_name in this.path_widget) break;
+		}
+		return this.path_widget[etemplate_name] ? this.path_widget[etemplate_name].get_value() : null;
 	},
 
 	/**
 	 * Open compose with already attached files
 	 *
 	 * @param {(string|string[])} attachments path(s)
+	 * @param {object} params
 	 */
-	open_mail: function(attachments)
+	open_mail: function(attachments, params)
 	{
-		if (typeof attachments == 'undefined') attachments = this.clipboard_files;
-		var params = {};
+		if (typeof attachments == 'undefined') attachments = this.get_clipboard_files();
+		if (!params || typeof params != 'object') params = {};
 		if (!(attachments instanceof Array)) attachments = [ attachments ];
 		for(var i=0; i < attachments.length; i++)
 		{
@@ -180,7 +259,18 @@ app.classes.filemanager = AppJS.extend(
 	 */
 	mail: function(_action, _elems)
 	{
-		this.open_mail(this._elems2paths(_elems));
+		this.open_mail(this._elems2paths(_elems), {
+			'preset[filemode]': _action.id.substr(5)
+		});
+	},
+
+	/**
+	 * Trigger Upload after each file is uploaded
+	 * @param {type} _event
+	 */
+	uploadOnOne: function (_event)
+	{
+		this.upload(_event,1);
 	},
 
 	/**
@@ -309,6 +399,35 @@ app.classes.filemanager = AppJS.extend(
 		}
 	},
 
+	/**
+	 * Get any files that are in the system clipboard
+	 *
+	 * @return {string[]} Paths
+	 */
+	get_clipboard_files: function()
+	{
+		var clipboard_files = [];
+		if (typeof window.localStorage != 'undefined' && typeof egw.getSessionItem('phpgwapi', 'egw_clipboard') != 'undefined')
+		{
+			var clipboard = JSON.parse(egw.getSessionItem('phpgwapi', 'egw_clipboard')) || {
+				type:[],
+				selected:[]
+			};
+			if(clipboard.type.indexOf('file') >= 0)
+			{
+				for(var i = 0; i < clipboard.selected.length; i++)
+				{
+					var split = clipboard.selected[i].id.split('::');
+					if(split[0] == 'filemanager')
+					{
+						clipboard_files.push(this.id2path(clipboard.selected[i].id));
+					}
+				}
+			}
+		}
+		return clipboard_files;
+	},
+
 	/**
 	 * Update clickboard tooltips in buttons
 	 */
@@ -318,7 +437,7 @@ app.classes.filemanager = AppJS.extend(
 		for(var i=0; i < paste_buttons.length; ++i)
 		{
 			var button = this.et2.getWidgetById(paste_buttons[i]);
-			if (button) button.set_statustext(this.clipboard_files.join(",\n"));
+			if (button) button.set_statustext(this.get_clipboard_files().join(",\n"));
 		}
 	},
 
@@ -331,21 +450,39 @@ app.classes.filemanager = AppJS.extend(
 	clipboard: function(_action, _elems)
 	{
 		this.clipboard_is_cut = _action.id == "cut";
-		if (_action.id != "add") this.clipboard_files = [];
-
-		this.clipboard_files = this.clipboard_files.concat(this._elems2paths(_elems));
-
-		if (_action.id == "add" && this.clipboard_files.length > 1)
+		var clipboard = JSON.parse(egw.getSessionItem('phpgwapi', 'egw_clipboard')) || {
+			type:[],
+			selected:[]
+		};
+		if(_action.id != "add")
 		{
-			// ToDo: make sure files are unique
+			clipboard = {
+				type:[],
+				selected:[]
+			};
+		};
+
+		// When pasting we need to know the type of data - pull from actions
+		var drag = _elems[0].getSelectedLinks('drag').links;
+		for(var k in drag)
+		{
+			if(drag[k].enabled && drag[k].actionObj.dragType.length > 0)
+			{
+				clipboard.type = clipboard.type.concat(drag[k].actionObj.dragType);
+			}
 		}
+		clipboard.type = jQuery.unique(clipboard.type);
+		// egwAction is a circular structure and can't be stringified so just take what we want
+		// Hopefully that's enough for the action handlers
+		for(var k in _elems)
+		{
+			if(_elems[k].id) clipboard.selected.push({id:_elems[k].id, data:_elems[k].data});
+		}
+
+		// Save it in session
+		egw.setSessionItem('phpgwapi', 'egw_clipboard', JSON.stringify(clipboard));
+
 		this.clipboard_tooltips();
-
-		// update localstore with files
-		if (typeof window.localStorage != 'undefined')
-		{
-			window.localStorage[this.clipboard_localstore_key] = JSON.stringify(this.clipboard_files);
-		}
 	},
 
 	/**
@@ -355,7 +492,8 @@ app.classes.filemanager = AppJS.extend(
 	 */
 	paste: function(_type)
 	{
-		if (this.clipboard_files.length == 0)
+		var clipboard_files = this.get_clipboard_files();
+		if (clipboard_files.length == 0)
 		{
 			alert(this.egw.lang('Clipboard is empty!'));
 			return;
@@ -363,22 +501,22 @@ app.classes.filemanager = AppJS.extend(
 		switch(_type)
 		{
 			case 'mailpaste':
-				this.open_mail(this.clipboard_files);
+				this.open_mail(clipboard_files);
 				break;
 
 			case 'paste':
-				this._do_action(this.clipboard_is_cut ? 'move' : 'copy', this.clipboard_files);
+				this._do_action(this.clipboard_is_cut ? 'move' : 'copy', clipboard_files);
 
 				if (this.clipboard_is_cut)
 				{
 					this.clipboard_is_cut = false;
-					this.clipboard_files = [];
+					clipboard_files = [];
 					this.clipboard_tooltips();
 				}
 				break;
 
 			case 'linkpaste':
-				this._do_action('symlink', this.clipboard_files);
+				this._do_action('symlink', clipboard_files);
 				break;
 		}
 	},
@@ -392,22 +530,41 @@ app.classes.filemanager = AppJS.extend(
 	action: function(_action, _elems)
 	{
 		var paths = this._elems2paths(_elems);
-		this._do_action(_action.id, paths);
+		var path = this.get_path(_action && _action.parent.data.nextmatch.getInstanceManager().uniqueId || false);
+		this._do_action(_action.id, paths,true, path);
 	},
 
 	/**
 	 * Prompt user for directory to create
+	 *
+	 * @param {egwAction|undefined|jQuery.Event} action Action, event or undefined if called directly
+	 * @param {egwActionObject[] | undefined} selected Selected row, or undefined if called directly
 	 */
-	createdir: function()
+	createdir: function(action, selected)
 	{
-		var dir = prompt(this.egw.lang('New directory'));
-
-		if (dir)
-		{
-			var path = this.get_path();
-			this._do_action('createdir', dir, true);	// true=synchronous request
-			this.change_dir((path == '/' ? '' : path)+'/'+dir);
-		}
+		var self = this;
+		et2_dialog.show_prompt(function(button, dir){
+			if (button && dir)
+			{
+				var path = self.get_path(action && action.parent ? action.parent.data.nextmatch.getInstanceManager().uniqueId : false);
+				if(action && action instanceof egwAction)
+				{
+					var paths = self._elems2paths(selected);
+					if(paths[0]) path = paths[0];
+					// check if target is a file --> use it's directory instead
+					if(selected[0].id || path)
+					{
+						var data = egw.dataGetUIDdata(selected[0].id || 'filemanager::'+path );
+						if (data && data.data.mime != 'httpd/unix-directory')
+						{
+							path = self.dirname(path);
+						}
+					}
+				}
+				self._do_action('createdir', dir, true, path);	// true=synchronous request
+				self.change_dir((path == '/' ? '' : path)+'/'+ dir);
+			}
+		},this.egw.lang('New directory'),this.egw.lang('Create directroy'));
 	},
 
 	/**
@@ -415,12 +572,13 @@ app.classes.filemanager = AppJS.extend(
 	 */
 	symlink: function()
 	{
-		var target = prompt(this.egw.lang('Link target'));
-
-		if (target)
-		{
-			this._do_action('symlink', target);
-		}
+		var self = this;
+		et2_dialog.show_prompt(function (button, target) {
+			if (button && target)
+			{
+				self._do_action('symlink', target);
+			}
+		},this.egw.lang('Link target'), this.egw.lang('Create link'));
 	},
 
 	/**
@@ -504,20 +662,83 @@ app.classes.filemanager = AppJS.extend(
 	/**
 	 * Change directory
 	 *
-	 * @param _dir directory to change to incl. '..' for one up
+	 * @param {string} _dir directory to change to incl. '..' for one up
+	 * @param {et2_widget} widget
 	 */
-	change_dir: function(_dir)
+	change_dir: function(_dir, widget)
 	{
+		for(var etemplate_name in this.path_widget) break;
+		if (widget) etemplate_name = widget.getInstanceManager().uniqueId;
+
 		switch (_dir)
 		{
 			case '..':
-				_dir = this.dirname(this.path_widget.getValue());
+				_dir = this.dirname(this.get_path(etemplate_name));
 				break;
 			case '~':
 				_dir = this.et2.getWidgetById('nm').options.settings.home_dir;
 				break;
 		}
-		this.path_widget.set_value(_dir);
+
+		this.path_widget[etemplate_name].set_value(_dir);
+	},
+
+	/**
+	 * Toggle view between tiles and rows
+	 *
+	 * @param {string|Event} [view] - Specify what to change the view to.  Either 'tile' or 'row'.
+	 *	Or, if this is used as a callback view is actually the event, and we need to find the view.
+	 * @param {et2_widget} [button_widget] - The widget that's calling
+	 */
+	change_view: function(view, button_widget)
+	{
+		var et2 = etemplate2.getById('filemanager-index');
+		var nm = false;
+		if(et2 && et2.widgetContainer.getWidgetById('nm'))
+		{
+			nm = et2.widgetContainer.getWidgetById('nm');
+		}
+		if(!nm)
+		{
+			egw.debug('warn', 'Could not find nextmatch to change view');
+
+			return;
+		}
+
+		if(!button_widget)
+		{
+			button_widget = nm.getWidgetById('button[change_view]');
+		}
+		if(button_widget && button_widget.instanceOf(et2_button))
+		{
+			// Switch view based on button icon, since controller can get re-created
+			if(typeof view != 'string')
+			{
+				view = button_widget.options.image.replace('list_','');
+			}
+
+			// Toggle button icon to the other view
+			button_widget.set_image("list_"+(view == nm.controller.VIEW_ROW ? nm.controller.VIEW_TILE : nm.controller.VIEW_ROW));
+
+			button_widget.set_label(view == nm.controller.VIEW_ROW ? this.egw.lang("Tile view") : this.egw.lang('List view'));
+		}
+
+		nm.set_view(view);
+		// Put it into active filters (but don't refresh)
+		nm.activeFilters.view = view;
+
+		// Change template to match
+		var template = view == nm.controller.VIEW_ROW ? 'filemanager.index.rows' : 'filemanager.tile';
+		nm.set_template(template);
+
+		// Wait for template to load, then refresh
+		template = nm.getWidgetById(template);
+		if(template && template.loading)
+		{
+			template.loading.done(function() {
+				nm.applyFilters({view: view});
+			})
+		}
 	},
 
 	/**
@@ -530,15 +751,24 @@ app.classes.filemanager = AppJS.extend(
 	{
 		var data = egw.dataGetUIDdata(_senders[0].id);
 		var path = this.id2path(_senders[0].id);
+		var mime = this.et2._inst.widgetContainer.getWidgetById('$row');
+		// try to get mime widget DOM node out of the row DOM
+		var mime_dom = jQuery(_senders[0].iface.getDOMNode()).find("span#filemanager-index_\\$row");
+
 		// symlinks dont have mime 'http/unix-directory', but server marks all directories with class 'isDir'
 		if (data.data.mime == 'httpd/unix-directory' || data.data['class'] && data.data['class'].split(/ +/).indexOf('isDir') != -1)
 		{
-			this.change_dir(path);
+			this.change_dir(path,_action.parent.data.nextmatch || this.et2);
+		}
+		else if(mime && data.data.mime.match(mime.mime_regexp) && mime_dom.length>0)
+		{
+			mime_dom.click();
 		}
 		else
 		{
-			egw.open({path: path, type: data.data.mime}, 'file');
+			egw.open({path: path, type: data.data.mime, download_url: data.data.download_url}, 'file','view',null,'_browser');
 		}
+		return false;
 	},
 
 	/**
@@ -549,12 +779,12 @@ app.classes.filemanager = AppJS.extend(
 	 */
 	editprefs: function(_action, _senders)
 	{
-		var path =  typeof _senders != 'undefined' ? this.id2path(_senders[0].id) : this.path_widget.getValue();
+		var path =  typeof _senders != 'undefined' ? this.id2path(_senders[0].id) : this.get_path(_action && _action.parent.data.nextmatch.getInstanceManager().uniqueId || false);
 
 		egw().open_link(egw.link('/index.php', {
 			menuaction: 'filemanager.filemanager_ui.file',
 			path: path
-		}), 'fileprefs', '495x425');
+		}), 'fileprefs', '510x425');
 	},
 
 	/**
@@ -568,17 +798,31 @@ app.classes.filemanager = AppJS.extend(
 	drop: function(_action, _elems, _target)
 	{
 		var src = this._elems2paths(_elems);
-		var dst = this.id2path(_target.id);
-		//alert(_action.id+': '+src.join(', ')+' --> '+dst);
 
-		// check if target is a file --> use it's directory instead
-		var data = egw.dataGetUIDdata(_target.id);
-		if (data.data.mime != 'httpd/unix-directory')
+
+		// Target will be missing ID if directory is empty
+		// so start with the current directory
+		var nm_dst = this.get_path(_action.parent.data.nextmatch.getInstanceManager().uniqueId || false);
+
+		// File(s) were dropped on a row, they want them inside
+		if(_target)
 		{
-			dst = this.dirname(dst);
+			var dst = '';
+			var paths = this._elems2paths([_target]);
+			if(paths[0]) dst = paths[0];
+
+			// check if target is a file --> use it's directory instead
+			if(_target.id)
+			{
+				var data = egw.dataGetUIDdata(_target.id);
+				if (!data || data.data.mime != 'httpd/unix-directory')
+				{
+					dst = this.dirname(dst);
+				}
+			}
 		}
 
-		this._do_action(_action.id == "file_drop_move" ? 'move' : 'copy', src, false, dst);
+		this._do_action(_action.id.replace("file_drop_",''), src, false, dst || nm_dst);
 	},
 
 	/**
@@ -599,12 +843,17 @@ app.classes.filemanager = AppJS.extend(
 		var widget = this.et2.getWidgetById('upload');
 
 		// Override finish to specify a potentially different path
+		var old_onfinishone = widget.options.onFinishOne;
 		var old_onfinish = widget.options.onFinish;
 
-		widget.options.onFinish = function(_event, _file_count) {
-			widget.options.onFinish = old_onfinish;
+		widget.options.onFinishOne = function(_event, _file_count) {
 			self.upload(_event, _file_count, path);
 		};
+
+		widget.options.onFinish = function() {
+			widget.options.onFinish = old_onfinish;
+			widget.options.onFinishOne = old_onfinishone;
+		}
 		// This triggers the upload
 		widget.set_value(files);
 
@@ -628,23 +877,26 @@ app.classes.filemanager = AppJS.extend(
 			this.readonly = [_path, _ro];
 			return;
 		}
-		var path =  this.path_widget.getValue();
-
-		if (_path == path)
+		for(var id in this.path_widget)
 		{
-			var ids = ['button[linkpaste]', 'button[paste]', 'button[createdir]', 'button[symlink]', 'upload'];
-			for(var i=0; i < ids.length; ++i)
+			var path = this.get_path(id);
+
+			if (_path == path)
 			{
-				var widget = this.et2.getWidgetById(ids[i]);
-				if (widget)
+				var ids = ['button[linkpaste]', 'button[paste]', 'button[createdir]', 'button[symlink]', 'upload'];
+				for(var i=0; i < ids.length; ++i)
 				{
-					if (widget._type == 'button' || widget._type == 'buttononly')
+					var widget = etemplate2.getById(id).widgetContainer.getWidgetById(ids[i]);
+					if (widget)
 					{
-						widget.set_readonly(_ro);
-					}
-					else
-					{
-						widget.set_disabled(_ro);
+						if (widget._type == 'button' || widget._type == 'buttononly')
+						{
+							widget.set_readonly(_ro);
+						}
+						else
+						{
+							widget.set_disabled(_ro);
+						}
 					}
 				}
 			}
@@ -724,7 +976,7 @@ app.classes.filemanager = AppJS.extend(
 					widget.set_label('Superuser');
 					widget.onclick = function(){
 						jQuery('.superuser').css('display','inline');
-					}
+					};
 			}
 		}
 	}
