@@ -5,13 +5,13 @@
  * @link http://www.egroupware.org
  * @package home
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: setup.inc.php 47348 2014-06-24 10:00:50Z ralfbecker $
+ * @version $Id$
  */
 
 /* Basic information about this app */
 $setup_info['home']['name']      = 'home';
 $setup_info['home']['title']     = 'Home';
-$setup_info['home']['version']   = '14.1';
+$setup_info['home']['version']   = '14.1.001';
 $setup_info['home']['app_order'] = 1;
 $setup_info['home']['enable']    = 1;
 $setup_info['home']['index']    = 'home.home_ui.index&ajax=true';
@@ -25,8 +25,7 @@ $setup_info['home']['maintainer'] = array(
 );
 
 /* The hooks this app includes, needed for hooks registration */
-$setup_info['home']['hooks']['hasUpdates'] = 'home.updates.hasUpdates';
-$setup_info['home']['hooks']['showUpdates'] = 'home.updates.showUpdates';
+$setup_info['home']['hooks']['sidebox_all'] = 'home_tutorial_ui::tutorial_menu';
 
 /* Dependencies for this app to work */
 $setup_info['home']['depends'][] = array(
