@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Cornelius Weiss <nelius@cwtech.de>
  * @copyright Cornelius Weiss <nelius@cwtech.de>
- * @version $Id: class.importexport_definitions_ui.inc.php 47809 2014-07-21 15:21:08Z nathangray $
+ * @version $Id$
  */
 
 /**
@@ -512,7 +512,7 @@ class importexport_definitions_ui
 		if(is_array($content) &&! $content['edit'])
 		{
 			if(self::_debug) error_log('importexport.wizard->$content '. print_r($content,true));
-			foreach($content as $key => $val) error_log(" $key : ".array2string($val));
+			//foreach($content as $key => $val) error_log(" $key : ".array2string($val));
 			// fetch plugin object
 			if($content['plugin'] && $content['application'])
 			{
@@ -607,7 +607,7 @@ class importexport_definitions_ui
 
 			unset($content['button']);
 			$content['wizard_content'] = $this->wizard_content_template;
-			$this->etpl->exec(self::_appname.'.importexport_definitions_ui.wizard',$content,$sel_options,$readonlys,$preserv);
+			$this->etpl->exec(self::_appname.'.importexport_definitions_ui.wizard',$content,$sel_options,$readonlys,$preserv,2);
 		}
 		else
 		{
