@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Cornelius Weiss <nelius@cwtech.de>
  * @copyright Cornelius Weiss <nelius@cwtech.de>
- * @version $Id: class.importexport_helper_functions.inc.php 48069 2014-08-13 17:36:34Z nathangray $
+ * @version $Id$
  */
 
 /**
@@ -590,12 +590,15 @@ class importexport_helper_functions {
 	{
 		$fields = array();
 		try {
-			if($record_classname == null) $record_classname = $app_name . '_egw_record';
-			if(!class_exists($record_classname)) throw new Exception('Bad class name ' . $record_classname);
-
 			$plugin = is_object($plugin_name) ? $plugin_name : new $plugin_name();
 			$plugin_name = get_class($plugin);
 
+			if($record_classname == null) $record_classname = $plugin::get_egw_record_class();
+			if(!class_exists($record_classname)) throw new Exception('Bad class name ' . $record_classname);
+
+			if($record_classname == null) $record_classname = $plugin::get_egw_record_class();
+			if(!class_exists($record_classname)) throw new Exception('Bad class name ' . $record_classname);
+
 			if(!$wizard_plugin)
 			{
 				$wizard_name = $app_name . '_wizard_' . str_replace($app_name . '_', '', $plugin_name);
@@ -658,7 +661,7 @@ class importexport_helper_functions {
 					$settings['tags'] = true;
 					break;
 				case 'select-account':
-					$settings['rows'] = '5,both';
+					$settings['account_type'] = 'both';
 					$settings['tags'] = true;
 					break;
 				case 'select':
