@@ -7,7 +7,7 @@
  * @package infolog
  * @copyright (c) 2003-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.infolog_hooks.inc.php 48341 2014-08-27 14:38:25Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -99,7 +99,11 @@ class infolog_hooks
 				'infolog list' => egw::link('/index.php',array(
 					'menuaction' => 'infolog.infolog_ui.index',
 					'ajax' => 'true')),
-				'Add' => "javascript:app.infolog.add_link_sidemenu();",
+				array(
+					'text' => lang('Add %1',lang(egw_link::get_registry($appname, 'entry'))),
+					'no_lang' => true,
+					'link' => "javascript:app.infolog.add_link_sidemenu();"
+				),
 				'Placeholders' => egw::link('/index.php','menuaction=infolog.infolog_merge.show_replacements')
 			);
 			display_sidebox($appname,$GLOBALS['egw_info']['apps']['infolog']['title'].' '.lang('Menu'),$file);
@@ -109,13 +113,13 @@ class infolog_hooks
 		{
 			$file = Array(
 				'Site configuration' => egw::link('/index.php',array(
-					'menuaction' => 'infolog.infolog_ui.admin' ), 'admin'),
+					'menuaction' => 'infolog.infolog_ui.admin' )),
 				'Global Categories'  => egw::link('/index.php',array(
 					'menuaction' => 'admin.admin_categories.index',
 					'appname'    => $appname,
 					'global_cats'=> True)),
 				'Custom fields, typ and status' => egw::link('/index.php',array(
-					'menuaction' => 'infolog.infolog_customfields.edit'), 'admin'),
+					'menuaction' => 'infolog.infolog_customfields.index')),
 			);
 			if ($location == 'admin')
 			{
@@ -451,7 +455,7 @@ class infolog_hooks
 				'type'   => 'multiselect',
 				'label'  => 'Which types should the calendar show',
 				'name'   => 'cal_show',
-				'values' => $info->enums['type'],
+				'values' => array(0 => lang('None')) + $info->enums['type'],
 				'help'   => 'Can be used to show further InfoLog types in the calendar or limit it to show eg. only tasks.',
 				'xmlrpc' => True,
 				'admin'  => False,
@@ -558,4 +562,17 @@ class infolog_hooks
 		unset($location);	// not used, but part of hook signature
 		return true;
 	}
+	
+	/**
+	 * Mail integration hook to import mail message contents into an infolog entry
+	 *
+	 * @return array
+	 */
+	public static function mail_import($args)
+	{
+		return array (
+			'menuaction' => 'infolog.infolog_ui.mail_import',
+			'popup' => egw_link::get_registry('infolog', 'edit_popup')
+		);
+	}
 }
