@@ -7,7 +7,7 @@
  * @package infolog
  * @copyright (c) 2003-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.infolog_so.inc.php 48341 2014-08-27 14:38:25Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -712,7 +712,7 @@ class infolog_so
 				if ($val[0] == '#')
 				{
 					$sortbycf = substr($val,1);
-					$val = "cfsortcrit";
+					$val = "cfsortcrit IS NULL,cfsortcrit";
 				}
 				else
 				{
@@ -798,8 +798,8 @@ class infolog_so
 
 		if ((int)$query['cat_id'])
 		{
-			//$filtermethod .= ' AND info_cat='.intval($query['cat_id']).' ';
-			$cats = $GLOBALS['egw']->categories->return_all_children((int)$query['cat_id']);
+			$categories = new categories('','infolog');
+			$cats = $categories->return_all_children((int)$query['cat_id']);
 			$filtermethod .= ' AND info_cat'.(count($cats)>1? ' IN ('.implode(',',$cats).') ' : '='.(int)$query['cat_id']);
 		}
 		$join = $distinct = '';
@@ -828,7 +828,8 @@ class infolog_so
 			if ($this->db->capabilities['like_on_text']) $columns[] = 'info_des';
 
 			$wildcard = $op = null;
-			$search = so_sql::search2criteria($query['search'], $wildcard, $op, null, $columns);
+			$so_sql = new so_sql('infolog', $this->info_table, $this->db);
+			$search = $so_sql->search2criteria($query['search'], $wildcard, $op, null, $columns);
 			$sql_query = 'AND ('.(is_numeric($query['search']) ? 'main.info_id='.(int)$query['search'].' OR ' : '').
 				implode($op, $search) .')';
 
@@ -862,7 +863,18 @@ class infolog_so
 			$info_customfield = '';
 			if ($sortbycf != '')
 			{
-				$info_customfield = ", (SELECT DISTINCT info_extra_value FROM $this->extra_table sub2 WHERE sub2.info_id=main.info_id AND info_extra_name=".$this->db->quote($sortbycf).") AS cfsortcrit ";
+				$sort_col = "(SELECT DISTINCT info_extra_value FROM $this->extra_table sub2 WHERE sub2.info_id=main.info_id AND info_extra_name=".$this->db->quote($sortbycf).")";
+				if (!isset($custom_fields)) $custom_fields = config::get_customfields('infolog');
+				switch($custom_fields[$sortbycf]['type'])
+				{
+					case 'int':
+						$sort_col = $this->db->to_int($sort_col);
+						break;
+					case 'float':
+						$sort_col = $this->db->to_double($sort_col);
+						break;
+				}
+				$info_customfield = ", $sort_col AS cfsortcrit ";
 			}
 			//echo "SELECT $distinct main.* $info_customfield $sql_query $ordermethod"."<br>";
 			do
