@@ -8,7 +8,7 @@
  * @subpackage setup
  * @copyright (c) 2003-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: tables_update.inc.php 47348 2014-06-24 10:00:50Z ralfbecker $
+ * @version $Id$
  */
 
 function infolog_upgrade0_9_11()
@@ -742,3 +742,58 @@ function infolog_upgrade1_9_006()
 {
 	return $GLOBALS['setup_info']['infolog']['currentver'] = '14.1';
 }
+
+function infolog_upgrade14_1()
+{
+	// Restore any hiding infologs
+	$GLOBALS['egw_setup']->db->query("UPDATE egw_infolog SET info_status='not-started' WHERE info_status IS NULL", __LINE__, __FILE__);
+
+	return $GLOBALS['setup_info']['infolog']['currentver'] = '14.2';
+}
+function infolog_upgrade14_2()
+{
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_infolog','info_responsible',array(
+		'type' => 'ascii',
+		'meta' => 'account-commasep',
+		'precision' => '255',
+		'nullable' => False,
+		'default' => '0',
+		'comment' => 'responsible users or groups (multiple)'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_infolog','info_access',array(
+		'type' => 'ascii',
+		'precision' => '10',
+		'default' => 'public',
+		'comment' => 'public or privat'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_infolog','info_confirm',array(
+		'type' => 'ascii',
+		'precision' => '10',
+		'default' => 'not'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_infolog','info_uid',array(
+		'type' => 'ascii',
+		'precision' => '128',
+		'comment' => 'unique id of the infolog-entry'
+	));
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_infolog','caldav_name',array(
+		'type' => 'ascii',
+		'precision' => '128',
+		'comment' => 'name part of CalDAV URL, if specified by client'
+	));
+
+	return $GLOBALS['setup_info']['infolog']['currentver'] = '14.2.001';
+}
+
+
+function infolog_upgrade14_2_001()
+{
+	$GLOBALS['egw_setup']->oProc->AlterColumn('egw_infolog_extra','info_extra_value',array(
+		'type' => 'varchar',
+		'meta' => 'cfvalue',
+		'precision' => '16384',
+		'nullable' => False
+	));
+
+	return $GLOBALS['setup_info']['infolog']['currentver'] = '14.3';
+}
