@@ -8,7 +8,7 @@
  * @author Ralf Becker <rb@stylite.de>
  * @author Nathan Gray <ng@stylite.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.jdots_framework.inc.php 50889 2014-10-14 07:56:11Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -48,7 +48,7 @@ class jdots_framework extends egw_framework
 	/**
 	 * Constructor
 	 *
-	 * @param string $template='idots' name of the template
+	 * @param string $template = 'idots' name of the template
 	 */
 	function __construct($template=self::APP)
 	{
@@ -68,7 +68,7 @@ class jdots_framework extends egw_framework
 	 */
 	public static function is_supported_user_agent()
 	{
-		if (html::$ua_mobile || html::$user_agent == 'msie' && html::$ua_version < 7)
+		if (html::$user_agent == 'msie' && html::$ua_version < 7)
 		{
 			return false;
 		}
@@ -152,9 +152,9 @@ class jdots_framework extends egw_framework
 	/**
 	 * Link url generator
 	 *
-	 * @param string	$string	The url the link is for
+	 * @param string $url The url the link is for
 	 * @param string|array	$extravars	Extra params to be passed to the url
-	 * @param string $link_app=null if appname or true, some templates generate a special link-handler url
+	 * @param string $link_app = null if appname or true, some templates generate a special link-handler url
 	 * @return string	The full url after processing
 	 */
 	static function link($url = '', $extravars = '', $link_app=null)
@@ -244,7 +244,7 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 	/**
 	 * Returns the html-header incl. the opening body tag
 	 *
-	 * @param array $extra=array() extra attributes passed as data-attribute to egw.js
+	 * @param array $extra = array() extra attributes passed as data-attribute to egw.js
 	 * @return string with html
 	 */
 	function header(array $extra=array())
@@ -263,8 +263,15 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 		// the instanciation of the template has to be here and not in the constructor,
 		// as the old Template class has problems if restored from the session (php-restore)
 		// todo: check if this is still true
-		$this->tpl = new Template(common::get_tpl_dir(static::APP));
-		$this->tpl->set_file(array('_head' => 'head.tpl'));
+		$this->tpl = new Template(EGW_SERVER_ROOT.$this->template_dir);
+		if (html::$ua_mobile || $GLOBALS['egw_info']['user']['preferences']['common']['theme'] == 'mobile')
+		{
+			$this->tpl->set_file(array('_head' => 'head_mobile.tpl'));
+		}
+		else
+		{
+			$this->tpl->set_file(array('_head' => 'head.tpl'));
+		}
 		$this->tpl->set_block('_head','head');
 		$this->tpl->set_block('_head','framework');
 
@@ -272,8 +279,10 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 		$do_framework = isset($_GET['cd']) && $_GET['cd'] === 'yes';
 
 		// load clientside link registry to framework only
-		$GLOBALS['egw_info']['flags']['js_link_registry'] = $do_framework;
-
+		if (!isset($GLOBALS['egw_info']['flags']['js_link_registry']))
+		{
+			$GLOBALS['egw_info']['flags']['js_link_registry'] = $do_framework;
+		}
 		// Loader
 		$this->tpl->set_var('loader_text', lang('please wait...'));
 
@@ -282,8 +291,17 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 			//echo __METHOD__.__LINE__.' do framework ...'.'<br>';
 			// framework javascript classes only need for framework
 			self::validate_file('jquery','jquery-ui');
-			self::validate_file('.', 'egw_fw', self::JS_INCLUDE_APP);
-			self::validate_file('.', 'egw_fw_ui', self::JS_INCLUDE_APP);
+			self::validate_file('framework', 'fw', self::JS_INCLUDE_APP);
+			self::validate_file('framework', 'fw_browser', self::JS_INCLUDE_APP);
+			self::validate_file('framework', 'fw_ui', self::JS_INCLUDE_APP);
+			if (html::$ua_mobile || $GLOBALS['egw_info']['user']['preferences']['common']['theme'] == 'mobile')
+			{
+				self::validate_file('.', 'fw_mobile', self::JS_INCLUDE_APP);
+			}
+			else
+			{
+				self::validate_file('.', 'fw_'.static::APP, static::JS_INCLUDE_APP);
+			}
 			self::validate_file('.', 'egw_fw_classes', self::JS_INCLUDE_APP);
 			self::validate_file('.','etemplate2','etemplate');
 
@@ -342,12 +360,14 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 		$this->tpl->set_var('sidebox_width', self::get_global_sidebar_width());
 		$this->tpl->set_var('sidebox_min_width', self::MIN_SIDEBAR_WIDTH);
 
-		// logout button
-		$this->tpl->set_var('title_logout', lang("Logout"));
-		$this->tpl->set_var('link_logout', egw::link('/logout.php'));
-
-		//Print button title
-		$this->tpl->set_var('title_print', lang("Print current view"));
+		if (!(html::$ua_mobile || $GLOBALS['egw_info']['user']['preferences']['common']['theme'] == 'mobile'))
+		{
+			// logout button
+			$this->tpl->set_var('title_logout', lang("Logout"));
+			$this->tpl->set_var('link_logout', egw::link('/logout.php'));
+			//Print button title
+			$this->tpl->set_var('title_print', lang("Print current view"));
+		}
 
 		// add framework div's
 		$this->tpl->set_var($this->_get_footer());
@@ -418,7 +438,15 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 		switch($app_data['name'])
 		{
 			case 'logout':
-				return;	// no need for logout in topmenu on jdots
+				if (html::$ua_mobile || $GLOBALS['egw_info']['user']['preferences']['common']['theme'] == 'mobile')
+				{
+
+				}
+				else
+				{
+					return;	// no need for logout in topmenu on jdots
+				}
+				break;
 
 			case 'manual':
 				$app_data['url'] = "javascript:callManual();";
@@ -433,15 +461,16 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 				}
 		}
 		$id = $app_data['id'] ? $app_data['id'] : ($app_data['name'] ? $app_data['name'] : $app_data['title']);
-		$this->topmenu_items[] = '<a id="topmenu_' . $id . '" href="'.htmlspecialchars($app_data['url']).'">'.
-			htmlspecialchars($alt_label ? $alt_label : $app_data['title']).'</a>';
+		$title = html::$ua_mobile || $GLOBALS['egw_info']['user']['preferences']['common']['theme'] == 'mobile'
+			? '' : htmlspecialchars($alt_label ? $alt_label : $app_data['title']);
+		$this->topmenu_items[] = '<a id="topmenu_' . $id . '" href="'.htmlspecialchars($app_data['url']).'" title="'.$app_data['title'].'">'.$title.'</a>';
 	}
 
 	/**
 	 * Add info items to the topmenu template class to be displayed
 	 *
 	 * @param string $content html of item
-	 * @param string $id=null
+	 * @param string $id = null
 	 * @access protected
 	 * @return void
 	 */
@@ -578,7 +607,7 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 	 * @param string $appname
 	 * @param string $menu_title
 	 * @param array $file
-	 * @param string $type=null 'admin', 'preferences', 'favorites', ...
+	 * @param string $type = null 'admin', 'preferences', 'favorites', ...
 	 */
 	public function sidebox($appname,$menu_title,$file,$type=null)
 	{
@@ -905,11 +934,6 @@ div .egw_fw_ui_sidemenu_entry_content > div {
 		{
 			$default_app = 'home';
 		}
-// until home works again, we use default_app from prefs (if set and not home) or calendar, if allowed, or first app found
-if ($default_app == 'home')
-{
-	$default_app = isset($GLOBALS['egw_info']['user']['apps']['calendar']) ? 'calendar' : key($GLOBALS['egw_info']['user']['apps']);
-}
 		if (isset($apps[$default_app]))
 		{
 			$apps[$default_app]['isDefault'] = true;
@@ -928,7 +952,12 @@ if ($default_app == 'home')
 		//whether the $active_tab really exists in the $apps array.
 		if ($active_tab && array_key_exists($active_tab, $apps))
 		{
-			$apps[$active_tab]['openOnce'] = preg_replace('/[&?]cd=yes/','',$url);
+			// Do not remove cd=yes if it's an ajax=true app
+			if (strpos( $apps[$active_tab]['url'],'ajax=true') !== False)
+			{
+				$url = preg_replace('/[&?]cd=yes/','',$url);
+			}
+			$apps[$active_tab]['openOnce'] = $url;
 			$store_prefs = true;
 		}
 		else
@@ -981,7 +1010,7 @@ if ($default_app == 'home')
 	/**
 	 * Returns the html from the closing div of the main application area to the closing html-tag
 	 *
-	 * @param boolean $no_framework=true
+	 * @param boolean $no_framework = true
 	 * @return string
 	 */
 	function footer($no_framework=true)
