@@ -7,7 +7,7 @@
  * @author Stylite AG [info@stylite.de]
  * @copyright (c) 2013 by Stylite AG <info-AT-stylite.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.mail_hooks.inc.php 51555 2015-02-03 12:34:48Z leithoff $
+ * @version $Id$
  */
 
 /**
@@ -93,7 +93,10 @@ class mail_hooks
 				'message/rfc822' => array(
 					'menuaction' => 'mail.mail_ui.importMessageFromVFS2DraftAndDisplay',
 					'mime_url'   => 'formData[file]',
+					'mime_data'  => 'formData[data]',
+					'formData[type]' => 'message/rfc822',
 					'mime_popup' => '870xavailHeight',
+					'mime_target' => '_blank'
 				),
 			),
 			'entry' => 'Mail',
@@ -337,39 +340,6 @@ class mail_hooks
 				'default'=> 0,
 				'admin'  => False,
 			),
-/*
-			// option requested by customer, removed for the new client
-			'prefaskformultipleforward' => array(
-				'type'   => 'select',
-				'label'  => 'Confirm attach message',
-				'help'  => 'Do you want to be asked for confirmation before attaching selected messages to new mail?',
-				'name'   => 'prefaskformultipleforward',
-				'values' => $no_yes,
-				'xmlrpc' => True,
-				'admin'  => False,
-				'forced' => '1',
-			),
-*/
-/*
-			'mainscreen_showmail' => array(
-				'type'   => 'select',
-				'label'  => 'show new messages on home page',
-				'help'   => 'Should new messages show up on the Home page',
-				'name'   => 'mainscreen_showmail',
-				'values' => $selectOptions,
-				'xmlrpc' => True,
-				'admin'  => False,
-			),
-			'mainscreen_showfolders' => array(
-				'type'   => 'multiselect',
-				'label'  => 'home page folders',
-				'help'   => 'if shown, which folders should appear on the Home page',
-				'name'   => 'mainscreen_showfolders',
-				'values' => $folderList,
-				'xmlrpc' => True,
-				'admin'  => False,
-			),
-*/
 			array(
 				'type'  => 'section',
 				'title' => lang('Configuration settings'),
@@ -417,57 +387,6 @@ class mail_hooks
 				'default'=> 0,
 				'admin'  => False,
 			),
-			/*'prefpreventmanagefolders' => array(
-				'type'   => 'select',
-				'label'  => 'Prevent managing folders',
-				'help'   => 'Do you want to prevent the managing of folders (creation, accessrights AND subscribtion)?',
-				'name'   => 'prefpreventmanagefolders',
-				'values' => $prefAllowManageFolders,
-				'xmlrpc' => True,
-				'admin'  => False,
-				'forced' => '0',
-			),
-			'prefpreventforwarding' => array(
-				'type'   => 'select',
-				'label'  => 'Prevent managing forwards',
-				'help'   => 'Do you want to prevent the editing/setup for forwarding of mails via settings (, even if SIEVE is enabled)?',
-				'name'   => 'prefpreventforwarding',
-				'values' => $no_yes,
-				'xmlrpc' => True,
-				'admin'  => False,
-				'forced' => '0',
-			),
-			'prefpreventnotificationformailviaemail' => array(
-				'type'   => 'select',
-				'label'  => 'Prevent managing notifications',
-				'help'   => 'Do you want to prevent the editing/setup of notification by mail to other emailadresses if emails arrive (, even if SIEVE is enabled)?',
-				'name'   => 'prefpreventnotificationformailviaemail',
-				'values' => $no_yes,
-				'xmlrpc' => True,
-				'admin'  => False,
-				'forced' => '1',
-			),
-			'prefpreventeditfilterrules' => array(
-				'type'   => 'select',
-				'label'  => 'Prevent managing filters',
-				'help'   => 'Do you want to prevent the editing/setup of filter rules (, even if SIEVE is enabled)?',
-				'name'   => 'prefpreventeditfilterrules',
-				'values' => $no_yes,
-				'xmlrpc' => True,
-				'admin'  => False,
-				'forced' => '0',
-			),
-			'prefpreventabsentnotice' => array(
-				'type'   => 'select',
-				'label'  => 'Prevent managing vacation notice',
-				'help'   => 'Do you want to prevent the editing/setup of the absent/vacation notice (, even if SIEVE is enabled)?',
-				'name'   => 'prefpreventabsentnotice',
-				'values' => $no_yes,
-				'xmlrpc' => True,
-				'admin'  => False,
-				'forced' => '0',
-			),
-			*/
 			'prefaskformove' => array(
 				'type'   => 'select',
 				'label'  => 'Confirm move to folder',
@@ -489,6 +408,17 @@ class mail_hooks
 				'admin'  => False,
 				'default'=> 'text',
 			),
+			'previewPane' => array(
+				'type' => 'select',
+				'label' => 'Preview pane',
+				'help' => 'Show/Hide preview pane in mail list view',
+				'name' => 'previewPane',
+				'values' => array(
+					'0' => lang('show'),
+					'1' => lang('hide')
+				),
+				'default' => '0'
+			),
 		);
 		if (!$GLOBALS['egw_info']['apps']['stylite']) unset($settingsArray['attachVCardAtCompose']);
 		return $settingsArray;
@@ -574,19 +504,15 @@ class mail_hooks
 			'link'=>false,
 			'icon' => false
 		);
-		$showMainScreenStuff = false;
-		// import Message link - only when the required library is available
-		if ((@include_once 'Mail/mimeDecode.php') !== false)
-		{
-			$linkData = array(
-				'menuaction' => 'mail.mail_ui.importMessage',
-			);
+		
+		$linkData = array(
+			'menuaction' => 'mail.mail_ui.importMessage',
+		);
 
-			$file += array(
-				'import message' => "javascript:egw_openWindowCentered2('".egw::link('/index.php', $linkData,false)."','importMessageDialog',600,100,'no','$appname');",
-			);
+		$file += array(
+			'import message' => "javascript:egw_openWindowCentered2('".egw::link('/index.php', $linkData,false)."','importMessageDialog',600,100,'no','$appname');",
+		);
 
-		}
 
 		// create account wizard
 		if (self::access('createaccount'))
@@ -788,8 +714,12 @@ class mail_hooks
 		//error_log(__METHOD__.__LINE__.' '.$feature.':'.array2string($config['deny_'.$feature]));
 		if (!empty($config['deny_'.$feature]))
 		{
-			$denied_groups = explode(',', $config['deny_'.$feature]);
-			return array_intersect($denied_groups, $GLOBALS['egw']->accounts->memberships($GLOBALS['egw_info']['user']['account_id'], true));
+			//error_log(__METHOD__.__LINE__.' feature:'.$feature.':'.array2string($config['deny_'.$feature]));
+			$denied_groups = (is_array($config['deny_'.$feature])?$config['deny_'.$feature]:explode(',', $config['deny_'.$feature]));
+			//error_log(__METHOD__.__LINE__.array2string($GLOBALS['egw']->accounts->memberships($GLOBALS['egw_info']['user']['account_id'], true)));
+			//error_log(__METHOD__.__LINE__.array2string(array_intersect($denied_groups, $GLOBALS['egw']->accounts->memberships($GLOBALS['egw_info']['user']['account_id'], true))));
+			// since access asks positively, the stored deny_$feature must return false if we find the denied group in the users membership-list
+			return (array_intersect($denied_groups, $GLOBALS['egw']->accounts->memberships($GLOBALS['egw_info']['user']['account_id'], true))?false:true);
 		}
 		return true;
 	}
