@@ -8,13 +8,13 @@
  * @author Alejandro Pedraza <alpeb(at)users.sourceforge.net>
  * @package phpbrain
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.uikb.inc.php 51484 2015-01-29 15:21:15Z leithoff $
+ * @version $Id$
  */
 
 /**
 * Presentation layer of the Knowledge Base
 *
-* Last Editor:	$Author: leithoff $
+* Last Editor:	$Author$
 */
 require_once(EGW_INCLUDE_ROOT.'/phpbrain/inc/class.bokb.inc.php');
 
@@ -520,6 +520,30 @@ class uikb extends bokb
 		}
 	}
 
+	static function transform_url2link($matches)
+	{
+		//error_log(__METHOD__.__LINE__.array2string($matches));
+		$webserverURL   = $GLOBALS['egw_info']['server']['webserver_url'];
+		$fullWebServerUrl = (substr(trim($webserverURL),0,1) == '/'?($GLOBALS['egw_info']['server']['enforce_ssl'] || $_SERVER['HTTPS'] ? 'https://' : 'http://').
+			($GLOBALS['egw_info']['server']['hostname'] ? $GLOBALS['egw_info']['server']['hostname'] : $_SERVER['HTTP_HOST']):$webserverURL);
+
+		$linkTextislink = false;
+		// this is the actual url
+		$matches[2] = trim(strip_tags($matches[2],'<img>'));
+		if ($matches[1]===$matches[2]) $linkTextislink = true;
+		$matches[1] = str_replace(' ','%20',$matches[1]);
+//error_log(__METHOD__.__LINE__.array2string($matches[1]));
+//error_log(__METHOD__.__LINE__.array2string($matches[2]));
+		//return ($linkTextislink?' ':'[ ').$matches[1].($linkTextislink?'':' -> '.$matches[2]).($linkTextislink?' ':' ]');
+		return '<a target="'.((stripos($matches[1],$webserverURL) !== false || stripos($matches[1],$fullWebServerUrl) !== false || substr(trim($matches[1]),0,1) == '/')?'_top':'_blank').'" href="'.$matches[1].'">'.($linkTextislink?$matches[1]:$matches[2]).'</a>';
+	}
+
+	function parseHREF (&$body) {
+		$body = preg_replace_callback('~<a[^>]+href=\"([^"]+)\"[^>]*>(.*?)</a>~si','self::transform_url2link',$body,-1,$counts);
+		return $counts;
+	}
+
+
 	/**
 	* Shows article details
 	*
@@ -691,6 +715,9 @@ class uikb extends bokb
 			$this->t->set_block('view_article', 'edit_btn_block', 'edit_btn');
 			$this->t->set_block('view_article', 'del_btn_block', 'del_btn');
 			$this->t->set_block('view_article', 'publish_btn_block', 'publish_btn');
+			$this->t->set_block('view_article', 'edit_btn_block2', 'edit_btn2');
+			$this->t->set_block('view_article', 'del_btn_block2', 'del_btn2');
+			$this->t->set_block('view_article', 'publish_btn_block2', 'publish_btn2');
 			$this->t->set_block('view_article', 'history_line_block', 'history_line');
 
 			if (!$this->sitemgr)
@@ -769,7 +796,7 @@ class uikb extends bokb
 			'img_src_del'			=> common::image('phpbrain', 'delete'),
 			'img_mail'				=> isset($ma)?common::image('phpbrain', 'mail'):'',
 			'alt_mail'				=> isset($ma)?lang('Mail article'):'',
-			'href_mail'				=> isset($ma)?$this->link('menuaction='.$ma['menuaction'].'&app=phpbrain&method=fetch&id='. $article_id):'',
+			'href_mail'				=> isset($ma)?$this->link('menuaction='.$ma['menuaction'].'&mimeType=html&app=phpbrain&method=fetch&id='. $article_id):'',
 			'form_article_action'	=> $this->link('menuaction=phpbrain.uikb.view_article&art_id=' . $article_id),
 			'form_del_action'		=> $this->link('menuaction=phpbrain.uikb.view_article&art_id=' . $article_id),
 			'url_bluedot'			=> common::image('phpbrain', 'bluedot')
@@ -798,10 +825,12 @@ class uikb extends bokb
 					'lang_edit_art'			=> lang('Edit article'),
 				));
 				$this->t->parse('edit_btn', 'edit_btn_block');
+				$this->t->parse('edit_btn2', 'edit_btn_block2');
 			}
 			else
 			{
 				$this->t->set_var('edit_btn', '');
+				$this->t->set_var('edit_btn2', '');
 			}
 			if($can_delete)
 			{
@@ -810,16 +839,20 @@ class uikb extends bokb
 					'lang_delete_article'	=> lang('Delete article')
 				));
 				$this->t->parse('del_btn', 'del_btn_block');
+				$this->t->parse('del_btn2', 'del_btn_block2');
 			}
 			else
 			{
 				$this->t->set_var('del_btn', '');
+				$this->t->set_var('del_btn2', '');
 			}
 		}
 		else
 		{
 			$this->t->set_var('edit_btn', '');
 			$this->t->set_var('del_btn', '');
+			$this->t->set_var('edit_btn2', '');
+			$this->t->set_var('del_btn2', '');
 		}
 
 		// show publish button if article is unpublish and user has publish rights on owner
@@ -833,13 +866,16 @@ class uikb extends bokb
 				'lang_publish_article'	=> lang('publish')
 			));
 			$this->t->parse('publish_btn', 'publish_btn_block');
+			$this->t->parse('publish_btn2', 'publish_btn_block2');
 		}
 		else
 		{
 			$this->t->set_var('publish_btn', '');
+			$this->t->set_var('publish_btn2', '');
 		}
 
 		$this->path = '';
+		$this->parseHREF($article['text']);
 		$this->t->set_var(array(
 			'art_id'			=> $article['art_id'],
 			'lang_unpublished'	=> $published,
@@ -850,7 +886,7 @@ class uikb extends bokb
 			'keywords'			=> $article['keywords'],
 			'createdby'			=> lang('Created by %1 on %2', $article['username'], common::show_date($article['created'], $GLOBALS['egw_info']['user']['preferences']['common']['dateformat'])),
 			'last_modif'		=> $lastmodif,
-			'content'			=> html::activate_links($article['text'])
+			'content'			=> $article['text'] //html::activate_links($article['text'])
 		));
 
 		$this->t->set_var(array(
