@@ -13,7 +13,7 @@
  * @package api
  * @subpackage accounts
  * @access public
- * @version $Id: class.accounts.inc.php 49146 2014-10-22 17:10:41Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -242,7 +242,7 @@ class accounts
 	 *	or integer group-id for a list of members of that group
 	 * @param $param['start'] int first account to return (returns offset or max_matches entries) or all if not set
 	 * @param $param['order'] string column to sort after, default account_lid if unset
-	 * @param $param['sort'] string 'ASC' or 'DESC', default 'DESC' if not set
+	 * @param $param['sort'] string 'ASC' or 'DESC', default 'ASC' if not set
 	 * @param $param['query'] string to search for, no search if unset or empty
 	 * @param $param['query_type'] string:
 	 *	'all'   - query all fields for containing $param[query]
@@ -335,6 +335,17 @@ class accounts
 		else
 		{
 			$account_search[$serial]['data'] = $this->backend->search($param);
+			if ($param['type'] !== 'accounts')
+			{
+				foreach($account_search[$serial]['data'] as &$account)
+				{
+					// add default description for Admins and Default group
+					if ($account['account_type'] === 'g' && empty($account['account_description']))
+					{
+						self::add_default_group_description($account);
+					}
+				}
+			}
 			$account_search[$serial]['total'] = $this->total = $this->backend->total;
 		}
 		//echo "<p>accounts::search(".array2string(unserialize($serial)).")= returning ".count($account_search[$serial]['data'])." of $this->total entries<pre>".print_r($account_search[$serial]['data'],True)."</pre>\n";
@@ -381,6 +392,7 @@ class accounts
 			case 'groups':
 				$type = $only_own ? 'memberships' : 'groups';
 				break;
+			case 'owngroups':
 			case 'groupmembers':
 			case 'memberships':
 				$type = $options['account_type'];
@@ -429,6 +441,12 @@ class accounts
 
 		$data = self::cache_read($id);
 
+		// add default description for Admins and Default group
+		if ($data['account_type'] === 'g' && empty($data['account_description']))
+		{
+			self::add_default_group_description($data);
+		}
+
 		if ($set_depricated_names && $data)
 		{
 			foreach($this->depricated_names as $name)
@@ -469,6 +487,27 @@ class accounts
 		return json_encode($account);
 	}
 
+	/**
+	 * Add a default description for stock groups: Admins, Default, NoGroup
+	 *
+	 * @param array &$data
+	 */
+	protected static function add_default_group_description(array &$data)
+	{
+		switch($data['account_lid'])
+		{
+			case 'Default':
+				$data['account_description'] = lang('EGroupware all users group, do NOT delete');
+				break;
+			case 'Admins':
+				$data['account_description'] = lang('EGroupware administrators group, do NOT delete');
+				break;
+			case 'NoGroup':
+				$data['account_description'] = lang('EGroupware anonymous users group, do NOT delete');
+				break;
+		}
+	}
+
 	/**
 	 * Saves / adds the data of one account
 	 *
@@ -490,6 +529,11 @@ class accounts
 				}
 			}
 		}
+		// add default description for Admins and Default group
+		if ($data['account_type'] === 'g' && empty($data['account_description']))
+		{
+			self::add_default_group_description($data);
+		}
 		if (($id = $this->backend->save($data)) && $data['account_type'] != 'g')
 		{
 			// if we are not on a pure LDAP system, we have to write the account-date via the contacts class now
@@ -519,7 +563,10 @@ class accounts
 				$this->set_memberships($memberships, $id);	// invalidates cache for account_id and primary group
 			}
 		}
-		self::cache_invalidate($data['account_id']);
+		// as some backends set (group-)members in save, we need to invalidate their members too!
+		$invalidate = isset($data['account_members']) ? $data['account_members'] : array();
+		$invalidate[] = $data['account_id'];
+		self::cache_invalidate($invalidate);
 
 		return $id;
 	}
