@@ -8,7 +8,7 @@
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
  * @package api
  * @subpackage accounts
- * @version $Id: class.accounts_ads.inc.php 49146 2014-10-22 17:10:41Z ralfbecker $
+ * @version $Id$
  */
 
 require_once EGW_API_INC.'/adldap/adLDAP.php';
@@ -578,10 +578,6 @@ class accounts_ads
 	protected function _save_group(array &$data, array $old=null)
 	{
 		//error_log(__METHOD__.'('.array2string($data).', old='.array2string($old).')');
-		if (empty($data['account_description']))
-		{
-			$data['account_description'] = 'EGroupware '.lang('Group').' '.$data['account_lid'];
-		}
 
 		if (!$old)	// new entry
 		{
@@ -592,7 +588,7 @@ class accounts_ads
 			$attributes = array();
 			foreach($new2adldap as $egw => $adldap)
 			{
-				if (isset($data[$egw])) $attributes[$adldap] = $data[$egw];
+				$attributes[$adldap] = (string)$data[$egw];
 			}
 			$attributes['container'] = $this->_get_container();
 
@@ -633,6 +629,10 @@ class accounts_ads
 			{
 				switch($egw)
 				{
+					case 'account_description':
+						$ldap[$adldap] = !empty($data[$egw]) ? $data[$egw] : array();
+						break;
+
 					default:
 						$ldap[$adldap] = $data[$egw];
 						break;
@@ -823,7 +823,7 @@ class accounts_ads
 	 *	or integer group-id for a list of members of that group
 	 * @param $param['start'] int first account to return (returns offset or max_matches entries) or all if not set
 	 * @param $param['order'] string column to sort after, default account_lid if unset
-	 * @param $param['sort'] string 'ASC' or 'DESC', default 'DESC' if not set
+	 * @param $param['sort'] string 'ASC' or 'DESC', default 'ASC' if not set
 	 * @param $param['query'] string to search for, no search if unset or empty
 	 * @param $param['query_type'] string:
 	 *	'all'   - query all fields for containing $param[query]
@@ -864,7 +864,7 @@ class accounts_ads
 			$query = ldap::quote(strtolower($param['query']));
 
 			$accounts = array();
-			if($param['type'] != 'groups')
+			if($param['type'] !== 'groups')
 			{
 				if (!empty($query) && $query != '*')
 				{
@@ -894,6 +894,11 @@ class accounts_ads
 							break;
 					}
 				}
+				if (is_numeric($param['type']))
+				{
+					$membership_filter = '(|(memberOf='.$this->id2name((int)$param['type'], 'account_dn').')(PrimaryGroupId='.abs($param['type']).'))';
+					$filter = $filter ? "(&$membership_filter$filter)" : $membership_filter;
+				}
 				foreach($this->filter($filter, 'u', self::$user_attributes) as $account_id => $data)
 				{
 					$account = $this->_ldap2user($data);
@@ -905,7 +910,7 @@ class accounts_ads
 					$accounts[$account_id] = $account;
 				}
 			}
-			if ($param['type'] == 'groups' || $param['type'] == 'both')
+			if ($param['type'] === 'groups' || $param['type'] === 'both')
 			{
 				$query = ldap::quote(strtolower($param['query']));
 
@@ -959,7 +964,7 @@ class accounts_ads
 	 *
 	 * @var string
 	 */
-	private $_callback_sort = 'DESC';
+	private $_callback_sort = 'ASC';
 	/**
 	 * column_names to sort by
 	 *
@@ -978,7 +983,7 @@ class accounts_ads
 	{
 		foreach($this->_callback_order as $col )
 		{
-			if($this->_callback_sort == 'ASC')
+			if($this->_callback_sort != 'DESC')
 			{
 				$cmp = strcasecmp( $a[$col], $b[$col] );
 			}
@@ -1437,6 +1442,80 @@ class adLDAPUsers_egw extends adLDAPUsers
     	return $result;
     }
 
+	/**
+	 * Check if we can to a real password change, not just a password reset
+	 *
+	 * Requires PHP 5.4 >= 5.4.26, PHP 5.5 >= 5.5.10 or PHP 5.6 >= 5.6.0
+	 *
+	 * @return boolean
+	 */
+	public static function changePasswordSupported()
+	{
+		return function_exists('ldap_modify_batch');
+	}
+
+    /**
+    * Set the password of a user - This must be performed over SSL
+    *
+    * @param string $username The username to modify
+    * @param string $password The new password
+    * @param bool $isGUID Is the username passed a GUID or a samAccountName
+	* @param string $old_password old password for password change, if supported
+    * @return bool
+    */
+    public function password($username, $password, $isGUID = false, $old_password=null)
+    {
+        if ($username === NULL) { return false; }
+        if ($password === NULL) { return false; }
+        if (!$this->adldap->getLdapBind()) { return false; }
+        if (!$this->adldap->getUseSSL() && !$this->adldap->getUseTLS()) {
+            throw new adLDAPException('SSL must be configured on your webserver and enabled in the class to set passwords.');
+        }
+
+        $userDn = $this->dn($username, $isGUID);
+        if ($userDn === false) {
+            return false;
+        }
+
+        $add=array();
+
+		if (empty($old_password) || !function_exists('ldap_modify_batch')) {
+			$add["unicodePwd"][0] = $this->encodePassword($password);
+
+			$result = @ldap_mod_replace($this->adldap->getLdapConnection(), $userDn, $add);
+		}
+		else {
+			$mods = array(
+				array(
+					"attrib"  => "unicodePwd",
+					"modtype" => LDAP_MODIFY_BATCH_REMOVE,
+					"values"  => array($this->encodePassword($old_password)),
+				),
+				array(
+					"attrib"  => "unicodePwd",
+					"modtype" => LDAP_MODIFY_BATCH_ADD,
+					"values"  => array($this->encodePassword($password)),
+				),
+			);
+			$result = ldap_modify_batch($this->adldap->getLdapConnection(), $userDn, $mods);
+		}
+        if ($result === false){
+            $err = ldap_errno($this->adldap->getLdapConnection());
+            if ($err) {
+                $msg = 'Error ' . $err . ': ' . ldap_err2str($err) . '.';
+                if($err == 53) {
+                    $msg .= ' Your password might not match the password policy.';
+                }
+                throw new adLDAPException($msg);
+            }
+            else {
+                return false;
+            }
+        }
+
+        return true;
+    }
+
     /**
     * Modify a user
     *
@@ -1515,7 +1594,6 @@ class adLDAPGroups_egw extends adLDAPGroups
 		if (!is_array($attributes)){ return "Attributes must be an array"; }
 		if (!array_key_exists("group_name", $attributes)){ return "Missing compulsory field [group_name]"; }
 		if (!array_key_exists("container", $attributes)){ return "Missing compulsory field [container]"; }
-		if (!array_key_exists("description", $attributes)){ return "Missing compulsory field [description]"; }
 		if (empty($attributes["container"])){ return "Container attribute must be an array or string."; }
 
 		//$member_array = array();
@@ -1526,7 +1604,7 @@ class adLDAPGroups_egw extends adLDAPGroups
 		$add["cn"] = $attributes["group_name"];
 		$add["samaccountname"] = $attributes["group_name"];
 		$add["objectClass"] = "Group";
-		$add["description"] = $attributes["description"];
+		if (!empty($attributes["description"])) $add["description"] = $attributes["description"];
 		//$add["member"] = $member_array; UNTESTED
 
 		// Determine the container
@@ -1554,4 +1632,24 @@ class adLDAPUtils_egw extends adLDAPUtils
 	{
 		return $this->adldap->encode8bit($item, $key);
 	}
+
+    /**
+    * Escape strings for the use in LDAP filters
+    *
+    * DEVELOPERS SHOULD BE DOING PROPER FILTERING IF THEY'RE ACCEPTING USER INPUT
+    * Ported from Perl's Net::LDAP::Util escape_filter_value
+    *
+    * @param string $str The string the parse
+    * @author Port by Andreas Gohr <andi@splitbrain.org>
+    * @return string
+    */
+    public function ldapSlashes($str){
+        return preg_replace_callback(
+      		'/([\x00-\x1F\*\(\)\\\\])/',
+        	function ($matches) {
+            	return "\\".join("", unpack("H2", $matches[1]));
+        	},
+        	$str
+    	);
+    }
 }
