@@ -20,7 +20,7 @@
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
  * @package api
  * @subpackage accounts
- * @version $Id: class.accounts_sql.inc.php 49146 2014-10-22 17:10:41Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -128,7 +128,9 @@ class accounts_sql
 				$this->contacts_table.'.tel_work AS account_phone,';
 				$join = 'LEFT JOIN '.$this->contacts_table.' ON '.$this->table.'.account_id='.$this->contacts_table.'.account_id';
 		}
-		else
+		// during setup emailadmin might not yet be installed and running below query
+		// will abort transaction in PostgreSQL
+		elseif (!isset($GLOBALS['egw_setup']) || in_array(emailadmin_smtp_sql::TABLE, $this->db->table_names(true)))
 		{
 			$extra_cols = emailadmin_smtp_sql::TABLE.'.mail_value AS account_email,';
 			$join = 'LEFT JOIN '.emailadmin_smtp_sql::TABLE.' ON '.$this->table.'.account_id=-'.emailadmin_smtp_sql::TABLE.'.account_id AND mail_type='.emailadmin_smtp_sql::TYPE_ALIAS;
@@ -380,7 +382,7 @@ class accounts_sql
 	 *	or integer group-id for a list of members of that group
 	 * @param $param['start'] int first account to return (returns offset or max_matches entries) or all if not set
 	 * @param $param['order'] string column to sort after, default account_lid if unset
-	 * @param $param['sort'] string 'ASC' or 'DESC', default 'DESC' if not set
+	 * @param $param['sort'] string 'ASC' or 'DESC', default 'ASC' if not set
 	 * @param $param['query'] string to search for, no search if unset or empty
 	 * @param $param['query_type'] string:
 	 *	'all'   - query all fields for containing $param[query]
@@ -414,6 +416,21 @@ class accounts_sql
 		}
 		if ($param['sort']) $order = implode(' '.$param['sort'].',', explode(',', $order)).' '.$param['sort'];
 
+		$search_cols = array('account_lid','n_family','n_given','email');
+		$join = $this->contacts_join;
+		$email_cols = array('email');
+
+		// Add in group email searching
+		if (!isset($GLOBALS['egw_setup']) || in_array(emailadmin_smtp_sql::TABLE, $this->db->table_names(true)))
+		{
+			$email_cols = array('coalesce('.$this->contacts_table.'.contact_email,'.emailadmin_smtp_sql::TABLE.'.mail_value) as email');
+			if ($this->db->Type == 'mysql' && !preg_match('/[\x80-\xFF]/', $param['query']))
+			{
+				$search_cols[] = emailadmin_smtp_sql::TABLE.'.mail_value';
+			}
+			$join .= ' LEFT JOIN '.emailadmin_smtp_sql::TABLE.' ON '.$this->table.'.account_id=-'.emailadmin_smtp_sql::TABLE.'.account_id AND mail_type='.emailadmin_smtp_sql::TYPE_ALIAS;
+		}
+
 		$filter = array();
 		switch($param['type'])
 		{
@@ -478,7 +495,7 @@ class accounts_sql
 				case 'all':
 				default:
 				case 'exact':
-					foreach(array('account_lid','n_family','n_given','email') as $col)
+					foreach($search_cols as $col)
 					{
 						$criteria[$col] = $query;
 					}
@@ -498,6 +515,11 @@ class accounts_sql
 				case 'account_email':
 				case 'email':
 					$criteria['email'] = $query;
+					// Group email
+					if(in_array(emailadmin_smtp_sql::TABLE, $this->db->table_names(true)))
+					{
+						$criteria[emailadmin_smtp_sql::TABLE.'.mail_value'] = $query;
+					}
 					break;
 			}
 		}
@@ -505,11 +527,11 @@ class accounts_sql
 
 		$accounts = array();
 		foreach((array) $GLOBALS['egw']->contacts->search($criteria,
-			"1,n_given,n_family,email,id,created,modified,$this->table.account_id AS account_id",
-			$order,"account_lid,account_type,account_status,account_expires,account_primary_group",
+			array_merge(array(1,'n_given','n_family','id','created','modified',$this->table.'.account_id AS account_id'),$email_cols),
+			$order,"account_lid,account_type,account_status,account_expires,account_primary_group,account_description",
 			$wildcard,false,$query[0] == '!' ? 'AND' : 'OR',
 			$param['offset'] ? array($param['start'], $param['offset']) : (is_null($param['start']) ? false : $param['start']),
-			$filter,$this->contacts_join) as $contact)
+			$filter,$join) as $contact)
 		{
 			if ($contact)
 			{
@@ -528,6 +550,7 @@ class accounts_sql
 					// addressbook_bo::search() returns everything in user-time, need to convert to server-time
 					'account_created'	=> egw_time::user2server($contact['created']),
 					'account_modified'	=> egw_time::user2server($contact['modified']),
+					'account_description' => $contact['account_description'],
 				);
 			}
 		}
@@ -572,6 +595,12 @@ class accounts_sql
 				$table = $this->table;
 				$cols .= ',account_type';
 				$where[$which] = $name;
+				// check if we need to treat username case-insensitive
+				if ($which == 'account_lid' && !$GLOBALS['egw_info']['server']['case_sensitive_username'])	// = is case sensitiv eg. on postgres, but not on mysql!
+				{
+					$where[] = 'account_lid '.$this->db->capabilities[egw_db::CAPABILITY_CASE_INSENSITIV_LIKE].' '.$this->db->quote($where['account_lid']);
+					unset($where['account_lid']);
+				}
 		}
 		if ($account_type)
 		{
