@@ -9,7 +9,7 @@
  * @license http://opensource.org/licenses/lgpl-license.php LGPL - GNU Lesser General Public License
  * @package api
  * @subpackage authentication
- * @version $Id: class.auth.inc.php 46642 2014-04-17 07:37:21Z ralfbecker $
+ * @version $Id$
  */
 
 // allow to set an application depending authentication type (eg. for syncml, groupdav, ...)
@@ -56,26 +56,25 @@ class auth
 	}
 
 	/**
-	 * check_password_age
 	 * check if users are supposed to change their password every x sdays, then check if password is of old age
 	 * or the devil-admin reset the users password and forced the user to change his password on next login.
 	 *
-	 * @param string $app to know where you are/ or where you want to go
-	 * @param string $class to know where you are/ or where you want to go
-	 * @param string $method to know where you are/ or where you want to go
-	 * @return boolean true if check determined, that you passed the test, otherwise void, as we get redirected
+	 * @param string& $message =null on return false: message why password needs to be changed
+	 * @return boolean true: all good, false: password change required, null: password expires in N days
 	 */
-	static function check_password_age($app='', $class='', $method='')
+	static function check_password_change(&$message=null)
 	{
 		// dont check anything for anonymous sessions/ users that are flagged as anonymous
 		if (is_object($GLOBALS['egw']->session) && $GLOBALS['egw']->session->session_flags == 'A') return true;
+
 		// some statics (and initialisation to make information and timecalculation a) more readable in conditions b) persistent per request
 		// if user has to be warned about an upcomming passwordchange, remember for the session, that he was informed
-		static $UserKnowsAboutPwdChange;
+		static $UserKnowsAboutPwdChange=null;
 		if (is_null($UserKnowsAboutPwdChange)) $UserKnowsAboutPwdChange =& egw_cache::getSession('phpgwapi','auth_UserKnowsAboutPwdChange');
+
 		// retrieve the timestamp regarding the last change of the password from auth system and store it with the session
-		static $alpwchange_val;
-		static $pwdTsChecked;
+		static $alpwchange_val=null;
+		static $pwdTsChecked=null;
 		if (is_null($pwdTsChecked) && is_null($alpwchange_val) || (string)$alpwchange_val === '0')
 		{
 			$alpwchange_val =& egw_cache::getSession('phpgwapi','auth_alpwchange_val'); // set that one with the session stored value
@@ -88,7 +87,7 @@ class auth
 				// on the other side, if your auth system does not require an forcedPasswordChange, you will not be asked.
 				if (method_exists($backend,'getLastPwdChange'))
 				{
-					$alpwchange_val = $backend->getLastPwdChange($GLOBALS['egw_info']['user']['account_lid']);
+					$alpwchange_val = $backend->getLastPwdChange($GLOBALS['egw']->session->account_lid);
 					$pwdTsChecked = true;
 				}
 				// if your authsystem does not provide that information, its likely, that you cannot change your password there,
@@ -100,21 +99,15 @@ class auth
 				//error_log(__METHOD__.__LINE__.'#'.$alpwchange_val.'# is null:'.is_null($alpwchange_val).'# is empty:'.empty($alpwchange_val).'# is set:'.isset($alpwchange_val));
 			}
 		}
-		static $passwordAgeBorder;
-		static $daysLeftUntilChangeReq;
-		// some debug output and develop options to move the horizons and warn levels around
-		//$GLOBALS['egw_info']['server']['change_pwd_every_x_days'] =35;
-		//$GLOBALS['egw_info']['server']['warn_about_upcoming_pwd_change']=5;
-		//echo egw_time::to('now','ts').'<br>';
-		//echo "User changed password at:".egw_time::to($GLOBALS['egw_info']['user'][$alpwchange]).'<br>';
-		//echo "User password is ".((egw_time::to('now','ts')-$GLOBALS['egw_info']['user'][$alpwchange])/86400)." days old<br>";
-		//echo "Users must change passwords every ".$GLOBALS['egw_info']['server']['change_pwd_every_x_days'].' days ('.($GLOBALS['egw_info']['server']['change_pwd_every_x_days']*86400).') seconds.<br>';
-		//error_log(__METHOD__.__LINE__.'#'.$alpwchange_val.'# is null:'.is_null($alpwchange_val).'# is empty:'.empty($alpwchange_val).'# is set:'.isset($alpwchange_val));
-		//echo egw_time::to('now','ts')-($GLOBALS['egw_info']['server']['change_pwd_every_x_days']*86400).'<br>';
+		static $passwordAgeBorder=null;
+		static $daysLeftUntilChangeReq=null;
+
 		// if neither timestamp isset return true, nothing to do (exept this means the password is too old)
 		if (is_null($alpwchange_val) &&
-			empty($GLOBALS['egw_info']['server']['change_pwd_every_x_days'])
-		) return true;
+			empty($GLOBALS['egw_info']['server']['change_pwd_every_x_days']))
+		{
+			return true;
+		}
 		if (is_null($passwordAgeBorder) && $GLOBALS['egw_info']['server']['change_pwd_every_x_days'])
 		{
 			$passwordAgeBorder = (egw_time::to('now','ts')-($GLOBALS['egw_info']['server']['change_pwd_every_x_days']*86400));
@@ -124,60 +117,55 @@ class auth
 			// maxage - passwordage = days left until change is required
 			$daysLeftUntilChangeReq = ($GLOBALS['egw_info']['server']['change_pwd_every_x_days'] - ((egw_time::to('now','ts')-($alpwchange_val?$alpwchange_val:0))/86400));
 		}
-		//echo "Warn about the upcomming change ".$GLOBALS['egw_info']['server']['warn_about_upcoming_pwd_change'].' days before that time is reached<br>';
-		//$result = $GLOBALS['egw_info']['server']['change_pwd_every_x_days'] - $daysLeftUntilChangeReq;
-		//echo $GLOBALS['egw_info']['server']['change_pwd_every_x_days'].' - '.$daysLeftUntilChangeReq.'='. $result.'<br>';
-		if  (!($app == 'preferences' && $class == 'preferences_password' && $method == 'change') &&
-			 (
-			  ($GLOBALS['egw_info']['server']['change_pwd_every_x_days'] &&
-			   ($GLOBALS['egw_info']['user']['apps']['preferences'] || $GLOBALS['egw_info']['user']['apps']['password']) &&
-			   (
-				($passwordAgeBorder > $alpwchange_val) ||
-				(
-				 $GLOBALS['egw_info']['server']['warn_about_upcoming_pwd_change'] &&
-				 $GLOBALS['egw_info']['server']['warn_about_upcoming_pwd_change'] > $daysLeftUntilChangeReq
-				)
-			   )
-			  ) || $alpwchange_val==0
-			 )
-			)
+		if ($alpwchange_val == 0 ||	// admin requested password change
+			$passwordAgeBorder > $alpwchange_val ||	// change password every N days policy requests change
+			// user should be warned N days in advance about change and is not yet
+			$GLOBALS['egw_info']['server']['change_pwd_every_x_days'] &&
+			$GLOBALS['egw_info']['user']['apps']['preferences'] &&
+			$GLOBALS['egw_info']['server']['warn_about_upcoming_pwd_change'] &&
+			$GLOBALS['egw_info']['server']['warn_about_upcoming_pwd_change'] > $daysLeftUntilChangeReq &&
+			$UserKnowsAboutPwdChange !== true)
 		{
-			if ($GLOBALS['egw']->acl->check('nopasswordchange', 1, 'preferences')) return true; // user has no rights to change password
-			if ($UserKnowsAboutPwdChange === true && !($passwordAgeBorder > $alpwchange_val || $alpwchange_val==0)) return true; // user has already been informed about the upcomming password expiration
-			if (!is_null($alpwchange_val))
+			if ($alpwchange_val == 0)
 			{
-				if ($alpwchange_val == 0)
-				{
-					$message = lang('an admin required that you must change your password upon login.');
-				}
-				elseif (($passwordAgeBorder < $alpwchange_val) ||
-						(
-						 $GLOBALS['egw_info']['server']['warn_about_upcoming_pwd_change'] &&
-						 $GLOBALS['egw_info']['server']['warn_about_upcoming_pwd_change'] > $daysLeftUntilChangeReq &&
-						 $daysLeftUntilChangeReq > 0
-						)
-					   )
+				$message = lang('An admin required that you must change your password upon login.');
+			}
+			elseif ($passwordAgeBorder > $alpwchange_val && $alpwchange_val > 0)
+			{
+				error_log(__METHOD__.' Password of '.$GLOBALS['egw_info']['user']['account_lid'].' ('.$GLOBALS['egw_info']['user']['account_fullname'].') is of old age.'.array2string(array(
+					'ts'=> $alpwchange_val,
+					'date'=>egw_time::to($alpwchange_val))));
+				$message = lang('It has been more then %1 days since you changed your password',$GLOBALS['egw_info']['server']['change_pwd_every_x_days']);
+			}
+			else
+			{
+				// login page does not inform user about passwords about to expire
+				if ($GLOBALS['egw_info']['flags']['currentapp'] != 'login' &&
+					($GLOBALS['egw_info']['flags']['currentapp'] != 'home' ||
+						strpos($_SERVER['SCRIPT_NAME'], '/home/') !== false))
 				{
 					$UserKnowsAboutPwdChange = true;
-					$message = lang('your password is about to expire in %1 days, you may change your password now',round($daysLeftUntilChangeReq));
 				}
-				elseif ($passwordAgeBorder > $alpwchange_val && $alpwchange_val > 0)
-				{
-					error_log(__METHOD__.' Password of '.$GLOBALS['egw_info']['user']['account_lid'].' ('.$GLOBALS['egw_info']['user']['account_fullname'].') is of old age.'.array2string(array(
-						'ts'=> $alpwchange_val,
-						'date'=>egw_time::to($alpwchange_val))));
-					$message = lang('it has been more then %1 days since you changed your password',$GLOBALS['egw_info']['server']['change_pwd_every_x_days']);
-				}
-				egw::redirect_link('/index.php',array(
-					'menuaction' => 'preferences.preferences_password.change',
-					'message'    => $message,
-					'nopopup'    => true,
-				));
+				$message = lang('Your password is about to expire in %1 days, you may change your password now',round($daysLeftUntilChangeReq));
+				// user has no rights to change password --> do NOT warn, as only forced check ignores rights
+				if ($GLOBALS['egw']->acl->check('nopasswordchange', 1, 'preferences')) return true;
+				return null;
 			}
+			return false;
 		}
 		return true;
 	}
 
+	/**
+	 * Retired password check method called all over the place
+	 *
+	 * @deprecated use check_password_change
+	 */
+	static function check_password_age()
+	{
+		return true;	// no change
+	}
+
 	/**
 	 * fetch the last pwd change for the user
 	 *
@@ -209,7 +197,7 @@ class auth
 	 *
 	 * @param string $username username of account to authenticate
 	 * @param string $passwd corresponding password
-	 * @param string $passwd_type='text' 'text' for cleartext passwords (default)
+	 * @param string $passwd_type ='text' 'text' for cleartext passwords (default)
 	 * @return boolean true if successful authenticated, false otherwise
 	 */
 	function authenticate($username, $passwd, $passwd_type='text')
@@ -235,7 +223,7 @@ class auth
 		}
 		if (($ret = $this->backend->change_password($old_passwd, $new_passwd, $account_id)))
 		{
-			if ($account_id == $GLOBALS['egw_info']['user']['account_id'])
+			if ($account_id == $GLOBALS['egw']->session->account_id)
 			{
 				// need to change current users password in session
 				egw_cache::setSession('phpgwapi', 'password', base64_encode($new_passwd));
@@ -304,9 +292,9 @@ class auth
 	 *
 	 * @param string $cleartext cleartext password
 	 * @param string $encrypted encrypted password, can have a {hash} prefix, which overrides $type
-	 * @param string $type_i type of encryption
+	 * @param string $type_in type of encryption
 	 * @param string $username used as optional key of encryption for md5_hmac
-	 * @param string &$type=null on return detected type of hash
+	 * @param string &$type =null on return detected type of hash
 	 * @return boolean
 	 */
 	static function compare_password($cleartext, $encrypted, $type_in, $username='', &$type=null)
@@ -314,6 +302,7 @@ class auth
 		// allow to specify the hash type to prefix the hash, to easy migrate passwords from ldap
 		$type = $type_in;
 		$saved_enc = $encrypted;
+		$matches = null;
 		if (preg_match('/^\\{([a-z_5]+)\\}(.+)$/i',$encrypted,$matches))
 		{
 			$type = strtolower($matches[1]);
@@ -419,9 +408,9 @@ class auth
 			}
 		}
 
-		$salt = substr($db_val, 0, $len);
-		$new_hash = crypt($form_val, $salt);
-		//error_log(__METHOD__."('$form_val', '$db_val') type=$type --> len=$len --> salt='$salt' --> new_hash='$new_hash' returning ".array2string($db_val === $new_hash));
+		$full_salt = substr($db_val, 0, $len);
+		$new_hash = crypt($form_val, $full_salt);
+		//error_log(__METHOD__."('$form_val', '$db_val') type=$type --> len=$len --> salt='$full_salt' --> new_hash='$new_hash' returning ".array2string($db_val === $new_hash));
 
 		return $db_val === $new_hash;
 	}
@@ -431,8 +420,8 @@ class auth
 	 *
 	 * uses the encryption type set in setup and calls the appropriate encryption functions
 	 *
-	 * @param $password password to encrypt
-	 * @param $type=null default to $GLOBALS['egw_info']['server']['ldap_encryption_type']
+	 * @param string $password password to encrypt
+	 * @param string $type =null default to $GLOBALS['egw_info']['server']['ldap_encryption_type']
 	 * @return string
 	 */
 	static function encrypt_ldap($password, $type=null)
@@ -495,7 +484,7 @@ class auth
 	 * Create a password for storage in the accounts table
 	 *
 	 * @param string $password
-	 * @param string $type=null default $GLOBALS['egw_info']['server']['sql_encryption_type']
+	 * @param string $type =null default $GLOBALS['egw_info']['server']['sql_encryption_type']
 	 * @return string hash
 	 */
 	static function encrypt_sql($password, $type=null)
@@ -545,7 +534,7 @@ class auth
 	/**
 	 * Get available password hashes sorted by securest first
 	 *
-	 * @param string &$securest=null on return securest available hash
+	 * @param string &$securest =null on return securest available hash
 	 * @return array hash => label
 	 */
 	public static function passwdhashes(&$securest=null)
@@ -603,12 +592,12 @@ class auth
 	 *
 	 * Windows compatible check is $reqstrength=3, $minlength=7, $forbid_name=true
 	 *
-	 * @param string $password
-	 * @param int $reqstrength=null defaults to whatever set in config for "force_pwd_strength"
-	 * @param int $minlength=null defaults to whatever set in config for "check_save_passwd"
-	 * @param string $forbid_name=null if "yes" username or full-name split by delimiters AND longer then 3 chars are
+	 * @param string $passwd
+	 * @param int $reqstrength =null defaults to whatever set in config for "force_pwd_strength"
+	 * @param int $minlength =null defaults to whatever set in config for "check_save_passwd"
+	 * @param string $forbid_name =null if "yes" username or full-name split by delimiters AND longer then 3 chars are
 	 *  forbidden to be included in password, default to whatever set in config for "passwd_forbid_name"
-	 * @param array|int $account=null array with account_lid and account_fullname or account_id for $forbid_name check
+	 * @param array|int $account =null array with account_lid and account_fullname or account_id for $forbid_name check
 	 * @return mixed false if password is considered "safe" (or no requirements) or a string $message if "unsafe"
 	 */
 	static function crackcheck($passwd, $reqstrength=null, $minlength=null, $forbid_name=null, $account=null)
@@ -756,12 +745,12 @@ class auth
 	 *
 	 * @param string $form_val user input value for comparison
 	 * @param string $db_val   stored value (from database)
-	 * @param string $key       key for md5_hmac-encryption (username for imported smf users)
+	 * @param string $_key       key for md5_hmac-encryption (username for imported smf users)
 	 * @return boolean	 True on successful comparison
 	 */
-	static function md5_hmac_compare($form_val,$db_val,$key)
+	static function md5_hmac_compare($form_val,$db_val,$_key)
 	{
-		$key = str_pad(strlen($key) <= 64 ? $key : pack('H*', md5($key)), 64, chr(0x00));
+		$key = str_pad(strlen($_key) <= 64 ? $_key : pack('H*', md5($_key)), 64, chr(0x00));
 		$md5_hmac = md5(($key ^ str_repeat(chr(0x5c), 64)) . pack('H*', md5(($key ^ str_repeat(chr(0x36), 64)). $form_val)));
 
 		return strcmp($md5_hmac,$db_val) == 0;
@@ -778,7 +767,7 @@ interface auth_backend
 	 *
 	 * @param string $username username of account to authenticate
 	 * @param string $passwd corresponding password
-	 * @param string $passwd_type='text' 'text' for cleartext passwords (default)
+	 * @param string $passwd_type ='text' 'text' for cleartext passwords (default)
 	 * @return boolean true if successful authenticated, false otherwise
 	 */
 	function authenticate($username, $passwd, $passwd_type='text');
