@@ -14,7 +14,7 @@
  * @package api
  * @subpackage categories
  * @access public
- * @version $Id: class.categories.inc.php 47417 2014-06-27 11:36:18Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -288,7 +288,7 @@ class categories
 		if (!($this->total_records = count($cats)))
 		{
 			//error_log(__METHOD__."($type,$start,$limit,$query,$sort,$order,$globals,parent=$parent_id,$lastmod,$column) account_id=$this->account_id, appname=$this->app_name = FALSE");
-			return false;
+			return array();
 		}
 		if (!$sort) $sort = 'ASC';
 		// order the entries if necessary (cache is already ordered in or default order: cat_main, cat_level, cat_name ASC)
@@ -1022,6 +1022,27 @@ class categories
 		}
 	}
 
+	/**
+	 * Have the framework include the CSS file
+	 *
+	 * We calculate the cachebuster timestamp from the last modified category in
+	 * the application.
+	 *
+	 * @param string appname
+	 *
+	 * @return Returns the URL, but you do not need to do anything with it.
+	 */
+	public static function css($appname)
+	{
+		$cats = new categories('',$appname);
+		$last_mod = $cats->return_array('all',0,1,'','DESC','last_mod', $appname == self::GLOBAL_APPNAME);
+		$time = count($last_mod) ? $last_mod[0]['last_mod'] : time();
+		$path = '/phpgwapi/categories.php?app='.$appname.'&'.$time;
+		egw_framework::includeCSS($path);
+
+		return $path;
+	}
+
 	/**
 	 * Delete categories belonging to a given account, when account got deleted
 	 *
