@@ -8,7 +8,7 @@
  * @subpackage db
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright (c) 2003-15 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @version $Id: class.db_backup.inc.php 51623 2015-02-06 14:38:50Z leithoff $
+ * @version $Id$
  */
 
 /**
@@ -368,7 +368,18 @@ class db_backup
 				$system_config[] = $row;
 			}
 		}
-		if (substr($this->db->Type,0,5) != 'mysql') $this->db->transaction_begin();
+
+		// as MySQL 5.7+ has sql_mode STRICT_(ALL|TRANS)_TABLES enabled by default,
+		// it will no longer restore '0000-00-00 00:00:00' in timestamps it created before,
+		// so switching strict-mode off temporary for the restore (we dont create these!)
+		if (substr($this->db->Type,0,5) == 'mysql')
+		{
+			$this->db->query("SET SESSION sql_mode=(SELECT REPLACE(REPLACE(@@sql_mode,'STRICT_ALL_TABLES',''),'STRICT_TRANS_TABLES',''))", __LINE__, __FILE__);
+		}
+		else
+		{
+			$this->db->transaction_begin();
+		}
 
 		// drop all existing tables
 		foreach($this->adodb->MetaTables('TABLES') as $table)
@@ -534,7 +545,8 @@ class db_backup
 			{
 				$charset = trim(substr($line,9));
 				// needed if mbstring.func_overload > 0, else eg. substr does not work with non ascii chars
-				@ini_set('mbstring.internal_encoding',$charset);
+				$ini_default_charset = version_compare(PHP_VERSION, '5.6', '<') ? 'mbstring.internal_encoding' : 'default_charset';
+				@ini_set($ini_default_charset, $charset);
 
 				// check if we really need to convert the charset, as it's not perfect and can do some damage
 				if ($convert_to_system_charset && !strcasecmp($this->schema_proc->system_charset, $charset))
@@ -687,6 +699,10 @@ class db_backup
 				}
 			}
 		}
+
+		// check if backup contained all indexes and create missing ones
+		$this->schema_proc->CheckCreateIndexes();
+
 		return $n;
 	}
 
@@ -944,7 +960,7 @@ class db_backup
 					empty($pk) || !$max ? false : $pk.' > '.$this->db->quote($max, $schema['fd'][$pk]['type']),
 					__LINE__, __FILE__,
 					empty($pk) ? false : 0,					// if no primary key, query all rows
-					empty($pk) ? '' : 'ORDER BY '.$pk.' ASC',	// order by primary key
+					empty($pk) ? '' : 'ORDER BY '.$this->db->name_quote($pk).' ASC',	// order by primary key
 					false, self::ROW_CHUNK) as $row)
 				{
 					if (!empty($pk)) $max = $row[$pk];
