@@ -9,7 +9,7 @@
  * @author	RalfBecker@outdoor-training.de
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
  * @package api
- * @version $Id: class.egw.inc.php 51001 2014-12-13 09:54:14Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -127,7 +127,7 @@ class egw extends egw_minimal
 		{
 			$GLOBALS['egw_info']['server'][$row['config_name']] = $row['config_value'];
 		}
-		if ($GLOBALS['egw_info']['server']['system_charset'])
+		if ($GLOBALS['egw_info']['server']['system_charset'] && $GLOBALS['egw_info']['server']['system_charset'] != 'utf-8')
 		{
 			$this->db->Link_ID->SetCharSet($GLOBALS['egw_info']['server']['system_charset']);
 		}
@@ -331,12 +331,7 @@ class egw extends egw_minimal
 				if (egw_session::get_sessionid()) $redirect .= 'cd=10&';
 			}
 			if ($relpath) $redirect .= 'phpgw_forward='.urlencode($relpath.(!empty($query) ? '?'.$query : ''));
-			//Header('Location: '.$GLOBALS['egw_info']['server']['webserver_url'].$redirect);
-			// do NOT redirect, but set top.location, as this works in framed template too
-			$redirect = $GLOBALS['egw_info']['server']['webserver_url'].$redirect;
-
-			echo "<html>\n<head>\n<script type='text/javascript'>top.location='$redirect';</script>\n</head>\n";
-			exit;
+			self::redirect_link($redirect);
 		}
 	}
 
@@ -344,6 +339,10 @@ class egw extends egw_minimal
 	 * Verify the user has rights for the requested app
 	 *
 	 * If the user has no rights for the app (eg. called via URL) he get a permission denied page (this function does NOT return)
+	 *
+	 * @throws egw_exception_redirect for anonymous user accessing something he has no rights to
+	 * @throws egw_exception_no_permission_admin
+	 * @throws egw_exception_no_permission_app
 	 */
 	function check_app_rights()
 	{
@@ -358,7 +357,12 @@ class egw extends egw_minimal
 				// present a login page, if anon user has no right for an application
 				if ($this->session->session_flags == 'A')
 				{
-					egw::redirect_link('/logout.php');
+					// need to destroy a basic auth session here, because it will only be available on current url
+					if (($sessionid = egw_session::get_sessionid(true)))
+					{
+						$GLOBALS['egw']->session->destroy($sessionid);
+					}
+					throw new egw_exception_redirect(egw::link('/logout.php'));
 				}
 				if ($currentapp == 'admin' || $GLOBALS['egw_info']['flags']['admin_only'])
 				{
@@ -439,9 +443,9 @@ class egw extends egw_minimal
 	/**
 	 * Link url generator
 	 *
-	 * @param string	$string	The url the link is for
-	 * @param string|array	$extravars=''	Extra params to be passed to the url
-	 * @param string $link_app=null if appname or true, some templates generate a special link-handler url
+	 * @param string $url url link is for
+	 * @param string|array $extravars ='' extra params to be added to url
+	 * @param string $link_app =null if appname or true, some templates generate a special link-handler url
 	 * @return string	The full url after processing
 	 */
 	static function link($url, $extravars = '', $link_app=null)
@@ -452,9 +456,9 @@ class egw extends egw_minimal
 	/**
 	 * Redirects direct to a generated link
 	 *
-	 * @param string	$string	The url the link is for
-	 * @param string/array	$extravars	Extra params to be passed to the url
-	 * @param string $link_app=null if appname or true, some templates generate a special link-handler url
+	 * @param string $url url link is for
+	 * @param string|array $extravars ='' extra params to be added to url
+	 * @param string $link_app =null if appname or true, some templates generate a special link-handler url
 	 * @return string	The full url after processing
 	 */
 	static function redirect_link($url, $extravars='', $link_app=null)
@@ -467,8 +471,8 @@ class egw extends egw_minimal
 	 *
 	 * This function handles redirects under iis and apache it assumes that $phpgw->link() has already been called
 	 *
-	 * @param  string The url ro redirect to
-	 * @param string $link_app=null appname to redirect for, default currentapp
+	 * @param string $url url to redirect to
+	 * @param string $link_app =null appname to redirect for, default currentapp
 	 */
 	static function redirect($url, $link_app=null)
 	{
@@ -546,7 +550,7 @@ class egw extends egw_minimal
 	 * garanties to run AFTER output send to user.
 	 *
 	 * @param callable $callback use array($classname, $method) for static methods
-	 * @param array $args=array()
+	 * @param array $args =array()
 	 */
 	public static function on_shutdown($callback, array $args=array())
 	{
@@ -571,6 +575,24 @@ class egw extends egw_minimal
 				egw_json_response::sendResult();
 			}
 
+			// run all on_shutdown callbacks with session in their name (eg. egw_link::save_session_cache), do NOT stop on exceptions
+			foreach(self::$shutdown_callbacks as $n => $data)
+			{
+				try {
+					//error_log(__METHOD__."() running ".array2string($data));
+					$callback = array_shift($data);
+					if (!is_array($callback) || strpos($callback[1], 'session') === false) continue;
+					call_user_func_array($callback, $data);
+				}
+				catch (Exception $ex) {
+					_egw_log_exception($ex);
+				}
+				unset(self::$shutdown_callbacks[$n]);
+			}
+			// now we can close the session
+			// without closing the session fastcgi_finish_request() will NOT send output to user
+			if (isset($GLOBALS['egw']->session) && is_object($GLOBALS['egw']->session)) $GLOBALS['egw']->session->commit_session();
+
 			// flush all output to user
 			/* does NOT work on Apache :-(
 			for($i = 0; ob_get_level() && $i < 10; ++$i)
@@ -579,7 +601,7 @@ class egw extends egw_minimal
 			}
 			flush();*/
 			// working for fastCGI :-)
-			if (function_exists('fastcgi_finish_request'))
+			if (function_exists('fastcgi_finish_request') && substr($_SERVER['PHP_SELF'], -32) != '/phpgwapi/cron/asyncservices.php')
 			{
 				fastcgi_finish_request();
 			}
@@ -684,26 +706,7 @@ class egw_minimal
 		switch($name)
 		{
 			case 'framework':
-				// default to idots, if no template_set set, to eg. not stall installations if settings use egw::link
-				if (empty($GLOBALS['egw_info']['server']['template_set'])) $GLOBALS['egw_info']['server']['template_set'] = 'idots';
-				// setup the new eGW framework (template sets)
-				$class = $GLOBALS['egw_info']['server']['template_set'].'_framework';
-				if (!class_exists($class))	// first try to autoload the class
-				{
-					require_once($file=EGW_INCLUDE_ROOT.'/phpgwapi/templates/'.$GLOBALS['egw_info']['server']['template_set'].'/class.'.$class.'.inc.php');
-					if (!in_array($file,(array)$_SESSION['egw_required_files']))
-					{
-						$_SESSION['egw_required_files'][] = $file;	// automatic load the used framework class, when the object get's restored
-					}
-				}
-				// fall back to idots if a template does NOT support current user-agent
-				if ($class != 'idots_framework' && method_exists($class,'is_supported_user_agent') &&
-					!call_user_func(array($class,'is_supported_user_agent')))
-				{
-					$GLOBALS['egw_info']['server']['template_set'] = 'idots';
-					return $this->__get('framework');
-				}
-				break;
+				return $this->framework = egw_framework::factory();
 			case 'template':	// need to be instancated for the current app
 				if (!($tpl_dir = common::get_tpl_dir($this->currentapp)))
 				{
