@@ -8,7 +8,7 @@
  * @subpackage cache
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright (c) 2009-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @version $Id: class.egw_cache.inc.php 47751 2014-07-17 12:35:14Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -89,6 +89,13 @@ class egw_cache
 	 */
 	static $max_expiration;
 
+	/**
+	 * Used to determine keys for tree- and instance-level caches
+	 *
+	 * @var string
+	 */
+	static $egw_server_root = EGW_SERVER_ROOT;
+
 	/**
 	 * Set some data in the cache
 	 *
@@ -639,7 +646,7 @@ class egw_cache
 			case self::TREE:
 				if (!isset($tree_key))
 				{
-					$tree_key = $level.'-'.str_replace(array(':','/','\\'),'-',EGW_SERVER_ROOT);
+					$tree_key = $level.'-'.str_replace(array(':','/','\\'),'-', self::$egw_server_root);
 					// add charset to key, if not utf-8 (as everything we store depends on charset!)
 					if (($charset = self::get_system_config('system_charset',false)) && $charset != 'utf-8')
 					{
