@@ -8,7 +8,7 @@
  * @package api
  * @subpackage tools
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.egw_ckeditor_config.inc.php 48451 2014-09-04 09:59:17Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -58,25 +58,44 @@ class egw_ckeditor_config
 
 	/**
 	 * Get available CKEditor Skins
-	 * @return array
+	 *
+	 * Only return skins existing in filesystem, as we disable / remove them if not compatible with supported browsers.
+	 *
+	 * @return array skin => label pairs alphabetical sorted with default moono first
 	 */
 	public static function getAvailableCKEditorSkins()
 	{
-		return array(
+		$labels = array(
 			'kama'  => lang('kama theme'),
 			'moono'	=> lang('moono theme (default)'),
-			'moonocolor'	=> lang('moono color theme'),
-			'moono-dark'	=> lang('dark moono theme'),
-			'bootstrapck'	=> lang('bootstrap theme for ckeditor'),
 		);
+		$skins = array();
+
+		foreach(scandir(EGW_SERVER_ROOT.'/phpgwapi/js/ckeditor/skins') as $skin)
+		{
+			if ($skin[0] == '.') continue;
+
+			if (isset($labels[$skin]))
+			{
+				$skins[$skin] = $labels[$skin];
+			}
+			else
+			{
+				$skins[$skin] = str_replace('_', '-', $skin).' '.lang('Theme');
+			}
+		}
+		uasort($skins, 'strcasecmp');
+
+		// return our default "moono" first
+		return isset($skins['moono']) ? array('moono' => $skins['moono'])+$skins : $skins;
 	}
 
 	/**
 	 * Get font size from preferences
 	 *
-	 * @param array $prefs=null default $GLOBALS['egw_info']['user']['preferences']
-	 * @param string &$size=null on return just size, without unit
-	 * @param string &$unit=null on return just unit
+	 * @param array $prefs =null default $GLOBALS['egw_info']['user']['preferences']
+	 * @param string &$size =null on return just size, without unit
+	 * @param string &$unit =null on return just unit
 	 * @return string font-size including unit
 	 */
 	public static function font_size_from_prefs(array $prefs=null, &$size=null, &$unit=null)
@@ -187,19 +206,18 @@ class egw_ckeditor_config
 			{
 				case 'kama':
 					$skin = "kama";
-					if (html::$user_agent=='firefox' || html::$user_agent=='msie') $skin='moonocolor';
+					//if (html::$user_agent=='firefox' || html::$user_agent=='msie') $skin='moonocolor';
 					break;
 				case 'silver':
 				case 'bootstrapck':
-					$skin = "bootstrapck";
-					break;
 				case 'moono-dark':
-					$skin = "moono-dark";
-					break;
+				case 'Moono_blue':
+				case 'icy_orange':
+				case 'office2013':
 				case 'office2003':
 				case 'moonocolor':
-					$skin = "moonocolor";
-					break;
+				case 'minimalist':
+				case 'flat':
 				case 'moono':
 				case 'default':
 				default:
@@ -207,9 +225,10 @@ class egw_ckeditor_config
 			}
 
 			//Check whether the skin actually exists, if not, switch to a default
-			if (!(file_exists($basePath.'skins/'.$skin) || file_exists($skin) || !empty($skin)))
+			if (!file_exists(EGW_SERVER_ROOT.'/phpgwapi/js/ckeditor/skins/'.$skin))
+			{
 				$skin = "moono"; //this is the basic skin for ckeditor
-
+			}
 			self::$skin = $skin;
 		}
 
@@ -284,7 +303,7 @@ class egw_ckeditor_config
 			// to use browsers native spellchecker, you have to hold CMD/CTRL button on rightclick to
 			// access the browsers spell correction options
 			if ($GLOBALS['egw_info']['server']['enabled_spellcheck']!='YesNoSCAYT') $config['disableNativeSpellChecker'] = false;
-			$spellchecker_button = 'SpellChecker';
+
 			if (!empty($GLOBALS['egw_info']['server']['aspell_path']) &&
 				is_executable($GLOBALS['egw_info']['server']['aspell_path']) &&
 				($GLOBALS['egw_info']['server']['enabled_spellcheck']!='YesUseWebSpellCheck' &&
@@ -292,7 +311,7 @@ class egw_ckeditor_config
 			)
 			{
 				$spellchecker_button = 'SpellCheck';
-				$config['extraPlugins'] = "aspell";
+				self::append_extraPlugins_config_array($config, array("aspell"));
 			}
 			if ($GLOBALS['egw_info']['server']['enabled_spellcheck']!='YesNoSCAYT' &&
 				$GLOBALS['egw_info']['server']['enabled_spellcheck']!='YesBrowserBased'
@@ -403,12 +422,38 @@ class egw_ckeditor_config
 		$spellchecker_button = null;
 
 		self::add_default_options($config, $height, $expanded_toolbar, $start_path);
+		$scayt_button = null;
 		self::add_spellchecker_options($config, $spellchecker_button, $scayt_button);
 		self::add_toolbar_options($config, $mode, $spellchecker_button, $scayt_button);
 		//error_log(__METHOD__."('$mode', $height, ".array2string($expanded_toolbar).") returning ".array2string($config));
+		// Add extra plugins
+		self::append_extraPlugins_config_array($config, array('uploadimage','uploadwidget','widget','notification','notificationaggregator','lineutils'));
 		return $config;
 	}
 
+	/**
+	 * Adds extra
+	 * @param array $config
+	 * @param array $plugins plugins name which needs to be appended into extraPlugins
+	 */
+	public static function append_extraPlugins_config_array (&$config, $plugins)
+	{
+		if (is_array($plugins))
+		{
+			foreach ($plugins as &$plugin)
+			{
+				if (!empty($config['extraPlugins']) && $config['extraPlugins'] !== '')
+				{
+					$config['extraPlugins'] .= ',' . $plugin;
+				}
+				else
+				{
+					$config['extraPlugins'] = $plugin;
+				}
+			}
+		}
+	}
+
 	/**
 	 * Returns a json encoded string containing the configuration for the ckeditor.
 	 * @param string $mode specifies the count of toolbar buttons available to the user. Possible
