@@ -8,7 +8,7 @@
  * @subpackage db
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright (c) 2003-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @version $Id: class.egw_db.inc.php 51559 2015-02-03 13:19:42Z ralfbecker $
+ * @version $Id$
  */
 
 if(empty($GLOBALS['egw_info']['server']['db_type']))
@@ -170,6 +170,13 @@ class egw_db
 	 */
 	static $tablealiases = array();
 
+	/**
+	 * Callback to check if selected node is healty / should be used
+	 *
+	 * @var callback throwing egw_exception_db_connection, if connected node should NOT be used
+	 */
+	static $health_check;
+
 	/**
 	 * db allows sub-queries, true for everything but mysql < 4.1
 	 *
@@ -298,16 +305,17 @@ class egw_db
 	}
 
 	/**
-	* Open a connection to a database
-	*
-	* @param string $Database name of database to use (optional)
-	* @param string $Host database host to connect to (optional)
-	* @param string $Port database port to connect to (optional)
-	* @param string $User name of database user (optional)
-	* @param string $Password password for database user (optional)
-	* @param string $Type type of database (optional)
-	* @return ADONewConnection
-	*/
+	 * Open a connection to a database
+	 *
+	 * @param string $Database name of database to use (optional)
+	 * @param string $Host database host to connect to (optional)
+	 * @param string $Port database port to connect to (optional)
+	 * @param string $User name of database user (optional)
+	 * @param string $Password password for database user (optional)
+	 * @param string $Type type of database (optional)
+	 * @throws egw_exception_db_connection
+	 * @return ADOConnection
+	 */
 	function connect($Database = NULL, $Host = NULL, $Port = NULL, $User = NULL, $Password = NULL,$Type = NULL)
 	{
 		/* Handle defaults */
@@ -339,45 +347,142 @@ class egw_db
 		{
 			$this->Type = $GLOBALS['egw_info']['server']['db_type'];
 		}
+		// on connection failure re-try with an other host
+		// remembering in session which host we used last time
+		$use_host_from_session = true;
+		while(($host = $this->get_host(!$use_host_from_session)))
+		{
+			try {
+				//error_log(__METHOD__."() this->Host(s)=$this->Host, n=$n --> host=$host");
+				$new_connection = !$this->Link_ID || !$this->Link_ID->IsConnected();
+				$this->_connect($host);
+				// check if connected node is healty
+				if ($new_connection && self::$health_check)
+				{
+					call_user_func(self::$health_check, $this);
+				}
+				//error_log(__METHOD__."() host=$host, new_connection=$new_connection, this->Type=$this->Type, this->Host=$this->Host, wsrep_local_state=".array2string($state));
+				return $this->Link_ID;
+			}
+			catch(egw_exception_db_connection $e) {
+				//_egw_log_exception($e);
+				$this->disconnect();	// force a new connect
+				$this->Type = $this->setupType;	// get set to "mysql" for "mysqli"
+				$use_host_from_session = false;	// re-try with next host from list
+			}
+		}
+		if (!isset($e))
+		{
+			$e = new egw_exception_db_connection('No DB host set!');
+		}
+		throw $e;
+	}
+
+	/**
+	 * Check if just connected Galera cluster node is healthy / fully operational
+	 *
+	 * A node in state "Donor/Desynced" will block updates at the end of a SST.
+	 * Therefore we try to avoid that node, if we have an alternative.
+	 *
+	 * To enable this check add the following to your header.inc.php:
+	 *
+	 * require_once(EGW_API_INC.'/class.egw_db.inc.php');
+	 * egw_db::$health_check = array('egw_db', 'galera_cluster_health');
+	 *
+	 * @param egw_db $db already connected egw_db instance to check
+	 * @throws egw_exception_db_connection if node should NOT be used
+	 */
+	static function galera_cluster_health(egw_db $db)
+	{
+		if (($state = $db->query("SHOW STATUS WHERE Variable_name in ('wsrep_cluster_size','wsrep_local_state','wsrep_local_state_comment')")->GetAssoc()))
+		{
+			if ($state['wsrep_local_state_comment'] == 'Synced' ||
+				// if we have only 2 nodes (2. one starting), we can only use the donor
+				$state['wsrep_local_state_comment'] == 'Donor/Desynced' &&
+					$state['wsrep_cluster_size'] == 2) return;
+
+			throw new egw_exception_db_connection('Node is NOT Synced! '.array2string($state));
+		}
+	}
+
+	/**
+	 * Get one of multiple (semicolon-separated) DB-hosts to use
+	 *
+	 * Which host to use is cached in session, default is first one.
+	 *
+	 * @param boolean $next =false	true: move to next host
+	 * @return boolean|string hostname or false, if already number-of-hosts plus 2 times called with $next == true
+	 */
+	public function get_host($next = false)
+	{
+		$hosts = explode(';', $this->Host[0] == '@' ? getenv(substr($this->Host, 1)) : $this->Host);
+		$num_hosts = count($hosts);
+		$n =& egw_cache::getSession(__CLASS__, $this->Host);
+		if (!isset($n)) $n = 0;
+
+		if ($next && ++$n >= $num_hosts+2)
+		{
+			$n = 0;	// start search again with default on next request
+			$ret = false;
+		}
+		else
+		{
+			$ret = $hosts[$n % $num_hosts];
+		}
+		//error_log(__METHOD__."(next=".array2string($next).") n=$n returning ".array2string($ret));
+		return $ret;
+	}
+
+	/**
+	 * Connect to given host
+	 *
+	 * @param string $Host host to connect to
+	 * @return ADOConnection
+	 * @throws egw_exception_db_connection
+	 */
+	protected function _connect($Host)
+	{
 		if (!$this->Link_ID)
 		{
-			foreach(array('Host','Database','User','Password') as $name)
+			$Database = $User = $Password = $Port = $Type = '';
+			foreach(array('Database','User','Password','Port','Type') as $name)
 			{
 				$$name = $this->$name;
+				if (${$name}[0] == '@' && $name != 'Password') $$name = getenv(substr($$name, 1));
 			}
-			$this->setupType = $php_extension = $type = $this->Type;
+			$this->setupType = $php_extension = $Type;
 
-			switch($this->Type)	// convert to ADO db-type-names
+			switch($Type)	// convert to ADO db-type-names
 			{
 				case 'pgsql':
-					$type = 'postgres'; // name in ADOdb
+					$Type = 'postgres'; // name in ADOdb
 					// create our own pgsql connection-string, to allow unix domain soccets if !$Host
-					$Host = "dbname=$this->Database".($this->Host ? " host=$this->Host".($this->Port ? " port=$this->Port" : '') : '').
-						" user=$this->User".($this->Password ? " password='".addslashes($this->Password)."'" : '');
+					$Host = "dbname=$Database".($Host ? " host=$Host".($Port ? " port=$Port" : '') : '').
+						" user=$User".($Password ? " password='".addslashes($Password)."'" : '');
 					$User = $Password = $Database = '';	// to indicate $Host is a connection-string
 					break;
 
 				case 'odbc_mssql':
 					$php_extension = 'odbc';
-					$this->Type = 'mssql';
+					$Type = 'mssql';
 					// fall through
 				case 'mssql':
-					if ($this->Port) $Host .= ','.$this->Port;
+					if ($Port) $Host .= ','.$Port;
 					break;
 
 				case 'odbc_oracle':
 					$php_extension = 'odbc';
-					$this->Type = 'oracle';
+					$Type = 'oracle';
 					break;
 				case 'oracle':
-					$php_extension = $type = 'oci8';
+					$php_extension = $Type = 'oci8';
 					break;
 
 				case 'sapdb':
-					$this->Type = 'maxdb';
+					$Type = 'maxdb';
 					// fall through
 				case 'maxdb':
-					$type ='sapdb';	// name in ADOdb
+					$Type ='sapdb';	// name in ADOdb
 					$php_extension = 'odbc';
 					break;
 
@@ -385,10 +490,10 @@ class egw_db
 					$php_extension = 'mysql';	// you can use $this->setupType to determine if it's mysqlt or mysql
 					// fall through
 				case 'mysqli':
-					$this->Type = 'mysql';
+					$this->Type = 'mysql';		// need to be "mysql", so apps can check just for "mysql"!
 					// fall through
 				default:
-					if ($this->Port) $Host .= ':'.$this->Port;
+					if ($Port) $Host .= ':'.$Port;
 					break;
 			}
 			if (!isset($GLOBALS['egw']->ADOdb) ||	// we have no connection so far
@@ -411,16 +516,22 @@ class egw_db
 				{
 					$this->privat_Link_ID = True;	// remember that we use a privat Link_ID for disconnect
 				}
-				$this->Link_ID = ADONewConnection($type);
+				$this->Link_ID = ADONewConnection($Type);
 				if (!$this->Link_ID)
 				{
-					throw new egw_exception_db_connection("No ADOdb support for '$type' ($this->Type) !!!");
+					throw new egw_exception_db_connection("No ADOdb support for '$Type' ($this->Type) !!!");
 				}
 				$connect = $GLOBALS['egw_info']['server']['db_persistent'] ? 'PConnect' : 'Connect';
 				if (($Ok = $this->Link_ID->$connect($Host, $User, $Password, $Database)))
 				{
 					$this->ServerInfo = $this->Link_ID->ServerInfo();
-					$this->set_capabilities($type,$this->ServerInfo['version']);
+					$this->set_capabilities($Type,$this->ServerInfo['version']);
+
+					// switch off MySQL 5.7+ ONLY_FULL_GROUP_BY sql_mode
+					if (substr($this->Type, 0, 5) == 'mysql' && $this->ServerInfo['version'] >= 5.7 && $this->ServerInfo['version'] < 10.0)
+					{
+						$this->query("SET SESSION sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''))", __LINE__, __FILE__);
+					}
 				}
 				if (!$Ok)
 				{
@@ -430,12 +541,12 @@ class egw_db
 				if ($this->Debug)
 				{
 					echo function_backtrace();
-					echo "<p>new ADOdb connection to $this->Type://$this->Host/$this->Database: Link_ID".($this->Link_ID === $GLOBALS['egw']->ADOdb ? '===' : '!==')."\$GLOBALS[egw]->ADOdb</p>";
+					echo "<p>new ADOdb connection to $Type://$Host/$Database: Link_ID".($this->Link_ID === $GLOBALS['egw']->ADOdb ? '===' : '!==')."\$GLOBALS[egw]->ADOdb</p>";
 					//echo "<p>".print_r($this->Link_ID->ServerInfo(),true)."</p>\n";
 					_debug_array($this);
 					echo "\$GLOBALS[egw]->db="; _debug_array($GLOBALS[egw]->db);
 				}
-				if ($this->Type == 'mssql')
+				if ($Type == 'mssql')
 				{
 					// this is the format ADOdb expects
 					$this->Link_ID->Execute('SET DATEFORMAT ymd');
@@ -443,6 +554,9 @@ class egw_db
 					ini_set('mssql.textlimit',2147483647);
 					ini_set('mssql.sizelimit',2147483647);
 				}
+				// set our default charset
+				$this->Link_ID->SetCharSet('utf-8');
+
 				$new_connection = true;
 			}
 			else
@@ -452,10 +566,15 @@ class egw_db
 		}
 		if (!$this->Link_ID->isConnected() && !$this->Link_ID->Connect())
 		{
-			$Host = preg_replace('/password=[^ ]+/','password=$Password',$this->Host);	// eg. postgres dsn contains password
-			throw new egw_exception_db_connection("ADOdb::$connect($Host, $this->User, \$Password, $this->Database) reconnect failed.");
+			$Host = preg_replace('/password=[^ ]+/','password=$Password',$Host);	// eg. postgres dsn contains password
+			throw new egw_exception_db_connection("ADOdb::$connect($Host, $User, \$Password, $Database) reconnect failed.");
+		}
+		// fix due to caching and reusing of connection not correctly set $this->Type == 'mysql'
+		if ($this->Type == 'mysqli')
+		{
+			$this->setupType = $this->Type;
+			$this->Type = 'mysql';
 		}
-
 		if ($new_connection)
 		{
 			foreach(get_included_files() as $file)
@@ -479,6 +598,24 @@ class egw_db
 		$this->connect();	// we need to re-connect
 	}
 
+	/**
+	 * Magic method called when object get's serialized
+	 *
+	 * We do NOT store Link_ID and private_Link_ID, as we need to reconnect anyway.
+	 * This also ensures reevaluating environment-data or multiple hosts in connection-data!
+	 *
+	 * @return array
+	 */
+	function __sleep()
+	{
+		if (!empty($this->setupType)) $this->Type = $this->setupType;	// restore Type eg. to mysqli
+
+		$vars = get_object_vars($this);
+		unset($vars['Link_ID']);
+		unset($vars['privat_Link_ID']);
+		return array_keys($vars);
+	}
+
 	/**
 	 * changes defaults set in class-var $capabilities depending on db-type and -version
 	 *
@@ -533,6 +670,8 @@ class egw_db
 		}
 		unset($this->Link_ID);
 		$this->Link_ID = 0;
+
+		if (!empty($this->setupType)) $this->Type = $this->setupType;
 	}
 
 	/**
@@ -686,6 +825,7 @@ class egw_db
 				"\n$this->Error ($this->Errno)".
 				($inputarr ? "\nParameters: '".implode("','",$inputarr)."'":''), $this->Errno);
 		}
+		elseif(empty($this->Query_ID->sql)) $this->Query_ID->sql = $Query_String;
 		return $this->Query_ID;
 	}
 
@@ -1126,14 +1266,23 @@ class egw_db
 		$currentPassword = $this->Password;
 		$currentDatabase = $this->Database;
 
+		if ($adminname != '')
+		{
+			$this->User = $adminname;
+			$this->Password = $adminpasswd;
+			$this->Database = $this->Type == 'pgsql' ? 'template1' : 'mysql';
+		}
+		$this->disconnect();
+
 		$sqls = array();
 		switch ($this->Type)
 		{
 			case 'pgsql':
-				$meta_db = 'template1';
 				$sqls[] = "CREATE DATABASE $currentDatabase";
 				break;
 			case 'mysql':
+			case 'mysqli':
+			case 'mysqlt':
 				$create = "CREATE DATABASE `$currentDatabase`";
 				if ($charset && isset($this->Link_ID->charset2mysql[$charset]) && (float) $this->ServerInfo['version'] >= 4.1)
 				{
@@ -1141,19 +1290,11 @@ class egw_db
 				}
 				$sqls[] = $create;
 				$sqls[] = "GRANT ALL ON `$currentDatabase`.* TO $currentUser@'$grant_host' IDENTIFIED BY ".$this->quote($currentPassword);
-				$meta_db = 'mysql';
 				break;
 			default:
 				echo "<p>db::create_database(user='$adminname',\$pw) not yet implemented for DB-type '$this->Type'</p>\n";
 				break;
 		}
-		if ($adminname != '')
-		{
-			$this->User = $adminname;
-			$this->Password = $adminpasswd;
-			$this->Database = $meta_db;
-		}
-		$this->disconnect();
 		foreach($sqls as $sql)
 		{
 			$this->query($sql,__LINE__,__FILE__);
@@ -1220,6 +1361,27 @@ class egw_db
 		return $sql;
 	}
 
+	/**
+	 * SQL returning character (not byte!) positions for $substr in $str
+	 *
+	 * @param string $str
+	 * @param string $substr
+	 * @return string SQL returning character (not byte!) positions for $substr in $str
+	 */
+	function strpos($str, $substr)
+	{
+		switch($this->Type)
+		{
+			case 'mysql':
+				return "LOCATE($substr,$str)";
+			case 'pgsql':
+				return "STRPOS($str,$substr)";
+			case 'mssql':
+				return "CHARINDEX($substr,$str)";
+		}
+		die(__METHOD__." not implemented for DB type '$this->Type'!");
+	}
+
 	/**
 	 * Convert a DB specific timestamp in a unix timestamp stored as integer, like MySQL: UNIX_TIMESTAMP(ts)
 	 *
@@ -1300,6 +1462,24 @@ class egw_db
 		return false;
 	}
 
+	/**
+	 * Cast a column or sql expression to integer, necessary at least for postgreSQL or MySQL for sorting
+	 *
+	 * @param string $expr
+	 * @return string
+	 */
+	function to_double($expr)
+	{
+		switch($this->Type)
+		{
+			case 'pgsql':
+				return $expr.'::double';
+			case 'mysql':
+				return 'CAST('.$expr.' AS DECIMAL(24,3))';
+		}
+		return $expr;
+	}
+
 	/**
 	 * Cast a column or sql expression to integer, necessary at least for postgreSQL
 	 *
@@ -1312,6 +1492,8 @@ class egw_db
 		{
 			case 'pgsql':
 				return $expr.'::integer';
+			case 'mysql':
+				return 'CAST('.$expr.' AS SIGNED)';
 		}
 		return $expr;
 	}
@@ -1366,7 +1548,7 @@ class egw_db
 		{
 			// if name contains special characters, quote it
 			// always quote for postgreSQL, as this is the only way to support mixed case names
-			if (preg_match('/\W/', $name) || $type == 'pgsql' && preg_match('/[A-Z]+/', $name))
+			if (preg_match('/\W/', $name) || $type == 'pgsql' && preg_match('/[A-Z]+/', $name) || $name == 'index')
 			{
 				return $quote . $name . $quote;
 			}
@@ -1526,9 +1708,10 @@ class egw_db
 				$column_type = is_array($column_definitions) ? @$column_definitions[$col]['type'] : False;
 				$not_null = is_array($column_definitions) && isset($column_definitions[$col]['nullable']) ? !$column_definitions[$col]['nullable'] : false;
 
+				$maxlength = null;
 				if ($truncate_varchar)
 				{
-					$maxlength = $column_definitions[$col]['type'] == 'varchar' ? $column_definitions[$col]['precision'] : null;
+					$maxlength = in_array($column_definitions[$col]['type'], array('varchar','ascii')) ? $column_definitions[$col]['precision'] : null;
 				}
 				// dont use IN ( ), if there's only one value, it's slower for MySQL
 				if (is_array($data) && count($data) == 1)
@@ -1556,6 +1739,7 @@ class egw_db
 				}
 				elseif (is_int($key) && $use_key===True)
 				{
+					if (empty($data)) continue;	// would give SQL error
 					$values[] = $data;
 				}
 				elseif ($glue != ',' && $use_key === True && !$not_null && is_null($data))
@@ -1769,7 +1953,9 @@ class egw_db
 			// the checked values need to be inserted too, value in data has precedence, also cant insert sql strings (numerical id)
 			foreach($where as $column => $value)
 			{
-				if (!is_numeric($column) && !isset($data[$column]))
+				if (!is_numeric($column) && !isset($data[$column]) &&
+					// skip auto-id of 0 or NULL, as PostgreSQL does NOT create an auto-id, if they are given
+					!(!$value && count($table_def['pk']) == 1 && $column == $table_def['pk'][0]))
 				{
 					$data[$column] = $value;
 				}
