@@ -9,7 +9,7 @@
  * @package api
  * @subpackage framework
  * @access public
- * @version $Id: class.egw_framework.inc.php 51545 2015-02-03 09:46:59Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -83,6 +83,40 @@ abstract class egw_framework
 		$this->template_dir = '/phpgwapi/templates/'.$template;
 	}
 
+	/**
+	 * Factory method to instanciate framework object
+	 *
+	 * @return egw_framwork
+	 */
+	public static function factory()
+	{
+		if ((html::$ua_mobile || $GLOBALS['egw_info']['user']['preferences']['common']['theme'] == 'mobile') &&
+			file_exists(EGW_SERVER_ROOT.'/pixelegg'))
+		{
+			$GLOBALS['egw_info']['server']['template_set'] = 'pixelegg';
+		}
+		// default to idots, if no template_set set, to eg. not stall installations if settings use egw::link
+		if (empty($GLOBALS['egw_info']['server']['template_set'])) $GLOBALS['egw_info']['server']['template_set'] = 'idots';
+		// setup the new eGW framework (template sets)
+		$class = $GLOBALS['egw_info']['server']['template_set'].'_framework';
+		if (!class_exists($class))	// first try to autoload the class
+		{
+			require_once($file=EGW_INCLUDE_ROOT.'/phpgwapi/templates/'.$GLOBALS['egw_info']['server']['template_set'].'/class.'.$class.'.inc.php');
+			if (!in_array($file,(array)$_SESSION['egw_required_files']))
+			{
+				$_SESSION['egw_required_files'][] = $file;	// automatic load the used framework class, when the object get's restored
+			}
+		}
+		// fall back to idots if a template does NOT support current user-agent
+		if ($class != 'idots_framework' && method_exists($class,'is_supported_user_agent') &&
+			!call_user_func(array($class,'is_supported_user_agent')))
+		{
+			$GLOBALS['egw_info']['server']['template_set'] = 'idots';
+			return self::factory();
+		}
+		return new $class($GLOBALS['egw_info']['server']['template_set']);
+	}
+
 	/**
 	 * Additional attributes or urls for CSP script-src 'self'
 	 *
@@ -180,18 +214,53 @@ abstract class egw_framework
 				//error_log(__METHOD__."() setting CSP script-src $attr ".function_backtrace());
 			}
 		}
-		//error_log(__METHOD__."(".array2string($set).") returned ".array2string(implode(' ', self::$csp_script_src_attrs)).' '.function_backtrace());
+		//error_log(__METHOD__."(".array2string($set).") returned ".array2string(implode(' ', self::$csp_connect_src_attrs)).' '.function_backtrace());
 		return implode(' ', self::$csp_connect_src_attrs);
 	}
 
 	/**
-	 * Query additional CSP frame-src from current app
+	 * Additional attributes or urls for CSP frame-src 'self'
 	 *
-	 * @return array
+	 * @var array
 	 */
-	protected function _get_csp_frame_src()
+	private static $csp_frame_src_attrs;
+
+	/**
+	 * Set/get Content-Security-Policy attributes for frame-src:
+	 *
+	 * Calling this method with an empty array sets no frame-src, but "'self'"!
+	 *
+	 * @param string|array $set =array() URL (incl. protocol!)
+	 * @return string with attributes eg. "'unsafe-inline'"
+	 */
+	public static function csp_frame_src_attrs($set=null)
 	{
-		return $GLOBALS['egw']->hooks->single('csp-frame-src', $GLOBALS['egw_info']['flags']['currentapp']);
+		// set frame-src attrs of API and apps via hook
+		if (!isset(self::$csp_frame_src_attrs) && !isset($set))
+		{
+			$frame_src = array('manual.egroupware.org', 'www.egroupware.org');
+			if (($app_additional = $GLOBALS['egw']->hooks->process('csp-frame-src')))
+			{
+				foreach($app_additional as $addtional)
+				{
+					if ($addtional) $frame_src = array_unique(array_merge($frame_src, $addtional));
+				}
+			}
+			return self::csp_frame_src_attrs($frame_src);
+		}
+
+		if (!isset(self::$csp_frame_src_attrs)) self::$csp_frame_src_attrs = array();
+
+		foreach((array)$set as $attr)
+		{
+			if (!in_array($attr, self::$csp_frame_src_attrs))
+			{
+				self::$csp_frame_src_attrs[] = $attr;
+				//error_log(__METHOD__."() setting CSP script-src $attr ".function_backtrace());
+			}
+		}
+		//error_log(__METHOD__."(".array2string($set).") returned ".array2string(implode(' ', self::$csp_frame_src_attrs)).' '.function_backtrace());
+		return implode(' ', self::$csp_frame_src_attrs);
 	}
 
 	/**
@@ -207,18 +276,19 @@ abstract class egw_framework
 		// - "connect-src 'self'" allows ajax requests only to self
 		// - "style-src 'self' 'unsave-inline'" allows only self and inline style, which we need
 		// - "frame-src 'self' manual.egroupware.org" allows frame and iframe content only for self or manual.egroupware.org
-		$frame_src = array("'self'", 'manual.egroupware.org');
-		if (($additional = $this->_get_csp_frame_src())) $frame_src = array_unique(array_merge($frame_src, $additional));
-
 		$csp = "script-src 'self' ".self::csp_script_src_attrs().
 			"; connect-src 'self' ".self::csp_connect_src_attrs().
 			"; style-src 'self' ".self::csp_style_src_attrs().
-			"; frame-src ".implode(' ', $frame_src);
+			"; frame-src 'self' ".self::csp_frame_src_attrs();
 
 		//$csp = "default-src * 'unsafe-eval' 'unsafe-inline'";	// allow everything
 		header("Content-Security-Policy: $csp");
 		header("X-Webkit-CSP: $csp");	// Chrome: <= 24, Safari incl. iOS
 		header("X-Content-Security-Policy: $csp");	// FF <= 22
+
+		// allow client-side to detect first load aka just logged in
+		$reload_count =& egw_cache::getSession(__CLASS__, 'framework-reload');
+		self::$extra['framework-reload'] = (int)(bool)$reload_count++;
 	}
 
 	/**
@@ -232,7 +302,6 @@ abstract class egw_framework
 
 			// allways load jquery (not -ui) and egw_json first
 			'/phpgwapi/js/jquery/jquery.js',
-			'/phpgwapi/js/./egw_json.js',
 			// always include javascript helper functions
 			'/phpgwapi/js/jsapi/jsapi.js',
 			'/phpgwapi/js/jsapi/egw.js',
@@ -339,9 +408,10 @@ abstract class egw_framework
 	 */
 	public static function refresh_opener($msg, $app, $id=null, $type=null, $targetapp=null, $replace=null, $with=null, $msg_type=null)
 	{
-		unset($msg, $app, $id, $type, $targetapp, $replace, $with, $msg_type);	// used only via func_get_args();
 		//error_log(__METHOD__.'('.array2string(func_get_args()).')');
 		self::$extra['refresh-opener'] = func_get_args();
+
+		unset($msg, $app, $id, $type, $targetapp, $replace, $with, $msg_type);	// used only via func_get_args();
 	}
 
 	/**
@@ -354,8 +424,9 @@ abstract class egw_framework
 	 */
 	public static function message($msg, $type='success')
 	{
-		unset($msg, $type);	// used only via func_get_args();
 		self::$extra['message'] = func_get_args();
+
+		unset($msg, $type);	// used only via func_get_args();
 	}
 
 	/**
@@ -367,10 +438,11 @@ abstract class egw_framework
 	 */
 	public static function popup($link, $target='_blank', $popup='640x480')
 	{
-		unset($link, $target, $popup);	// used only via func_get_args()
 		// default params are not returned by func_get_args!
 		$args = func_get_args()+array(null, '_blank', '640x480');
 
+		unset($link, $target, $popup);	// used only via func_get_args()
+
 		if (egw_json_request::isJSONRequest())
 		{
 			egw_json_response::get()->apply('egw.open_link', $args);
@@ -506,24 +578,48 @@ abstract class egw_framework
 	abstract function footer();
 
 	/**
-	* displays a login screen
-	*
-	* @param string $extra_vars for login url
+	 * Displays the login screen
+	 *
+	 * @param string $extra_vars for login url
+	 * @param string $change_passwd =null string with message to render input fields for password change
 	*/
-	function login_screen($extra_vars)
+	function login_screen($extra_vars, $change_passwd=null)
 	{
-		//allow to include JSONP file with social media urls from egroupware.org
-		self::csp_script_src_attrs('https://www.egroupware.org');
+		self::csp_frame_src_attrs(array());	// array() no external frame-sources
 
-		//error_log(__METHOD__."() server[template_dir]=".array2string($GLOBALS['egw_info']['server']['template_dir']).", this->template=$this->template, this->template_dir=$this->template_dir, get_class(this)=".get_class($this));
-		$tmpl = new Template($GLOBALS['egw_info']['server']['template_dir']);
+		//error_log(__METHOD__."() this->template=$this->template, this->template_dir=$this->template_dir, get_class(this)=".get_class($this));
+		$tmpl = new Template(EGW_SERVER_ROOT.$this->template_dir);
 
 		$tmpl->set_file(array('login_form' => 'login.tpl'));
 
 		$tmpl->set_var('lang_message',$GLOBALS['loginscreenmessage']);
 
-		$last_loginid = $_COOKIE['last_loginid'];
-
+		// hide change-password fields, if not requested
+		if (!$change_passwd)
+		{
+			$tmpl->set_block('login_form','change_password');
+			$tmpl->set_var('change_password', '');
+			$tmpl->set_var('lang_password',lang('password'));
+			$tmpl->set_var('cd',check_logoutcode($_GET['cd']));
+			$tmpl->set_var('cd_class', isset($_GET['cd']) && $_GET['cd'] != 1 ? 'error' : '');
+			$last_loginid = $_COOKIE['last_loginid'];
+			$last_domain  = $_COOKIE['last_domain'];
+			$tmpl->set_var('passwd', '');
+			$tmpl->set_var('autofocus_login', 'autofocus');
+		}
+		else
+		{
+			$tmpl->set_var('lang_password',lang('Old password'));
+			$tmpl->set_var('lang_new_password',lang('New password'));
+			$tmpl->set_var('lang_repeat_password',lang('Repeat password'));
+			$tmpl->set_var('cd', $change_passwd);
+			$tmpl->set_var('cd_class', 'error');
+			$last_loginid = $_POST['login'];
+			$last_domain  = $_POST['domain'];
+			$tmpl->set_var('passwd', $_POST['passwd']);
+			$tmpl->set_var('autofocus_login', '');
+			$tmpl->set_var('autofocus_new_passwd', 'autofocus');
+		}
 		if($GLOBALS['egw_info']['server']['show_domain_selectbox'])
 		{
 			foreach(array_keys($GLOBALS['egw_domain']) as $domain)
@@ -532,7 +628,7 @@ abstract class egw_framework
 			}
 			$tmpl->set_var(array(
 				'lang_domain'   => lang('domain'),
-				'select_domain' => html::select('logindomain',$_COOKIE['last_domain'],$domains,true,'tabindex="2"',0,false),
+				'select_domain' => html::select('logindomain',$last_domain,$domains,true,'tabindex="2"',0,false),
 			));
 		}
 		else
@@ -547,9 +643,9 @@ abstract class egw_framework
 				reset($GLOBALS['egw_domain']);
 				list($default_domain) = each($GLOBALS['egw_domain']);
 
-				if($_COOKIE['last_domain'] != $default_domain && !empty($_COOKIE['last_domain']))
+				if(!empty ($last_domain) && $last_domain != $default_domain)
 				{
-					$last_loginid .= '@' . $_COOKIE['last_domain'];
+					$last_loginid .= '@' . $last_domain;
 				}
 			}
 		}
@@ -589,12 +685,10 @@ abstract class egw_framework
 		}
 
 		$tmpl->set_var('login_url', $GLOBALS['egw_info']['server']['webserver_url'] . '/login.php' . $extra_vars);
-		$tmpl->set_var('version',$GLOBALS['egw_info']['server']['versions']['phpgwapi']);
-		$tmpl->set_var('cd',check_logoutcode($_GET['cd']));
-		$tmpl->set_var('cookie',$last_loginid);
+		$tmpl->set_var('version', $GLOBALS['egw_info']['server']['versions']['phpgwapi']);
+		$tmpl->set_var('login', $last_loginid);
 
 		$tmpl->set_var('lang_username',lang('username'));
-		$tmpl->set_var('lang_password',lang('password'));
 		$tmpl->set_var('lang_login',lang('login'));
 
 		$tmpl->set_var('website_title', $GLOBALS['egw_info']['server']['site_title']);
@@ -669,7 +763,7 @@ abstract class egw_framework
 	*/
 	function denylogin_screen()
 	{
-		$tmpl = new Template($GLOBALS['egw_info']['server']['template_dir']);
+		$tmpl = new Template(EGW_SERVER_ROOT.$this->template_dir);
 
 		$tmpl->set_file(array(
 			'login_form' => 'login_denylogin.tpl'
@@ -760,6 +854,14 @@ abstract class egw_framework
 	 */
 	protected function _get_header(array $extra=array())
 	{
+		// display password expires in N days message once per session
+		$message = null;
+		if ($GLOBALS['egw_info']['flags']['currentapp'] != 'login' &&
+			auth::check_password_change($message) !== true)
+		{
+			self::message($message, 'info');
+		}
+
 		// get used language code (with a little xss check, if someone tries to sneak something in)
 		if (preg_match('/^[a-z]{2}(-[a-z]{2})?$/',$GLOBALS['egw_info']['user']['preferences']['common']['lang']))
 		{
@@ -1171,8 +1273,6 @@ abstract class egw_framework
 		$apps = array();
 		foreach($GLOBALS['egw_info']['user']['apps'] as $app => $data)
 		{
-// disable home until we get it working again
-if ($app == 'home') continue;
 			if (is_long($app))
 			{
 				continue;
@@ -1291,16 +1391,23 @@ if ($app == 'home') continue;
 			// eTemplate2 - load in top so sidebox has styles too
 			self::includeCSS('/etemplate/templates/default/etemplate2.css');
 
-			// Theme likes to override eTemplate2
-			$theme_css = $this->template_dir.'/css/'.$GLOBALS['egw_info']['user']['preferences']['common']['theme'].'.css';
-			if(!file_exists(EGW_SERVER_ROOT.$theme_css))
+			// Category styles
+			categories::css(categories::GLOBAL_APPNAME);
+
+			// For mobile user-agent we prefer mobile theme over selected one with a final fallback to theme named as template
+			$themes_to_check = array();
+			if (html::$ua_mobile) $themes_to_check[] = $this->template_dir.'/css/mobile.css';
+			$themes_to_check[] = $this->template_dir.'/css/'.$GLOBALS['egw_info']['user']['preferences']['common']['theme'].'.css';
+			$themes_to_check[] = $this->template_dir.'/css/'.$this->template.'.css';
+			foreach($themes_to_check as $theme_css)
 			{
-				$theme_css = $this->template_dir.'/css/'.$this->template.'.css';
+				if (file_exists(EGW_SERVER_ROOT.$theme_css)) break;
 			}
 			self::includeCSS($theme_css);
 
 			// search for app specific css file, so it can customize the theme
-			self::includeCSS($GLOBALS['egw_info']['flags']['currentapp'], 'app');
+			self::includeCSS($GLOBALS['egw_info']['flags']['currentapp'], 'app-'.$GLOBALS['egw_info']['user']['preferences']['common']['theme']) ||
+				self::includeCSS($GLOBALS['egw_info']['flags']['currentapp'], 'app');
 
 			// sending print css last, so it can overwrite anything
 			$print_css = $this->template_dir.'/print.css';
@@ -1320,11 +1427,13 @@ if ($app == 'home') continue;
 		{
 			foreach(self::resolve_css_includes($path) as $path)
 			{
-				if (($mod = filemtime(EGW_SERVER_ROOT.$path)) > $max_modified) $max_modified = $mod;
+				list($file,$query) = explode('?',$path,2);
+				if (($mod = filemtime(EGW_SERVER_ROOT.$file)) > $max_modified) $max_modified = $mod;
 
-				if ($debug_minify || substr($path, -8) == '/app.css')	// do NOT include app.css, as it changes from app to app
+				// do NOT include app.css or categories.php, as it changes from app to app
+				if ($debug_minify || substr($path, -8) == '/app.css' || substr($file,-14) == 'categories.php')
 				{
-					$css_files .= '<link href="'.$GLOBALS['egw_info']['server']['webserver_url'].$path.'?'.$mod.'" type="text/css" rel="StyleSheet" />'."\n";
+					$css_files .= '<link href="'.$GLOBALS['egw_info']['server']['webserver_url'].$path.($query ? '&' : '?').$mod.'" type="text/css" rel="StyleSheet" />'."\n";
 				}
 				else
 				{
@@ -1356,7 +1465,9 @@ if ($app == 'home') continue;
 	protected static function resolve_css_includes($path, &$pathes=array())
 	{
 		$matches = null;
-		if (($to_check = file_get_contents (EGW_SERVER_ROOT.$path, false, null, -1, 1024)) &&
+
+		list($file,$query) = explode('?',$path,2);
+		if (($to_check = file_get_contents (EGW_SERVER_ROOT.$file, false, null, -1, 1024)) &&
 			stripos($to_check, '/*@import') !== false && preg_match_all('|/\*@import url\("([^"]+)"|i', $to_check, $matches))
 		{
 			foreach($matches[1] as $import_path)
@@ -1567,8 +1678,8 @@ if ($app == 'home') continue;
 				'id'    => 'password',
 				'name'  => 'preferences',
 				'title' => lang('Password'),
-				'url'   => 'javascript:egw.open_link("'.
-					egw::link('/index.php?menuaction=preferences.preferences_password.change').'","_blank","400x270")',
+				'url'   => "javascript:egw.open_link('".
+					egw::link('/index.php?menuaction=preferences.preferences_password.change')."','_blank','400x270')",
 			));
 		}
 		/* disable help until content is reworked
@@ -1943,6 +2054,7 @@ if ($app == 'home') continue;
 			{
 				if (($bundle = $file2bundle[$file]))
 				{
+					//error_log(__METHOD__."() requiring boundle $bundle for $file");
 					if (!in_array($bundle, $included_bundles))
 					{
 						$max_modified = 0;
@@ -2066,6 +2178,11 @@ if ($app == 'home') continue;
 		$inc_mgr->include_js_file('/phpgwapi/js/egw_action/egw_dragdrop_dhtmlx_tree.js');
 		$inc_mgr->include_js_file('/phpgwapi/js/egw_action/egw_menu.js');
 		$inc_mgr->include_js_file('/phpgwapi/js/egw_action/egw_menu_dhtmlx.js');
+		// include choosen in api, as old eTemplate uses it and fail if it pulls in half of et2
+		$inc_mgr->include_js_file('/phpgwapi/js/jquery/chosen/chosen.jquery.js');
+		// include CKEditor in api, as old eTemplate uses it too
+		$inc_mgr->include_js_file('/phpgwapi/js/ckeditor/ckeditor.js');
+		$inc_mgr->include_js_file('/phpgwapi/js/ckeditor/config.js');
 		$bundles['api'] = $inc_mgr->get_included_files();
 		self::bundle_urls($bundles['api'], $api_max_mod);
 
@@ -2076,14 +2193,13 @@ if ($app == 'home') continue;
 		self::bundle_urls($bundles['et2'], $et2_max_mod);
 
 		// generate jdots bundle, if installed
+		/* switching jdots bundle off, as fw_pixelegg will cause whole jdots bundle incl. fw_jdots to include
 		if (file_exists(EGW_SERVER_ROOT.'/jdots'))
 		{
-			$inc_mgr->include_js_file('/jdots/js/egw_fw.js');
-			$inc_mgr->include_js_file('/jdots/js/egw_fw_ui.js');
-			$inc_mgr->include_js_file('/jdots/js/egw_fw_classes.js');
+			$inc_mgr->include_js_file('/jdots/js/fw_jdots.js');
 			$bundles['jdots'] = array_diff($inc_mgr->get_included_files(), call_user_func_array('array_merge', $bundles));
 			self::bundle_urls($bundles['jdots'], $jdots_max_mod);
-		}
+		}*/
 
 		// automatic split bundles with more then MAX_BUNDLE_FILES (=50) files
 		foreach($bundles as $name => $files)
@@ -2145,7 +2261,7 @@ if ($app == 'home') continue;
 		{
 			$path = $app;
 		}
-		if (!file_exists(EGW_SERVER_ROOT.$path))
+		if (!file_exists(EGW_SERVER_ROOT.$path) && !file_exists(EGW_SERVER_ROOT . parse_url($path,PHP_URL_PATH)))
 		{
 			//error_log(__METHOD__."($app,$name) $path NOT found!");
 			return false;
@@ -2173,7 +2289,8 @@ if ($app == 'home') continue;
 		$app = $GLOBALS['egw_info']['flags']['currentapp'];
 
 		// try to add app specific css file
-		self::includeCSS($app,'app');
+		self::includeCSS($app, 'app-'.$GLOBALS['egw_info']['user']['preferences']['common']['theme']) ||
+			self::includeCSS($app,'app');
 
 		// add all css files from egw_framework::includeCSS()
 		$query = null;
@@ -2181,7 +2298,7 @@ if ($app == 'home') continue;
 		{
 			unset($query);
 			list($path,$query) = explode('?',$path,2);
-			$path .= '?'. filemtime(EGW_SERVER_ROOT.$path).($query ? '&'.$query : '');
+			$path .= '?'. ($query ? $query : filemtime(EGW_SERVER_ROOT.$path));
 			$response->includeCSS($GLOBALS['egw_info']['server']['webserver_url'].$path);
 		}
 
