@@ -17,7 +17,7 @@
  * @package api
  * @subpackage session
  * @author Ralf Becker <ralfbecker@outdoor-training.de> since 2003 on
- * @version $Id: class.egw_session.inc.php 48249 2014-08-25 14:21:21Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -40,6 +40,8 @@ class egw_session
 {
 	/**
 	 * Write debug messages about session verification and creation to the error_log
+	 *
+	 * This will contain passwords! Don't leave it permanently switched on!
 	 */
 	const ERROR_LOG_DEBUG = false;
 
@@ -166,10 +168,28 @@ class egw_session
 	 */
 	var $required_files;
 
+	/**
+	 * Nummeric code why session creation failed
+	 *
+	 * @var int
+	 */
+	var $cd_reason;
+	const CD_BAD_LOGIN_OR_PASSWORD = 5;
+	const CD_FORCE_PASSWORD_CHANGE = 97;
+	const CD_ACCOUNT_EXPIRED = 98;
+	const CD_BLOCKED = 99;	// to many failed attempts to loing
+
+	/**
+	 * Verbose reason why session creation failed
+	 *
+	 * @var string
+	 */
+	var $reason;
+
 	/**
 	 * Constructor just loads up some defaults from cookies
 	 *
-	 * @param array $domain_names=null domain-names used in this install
+	 * @param array $domain_names =null domain-names used in this install
 	 */
 	function __construct(array $domain_names=null)
 	{
@@ -323,8 +343,8 @@ class egw_session
 	 *
 	 * @param array &$arr
 	 * @param string $label
-	 * @param boolean $recursion=true if true call itself for every item > $limit
-	 * @param int $limit=1000 log only differences > $limit
+	 * @param boolean $recursion =true if true call itself for every item > $limit
+	 * @param int $limit =1000 log only differences > $limit
 	 */
 	static function log_session_usage(&$arr,$label,$recursion=true,$limit=1000)
 	{
@@ -375,8 +395,8 @@ class egw_session
 	 *
 	 * @param string $kp3 mcrypt key transported via cookie or get parameter like the session id,
 	 *	unlike the session id it's not know on the server, so only the client-request can decrypt the session!
-	 * @param string $algo=self::MCRYPT_ALGO
-	 * @param string $mode=self::MCRYPT_MODE
+	 * @param string $algo =self::MCRYPT_ALGO
+	 * @param string $mode =self::MCRYPT_MODE
 	 * @return boolean true if encryption is used, false otherwise
 	 */
 	static private function init_crypt($kp3,$algo=self::MCRYPT_ALGO,$mode=self::MCRYPT_MODE)
@@ -420,194 +440,196 @@ class egw_session
 	 * @param string $login user login
 	 * @param string $passwd user password
 	 * @param string $passwd_type type of password being used, ie plaintext, md5, sha1
-	 * @param boolean $no_session_needed=false dont create a real session, eg. for GroupDAV clients using only basic auth, no cookie support
-	 * @param boolean $auth_check=true if false, the user is loged in without checking his password (eg. for single sign on), default = true
-	 * @return string session id
+	 * @param boolean $no_session =false dont create a real session, eg. for GroupDAV clients using only basic auth, no cookie support
+	 * @param boolean $auth_check =true if false, the user is loged in without checking his password (eg. for single sign on), default = true
+	 * @param boolean $fail_on_forced_password_change =false true: do NOT create session, if password change requested
+	 * @return string|boolean session id or false if session was not created, $this->(cd_)reason contains cause
 	 */
-	function create($login,$passwd = '',$passwd_type = '',$no_session=false,$auth_check=true)
+	function create($login,$passwd = '',$passwd_type = '',$no_session=false,$auth_check=true,$fail_on_forced_password_change=false)
 	{
-		if (is_array($login))
-		{
-			$this->login       = $login['login'];
-			$this->passwd      = $login['passwd'];
-			$this->passwd_type = $login['passwd_type'];
-			$login             = $this->login;
-		}
-		else
-		{
-			$this->login       = $login;
-			$this->passwd      = $passwd;
-			$this->passwd_type = $passwd_type;
-		}
-		if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) starting ...");
-
-		self::split_login_domain($login,$this->account_lid,$this->account_domain);
-		// add domain to the login, if not already there
-		if (substr($this->login,-strlen($this->account_domain)-1) != '@'.$this->account_domain)
-		{
-			$this->login .= '@'.$this->account_domain;
-		}
-		$now = time();
-		//error_log(__METHOD__."($login,$passwd,$passwd_type,$no_session,$auth_check) account_lid=$this->account_lid, account_domain=$this->account_domain, default_domain={$GLOBALS['egw_info']['server']['default_domain']}, user/domain={$GLOBALS['egw_info']['user']['domain']}");
-
-		// This is to ensure that we authenticate to the correct domain (might not be default)
-		// if no domain is given we use the default domain, so we dont need to re-create everything
-		if (!$GLOBALS['egw_info']['user']['domain'] && $this->account_domain == $GLOBALS['egw_info']['server']['default_domain'])
-		{
-			$GLOBALS['egw_info']['user']['domain'] = $this->account_domain;
-		}
-		elseif (!$this->account_domain && $GLOBALS['egw_info']['user']['domain'])
-		{
-			$this->account_domain = $GLOBALS['egw_info']['user']['domain'];
-		}
-		elseif($this->account_domain != $GLOBALS['egw_info']['user']['domain'])
-		{
-			throw new Exception("Wrong domain! '$this->account_domain' != '{$GLOBALS['egw_info']['user']['domain']}'");
-/*			$GLOBALS['egw']->ADOdb = null;
-			$GLOBALS['egw_info']['user']['domain'] = $this->account_domain;
-			// reset the db and all other (non-header!) egw_info/server data
-			$GLOBALS['egw_info']['server'] = array(
-				'sessions_type'  => $GLOBALS['egw_info']['server']['sessions_type'],
-				'default_domain' => $GLOBALS['egw_info']['server']['default_domain'],
-			);
-			$GLOBALS['egw_info']['server']['db_host'] = $GLOBALS['egw_domain'][$this->account_domain]['db_host'];
-			$GLOBALS['egw_info']['server']['db_port'] = $GLOBALS['egw_domain'][$this->account_domain]['db_port'];
-			$GLOBALS['egw_info']['server']['db_name'] = $GLOBALS['egw_domain'][$this->account_domain]['db_name'];
-			$GLOBALS['egw_info']['server']['db_user'] = $GLOBALS['egw_domain'][$this->account_domain]['db_user'];
-			$GLOBALS['egw_info']['server']['db_pass'] = $GLOBALS['egw_domain'][$this->account_domain]['db_pass'];
-			$GLOBALS['egw_info']['server']['db_type'] = $GLOBALS['egw_domain'][$this->account_domain]['db_type'];
-			$GLOBALS['egw']->setup('',false);
-*/
-		}
-		unset($GLOBALS['egw_info']['server']['default_domain']); // we kill this for security reasons
-
-		//echo "<p>session::create(login='$login'): lid='$this->account_lid', domain='$this->account_domain'</p>\n";
-		$user_ip = self::getuser_ip();
-
-		$this->account_id = $GLOBALS['egw']->accounts->name2id($this->account_lid,'account_lid','u');
-
-		if (($blocked = $this->login_blocked($login,$user_ip)) ||	// too many unsuccessful attempts
-			$GLOBALS['egw_info']['server']['global_denied_users'][$this->account_lid] ||
-			$auth_check && !$GLOBALS['egw']->auth->authenticate($this->account_lid, $this->passwd, $this->passwd_type) ||
-			$this->account_id && $GLOBALS['egw']->accounts->get_type($this->account_id) == 'g')
-		{
-			$this->reason = $blocked ? 'blocked, too many attempts' : 'bad login or password';
-			$this->cd_reason = $blocked ? 99 : 5;
-
-			// we dont log anon users as it would block the website
-			if (!$GLOBALS['egw']->acl->get_specific_rights_for_account($this->account_id,'anonymous','phpgwapi'))
+		try {
+			if (is_array($login))
 			{
-				$this->log_access($this->reason,$login,$user_ip,0);	// log unsuccessfull login
+				$this->login       = $login['login'];
+				$this->passwd      = $login['passwd'];
+				$this->passwd_type = $login['passwd_type'];
+				$login             = $this->login;
 			}
-			if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) UNSUCCESSFULL ($this->reason)");
-			return false;
-		}
-		if (!$this->account_id && $GLOBALS['egw_info']['server']['auto_create_acct'])
-		{
-			if ($GLOBALS['egw_info']['server']['auto_create_acct'] == 'lowercase')
+			else
 			{
-				$this->account_lid = strtolower($this->account_lid);
+				$this->login       = $login;
+				$this->passwd      = $passwd;
+				$this->passwd_type = $passwd_type;
 			}
-			$this->account_id = $GLOBALS['egw']->accounts->auto_add($this->account_lid, $passwd);
-		}
-		// fix maybe wrong case in username, it makes problems eg. in filemanager (name of homedir)
-		if ($this->account_lid != ($lid = $GLOBALS['egw']->accounts->id2name($this->account_id)))
-		{
-			$this->account_lid = $lid;
-			$this->login = $lid.substr($this->login,strlen($lid));
-		}
+			if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) starting ...");
 
-		$GLOBALS['egw_info']['user']['account_id'] = $this->account_id;
-		$GLOBALS['egw']->accounts->accounts($this->account_id);
-
-		// for WebDAV and GroupDAV we use a pseudo sessionid created from md5(user:passwd)
-		// --> allows this stateless protocolls which use basic auth to use sessions!
-		if (($this->sessionid = self::get_sessionid(true)))
-		{
-			$no_session = true;	// no need to set cookie
-			session_id($this->sessionid);
-		}
-		else
-		{
-			self::cache_control();
-			session_start();
-			// set a new session-id, if not syncml (already done in Horde code and can NOT be changed)
-			if (!$no_session && $GLOBALS['egw_info']['flags']['currentapp'] != 'syncml')
+			self::split_login_domain($login,$this->account_lid,$this->account_domain);
+			// add domain to the login, if not already there
+			if (substr($this->login,-strlen($this->account_domain)-1) != '@'.$this->account_domain)
 			{
-				session_regenerate_id(true);
+				$this->login .= '@'.$this->account_domain;
 			}
-			$this->sessionid = session_id();
-		}
-		$this->kp3       = common::randomstring(24);
+			$now = time();
+			//error_log(__METHOD__."($login,$passwd,$passwd_type,$no_session,$auth_check) account_lid=$this->account_lid, account_domain=$this->account_domain, default_domain={$GLOBALS['egw_info']['server']['default_domain']}, user/domain={$GLOBALS['egw_info']['user']['domain']}");
 
-		$GLOBALS['egw_info']['user'] = $this->read_repositories();
-		if ($GLOBALS['egw']->accounts->is_expired($GLOBALS['egw_info']['user']))
-		{
-			$this->reason = 'account is expired';
-			$this->cd_reason = 98;
-
-			if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) UNSUCCESSFULL ($this->reason)");
-			return false;
-		}
-
-		egw_cache::setSession('phpgwapi', 'password', base64_encode($this->passwd));
-
-		if ($GLOBALS['egw']->acl->check('anonymous',1,'phpgwapi'))
-		{
-			$this->session_flags = 'A';
-		}
-		else
-		{
-			$this->session_flags = 'N';
-		}
-
-		if (($hook_result = $GLOBALS['egw']->hooks->process(array(
-			'location'       => 'session_creation',
-			'sessionid'      => $this->sessionid,
-			'session_flags'  => $this->session_flags,
-			'account_id'     => $this->account_id,
-			'account_lid'    => $this->account_lid,
-			'passwd'         => $this->passwd,
-			'account_domain' => $this->account_domain,
-			'user_ip'        => $user_ip,
-		),'',true)))	// true = run hooks from all apps, not just the ones the current user has perms to run
-		{
-			foreach($hook_result as $app => $reason)
+			// This is to ensure that we authenticate to the correct domain (might not be default)
+			// if no domain is given we use the default domain, so we dont need to re-create everything
+			if (!$GLOBALS['egw_info']['user']['domain'] && $this->account_domain == $GLOBALS['egw_info']['server']['default_domain'])
 			{
-				if ($reason)	// called hook requests to deny the session
+				$GLOBALS['egw_info']['user']['domain'] = $this->account_domain;
+			}
+			elseif (!$this->account_domain && $GLOBALS['egw_info']['user']['domain'])
+			{
+				$this->account_domain = $GLOBALS['egw_info']['user']['domain'];
+			}
+			elseif($this->account_domain != $GLOBALS['egw_info']['user']['domain'])
+			{
+				throw new Exception("Wrong domain! '$this->account_domain' != '{$GLOBALS['egw_info']['user']['domain']}'");
+			}
+			unset($GLOBALS['egw_info']['server']['default_domain']); // we kill this for security reasons
+
+			//echo "<p>session::create(login='$login'): lid='$this->account_lid', domain='$this->account_domain'</p>\n";
+			$user_ip = self::getuser_ip();
+
+			$this->account_id = $GLOBALS['egw']->accounts->name2id($this->account_lid,'account_lid','u');
+
+			if (($blocked = $this->login_blocked($login,$user_ip)) ||	// too many unsuccessful attempts
+				$GLOBALS['egw_info']['server']['global_denied_users'][$this->account_lid] ||
+				$auth_check && !$GLOBALS['egw']->auth->authenticate($this->account_lid, $this->passwd, $this->passwd_type) ||
+				$this->account_id && $GLOBALS['egw']->accounts->get_type($this->account_id) == 'g')
+			{
+				$this->reason = $blocked ? 'blocked, too many attempts' : 'bad login or password';
+				$this->cd_reason = $blocked ? self::CD_BLOCKED : self::CD_BAD_LOGIN_OR_PASSWORD;
+
+				// we dont log anon users as it would block the website
+				if (!$GLOBALS['egw']->acl->get_specific_rights_for_account($this->account_id,'anonymous','phpgwapi'))
 				{
-					$this->reason = $this->cd_reason = $reason;
-					$this->log_access($this->reason,$login,$user_ip,0);		// log unsuccessfull login
-					if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) UNSUCCESSFULL ($this->reason)");
-					return false;
+					$this->log_access($this->reason,$login,$user_ip,0);	// log unsuccessfull login
+				}
+				if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) UNSUCCESSFULL ($this->reason)");
+				return false;
+			}
+			if ($fail_on_forced_password_change && auth::check_password_change($this->reason) === false)
+			{
+				$this->cd_reason = self::CD_FORCE_PASSWORD_CHANGE;
+				return false;
+			}
+			if (!$this->account_id && $GLOBALS['egw_info']['server']['auto_create_acct'])
+			{
+				if ($GLOBALS['egw_info']['server']['auto_create_acct'] == 'lowercase')
+				{
+					$this->account_lid = strtolower($this->account_lid);
+				}
+				$this->account_id = $GLOBALS['egw']->accounts->auto_add($this->account_lid, $passwd);
+			}
+			// fix maybe wrong case in username, it makes problems eg. in filemanager (name of homedir)
+			if ($this->account_lid != ($lid = $GLOBALS['egw']->accounts->id2name($this->account_id)))
+			{
+				$this->account_lid = $lid;
+				$this->login = $lid.substr($this->login,strlen($lid));
+			}
+
+			$GLOBALS['egw_info']['user']['account_id'] = $this->account_id;
+			$GLOBALS['egw']->accounts->accounts($this->account_id);
+
+			// for *DAV and eSync we use a pseudo sessionid created from md5(user:passwd)
+			// --> allows this stateless protocolls which use basic auth to use sessions!
+			if (($this->sessionid = self::get_sessionid(true)))
+			{
+				session_id($this->sessionid);
+			}
+			else
+			{
+				self::cache_control();
+				session_start();
+				// set a new session-id, if not syncml (already done in Horde code and can NOT be changed)
+				if (!$no_session && $GLOBALS['egw_info']['flags']['currentapp'] != 'syncml')
+				{
+					session_regenerate_id(true);
+				}
+				$this->sessionid = session_id();
+			}
+			$this->kp3       = common::randomstring(24);
+
+			$GLOBALS['egw_info']['user'] = $this->read_repositories();
+			if ($GLOBALS['egw']->accounts->is_expired($GLOBALS['egw_info']['user']))
+			{
+				$this->reason = 'account is expired';
+				$this->cd_reason = self::CD_ACCOUNT_EXPIRED;
+
+				if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) UNSUCCESSFULL ($this->reason)");
+				return false;
+			}
+
+			egw_cache::setSession('phpgwapi', 'password', base64_encode($this->passwd));
+
+			if ($GLOBALS['egw']->acl->check('anonymous',1,'phpgwapi'))
+			{
+				$this->session_flags = 'A';
+			}
+			else
+			{
+				$this->session_flags = 'N';
+			}
+
+			if (($hook_result = $GLOBALS['egw']->hooks->process(array(
+				'location'       => 'session_creation',
+				'sessionid'      => $this->sessionid,
+				'session_flags'  => $this->session_flags,
+				'account_id'     => $this->account_id,
+				'account_lid'    => $this->account_lid,
+				'passwd'         => $this->passwd,
+				'account_domain' => $this->account_domain,
+				'user_ip'        => $user_ip,
+			),'',true)))	// true = run hooks from all apps, not just the ones the current user has perms to run
+			{
+				foreach($hook_result as $reason)
+				{
+					if ($reason)	// called hook requests to deny the session
+					{
+						$this->reason = $this->cd_reason = $reason;
+						$this->log_access($this->reason,$login,$user_ip,0);		// log unsuccessfull login
+						if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) UNSUCCESSFULL ($this->reason)");
+						return false;
+					}
 				}
 			}
-		}
-		$GLOBALS['egw']->db->transaction_begin();
-		$this->register_session($this->login,$user_ip,$now,$this->session_flags);
-		if ($this->session_flags != 'A')		// dont log anonymous sessions
-		{
-			$this->sessionid_access_log = $this->log_access($this->sessionid,$login,$user_ip,$this->account_id);
-		}
-		self::appsession('account_previous_login','phpgwapi',$GLOBALS['egw']->auth->previous_login);
-		$GLOBALS['egw']->accounts->update_lastlogin($this->account_id,$user_ip);
-		$GLOBALS['egw']->db->transaction_commit();
+			$GLOBALS['egw']->db->transaction_begin();
+			$this->register_session($this->login,$user_ip,$now,$this->session_flags);
+			if ($this->session_flags != 'A')		// dont log anonymous sessions
+			{
+				$this->sessionid_access_log = $this->log_access($this->sessionid,$login,$user_ip,$this->account_id);
+			}
+			self::appsession('account_previous_login','phpgwapi',$GLOBALS['egw']->auth->previous_login);
+			$GLOBALS['egw']->accounts->update_lastlogin($this->account_id,$user_ip);
+			$GLOBALS['egw']->db->transaction_commit();
 
-		if ($GLOBALS['egw_info']['server']['usecookies'] && !$no_session)
-		{
-			self::egw_setcookie(self::EGW_SESSION_NAME,$this->sessionid);
-			self::egw_setcookie('kp3',$this->kp3);
-			self::egw_setcookie('domain',$this->account_domain);
-		}
-		if ($GLOBALS['egw_info']['server']['usecookies'] && !$no_session || isset($_COOKIE['last_loginid']))
-		{
-			self::egw_setcookie('last_loginid', $this->account_lid ,$now+1209600); /* For 2 weeks */
-			self::egw_setcookie('last_domain',$this->account_domain,$now+1209600);
-		}
-		//if (!$this->sessionid) echo "<p>session::create(login='$login') = '$this->sessionid': lid='$this->account_lid', domain='$this->account_domain'</p>\n";
-		if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) successfull sessionid=$this->sessionid");
+			if ($GLOBALS['egw_info']['server']['usecookies'] && !$no_session)
+			{
+				self::egw_setcookie(self::EGW_SESSION_NAME,$this->sessionid);
+				self::egw_setcookie('kp3',$this->kp3);
+				self::egw_setcookie('domain',$this->account_domain);
+			}
+			if ($GLOBALS['egw_info']['server']['usecookies'] && !$no_session || isset($_COOKIE['last_loginid']))
+			{
+				self::egw_setcookie('last_loginid', $this->account_lid ,$now+1209600); /* For 2 weeks */
+				self::egw_setcookie('last_domain',$this->account_domain,$now+1209600);
+			}
+			//if (!$this->sessionid) echo "<p>session::create(login='$login') = '$this->sessionid': lid='$this->account_lid', domain='$this->account_domain'</p>\n";
+			if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($this->login,$this->passwd,$this->passwd_type,$no_session,$auth_check) successfull sessionid=$this->sessionid");
 
-		return $this->sessionid;
+			return $this->sessionid;
+		}
+		// catch all exceptions, as their (allways logged) trace (eg. on a database error) would contain the user password
+		catch(Exception $e) {
+			$this->reason = $this->cd_reason = $e->getMessage();
+			error_log(__METHOD__."('$login', ".array2string(str_repeat('*', strlen($passwd))).
+				", '$passwd_type', no_session=".array2string($no_session).
+				", auth_check=".array2string($auth_check).
+				", fail_on_forced_password_change=".array2string($fail_on_forced_password_change).
+				") Exception ".$e->getMessage());
+			return false;
+		}
 	}
 
 	/**
@@ -649,9 +671,9 @@ class egw_session
     * Write or update (for logout) the access_log
 	*
 	* @param string|int $sessionid nummeric or PHP session id or 0 for unsuccessful logins
-	* @param string $login='' account_lid (evtl. with domain) or '' for setting the logout-time
-	* @param string $user_ip='' ip to log
-	* @param int $account_id=0 numerical account_id
+	* @param string $login ='' account_lid (evtl. with domain) or '' for setting the logout-time
+	* @param string $user_ip ='' ip to log
+	* @param int $account_id =0 numerical account_id
 	* @return int $sessionid primary key of egw_access_log for login, null otherwise
 	*/
 	private function log_access($sessionid,$login='',$user_ip='',$account_id=0)
@@ -708,7 +730,6 @@ class egw_session
 	 */
 	private function login_blocked($login,$ip)
 	{
-		$blocked = false;
 		$block_time = time() - $GLOBALS['egw_info']['server']['block_time'] * 60;
 
 		$false_id = $false_ip = 0;
@@ -751,21 +772,21 @@ class egw_session
 		if ($blocked && $GLOBALS['egw_info']['server']['admin_mails'] &&
 			$GLOBALS['egw_info']['server']['login_blocked_mail_time'] < time()-5*60)	// max. one mail every 5mins
 		{
-			// notify admin(s) via email
-			$from    = 'eGroupWare@'.$GLOBALS['egw_info']['server']['mail_suffix'];
-			$subject = lang("eGroupWare: login blocked for user '%1', IP %2",$login,$ip);
-			$body    = lang("Too many unsucessful attempts to login: %1 for the user '%2', %3 for the IP %4",$false_id,$login,$false_ip,$ip);
-
-			$admin_mails = explode(',',$GLOBALS['egw_info']['server']['admin_mails']);
-			foreach($admin_mails as $to)
-			{
-				try {
-						$GLOBALS['egw']->send->msg('email',$to,$subject,$body,'','','',$from,$from);
-				}
-				catch(Exception $e) {
-					// ignore exception, but log it, to block the account and give a correct error-message to user
-					error_log(__METHOD__."('$log', '$ip') ".$e->getMessage());
+			try {
+				$mailer = new egw_mailer();
+				// notify admin(s) via email
+				$mailer->setFrom('eGroupWare@'.$GLOBALS['egw_info']['server']['mail_suffix']);
+				$mailer->addHeader('Subject', lang("eGroupWare: login blocked for user '%1', IP %2",$login,$ip));
+				$mailer->setBody(lang("Too many unsucessful attempts to login: %1 for the user '%2', %3 for the IP %4",$false_id,$login,$false_ip,$ip));
+				foreach(preg_split('/,\s*/',$GLOBALS['egw_info']['server']['admin_mails']) as $mail)
+				{
+					$mailer->addAddress($mail);
 				}
+				$mailer->send();
+			}
+			catch(Exception $e) {
+				// ignore exception, but log it, to block the account and give a correct error-message to user
+				error_log(__METHOD__."('$login', '$ip') ".$e->getMessage());
 			}
 			// save time of mail, to not send to many mails
 			$config = new config('phpgwapi');
@@ -776,10 +797,19 @@ class egw_session
 		return $blocked;
 	}
 
+	/**
+	 * Basename of scripts for which we create a pseudo session-id based on user-credentials
+	 *
+	 * @var array
+	 */
+	static $pseudo_session_scripts = array(
+		'webdav.php', 'groupdav.php', 'remote.php', 'share.php'
+	);
+
 	/**
 	 * Get the sessionid from Cookie, Get-Parameter or basic auth
 	 *
-	 * @param boolean $only_basic_auth=false return only a basic auth pseudo sessionid, default no
+	 * @param boolean $only_basic_auth =false return only a basic auth pseudo sessionid, default no
 	 * @return string
 	 */
 	static function get_sessionid($only_basic_auth=false)
@@ -787,25 +817,27 @@ class egw_session
 		// for WebDAV and GroupDAV we use a pseudo sessionid created from md5(user:passwd)
 		// --> allows this stateless protocolls which use basic auth to use sessions!
 		if (isset($_SERVER['PHP_AUTH_USER']) && isset($_SERVER['PHP_AUTH_PW']) &&
-			(in_array(basename($_SERVER['SCRIPT_NAME']),array('webdav.php','groupdav.php','remote.php')) ||
+			(in_array(basename($_SERVER['SCRIPT_NAME']), self::$pseudo_session_scripts) ||
 				$_SERVER['SCRIPT_NAME'] === '/Microsoft-Server-ActiveSync'))
 		{
 			// we generate a pseudo-sessionid from the basic auth credentials
 			$sessionid = md5($_SERVER['PHP_AUTH_USER'].':'.$_SERVER['PHP_AUTH_PW'].':'.$_SERVER['HTTP_HOST'].':'.
 				EGW_SERVER_ROOT.':'.self::getuser_ip().':'.filemtime(EGW_SERVER_ROOT.'/phpgwapi/setup/setup.inc.php').
 				// for ActiveSync we add the DeviceID
-				(isset($_GET['DeviceId']) && $_SERVER['SCRIPT_NAME'] === '/Microsoft-Server-ActiveSync' ? ':'.$_GET['DeviceId'] : ''));
+				(isset($_GET['DeviceId']) && $_SERVER['SCRIPT_NAME'] === '/Microsoft-Server-ActiveSync' ? ':'.$_GET['DeviceId'] : '').
+				':'.$_SERVER['HTTP_USER_AGENT']);
 			//error_log(__METHOD__."($only_basic_auth) HTTP_HOST=$_SERVER[HTTP_HOST], PHP_AUTH_USER=$_SERVER[PHP_AUTH_USER], DeviceId=$_GET[DeviceId]: sessionid=$sessionid");
 		}
 		// same for digest auth
 		elseif (isset($_SERVER['PHP_AUTH_DIGEST']) &&
-			in_array(basename($_SERVER['SCRIPT_NAME']),array('webdav.php','groupdav.php','remote.php')))
+			in_array(basename($_SERVER['SCRIPT_NAME']), self::$pseudo_session_scripts))
 		{
 			// we generate a pseudo-sessionid from the digest username, realm and nounce
 			// can't use full $_SERVER['PHP_AUTH_DIGEST'], as it changes (contains eg. the url)
 			$data = egw_digest_auth::parse_digest($_SERVER['PHP_AUTH_DIGEST']);
 			$sessionid = md5($data['username'].':'.$data['realm'].':'.$data['nonce'].':'.$_SERVER['HTTP_HOST'].
-				EGW_SERVER_ROOT.':'.self::getuser_ip().':'.filemtime(EGW_SERVER_ROOT.'/phpgwapi/setup/setup.inc.php'));
+				EGW_SERVER_ROOT.':'.self::getuser_ip().':'.filemtime(EGW_SERVER_ROOT.'/phpgwapi/setup/setup.inc.php').
+				':'.$_SERVER['HTTP_USER_AGENT']);
 		}
 		elseif(!$only_basic_auth && isset($_REQUEST[self::EGW_SESSION_NAME]))
 		{
@@ -897,20 +929,6 @@ class egw_session
 		if($GLOBALS['egw_info']['user']['domain'] && $this->account_domain != $GLOBALS['egw_info']['user']['domain'])
 		{
 			return false;	// session not verified, domain changed
-
-			throw new Exception("Wrong domain! '$this->account_domain' != '{$GLOBALS['egw_info']['user']['domain']}'");
-/*			if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."('$sessionid','$kp3') account_domain='$this->account_domain' != '{$GLOBALS['egw_info']['user']['domain']}'=egw_info[user][domain]");
-			$GLOBALS['egw']->ADOdb = null;
-			$GLOBALS['egw_info']['user']['domain'] = $this->account_domain;
-			// reset the db
-			$GLOBALS['egw_info']['server']['db_host'] = $GLOBALS['egw_domain'][$this->account_domain]['db_host'];
-			$GLOBALS['egw_info']['server']['db_port'] = $GLOBALS['egw_domain'][$this->account_domain]['db_port'];
-			$GLOBALS['egw_info']['server']['db_name'] = $GLOBALS['egw_domain'][$this->account_domain]['db_name'];
-			$GLOBALS['egw_info']['server']['db_user'] = $GLOBALS['egw_domain'][$this->account_domain]['db_user'];
-			$GLOBALS['egw_info']['server']['db_pass'] = $GLOBALS['egw_domain'][$this->account_domain]['db_pass'];
-			$GLOBALS['egw_info']['server']['db_type'] = $GLOBALS['egw_domain'][$this->account_domain]['db_type'];
-			$GLOBALS['egw']->setup('',false);
-*/
 		}
 		$GLOBALS['egw_info']['user']['kp3'] = $this->kp3;
 
@@ -1022,7 +1040,7 @@ class egw_session
 		}
 		$this->log_access($sessionid);	// log logout-time
 
-		if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($sessionid,$kp3) parent::destroy()=$ret");
+		if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($sessionid,$kp3)");
 
 		if (is_numeric($sessionid))	// do we have a access-log-id --> get PHP session id
 		{
@@ -1127,7 +1145,7 @@ class egw_session
 		}
 
 		// check if the url already contains a query and ensure that vars is an array and all strings are in extravars
-		list($url,$othervars) = explode('?',$url,2);
+		list($ret_url,$othervars) = explode('?', $url, 2);
 		if ($extravars && is_array($extravars))
 		{
 			$vars += $extravars;
@@ -1174,18 +1192,17 @@ class egw_session
 					$query[] = $key.'='.urlencode($value);
 				}
 			}
-			$url .= '?' . implode('&',$query);
+			$ret_url .= '?' . implode('&',$query);
 		}
-		//echo " = '$url'</p>\n";
-		return $url;
+		return $ret_url;
 	}
 
 	/**
 	 * Stores or retrieve applications data in/form the eGW session
 	 *
 	 * @param string $location free lable to store the data
-	 * @param string $appname='' default current application (egw_info[flags][currentapp])
-	 * @param mixed $data='##NOTHING##' if given, data to store, if not specified
+	 * @param string $appname ='' default current application (egw_info[flags][currentapp])
+	 * @param mixed $data ='##NOTHING##' if given, data to store, if not specified
 	 * @deprecated use egw_cache::setSession($appname, $location, $data) or egw_cache::getSession($appname, $location)
 	 * @return mixed session data or false if no data stored for $appname/$location
 	 */
@@ -1251,13 +1268,21 @@ class egw_session
 	 */
 	private static $cookie_path = '/';
 
+	/**
+	 * iOS web-apps will loose cookie if set with a livetime of 0 / session-cookie
+	 *
+	 * Therefore we set a fixed lifetime of 24h from session-start instead.
+	 * Server-side session will timeout earliert anyway, if there's no activity.
+	 */
+	const IOS_SESSION_COOKIE_LIFETIME = 86400;
+
 	/**
 	 * Set a cookie with eGW's cookie-domain and -path settings
 	 *
 	 * @param string $cookiename name of cookie to be set
-	 * @param string $cookievalue='' value to be used, if unset cookie is cleared (optional)
-	 * @param int $cookietime=0 when cookie should expire, 0 for session only (optional)
-	 * @param string $cookiepath=null optional path (eg. '/') if the eGW install-dir should not be used
+	 * @param string $cookievalue ='' value to be used, if unset cookie is cleared (optional)
+	 * @param int $cookietime =0 when cookie should expire, 0 for session only (optional)
+	 * @param string $cookiepath =null optional path (eg. '/') if the eGW install-dir should not be used
 	 */
 	public static function egw_setcookie($cookiename,$cookievalue='',$cookietime=0,$cookiepath=null)
 	{
@@ -1267,14 +1292,19 @@ class egw_session
 		}
 		if (self::ERROR_LOG_DEBUG) error_log(__METHOD__."($cookiename,$cookievalue,$cookietime,$cookiepath,".self::$cookie_domain.")");
 
+		// if we are installed in iOS as web-app, we must not set a cookietime==0 (session-cookie),
+		// as every change between apps will cause the cookie to get lost
+		static $is_iOS = null;
+		if (!$cookietime && !isset($is_iOS)) $is_iOS = (bool)preg_match('/^(iPhone|iPad|iPod)/i', html::$ua_mobile);
+
 		if(!headers_sent())	// gives only a warning, but can not send the cookie anyway
 		{
-			$rv = setcookie($cookiename,$cookievalue,$cookietime,
+			setcookie($cookiename, $cookievalue,
+				!$cookietime && $is_iOS ? time()+self::IOS_SESSION_COOKIE_LIFETIME : $cookietime,
 				is_null($cookiepath) ? self::$cookie_path : $cookiepath,self::$cookie_domain,
 				// if called via HTTPS, only send cookie for https and only allow cookie access via HTTP (true)
 				empty($GLOBALS['egw_info']['server']['insecure_cookies']) && !empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off', true);
 		}
-		//error_log(__METHOD__." $cookiename->$cookievalue".' returned:'.print_r($rv,true).print_r($_COOKIE,true));
 	}
 
 	/**
@@ -1293,6 +1323,7 @@ class egw_session
 			self::$cookie_domain = isset($_SERVER['HTTP_X_FORWARDED_HOST']) ?  $_SERVER['HTTP_X_FORWARDED_HOST'] : $_SERVER['HTTP_HOST'];
 		}
 		// remove port from HTTP_HOST
+		$arr = null;
 		if (preg_match("/^(.*):(.*)$/",self::$cookie_domain,$arr))
 		{
 			self::$cookie_domain = $arr[1];
@@ -1305,11 +1336,11 @@ class egw_session
 		if (!$GLOBALS['egw_info']['server']['cookiepath'] ||
 			!(self::$cookie_path = parse_url($GLOBALS['egw_info']['server']['webserver_url'],PHP_URL_PATH)))
 		{
-			 self::$cookie_path = '/';
+			self::$cookie_path = '/';
 		}
 		//echo "<p>cookie_path='self::$cookie_path', cookie_domain='self::$cookie_domain'</p>\n";
 
-		session_set_cookie_params(0, $path, $domain,
+		session_set_cookie_params(0, self::$cookie_path, self::$cookie_domain,
 			// if called via HTTPS, only send cookie for https and only allow cookie access via HTTP (true)
 			empty($GLOBALS['egw_info']['server']['insecure_cookies']) && !empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off', true);
 	}
@@ -1320,8 +1351,8 @@ class egw_session
 	 * @param string $login on login $_POST['login'], $_SERVER['PHP_AUTH_USER'] or $_SERVER['REMOTE_USER']
 	 * @param string $domain_requested usually self::get_request('domain')
 	 * @param string &$default_domain usually $default_domain get's set eg. by sitemgr
-	 * @param string|array $server_name usually array($_SERVER['HTTP_HOST'], $_SERVER['SERVER_NAME'])
-	 * @param array $domains=null defaults to $GLOBALS['egw_domain'] from the header
+	 * @param string|array $server_names usually array($_SERVER['HTTP_HOST'], $_SERVER['SERVER_NAME'])
+	 * @param array $domains =null defaults to $GLOBALS['egw_domain'] from the header
 	 * @return string $GLOBALS['egw_info']['user']['domain'] set with the domain/instance to use
 	 */
 	public static function search_instance($login,$domain_requested,&$default_domain,$server_names,array $domains=null)
@@ -1342,9 +1373,9 @@ class egw_session
 				}
 				else
 				{
-					$domain_part = explode('.',$server_name);
-					array_shift($domain_part);
-					$domain_part = implode('.',$domain_part);
+					$parts = explode('.', $server_name);
+					array_shift($parts);
+					$domain_part = implode('.', $parts);
 					if(isset($domains[$domain_part]))
 					{
 						$default_domain = $domain_part;
@@ -1385,7 +1416,7 @@ class egw_session
 	/**
 	 * Update session_action and session_dla (session last used time)
 	 *
-	 * @param boolean $update_access_log=true false: dont update egw_access_log table
+	 * @param boolean $update_access_log =true false: dont update egw_access_log table
 	 * @return string action as written to egw_access_log.session_action
 	 */
 	private function update_dla($update_access_log=true)
@@ -1538,7 +1569,7 @@ class egw_session
 	 *
 	 * @param string $user username
 	 * @param string $password password or md5 hash of password if $allow_password_md5
-	 * @param boolean $allow_password_md5=false can password alread be an md5 hash
+	 * @param boolean $allow_password_md5 =false can password alread be an md5 hash
 	 * @return string
 	 */
 	static function user_pw_hash($user,$password,$allow_password_md5=false)
@@ -1598,7 +1629,7 @@ class egw_session
 	 * - true --> private caching by browser (no expires header)
 	 * - "public" or integer --> public caching with given cache_expire in minutes or php.ini default session_cache_expire
 	 *
-	 * @param int $expire=null expiration time in seconds, default $GLOBALS['egw_info']['flags']['nocachecontrol'] or php.ini session.cache_expire
+	 * @param int $expire =null expiration time in seconds, default $GLOBALS['egw_info']['flags']['nocachecontrol'] or php.ini session.cache_expire
 	 */
 	public static function cache_control($expire=null)
 	{
@@ -1635,6 +1666,7 @@ class egw_session
 			if ($expire && (session_cache_limiter() !== ($expire===true?'private_no_expire':'public') ||
 				is_int($expire) && $expire/60 !== session_cache_expire()))
 			{
+				$file = $line = null;
 				if (headers_sent($file, $line))
 				{
 					error_log(__METHOD__."($expire) called, but header already sent in $file: $line");
@@ -1667,10 +1699,10 @@ class egw_session
 	 * Get a session list (of the current instance)
 	 *
 	 * @param int $start
-	 * @param string $sort='DESC' ASC or DESC
-	 * @param string $order='session_dla' session_lid, session_id, session_started, session_logintime, session_action, or (default) session_dla
-	 * @param boolean $all_no_sort=False skip sorting and limiting to maxmatchs if set to true
-	 * @param array $filter=array() extra filter for sessions
+	 * @param string $sort ='DESC' ASC or DESC
+	 * @param string $order ='session_dla' session_lid, session_id, session_started, session_logintime, session_action, or (default) session_dla
+	 * @param boolean $all_no_sort =False skip sorting and limiting to maxmatchs if set to true
+	 * @param array $filter =array() extra filter for sessions
 	 * @return array with sessions (values for keys as in $sort)
 	 */
 	public static function session_list($start,$sort='DESC',$order='session_dla',$all_no_sort=False,array $filter=array())
@@ -1758,6 +1790,6 @@ class egw_session
 	 */
 	function delete_cache($accountid='')
 	{
-
+		unset($accountid);	// not used, but required by function signature
 	}
 }
