@@ -7,16 +7,20 @@
  * @package api
  * @subpackage groupdav
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @copyright (c) 2007-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
- * @version $Id: class.groupdav.inc.php 51368 2015-01-23 09:40:19Z ralfbecker $
+ * @copyright (c) 2007-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
+ * @version $Id$
  */
 
+if (strpos(ini_get('include_path'), EGW_API_INC) === false)
+{
+	ini_set('include_path', EGW_API_INC.PATH_SEPARATOR.ini_get('include_path'));
+}
 require_once('HTTP/WebDAV/Server.php');
 
 /**
  * EGroupware: GroupDAV access
  *
- * Using a modified PEAR HTTP/WebDAV/Server class from egw-pear!
+ * Using a modified PEAR HTTP/WebDAV/Server class from API!
  *
  * One can use the following url's releative (!) to http://domain.com/egroupware/groupdav.php
  *
@@ -29,6 +33,7 @@ require_once('HTTP/WebDAV/Server.php');
  * - /<current-username>/addressbook-<other-username>/ shared addressbooks from other user or group
  * - /<current-username>/addressbook-accounts/ all accounts current user has rights to see
  * - /<username>/calendar/    calendar of user <username> given the user has rights to view it
+ * - /<username>/calendar/?download download whole calendar as .ics file (GET request!)
  * - /<current-username>/calendar-<other-username>/ shared calendar from other user or group (only current <username>!)
  * - /<username>/inbox/       scheduling inbox of user <username>
  * - /<username>/outbox/      scheduling outbox of user <username>
@@ -212,6 +217,9 @@ class groupdav extends HTTP_WebDAV_Server
 
 	function __construct()
 	{
+		// log which CalDAVTester test is currently running, set as User-Agent header
+		if (substr($_SERVER['HTTP_USER_AGENT'], 0, 14) == 'scripts/tests/') error_log('****** '.$_SERVER['HTTP_USER_AGENT']);
+
 		if (!$this->debug) $this->debug = (int)$GLOBALS['egw_info']['user']['preferences']['groupdav']['debug_level'];
 
 		if ($this->debug > 2) error_log('groupdav: $_SERVER='.array2string($_SERVER));
@@ -378,6 +386,7 @@ class groupdav extends HTTP_WebDAV_Server
 					'location' => 'groupdav_root_props',
 					'props' => &$files['files'][0]['props'],
 					'options' => $options,
+					'caldav' => $this,
 				));
 			}
 
@@ -412,8 +421,10 @@ class groupdav extends HTTP_WebDAV_Server
 				$files['files'][0] = $this->add_app($app,$app=='addressbook'&&$handler->get_agent()=='kde',$user,
 					$this->_slashify($options['path']));
 
-				// Hack for iOS 5.0.1 addressbook to stop asking directory gateway permissions with depth=1
-				if ($method == 'PROPFIND' && $options['path'] == '/addressbook/' && $handler->get_agent() == 'dataaccess')
+				// Hack for iOS 5.0.1 addressbook to stop asking directory gateway permissions with depth != 0
+				// values for depth are 0, 1, "infinit" or not set which has to be interpreted as "infinit"
+				if ($method == 'PROPFIND' && $options['path'] == '/addressbook/' &&
+					(!isset($options['depth']) || $options['depth']) && $handler->get_agent() == 'dataaccess')
 				{
 					$this->log(__CLASS__."::$method(".array2string($options).') Enabling hack for iOS 5.0.1 addressbook: force Depth: 0 on PROPFIND for directory gateway!');
 					return true;
@@ -1298,6 +1309,19 @@ class groupdav extends HTTP_WebDAV_Server
 				}
 				else
 				{
+					// check for rename of attachment via Content-Disposition:filename=
+					if (isset($this->_SERVER['HTTP_CONTENT_DISPOSITION']) &&
+						substr($this->_SERVER['HTTP_CONTENT_DISPOSITION'], 0, 10) === 'attachment' &&
+						preg_match('/filename="?([^";]+)/', $this->_SERVER['HTTP_CONTENT_DISPOSITION'], $matches) &&
+						($filename = egw_vfs::basename($matches[1])) != egw_vfs::basename($path))
+					{
+						$old_path = $path;
+						if (!egw_vfs::rename($old_path, $path = egw_vfs::concat(egw_vfs::dirname($path), $filename)))
+						{
+							self::xml_error(self::mkprop(self::CALDAV, 'valid-managed-id-parameter', ''));
+							return '403 Forbidden';
+						}
+					}
 					if (!($to = egw_vfs::fopen($path, 'w')) ||
 						isset($options['stream']) && ($copied=stream_copy_to_stream($options['stream'], $to)) === false ||
 						isset($options['content']) && ($copied=fwrite($to, $options['content'])) === false)
@@ -1340,6 +1364,7 @@ class groupdav extends HTTP_WebDAV_Server
 	 * @param int|string $id
 	 * @param array $attach array of array with values for keys 'name', 'params', 'value'
 	 * @param boolean $delete_via_put
+	 * @return boolean false on error, eg. invalid managed id, for false an xml-error body has been send
 	 */
 	public static function handle_attach($app, $id, $attach, $delete_via_put=false)
 	{
@@ -1376,10 +1401,37 @@ class groupdav extends HTTP_WebDAV_Server
 		// turn inline attachments into managed ones
 		foreach($attach as $key => $attr)
 		{
-			if ($attr['params']['VALUE'] === 'BINARY')
+			if (!empty($attr['params']['FMTTYPE']))
 			{
+				if (isset($attr['params']['MANAGED-ID']))
+				{
+					// invalid managed-id
+					if (!($path = self::managed_id2path($attr['params']['MANAGED-ID'])) || !egw_vfs::is_readable($path))
+					{
+						error_log(__METHOD__."('$app', $id, ...) invalid MANAGED-ID ".array2string($attr));
+						self::xml_error(self::mkprop(self::CALDAV, 'valid-managed-id', ''));
+						return false;
+					}
+					if($path == ($link = egw_link::vfs_path($app, $id, egw_vfs::basename($path))))
+					{
+						error_log(__METHOD__."('$app', $id, ...) trying to modify existing MANAGED-ID --> ignored! ".array2string($attr));
+						continue;
+					}
+					// reuse valid managed-id --> symlink attachment
+					if (egw_vfs::file_exists($link))
+					{
+						if (egw_vfs::readlink($link) === $path) continue;	// no need to recreate identical link
+						egw_vfs::unlink($link);		// symlink will fail, if $link exists
+					}
+					if (!egw_vfs::symlink($path, $link))
+					{
+						error_log(__METHOD__."('$app', $id, ...) failed to symlink($path, $link) --> ignored!");
+					}
+					continue;
+				}
 				if (!($to = self::fopen_attachment($app, $id, $filename=$attr['params']['FILENAME'], $attr['params']['FMTTYPE'], $path)) ||
-					($copied=fwrite($to, $attr['value'])) === false)
+					// Horde Icalendar does NOT decode automatic
+					($copied=fwrite($to, $attr['params']['ENCODING'] == 'BASE64' ? base64_decode($attr['value']) : $attr['value'])) === false)
 				{
 					error_log(__METHOD__."('$app', $id, ...) failed to add attachment ".array2string($attr).") ");
 					continue;
@@ -1474,17 +1526,28 @@ class groupdav extends HTTP_WebDAV_Server
 			'need_mime' => true,
 		), true) as $path => $stat)
 		{
+			// handle symlinks --> return target size and mime-type
+			if (($target = egw_vfs::readlink($path)))
+			{
+				if (!($stat = egw_vfs::stat($target))) continue;	// broken or inaccessible symlink
+
+				// check if target is in /apps, probably reused MANAGED-ID --> return it
+				if (substr($target, 0, 6) == '/apps/')
+				{
+					$path = $target;
+				}
+			}
 			$attributes['ATTACH'][] = self::path2location($path);
 			$parameters['ATTACH'][] = array(
 				'MANAGED-ID' => groupdav::path2managed_id($path),
 				'FMTTYPE'    => $stat['mime'],
-				'SIZE'       => $stat['size'],
+				'SIZE'       => (string)$stat['size'],	// Horde_Icalendar renders int as empty string
 				'FILENAME'   => egw_vfs::basename($path),
 			);
+			// if we have attachments, set X-attribute to enable deleting them by put
+			// (works around events synced before without ATTACH attributes)
+			$attributes['X-EGROUPWARE-ATTACH-INCLUDED'] = 'TRUE';
 		}
-		// if we have attachments, set X-attribute to enable deleting them by put
-		// (works around events synced before without ATTACH attributes)
-		if ($attributes['ATTACH']) $attributes['X-EGROUPWARE-ATTACH-INCLUDED'] = 'TRUE';
 	}
 
 	/**
@@ -1653,9 +1716,13 @@ class groupdav extends HTTP_WebDAV_Server
 			if (is_bool($status)) $status = $status ? '204 No Content' : '400 Something went wrong';
 
 			// check/handle Prefer: return-representation
-			if ($status[0] === '2')
+			if ($status[0] === '2' || $status === true)
 			{
-				$handler->check_return_representation($options, $id, $user);
+				// we can NOT use 204 No content (forbidds a body) with return=representation, therefore we need to use 200 Ok instead!
+				if ($handler->check_return_representation($options, $id, $user) && (int)$status == 204)
+				{
+					$status = '200 Ok';
+				}
 			}
 			return $status;
 		}
