@@ -6,10 +6,10 @@
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de> complete rewrite in 6/2006 and earlier modifications
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
  * @author RalfBecker-AT-outdoor-training.de
- * @copyright 2001-2009 by RalfBecker@outdoor-training.de
+ * @copyright 2001-2014 by RalfBecker@outdoor-training.de
  * @package api
  * @subpackage html
- * @version $Id: class.html.inc.php 51186 2015-01-09 19:00:13Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -25,8 +25,8 @@ class html
 	 */
 	static $user_agent;
 	/**
-	 * User agent is a mobile browser
-	 * @var boolean
+	 * User agent is mobile browser: "iphone", "ipod", "ipad", "android", "symbianos", "blackberry", "kindle", "opera mobi", "windows phone"
+	 * @var string with name of mobile browser or null, if not mobile browser
 	 */
 	static $ua_mobile;
 	/**
@@ -62,11 +62,19 @@ class html
 	static function _init_static()
 	{
 		// should be Ok for all HTML 4 compatible browsers
-		$parts = null;
+		$parts = $all_parts = null;
 		if(!preg_match('/compatible; ([a-z]+)[\/ ]+([0-9.]+)/i',$_SERVER['HTTP_USER_AGENT'],$parts))
 		{
-			preg_match_all('/([a-z]+)\/([0-9.]+)/i',$_SERVER['HTTP_USER_AGENT'],$parts,PREG_SET_ORDER);
-			$parts = $parts[2][1] == 'Chrome' ? $parts[2] : array_pop($parts);
+			preg_match_all('/([a-z]+)\/([0-9.]+)/i',$_SERVER['HTTP_USER_AGENT'],$all_parts,PREG_SET_ORDER);
+			$parts = array_pop($all_parts);
+			foreach($all_parts as $p)
+			{
+				if ($p[1] == 'Chrome' && $parts[1] != 'Edge')
+				{
+					$parts = $p;
+					break;
+				}
+			}
 		}
 		list(,self::$user_agent,self::$ua_version) = $parts;
 		if ((self::$user_agent = strtolower(self::$user_agent)) == 'version') self::$user_agent = 'opera';
@@ -78,7 +86,19 @@ class html
 			self::$ua_version = preg_match('|Trident/[0-9.]+; rv:([0-9.]+)|i', $_SERVER['HTTP_USER_AGENT'], $matches) ?
 				$matches[1] : 11.0;
 		}
-		self::$ua_mobile = preg_match('/(iPhone|iPad|Android|SymbianOS)/i',$_SERVER['HTTP_USER_AGENT']);
+		// iceweasel is based on mozilla and we treat it like as firefox
+		if (self::$user_agent == 'iceweasel')
+		{
+			self::$user_agent = 'firefox';
+		}
+		// MS Edge sometimes reports just "Mozilla/5.0 (Windows NT 10.0; Win64; x64)"
+		if (self::$user_agent == 'mozilla' && self::$ua_version == '5.0')
+		{
+			self::$user_agent = 'edge';
+			self::$ua_version = '12.0';
+		}
+		self::$ua_mobile = preg_match('/(iPhone|iPod|iPad|Android|SymbianOS|Blackberry|Kindle|Opera Mobi|Windows Phone)/i',
+			$_SERVER['HTTP_USER_AGENT'], $matches) ? strtolower($matches[1]) : null;
 
 		self::$netscape4 = self::$user_agent == 'mozilla' && self::$ua_version < 5;
 		self::$prefered_img_title = self::$netscape4 ? 'alt' : 'title';
@@ -97,9 +117,9 @@ class html
 	* Please note: it need to be called before the call to egw_header() !!!
 	*
 	* @param string $name the name of the input-field
-	* @param string $value='' the actual value for the input-field, default ''
-	* @param string $title='' tooltip/title for the picker-activation-icon
-	* @param string $options='' options for input
+	* @param string $value ='' the actual value for the input-field, default ''
+	* @param string $title ='' tooltip/title for the picker-activation-icon
+	* @param string $options ='' options for input
 	* @return string the html
 	*/
 	static function inputColor($name,$value='',$title='',$options='')
@@ -179,30 +199,84 @@ class html
 
 		// spamsaver emailaddress
 		$result = preg_replace('/'.$NotAnchor.'mailto:([a-z0-9._-]+)@([a-z0-9_-]+)\.([a-z0-9._-]+)/i',
-			'<a href="#" onclick="document.location=\'mai\'+\'lto:\\1\'+unescape(\'%40\')+\'\\2.\\3\'; return false;">\\1 AT \\2 DOT \\3</a>',
+//			'<a href="#" onclick="document.location=\'mai\'+\'lto:\\1\'+unescape(\'%40\')+\'\\2.\\3\'; return false;">\\1 AT \\2 DOT \\3</a>',
+			"<a href=\"mailto:$1@$2.$3\" target=\"_blank\">$1 AT $2 DOT $3</a>",
 			$content);
 
 		//  First match things beginning with http:// (or other protocols)
+		$optBracket0 = '(<|&lt;)';
 		$Protocol = '(http:\/\/|(ftp:\/\/|https:\/\/))';	// only http:// gets removed, other protocolls are shown
 		$Domain = '([\w-]+\.[\w-.]+)';
 		$Subdir = '([\w\-\.,@?^=%&;:\/~\+#]*[\w\-\@?^=%&\/~\+#])?';
 		$optBracket = '';
-		//$optBracket = '(>|&gt;)';
-		$Expr = '/' . $NotAnchor . $Protocol . $Domain . $Subdir . $optBracket . '/i';
-
-		$result2 = preg_replace( $Expr, "<a href=\"$0\" target=\"_blank\">$2$3$4</a>", $result );
-		//$result = preg_replace( $Expr, "<a href=\"$1$2$3$4\" target=\"_blank\">$2$3$4</a>$5 ", $result );
-
+		$optBracket = '(>|&gt;)';
+		$Expr = '/' .$optBracket0. $NotAnchor . $Protocol . $Domain . $Subdir . $optBracket . '/i';
+		// use preg_replace_callback as we experienced problems with https links
+		$result2 = preg_replace_callback( $Expr, function ($match) {
+				//error_log(__METHOD__.__LINE__.array2string($match));
+				return $match[1]."<a href=\"".($match[2]&&!$match[3]?$match[2]:'').($match[3]?$match[3]:'').$match[4].$match[5]."\" target=\"_blank\">".$match[4].$match[5]."</a>".$match[6];
+			}, $result );
+//error_log(__METHOD__.__LINE__.array2string($Expr));
+//error_log(__METHOD__.__LINE__.array2string($result2));
+		//  First match things beginning with http:// (or other protocols)
+		$Protocol = '(http:\/\/|(ftp:\/\/|https:\/\/))';	// only http:// gets removed, other protocolls are shown
+		$Domain = '([\w-]+\.[\w-.]+)';
+		$Subdir = '([\w\-\.,@?^=%&;:\/~\+#]*[\w\-\@?^=%&\/~\+#])?';
+		$optStuff = '(&quot;|&quot|;)?';
+		$Expr = '/' . $NotAnchor . $Protocol . $Domain . $Subdir . $optStuff . '/i';
+		// use preg_replace_callback as we experienced problems with https links
+		$result3 = preg_replace_callback( $Expr, function ($match) {
+				$additionalQuote="";//at the end, ...
+				// only one &quot at the end is found. chance is, it is not belonging to the URL
+				if ($match[5]==';' && (strlen($match[4])-6) >=0 && strpos($match[4],'&quot',strlen($match[4])-6)!==false && strpos(substr($match[4],0,strlen($match[4])-6),'&quot')===false)
+				{
+					$match[4] = substr($match[4],0,strpos($match[4],'&quot',strlen($match[4])-6));
+					$additionalQuote = "&quot;";
+				}
+				// if there is quoted stuff within the URL then we have at least one more &quot; in match[4], so chance is the last &quot is matched by the one within
+				if ($match[5]==';' && (strlen($match[4])-6) >=0 && strpos($match[4],'&quot',strlen($match[4])-6)!==false && strpos(substr($match[4],0,strlen($match[4])-6),'&quot')!==false)
+				{
+					$match[4] .= $match[5];
+				}
+				if ($match[5]==';'&&$match[4]=="&quot")
+				{
+					$match[4] ='';
+					$additionalQuote = "&quot;";
+				}
+				//error_log(__METHOD__.__LINE__.array2string($match));
+				return "<a href=\"".($match[1]&&!$match[2]?$match[1]:'').($match[2]?$match[2]:'').$match[3].$match[4]."\" target=\"_blank\">".$match[3].$match[4]."</a>$additionalQuote";
+			}, $result2 );
+//error_log(__METHOD__.__LINE__.array2string($Expr));
+//error_log(__METHOD__.__LINE__.array2string($result3));
 		//  Now match things beginning with www.
+		$optBracket0 = '(<|&lt;)?';
 		$NotHTTP = '(?<!:\/\/|" target=\"_blank\">)';	//	avoid running again on http://www links already handled above
 		$Domain2 = 'www(\.[\w-.]+)';
 		$Subdir2 = '([\w\-\.,@?^=%&:\/~\+#]*[\w\-\@?^=%&\/~\+#])?';
-		$Expr = '/' . $NotAnchor . $NotHTTP . $Domain2 . $Subdir2 . '/i';
-		//$optBracket = '(>|&gt;)';
+		$optBracket = '(>|&gt;|&gt|;)?';
+		$Expr = '/' .$optBracket0. $NotAnchor . $NotHTTP . $Domain2 . $Subdir2 .$optBracket. '/i';
 		//$Expr = '/' . $NotAnchor . $NotHTTP . $Domain . $Subdir . $optBracket . '/i';
+		// use preg_replace_callback as we experienced problems with links such as <www.example.tld/pfad/zu/einer/pdf-Datei.pdf>
+		$result4 = preg_replace_callback( $Expr, function ($match) {
+				//error_log(__METHOD__.__LINE__.array2string($match));
+				if ($match[4]==';' && (strlen($match[3])-4) >=0 && strpos($match[3],'&gt',strlen($match[3])-4)!==false)
+				{
+					$match[3] = substr($match[3],0,strpos($match[3],'&gt',strlen($match[3])-4));
+					$match[4] = "&gt;";
+				}
+				if ($match[4]==';'&&$match[3]=="&gt")
+				{
+					$match[3] ='';
+					$match[4] = "&gt;";
+				}
+				//error_log(__METHOD__.__LINE__.array2string($match));
+				return $match[1]."<a href=\"http://www".$match[2].$match[3]."\" target=\"_blank\">"."www".$match[2].$match[3]."</a>".$match[4];
+			}, $result3 );
 
-		return preg_replace( $Expr, "<a href=\"http://$0\" target=\"_blank\">$0</a>", $result2 );
 		//return preg_replace( $Expr, "<a href=\"http://www$1$2\" target=\"_blank\">www$1$2</a>$3 ", $result );
+//error_log(__METHOD__.__LINE__.array2string($Expr));
+//error_log(__METHOD__.__LINE__.array2string($result4));
+		return $result4;
 	}
 
 	/**
@@ -214,7 +288,7 @@ class html
 	 * - &nbsp; &lt; &gt; for convenience -> should not happen anymore, as we do not doubleencode anymore (20101020)
 	 *
 	 * @param string $str string to escape
-	 * @param boolean $double_encoding=false do we want double encoding or not, default no
+	 * @param boolean $double_encoding =false do we want double encoding or not, default no
 	 * @return string
 	 */
 	static function htmlspecialchars($str, $double_encoding=false)
@@ -233,7 +307,7 @@ class html
 	 * allows to show and select one item from an array
 	 *
 	 * @param string $name	string with name of the submitted var which holds the key of the selected item form array
-	 * @param string/array $key key(s) of already selected item(s) from $arr, eg. '1' or '1,2' or array with keys
+	 * @param string|array $key key(s) of already selected item(s) from $arr, eg. '1' or '1,2' or array with keys
 	 * @param array $arr array with items to select, eg. $arr = array ( 'y' => 'yes','n' => 'no','m' => 'maybe');
 	 * @param boolean $no_lang NOT run the labels of the options through lang(), default false=use lang()
 	 * @param string $options additional options (e.g. 'width')
@@ -321,13 +395,13 @@ class html
 	 * in the same way. Therefor I made it an extra function.
 	 *
 	 * @param string $name	string with name of the submitted var which holds the key of the selected item form array
-	 * @param string/array $key key(s) of already selected item(s) from $arr, eg. '1' or '1,2' or array with keys
+	 * @param string|array $key key(s) of already selected item(s) from $arr, eg. '1' or '1,2' or array with keys
 	 * @param array $arr array with items to select, eg. $arr = array ( 'y' => 'yes','n' => 'no','m' => 'maybe');
 	 * @param boolean $no_lang NOT run the labels of the options through lang(), default false=use lang()
 	 * @param string $options additional options (e.g. 'width')
 	 * @param int $multiple number of lines for a multiselect, default 3
 	 * @param boolean $selected_first show the selected items before the not selected ones, default true
-	 * @param string $style='' extra style settings like "width: 100%", default '' none
+	 * @param string $style ='' extra style settings like "width: 100%", default '' none
 	 * @return string to set for a template or to echo into html page
 	 */
 	static function checkbox_multiselect($name, $key, $arr=0,$no_lang=false,$options='',$multiple=3,$selected_first=true,$style='',$enhanced = null)
@@ -451,7 +525,7 @@ class html
 	/**
 	 * generate one or more hidden input tag(s)
 	 *
-	 * @param array/string $vars var-name or array with name / value pairs
+	 * @param array|string $vars var-name or array with name / value pairs
 	 * @param string $value value if $vars is no array, default ''
 	 * @param boolean $ignore_empty if true all empty, zero (!) or unset values, plus filer=none
 	 * @param string html
@@ -482,7 +556,7 @@ class html
 	 * @param string $name name attr. of the tag
 	 * @param string $value default
 	 * @param boolean $ignore_empty if true all empty, zero (!) or unset values, plus filer=none
-	 * @param boolean $double_encoding=false do we want double encoding or not, default no
+	 * @param boolean $double_encoding =false do we want double encoding or not, default no
 	 * @param string html
 	 */
 	static function textarea($name,$value='',$options='',$double_encoding=false)
@@ -497,11 +571,6 @@ class html
 	 */
 	static function htmlarea_availible()
 	{
-		//error_log(__METHOD__.__LINE__.' userAgent:'.$_SERVER[HTTP_USER_AGENT]);
-		// we check for the useragent to be able to recognize andoid machines that do not support ckeditor yet.
-		// ckeditors fallback is rather unpleaseant.
-		if (stripos($_SERVER[HTTP_USER_AGENT],'android') !== false) return false;
-
 		// this one is for testing how it will turn out, if you do not have the device or agent ready at your fingertips
 		// if (stripos($_SERVER[HTTP_USER_AGENT],'mozilla') !== false) return false;
 
@@ -531,12 +600,12 @@ class html
 	* @param string $_content of the tinymce (will be run through htmlspecialchars !!!), default ''
 	* @param string $_mode display mode of the tinymce editor can be: simple, extended or advanced
 	* @param array  $_options (toolbar_expanded true/false)
-	* @param string $_height='400px'
-	* @param string $_width='100%'
-	* @param string $_start_path='' if passed activates the browser for image at absolute path passed
-	* @param boolean $_purify=true run $_content through htmlpurifier before handing it to fckEditor
+	* @param string $_height ='400px'
+	* @param string $_width ='100%'
+	* @param string $_start_path ='' if passed activates the browser for image at absolute path passed
+	* @param boolean $_purify =true run $_content through htmlpurifier before handing it to fckEditor
 	* @param mixed (boolean/string) $_focusToBody=false USED only for CKEDIOR true means yes, focus on top, you may specify TOP or BOTTOM (to focus on the end of the editor area)
-	* @param string $_executeJSAfterInit='' Javascript to be executed after InstanceReady of CKEditor
+	* @param string $_executeJSAfterInit ='' Javascript to be executed after InstanceReady of CKEditor
 	* @return string the necessary html for the textarea
 	*/
 	static function fckEditor($_name, $_content, $_mode, $_options=array('toolbar_expanded' =>'true'),
@@ -615,13 +684,13 @@ egw_LAB.wait(function() {
 	*
 	* @param string $_name name and id of the input-field
 	* @param string $_mode display mode of the tinymce editor can be: simple, extended or advanced
-	* @param string $_content='' of the tinymce (will be run through htmlspecialchars !!!), default ''
-	* @param string $_height='400px'
-	* @param string $_width='100%'
-	* @param boolean $_purify=true
-	* @param string $_border='0px' NOT used for CKEditor
+	* @param string $_content ='' of the tinymce (will be run through htmlspecialchars !!!), default ''
+	* @param string $_height ='400px'
+	* @param string $_width ='100%'
+	* @param boolean $_purify =true
+	* @param string $_border ='0px' NOT used for CKEditor
 	* @param mixed (boolean/string) $_focusToBody=false USED only for CKEDIOR true means yes, focus on top, you may specify TOP or BOTTOM (to focus on the end of the editor area)
-	* @param string $_executeJSAfterInit='' Javascript to be executed after InstanceReady of CKEditor
+	* @param string $_executeJSAfterInit ='' Javascript to be executed after InstanceReady of CKEditor
 	* @return string the necessary html for the textarea
 	*/
 	static function fckEditorQuick($_name, $_mode, $_content='', $_height='400px', $_width='100%',$_purify=true, $_border='0px',$_focusToBody=false,$_executeJSAfterInit='')
@@ -717,10 +786,6 @@ egw_LAB.wait(function() {
 			$image = ' src="'.$path.'"';
 			$classes[] = 'image_button';
 		}
-		else
-		{
-			$classes[] = 'et_button_text';
-		}
 		if (!$no_lang)
 		{
 			$label = lang($label);
@@ -780,24 +845,24 @@ egw_LAB.wait(function() {
 	 * Example link('/index.php?menuaction=infolog.uiinfolog.get_list',array('info_id' => 123))
 	 *	gives 'http://domain/phpgw-path/index.php?menuaction=infolog.uiinfolog.get_list&info_id=123'
 	 *
-	 * @param string $url phpgw-relative link, may include query / get-vars
-	 * @param array/string $vars query or array ('name' => 'value', ...) with query
+	 * @param string $_url egw-relative link, may include query / get-vars
+	 * @param array|string $vars query or array ('name' => 'value', ...) with query
 	 * @return string absolut link already run through $phpgw->link
 	 */
-	static function link($url,$vars='')
+	static function link($_url,$vars='')
 	{
 		//echo "<p>html::link(url='$url',vars='"; print_r($vars); echo "')</p>\n";
 		if (!is_array($vars))
 		{
 			parse_str($vars,$vars);
 		}
-		list($url,$v) = explode('?',$url);	// url may contain additional vars
+		list($url,$v) = explode('?', $_url);	// url may contain additional vars
 		if ($v)
 		{
 			parse_str($v,$v);
 			$vars += $v;
 		}
-		return $GLOBALS['egw']->link($url,$vars);
+		return egw::link($url,$vars);
 	}
 
 	/**
@@ -819,16 +884,16 @@ egw_LAB.wait(function() {
 	 *
 	 * @param string $content of the form, if '' only the opening tag gets returned
 	 * @param array $hidden_vars array with name-value pairs for hidden input fields
-	 * @param string $url eGW relative URL, will be run through the link function, if empty the current url is used
-	 * @param string/array $url_vars parameters for the URL, send to link static function too
+	 * @param string $_url eGW relative URL, will be run through the link function, if empty the current url is used
+	 * @param string|array $url_vars parameters for the URL, send to link static function too
 	 * @param string $name name of the form, defaul ''=none
 	 * @param string $options attributes for the tag, default ''=none
 	 * @param string $method method of the form, default 'POST'
 	 * @return string html
 	 */
-	static function form($content,$hidden_vars,$url,$url_vars='',$name='',$options='',$method='POST')
+	static function form($content,$hidden_vars,$_url,$url_vars='',$name='',$options='',$method='POST')
 	{
-		$url = $url ? self::link($url,$url_vars) : $_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING'];
+		$url = $_url ? self::link($_url, $url_vars) : $_SERVER['PHP_SELF'].'?'.$_SERVER['QUERY_STRING'];
 		$html = "<form method=\"$method\" ".($name != '' ? "name=\"$name\" " : '')."action=\"$url\" $options>\n";
 		$html .= self::input_hidden($hidden_vars);
 
@@ -847,7 +912,7 @@ egw_LAB.wait(function() {
 	 * @param string $label label of the button
 	 * @param array $hidden_vars array with name-value pairs for hidden input fields
 	 * @param string $url eGW relative URL, will be run through the link function
-	 * @param string/array $url_vars parameters for the URL, send to link static function too
+	 * @param string|array $url_vars parameters for the URL, send to link static function too
 	 * @param string $options attributes for the tag, default ''=none
 	 * @param string $form_name name of the form, defaul ''=none
 	 * @param string $method method of the form, default 'POST'
@@ -964,21 +1029,21 @@ egw_LAB.wait(function() {
 	/**
 	 * html-widget showing progessbar with a view div's (html4 only, textual percentage otherwise)
 	 *
-	 * @param mixed $percent percent-value, gets casted to int
-	 * @param string $title title for the progressbar, default ''=the percentage itself
+	 * @param mixed $_percent percent-value, gets casted to int
+	 * @param string $_title title for the progressbar, default ''=the percentage itself
 	 * @param string $options attributes for the outmost div (may include onclick="...")
 	 * @param string $width width, default 30px
 	 * @param string $color color, default '#D00000' (dark red)
 	 * @param string $height height, default 5px
 	 * @return string html
 	 */
-	static function progressbar( $percent,$title='',$options='',$width='',$color='',$height='' )
+	static function progressbar($_percent, $_title='',$options='',$width='',$color='',$height='' )
 	{
-		$percent = (int) $percent;
+		$percent = (int)$_percent;
 		if (!$width) $width = '30px';
 		if (!$height)$height= '5px';
 		if (!$color) $color = '#D00000';
-		$title = $title ? self::htmlspecialchars($title) : $percent.'%';
+		$title = $_title ? self::htmlspecialchars($_title) : $percent.'%';
 
 		if (self::$netscape4)
 		{
@@ -999,7 +1064,7 @@ egw_LAB.wait(function() {
 	 * This way session-information gets passed, eg. $name=array('menuaction'=>'myapp.class.image','id'=>123).
 	 *
 	 * @param string $app app-name to search the image
-	 * @param string/array $name image-name or URL (incl. vfs:/) or array with get-vars
+	 * @param string|array $name image-name or URL (incl. vfs:/) or array with get-vars
 	 * @param string $title tooltip, default '' = none
 	 * @param string $options further options for the tag, default '' = none
 	 * @return string the html
@@ -1075,7 +1140,7 @@ egw_LAB.wait(function() {
 	 *
 	 * @param string $content of the link, if '' only the opening tag gets returned
 	 * @param string $url eGW relative URL, will be run through the link function
-	 * @param string/array $vars parameters for the URL, send to link static function too
+	 * @param string|array $vars parameters for the URL, send to link static function too
 	 * @param string $options attributes for the tag, default ''=none
 	 * @return string the html
 	 */
@@ -1247,17 +1312,17 @@ egw_LAB.wait(function() {
 	 * @author Lars Kneschke <lars-AT-kneschke.de> original code in felamimail
 	 * @param array $_folders array of folders: pairs path => node (string label or array with keys: label, (optional) image, (optional) title, (optional) checked)
 	 * @param string $_selected path of selected folder
-	 * @param mixed $_topFolder=false node of topFolder or false for none
-	 * @param string $_onNodeSelect='alert' js static function to call if node gets selected
-	 * @param string $_tree='foldertree' id of the div and name of the variable containing the tree object
-	 * @param string $_divClass='' css class of the div
-	 * @param string $_leafImage='' default image of a leaf-node, ''=default of foldertree, set it eg. 'folderClosed.gif' to show leafs as folders
-	 * @param boolean|string $_onCheckHandler=false string with handler-name to display a checkbox for each folder, or false (default), 'null' switches checkboxes on without an handler!
-	 * @param string $delimiter='/' path-delimiter, default /
-	 * @param string $folderImageDir=null string path to the tree menu images, null uses default path
-	 * @param string|array $autoLoading=null EGw relative path or array with get parameter, both send through egw::link
-	 * @param string $dataMode='JSON' data type for autoloading: XML, JSON, CSV
-	 * @param boolean $dragndrop=false true to enable drag-n-drop (must be before autoloading get enabled!)
+	 * @param mixed $_topFolder =false node of topFolder or false for none
+	 * @param string $_onNodeSelect ='alert' js static function to call if node gets selected
+	 * @param string $tree ='foldertree' id of the div and name of the variable containing the tree object
+	 * @param string $_divClass ='' css class of the div
+	 * @param string $_leafImage ='' default image of a leaf-node, ''=default of foldertree, set it eg. 'folderClosed.gif' to show leafs as folders
+	 * @param boolean|string $_onCheckHandler =false string with handler-name to display a checkbox for each folder, or false (default), 'null' switches checkboxes on without an handler!
+	 * @param string $delimiter ='/' path-delimiter, default /
+	 * @param string $folderImageDir =null string path to the tree menu images, null uses default path
+	 * @param string|array $autoLoading =null EGw relative path or array with get parameter, both send through egw::link
+	 * @param string $dataMode ='JSON' data type for autoloading: XML, JSON, CSV
+	 * @param boolean $dragndrop =false true to enable drag-n-drop (must be before autoloading get enabled!)
 	 *
 	 * @return string the html code, to be added into the template
 	 */
@@ -1451,11 +1516,11 @@ egw_LAB.wait(function() {
 	 * Runs HTMLPurifier over supplied html to remove malicious code
 	 *
 	 * @param string $html
-	 * @param array/string $config=null - config to influence the behavior of current purifying engine
-	 * @param array/string $spec=null - spec to influence the behavior of current purifying engine
+	 * @param array|string $config =null - config to influence the behavior of current purifying engine
+	 * @param array|string $spec =null - spec to influence the behavior of current purifying engine
 	 *		The $spec argument can be used to disallow an otherwise legal attribute for an element,
 	 *		or to restrict the attribute's values
-	 * @param boolean $_force=null - force the config passed to be used without merging to the default
+	 * @param boolean $_force =null - force the config passed to be used without merging to the default
 	 */
 	static function purify($html,$config=null,$spec=array(),$_force=false)
 	{
@@ -1487,14 +1552,101 @@ egw_LAB.wait(function() {
 	}
 
 	/**
-	 * Output content headers for file downloads
+	 * Output content headers for user-content, mitigating risk of javascript or html
+	 *
+	 * Mitigate risk of serving javascript or css from our domain,
+	 * which will get around same origin policy and CSP!
+	 *
+	 * Mitigate risk of html downloads by using CSP or force download for IE
+	 *
+	 * @param resource|string &$content content might be changed by this call
+	 * @param string $path filename or path for content-disposition header
+	 * @param string &$mime ='' mimetype or '' (default) to detect it from filename, using mime_magic::filename2mime()
+	 *	on return used, maybe changed, mime-type
+	 * @param int $length =0 content length, default 0 = skip that header
+	 *  on return changed size
+	 * @param boolean $nocache =true send headers to disallow browser/proxies to cache the download
+	 * @param boolean $force_download =true send content-disposition attachment header
+	 * @param boolean $no_content_type =false do not send actual content-type and content-length header, just content-disposition
+	 */
+	public static function safe_content_header(&$content, $path, &$mime='', &$length=0, $nocache=true, $force_download=true, $no_content_type=false)
+	{
+		// change old/aliased mime-types to new one, eg. image/pdf to application/pdf
+		$mime = mime_magic::fix_mime_type($mime);
+
+		// mitigate risk of serving javascript or css via webdav from our domain,
+		// which will get around same origin policy and CSP
+		list($type, $subtype) = explode('/', strtolower($mime));
+		if (!$force_download && in_array($type, array('application', 'text')) &&
+			in_array($subtype, array('javascript', 'x-javascript', 'ecmascript', 'jscript', 'vbscript', 'css')))
+		{
+			// unfortunatly only Chrome and IE >= 8 allow to switch content-sniffing off with X-Content-Type-Options: nosniff
+			if (html::$user_agent == 'chrome' || html::$user_agent == 'msie' && html::$ua_version >= 8)
+			{
+				$mime = 'text/plain';
+				header('X-Content-Type-Options: nosniff');	// stop IE & Chrome from content-type sniffing
+			}
+			// for the rest we change mime-type to text/html and let code below handle it safely
+			// this stops Safari and Firefox from using it as src attribute in a script tag
+			// but only for "real" browsers, we dont want to modify data for our WebDAV clients
+			elseif (isset($_SERVER['HTTP_REFERER']))
+			{
+				$mime = 'text/html';
+				if (is_resource($content))
+				{
+					$data = fread($content, $length);
+					fclose($content);
+					$content =& $data;
+					unset($data);
+				}
+				$content = '<pre>'.$content;
+				$length += 5;
+			}
+		}
+		// mitigate risk of html downloads by using CSP or force download for IE
+		if (!$force_download && in_array($mime, array('text/html', 'application/xhtml+xml')))
+		{
+			// use CSP only for current user-agents/versions I was able to positivly test
+			if (html::$user_agent == 'chrome' && html::$ua_version >= 24 ||
+				// mobile FF 24 on Android does NOT honor CSP!
+				html::$user_agent == 'firefox' && !html::$ua_mobile && html::$ua_version >= 24 ||
+				html::$user_agent == 'safari' && !html::$ua_mobile && html::$ua_version >= 536 ||	// OS X
+				html::$user_agent == 'safari' && html::$ua_mobile && html::$ua_version >= 9537)	// iOS 7
+			{
+				$csp = "script-src 'none'";	// forbid to execute any javascript
+				header("Content-Security-Policy: $csp");
+				header("X-Webkit-CSP: $csp");	// Chrome: <= 24, Safari incl. iOS
+				//header("X-Content-Security-Policy: $csp");	// FF <= 22
+				//error_log(__METHOD__."('$options[path]') ".html::$user_agent.'/'.html::$ua_version.(html::$ua_mobile?'/mobile':'').": using Content-Security-Policy: $csp");
+			}
+			else	// everything else get's a Content-dispostion: attachment, to be on save side
+			{
+				//error_log(__METHOD__."('$options[path]') ".html::$user_agent.'/'.html::$ua_version.(html::$ua_mobile?'/mobile':'').": using Content-disposition: attachment");
+				$force_download = true;
+			}
+		}
+		if ($no_content_type)
+		{
+			if ($force_download) self::content_disposition_header(egw_vfs::basename($path), $force_download);
+		}
+		else
+		{
+			self::content_header(egw_vfs::basename($path), $mime, $length, $nocache, $force_download);
+		}
+	}
+
+	/**
+	 * Output content-type headers for file downloads
+	 *
+	 * This function should only be used for non-user supplied content!
+	 * For uploaded files, mail attachmentes, etc, you have to use safe_content_header!
 	 *
 	 * @author Miles Lott originally in browser class
 	 * @param string $fn filename
-	 * @param string $mime='' mimetype or '' (default) to detect it from filename, using mime_magic::filename2mime()
-	 * @param int $length=0 content length, default 0 = skip that header
-	 * @param boolean $nocache=true send headers to disallow browser/proxies to cache the download
-	 * @param boolean $forceDownload=true send headers to handle as attachment/download
+	 * @param string $mime ='' mimetype or '' (default) to detect it from filename, using mime_magic::filename2mime()
+	 * @param int $length =0 content length, default 0 = skip that header
+	 * @param boolean $nocache =true send headers to disallow browser/proxies to cache the download
+	 * @param boolean $forceDownload =true send headers to handle as attachment/download
 	 */
 	public static function content_header($fn,$mime='',$length=0,$nocache=True,$forceDownload=true)
 	{
@@ -1528,25 +1680,20 @@ egw_LAB.wait(function() {
 	 * Output content-disposition header for file downloads
 	 *
 	 * @param string $fn filename
-	 * @param boolean $forceDownload=true send headers to handle as attachment/download
+	 * @param boolean $forceDownload =true send headers to handle as attachment/download
 	 */
 	public static function content_disposition_header($fn,$forceDownload=true)
 	{
-			if ($forceDownload===true)
-			{
-				$attachment = ' attachment;';
-			}
-			else
-			{
-				$attachment = ' inline;';
-			}
-			// limit IE hack (no attachment in Content-disposition header) to IE < 9
-			if(self::$user_agent == 'msie' && self::$ua_version < 9)
-			{
-				$attachment = '';
-			}
+		if ($forceDownload)
+		{
+			$attachment = ' attachment;';
+		}
+		else
+		{
+			$attachment = ' inline;';
+		}
 
-			header('Content-disposition:'.$attachment.' filename="'.translation::to_ascii($fn).'"; filename*=utf-8\'\''.rawurlencode($fn));
+		header('Content-disposition:'.$attachment.' filename="'.translation::to_ascii($fn).'"; filename*=utf-8\'\''.rawurlencode($fn));
 	}
 
 	/**
