@@ -9,7 +9,7 @@
  * Copyright (C) 2000, 2001 Joseph Engo
  * @license http://opensource.org/licenses/lgpl-license.php LGPL - GNU Lesser General Public License
  * @package api
- * @version $Id: class.preferences.inc.php 47849 2014-07-22 16:53:27Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -98,7 +98,7 @@ class preferences
 	/**
 	 * Contstructor
 	 *
-	 * @param int|string $account_id=''
+	 * @param int|string $account_id =''
 	 * @return preferences
 	 */
 	function __construct($account_id = '')
@@ -167,7 +167,7 @@ class preferences
 	/**
 	 * Old PHP4 contstructor
 	 *
-	 * @param int|string $account_id=''
+	 * @param int|string $account_id =''
 	 * @return preferences
 	 * @deprecated
 	 */
@@ -231,9 +231,9 @@ class preferences
 	 * parses a notify and replaces the substitutes
 	 *
 	 * @param string $msg message to parse / substitute
-	 * @param array $values=array() extra vars to replace in addition to $this->values, vars are in an array with \
+	 * @param array $values =array() extra vars to replace in addition to $this->values, vars are in an array with \
 	 * 	$key => $value pairs, $key does not include the $'s and is the *untranslated* name
-	 * @param boolean $use_standard_values=true should the standard values are used
+	 * @param boolean $use_standard_values =true should the standard values are used
 	 * @return string with parsed notify-msg
 	 */
 	function parse_notify($msg,$values=array(),$use_standard_values=True)
@@ -258,9 +258,9 @@ class preferences
 	 * replaces the english key's with translated ones, or if $un_lang the opposite
 	 *
 	 * @param string $msg message to translate
-	 * @param array $values=array() extra vars to replace in addition to $this->values, vars are in an array with \
+	 * @param array $vals =array() extra vars to replace in addition to $this->values, vars are in an array with \
 	 * 	$key => $value pairs, $key does not include the $'s and is the *untranslated* name
-	 * @param boolean $un_lang=false if true translate back
+	 * @param boolean $un_lang =false if true translate back
 	 * @return string
 	 */
 	function lang_notify($msg,$vals=array(),$un_lang=False)
@@ -397,7 +397,7 @@ class preferences
 	 *
 	 * the function ready all 3 prefs user/default/forced and merges them to the effective ones
 	 *
-	 * @param boolean $use_session=true should the session prefs get used (default true) or not (false)
+	 * @param boolean $use_session =true should the session prefs get used (default true) or not (false)
 	 * @return array with effective prefs ($this->data)
 	 */
 	function read_repository($use_session=true)
@@ -434,7 +434,7 @@ class preferences
 				default:
 					foreach($values as $app => $vals)
 					{
-						$this->group[$app] = $vals + (array)$this->group[$app];
+						$this->group[$app] = (array)$vals + (array)$this->group[$app];
 					}
 					break;
 			}
@@ -512,8 +512,8 @@ class preferences
 	/**
 	 * Get default preferences (also taking forced preferences into account!)
 	 *
-	 * @param string $app=null
-	 * @param string $name=null
+	 * @param string $app =null
+	 * @param string $name =null
 	 * @return mixed
 	 */
 	function default_prefs($app=null,$name=null)
@@ -582,8 +582,8 @@ class preferences
 	 *
 	 * @param string $app_name name of the app
 	 * @param string $var name of preference to be stored
-	 * @param mixed $value='##undef##' value of the preference, if not given $GLOBALS[$var] is used
-	 * @param $type='user' of preference to set: forced, default, user
+	 * @param mixed $value ='##undef##' value of the preference, if not given $GLOBALS[$var] is used
+	 * @param string $type ='user' of preference to set: forced, default, user
 	 * @return array with new effective prefs (even when forced or default prefs are set !)
 	 */
 	function add($app_name,$var,$value = '##undef##',$type='user')
@@ -640,8 +640,8 @@ class preferences
 	 * the effektive prefs ($this->data) are updated to reflect the change
 	 *
 	 * @param string $app_name name of app
-	 * @param string $var=false variable to be deleted
-	 * @param string $type='user' of preference to set: forced, default, user
+	 * @param string $var =false variable to be deleted
+	 * @param string $type ='user' of preference to set: forced, default, user
 	 * @return array with new effective prefs (even when forced or default prefs are deleted!)
 	 */
 	function delete($app_name, $var = False,$type = 'user')
@@ -729,9 +729,9 @@ class preferences
 	/**
 	 * Change single value in preferences of all users (incl. groups, default and forced)
 	 *
-	 * @param string $app
-	 * @param string $name
-	 * @param string $value new value to set, or null or '' to delete it
+	 * @param string $app app-name or null for all apps
+	 * @param string $name attribute name or regular expression (enclosed in /) to match attribute-name eg. '/^favorite_/'
+	 * @param string|callable $value new value to set, or null or '' to delete it or callable returning new value: function($attr, $old_value, $owner)
 	 * @param string $old_value if given, only change if that's current value
 	 * @param string $type if given limit to "user", "forced", "default", "group"
 	 */
@@ -739,9 +739,9 @@ class preferences
 	{
 		$db = isset($GLOBALS['egw_setup']->db) ? $GLOBALS['egw_setup']->db : $GLOBALS['egw']->db;
 
-		$where = array(
-			'preference_app' => $app,
-		);
+		$where = array();
+		if ($app) $where['preference_app'] = $app;
+
 		switch($type)
 		{
 			case 'forced':
@@ -760,31 +760,55 @@ class preferences
 		foreach($db->select(self::TABLE, '*', $where, __LINE__, __FILE__) as $row)
 		{
 			$prefs = self::unserialize($row['preference_value']);
+			if (!is_array($prefs)) $prefs = array();	// would stall update otherwise
 
-			if (isset($old_value) && $prefs[$name] != $old_value) continue;
-
-			if ((string)$value !== '')
+			if ($name[0] == '/' && substr($name, -1) == '/')
 			{
-				$prefs[$name] = $value;
+				$attrs = array_filter(array_keys($prefs), function($n) use ($name)
+				{
+					return preg_match($name, $n);
+				});
 			}
 			else
 			{
-				unset($prefs[$name]);
+				$attrs = array($name);
 			}
 
-			$db->update(self::TABLE, array(
-				'preference_value' => json_encode($prefs),
-			), array(
-				'preference_owner' => $row['preference_owner'],
-				'preference_app'   => $row['preference_app'],
-			), __LINE__, __FILE__);
-
-			// update instance-wide cache
-			$cached = egw_cache::getInstance(__CLASS__, $row['preference_owner']);
-			if($cached && $cached[$row['preference_app']])
+			$updated = false;
+			foreach($attrs as $attr)
 			{
-				$cached[$row['preference_app']] = $prefs;
-				egw_cache::setInstance(__CLASS__, $row['preference_owner'], $cached);
+				if (isset($old_value) && $prefs[$attr] != $old_value) continue;
+
+				$val = is_callable($value) ? call_user_func($value, $attr, $prefs[$attr], $row['preference_owner']) : $value;
+				if ($val === $prefs[$attr]) continue;
+
+				$updated = true;
+				if ((string)$val !== '')
+				{
+					$prefs[$attr] = $val;
+				}
+				else
+				{
+					unset($prefs[$attr]);
+				}
+			}
+			// if somethings changed or old row was php-serialized --> store it again json-encoded
+			if ($updated || $row['preference_value'][0] == 'a' && $row['preference_value'][1] == ':')
+			{
+				$db->update(self::TABLE, array(
+					'preference_value' => json_encode($prefs),
+				), array(
+					'preference_owner' => $row['preference_owner'],
+					'preference_app'   => $row['preference_app'],
+				), __LINE__, __FILE__);
+
+				// update instance-wide cache
+				$cached = egw_cache::getInstance(__CLASS__, $row['preference_owner']);
+				if($cached && $cached[$row['preference_app']])
+				{
+					$cached[$row['preference_app']] = $prefs;
+					egw_cache::setInstance(__CLASS__, $row['preference_owner'], $cached);
+				}
 			}
 		}
 	}
@@ -794,7 +818,7 @@ class preferences
 	 *
 	 * @param string $app Application name
 	 * @param string $name Preference name
-	 * @param string $type='user' of preference to set: forced, default, user
+	 * @param string $type ='user' of preference to set: forced, default, user
 	 */
 	public static function delete_preference($app, $name, $type='user')
 	{
@@ -806,7 +830,7 @@ class preferences
 	 *
 	 * @param string $from_app
 	 * @param string $to_app
-	 * @param array $names=null array of names to copy or null for all
+	 * @param array $names =null array of names to copy or null for all
 	 */
 	public static function copy_preferences($from_app, $to_app, array $names=null)
 	{
@@ -830,7 +854,7 @@ class preferences
 			{
 				$prefs = array_merge(self::unserialize($values), $prefs);
 			}
-
+			unset($row['preference_id']);
 			//error_log(__LINE__.': '.__METHOD__."() inserting app=$row[preference_app], owner=$row[preference_owner]: ".array2string($prefs));
 			$db->insert(self::TABLE, array(
 				'preference_value' => json_encode($prefs)
@@ -850,9 +874,9 @@ class preferences
 	 *
 	 * User prefs for saveing are in $this->user not in $this->data, which are the effectiv prefs only!
 	 *
-	 * @param boolean $update_session_info=false old param, seems not to be used (not used anymore)
-	 * @param string $type='user' which prefs to update: user/default/forced
-	 * @param boolean $invalid_cache=true should we invalidate the cache, default true (not used anymore)
+	 * @param boolean $update_session_info =false old param, seems not to be used (not used anymore)
+	 * @param string $type ='user' which prefs to update: user/default/forced
+	 * @param boolean $invalid_cache =true should we invalidate the cache, default true (not used anymore)
 	 * @return array with new effective prefs (even when forced or default prefs are deleted!)
 	 */
 	function save_repository($update_session_info = False,$type='user',$invalid_cache=true)
@@ -880,7 +904,7 @@ class preferences
 		//echo "<p>preferences::save_repository(,$type): account_id=$account_id, prefs="; print_r($prefs); echo "</p>\n";
 
 		if (isset($GLOBALS['egw_setup']) || !$GLOBALS['egw']->acl->check('session_only_preferences',1,'preferences') &&
-			(!($old_prefs = $this->cache_read($account_id)) || $old_prefs[$account_id] != $prefs))
+			(!($old_prefs = $this->cache_read($account_id)) || $old_prefs != $prefs))
 		{
 			//error_log(__METHOD__."(type=$type) saved, because old_prefs[$account_id] != prefs=".array2string($prefs));
 			$changed = 0;
@@ -938,7 +962,7 @@ class preferences
 	 * In the absence of a custom ['email']['address'], this function should be used to set it.
 	 *
 	 * @access public
-	 * @param int $accountid - as determined in and/or passed to "create_email_preferences"
+	 * @param int $account_id as determined in and/or passed to "create_email_preferences"
 	 * @return string with email-address
 	 */
 	function email_address($account_id='')
