@@ -6,7 +6,7 @@
  * @copyright 2011 by Andreas Stöckel
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
  * @package egw_action
- * @version $Id: egw_action.js 49179 2014-10-24 12:57:16Z ralfbecker $
+ * @version $Id$
  */
 
 /*egw:uses
@@ -91,7 +91,7 @@ function egw_getObjectManager(_id, _create, _search_depth) {
 		res = egw_globalObjectManager.getObjectById(_id, _search_depth);
 		if (res == null && _create) {
 			res = new egwActionObjectManager(_id,
-				egw_getActionManager(_id));
+				egw_getActionManager(_id,true,_search_depth));
 			egw_globalObjectManager.addObject(res);
 		}
 	}
@@ -1917,7 +1917,7 @@ var EGW_AO_EXEC_THIS = 1;
  *
  * @param {object} _implContext is data which should be delivered to the action implementation.
  * 	E.g. in case of the popup action implementation, the x and y coordinates where the
- * 	menu should open are transmitted.
+ * 	menu should open, and contextmenu event are transmitted.
  * @param {string} _implType is the action type for which the implementation should be
  * 	executed.
  * @param {number} _execType specifies in which context the execution should take place.
