@@ -6,7 +6,7 @@
  * @copyright 2011 by Andreas Stöckel
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
  * @package egw_action
- * @version $Id: egw_action_dragdrop.js 51264 2015-01-14 16:36:55Z hnategh $
+ * @version $Id$
  */
 
 /*egw:uses
@@ -302,6 +302,8 @@ function egwDragActionImplementation()
 							},
 							mouseup: function (){
 								$j(node).draggable("enable");
+								// Set cursor back to auto. Seems FF can't handle cursor reversion
+								$j('body').css({cursor:'auto'});
 							}
 				});
 			}
@@ -368,6 +370,7 @@ function egwDragActionImplementation()
 									egw.lang("Ctrl") : egw.lang("Command ⌘");
 								egw.message(egw.lang('Hold [%1] key to select text eg. to copy it', key), 'info');
 							}
+							
 							// Invalid target
 							return true;
 						}
@@ -382,7 +385,8 @@ function egwDragActionImplementation()
 					"refreshPositions": true,
 					"scroll": false,
 					//"containment": "document",
-					"iframeFix": true
+					"iframeFix": true,
+					"delay": 300
 				}
 			);
 
@@ -634,12 +638,20 @@ function egwDropActionImplementation()
 							// set of properties.
 							var popup = getPopupImplementation();
 							var pos = popup._getPageXY(event.originalEvent);
+							
+							// Don't add paste actions, this is a drop
+							popup.auto_paste = false;
+							
 							window.setTimeout(function() {
 								popup.doExecuteImplementation(pos, selected, links,
 									_context);
+								// Reset, popup is reused
+								popup.auto_paste = true;
 							}, 0); // Timeout is needed to have it working in IE
 						}
-
+						// Set cursor back to auto. Seems FF can't handle cursor reversion
+						$j('body').css({cursor:'auto'});
+						
 						_aoi.triggerEvent(EGW_AI_DRAG_OUT);
 					},
 					"over": function() {
