@@ -6,13 +6,14 @@
  * @subpackage api
  * @link http://www.egroupware.org
  * @author Nathan Gray
- * @version $Id: app_base.js 49165 2014-10-23 13:45:11Z hnategh $
+ * @version $Id$
  */
 
 "use strict";
 
 /*egw:uses
-        egw_inheritance;
+	egw_inheritance;
+	/phpgwapi/js/es6-promise.min.js;
 */
 
 /**
@@ -141,6 +142,12 @@ var AppJS = Class.extend(
 		}
 		this.et2 = et2.widgetContainer;
 		this._fix_iFrameScrolling();
+		if (this.egw && this.egw.is_popup()) this._set_Window_title();
+
+		// Highlights the favorite based on initial list state
+		this.highlight_favorite();
+
+
 	},
 
 	/**
@@ -247,10 +254,10 @@ var AppJS = Class.extend(
 	 *
 	 * @param {{name: string, state: object}|string} state Object (or JSON string) for a state.
 	 *	Only state is required, and its contents are application specific.
-	 *
+	 * @param {string} template template name to check, instead of trying all templates of current app
 	 * @return {boolean} false - Returns false to stop event propagation
 	 */
-	setState: function(state)
+	setState: function(state, template)
 	{
 		// State should be an object, not a string, but we'll parse
 		if(typeof state == "string")
@@ -278,7 +285,7 @@ var AppJS = Class.extend(
 
 		// Try and find a nextmatch widget, and set its filters
 		var nextmatched = false;
-		var et2 = etemplate2.getByApplication(this.appname);
+		var et2 = template ? etemplate2.getByTemplate(template) : etemplate2.getByApplication(this.appname);
 		for(var i = 0; i < et2.length; i++)
 		{
 			et2[i].widgetContainer.iterateOver(function(_widget) {
@@ -290,17 +297,32 @@ var AppJS = Class.extend(
 				{
 					_widget.sortBy(state.state.sort.id, state.state.sort.asc,false);
 				}
+				if(state.state && state.state.selectcols)
+				{
+					// Make sure it's a real array, not an object, then set cols
+					_widget.set_columns(jQuery.extend([],state.state.selectcols));
+				}
 				_widget.applyFilters(state.state || state.filter || {});
 				nextmatched = true;
 			}, this, et2_nextmatch);
 			if(nextmatched) return false;
 		}
 
-		// Try a redirect to list
 		// 'blank' is the special name for no filters, send that instead of the nice translated name
 		var safe_name = jQuery.isEmptyObject(state) || jQuery.isEmptyObject(state.state||state.filter) ? 'blank' : state.name.replace(/[^A-Za-z0-9-_]/g, '_');
-		egw.open('',this.appname,'list',{'favorite': safe_name},this.appname);
+		var url = '/'+this.appname+'/index.php';
 
+		// Try a redirect to list, if app defines a "list" value in registry
+		if (egw.link_get_registry(this.appname, 'list'))
+		{
+			url = egw.link('/index.php', jQuery.extend({'favorite': safe_name}, egw.link_get_registry(this.appname, 'list')));
+		}
+		// if no list try index value from application
+		else if (egw.app(this.appname).index)
+		{
+			url = egw.link('/index.php', 'menuaction='+egw.app(this.appname).index+'&favorite='+safe_name);
+		}
+		egw.open_link(url, undefined, undefined, this.appname);
 		return false;
 	},
 
@@ -339,6 +361,18 @@ var AppJS = Class.extend(
 	 */
 	_init_sidebox: function(sidebox)
 	{
+		// Initialize egw tutorial sidebox, but only for non-popups, as calendar edit app.js has this.et2 set to tutorial et2 object
+		if (!this.egw.is_popup())
+		{
+			var egw_fw = egw_getFramework();
+			var tutorial = $j('#egw_tutorial_'+this.appname+'_sidebox', egw_fw ? egw_fw.sidemenuDiv : document);
+			// _init_sidebox gets currently called multiple times, which needs to be fixed
+			if (tutorial.length && !this.tutorial_initialised)
+			{
+				this.egwTutorial_init(tutorial[0]);
+				this.tutorial_initialised = true;
+			}
+		}
 		if(sidebox.length)
 		{
 			var self = this;
@@ -347,25 +381,52 @@ var AppJS = Class.extend(
 			sidebox
 				.off()
 				// removed .on("mouse(enter|leave)" (wrapping trash icon), as it stalls delete in IE11
-				.on("click","div.ui-icon-trash", this, this.delete_favorite)
+				.on("click.sidebox","div.ui-icon-trash", this, this.delete_favorite)
 				// need to install a favorite handler, as we switch original one off with .off()
-				.on('click','li[data-id]', this, function(event) {
-					var href = jQuery('a[href^="javascript:"]', this).prop('href');
-					var matches = href ? href.match(/^javascript:([^\(]+)\((.*)?\);?$/) : null;
-					if (matches && matches.length > 1 && matches[2] !== undefined)
+				.on('click.sidebox','li[data-id]', this, function(event) {
+					var li = $j(this);
+					li.siblings().removeClass('ui-state-highlight');
+
+					// Wait an arbitrary 50ms to avoid having the class removed again
+					// by the change handler.
+					if(li.attr('data-id') !== 'blank')
+					{
+						window.setTimeout(function() {
+							li.addClass('ui-state-highlight');
+						},50);
+					}
+
+					var state = {};
+					var pref = egw.preference('favorite_' + this.dataset.id, self.appname);
+					if(pref)
+					{
+						// Extend, to prevent changing the preference by reference
+						jQuery.extend(true, state, pref);
+					}
+					if(this.dataset.id != 'add')
 					{
 						event.stopImmediatePropagation();
-						self.setState.call(self, JSON.parse(decodeURI(matches[2])));
+						self.setState.call(self, state);
 						return false;
 					}
 				})
 				.addClass("ui-helper-clearfix");
 
-				//Add Sortable handler to sideBox fav. menu
-				 jQuery('ul','#favorite_sidebox_'+this.appname).sortable({
-
+			//Add Sortable handler to sideBox fav. menu
+			jQuery('ul','#favorite_sidebox_'+this.appname).sortable({
 					items:'li:not([data-id$="add"])',
 					placeholder:'ui-fav-sortable-placeholder',
+					delay:250, //(millisecond) delay before the sorting should start
+					helper: function(event, item) {
+						// We'll need to know which app this is for
+						item.attr('data-appname',self.appname);
+						// Create custom helper so it can be dragged to Home
+						var h_parent = item.parent().parent().clone();
+						h_parent.find('li').not('[data-id="'+item.attr('data-id')+'"]').remove();
+						h_parent.appendTo('body');
+						return h_parent;
+					},
+					refreshPositions: true,
 					update: function (event, ui)
 					{
 						var favSortedList = jQuery(this).sortable('toArray', {attribute:'data-id'});
@@ -375,6 +436,18 @@ var AppJS = Class.extend(
 						self._refresh_fav_nm();
 					}
 				});
+
+			// Bind favorite de-select
+			var egw_fw = egw_getFramework();
+			if(egw_fw && egw_fw.applications[this.appname] && egw_fw.applications[this.appname].browser
+				&& egw_fw.applications[this.appname].browser.baseDiv)
+			{
+				$j(egw_fw.applications[this.appname].browser.baseDiv)
+					.off('.sidebox')
+					.on('change.sidebox', function() {
+						self.highlight_favorite();
+					});
+			}
 			return true;
 		}
 		return false;
@@ -504,7 +577,7 @@ var AppJS = Class.extend(
 			<ul id="'+this.appname+'_favorites_popup_state"/>\
 			</form>\
 			</div>'
-		).appendTo(this.et2 ? this.et2.getDOMNode() : $j('body'));
+		).appendTo(this.et2 && this.appname != 'calendar' ? this.et2.getDOMNode() : $j('body'));
 
 		$j(".ui-icon-circle-plus",this.favorite_popup).prev().andSelf().click(function() {
 			var details = $j("#"+self.appname+"_favorites_popup_state",self.favorite_popup)
@@ -524,7 +597,7 @@ var AppJS = Class.extend(
 				empty_label: "Groups",
 				no_lang: true,
 				parent_node: this.appname+'_favorites_popup_admin'
-			},this.et2 || null);
+			},(this.et2 && this.appname != 'calendar'? this.et2:null));
 			this.favorite_popup.group.loadingFinished();
 		}
 
@@ -694,9 +767,104 @@ var AppJS = Class.extend(
 
 		return false;
 	},
-	
+
 	/**
-	 * Fix scrolling iframe browsed by iPhone/iPod/iPad touch devices 
+	 * Mark the favorite closest matching the current state
+	 *
+	 * Closest matching takes into account not set values, so we pick the favorite
+	 * with the most matching values without a value that differs.
+	 */
+	highlight_favorite: function() {
+		if(!this.sidebox) return;
+
+		var state = this.getState();
+		var best_match = false;
+		var best_count = 0;
+		var self = this;
+
+		$j('li[data-id]',this.sidebox).removeClass('ui-state-highlight');
+
+		$j('li[data-id]',this.sidebox).each(function(i,href) {
+			var favorite = {};
+			if(this.dataset.id && egw.preference('favorite_'+this.dataset.id,self.appname))
+			{
+				favorite = egw.preference('favorite_'+this.dataset.id,self.appname);
+			}
+			if(!favorite || jQuery.isEmptyObject(favorite)) return;
+
+			var match_count = 0;
+			for(var state_key in state)
+			{
+				if(typeof favorite.state != 'undefined' && typeof state[state_key] != 'undefined'&&typeof favorite.state[state_key] != 'undefined' && ( state[state_key] == favorite.state[state_key] || !state[state_key] && !favorite.state[state_key]))
+				{
+					match_count++;
+				}
+				else if (typeof state[state_key] != 'undefined' && state[state_key] && typeof state[state_key] === 'object'
+							&& typeof favorite.state != 'undefined' && typeof favorite.state[state_key] != 'undefined' && favorite.state[state_key] && typeof favorite.state[state_key] === 'object')
+				{
+					if((typeof state[state_key].length !== 'undefined' || typeof state[state_key].length !== 'undefined')
+							&& (state[state_key].length || Object.keys(state[state_key]).length) != (favorite.state[state_key].length || Object.keys(favorite.state[state_key]).length ))
+					{
+						// State or favorite has a length, but the other does not
+						if((state[state_key].length === 0 || Object.keys(state[state_key]).length === 0) &&
+							(favorite.state[state_key].length == 0 || Object.keys(favorite.state[state_key]).length === 0))
+						{
+							// Just missing, or one is an array and the other is an object
+							continue;
+						}
+						// One has a value and the other doesn't, no match
+						return;
+					}
+					// Consider sub-objects (column filters) individually
+					for(var sub_key in state[state_key])
+					{
+						if(state[state_key][sub_key] == favorite.state[state_key][sub_key] || !state[state_key][sub_key] && !favorite.state[state_key][sub_key])
+						{
+							match_count++;
+						}
+						else if (state[state_key][sub_key] && favorite.state[state_key][sub_key] &&
+							typeof state[state_key][sub_key] === 'object' && typeof favorite.state[state_key][sub_key] === 'object')
+						{
+							// Too deep to keep going, just string compare for perfect match
+							if(JSON.stringify(state[state_key][sub_key]) === JSON.stringify(favorite.state[state_key][sub_key]))
+							{
+								match_count++;
+							}
+						}
+						else if(state[state_key][sub_key] && state[state_key][sub_key] != favorite.state[state_key][sub_key])
+						{
+							// Different values, do not match
+							return;
+						}
+
+					}
+				}
+				else if (state_key == 'selectcols')
+				{
+					// Skip, might be set, might not
+				}
+				else if (typeof state[state_key] !== 'undefined'
+						 && typeof favorite.state != 'undefined'&&typeof favorite.state[state_key] !== 'undefined'
+						 && state[state_key] != favorite.state[state_key])
+				{
+					// Different values, do not match
+					return;
+				}
+			}
+			if(match_count > best_count)
+			{
+				best_match = this.dataset.id;
+				best_count = match_count;
+			}
+		});
+		if(best_match)
+		{
+			$j('li[data-id="'+best_match+'"]',this.sidebox).addClass('ui-state-highlight');
+		}
+	},
+
+	/**
+	 * Fix scrolling iframe browsed by iPhone/iPod/iPad touch devices
 	 */
 	_fix_iFrameScrolling: function()
 	{
@@ -720,5 +888,180 @@ var AppJS = Class.extend(
 				}
 			});
 		}
-	}
+	},
+
+	/**
+	 * Set document title, uses getWindowTitle to get the correct title,
+	 * otherwise set it with uniqueID as default title
+	 */
+	_set_Window_title: function ()
+	{
+		var title = this.getWindowTitle();
+		if (title)
+		{
+			document.title = this.et2._inst.uniqueId + ": " + title;
+		}
+	},
+
+	/**
+	 * Window title getter function in order to set the window title
+	 * this can be overridden on each application app.js file to customize the title value
+	 *
+	 * @returns {string} window title
+	 */
+	getWindowTitle: function ()
+	{
+		var titleWidget = this.et2.getWidgetById('title');
+		if (titleWidget)
+		{
+			return titleWidget.options.value;
+		}
+		else
+		{
+			return this.et2._inst.uniqueId;
+		}
+	},
+
+	/**
+	 * Handler for drag and drop when dragging nextmatch rows from mail app
+	 * and dropped on a row in the current application.  We copy the mail into
+	 * the filemanager to link it since we can't link directly.
+	 *
+	 * This doesn't happen automatically.  Each application must indicate that
+	 * it will accept dropped mail by it's nextmatch actions:
+	 *
+	 * $actions['info_drop_mail'] = array(
+	 *		'type' => 'drop',
+	 *		'acceptedTypes' => 'mail',
+	 *		'onExecute' => 'javaScript:app.infolog.handle_dropped_mail',
+	 *		'hideOnDisabled' => true
+	 *	);
+	 *
+	 * This action, when defined, will not affect the automatic linking between
+	 * normal applications.
+	 *
+	 * @param {egwAction} _action
+	 * @param {egwActionObject[]} _selected Dragged mail rows
+	 * @param {egwActionObject} _target Current application's nextmatch row the mail was dropped on
+	 */
+	handle_dropped_mail: function(_action, _selected, _target)
+	{
+		/**
+		 * Mail doesn't support link system, so we copy it to VFS
+		 */
+		var ids = _target.id.split("::");
+		if(ids.length != 2 || ids[0] == 'mail') return;
+
+		var vfs_path = "/apps/"+ids[0]+"/"+ids[1];
+		var mail_ids = [];
+
+		for(var i = 0; i < _selected.length; i++)
+		{
+			mail_ids.push(_selected[i].id);
+		}
+		if(mail_ids.length)
+		{
+			egw.message(egw.lang("Please wait..."));
+			this.egw.json('filemanager.filemanager_ui.ajax_action',['mail',mail_ids, vfs_path],function(data){
+				// Trigger an update (minimal, no sorting changes) to display the new link
+				egw.refresh(data.msg||'',ids[0],ids[1],'update');
+			}).sendRequest(true);
+		}
+	},
+	
+	/**
+	 * Get json data for videos from the given url
+	 *
+	 * @return {Promise, object} return Promise, json object as resolved result and error message in case of failure
+	 */
+	egwTutorialGetData: function(){
+		var self = this;
+		return new Promise (function(_resolve, _reject)
+		{
+			var resolve = _resolve;
+			var reject = _reject;
+			self.egw.json('home.home_tutorial_ui.ajax_data', [self.egw.app_name()], function(_data){
+					resolve(_data);
+			}).sendRequest();
+		});
+	},
+
+	/**
+	 * Create and Render etemplate2 for egroupware tutorial
+	 * sidebox option. The .xet file is stored in etemplate/templates/default/egw_tutorials
+	 *
+	 * @description tutorials json object should have the following structure:
+	 *	object:
+	 *		{
+	 *			[app name]:{
+	 *				[language tag]:[
+	 *					{src:"",thumbnail:"",title:"",desc:""}
+	 *				]
+	 *			}
+	 *		}
+	 *
+	 *	*Note: "desc" and "title" are optional attributes, which "desc" would appears as tooltip for the video.
+	 *
+	 *	example:
+	 *		{
+	 *			"mail":{
+	 *				"en":[
+	 *					{src:"https://www.youtube.com/embed/mCDJndpjO40", thumbnail:"http://img.youtube.com/vi/mCDJndpjO40/0.jpg", "title":"PGP Encryption", "desc":""},
+	 *					{src:"https://www.youtube.com/embed/mCDJndpjO", thumbnail:"http://img.youtube.com/vi/mCDJndpjO/0.jpg", "title":"Subscription", "desc":""},
+	 *				],
+	 *				"de":[
+	 *					{src:"https://www.youtube.com/embed/m40", thumbnail:"http://img.youtube.com/vi/m40/0.jpg", "title":"PGP Verschlüsselung", "desc":""},
+	 *					{src:"https://www.youtube.com/embed/mpjO", thumbnail:"http://img.youtube.com/vi/mpjO/0.jpg", "title":"Ordner Abonnieren", "desc":""},
+	 *				]
+	 *			}
+	 *		}
+	 */
+	egwTutorial_init: function(div)
+	{
+		// et2 object
+		var etemplate = new etemplate2 (div, false);
+		var template = egw.webserverUrl+'/etemplate/templates/default/egw_tutorial.xet?1';
+
+		this.egwTutorialGetData().then(function(_data){
+			var lang = egw.preference('lang');
+			var content = {content:{list:[]}};
+			if (_data && _data[egw.app_name()])
+			{
+				if (!_data[egw.app_name()][lang]) lang = 'en';
+				if (typeof _data[egw.app_name()][lang] !='undefined'
+					&& _data[egw.app_name()][lang].length > 0)
+				{
+					for (var i=0;i < _data[egw.app_name()][lang].length;i++)
+					{
+						var tuid = egw.app_name() + '-' +lang + '-' + i;
+						_data[egw.app_name()][lang][i]['onclick'] = 'app.'+egw.app_name()+'.egwTutorialPopup("'+tuid+'")';
+					}
+					content.content.list = _data[egw.app_name()][lang];
+
+					if (template.indexOf('.xet') >0)
+					{
+						etemplate.load ('',template , content, function(){});
+					}
+					else
+					{
+						etemplate.load (template, '', content);
+					}
+				}
+			}
+		},
+		function(_err){
+			console.log(_err);
+		});
+	},
+
+	/**
+	 * Open popup to show given tutorial id
+	 * @param {string} _tuid tutorial object id
+	 *	- tuid: appname-lang-index
+	 */
+	egwTutorialPopup: function (_tuid)
+	{
+		var url = egw.link('/index.php', 'menuaction=home.home_tutorial_ui.popup&tuid='+_tuid);
+		egw.open_link(url,'_blank','960x580');
+	},
 });
