@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Andreas Stöckel (as AT stylite.de)
  * @author Ralf Becker <RalfBecker@outdoor-training.de>
- * @version $Id: egw_core.js 46338 2014-03-20 09:40:37Z ralfbecker $
+ * @version $Id$
  */
 
 "use strict";
@@ -449,7 +449,7 @@
 			{
 				case 0:
 					// Return the global instance
-					return egw;
+					return instances['~global~'][0]['instance'];
 
 				case 1:
 					if (typeof arguments[0] === 'string')
