@@ -7,11 +7,12 @@
  * @package projectmanager
  * @copyright (c) 2005-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.projectmanager_bo.inc.php 47436 2014-06-30 09:47:17Z ralfbecker $
+ * @version $Id$
  */
 
 define('EGW_ACL_BUDGET',EGW_ACL_CUSTOM_1);
 define('EGW_ACL_EDIT_BUDGET',EGW_ACL_CUSTOM_2);
+define('EGW_ACL_ADD_TIMESHEET', EGW_ACL_CUSTOM_3);
 
 /**
  * General business object of the projectmanager
@@ -402,7 +403,6 @@ class projectmanager_bo extends projectmanager_so
 		{
 			if (isset($data[$name]) && $data[$name]) $data[$name] += $this->tz_offset_s;
 		}
-		if (is_numeric($data['pm_completion'])) $data['pm_completion'] .= '%';
 
 		return $data;
 	}
@@ -526,7 +526,7 @@ class projectmanager_bo extends projectmanager_so
 		if(!$index && $this->not_unique(array('pm_number' => $pm_format)))	// no index given and not unique
 		{
 			// have to use default
-			$pm_format = $parent === '' ? sprintf('P-%04Y-%04d', date('Y')) : $parent.'/%04d';
+			$pm_format = $parent === '' ? sprintf('P-%04s-%%04d', date('Y')) : $parent.'/%04d';
 		}
 		elseif(!$index)
 		{
@@ -552,7 +552,7 @@ class projectmanager_bo extends projectmanager_so
 	 *
 	 * Rights are given via owner grants or role based acl
 	 *
-	 * @param int $required EGW_ACL_READ, EGW_ACL_WRITE, EGW_ACL_ADD, EGW_ACL_DELETE, EGW_ACL_BUDGET, EGW_ACL_EDIT_BUDGET
+	 * @param int $required EGW_ACL_READ, EGW_ACL_WRITE, EGW_ACL_ADD, EGW_ACL_ADD_TIMESHEET, EGW_ACL_DELETE, EGW_ACL_BUDGET, EGW_ACL_EDIT_BUDGET
 	 * @param array|int $data=null project or project-id to use, default the project in $this->data
 	 * @param boolean $no_cache=false should a cached value be used, if availible, or not
 	 * @param int $user=null for which user to check, default current user
@@ -622,9 +622,9 @@ class projectmanager_bo extends projectmanager_so
 		{
 			$access = false;
 		}
-		elseif ($required & EGW_ACL_READ)	// read-rights are implied by all other rights
+		elseif ($required & EGW_ACL_READ)       // read-rights are implied by all other rights, but EGW_ACL_ADD_TIMESHEET
 		{
-			$access = $rights != 0;
+			$access = (boolean) ($rights & ~EGW_ACL_ADD_TIMESHEET);
 		}
 		else
 		{
@@ -670,7 +670,17 @@ class projectmanager_bo extends projectmanager_so
 	{
 		if (!is_array($entry))
 		{
+			$pm_id = $entry;
+			$backup = $this->data;
 			$entry = $this->read( $entry );
+			$this->data = $backup;
+
+			// Even though ADD_TIMESHEET means no read, we let them
+			// see the title
+			if(!$entry && $this->check_acl(EGW_ACL_ADD_TIMESHEET, $pm_id))
+			{
+				$entry = $this->data;
+			}
 		}
 		if (!$entry)
 		{
