@@ -7,7 +7,7 @@
  * @package projectmanager
  * @copyright (c) 2005-8 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.projectmanager_elements_bo.inc.php 49309 2014-11-06 13:33:52Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -245,7 +245,9 @@ class projectmanager_elements_bo extends projectmanager_elements_so
 				$this->data['pe_status']= 'new';
 			}
 			// if user linking has no ADD rights, the entry is set to ignored
-			if (!$this->check_acl(EGW_ACL_ADD,array('pm_id'=>$pm_id)))
+			if (!$this->check_acl(EGW_ACL_ADD,array('pm_id'=>$pm_id)) && !
+				($this->check_acl(EGW_ACL_ADD_TIMESHEET, array('pm_id'=>$pm_id)) && $app == 'timesheet')
+			)
 			{
 				$this->data['pe_status']= 'ignore';
 			}
@@ -680,7 +682,7 @@ class projectmanager_elements_bo extends projectmanager_elements_so
 	{
 		if ((int) $this->debug >= 2 || $this->debug == 'copytree') $this->debug_message("projectmanager_elements_bo::copytree($source) this->pm_id=$this->pm_id");
 
-		$elements =& $this->search(array('pm_id' => $source),false,'pe_planned_start,pe_title');
+		$elements =& $this->search('',false,'pe_planned_start,pe_title','','',false,'AND',false,array('pm_id'=>$source));
 		if (!$elements) return array();
 
 		$copied = $apps_copied = $callbacks = $params = array();
