@@ -7,7 +7,7 @@
  * @package projectmanager
  * @copyright (c) 2005-8 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.projectmanager_elements_ui.inc.php 51050 2014-12-16 17:22:22Z nathangray $
+ * @version $Id$
  */
 
 /**
@@ -70,19 +70,16 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 		parent::__construct($pm_id);
 
 
-		$GLOBALS['egw']->preferences->add('projectmanager','current_project', $pm_id);
-		$GLOBALS['egw']->preferences->save_repository();
-
 		// check if we have at least read-access to this project
 		if (!$this->project->check_acl(EGW_ACL_READ))
 		{
-			$this->tpl->location(array(
-				'menuaction' => 'projectmanager.projectmanager_ui.index',
-				'msg'        => lang('Permission denied !!!'),
-				'ajax'       => 'true'
-			));
+			egw_framework::message(lang('Permission denied !!!'),'error');
+			$pm_id = 0;
 		}
 
+		$GLOBALS['egw']->preferences->add('projectmanager','current_project', $pm_id);
+		$GLOBALS['egw']->preferences->save_repository();
+
 		$this->status_labels = array(
 			'all'     => lang('all'),
 			'used'    => lang('used'),
@@ -449,14 +446,30 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 	 */
 	function get_rows(&$query_in,&$rows,&$readonlys)
 	{
-		if($query_in['col_filter']['pm_id'])
+		if(!$query_in['csv_export'])
 		{
-			$this->pm_id = (int)$query_in['col_filter']['pm_id'];
-		}
-		else
-		{
-			return 0;
+			if($query_in['col_filter']['pm_id'])
+			{
+				$this->pm_id = (int)$query_in['col_filter']['pm_id'];
+			}
+			else if ( $GLOBALS['egw_info']['user']['preferences']['projectmanager']['current_project'])
+			{
+				$this->pm_id = $GLOBALS['egw_info']['user']['preferences']['projectmanager']['current_project'];
+			}
+			if (!property_exists ($this, 'pm_id') || !$this->pm_id)
+			{
+				return 0;
+			}
+			if(!$this->project->check_acl(Acl::READ, $this->pm_id))
+			{
+				return 0;
+			}
 		}
+
+		// Check for filter change, need to get totals
+		$session = egw_cache::getSession('projectmanager', 'projectelements_list');
+		$get_totals =  $query_in['start'] === 0 || ($session && $session['filter'] != $query_in['filter']) || !$session && $query_in['filter'];
+
 		$GLOBALS['egw']->session->appsession('projectelements_list','projectmanager',$query=$query_in);
 
 		//echo "<p>project_elements_ui::get_rows(".print_r($query,true).")</p>\n";
@@ -628,6 +641,7 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 						}
 					}
 				}
+				$row['ignored'] = ($row['pe_status'] == 'ignore') ? 'projectmanager/ignored' : '';
 			}
 		}
 		if (!$this->project->check_acl(EGW_ACL_BUDGET))
@@ -653,9 +667,8 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 			$query_in['options-selectcols']['pm_used_budget'] = $query_in['options-selectcols']['pm_planned_budget'] = false;
 		}
 		if ($query['cat_id']) $rows['no_cat_id'] = true;
-		// calculate the filter-specific summary if we have a filter, beside the default pe_status=used=array(new,regular)
-		unset($query['col_filter']['pe_app']); //pe_app should not change summary
-		if (array_diff(array_keys($query['col_filter']),array(0,'pe_status','pm_id')) || !is_array($query['col_filter']['pe_status']))
+		// calculate the filter-specific summary if we have a filter change
+		if ($get_totals)
 		{
 			$totals = $this->summary(null,$query['col_filter']);
 			foreach($totals as $field => $value)
@@ -664,6 +677,11 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 				{
 					$rows['total_' . $field] = $value;
 				}
+				// etemplate requires unique IDs, these ones are in the second times column
+				if(in_array($field, array('pe_planned_time', 'pe_used_time')))
+				{
+					$rows['total_' . $field . '_2'] = $value;
+				}
 			}
 		}
 		if ((int)$this->debug >= 2 || $this->debug == 'get_rows')
@@ -759,6 +777,14 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 				'disableClass' => 'rowNoEdit',
 
 			),
+			'ignore' => array(
+				'caption' => 'Ignore that entry',
+				'group' => $group,
+				'disableClass' => 'rowNoEdit',
+				'checkbox' => true,
+				'isChecked' => 'javaScript:app.projectmanager.is_ignored',
+				'onExecute' => 'javaScript:app.projectmanager.ignore_action'
+			),
 			'timesheet' => array(
 				'icon' => 'timesheet/navbar',
 				'caption' => 'Timesheet',
@@ -773,9 +799,10 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 				'group' => $group,
 				'allowOnMultiple' => false,
 				'enableId' => '^infolog:',
-				'enableClass' => 'projectmanager_rowHasSubs',
+				'enableClass' => 'infolog_rowHasSubs',
 				'url' => 'menuaction=infolog.infolog_ui.index&action=sp&action_id=$id',
 				'targetapp' => 'infolog',
+				'hideOnDisabled' => true
 			),
 		);
 		if ($GLOBALS['egw_info']['user']['apps']['filemanager'])
@@ -1035,6 +1062,7 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 		}
 		if (substr($action,0,4) == 'cat_') list($action,$cat_id) = explode('_',$action);
 		if (substr($action,0,6) == 'erole_') list($action,$erole) = explode('_',$action);
+		if (substr($action,0,7) == 'ignore_') list($action,$ignore) = explode('_',$action);
 
 		switch($action)
 		{
@@ -1098,6 +1126,23 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 					}
 				}
 				break;
+			case 'ignore':
+				$success = $failed = 0;
+				foreach($checked as $id)
+				{
+					$element = $this->read($id);
+					if(!$this->save(array('pe_status' => $ignore ? 'ignore' : 'new')))
+					{
+						$success++;
+					}
+					else
+					{
+						$failed++;
+					}
+				}
+				$msg = lang('%1 element(s) updated',$success);
+				return $failed==0;
+				break;
 			case 'delete':
 				if (!$this->project->check_acl(EGW_ACL_ADD))
 				{
@@ -1190,6 +1235,44 @@ class projectmanager_elements_ui extends projectmanager_elements_bo
 		return false;
 	}
 
+/**
+	 * Run given action on given path(es) and return array/object with values for keys 'msg', 'errs', 'dirs', 'files'
+	 *
+	 * @param string $action eg. 'delete', ...
+	 * @param array $selected selected path(s)
+	 * @param string $data Action specific data
+	 * @see static::action()
+	 */
+	public static function ajax_action($action, $selected, $data = array())
+	{
+
+		$response = egw_json_response::get();
+
+		switch($action)
+		{
+			case 'ignore':
+				$ui = new projectmanager_elements_ui();
+				$checked = array();
+				foreach($selected as $entry)
+				{
+					list($prefix,$checked[]) = explode('::',$entry);
+				}
+				$msg = '';
+				if($ui->action('ignore_'.(!!$data),$checked,$msg, $add_existing))
+				{
+					// We could just update the selected rows here, but this is easier and gets
+					// the totals too
+					$response->call('egw.refresh',$msg,'projectmanager');
+				}
+				else
+				{
+					$response->error($msg);
+				}
+				break;
+		}
+
+		//error_log(__METHOD__."('$action',".array2string($selected).') returning '.array2string($arr));
+	}
 
 	/**
 	 * Download a document with inserted contact(s)
