@@ -6,7 +6,7 @@
  * @author Nathan Gray <ng@stylite.de>
  * @package projectmanager
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.projectmanager_gantt.inc.php 49309 2014-11-06 13:33:52Z ralfbecker $
+ * @version $Id$
  */
 
 class projectmanager_gantt extends projectmanager_elements_ui {
@@ -20,7 +20,8 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 		parent::__construct();
 	}
 
-	public function chart($data = array()) {
+	public function chart($data = array())
+	{
 
 		// Find out which project we're working with
 		if (isset($_REQUEST['pm_id']))
@@ -117,7 +118,14 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 		$template->read('projectmanager.gantt');
 
 		$template->setElementAttribute('gantt','actions', $this->get_gantt_actions());
+		$template->setElementAttribute('gantt','columns', $this->get_gantt_columns());
 		
+		// Allow Stylite extensions a chance
+		if(class_exists('stylite_projectmanager_gantt'))
+		{
+			stylite_projectmanager_gantt::chart($template, $data, $sel_options, $readonlys);
+		}
+
 		$template->exec('projectmanager.projectmanager_gantt.chart', $data, $sel_options, $readonlys);
 	}
 
@@ -162,6 +170,103 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 		return $actions;
 	}
 
+	/**
+	 * Get a list of fields to be offered as options for display in the
+	 * left side of the gantt chart.
+	 * name and label are required.  tree is only for the first column.
+	 * Additionally, width (numeric), hide (boolean), and widget(et2_widget name)
+	 * can be specified to indicate desired width, start hidden, and to try
+	 * to use the specified et2_widget to render the value.
+	 *
+	 * @see http://docs.dhtmlx.com/gantt/api__gantt_columns_config.html
+	 *
+	 * @return array
+	 */
+	protected function get_gantt_columns()
+	{
+		$time_attributes = array();
+		list($time_attributes['display_format'],$time_attributes['hours_per_day']) = explode(',',$this->config['duration_format']);
+		$columns = array(
+			array(
+				'name' => 'text',
+				'label'=> lang('Title'),
+				'tree'=> true,
+				'width'=> 300
+			),
+			array(
+				'name' => 'pe_completion',
+				'label'=>lang('Status'),
+				'width' => 60,
+				'hide'=> true
+			),
+			array(
+				'name' => 'pe_planned_time',
+				'label'=>lang('Planned time'),
+				'width' => 80,
+				'hide'=> true,
+				'widget'=>'date-duration',
+				'widget_attributes' => $time_attributes
+			),
+			array(
+				'name' => 'pe_used_time',
+				'label'=>lang('Used time'),
+				'width' => 80,
+				'hide'=> true,
+				'widget'=>'date-duration',
+				'widget_attributes' => $time_attributes
+			),
+			array(
+				'name' => 'pe_planned_budget',
+				'label'=>lang('Planned budget'),
+				'width' => 80,
+				'align' => 'right',
+				'hide'=> true
+			),
+			array(
+				'name' => 'pe_used_budget',
+				'label'=>lang('Used budget'),
+				'width' => 80,
+				'align' => 'right',
+				'hide'=> true
+			),
+			array(
+				'name' => 'start_date',
+				'label' => lang('Start date'),
+				'width' => 120,
+				'hide'=> true,
+				'widget'=>'date-time'
+			),
+			array(
+				'name' => 'end_date',
+				'label' => lang('End date'),
+				'width' => 120,
+				'hide'=> true,
+				'widget'=>'date-time'
+			),
+			array(
+				'name' => 'pe_resources',
+				'label' => lang('Resources'),
+				'width' => 150,
+				'hide'=> true,
+				'widget'=>'select-account'
+			)
+		);
+
+		// Apply preferences
+		$pref = $GLOBALS['egw_info']['user']['preferences']['projectmanager']['gantt_columns_gantt'];
+		if($pref)
+		{
+			foreach($columns as &$col)
+			{
+				if(in_array($col['name'], $pref))
+				{
+					$col['hide'] = false;
+				}
+			}
+		}
+		return $columns;
+	}
+
 	/**
 	 * Ajax callback to load the elements for a project.  The project itself
 	 * (and it's elements) are already in the gantt, we're loading one level
@@ -206,9 +311,9 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 
 	// Get the data into required format
 	protected function &add_project(&$data, $pm_id, $params) {
-		if(!$pm_id)
+		if(!$pm_id || !$this->project->check_acl(EGW_ACL_READ, $pm_id))
 		{
-			return;
+			return array();
 		}
 		if ($pm_id != $this->project->data['pm_id'])
 		{
@@ -245,6 +350,18 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 		{
 			$project['duration'] = $params['planned_times'] ? $this->project->data['pm_planned_time'] : 1;
 		}
+		// Add in planned start & end, if different
+		if(!$params['planned_times'])
+		{
+			if($project['pm_planned_start'] && $project['pm_planned_start'] != egw_time::to($project['start_date'],egw_time::DATABASE))
+			{
+				$project['planned_start'] = egw_time::to((int)$project['pm_planned_start'], egw_time::DATABASE);
+			}
+			if($project['pm_planned_end'] && $project['pm_planned_end'] != $project['pm_real_end'])
+			{
+				$project['planned_end'] = egw_time::to((int)$project['pm_planned_end'], egw_time::DATABASE);
+			}
+		}
 		
 		// Not sure how it happens, but it causes problems
 		if($project['start'] && $project['start'] < 10) $project['start'] = 0;
@@ -388,13 +505,23 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 			$pe['text'] = $this->prefs['gantt_element_title_length'] ? substr($pe['pe_title'], 0, $this->prefs['gantt_element_title_length']) : $pe['pe_title'];
 			$pe['parent'] = 'projectmanager::'.$pm_id;
 			$pe['start_date'] = egw_time::to((int)$pe['pe_start'],egw_time::DATABASE);
+			if($pe['pe_planned_start'] && $pe['pe_planned_start'] != $pe['pe_start'])
+			{
+				$pe['start_date'] = egw_time::to($pe['pe_real_start'], egw_time::DATABASE);
+				$pe['planned_start'] = egw_time::to((int)$pe['pe_planned_start'], egw_time::DATABASE);
+			}
 			$pe['duration'] = self::get_duration($data,(float)($params['planned_times'] ? $pe['pe_planned_time'] : $pe['pe_used_time']));
 			if($pe['pe_end'])
 			{
 				// Make sure we don't kill the gantt chart with too large a time span - limit to 10 years
 				$pe['end_date'] = egw_time::to(min($pe['pe_end'],strtotime('+10 years',$pe['pe_start'])),egw_time::DATABASE	);
 			}
-			$pe['progress'] = ((int)substr($this->project->data['pe_completion'],0,-1))/100;
+			if($pe['pe_planned_end'] && $pe['pe_planned_end'] != $pe['pe_end'])
+			{
+				$pe['end_date'] = egw_time::to($pe['pe_real_end'], egw_time::DATABASE);
+				$pe['planned_end'] = egw_time::to((int)$pe['pe_planned_end'], egw_time::DATABASE);
+			}
+			$pe['progress'] = ((int)substr($pe['pe_completion'],0,-1))/100;
 			$pe['edit'] = $this->check_acl(EGW_ACL_EDIT, $pe);
 
 			// Set field for filter to filter on
@@ -454,7 +581,7 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 	/**
 	 * User updated start date or duration from gantt chart
 	 */
-	public static function ajax_update($values, $params)
+	public static function ajax_update($values, $mode, $params)
 	{
 		if($params['planned_times'] == 'false') $params['planned_times'] = false;
 
@@ -464,9 +591,31 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 			unset($values['pe_id']);
 		}
 
-		if(class_exists('stylite_projectmanager_gantt'))
+		// Handle duration scaling
+		if($params['gantt']['duration_unit'] && $values['duration'])
 		{
-			$handled = stylite_projectmanager_gantt::ajax_update($values, $params);
+			switch($params['gantt']['duration_unit'])
+			{
+				case 'year':
+					$values['duration'] *= 52;
+				case 'week':
+					$values['duration'] *= 7;
+				case 'day':
+					$values['duration'] *= 24;
+				case 'hour':
+					$values['duration'] *= 60;
+			}
+		}
+			
+		// Don't change timesheets
+		if($values['pe_app'] == 'timesheet')
+		{
+			egw_json_response::get()->call('egw.message',lang('Editing timesheets is not supported, edit the entry directly'), 'error');
+			return;
+		}
+		if(class_exists('stylite_projectmanager_gantt') && $params['datasource_update'])
+		{
+			$handled = stylite_projectmanager_gantt::ajax_update($values, $mode, $params);
 			if($handled) return;
 		}
 		else if(!$GLOBALS['egw_info']['user']['preferences']['projectmanager']['skip_stylite_warning'])
@@ -494,7 +643,7 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 			$update_mask = $update_mask | $pe_bo->data['pe_overwrite'];
 			$keys = array('pe_overwrite' => $update_mask);
 			$keys['pe_completion'] = (int)($values['progress'] * 100).'%';
-			if(array_key_exists('duration', $values))
+			if($mode == 'resize' && array_key_exists('duration', $values))
 			{
 				$keys['pe_' . ($params['planned_times'] ? 'planned' : 'used') .'_time'] = $values['duration'];
 			}
@@ -523,10 +672,6 @@ class projectmanager_gantt extends projectmanager_elements_ui {
 			$pm_bo = new projectmanager_bo((int)$values['pm_id']);
 			$keys = array('pm_overwrite' => $update_mask | $pm_bo->data['pm_overwrite']);
 			$keys['pm_completion'] = (int)($values['progress'] * 100).'%';
-			if(array_key_exists('duration', $values))
-			{
-				$keys['pm_' . ($params['planned_times'] ? 'planned' : 'used') .'_time'] = $values['duration'];
-			}
 			if(array_key_exists('start_date', $values))
 			{
 				$keys['pm_' . ($params['planned_times'] ? 'planned' : 'real') . '_start'] = egw_time::to($values['start_date'],'ts');
