@@ -7,7 +7,7 @@
  * @package projectmanager
  * @copyright (c) 2005-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.projectmanager_hooks.inc.php 51028 2014-12-15 19:43:57Z nathangray $
+ * @version $Id$
  */
 
 /**
@@ -151,22 +151,27 @@ class projectmanager_hooks
 				'Projectlist' => egw::link('/index.php',array(
 					'menuaction' => 'projectmanager.projectmanager_ui.index',
 					'ajax' => 'true',
-				)),
-				array(
-					'text' => 'Elementlist',
-					'link' =>  egw::link('/index.php',array(
-						'menuaction' => 'projectmanager.projectmanager_ui.index',
-						'ajax' => 'true',
-					)),
-				),
-				array(
-					'text' => 'Gantt chart',
-					'link' =>  egw::link('/index.php',array(
-						'menuaction' => 'projectmanager.projectmanager_ui.index',
-						'ajax' => 'true',
-					)),
-				)
+				))
 			);
+			if($GLOBALS['projectmanager_bo']->check_acl(EGW_ACL_READ))
+			{
+				$file += array(
+					array(
+						'text' => 'Elementlist',
+						'link' =>  egw::link('/index.php',array(
+							'menuaction' => 'projectmanager.projectmanager_ui.index',
+							'ajax' => 'true',
+						)),
+					),
+					array(
+						'text' => 'Gantt chart',
+						'link' =>  egw::link('/index.php',array(
+							'menuaction' => 'projectmanager.projectmanager_ui.index',
+							'ajax' => 'true',
+						)),
+					)
+				);
+			}
 			// show pricelist menuitem only if we use pricelists
 			if (!self::$config['accounting_types'] || in_array('pricelist',(is_array(self::$config['accounting_types'])?self::$config['accounting_types']:explode(',',self::$config['accounting_types']))))
 			{
@@ -215,7 +220,7 @@ class projectmanager_hooks
 		{
 			$file = Array(
 				'Site configuration' => egw::link('/index.php','menuaction=projectmanager.projectmanager_admin.config'),
-				'Custom fields' => egw::link('/index.php','menuaction=admin.customfields.edit&appname=projectmanager&use_private=1'),
+				'Custom fields' => egw::link('/index.php','menuaction=admin.customfields.index&appname=projectmanager&use_private=1'),
 				'Global Categories'  => egw::link('/index.php',array(
 					'menuaction' => 'admin.admin_categories.index',
 					'appname'    => $appname,
@@ -371,6 +376,19 @@ class projectmanager_hooks
 				'admin'  => False,
 				'forced'=> False,
 		);
+		$pm_list_options = array(
+			'~edit~'    => lang('Edit project'),
+			'list' => lang('Element list'),
+		);
+		$settings['pm_list'] = array(
+			'type'   => 'select',
+			'label'  => 'Default action on double-click',
+			'name'   => 'pm_list',
+			'values' => $pm_list_options,
+			'xmlrpc' => True,
+			'admin'  => false,
+			'default'=> 'list',
+		);
 		$settings['show_projectselection'] = array(
 			'type'   => 'select',
 			'label'  => 'Show the project selection as',
@@ -401,7 +419,8 @@ class projectmanager_hooks
 			'label'  => 'Order of sub-project bars in ganttcharts',
 			'name'   => 'gantt_pm_elementbars_order',
 			'values' => array(
-				'pe_start,pe_end'    => lang('Start Date, End Date'),
+				'pe_start,pe_end'    => lang('Start Date'),
+				'pe_end'             => lang('End Date'),
 				'pe_title'           => lang('Project ID'),
 			),
 			'help'   => 'Set order to show sub-project bars in ganttcharts.',
