@@ -7,7 +7,7 @@
  * @package projectmanager
  * @copyright (c) 2005-11 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.projectmanager_ui.inc.php 48901 2014-10-06 16:02:52Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -79,16 +79,39 @@ class projectmanager_ui extends projectmanager_bo
 	 */
 	public function index(array $content = null)
 	{
-		$this->pm_list();
+		// Check ACL first, no read access will trigger redirects
+		if ((int) $_REQUEST['pm_id'])
+		{
+			$pm_id = (int) $_REQUEST['pm_id'];
+			// store the current project (only for index, as popups may be called by other parent-projects)
+		}
+		else if ($_GET['pm_id'])
+		{
+			// AJAX requests have pm_id only in GET, not REQUEST
+			$pm_id = (int)$_GET['pm_id'];
+		}
+		else
+		{
+			$pm_id = $GLOBALS['egw_info']['user']['preferences']['projectmanager']['current_project'];
+		}
+		if(!$this->check_acl(EGW_ACL_READ,$pm_id))
+		{
+			$pm_id = $_GET['pm_id'] = $GLOBALS['egw_info']['user']['preferences']['projectmanager']['current_project'] = 0;
 
-		$element_list = new projectmanager_elements_ui();
-		$element_list->index();
+		}
+		if($this->check_acl(EGW_ACL_READ, $pm_id))
+		{
+			$this->pm_list();
 
-		$gantt = new projectmanager_gantt();
-		$gantt->chart();
+			$element_list = new projectmanager_elements_ui();
+			$element_list->index();
 
-		$prices = new projectmanager_pricelist_ui();
-		$prices->index();
+			$gantt = new projectmanager_gantt();
+			$gantt->chart();
+
+			$prices = new projectmanager_pricelist_ui();
+			$prices->index();
+		}
 	}
 
 	/**
@@ -129,7 +152,7 @@ class projectmanager_ui extends projectmanager_bo
 			{
 				//echo "content="; _debug_array($content);
 				$this->data_merge($content);
-				//echo "after data_merge data="; _debug_array($this->data);
+				//error_log("after data_merge data="); error_log(array2string($this->data));
 
 				// set the data of the pe_summary, if the project-value is unset
 				$pe_summary = $this->pe_summary();
@@ -141,7 +164,7 @@ class projectmanager_ui extends projectmanager_bo
 					if (($content[$pm_name] || $pm_name == 'pm_completion' && $content[$pm_name] !== '') &&
 						($content[$pm_name] != $this->data[$pm_name] || !($this->data['pm_overwrite'] & $id)))
 					{
-						//echo "$pm_name set to '".$this->data[$pm_name]."'<br>\n";
+						//error_log( "$pm_name set to '".$this->data[$pm_name]);
 						$this->data['pm_overwrite'] |= $id;
 					}
 					// or check if a field is no longer set, or the datasource changed => set it from the datasource
@@ -224,6 +247,15 @@ class projectmanager_ui extends projectmanager_bo
 					if (!$content['pm_id'] && is_array($content['link_to']['to_id']))
 					{
 						egw_link::link('projectmanager',$this->data['pm_id'],$content['link_to']['to_id']);
+
+						// check if we have dragged in images and fix their image urls
+						if (etemplate_widget_vfs::fix_html_dragins('projectmanager', $this->data['pm_id'],
+							$content['link_to']['to_id'], $content['pm_description']))
+						{
+							so_sql::update(array(
+								'pm_description' => $content['pm_description'],
+							));
+						}
 					}
 					if ($content['template'] && $new_id = $this->copy($content['template'],2))
 					{
@@ -367,7 +399,7 @@ class projectmanager_ui extends projectmanager_bo
 					($this->config['accounting_types'] && count($this->config['accounting_types']) == 1 ||
 					!($this->data['pm_creator'] == $this->user || $this->data['pm_members'][$this->user]['role_id'] == 1 ||
 					$coord_from_groups_roles)) ||
-					$this->config['accounting_types'] == 'status' || $this->config['accounting_types'] == 'times',
+					$this->config['accounting_types'] == array('status') || $this->config['accounting_types'] == array('times'),
 				'custom' => !count($this->customfields),	// only show customfields tab, if there are some
 				'history' => !$this->data['pm_id'],        //suppress history for the first loading without ID
 			),
@@ -450,6 +482,12 @@ class projectmanager_ui extends projectmanager_bo
 			}
 			if (count($sel_options['pm_accounting_type']) == 1)
 			{
+				if(!$content['pm_accounting_type'])
+				{
+					reset($sel_options['pm_accounting_type']);
+					$content['pm_accounting_type'] = $preserv['pm_accounting_type'] =
+						key($sel_options['pm_accounting_type']);
+				}
 				$readonlys['pm_accounting_type'] = true;
 			}
 		}
@@ -483,6 +521,17 @@ class projectmanager_ui extends projectmanager_bo
 	 */
 	function get_rows(&$query_in,&$rows,&$readonlys)
 	{
+		// for unknown reason, order is sometimes set to an element column, eg. pe_modified
+		// need to fix that, as it gives a sql error otherwise
+		if (substr($query_in['order'], 0, 3) === 'pe_')
+		{
+			$query_in['order'] = 'pm_'.substr($query_in['order'], 3);
+		}
+		if (!$this->db->get_column_attribute($query_in['order'], $this->table_name,'projectmanager'))
+		{
+			$query_in['order'] = 'pm_modified';
+		}
+
 		$GLOBALS['egw']->session->appsession('project_list','projectmanager',$query=$query_in);
 
 		//echo "<p>projectmanager_ui::get_rows(".print_r($query,true).")</p>\n";
@@ -508,12 +557,21 @@ class projectmanager_ui extends projectmanager_bo
 			if ($query[$nm_name]) $query['col_filter'][$pm_name] = $query[$nm_name];
 		}
 		$query['col_filter']['subs_or_mains'] = $query['filter'];
+		unset($query['col_filter']['pm_id']);
 
-		$total = parent::get_rows($query,$rows,$readonlys,true,true);
+		$total = parent::get_rows($query,$rows,$readonlys,'',true);
 
 		$readonlys = array();
 		foreach($rows as &$row)
 		{
+			// Hide as much as possible for users without read access, but still have other permissions
+			if(!$this->check_acl(EGW_ACL_READ,$row['pm_id']))
+			{
+				foreach($row as $key => &$value)
+				{
+					if(!in_array($key, array('pm_id','pm_number','pm_title'))) $value = '';
+				}
+			}
 			if (!$this->check_acl(EGW_ACL_EDIT,$row))
 			{
 				$row['class'] .= ' rowNoEdit';
@@ -541,9 +599,8 @@ class projectmanager_ui extends projectmanager_bo
 			$roles = $this->roles->query_list();
 
 			$all_members = $this->read_members($pm_ids);
-			foreach($rows as $n => $val)
+			foreach($rows as &$row)
 			{
-				$row =& $rows[$n];
 				$members = $row['pm_members'] = $all_members[$row['pm_id']];
 				// Set a value even if empty, or previous row won't be cleared.
 				for($i = 0; $i < 5; $i++)
@@ -695,7 +752,7 @@ class projectmanager_ui extends projectmanager_bo
 		$this->instanciate('roles');
 		$roles = $this->roles->query_list();
 		$role_count = 0;
-		foreach($roles as $role_id => $role_name)
+		foreach($roles as $role_name)
 		{
 			if($role_count > 5)
 			{
@@ -756,7 +813,7 @@ class projectmanager_ui extends projectmanager_bo
 	 *
 	 * @return array see nextmatch_widget::egw_actions()
 	 */
-	private function get_actions()
+	public function get_actions()
 	{
 		$actions = array(
 			'view' => array(
@@ -766,13 +823,14 @@ class projectmanager_ui extends projectmanager_bo
 				'onExecute' => 'javaScript:app.projectmanager.set_project',
 				'target' => '_self',
 				'group' => $group=1,
-				'default' => true,
+				'default' => $GLOBALS['egw_info']['user']['preferences']['projectmanager']['pm_list'] != '~edit~',
 			),
 			'open' => array(	// does edit if allowed, otherwise view
 				'caption' => 'Open',
 				'allowOnMultiple' => false,
 				'egw_open' => 'edit-projectmanager',
 				'group' => $group,
+				'default' => $GLOBALS['egw_info']['user']['preferences']['projectmanager']['pm_list'] == '~edit~',
 			),
 			'add' => array(
 				'caption' => 'Add',
@@ -1034,6 +1092,11 @@ class projectmanager_ui extends projectmanager_bo
 			else
 			{
 				$filter = array();
+				if($parent_pm_id)
+				{
+					$project = $GLOBALS['projectmanager_bo']->read($parent_pm_id);
+					$filter['pm_status'] = $project['pm_status'];
+				}
 			}
 			self::_project_tree_leaves($filter,$parent_pm_id?$parent_pm_id : 'mains',$_pm_id ? $_pm_id : $parent_pm_id,$nodes);
 		}
