@@ -6,7 +6,7 @@
  * @author Nahtan Gray
  * @copyright (c) 2013 by Nathan Gray
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: app.js 49815 2014-12-10 16:50:27Z nathangray $
+ * @version $Id$
  */
 
 /**
@@ -19,7 +19,7 @@ app.classes.projectmanager = AppJS.extend(
 	appname: 'projectmanager',
 
 	// Variables for having all templates loaded & switching
-	view: null,
+	view: 'list',
 	views: {
 		// Name = key, etemplate is filled in when loaded, sidemenu is untranslated text from hooks
 		list: {name: 'list', etemplate: null, sidemenu: 'Projectlist'},
@@ -106,7 +106,7 @@ app.classes.projectmanager = AppJS.extend(
 				$j(et2.DOMContainer).hide();
 			}
 
-			if(this.view == null && view.name == 'list')
+			if(view.name == 'list')
 			{
 				// First load, bind filemanager too
 				var list = view;
@@ -298,7 +298,7 @@ app.classes.projectmanager = AppJS.extend(
 			return egw.open(state.state,undefined,undefined,{},'_self');
 		}
 
-		if(state.state.view)
+		if(state.state && state.state.view)
 		{
 			this.show(state.state.view, state.state.pm_id||false);
 			// Avoid any potential conflicts when setting others, below
@@ -390,6 +390,10 @@ app.classes.projectmanager = AppJS.extend(
 			{
 				et2.widgetContainer.iterateOver(function(_widget) {
 					state = _widget.getValue();
+					
+					// These aren't considered for state
+					delete state.link_add;
+					delete state.link_addapp;
 				}, this, et2_nextmatch);
 			}
 		}
@@ -495,7 +499,7 @@ app.classes.projectmanager = AppJS.extend(
 							egw.dataRefreshUIDs(rex,'delete');
 						}
 					}
-				}	
+				}
 		}
 
 		// Update current view with new info
@@ -508,6 +512,13 @@ app.classes.projectmanager = AppJS.extend(
 					nm.refresh(_id,_type);
 				}
 				return false;
+			case 'elements':
+				var nm = this.views.elements.etemplate ? this.views.elements.etemplate.widgetContainer.getWidgetById('nm') : null
+				if(nm)
+				{
+					nm.refresh(_id,_type);
+				}
+				return false;
 			case 'gantt':
 				var ids = [];
 				var gantt = this.views.gantt.etemplate.widgetContainer.getWidgetById('gantt');
@@ -592,17 +603,17 @@ app.classes.projectmanager = AppJS.extend(
 				menuaction: (content.data['caller'])? content.data['caller'] :'projectmanager.projectmanager_elements_ui.index',
 				delete: id
 			});
-		window.close();
+		egw(window).close();
 	},
 
 	/**
 	 * Limits constraint target link-entry widget to current project
 	 *
 	 * @param {object} request
-	 * @param {object} response
+	 * @param {et2_widget_entry} widget
 	 * @returns {boolean} true to do the search
 	 */
-	element_constraint_pre_query: function(request,response)
+	element_constraint_pre_query: function(request,widget)
 	{
 		if(!request.options) request.options = {};
 		request.options.pm_id = this.et2.getInstanceManager().widgetContainer.getArrayMgr('content').getEntry('pm_id');
@@ -863,6 +874,40 @@ app.classes.projectmanager = AppJS.extend(
 		return allowed;
 	},
 
+	/**
+	 * Is the selected entry ignored?
+	 */
+	is_ignored: function(action, selected)
+	{
+		var ignored = false;
+		
+		for(var i = 0; i < selected.length; i++)
+		{
+			var data = egw.dataGetUIDdata(selected[i].id);
+			ignored = ignored || !!(data && data.data && data.data.ignored);
+		}
+
+		return ignored;
+	},
+
+	/**
+	 * Toggle the ignore flag on the selected entries
+	 *
+	 * @param {egwAction} action
+	 * @param {egwActionObject[]} selected
+	 */
+	ignore_action: function(action, selected)
+	{
+		var ids = [];
+		for(var i = 0; i < selected.length; i++)
+		{
+			ids.push(selected[i].id);
+		}
+		egw.json('projectmanager_elements_ui::ajax_action', [action.id, ids, action.checked],
+			false, this, true, this
+		).sendRequest(true);
+	},
+
 	/**
 	 * Enabled check for project element action, used by context menu
 	 *
@@ -900,19 +945,29 @@ app.classes.projectmanager = AppJS.extend(
 	add_new: function (action, selected)
 	{
 		var tree = this.views.list.etemplate.widgetContainer.getWidgetById('project_tree');
+		var pm_id = '';
 		if (tree)
 		{
-			var pm_id = tree.getValue();
+			pm_id = tree.getValue();
 
 			// Gantt chart can have multiple selected
 			if(jQuery.isArray(pm_id)) pm_id = pm_id[0];
 
 			pm_id = pm_id.replace('::',':');
-			if (typeof action != 'undefined')
+		}
+		// No tree, or could not find project there
+		if(!pm_id && selected[0] && egw.dataGetUIDdata(selected[0].id))
+		{
+			var data = egw.dataGetUIDdata(selected[0].id);
+			if(data && data.data && data.data.pm_id)
 			{
-				this.egw.open(pm_id, action.id.replace('act-',''), 'add');
+				pm_id = 'projectmanager:'+data.data.pm_id;
 			}
 		}
+		if (typeof action !== 'undefined')
+		{
+			return this.egw.open(pm_id, action.id.replace('act-',''), 'add');
+		}
 	},
 
 	/**
@@ -928,5 +983,15 @@ app.classes.projectmanager = AppJS.extend(
 		var sidebox = $j('a:contains("' +app.projectmanager.egw.lang(label)+'")',app.projectmanager.sidebox.parentsUntil('#egw_fw_sidemenu,#tdSidebox').last());
 		sidebox.off('click.projectmanager');
 		sidebox.on('click.projectmanager', click);
+	},
+	
+	/**
+	 * Get title in order to set it as document title
+	 * @returns {string}
+	 */
+	getWindowTitle: function()
+	{
+		var widget = this.et2.getWidgetById('pm_title');
+		if(widget) return widget.options.value;
 	}
 });
