@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Cornelius Weiss <egw@von-und-zu-weiss.de>
  * @author Lukas Weiss <wnz_gh05t@users.sourceforge.net>
- * @version $Id: class.resources_bo.inc.php 48317 2014-08-27 08:14:07Z hnategh $
+ * @version $Id$
  */
 
 
@@ -251,7 +251,17 @@ class resources_bo
 			}
 			else
 			{
-				$rows[$num]['picture_thumb'] = $this->get_picture($resource, true);
+				$rows[$num]['picture_thumb'] = $this->get_picture($resource, false);
+				if ($rows[$num]['picture_src'] == 'own_src')
+				{
+					// VFS picture fullsize
+					$rows[$num]['picture_original'] = 'webdav.php/apps/resources/'.$rows[$num]['res_id'].'/.picture.jpg';
+				}
+				else
+				{
+					// cat or generic icon fullsize
+					$rows[$num]['picture_original'] = $this->get_picture($resource, true);
+				}
 			}
 			$rows[$num]['admin'] = $this->acl->get_cat_admin($resource['cat_id']);
 		}
@@ -588,11 +598,11 @@ class resources_bo
 		$data = $this->so->search($criteria,$only_keys,$order_by='name',$extra_cols='',$wildcard='%',$empty,$op='OR',$limit,$filter);
 		// maybe we need to check disponibility of the searched resources in the calendar if $pattern ['exec'] contains some extra args
 		$show_conflict=False;
-		if (is_array($pattern) && isset($pattern['exec']) )
+		if ($options['exec'])
 		{
 			// we'll use a cache for resources info taken from database
 			static $res_info_cache = array();
-			$cal_info=$pattern['exec'];
+			$cal_info=$options['exec'];
 			if ( isset($cal_info['start']) && isset($cal_info['duration']))
 			{
 				//get a calendar objet for reservations
@@ -601,22 +611,14 @@ class resources_bo
 					require_once(EGW_INCLUDE_ROOT.'/calendar/inc/class.calendar_bo.inc.php');
 					$this->bocal =& CreateObject('calendar.calendar_bo');
 				}
-
-				//get the real timestamps from infos we have on the event
-				//use etemplate date widget to handle date values
-				require_once(EGW_INCLUDE_ROOT.'/phpgwapi/inc/class.jscalendar.inc.php');
-				$jscal=& CreateObject('phpgwapi.jscalendar');
-				$startarr= $jscal->input2date($cal_info['start']['str'],$raw='raw',$day='day',$month='month',$year='year');
+				$start = new egw_time($cal_info['start']);
+				$startarr= getdate($start->format('ts'));
 				if (isset($cal_info['whole_day'])) {
 					$startarr['hour'] = $startarr['minute'] = 0;
-					unset($startarr['raw']);
-					$start = $this->bocal->date2ts($startarr);
-					$end = $start + 86399;
+					$start = new egw_time($startarr);
+					$end = $start->format('ts') + 86399;
 				} else {
-					$startarr['hour'] = $cal_info['start']['H'];
-					$startarr['minute'] = $cal_info['start']['i'];
-					unset($startarr['raw']);
-					$start = $this->bocal->date2ts($startarr);
+					$start = $start->format('ts');
 					$end = $start + ($cal_info['duration']);
 				}
 
@@ -673,7 +675,7 @@ class resources_bo
 			}
 		}
 		if (isset($res_info_cache)) {
-			$show_conflict= (isset($pattern['exec']['show_conflict'])&& ($pattern['exec']['show_conflict']=='0'))? False:True;
+			$show_conflict= (isset($options['exec']['show_conflict'])&& ($options['exec']['show_conflict']=='0'))? False:True;
 			// if we have this array indexed on resource id it means non-bookable resource are removed and we are working for calendar
 			// so we'll loop on this one and not $data
 			foreach($res_info_cache as $id => $resource) {
@@ -759,7 +761,7 @@ class resources_bo
 	}
 
 	/**
-	 * resizes and saves an pictures in vfs
+	 * saves a pictures in vfs
 	 *
 	 * Cornelius Weiss <egw@von-und-zu-weiss.de>
 	 * @param array $file array with key => value
@@ -785,23 +787,8 @@ class resources_bo
 				return lang('Picture type is not supported, sorry!');
 		}
 
-		$src_img_size = getimagesize($file['tmp_name']);
-		$dst_img_size = array( 0 => 320, 1 => 240);
-
 		$tmp_name = tempnam($GLOBALS['egw_info']['server']['temp_dir'],'resources-picture');
-		if($src_img_size[0] > $dst_img_size[0] || $src_img_size[1] > $dst_img_size[1])
-		{
-			$f = $dst_img_size[0] / $src_img_size[0];
-			$f = $dst_img_size[1] / $src_img_size[1] < $f ? $dst_img_size[1] / $src_img_size[1] : $f;
-			$dst_img = imagecreatetruecolor($src_img_size[0] * $f, $src_img_size[1] * $f);
-			imagecopyresized($dst_img,$src_img,0,0,0,0,$src_img_size[0] * $f,$src_img_size[1] * $f,$src_img_size[0],$src_img_size[1]);
-			imagejpeg($dst_img,$tmp_name);
-			imagedestroy($dst_img);
-		}
-		else
-		{
-			imagejpeg($src_img,$tmp_name);
-		}
+		imagejpeg($src_img,$tmp_name);
 		imagedestroy($src_img);
 
 		egw_link::attach_file('resources',$resouce_id,array(
@@ -830,6 +817,12 @@ class resources_bo
 				{
 					$picture = egw::link(egw_vfs::download_url($picture));
 				}
+				else
+				{
+					$picture = egw::link('/etemplate/thumbnail.php', array(
+						'path' => $picture
+					));
+				}
 				break;
 
 			case 'cat_src':
@@ -837,14 +830,14 @@ class resources_bo
 				$picture = unserialize($picture['data']);
 				if($picture['icon'])
 				{
-					$picture = $GLOBALS['egw_info']['server']['webserver_url'].'/phpgwapi/images/'.$picture['icon'];
+					$picture = !$fullsize?$GLOBALS['egw_info']['server']['webserver_url'].'/phpgwapi/images/'.$picture['icon']:'/phpgwapi/images/'.$picture['icon'];
 					break;
 				}
 				// fall through
 			case 'gen_src':
 			default :
 				$src = $resource['picture_src'];
-				$picture = $GLOBALS['egw_info']['server']['webserver_url'].$this->resource_icons;
+				$picture = !$fullsize?$GLOBALS['egw_info']['server']['webserver_url'].$this->resource_icons:$this->resource_icons;
 				$picture .= strpos($src,'.') !== false ? $src : 'generic.png';
 		}
 		return $picture;
