@@ -7,7 +7,7 @@
  * @link http://www.egroupware.org
  * @author Cornelius Weiss <egw@von-und-zu-weiss.de>
  * @author Lukas Weiss <wnz_gh05t@users.sourceforge.net>
- * @version $Id: class.resources_ui.inc.php 46312 2014-03-18 14:14:00Z hnategh $
+ * @version $Id$
  */
 
 /**
@@ -181,7 +181,7 @@ class resources_ui
 	 *
 	 * @return array see nextmatch_widget::egw_actions()
 	 */
-	protected function get_actions()
+	public function get_actions()
 	{
 		$actions = array(
 			'edit' => array(
@@ -505,7 +505,11 @@ class resources_ui
 		if ($_GET['msg']) $content['msg'] = strip_tags($_GET['msg']);
 
 		// some presetes
-		$content['resource_picture'] = $this->bo->get_picture($content['res_id'],$content['picture_src'],$size=true);
+		$content['resource_picture'] = $this->bo->get_picture($content['res_id'],false);
+		// Set original size picture
+		$content['picture_original'] = $content['picture_src'] == 'own_src'?
+				'webdav.php/apps/resources/'.$content['res_id'].'/.picture.jpg': $this->bo->get_picture($content['res_id'],true);
+
 		$content['quantity'] = $content['quantity'] ? $content['quantity'] : 1;
 		$content['useable'] = $content['useable'] ? $content['useable'] : 1;
 		$content['accessory_of'] = $content['accessory_of'] ? $content['accessory_of'] : $accessory_of;
@@ -643,10 +647,10 @@ class resources_ui
 			$selectbox = html::select(
 				'owner',
 				$selected,
-				array_merge($resources,$res_cats),
-				$no_lang=true,
-				$options=' style="width: 100%;" id="uical_select_resource"',
-				$multiple=4,
+				$options=array_merge(array('r0' => lang('None')),$resources,$res_cats),
+				true,	// no lang
+				' style="width: 100%;" id="uical_select_resource"',
+				min(4, count($options)),	// multiple
 				false
 			);
 			return array(
