@@ -7,7 +7,7 @@
  * @author Miles Lott <milos@groupwhere.org>
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: check_install.php 51016 2014-12-15 14:30:45Z ralfbecker $
+ * @version $Id$
  */
 $run_by_webserver = !!$_SERVER['PHP_SELF'];
 $is_windows = strtoupper(substr(PHP_OS,0,3)) == 'WIN';
@@ -141,7 +141,7 @@ $checks = array(
 		'func' => 'extension_check',
 		'error' => lang('The PDO extension plus a database specific driver is needed by the VFS (virtual file system)!'),
 	),
-	'mysql' => array(
+	'mysqli' => array(
 		'func' => 'extension_check',
 		'warning' => lang('The %1 extension is needed, if you plan to use a %2 database.','mysql','MySQL')
 	),
@@ -196,10 +196,6 @@ $checks = array(
 		'func' => 'pear_check',
 		'error' => lang('PEAR extensions are required by many eGroupware applications, PEAR itself is the required basis for each extension!'),
 	),
-	'HTTP_WebDAV_Server' => array(
-		'func' => 'pear_check',
-		'from' => 'WebDAV',
-	),
 	realpath('..') => array(
 		'func' => 'permission_check',
 		'is_world_writable' => False,
@@ -223,6 +219,10 @@ $checks = array(
 		'func' => 'extension_check',
 		'error' => lang('The json extension is required by EGroupware for AJAX.'),
 	),
+	'zip' => array(
+		'func' => 'extension_check',
+		'warning' => lang('The zip extension is required for merge-print with office documents.'),
+	),
 	'tidy' => array(
 		'func' => 'extension_check',
 		'warning' => lang('The tidy extension is need in merge-print to clean up html before inserting it in office documents.'),
@@ -300,11 +300,11 @@ foreach(array('php_version','php_ini_check','extension_check','pear_check','gd_c
 	}
 }
 if ($checks) $sorted_checks += $checks;
-$checks =& $sorted_checks;
 
 function php_version($name,$args)
 {
 	global $passed_icon, $error_icon;
+	unset($name);	// not used, but required by function signature
 
 	$version_ok = version_compare(phpversion(),$args['value']) >= 0;
 
@@ -313,10 +313,33 @@ function php_version($name,$args)
 		phpversion().' ==> '.($version_ok ? lang('True') : lang('False'))."</span></div>\n";
 }
 
+/**
+ * Check if given package is installed via composer in EGroupware's vendor directory
+ *
+ * @param string $package package-name in composer notation eg. "pear-pear.horde.org/Horde_Imap_Client" or "pear-pear.php.net/Net_Sieve"
+ */
+function composer_check($package)
+{
+	static $installed=null;
+	if (!isset($installed))
+	{
+		$installed = array();
+		if (file_exists(EGW_SERVER_ROOT.'/vendor') && file_exists($json=EGW_SERVER_ROOT.'/vendor/composer/installed.json'))
+		{
+			foreach(json_decode(file_get_contents($json), true) as $package_data)
+			{
+				$installed[strtolower($package_data['name'])] = $package_data['version'];
+			}
+		}
+	}
+	//error_log(__METHOD__."('$package') returning ".array2string($installed[strtolower($package)]));
+	return $installed[strtolower($package)];
+}
+
 /**
  * quering the pear registry to find out which pear packages and versions are installed
  *
- * @param $channel=null use default or given channel
+ * @param $channel =null use default or given channel
  * @return array with package-name => version pairs, eg. array('Log' => '1.9.8','PEAR' => '1.4.11')
  */
 function get_installed_pear_packages($channel=null)
@@ -376,44 +399,44 @@ function pear_check($package,$args)
 	}
 	$min_version = isset($args['version']) ? $args['version'] : null;
 
-	if (!isset($channel_packages[(string)$channel]))
+	// first check if package is available via composer installed vendor directory
+	$composer_name = 'pear-'.($channel ? $channel : 'pear.php.net').'/'.($package ? $package : 'pear');
+	$version_available = composer_check($composer_name);
+	if ((!($found = !empty($version_available))))
 	{
-		$channel_packages[(string)$channel] = get_installed_pear_packages($channel);
-	}
-	$pear_packages = $channel_packages[(string)$channel];
-	$version_available = false;
-
-	// check if egw-pear is availible and packages is included
-	if ($package && is_dir('../egw-pear') && file_exists('../egw-pear/'.str_replace('_','/',$package).'.php'))
-	{
-		$available = $found = true;
-		$version_available = '999.egw-pear';
-	}
-	// packages found in the pear registry --> use that info
-	elseif ($pear_packages)
-	{
-		$pear_available = $found = true;
-		// check if package is installed
-		if ($package && isset($pear_packages[$package])) $available = true;
-		// check if it's the right version
-		$version_available = $pear_packages[$package ? $package : 'PEAR'];
-	}
-	else	// use the old checks as fallback
-	{
-		if (is_null($pear_available))
+		if (!isset($channel_packages[(string)$channel]))
 		{
-			$pear_available = @include_once('PEAR.php');
-
-			if (!class_exists('PEAR')) $pear_available = false;
+			$channel_packages[(string)$channel] = get_installed_pear_packages($channel);
 		}
-		$found = $pear_available;
-		if ($pear_available && $package)
+		$pear_packages = $channel_packages[(string)$channel];
+		$version_available = false;
+
+		// packages found in the pear registry --> use that info
+		if ($pear_packages)
 		{
-			$file = str_replace('_','/',$package == 'Mail_Mime' ? 'Mail_mime' : $package).'.php';
+			$pear_available = $found = true;
+			// check if package is installed
+			if ($package && isset($pear_packages[$package])) $available = true;
+			// check if it's the right version
+			$version_available = $pear_packages[$package ? $package : 'PEAR'];
+		}
+		else	// use the old checks as fallback
+		{
+			if (is_null($pear_available))
+			{
+				$pear_available = @include_once('PEAR.php');
 
-			$found = @include_once($file);
+				if (!class_exists('PEAR')) $pear_available = false;
+			}
+			$found = $pear_available;
+			if ($pear_available && $package)
+			{
+				$file = str_replace('_','/',$package == 'Mail_Mime' ? 'Mail_mime' : $package).'.php';
 
-			if (!class_exists($package)) $found = false;
+				$found = @include_once($file);
+
+				if (!class_exists($package)) $found = false;
+			}
 		}
 	}
 	// is the right version availible
@@ -428,25 +451,37 @@ function pear_check($package,$args)
 		echo '<div class="setup_info">' . lang('PEAR%1 is needed by: %2.',$package ? '::'.$package : '',
 			is_array($args['from']) ? implode(', ',$args['from']) : $args['from']);
 
-		if (!$pear_available)
-		{
-			echo '<br/>'.lang('PEAR (%1) is a PHP repository and is usually in a package called %2.',
-				'<a href="http://pear.php.net" target="_blank">pear.php.net</a>','php-pear');
-		}
-		elseif ($package && !$found)
+		// if using Composer, dont confuse user with PEAR ;-)
+		if (file_exists(EGW_SERVER_ROOT.'/vendor'))
 		{
 			echo '<br/>'.lang('You can install it by running:').
-				($channel ? ' pear channel-discover '.$channel.' ;' : '').
-				' pear install '.($channel ? $channel.'/' : '').$package;
+				' cd '.realpath(EGW_SERVER_ROOT).'; php composer.phar install';
 		}
-		elseif ($min_version && !$version_available)
+		else
 		{
-			echo '<br/>'.lang('We could not determine the version of %1, please make sure it is at least %2',$package,$min_version);
-		}
-		elseif ($min_version && version_compare($min_version,$version_available) > 0)
-		{
-			echo '<br/>'.lang('Your installed version of %1 is %2, required is at least %3, please run: ',
-				$package,$version_available,$min_version).' pear upgrade '.$package;
+			if (!$pear_available)
+			{
+				echo '<br/>'.lang('PEAR (%1) is a PHP repository and is usually in a package called %2.',
+					'<a href="http://pear.php.net" target="_blank">pear.php.net</a>','php-pear');
+			}
+			elseif ($package && !$found)
+			{
+				echo '<br/>'.lang('You can install it by running:').
+					($channel ? ' pear channel-discover '.$channel.' ;' : '').
+					' pear install '.($channel ? $channel.'/' : '').$package;
+			}
+			elseif ($min_version && !$version_available)
+			{
+				echo '<br/>'.lang('We could not determine the version of %1, please make sure it is at least %2',$package,$min_version);
+			}
+			elseif ($min_version && version_compare($min_version,$version_available) > 0)
+			{
+				echo '<br/>'.lang('Your installed version of %1 is %2, required is at least %3, please run: ',
+					$package,$version_available,$min_version).' pear upgrade '.$package;
+			}
+			echo '<br/>'.lang('Alternatively you can use %1Composer%2 to install all requirements at once. Downloading it and run:',
+				'<a href="https://getcomposer.org/" target="_blank">', '</a>').
+				' cd '.realpath(EGW_SERVER_ROOT).'; php composer.phar install';
 		}
 		echo "</div>";
 	}
@@ -458,7 +493,7 @@ function pear_check($package,$args)
 function extension_check($name,$args)
 {
 	//echo "<p>extension_check($name,".print_r($args,true).")</p>\n";
-	global $passed_icon, $error_icon, $warning_icon, $is_windows;
+	global $passed_icon, $warning_icon, $is_windows;
 
 	if (isset($args['win_only']) && $args['win_only'] && !$is_windows)
 	{
@@ -485,7 +520,7 @@ function extension_check($name,$args)
 
 function function_check($name,$args)
 {
-	global $passed_icon, $error_icon, $warning_icon, $is_windows;
+	global $passed_icon, $warning_icon;
 
 	$available = function_exists($name);
 
@@ -606,16 +641,14 @@ function permission_check($name,$args,$verbose=True)
 		$checks[] = lang('world writable');
 		$check_not = (!$args['is_world_writable']?lang('not'):'');
 	}
-	$checks = implode(', ',$checks);
 
-	$icon = $passed_icon;
 	if (isset($args['msg']) && ($msg = $args['msg']))
 	{
 		$msg .= ': '.$perms."<br />\n";
 	}
 	else
 	{
-		$msg = lang('Checking file-permissions of %1 for %2 %3: %4',$rel_name,$check_not,$checks,$perms)."<br />\n";
+		$msg = lang('Checking file-permissions of %1 for %2 %3: %4',$rel_name,$check_not,implode(', ',$checks),$perms)."<br />\n";
 	}
 	$extra_error_msg = '';
 	if (isset($args['error']) && $args['error'])
@@ -686,52 +719,12 @@ function permission_check($name,$args,$verbose=True)
 
 function mk_value($value)
 {
+	$matches = null;
 	if (!preg_match('/^([0-9]+)([mk]+)$/i',$value,$matches)) return $value;
 
 	return (strtolower($matches[2]) == 'm' ? 1024*1024 : 1024) * (int) $matches[1];
 }
 
-function tnef_check($name,$args)
-{
-	global $passed_icon, $error_icon, $warning_icon;
-
-	$available = false;
-	$tnef = array();
-
-	if (file_exists('/usr/bin/tnef'))
-	{
-		$available = true;
-		$tnef[] = '/usr/bin/tnef';
-	}
-	// Mac ports location
-	elseif (file_exists('/opt/local/bin/tnef'))
-	{
-		$available = true;
-		$tnef[] = '/opt/local/bin/tnef';
-	}
-	elseif (exec("which tnef", $tnef))
-	{
-		$available = true;
-	}
-	elseif (exec("which ytnef", $tnef))
-	{
-		$available = true;
-	}
-
-	echo "<div>".($available ? $passed_icon : $warning_icon).
-		' <span'.($available?'':' class="setup_warning"').'>'.
-		lang('Checking for tnef application').': '.
-		($available ? $tnef[0] : lang('False'));
-
-	if (!$available)
-	{
-		echo "<br />\n".lang('You dont have tnef or ytnef installed! It is needed to decode winmail.dat attachments in felamimail.')."\n";
-	}
-	echo "</span></div>\n";
-
-	return $available;
-}
-
 function php_ini_check($name,$args)
 {
 	global $passed_icon, $error_icon, $warning_icon, $is_windows;
@@ -781,6 +774,7 @@ function php_ini_check($name,$args)
 			unset($tz);
 		}
 		catch(Exception $e) {
+			unset($e);
 			$result = false;	// no valid timezone
 		}
 	}
@@ -820,50 +814,6 @@ function php_ini_check($name,$args)
 	return $result;
 }
 
-function jpgraph_check($name,array $args)
-{
-	global $passed_icon, $error_icon, $warning_icon;
-
-	$egw_path = defined(EGW_SERVER_ROOT) ? EGW_SERVER_ROOT : dirname(dirname(__FILE__));
-	if (!($jpgraph_path = realpath($egw_path.'/..')))
-	{
-		$jpgraph_path = dirname($egw_path);	// realpath can fail because eg. open_basedir does NOT include ..
-	}
-	$jpgraph_path .= SEP.'jpgraph';
-
-	$min_version = isset($args['min_version']) ? $args['min_version'] : '1.13';
-
-	$available = false;
-	if (($check = file_exists($jpgraph_path) && is_dir($jpgraph_path)) &&
-		(file_exists($jpgraph_path.'/src/jpgraph.php') || file_exists($jpgraph_path.'/jpgraph.php')))
-	{
-		if (file_exists($jpgraph_path.'/VERSION') && preg_match('/Version: v([0-9.]+)/',file_get_contents($jpgraph_path.'/VERSION'),$matches))
-		{
-			$version = $matches[1];
-			$available = version_compare($version,$min_version,'>=');
-		}
-		else
-		{
-			$version = 'unknown';
-		}
-	}
-	echo "<div>".($available ? $passed_icon : (isset($version) ? $warning_icon : $error_icon)).
-		' <span'.($available?'':' class="setup_warning"').'>'.
-		lang('Checking for JPGraph in %1',$jpgraph_path).': '.
-		(isset($version) ? lang('Version').' '.$version : lang('False'));
-
-	if (!$available)
-	{
-		echo "<br />\n".lang('You dont have JPGraph version %1 or higher installed! It is needed from ProjectManager for Ganttcharts.',$min_version)."<br />\n";
-		echo lang('Please download a recent version from %1 and install it as %2.',
-			'<a href="http://jpgraph.net/download/" target="_blank">jpgraph.net/download/</a>',
-			$jpgraph_path);
-	}
-	echo "</span></div>\n";
-
-	return $available;
-}
-
 function get_php_ini()
 {
 	ob_start();
@@ -871,6 +821,7 @@ function get_php_ini()
 	$phpinfo = ob_get_contents();
 	ob_end_clean();
 
+	$found = null;
 	return preg_match('/\(php.ini\).*<\/td><td[^>]*>([^ <]+)/',$phpinfo,$found) ? $found[1] : False;
 }
 
@@ -928,7 +879,7 @@ else
 }
 
 $Ok = True;
-foreach ($checks as $name => $args)
+foreach ($sorted_checks as $name => $args)
 {
 	$check_ok = $args['func']($name,$args);
 	$Ok = $Ok && $check_ok;
