@@ -10,7 +10,7 @@
  * @author Miles Lott <milos@groupwhere.org>
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.setup.inc.php 49380 2014-11-12 08:42:00Z ralfbecker $
+ * @version $Id$
  */
 
 class setup
@@ -75,7 +75,7 @@ class setup
 	 */
 	var $recommended_php_version = '5.5';
 
-	function setup($html=False, $translation=False)
+	function __construct($html=False, $translation=False)
 	{
 		// setup us as $GLOBALS['egw_setup'], as this gets used in our sub-objects
 		$GLOBALS['egw_setup'] =& $this;
@@ -88,7 +88,7 @@ class setup
 		$this->detection = new setup_detection();
 		$this->process   = new setup_process();
 
-		if ($_REQUEST['system_charset']) $this->system_charset = $_REQUEST['system_charset'];
+		if (preg_match('/^[a-z0-9-]+$/i', $_REQUEST['system_charset'])) $this->system_charset = $_REQUEST['system_charset'];
 
 		/* The setup application needs these */
 		if ($html) $this->html = new setup_html();
@@ -362,7 +362,7 @@ class setup
     * @param string $hash hash to check password agains (no {prefix} for plain and md5!)
     * @returns bool true on success
     */
-	function check_auth($user, $pw, $conf_user, $hash)
+	static function check_auth($user, $pw, $conf_user, $hash)
 	{
 		if ($user !== $conf_user)
 		{
@@ -416,7 +416,7 @@ class setup
 				return True;	// match
 			}
 		}
-		$GLOBALS['egw_info']['setup']['ConfigLoginMSG'] = lang('Invalid IP address');
+		$GLOBALS['egw_info']['setup']['ConfigLoginMSG'] = lang('Invalid IP address').' '.$remoteip;
 		error_log(__METHOD__.'-> checking IP failed:'.print_r($remoteip,true));
 		return False;
 	}
@@ -498,7 +498,7 @@ class setup
 					'app_name'		=> $appname,
 					'app_enabled'	=> $enable,
 					'app_order'		=> $setup_info[$appname]['app_order'],
-					'app_tables'	=> $tables,
+					'app_tables'	=> (string)$tables,	// app_tables is NOT NULL
 					'app_version'	=> $setup_info[$appname]['version'],
 					'app_index'     => $setup_info[$appname]['index'],
 					'app_icon'      => $setup_info[$appname]['icon'],
@@ -580,7 +580,7 @@ class setup
 			$this->db->update($this->applications_table,array(
 					'app_enabled'	=> $setup_info[$appname]['enable'],
 					'app_order'		=> $setup_info[$appname]['app_order'],
-					'app_tables'	=> $tables,
+					'app_tables'	=> (string)$tables,	// app_tables is NOT NULL
 					'app_version'	=> $setup_info[$appname]['version'],
 					'app_index'     => $setup_info[$appname]['index'],
 					'app_icon'      => $setup_info[$appname]['icon'],
