@@ -7,7 +7,7 @@
  * @package setup
  * @copyright (c) 2007-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.setup_cmd_config.inc.php 49647 2014-11-30 13:46:59Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -199,7 +199,7 @@ class setup_cmd_config extends setup_cmd
 		'--ldap-context' => 'ldap_context',
 		'--ldap-search-filter' => 'ldap_search_filter',
 		'--ldap-group-context' => 'ldap_group_context',
-		'--sambaadmin-sid' => 'sambaadmin/sambaSID',
+		'--sambaadmin-sid' => 'sambaadmin/sambasid',
 		'--allow-remote-admin' => 'allow_remote_admin',
 		'--install-id' => 'install_id',
 		'--ads-host' => 'ads_host',
