@@ -7,7 +7,7 @@
  * @package setup
  * @copyright (c) 2007-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.setup_cmd_ldap.inc.php 49443 2014-11-17 12:20:15Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -45,10 +45,10 @@
  * - copies mail-attributes from ldap to AD (example is from Mandriva mailAccount schema, need to adapt to other schema!)
  *   (no_sid_check=1 uses all objectClass=posixAccount, not checking for having a SID and uid not ending in $ for computer accounts)
  *
- * setup/setup-cli.php [--dry-run] --setup_cmd_ldap <domain>,<config-user>,<config-pw> sub_command=passwords_to_sql \
- * 	ldap_base=dc=local ldap_root_dn=cn=admin,dc=local ldap_root_pw=secret ldap_host=localhost
+ * setup/setup-cli.php [--dry-run] --setup-cmd-ldap <domain>,<config-user>,<config-pw> sub_command=passwords_to_sql \
+ * 	ldap_context=ou=accounts,dc=local ldap_root_dn=cn=admin,dc=local ldap_root_pw=secret ldap_host=localhost
  *
- * Updating passwords for existing users in SQL from LDAP, eg. to switch off authentication to LDAP on a SQL install.
+ * - updating passwords for existing users in SQL from LDAP, eg. to switch off authentication to LDAP on a SQL install.
  */
 class setup_cmd_ldap extends setup_cmd
 {
@@ -146,6 +146,7 @@ class setup_cmd_ldap extends setup_cmd
 				break;
 			case 'migrate_to_ldap':
 			case 'migrate_to_sql':
+			case 'migrate_to_univention':
 			case 'passwords_to_sql':
 				$msg = $this->migrate($this->sub_command);
 				break;
@@ -421,18 +422,21 @@ class setup_cmd_ldap extends setup_cmd
 	/**
 	 * Migrate to other account storage
 	 *
-	 * @param boolean|string $to_ldap true: sql --> ldap, false: ldap --> sql, "passwords_to_sql", "migrate_to_(sql|ldap)"
+	 * @param string $mode "passwords_to_sql", "migrate_to_(sql|ldap|univention)"
 	 * @return string with success message
 	 * @throws Exception on error
 	 */
-	private function migrate($to_ldap)
+	private function migrate($mode)
 	{
-		$passwords2sql = $to_ldap === "passwords_to_sql";
-		if (is_string($to_ldap)) $to_ldap = substr($to_ldap, -8) == '_to_ldap';
+		// support old boolean mode
+		if (is_bool($mode)) $mode = $mode ? 'migrate_to_ldap' : 'migrate_to_sql';
+
+		$passwords2sql = $mode === "passwords_to_sql";
+		list(,$to) = explode('_to_', $mode);
 
 		$msg = array();
 		// if migrating to ldap, check ldap and create context if not yet exiting
-		if ($to_ldap && !empty($this->ldap_admin_pw))
+		if ($to == 'ldap' && !empty($this->ldap_admin_pw))
 		{
 			$msg[] = $this->create();
 		}
@@ -441,20 +445,20 @@ class setup_cmd_ldap extends setup_cmd
 			$msg[] = $this->connect();
 		}
 		// read accounts from old store
-		$accounts = $this->accounts(!$to_ldap, $passwords2sql ? 'accounts' : 'both');
+		$accounts = $this->accounts($to == 'sql' ? 'ldap' : 'sql', $passwords2sql ? 'accounts' : 'both');
 
 		// clean up SQL before migration
-		if (!$to_ldap && $this->truncate_egw_accounts)
+		if ($to == 'sql' && $this->truncate_egw_accounts)
 		{
-		        $GLOBALS['egw']->db->query('TRUNCATE TABLE egw_accounts', __LINE__, __FILE__);
-		        $GLOBALS['egw']->db->query('DELETE FROM egw_addressbook WHERE account_id IS NOT NULL', __LINE__, __FILE__);
+			$GLOBALS['egw']->db->query('TRUNCATE TABLE egw_accounts', __LINE__, __FILE__);
+			$GLOBALS['egw']->db->query('DELETE FROM egw_addressbook WHERE account_id IS NOT NULL', __LINE__, __FILE__);
 		}
 		// instanciate accounts obj for new store
-		$accounts_obj = $this->accounts_obj($to_ldap);
+		$accounts_obj = $this->accounts_obj($to);
 
 		$accounts_created = $groups_created = $errors = $egw_info_set = 0;
 		$emailadmin_src = $ldap_class = null;
-		$target = $to_ldap ? 'LDAP' : 'SQL';
+		$target = strtoupper($to);
 		foreach($accounts as $account_id => $account)
 		{
 			if (isset($this->only) && !in_array($account_id,$this->only))
@@ -464,6 +468,12 @@ class setup_cmd_ldap extends setup_cmd
 			$what = ($account['account_type'] == 'u' ? lang('User') : lang('Group')).' '.
 				$account_id.' ('.$account['account_lid'].')';
 
+			// if we migrate passwords from an authentication source, we need to use account_lid, not numerical id
+			if ($passwords2sql && ($id = $accounts_obj->name2id($account['account_lid'], 'account_lid', 'u')))
+			{
+				$account_id = $id;
+			}
+
 			// invalidate cache: otherwise no migration takes place, if cached results says account already exists
 			accounts::cache_invalidate($account_id);
 
@@ -474,6 +484,11 @@ class setup_cmd_ldap extends setup_cmd
 					$msg[] = lang('%1 does NOT exist in %2.',$what,$target);
 					$errors++;
 				}
+				elseif(empty($account['account_pwd']))
+				{
+					$msg[] = lang('%1 does NOT have a password (userPassword attribute) or we are not allowed to read it!',$what);
+					$errors++;
+				}
 				else
 				{
 					$sql_account['account_passwd'] = self::hash_ldap2sql($account['account_pwd']);
@@ -500,7 +515,7 @@ class setup_cmd_ldap extends setup_cmd
 					$errors++;
 					continue;
 				}
-				if ($to_ldap)
+				if ($to != 'sql')
 				{
 					if ($GLOBALS['egw_info']['server']['ldap_extra_attributes'])
 					{
@@ -548,10 +563,8 @@ class setup_cmd_ldap extends setup_cmd
 				{
 					if (!isset($emailadmin_src))
 					{
-						// add egw-pear to include_path, as setup does NOT do it (only includes common_functions.inc.php from API)
-						set_include_path(EGW_SERVER_ROOT.'/egw-pear'.PATH_SEPARATOR.get_include_path());
 						include_once(EGW_INCLUDE_ROOT.'/emailadmin/inc/class.'.$ldap_class.'.inc.php');
-						if ($to_ldap)
+						if ($to != 'sql')
 						{
 							$emailadmin_src = new emailadmin_smtp_sql();
 							$emailadmin_dst = new $ldap_class();
@@ -653,7 +666,7 @@ class setup_cmd_ldap extends setup_cmd
 			}
 		}
 		ob_start();
-		$addressbook->migrate2ldap($to_ldap ? 'accounts' : 'accounts-back');
+		$addressbook->migrate2ldap($to != 'sql' ? 'accounts' : 'accounts-back');
 		$msgs = array_merge($msg, explode("\n", strip_tags(ob_get_clean())));
 
 		$this->restore_db();
@@ -714,16 +727,16 @@ class setup_cmd_ldap extends setup_cmd
 	/**
 	 * Read all accounts from sql or ldap
 	 *
-	 * @param boolean $from_ldap =true true: ldap, false: sql
+	 * @param string $from ='ldap', 'ldap', 'sql', 'univention'
 	 * @param string $type ='both'
 	 * @return array
 	 */
-	public function accounts($from_ldap=true, $type='both')
+	public function accounts($from='ldap', $type='both')
 	{
-		$accounts_obj = $this->accounts_obj($from_ldap);
+		$accounts_obj = $this->accounts_obj($from);
 		//error_log(__METHOD__."(from_ldap=".array2string($from_ldap).') get_class(accounts_obj->backend)='.get_class($accounts_obj->backend));
 
-		$accounts = $accounts_obj->search(array('type' => $type, 'objectclass' => true));
+		$accounts = $accounts_obj->search(array('type' => $type, 'objectclass' => true, 'active' => false));
 
 		foreach($accounts as $account_id => &$account)
 		{
@@ -751,10 +764,10 @@ class setup_cmd_ldap extends setup_cmd
 	/**
 	 * Instancate accounts object from either sql of ldap
 	 *
-	 * @param boolean $ldap true: ldap, false: sql
+	 * @param string $type 'ldap', 'sql', 'univention'
 	 * @return accounts
 	 */
-	private function accounts_obj($ldap)
+	private function accounts_obj($type)
 	{
 		static $enviroment_setup=null;
 		if (!$enviroment_setup)
@@ -762,19 +775,19 @@ class setup_cmd_ldap extends setup_cmd
 			parent::_setup_enviroment($this->domain);
 			$enviroment_setup = true;
 		}
-		if ($ldap) $this->connect();	// throws exception, if it can NOT connect
+		if ($type != 'sql') $this->connect();	// throws exception, if it can NOT connect
 
 		// otherwise search does NOT work, as accounts_sql uses addressbook_bo for it
-		$GLOBALS['egw_info']['server']['account_repository'] = $ldap ? 'ldap' : 'sql';
+		$GLOBALS['egw_info']['server']['account_repository'] = $type;
 
 		if (!self::$egw_setup->setup_account_object(
 			array(
 				'account_repository' => $GLOBALS['egw_info']['server']['account_repository'],
 			) + $this->as_array()) ||
 			!is_a(self::$egw_setup->accounts,'accounts') ||
-			!is_a(self::$egw_setup->accounts->backend,'accounts_'.($ldap?'ldap':'sql')))
+			!is_a(self::$egw_setup->accounts->backend,'accounts_'.$type))
 		{
-			throw new Exception(lang("Can NOT instancate accounts object for %1",$ldap?'LDAP':'SQL'));
+			throw new Exception(lang("Can NOT instancate accounts object for %1", strtoupper($type)));
 		}
 		return self::$egw_setup->accounts;
 	}
@@ -1006,7 +1019,6 @@ class setup_cmd_ldap extends setup_cmd
 	 */
 	private function _create_node($dn,$extra=array())
 	{
-		//echo "<p>_create_node($dn,".array2string($extra).")</p>\n";
 		// check if the node already exists and return if it does
 		if (@ldap_read($this->test_ldap->ds,$dn,'objectClass=*'))
 		{
