@@ -7,7 +7,7 @@
  * @author Miles Lott <milos@groupwhere.org>
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: index.php 47342 2014-06-24 08:43:46Z ralfbecker $
+ * @version $Id$
  */
 
 $GLOBALS['DEBUG'] = False;
@@ -70,7 +70,11 @@ $GLOBALS['egw_setup']->html->show_header(
 	$GLOBALS['egw_info']['setup']['header_msg'],
 	False,
 	'config',
-	$GLOBALS['egw_setup']->ConfigDomain . '(' . $GLOBALS['egw_domain'][$GLOBALS['egw_setup']->ConfigDomain]['db_type'] . ')'
+	$GLOBALS['egw_setup']->ConfigDomain . ' (' . $GLOBALS['egw_domain'][$GLOBALS['egw_setup']->ConfigDomain]['db_type'].'://'.
+		$GLOBALS['egw_domain'][$GLOBALS['egw_setup']->ConfigDomain]['db_user'].'@'.
+		$GLOBALS['egw_domain'][$GLOBALS['egw_setup']->ConfigDomain]['db_host'].':'.
+		$GLOBALS['egw_domain'][$GLOBALS['egw_setup']->ConfigDomain]['db_port'].'/'.
+		$GLOBALS['egw_domain'][$GLOBALS['egw_setup']->ConfigDomain]['db_name'] . ')'
 );
 /* Add cleaning of app_sessions per skeeter, but with a check for the table being there, just in case */
 /* $GLOBALS['egw_setup']->clear_session_cache(); */
@@ -332,7 +336,7 @@ switch($GLOBALS['egw_info']['setup']['stage']['db'])
 					{
 						$setup_info = $GLOBALS['egw_setup']->detection->upgrade_exclude($setup_info);
 						// Set the DB's client charset if a system-charset is set
-						if ($_REQUEST['system_charset'])
+						if (preg_match('/^[a-z0-9-]+$/i', $_REQUEST['system_charset']))
 						{
 							$GLOBALS['egw_setup']->system_charset = $_REQUEST['system_charset'];
 							$GLOBALS['egw_setup']->db->Link_ID->SetCharSet($_REQUEST['system_charset']);
@@ -528,7 +532,7 @@ switch($GLOBALS['egw_info']['setup']['stage']['db'])
 			}
 		}
 		// warn if essential apps are not installed
-		foreach(array('phpgwapi','etemplate','egw-pear','home','admin','preferences') as $app)
+		foreach(array('phpgwapi','etemplate','home','admin','preferences','emailadmin') as $app)
 		{
 			if (!isset($setup_info[$app]) || empty($setup_info[$app]['currentver']))
 			{
