@@ -8,7 +8,7 @@
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @copyright (c) 2006-14 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: setup-cli.php 49326 2014-11-07 13:15:24Z ralfbecker $
+ * @version $Id$
  */
 
 chdir(dirname(__FILE__));	// to enable our relative pathes to work
@@ -17,7 +17,7 @@ if (php_sapi_name() !== 'cli')	// security precaution: forbit calling setup-cli
 {
 	die('<h1>setup-cli.php must NOT be called as web-page --> exiting !!!</h1>');
 }
-error_reporting(E_ALL & ~E_NOTICE & ~E_STRICT);
+error_reporting(E_ALL & ~E_NOTICE & ~E_STRICT & ~E_DEPRECATED);
 
 $dry_run = false;
 array_shift($_SERVER['argv']);
@@ -126,7 +126,7 @@ switch($action)
 			is_subclass_of($class,'admin_cmd') && @constant($class.'::SETUP_CLI_CALLABLE'))
 		{
 			$args = array();
-			$args['domain'] = array_shift($arguments);	// domain must be first argument, to ensure right domain get's selected in header-include
+			list($args['domain']) = explode(',', array_shift($arguments));	// domain must be first argument, to ensure right domain get's selected in header-include
 			foreach($arguments as $arg)
 			{
 				list($name,$value) = explode('=',$arg,2);
