@@ -6,7 +6,7 @@
  * @package sitemgr
  * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.Common_UI.inc.php 47686 2014-07-15 14:07:36Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -327,7 +327,7 @@ class Common_UI
 					break;
 			}
 		}
-		config::save_value('custom_fields', $cfs, 'sitemgr');
+		egw_customfields::save('sitemgr', $cfs);
 	}
 
 	function DisplayMenu()
