@@ -7,7 +7,7 @@
  * @package timesheet
  * @copyright (c) 2005-13 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.timesheet_ui.inc.php 49486 2014-11-19 08:41:08Z hnategh $
+ * @version $Id$
  */
 
 /**
@@ -917,7 +917,7 @@ class timesheet_ui extends timesheet_bo
 	 *
 	 * @return array see nextmatch_widget::egw_actions()
 	 */
-	private function get_actions(Array $query)
+	public function get_actions(Array $query)
 	{
 		$actions = array(
 			'open' => array(	// does edit if allowed, otherwise view
