@@ -7,7 +7,7 @@
  * @package tracker
  * @copyright (c) 2006-10 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.tracker_admin.inc.php 48845 2014-10-01 15:46:50Z hnategh $
+ * @version $Id$
  */
 
 /**
@@ -538,7 +538,8 @@ class tracker_admin extends tracker_bo
 			'allow_voting' => array('No','Yes'),
 			'allow_bounties' => array('No','Yes'),
 			'autoassign' => $this->get_staff($tracker),
-			'lang' => $GLOBALS['egw']->translation->get_installed_langs(),
+			'lang' => ($tracker ? array('' => lang('default')) : array() )+
+				translation::get_installed_langs(),
 			'cat_id' => $this->get_tracker_labels('cat',$tracker),
 			// Mail handling
 			'interval' => array(
