@@ -7,7 +7,7 @@
  * @package tracker
  * @copyright (c) 2006-12 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.tracker_bo.inc.php 49798 2014-12-09 20:25:13Z nathangray $
+ * @version $Id$
  */
 
 /**
@@ -360,13 +360,7 @@ class tracker_bo extends tracker_so
 		'3 years ago' => array(-3,0,0,0, -2,0,0,0),
 	);
 
-	/**
-	 * Maximum number of line characters (-_+=~) allowed in a mail, to not stall the layout.
-	 * Longer lines / biger number of these chars are truncated to that max. number or chars.
-	 *
-	 * @var int
-	 */
-	var $max_line_chars = 40;
+
 
 	/**
 	 * Constructor
@@ -391,7 +385,7 @@ class tracker_bo extends tracker_so
 	 *
 	 * Reimplemented to set some defaults
 	 *
-	 * @param array $keys=array() array with keys in form internalName => value
+	 * @param array $keys = array() array with keys in form internalName => value
 	 * @return array internal data after init
 	 */
 	function init($keys=array())
@@ -446,8 +440,6 @@ class tracker_bo extends tracker_so
 		// Consider due date independent of overdue days
 		$data['overdue'] |= ($data['tr_duedate'] && $this->now > $data['tr_duedate'] && !in_array($data['tr_status'], $this->get_tracker_stati(null,true)));
 
-		if (is_numeric($data['tr_completion'])) $data['tr_completion'] .= '%';
-
 		// Keep a copy of the timestamps in server time, so notifications can change them for each user
 		foreach($this->timestamps as $field)
 		{
@@ -484,17 +476,17 @@ class tracker_bo extends tracker_so
 	 * @param array $keys array with keys in form internalName => value, may be a scalar value if only one key
 	 * @param string|array $extra_cols string or array of strings to be added to the SELECT, eg. "count(*) as num"
 	 * @param string $join sql to do a join, added as is after the table-name, eg. ", table2 WHERE x=y" or
-	 * @param int $user=null for which user to check, default current user
+	 * @param int $user = null for which user to check, default current user
 	 * @return array|boolean data if row could be retrived else False
 	*/
 	function read($keys,$extra_cols='',$join='',$user=null)
 	{
-		// Read restricted doesn't include assigned, it's not known yet
-		$read_restricted = $this->is_admin(is_array($keys) ? $keys['tr_id'] : $keys, $user) ||
-			$this->is_technician(is_array($keys) ? $keys['tr_id'] : $keys, $user);
-
-		if (($ret = parent::read($keys, $extra_cols, $join, $read_restricted, $user)))
+		if (($ret = parent::read($keys, $extra_cols, $join)))
 		{
+			// read_extras need to know if $user is admin and/or technician of queue of ticket
+			$ret = $this->read_extra($this->is_admin($this->data['tr_tracker'], $user),
+				$this->is_technician($this->data['tr_tracker'], $user), $user);
+
 			$this->data['old_status'] = $this->data['tr_status'];
 
 			if ($this->deny_private()) $ret = $this->data = false;
@@ -669,7 +661,7 @@ class tracker_bo extends tracker_so
 	/**
 	 * Get a list of all groups
 	 *
-	 * @param boolean $primary=false, when not ACL to change the group, return primary group only on new tickets
+	 * @param boolean $primary = false, when not ACL to change the group, return primary group only on new tickets
 	 * @return array with gid => group-name pairs
 	 */
 	function &get_groups($primary=false)
@@ -706,8 +698,8 @@ class tracker_bo extends tracker_so
 	 * Get the staff (technicians or admins) of a tracker
 	 *
 	 * @param int $tracker tracker-id or 0, 0 = staff of all trackers!
-	 * @param int $return_groups=2 0=users, 1=groups+users, 2=users+groups
-	 * @param string $what='technicians' technicians=technicians (incl. admins), admins=only admins, users=only users
+	 * @param int $return_groups = 2 0=users, 1=groups+users, 2=users+groups
+	 * @param string $what = 'technicians' technicians=technicians (incl. admins), admins=only admins, users=only users
 	 * @return array with uid => user-name pairs
 	 */
 	function &get_staff($tracker,$return_groups=2,$what='technicians')
@@ -809,15 +801,21 @@ class tracker_bo extends tracker_so
 	 * Check if a user (default current user) is an admin for the given tracker
 	 *
 	 * @param int $tracker ID of tracker
-	 * @param int $user=null ID of user, default current user $this->user
+	 * @param int $user = null ID of user, default current user $this->user
+	 * @param boolean $checkGivenUser = false flag to force the check If the given User is admin, no matter if $this->user=0
 	 * @return boolean
 	 */
-	function is_admin($tracker,$user=null)
+	function is_admin($tracker,$user=null,$checkGivenUser=false)
 	{
 		if (is_null($user)) $user = $this->user;
 
 		$admins =& $this->get_staff($tracker,0,'admins');
-
+		// evaluate $checkGivenUser flag to force the check If the given User is admin, no matter if $this->user=0
+		// this is used and needed to control (email)notification on close-pending
+		if ($checkGivenUser)
+		{
+			return isset($admins[$user]);
+		}
 		return $this->user===0 || isset($admins[$user]); // this->user is set to 0 by close_pending
 	}
 
@@ -843,7 +841,7 @@ class tracker_bo extends tracker_so
 	 * If queue ACL access is NOT enabled, we return is_tracker_user() (user is non-anonymous and has tracker run-rights)
 	 *
 	 * @param int $tracker ID of tracker
-	 * @param int $user=null ID of user, default current user $this->user
+	 * @param int $user = null ID of user, default current user $this->user
 	 * @return boolean
 	 */
 	function is_user($tracker,$user=null)
@@ -859,7 +857,7 @@ class tracker_bo extends tracker_so
 	 * Check if a user (default current user) is staff member for the given tracker
 	 *
 	 * @param int $tracker ID of tracker
-	 * @param int $user=null ID of user, default current user $this->user
+	 * @param int $user = null ID of user, default current user $this->user
 	 * @return boolean
 	 */
 	function is_staff($tracker,$user=null)
@@ -872,7 +870,7 @@ class tracker_bo extends tracker_so
 	/**
 	 * Check if a user (default current user) is anonymous
 	 *
-	 * @param int $user=null ID of user, default current user $this->user
+	 * @param int $user = null ID of user, default current user $this->user
 	 * @return boolean
 	 */
 	function is_anonymous($user=null)
@@ -901,7 +899,7 @@ class tracker_bo extends tracker_so
 	/**
 	 * Check if a user (default current user) is a non-anoymous user with run-rights for tracker
 	 *
-	 * @param int $user=null ID of user, default current user $this->user
+	 * @param int $user = null ID of user, default current user $this->user
 	 * @return boolean
 	 */
 	function is_tracker_user($user=null)
@@ -938,27 +936,29 @@ class tracker_bo extends tracker_so
 	/**
 	 * Check if given or current ticket is private and user is not creator, assignee or admin
 	 *
-	 * @param array $data=null array with ticket or null for $this->data
-	 * @param int $user=null account_id or null for current user
+	 * @param array $data = null array with ticket or null for $this->data
+	 * @param int $user = null account_id or null for current user
 	 * @return boolean true = deny access to private ticket, false grant access (ticket not private or access allowed)
 	 */
 	function deny_private(array $data=null,$user=null)
 	{
 		if (!$user) $user = $this->user;
 		if (!$data) $data = $this->data;
+		$memberships = $GLOBALS['egw']->accounts->memberships($user, true);
+		$memberships[] = $user;
 
 		return $data['tr_private'] && !($user == $data['tr_creator'] || $this->is_admin($data['tr_tracker'],$user) ||
-			$data['tr_assigned'] && in_array($user,$data['tr_assigned']));
+			$data['tr_assigned'] && array_intersect($memberships, $data['tr_assigned']));
 	}
 
 	/**
 	 * Check what rights the current user has on a given or the current tracker item ($this->data) or a given tracker
 	 *
 	 * @param int $needed or'ed together: TRACKER_ADMIN|TRACKER_TECHNICIAN|TRACKER_ITEM_CREATOR|TRACKER_ITEM_ASSIGNEE
-	 * @param int $check_only_tracker=null should only the given tracker be checked and NO $this->data specific checks be performed, default no
-	 * @param int|array $data=null array with tracker item, integer tr_id or default null for loaded tracker item ($this->tracker)
-	 * @param int $user=null for which user to check, default current user
-	 * @param string $name=null something to put in error_log
+	 * @param int $check_only_tracker = null should only the given tracker be checked and NO $this->data specific checks be performed, default no
+	 * @param int|array $data = null array with tracker item, integer tr_id or default null for loaded tracker item ($this->tracker)
+	 * @param int $user = null for which user to check, default current user
+	 * @param string $name = null something to put in error_log
 	 * @return boolean true if user has the $needed rights, false otherwise
 	 */
 	function check_rights($needed,$check_only_tracker=null,$data=null,$user=null,$name=null)
@@ -1086,8 +1086,8 @@ class tracker_bo extends tracker_so
 	 *
 	 * @param int|array $id id of entry or entry array
 	 * @param int $check EGW_ACL_READ for read and EGW_ACL_EDIT for write or delete access
-	 * @param string $rel_path=null currently not used in Tracker
-	 * @param int $user=null for which user to check, default current user
+	 * @param string $rel_path = null currently not used in Tracker
+	 * @param int $user = null for which user to check, default current user
 	 * @return boolean true if access is granted or false otherwise
 	 */
 	function file_access($id,$check,$rel_path=null,$user=null)
@@ -1115,7 +1115,7 @@ class tracker_bo extends tracker_so
 	/**
 	 * Check if users is allowed to vote and has not already voted
 	 *
-	 * @param int $tr_id=null tracker-id, default current tracker-item ($this->data)
+	 * @param int $tr_id = null tracker-id, default current tracker-item ($this->data)
 	 * @return int|boolean true for no rights, timestamp voted or null
 	 */
 	function check_vote($tr_id=null)
@@ -1138,8 +1138,8 @@ class tracker_bo extends tracker_so
 	/**
 	 * Cast vote for given tracker-item
 	 *
-	 * @param int $tr_id=null tracker-id, default current tracker-item ($this->data)
-	 * @return boolean true=vote casted, false=already voted before
+	 * @param int $tr_id = null tracker-id, default current tracker-item ($this->data)
+	 * @return boolean true = vote casted, false=already voted before
 	 */
 	function cast_vote($tr_id=null)
 	{
@@ -1159,9 +1159,9 @@ class tracker_bo extends tracker_so
 	 * The "cat_data" column stores if a tracker-cat is a "tracker", "version", "cat" or empty.
 	 * Labels need to be either tracker specific or global and NOT in denyglobal.
 	 *
-	 * @param string $type='tracker' 'tracker', 'version', 'cat', 'resolution'
-	 * @param int $tracker=null tracker to use or null to use $this->data['tr_tracker']
-	 * @param int &$default=null on return default, if it is set
+	 * @param string $type = 'tracker' 'tracker', 'version', 'cat', 'resolution'
+	 * @param int $tracker = null tracker to use or null to use $this->data['tr_tracker']
+	 * @param int &$default = null on return default, if it is set
 	 */
 	function get_tracker_labels($type='tracker', $tracker=null, &$default=null)
 	{
@@ -1235,7 +1235,7 @@ class tracker_bo extends tracker_so
 	 *
 	 * There's a bunch of pre-defined stati, plus statis stored as labels, which can be per tracker
 	 *
-	 * @param int $tracker=null tracker to use of null to use $this->data['tr_tracker']
+	 * @param int $tracker = null tracker to use of null to use $this->data['tr_tracker']
 	 * @param boolean $closed True to get 'closed' stati, false to get open stati, null for all
 	 */
 	function get_tracker_stati($tracker=null, $closed = null)
@@ -1260,9 +1260,9 @@ class tracker_bo extends tracker_so
 	 *
 	 * Currently priorities are a fixed list with numeric values from 1 to 9 as keys and customizable labels
 	 *
-	 * @param int $tracker=null tracker to use or null to use tracker unspecific priorities
-	 * @param int $cat_id=null category to use or null to use categorie unspecific priorities
-	 * @param boolean $remove_empty=true should empty labels be displayed, default no
+	 * @param int $tracker = null tracker to use or null to use tracker unspecific priorities
+	 * @param int $cat_id = null category to use or null to use categorie unspecific priorities
+	 * @param boolean $remove_empty = true should empty labels be displayed, default no
 	 * @return array
 	 */
 	function get_tracker_priorities($tracker=null,$cat_id=null,$remove_empty=true)
@@ -1432,7 +1432,9 @@ class tracker_bo extends tracker_so
 		if($options['start'] || $options['num_rows']) {
 			$limit = array($options['start'], $options['num_rows']);
 		}
-		foreach((array) $this->search($pattern,false,'tr_modified DESC','','%',false,'OR',$limit,array('tr_status != '. self::STATUS_DELETED)) as $item )
+		$filter[] = array('tr_status != '. self::STATUS_DELETED);
+		$filter['tr_tracker'] = array_keys($this->trackers);
+		foreach((array) $this->search($pattern,false,'tr_modified DESC','','%',false,'OR',$limit,$filter) as $item )
 		{
 			if ($item) $result[$item['tr_id']] = $this->link_title($item);
 		}
@@ -1447,9 +1449,9 @@ class tracker_bo extends tracker_so
 	 *	For other keys like 'filter', 'cat_id' you have to reimplement this method in a derived class.
 	 * @param array &$rows returned rows/competitions
 	 * @param array &$readonlys eg. to disable buttons based on acl, not use here, maybe in a derived class
-	 * @param string $join='' sql to do a join, added as is after the table-name, eg. ", table2 WHERE x=y" or
+	 * @param string $join = '' sql to do a join, added as is after the table-name, eg. ", table2 WHERE x=y" or
 	 *	"LEFT JOIN table2 ON (x=y)", Note: there's no quoting done on $join!
-	 * @param boolean $need_full_no_count=false If true an unlimited query is run to determine the total number of rows, default false
+	 * @param boolean $need_full_no_count = false If true an unlimited query is run to determine the total number of rows, default false
 	 * @return int total number of rows
 	 */
 	function get_rows(&$query,&$rows,&$readonlys,$join=true,$need_full_no_count=false)
@@ -1650,7 +1652,7 @@ class tracker_bo extends tracker_so
 	/**
 	 * Check if exist and if not start or stop an async job to close pending items
 	 *
-	 * @param boolean $start=true true=start, false=stop
+	 * @param boolean $start = true true=start, false=stop
 	 */
 	static function set_async_job($start=true)
 	{
@@ -1784,7 +1786,7 @@ class tracker_bo extends tracker_so
 	/**
 	 * Delete a bounty, the bounty must not be confirmed and you must be an tracker-admin!
 	 *
-	 * @param int $bounty_id
+	 * @param int $id
 	 * @return boolean true on success or false otherwise
 	 */
 	function delete_bounty($id)
@@ -1809,7 +1811,7 @@ class tracker_bo extends tracker_so
 	 *
 	 * @internal
 	 * @param array $new new value
-	 * @param array $old=null old value
+	 * @param array $old = null old value
 	 */
 	function _bounty2history($new,$old=null)
 	{
@@ -1846,6 +1848,19 @@ class tracker_bo extends tracker_so
 			$bounty['bounty_amount'].' '.$this->currency.($bounty['bounty_confirmed'] ? ' Ok' : '');
 	}
 
+	/**
+	 * Provide response data of get_ticketId to client-side
+	 * JSON response to client with data = (int)ticket_id
+	 * or 0 if there was no ticket registered for the given subject
+	 *
+	 * @param type $_subject
+	 */
+	function ajax_getTicketId($_subject)
+	{
+		$response  = egw_json_response::get();
+		$response->data($this->get_ticketId($_subject));
+	}
+
 	/**
 	 * Try to extract a ticket number from a subject line
 	 *
@@ -1900,39 +1915,38 @@ class tracker_bo extends tracker_so
 	 * prepares the content of an email to be imported as tracker
 	 *
 	 * @author Klaus Leithoff <kl@stylite.de>
-	 * @param string $_email_address rfc822 conform emailaddresses
+	 * @param array $_addresses array of addresses
+	 *	- array (email,name)
 	 * @param string $_subject
 	 * @param string $_message
 	 * @param array $_attachments
-	 * @param string $_date
+	 * @param string $_ticket_id ticket id
 	 * @param int $_queue optional param to pass queue
 	 * @return array $content array for tracker_ui
 	 */
-	function prepare_import_mail($_email_address,$_subject,$_message,$_attachments,$_date,$_queue=0)
+	function prepare_import_mail($_addresses, $_subject, $_message, $_attachments, $_ticket_id, $_queue = 0)
 	{
-		unset($_date);	// unused, but required by function signature
-		$address_array = imap_rfc822_parse_adrlist($_email_address,'');
-		$email = $name = array();
-		foreach ((array)$address_array as $address)
+		foreach((array)$_addresses as $address)
 		{
-			$email[] = $emailadr = sprintf('%s@%s',
-				trim($address->mailbox),
-				trim($address->host));
-			$name[] = !empty($address->personal) ? $address->personal : $emailadr;
+			if (is_array($address) && isset($address['email']))
+			{
+				$emails[] =$address['email'];
+			}
+			else
+			{
+				$parsedAddresses = emailadmin_imapbase::parseAddressList($address);
+				foreach($parsedAddresses as $i => $adr)	$emails[] = $adr->mailbox.'@'.$adr->host;
+			}
 		}
-		// shorten long (> $this->max_line_chars) lines of "line" chars (-_+=~) in mails
-		$_message = preg_replace_callback('/[-_+=~\.]{'.$this->max_line_chars.',}/m',
-			create_function('$matches',"return substr(\$matches[0],0,$this->max_line_chars);"),$_message);
-		//$type = isset($this->enums['type']['email']) ? 'email' : 'note';
-		//$status = isset($this->status['defaults'][$type]) ? $this->status['defaults'][$type] : 'done';
-		$ticketId = $this->get_ticketId($_subject);
+
+		$ticketId = $_ticket_id? $_ticket_id: $this->get_ticketId($_subject);
 		//_debug_array('TickedId found:'.$ticketId);
 		// we have to check if we know this ticket before proceeding
 		if ($ticketId == 0)
 		{
 			$trackerentry = array(
 				'tr_id' => 0,
-				'tr_cc' => implode(', ',$email),
+				'tr_cc' => implode(', ',$emails),
 				'tr_summary' => $_subject,
 				'tr_description' => $_message,
 				'referer' => false,
@@ -1945,17 +1959,18 @@ class tracker_bo extends tracker_so
 			// find the addressbookentry to link with
 			$addressbook = new addressbook_bo();
 			$contacts = array();
-			foreach ($email as $mailadr)
+			$filter['owner'] = 0;
+			foreach ($emails as $mailadr)
 			{
 				$contacts = array_merge($contacts,(array)$addressbook->search(
 					array(
 						'email' => $mailadr,
 						'email_home' => $mailadr
-					),'contact_id,contact_email,contact_email_home,egw_addressbook.account_id as account_id','','','',false,'OR',false,null,'',false));
+					),'contact_id,contact_email,contact_email_home,egw_addressbook.account_id as account_id','','','',false,'OR',false,$filter,'',false));
 			}
 			if (!$contacts || !is_array($contacts) || !is_array($contacts[0]))
 			{
-				$trackerentry['msg'] = lang('Attention: No Contact with address %1 found.',implode(', ',$email));
+				$trackerentry['msg'] = lang('Attention: No Contact with address %1 found.',implode(', ',$emails));
 				$trackerentry['tr_creator'] = $this->user;	// use current user as creator instead
 			}
 			else
@@ -1996,7 +2011,7 @@ class tracker_bo extends tracker_so
 				}
 				if (empty($trackerentry['tr_creator']))
 				{
-					$trackerentry['msg'] = lang('Attention: No Contact with address %1 found.',implode(', ',$email));
+					$trackerentry['msg'] = lang('Attention: No Contact with address %1 found.',implode(', ',$emails));
 					$trackerentry['tr_creator']=$this->user;
 				}
 			}
@@ -2006,13 +2021,14 @@ class tracker_bo extends tracker_so
 			// find the addressbookentry to idetify the reply creator
 			$addressbook = new addressbook_bo();
 			$contacts = array();
-			foreach ($email as $mailadr)
+			$filter['owner'] = 0;
+			foreach ($emails as $mailadr)
 			{
 				$contacts = array_merge($contacts,(array)$addressbook->search(
 					array(
 						'email' => $mailadr,
 						'email_home' => $mailadr
-					),'contact_id,contact_email,contact_email_home,egw_addressbook.account_id as account_id','','','',false,'OR',false,null,'',false));
+					),'contact_id,contact_email,contact_email_home,egw_addressbook.account_id as account_id','','','',false,'OR',false,$filter,'',false));
 			}
 			$found= false;
 			if (!$contacts || !is_array($contacts) || !is_array($contacts[0]))
@@ -2048,7 +2064,7 @@ class tracker_bo extends tracker_so
 					}
 				}
 			}
-			if($found===false) $msg['msg'] = lang('Attention: No Contact with address %1 found.',implode(', ',$email));
+			if($found===false) $msg['msg'] = lang('Attention: No Contact with address %1 found.',implode(', ',$emails));
 			$this->read($ticketId);
 			//echo "<p>data[tr_edit_mode]={$this->data['tr_edit_mode']}, this->htmledit=".array2string($this->htmledit)."</p>\n";
 			// Ascii Replies are converted to html, if htmledit is disabled (default), we allways convert, as this detection is weak
@@ -2076,12 +2092,12 @@ class tracker_bo extends tracker_so
 		{
 			foreach ($_attachments as $attachment)
 			{
-				$is_vfs = false;
-				if (parse_url($attachment['tmp_name'],PHP_URL_SCHEME) == 'vfs' && egw_vfs::is_readable($attachment['tmp_name']))
+				if($attachment['egw_data'])
 				{
-					$is_vfs = true;
+					egw_link::link('tracker',$trackerentry['link_to']['to_id'],egw_link::DATA_APPNAME,$attachment);
 				}
-				if(is_readable($attachment['tmp_name']) || $is_vfs)
+				else if(is_readable($attachment['tmp_name']) ||
+					(egw_vfs::is_readable($attachment['tmp_name']) && parse_url($attachment['tmp_name'], PHP_URL_SCHEME) === 'vfs'))
 				{
 					egw_link::link('tracker',$trackerentry['link_to']['to_id'],'file',$attachment);
 				}
@@ -2095,104 +2111,34 @@ class tracker_bo extends tracker_so
 	 *
 	 * @param string $name
 	 * @param int &$start
-	 * @param int &$end_param
+	 * @param int &$end
 	 * @return string
 	 */
-	function date_filter($name,&$start,&$end_param)
+	function date_filter($name,&$start,&$end)
 	{
-		$end = $end_param;
-
-		if ($name == 'custom' && $start)
+		switch(strtolower($name))
 		{
-			if ($end)
-			{
-				$end += 24*60*60;
-			}
-			else
-			{
-				$end = $start + 8*24*60*60;
-			}
-		}
-		else if (strtolower($name) == 'overdue')
-		{
-                        $limit = $this->now - $this->overdue_days * 24*60*60;
+			case 'overdue':
+				$limit = $this->now - $this->overdue_days * 24*60*60;
 
-			return "(tr_duedate IS NOT NULL and tr_duedate < {$this->now}
+				return "(tr_duedate IS NOT NULL and tr_duedate < {$this->now}
 OR tr_duedate IS NULL AND
-    CASE
-        WHEN tr_modified IS NULL
-        THEN
-            tr_created < $limit
-        ELSE
-            tr_modified < $limit
-    END
-			) ";
-		}
-		else if (strtolower($name) == 'started')
-		{
-			return "(tr_startdate IS NULL OR tr_startdate < {$this->now} )" ;
-		}
-		else if (strtolower($name) == 'upcoming')
-		{
-			return "(tr_startdate IS NOT NULL and tr_startdate > {$this->now} )";
-		}
-		else
-		{
-			if (!isset($this->date_filters[$name]))
-			{
-				return '1=1';
-			}
-			$year  = (int) date('Y',$this->today);
-			$month = (int) date('m',$this->today);
-			$day   = (int) date('d',$this->today);
+	CASE
+		WHEN tr_modified IS NULL
+		THEN
+			tr_created < $limit
+		ELSE
+			tr_modified < $limit
+	END
+) ";
 
-			list($syear,$smonth,$sday,$sweek,$eyear,$emonth,$eday,$eweek) = $this->date_filters[$name];
+			case 'started':
+				return "(tr_startdate IS NULL OR tr_startdate < {$this->now} )" ;
 
-			if(stripos($name, 'quarter') !== false)
-			{
-				// Handle quarters
-				$start = mktime(0,0,0,((int)floor(($smonth+$month) / 3.1)) * 3 + 1, 1, $year);
-				$end = mktime(0,0,0,((int)floor(($emonth+$month) / 3.1)+1) * 3 + 1, 1, $year);
-			}
-			elseif ($syear || $eyear)
-			{
-				$start = mktime(0,0,0,1,1,$syear+$year);
-				$end   = mktime(0,0,0,1,1,$eyear+$year);
-			}
-			elseif ($smonth || $emonth)
-			{
-				$start = mktime(0,0,0,$smonth+$month,1,$year);
-				$end   = mktime(0,0,0,$emonth+$month,1,$year);
-			}
-			elseif ($sday || $eday)
-			{
-				$start = mktime(0,0,0,$month,$sday+$day,$year);
-				$end   = mktime(0,0,0,$month,$eday+$day,$year);
-			}
-			elseif ($sweek || $eweek)
-			{
-				$wday = (int) date('w',$this->today); // 0=sun, ..., 6=sat
-				switch($GLOBALS['egw_info']['user']['preferences']['calendar']['weekdaystarts'])
-				{
-					case 'Sunday':
-						$weekstart = $this->today - $wday * 24*60*60;
-						break;
-					case 'Saturday':
-						$weekstart = $this->today - (6-$wday) * 24*60*60;
-						break;
-					case 'Moday':
-					default:
-						$weekstart = $this->today - ($wday ? $wday-1 : 6) * 24*60*60;
-						break;
-				}
-				$start = $weekstart + $sweek*7*24*60*60;
-				$end   = $weekstart + $eweek*7*24*60*60;
-			}
-			$end_param = $end - 24*60*60;
+			case 'upcoming':
+				return "(tr_startdate IS NOT NULL and tr_startdate > {$this->now} )";
 		}
-		//echo "<p align='right'>date_filter($name,$start,$end) today=".date('l, Y-m-d H:i',$this->today)." ==> ".date('l, Y-m-d H:i:s',$start)." <= date < ".date('l, Y-m-d H:i:s',$end)."</p>\n";
-		// convert start + end from user to servertime for the filter
-		return '('.($start-$this->tz_offset_s).' <= tr_created AND tr_created < '.($end-$this->tz_offset_s).')';
+		return egw_time::sql_filter($name, $start, $end, 'tr_created', $this->date_filters);
 	}
 
 	/**
