@@ -10,7 +10,7 @@
  * @package tracker
  * @copyright (c) 2008 by Oscar van Eijk <oscar.van.eijk-AT-oveas.com>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.tracker_mailhandler.inc.php 49424 2014-11-13 15:07:51Z leithoff $
+ * @version $Id$
  */
 
 class tracker_mailhandler extends tracker_bo
@@ -276,6 +276,7 @@ class tracker_mailhandler extends tracker_bo
 			$this->mailBox = self::get_mailbox($queue);
 		}
 		if (self::LOG_LEVEL>1) error_log(__METHOD__.__LINE__." for $queue on".' # Instance='.$GLOBALS['egw_info']['user']['domain']);
+
 		if ($this->mailBox === false)
 		{
 			if ($TestConnection) throw new egw_exception_wrong_userinput(lang("incomplete server profile for mailhandling provided; Disabling mailhandling for Queue %1", $queue));
@@ -296,7 +297,7 @@ class tracker_mailhandler extends tracker_bo
 				}
 				try
 				{
-					$this->smtpMail = new send();
+					$this->smtpMail = new egw_mailer();
 					if (self::LOG_LEVEL>2) error_log(__METHOD__.__LINE__.array2string($this->smtpMail));
 				} catch(Exception $e) {
 					// ignore exception, but log it, to block the account and give a correct error-message to user
@@ -1381,7 +1382,7 @@ class tracker_mailhandler extends tracker_bo
 			$mailcontent['subject'],
 			$mailcontent['message'],
 			$mailcontent['attachments'],
-			strtotime($mailcontent['headers']['DATE']),
+			($tId?$tId:0),
 			$queue
 		);
 		if (self::LOG_LEVEL>2) error_log(__METHOD__.__LINE__.array2string($this->data));
