@@ -7,7 +7,7 @@
  * @package tracker
  * @copyright (c) 2006-12 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
  * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
- * @version $Id: class.tracker_ui.inc.php 51201 2015-01-12 10:08:59Z ralfbecker $
+ * @version $Id$
  */
 
 /**
@@ -24,7 +24,7 @@ class tracker_ui extends tracker_bo
 		'edit'  => true,
 		'index' => true,
 		'tprint'=> true,
-		'import_mail' => True,
+		'mail_import' => True,
 	);
 	/**
 	 * Displayed instead of the '@' in email-addresses
@@ -60,7 +60,7 @@ class tracker_ui extends tracker_bo
 		if ($GLOBALS['egw_info']['apps']['projectmanager'])
 		{
 			$pm_config = config::read('projectmanager');
-			$this->duration_format = str_replace(',','',$pm_config['duration_units']).','.$pm_config['hours_per_workday'];
+			$this->duration_format = str_replace(',','',implode('', (array)$pm_config['duration_units'])).','.$pm_config['hours_per_workday'];
 			unset($pm_config);
 		}
 	}
@@ -135,8 +135,7 @@ class tracker_ui extends tracker_bo
 				$own_referer = common::get_referer();
 				if (!$this->read($_GET['tr_id']))
 				{
-					$msg = lang('Tracker item not found !!!');
-					$this->init();
+					egw_framework::window_close(lang('Tracker item not found !!!'));
 				}
 				else
 				{
@@ -325,7 +324,10 @@ class tracker_ui extends tracker_bo
 					{
 						$msg = lang('Entry saved');
 						//apply defaultlinks
-						usort($this->all_cats,create_function('$a,$b','return strcasecmp($a["name"],$b["name"]);'));
+						usort($this->all_cats,function($a, $b)
+						{
+							return strcasecmp($a['name'], $b['name']);
+						});
 						foreach($this->all_cats as $cat)
 						{
 							if (!is_array($data = $cat['data'])) $data = array('type' => $data);
@@ -342,6 +344,15 @@ class tracker_ui extends tracker_bo
 						if (is_array($content['link_to']['to_id']) && count($content['link_to']['to_id']))
 						{
 							egw_link::link('tracker',$this->data['tr_id'],$content['link_to']['to_id']);
+
+							// check if we have dragged in images and fix their image urls
+							if (etemplate_widget_vfs::fix_html_dragins('tracker', $this->data['tr_id'],
+								$content['link_to']['to_id'], $content['tr_description']))
+							{
+								$this->update(array(
+									'tr_description' => $content['tr_description'],
+								));
+							}
 						}
 						$state = egw_session::appsession('index','tracker');
 						egw_framework::refresh_opener($msg, 'tracker',$this->data['tr_id'],'edit');
@@ -497,7 +508,7 @@ class tracker_ui extends tracker_bo
 			'tr_description_options' => $tr_description_options,
 			'tr_description_mode'    => $readonlys['tr_description'],
 			'tr_reply_options' => $tr_reply_options,
-			'on_cancel' => $popup ? 'window.close();' : '',
+			'on_cancel' => $popup ? 'egw(window).close();' : '',
 			'no_vote' => '',
 			'show_dates' => $this->show_dates,
 			'link_to' => array(
@@ -635,6 +646,7 @@ class tracker_ui extends tracker_bo
 		$content['no_comment_visibility'] = !$this->check_rights(TRACKER_ADMIN|TRACKER_TECHNICIAN|TRACKER_ITEM_ASSIGNEE,null,null,null,'no_comment_visibility') ||
 			!$this->allow_restricted_comments;
 
+		$account_select_pref = $GLOBALS['egw_info']['user']['preferences']['common']['account_selection'];
 		$sel_options = array(
 			'tr_tracker'  => &$this->trackers,
 			'cat_id'      => $this->get_tracker_labels('cat',$tracker),
@@ -642,10 +654,10 @@ class tracker_ui extends tracker_bo
 			'tr_priority' => $this->get_tracker_priorities($tracker,$content['cat_id']),
 			'tr_status'   => &$statis,
 			'tr_resolution' => $this->get_tracker_labels('resolution',$tracker),
-			'tr_assigned' => $this->get_staff($tracker,$this->allow_assign_groups,$this->allow_assign_users?'usersANDtechnicians':'technicians'),
+			'tr_assigned' => $account_select_pref == 'none' ? array() : $this->get_staff($tracker,$this->allow_assign_groups,$this->allow_assign_users?'usersANDtechnicians':'technicians'),
 			'tr_creator'  => $creators,
 			// New items default to primary group is no right to change the group
-			'tr_group' => $this->get_groups(!$this->check_rights($this->field_acl['tr_group'],$tracker,null,null,'tr_group') && !$this->data['tr_id']),
+			'tr_group' => $account_select_pref == 'none' ? array() : $this->get_groups(!$this->check_rights($this->field_acl['tr_group'],$tracker,null,null,'tr_group') && !$this->data['tr_id']),
 			'canned_response' => $this->get_tracker_labels('response'),
 		);
 
@@ -664,6 +676,9 @@ class tracker_ui extends tracker_bo
 			'bounties' => !$this->allow_bounties,
 			'custom'   => !config::get_customfields('tracker', false, $content['tr_tracker']),
 		);
+		// Make link_to readonly if the user has no EDIT access
+		$readonlys['link_to'] = !$this->file_access($tr_id, EGW_ACL_EDIT);
+
 		if ($tr_id && $readonlys['reply_message'])
 		{
 			$readonlys['button[save]'] = true;
@@ -673,6 +688,13 @@ class tracker_ui extends tracker_bo
 			$msg = lang('Permission denied !!!');
 			$readonlys['button[save]'] = true;
 		}
+		// Assigned & group are not select-account widgets, so we need to apply
+		// none preference (no value, no options) here.
+		if($account_select_pref == 'none')
+		{
+			$readonlys['tr_assigned'] = true;
+			$readonlys['tr_group'] = true;
+		}
 		if (!$this->allow_voting || !$tr_id || $readonlys['vote'] || ($voted = $this->check_vote()))
 		{
 			$readonlys['button[vote]'] = true;
@@ -738,9 +760,9 @@ class tracker_ui extends tracker_bo
 			!$this->allow_bounties && $query_in['order'] == 'bounties') $query_in['order'] = 'tr_id';
 
 		$query = $query_in;
-		if (!$query['csv_export'] && !$query['action'])	// do not store query for csv-export in session
+		if (!$query['csv_export'])	// do not store query for csv-export in session
 		{
-			egw_session::appsession('index','tracker'.($query_in['only_tracker'] ? '-'.$query_in['only_tracker'] : ''),$query);
+			egw_session::appsession($query['session_for'] ? $query['session_for'] : 'index','tracker'.($query_in['only_tracker'] ? '-'.$query_in['only_tracker'] : ''),$query);
 		}
 		// save the state of the index page (filters) in the user prefs
 		// need to save state, before resolving diverse col-filters, eg. to all group-members or sub-cats
@@ -800,6 +822,7 @@ class tracker_ui extends tracker_bo
 			if (!($linked = egw_link::get_links_multiple($app,$id,true,'tracker')))
 			{
 				$rows = array();	// no entries linked to selected link --> no rows to return
+				$this->get_rows_options($rows, $tracker);
 				return 0;
 			}
 
@@ -926,7 +949,33 @@ class tracker_ui extends tracker_bo
 			$id=$row['tr_id'];
 		}
 
-		$rows['duration_format'] = ','.$this->duration_format.',,1';
+		$this->get_rows_options($rows,$tracker,$trackers);
+
+		// disable start date / due date column, if disabled in config
+		if(!$this->show_dates)
+		{
+			$rows['no_tr_startdate_tr_duedate'] = true;
+		}
+
+		return $total;
+	}
+
+	/**
+	 * Selectbox options vary depending on the selected tracker.
+	 *
+	 * @param Array $rows List of rows, we'll add the sel_options in
+	 * @param String[] $tracker List of tracker IDs
+	 */
+	protected function get_rows_options(&$rows, $selected_trackers, $visible_trackers=array())
+	{
+		if(!is_array($selected_trackers) && strpos($selected_trackers,',') !== false)
+		{
+			$tracker = explode(',',$selected_trackers);
+		}
+		else
+		{
+			$tracker = $selected_trackers ? $selected_trackers : array_keys($this->trackers);
+		}
 		$rows['sel_options']['tr_assigned'] = array('not' => lang('Not assigned'));
 
 		// Add allowed staff
@@ -947,7 +996,7 @@ class tracker_ui extends tracker_bo
 			$statis += $this->get_tracker_stati($tr_id);
 		}
 
-		$trackers = array_unique($trackers);
+		$trackers = array_unique($visible_trackers);
 		if($trackers)
 		{
 			foreach($trackers as $tracker_id)
@@ -962,6 +1011,7 @@ class tracker_ui extends tracker_bo
 		$rows['sel_options']['filter2'] = array(lang('All'))+$versions;
 		$rows['sel_options']['tr_version'] =& $versions;
 		$rows['sel_options']['tr_resolution'] =& $resolutions;
+
 		if ($this->is_admin($tracker))
 		{
 			$rows['sel_options']['canned_response'] = $this->get_tracker_labels('response',$tracker);
@@ -983,14 +1033,6 @@ class tracker_ui extends tracker_bo
 
 		// enable tracker column if all trackers are shown
 		if ($tracker && !$query['multi_queue']) $rows['no_tr_tracker'] = true;
-
-		// disable start date / due date column, if disabled in config
-		if(!$this->show_dates)
-		{
-			$rows['no_tr_startdate_tr_duedate'] = true;
-		}
-
-		return $total;
 	}
 
 	/**
@@ -1142,6 +1184,7 @@ class tracker_ui extends tracker_bo
 		{
 			$only_tracker = $content['only_tracker']; unset($content['only_tracker']);
 			$tracker = $content['nm']['col_filter']['tr_tracker'];
+			$this->called_by = $content['called_by']; unset($content['called_by']);
 
 			// Multiple queues at once
 			list($multi_queue_filter) = @each($content['nm']['col_filter']['multi_queue']);
@@ -1229,6 +1272,7 @@ class tracker_ui extends tracker_bo
 			'tr_status'   => $this->filters + $this->get_tracker_stati($tracker),
 			'tr_priority' => $this->get_tracker_priorities($tracker,$content['cat_id']),
 			'tr_resolution' => $this->get_tracker_labels('resolution',$tracker),
+			// Still need to provide options for the column filter
 			'tr_private'  => array('No', 'Yes'),
 		);
 		if (($escalations = ExecMethod2('tracker.tracker_escalations.query_list','esc_title','esc_id')))
@@ -1250,12 +1294,12 @@ class tracker_ui extends tracker_bo
 		}
 
 		if (!is_array($content)) $content = array();
-		$content['nm'] = egw_session::appsession('index','tracker'.($only_tracker ? '-'.$only_tracker : ''));
+		$content['nm'] = egw_session::appsession($this->called_by ? $this->called_by : 'index', 'tracker'.($only_tracker ? '-'.$only_tracker : ''));
 		$content['msg'] = $msg;
 		$content['status_help'] = !$this->pending_close_days ? lang('Pending items never get close automatic.') :
 				lang('Pending items will be closed automatic after %1 days without response.',$this->pending_close_days);
 
-		if (!is_array($content['nm']))
+		if (!is_array($content['nm']) || !$content['nm']['get_rows'])
 		{
 			$date_filters = array(lang('All'));
 			foreach(array_keys($this->date_filters) as $name)
@@ -1286,12 +1330,12 @@ class tracker_ui extends tracker_bo
 				'row_modified'   => 'tr_modified'
 			);
 			// use the state of the last session stored in the user prefs
-			if (($state = @unserialize($GLOBALS['egw_info']['user']['preferences']['tracker']['index_state'])))
+			if (!$this->called_by && ($state = @unserialize($GLOBALS['egw_info']['user']['preferences']['tracker']['index_state'])))
 			{
 				$content['nm'] = array_merge($content['nm'],$state);
 				$tracker = $content['nm']['col_filter']['tr_tracker'];
 			}
-			elseif (!$tracker)
+			elseif (!$this->called_by && !$tracker)
 			{
 				reset($this->trackers);
 				list($tracker) = @each($this->trackers);
@@ -1309,9 +1353,7 @@ class tracker_ui extends tracker_bo
 				$content['nm']['options-selectcols']['tr_duedate'] = false;
 			}
 		}
-		$content['nm']['actions'] = $this->get_actions($tracker, $content['cat_id']);
-		$content['nm']['multi_queue'] = $multi_queue;
-
+		if (!$content['nm']['session_for'] && $this->called_by) $content['nm']['session_for'] = $this->called_by;
 		if($_GET['search'])
 		{
 			$content['nm']['search'] = $_GET['search'];
@@ -1332,10 +1374,17 @@ class tracker_ui extends tracker_bo
 			$content['nm']['col_filter']['tr_tracker'] = $tracker;
 		}
 
-		// Turn on multi-queue widget
-		$content['nm']['header_left'] = $content['nm']['multi_queue'] ? 'tracker.index.left_multiqueue' : 'tracker.index.left';
-
-		$content['nm']['favorites'] = true; // Enable favorites
+		//
+		// disable favories dropdown button, if not running as infolog
+		if ($this->called_as || $content['nm']['session_for'])
+		{
+			$content['nm']['favorites'] = false;
+		}
+		else
+		{
+			$content['nm']['favorites'] = true; // Enable favorites
+		}
+		$content['duration_format'] = ','.$this->duration_format.',,1';
 
 		$content['is_admin'] = $this->is_admin($tracker);
 		//_debug_array($content);
@@ -1353,6 +1402,11 @@ class tracker_ui extends tracker_bo
 			$tpl->set_dom_id("{$tpl->name}-{$this->called_by}");
 		}
 
+		$content['nm']['actions'] = $this->get_actions($tracker, $content['cat_id']);
+		$content['nm']['multi_queue'] = $multi_queue;
+		// Turn on multi-queue widget
+		$content['nm']['header_left'] = $content['nm']['multi_queue'] ? 'tracker.index.left_multiqueue' : 'tracker.index.left';
+
 		// disable filemanager icon, if user has no access to it
 		$readonlys['filemanager/navbar'] = !isset($GLOBALS['egw_info']['user']['apps']['filemanager']);
 
@@ -1385,7 +1439,12 @@ width:100%;
 }</style>';
 		}
 
-		return $tpl->exec('tracker.tracker_ui.index',$content,$sel_options,$readonlys,array('only_tracker' => $only_tracker),$return_html);
+		$preserve = array(
+			'only_tracker' => $only_tracker,
+			'called_by' => $this->called_by
+		);
+
+		return $tpl->exec('tracker.tracker_ui.index',$content,$sel_options,$readonlys,$preserve,$return_html);
 	}
 
 	/**
@@ -1395,7 +1454,7 @@ width:100%;
 	 * @param int $cat_id =null
 	 * @return array see nextmatch_widget::get_actions()
 	 */
-	private function get_actions($tracker=null, $cat_id=null)
+	public function get_actions($tracker=null, $cat_id=null)
 	{
 		for($i = 0; $i <= 100; $i += 10)
 		{
@@ -1558,7 +1617,7 @@ width:100%;
 			);
 		}
 
-		$actions['documents'] = infolog_merge::document_action(
+		$actions['documents'] = tracker_merge::document_action(
 			$this->prefs['document_dir'], ++$group, 'Insert in document', 'document_',
 			$this->prefs['default_document']
 		);
@@ -1568,206 +1627,28 @@ width:100%;
 	}
 
 	/**
-	 * imports a mail as tracker
-	 * two possible calls:
-	 * 1. with function args set. (we come from send mail)
-	 * 2. with $_GET['uid] = someuid (we come from display mail)
+	 * imports a mail as Tracker
 	 *
-	 * @author klaus Leithoff <kl@stylite.de>
-	 * @param string $_to_emailAddress
-	 * @param string $_subject
-	 * @param string $_body
-	 * @param array $_attachments
-	 * @param string $_date
-	 * @param string $_rawMailHeader
-	 * @param string $_rawMailBody
+	 * @param array $mailContent = null mail content
+	 * @return  array
 	 */
-	function import_mail($_to_emailAddress=false,$_subject=false,$_body=false,$_attachments=false,$_date=false,$_rawMailHeader=null,$_rawMailBody=null)
+	function mail_import(array $mailContent=null)
 	{
-		$uid = $_GET['uid'];
-		$partid = $_GET['part'];
-		$mailbox = base64_decode($_GET['mailbox']);
-		$mailClass = 'mail_bo';
-		$sessionLocation = 'mail';
-		// if rowid is set, we are called from new mail module.
-		if (method_exists('mail_ui','splitRowID') && isset($_GET['rowid']) && !empty($_GET['rowid']))
+		// It would get called from compose as a popup with egw_data
+		if (!is_array($mailContent) && ($_GET['egw_data']))
 		{
-			// rowid holds all needed information: server, folder, uid, etc.
-			$rowID = $_GET['rowid'];
-			$hA = mail_ui::splitRowID($rowID);
-			$sessionLocation = $hA['app']; // THIS is part of the row ID, we may use this for validation
-			if ($sessionLocation != 'mail') throw new egw_exception_assertion_failed(lang('Application mail expected but got: %1',$sessionLocation));
-			$uid = $hA['msgUID'];
-			$mailbox = $hA['folder'];
-			$icServerID = $hA['profileID'];
-			$mailClass = 'mail_bo';
+			// get the mail raw data
+			egw_link::get_data ($_GET['egw_data']);
+			return false;
 		}
-		// load lang stuff for mailheaderInfoSection creation
-		translation::add_app($sessionLocation);
-		if ($_date == false || empty($_date)) $_date = $this->bo->user_time_now;
-		if (!empty($_to_emailAddress))
-		{
-			$GLOBALS['egw_info']['flags']['currentapp'] = 'tracker';
-			if (!($GLOBALS['egw_info']['user']['preferences'][$sessionLocation]['saveAsOptions']==='text_only')&&is_array($_attachments))
-			{
-				//echo __METHOD__.'<br>';
-				//_debug_array($_attachments);
-				$icServerID =& egw_cache::getSession($sessionLocation,'activeProfileID');
-				$mailobject	= $mailClass::getInstance(true,$icServerID);
-				$mailobject->openConnection();
-				foreach ($_attachments as $attachment)
-				{
-					if (trim(strtoupper($attachment['type'])) == 'MESSAGE/RFC822' && !empty($attachment['uid']) && !empty($attachment['folder']))
-					{
-						$mailobject->reopen($attachment['folder']);
+		// Wrap a pre tag if we are using html editor
+		$message = $this->htmledit? "<pre>".$mailContent['message']."</pre>": $mailContent['message'];
 
-						// get the message itself, and attach it, as we are able to display it in egw
-						// instead of fetching only the attachments attached files (as we did previously)
-						$message = $mailobject->getMessageRawBody($attachment['uid'],$attachment['partID'],($attachment['folder']?$attachment['folder']:$mailbox));
-						$headers = $mailobject->getMessageHeader($attachment['uid'],$attachment['partID'],true,false,($attachment['folder']?$attachment['folder']:$mailbox));
-						$subject = str_replace('$$','__',($headers['SUBJECT']?$headers['SUBJECT']:lang('(no subject)')));
-						$attachment_file =tempnam($GLOBALS['egw_info']['server']['temp_dir'],$GLOBALS['egw_info']['flags']['currentapp']."_");
-						$tmpfile = fopen($attachment_file,'w');
-						fwrite($tmpfile,$message);
-						fclose($tmpfile);
-						$size = filesize($attachment_file);
-						$attachments[] = array(
-								'name' => trim($subject).'.eml',
-								'mimeType' => 'message/rfc822',
-								'type' => 'message/rfc822',
-								'tmp_name' => $attachment_file,
-								'size' => $size,
-							);
-					}
-					else
-					{
-						if (!empty($attachment['folder']))
-						{
-							$is_winmail = $_GET['is_winmail'] ? $_GET['is_winmail'] : 0;
-							$mailobject->reopen($attachment['folder']);
-							$attachmentData = $mailobject->getAttachment($attachment['uid'],$attachment['partID'],$is_winmail);
-							$attachment['file'] =tempnam($GLOBALS['egw_info']['server']['temp_dir'],$GLOBALS['egw_info']['flags']['currentapp']."_");
-							$tmpfile = fopen($attachment['file'],'w');
-							fwrite($tmpfile,$attachmentData['attachment']);
-							fclose($tmpfile);
-						}
-						//make sure we search for our attached file in our configured temp_dir
-						if (isset($attachment['file']) && parse_url($attachment['file'],PHP_URL_SCHEME) != 'vfs' &&
-							file_exists($GLOBALS['egw_info']['server']['temp_dir'].SEP.basename($attachment['file'])))
-						{
-							$attachment['file'] = $GLOBALS['egw_info']['server']['temp_dir'].SEP.basename($attachment['file']);
-						}
-
-						$attachments[] = array(
-							'name' => $attachment['name'],
-							'mimeType' => $attachment['type'],
-							'type' => $attachment['type'],
-							'tmp_name' => $attachment['file'],
-							'size' => $attachment['size'],
-						);
-					}
-				}
-				$mailobject->closeConnection();
-			}
-
-			// this one adds the mail itself (as message/rfc822 (.eml) file) to the infolog as additional attachment
-			// this is done to have a simple archive functionality (ToDo: opening .eml in email module)
-			if ($_rawMailHeader && $_rawMailBody && $GLOBALS['egw_info']['user']['preferences'][$sessionLocation]['saveAsOptions']==='add_raw')
-			{
-				$message = ltrim(str_replace("\n","\r\n",$_rawMailHeader)).str_replace("\n","\r\n",$_rawMailBody);
-				$subject = $mailClass::adaptSubjectForImport($_subject);
-				$attachment_file =tempnam($GLOBALS['egw_info']['server']['temp_dir'],$GLOBALS['egw_info']['flags']['currentapp']."_");
-				$tmpfile = fopen($attachment_file,'w');
-				fwrite($tmpfile,$message);
-				fclose($tmpfile);
-				$size = filesize($attachment_file);
-				$attachments[] = array(
-						'name' => trim($subject).'.eml',
-						'mimeType' => 'message/rfc822',
-						'type' => 'message/rfc822',
-						'tmp_name' => $attachment_file,
-						'size' => $size,
-					);
-			}
-
-			//_debug_array($_to_emailAddress);
-			$toaddr = array();
-			foreach(array('to','cc','bcc') as $x)
-			{
-				if (is_array($_to_emailAddress[$x]) && !empty($_to_emailAddress[$x])) $toaddr = array_merge($toaddr,$_to_emailAddress[$x]);
-			}
-			//_debug_array($attachments);
-			//we need to fix broken tags (or just stuff like "<800 USD/p" )
-			$_body = htmlspecialchars_decode(strip_tags($_body), ENT_QUOTES);
-			$_subject = $mailClass::adaptSubjectForImport($_subject);
-			$tId = $this->get_ticketId($_subject);
-			if ($tId)
-			{
-				$t = $this->read($tId);
-				$this->htmledit = $t['tr_edit_mode']=='html';
-			}
-			$body = $mailClass::createHeaderInfoSection(array('FROM'=>$_to_emailAddress['from'],
-				'TO'=>(!empty($_to_emailAddress['to'])?implode(',',$_to_emailAddress['to']):null),
-				'CC'=>(!empty($_to_emailAddress['cc'])?implode(',',$_to_emailAddress['cc']):null),
-				'BCC'=>(!empty($_to_emailAddress['bcc'])?implode(',',$_to_emailAddress['bcc']):null),
-				'SUBJECT'=>$_subject,
-				'DATE'=>$mailClass::_strtotime($_date)),'',$this->htmledit).$_body;
-			if ($this->htmledit) $body = '<pre>'.$body.'</pre>';
-			$this->edit($this->prepare_import_mail(
-				implode(',',$toaddr),$_subject,$body,$attachments,$_date
-			));
-			exit;
-		}
-		elseif ($uid && $mailbox)
-		{
-			$icServerID =& egw_cache::getSession($sessionLocation,'activeProfileID');
-			$mailobject	= $mailClass::getInstance(true,$icServerID);
-			$mailobject->openConnection();
-			$mailobject->reopen($mailbox);
-			$headers = $mailobject->getMessageHeader($uid, $partid,true,false,$mailbox);
-			$subject = $mailClass::adaptSubjectForImport($headers['SUBJECT']);
-			$tId = $this->get_ticketId($subject);
-			if ($tId)
-			{
-				$t = $this->read($tId);
-				$this->htmledit = $t['tr_edit_mode']=='html';
-			}
-
-			$mailcontent = $mailClass::get_mailcontent($mailobject,$uid,$partid,$mailbox,$this->htmledit,true,(!($GLOBALS['egw_info']['user']['preferences'][$sessionLocation]['saveAsOptions']==='text_only')));
-
-			// this one adds the mail itself (as message/rfc822 (.eml) file) to the infolog as additional attachment
-			// this is done to have a simple archive functionality (ToDo: opening .eml in email module)
-			if ($GLOBALS['egw_info']['user']['preferences'][$sessionLocation]['saveAsOptions']==='add_raw')
-			{
-				$message = $mailobject->getMessageRawBody($uid, $partid,$mailbox);
-				$headers = $mailobject->getMessageHeader($uid, $partid,true,false,$mailbox);
-				$subject = $mailClass::adaptSubjectForImport($headers['SUBJECT']);
-				$attachment_file =tempnam($GLOBALS['egw_info']['server']['temp_dir'],$GLOBALS['egw_info']['flags']['currentapp']."_");
-				$tmpfile = fopen($attachment_file,'w');
-				fwrite($tmpfile,$message);
-				fclose($tmpfile);
-				$size = filesize($attachment_file);
-				$mailcontent['attachments'][] = array(
-						'name' => trim($subject).'.eml',
-						'mimeType' => 'message/rfc822',
-						'type' => 'message/rfc822',
-						'tmp_name' => $attachment_file,
-						'size' => $size,
-					);
-			}
-			$mailcontent['subject'] = $mailClass::adaptSubjectForImport($mailcontent['subject']);
-			return $this->edit($this->prepare_import_mail(
-				$mailcontent['mailaddress'],
-				$mailcontent['subject'],
-				$mailcontent['message'],
-				$mailcontent['attachments'],
-				strtotime($mailcontent['headers']['DATE'])
-			));
-		}
-		common::egw_header();
-		egw_framework::window_close(lang('Error: no mail (Mailbox / UID) given!'));
-		common::egw_exit();
-		exit;
+		$this->edit($this->prepare_import_mail($mailContent['addresses'],
+				$mailContent['subject'],
+				$message,
+				$mailContent['attachments'],
+				$mailContent['entry_id']));
 	}
 
 	/**
@@ -2021,8 +1902,6 @@ width:100%;
 		// Load JS for tracker actions
 		egw_framework::validate_file('.','app','tracker');
 
-		$state=egw_session::appsession('index','tracker');
-
 		switch ($args['location'])
 		{
 			case 'addressbook_view':
@@ -2039,7 +1918,10 @@ width:100%;
 			return False;
 		}
 		$this->called_by = $app;	// for read/save_sessiondata, to have different sessions for the hooks
-		$GLOBALS['egw_info']['flags']['currentapp'] = 'tracker';
+
+		// Set to calling app, so actions wind up in the correct place client side
+		$GLOBALS['egw_info']['flags']['currentapp'] = $app;
+
 		translation::add_app('tracker');
 
 		$this->index(null);
